<?php


require_once 'Zend/Application/Bootstrap/ResourceBootstrapper.php';
require_once 'Zend/Application/Bootstrap/Bootstrapper.php';
require_once 'Zend/Application/Bootstrap/BootstrapAbstract.php';
require_once 'Zend/Application/Bootstrap/Bootstrap.php';
require_once 'Zend/Loader/Autoloader.php';
require_once(APPLICATION_PATH . "/abc.php");
ini_set('precision', 16);

class Bootstrap extends Zend_Application_Bootstrap_Bootstrap
{

    protected function _initSession()
    {
        Zend_Session::start();
    }

    protected function _initCache(array $options = array())
    {
        $options = $this->getOption('cache');

        $frontendOptions = array('automatic_serialization' => true, 'lifetime' => 3600);
        $longFrontendOptions = array('automatic_serialization' => false, 'lifetime' => 24*3600);

        if (isset($options['backendOptions'])) {
            $backendOptions = $options['backendOptions'];
        } else {
            $backendOptions = array();
        }
	    $cache = Zend_Cache::factory('Core', $options['backend'], $frontendOptions, $backendOptions);
        // We are using the same cache-path, hopefully this is not a problem
        
        $tsCache = Zend_Cache::factory('Core', $options['backend'], $frontendOptions, $backendOptions);

        Zend_Registry::set('cache', $cache);
        Zend_Registry::set('timestamp-cache', $tsCache);

        $weatherCache = Zend_Cache::factory('Core', $options['backend'], $longFrontendOptions, $backendOptions);
        $shortWeatherCache = Zend_Cache::factory('Core', $options['backend'], $frontendOptions, $backendOptions);
        Zend_Registry::set('short-weather-cache', $shortWeatherCache);
        Zend_Registry::set('weather-cache', $weatherCache);
 
        return $cache;
    }

    protected function _initDb(array $options = array())
    {
        $options = $this->getOption('database');
        $db = Zend_Db::factory($options['adapter'], $options['params']);
        $db->setFetchMode(Zend_Db::FETCH_OBJ);
        Zend_Db_Table_Abstract::setDefaultAdapter($db);
        Zend_Registry::set('db', $db);

        if ($this->getEnvironment() == 'production') {
            $this->bootstrap('cache');
            $cache = $this->getResource('cache');
            Zend_Db_Table_Abstract::setDefaultMetadataCache($cache);
        } elseif ($this->getEnvironment() == 'development') {
            //$profiler = new Zend_Db_Profiler_Firebug('All DB Queries');
            //$profiler->setEnabled(true);
            #$db->setProfiler($profiler);
        }
        return $db;
    }

    protected function _initView(array $options = array())
    {
        $this->bootstrap('Constants');
        $view = new Zend_View();

        $view->doctype('XHTML1_STRICT');
        $view->setEncoding('utf-8');

        $view->addHelperPath('CM/View/Helper', 'CM_View_Helper');
        $view->addHelperPath('WebReport/View/Helper', 'WebReport_View_Helper');
        $view->addHelperPath('Talking/View/Helper', 'Talking_View_Helper');
	$view->addScriptPath(MODULES_PATH . '/core/views/scripts');

        $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('ViewRenderer');
        $viewRenderer->setView($view);

        return $view;
    }

    protected function _initModules(array $options = array())
    {
        $this->bootstrap('Constants');
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');
        $this->bootstrap('db');
        $db = $this->getResource('db');

        if(false && Zend_Registry::isRegistered('quickstart') && Zend_Registry::get('quickstart')) {
            $modules = array();
            $modules[] = new stdClass();
            $modules[0]->alias = 'mod_core';
        } else {
            #MODULES-CLEAN Trying to get rid of modules DB table. WIP.
            #$modules = $db->fetchAll($db->select()->from('module')->order('order'));
            $modules = array_keys($front->getControllerDirectory());
        }
        $router = $front->getRouter();

        $ret = array();
        foreach ($modules as $module) {
            #alias not needed anymore when moving from module db table.
            #$alias = substr($module->alias, strpos($module->alias, "_") + 1);

            $ret[] = $module;
            $modulePath = MODULES_PATH . '/' . $module;

            $resourceLoader = new Zend_Loader_Autoloader_Resource(array(
                        'basePath' => $modulePath,
                        'namespace' => Talking_Toolbox_String::toCamel(str_replace("-", "_", ucfirst($module))),
                    ));

            $resourceLoader->addResourceType('plugin', 'plugins/', 'Plugin')
                    ->addResourceType('form', 'forms/', 'Form')
                    ->addResourceType('model', 'models/', 'Model');

            $appOptions = $this->getOptions();
            $configPath = $modulePath . '/config/config.ini';
            if (is_file($configPath)) {
                $o = new Zend_Config_Ini($configPath, $this->getEnvironment());
                $options = $o->toArray();

                if (array_key_exists($module, $appOptions)) {
                    if (is_array($appOptions[$module])) {
                        $appOptions[$module] = array_merge($appOptions[$module], $options);
                    } else {
                        $appOptions[$module] = $options;
                    }
                } else {
                    $appOptions[$module] = $options;
                }

                if (isset($options["autoloaderResourceTypes"])) {
                    foreach ($options["autoloaderResourceTypes"] as $name) {
                        $resourceLoader->addResourceType($name, $name . "/", $name);
                    }
                }
            }



            $this->setOptions($appOptions);
        }


        return $ret;
    }

    protected function _initTranslate(array $options = array())
    {
        $this->bootstrap('Constants');

        $this->bootstrap('modules');
        $modules = $this->getResource('modules');
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');

        $this->bootstrap('locale');
        $locale = $this->getResource('locale');

        $this->bootstrap('loggers');
        $loggers = $this->getResource('loggers');

        $this->bootstrap('auth');
    
        $cache = $this->getResource('cache');
        if($cache) {
            Zend_Translate::setCache($cache);
            // This does not work. Don't know why.
            //Zend_Date::setOptions(array('format_type' => 'php', 'cache' => $cache));
        }
        
        $options = $this->getOption('general');
        $languages = $options['languages'];
        Zend_Registry::set('languages', $languages);

        if($cache && $t = $cache->load("trans")) {
            Zend_Validate_Abstract::setDefaultTranslator($t);
            $front->registerPlugin(new Admin_Plugin_ChangeLanguage());
            $t->setLocale($locale);
            Zend_Registry::set('Zend_Translate', $t);
            return;
        }
        $translate = null;

        /* @var $locale Zend_Locale  */
        foreach ($languages as $language) {
            $languageFile = APPLICATION_PATH . '/languages/' . $language . '.mo';
            if (Talking_Filesystem_Dir::exists($languageFile)) {
                if ($translate == null) {
                    $translate = new Zend_Translate('gettext', $languageFile, $language);
                } else {
                    $translate->addTranslation($languageFile, $language);
                }
            }
        }
        
        $language = $locale->getLanguage();
/*
        $translate->setOptions(array(
            'log' => $loggers->getLogger(WebReport_Log::TRANSLATION_LOG),
            'logUntranslated' => true
        ));
*/

        $front->registerPlugin(new Admin_Plugin_ChangeLanguage());
        $translate->setLocale($locale);
        Zend_Registry::set('Zend_Translate', $translate);
        if($cache) $cache->save($translate,"trans");
        $translator = new Zend_Translate(
                        'array',
                        APPLICATION_PATH . '/../library/languages/Zend_Validate/',
                        $language,
                        array('scan' => Zend_Translate::LOCALE_DIRECTORY)
        );
        Zend_Validate_Abstract::setDefaultTranslator($translator);
        return $translate;
    }

    protected function _initRoutes(array $options = array())
    {
        $this->bootstrap('modules');
        $modules = $this->getResource('modules');
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');

        $this->bootstrap('Constants');


        $router = $front->getRouter();

        foreach ($modules as $module) {

            $modulePath = MODULES_PATH . '/' . $module;
            $moduleConfigPath = $modulePath . '/config';
            $routesConfigFile = $moduleConfigPath . '/routes.ini';

            if (Talking_Filesystem_Dir::exists($routesConfigFile)) {
                $routesConfig = new Zend_Config_Ini($routesConfigFile, $this->getEnvironment());
                $router->addConfig($routesConfig);
            }
        }

        $front->setRouter($router);
        return $router;
    }

    protected function _initForms(array $options = array())
    {
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');

        $this->bootstrap('translate');
        $translate = $this->getResource('translate');

        $this->bootstrap('view');
        $view = $this->getResource('view');
        $view->setEncoding("UTF-8");

        UniForm_Form::setCssDirectory($front->getBaseUrl() . '/templates/' . $view->template() . '/css/uni-form');

        Zend_Form::setDefaultTranslator($translate);
    }

    protected function _initAmCharts(array $options = array())
    {
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');

        AmCharts_Abstract::setSwfObjectScriptPath($front->getBaseUrl() . '/library/javascript/swfobject.js');
        AmCharts_Abstract::setSwfFilesPath($front->getBaseUrl() . '/modules/charts/amcharts/');
    }

    protected function _initLoggers(array $options = array())
    {
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');
        $this->bootstrap('db');

        if ($this->getEnvironment() == 'development' || $this->getEnvironment() == 'testing') {
            Zend_Registry::set("Zend_Logger_FireBug", new Zend_Log(new Zend_Log_Writer_Firebug()));
        } else if ($this->getEnvironment() == 'production') {
            Zend_Registry::set("Zend_Logger_FireBug", new Zend_Log(new Zend_Log_Writer_Null()));
        }

        $loggers = new WebReport_Loggers();

        Zend_Registry::set('loggers', $loggers);
        Zend_Controller_Action_HelperBroker::addHelper(new WebReport_Controller_Action_Helper_Loggers());

        return $loggers;
    }

    protected function _initAuth(array $options = array())
    {

        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');

        $auth = Zend_Auth::getInstance();

        /* TODO: load specific user configs */
        /*
          $localeString = 'en';
          if ($auth->hasIdentity()) {
          $read = $auth->getStorage()->read();
          } */

        $front->registerPlugin(new Admin_Plugin_CheckAuth());

        return $auth;
    }

    protected function _initLocale(array $options = array())
    {
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');
        $this->bootstrap('auth');
        $auth = $this->getResource('auth');
        $options = $this->getOption('general');
        $languages = $options['languages'];

        if ($this->getEnvironment() == 'production') {
            $this->bootstrap('cache');
            $cache = $this->getResource('cache');
            Zend_Locale::setCache($cache);
        }


        try {

            $locale = new Zend_Locale(Zend_Locale::BROWSER);
        } catch (Exception $e) {
            $locale = new Zend_Locale('en');
            // changed from en_US to a lang that we actually have translations for -- eriksod
        }

        if (!in_array($locale->getLanguage(), $languages)) {
            $locale->setLocale("en_" . $locale->getRegion());
        }

        if ($auth->hasIdentity()) {

            $identity = $auth->getIdentity();

            $l = $identity->language;
            if ($identity->country) {
                $l = $l . '_' . strtoupper($identity->country);
            }

            $locale = new Zend_Locale($l);
        }

        Zend_Registry::set('Zend_Locale', $locale);

        return $locale;
    }

    protected function _initCountDataFilters(array $options = array())
    {
        Core_Model_CountData_Filter_Parts::addPart(new Core_Model_CountData_Filter_Part_CameraName());
        Core_Model_CountData_Filter_Parts::addPart(new Core_Model_CountData_Filter_Part_CountType());
        //Core_Model_CountDataFilters::addFilter(new Core_CountDataFilter_Count());
        //Core_Model_CountDataFilters::addFilter(new Core_CountDataFilter_CountTime());
    }

    protected function _initWarnings()
    {
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');
        $this->bootstrap('Auth');
        $auth = $this->getResource('Auth');

        if ($auth->hasIdentity()) {
            $front->registerPlugin(new Admin_Plugin_WarningsLoader());
        }
    }

    protected function _initZFDebug()
    {
        if ($this->getEnvironment() == 'xdevelopment') {
            $autoloader = Zend_Loader_Autoloader::getInstance();
            $autoloader->registerNamespace('ZFDebug');
            $this->bootstrap('db');
            $db = $this->getResource('db');
            $this->bootstrap('cache');
            $cache = $this->getResource('cache');
            $options = array(
                'plugins' => array('Variables',
                    'Database' => array('adapter' => $db),
                    'File' => array('basePath' => APPLICATION_PATH),
                    'Memory',
                    'Time',
                    'Registry',
                    'Cache' => array('backend' => $cache->getBackend()),
                    'Exception')
            );
            $debug = new ZFDebug_Controller_Plugin_Debug($options);

            $this->bootstrap('FrontController');
            $frontController = $this->getResource('FrontController');
            $frontController->registerPlugin($debug);
        }
    }

    protected function _initTidy()
    {
        $this->bootstrap('Loggers');
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');

        if (class_exists("tidy", false)) {
            $tidyPlugin = new Talking_Controller_Plugin_TidyOutput();
            if (defined('WEBREPORT_DEBUG_TIDY_OUTPUT')) {
                $logger = Zend_Registry::get("Zend_Logger_FireBug");
                $tidyPlugin->setLogger($logger);
            }
            $front->registerPlugin($tidyPlugin);
        }
    }

}
