<?php

// Define application environment
defined('APPLICATION_ENVIRONMENT')
    || define('APPLICATION_ENVIRONMENT',
              (getenv('APPLICATION_ENVIRONMENT') ? getenv('APPLICATION_ENVIRONMENT')
                                         : 'production'));

if (APPLICATION_ENVIRONMENT == 'development') {
//	define('WEBREPORT_DEBUG_TIDY_OUTPUT', true);
}

if ( APPLICATION_ENVIRONMENT == "testing" ) {
    define("APPLICATION_LICENSE_PATH", APPLICATION_PATH."/license.test.ini");
    define("APPLICATION_CONFIG_PATH", APPLICATION_PATH."/config.test.ini");
} else {
    define("APPLICATION_LICENSE_PATH", APPLICATION_PATH."/license.ini");
    define("APPLICATION_CONFIG_PATH", APPLICATION_PATH."/config.ini");
}

set_include_path(implode(PATH_SEPARATOR, array(
    dirname(dirname(__FILE__)) . '/library/php',
    get_include_path(),
)));


/** Zend_Application */
require_once 'Zend/Exception.php';
require_once 'Zend/Application.php';
require_once 'Zend/Config/Exception.php';
require_once 'Zend/Config/Ini.php';
require_once 'Zend/Config/Writer.php';
require_once 'Zend/Config/Writer/FileAbstract.php';
require_once 'Zend/Config/Writer/Ini.php';
require_once 'Zend/Config/Writer/IniNice.php';
require_once 'Zend/Registry.php';
require_once APPLICATION_PATH . '/Bootstrap.php';
require_once APPLICATION_PATH . '/modules/core/models/Mutex.php';

$mutex = new Core_Model_Mutex();
$mutex->init(Core_Model_Mutex::MUTEX_MODIFY_CONFIG);
if($mutex->acquire() == FALSE) throw new WebReport_Exception("Could not require semaphore");
$config = new Zend_Config_Ini(APPLICATION_CONFIG_PATH, APPLICATION_ENVIRONMENT, array('allowModifications' => true));
if($mutex->release() == FALSE) throw new WebReport_Exception("Could not release semaphore");

//Redirect to installation if installation is not completed.
if(!$config->general->installationCompleted) {
    header('Location: installation');
    return;
}


if(!file_exists(APPLICATION_LICENSE_PATH))
{
	$config_license = new Zend_Config(array(), true);
	$config_license->production = array();
	$config_license->development = array();
	$config_license->setExtend('development', 'production');
	
	$config_license->production->registrationCode = $config->general->registrationCode;
	$config_license->production->installationId = $config->general->installationId;
	$config_license->production->activationCode = $config->general->activationCode;
	
	$writer = new Zend_Config_Writer_IniNice();
	$writer->write(APPLICATION_LICENSE_PATH, $config_license); 
} else {
	$config_license = new Zend_Config_Ini(APPLICATION_LICENSE_PATH, APPLICATION_ENVIRONMENT, array('allowModifications' => true));
}

date_default_timezone_set($config->general->defaultTimezone);
ini_set('memory_limit', $config->php->memory_limit);
set_time_limit((int)$config->php->max_execution_time);
$session_path = $config->php->session_save_path;
// Fixes old corrupt paths
$session_path = str_replace(" .","/",$session_path);
session_save_path($session_path);

if(!function_exists("_T")) {
    function _T($s) {
        return $s;
    }
}
if(!function_exists("r_log")) {
    function r_log($obj, $pre=""){
        error_log($pre . " " . print_r($obj,true));
    }
}

Zend_Registry::set('__us', 1000.0*microtime(true));
function tic() {
    Zend_Registry::set('__us', 1000.0*microtime(true));
}
function toc($err = false) {
    $dms = floor((1000.0*microtime(true) - Zend_Registry::get('__us')));
    if($err)
        error_log($dms . "us elapsed");
    return $dms;
}
		
Zend_Registry::set('config', $config);
Zend_Registry::set('config-license', $config_license);
// Create application, bootstrap, and run
$application = new Zend_Application(
    APPLICATION_ENVIRONMENT,
    $config
);
