<?php

include_once 'core/models/AclResources.php';
include_once 'core/models/AclPrivileges.php';
include_once 'core/models/AclPrivilegeGroups.php';
include_once 'core/models/AclPrivilegeGroupAssemblies.php';
include_once 'core/models/AclPrivilegeGroupAssemblyPrivilege.php';


class Admin_AclAdminController extends WebReport_Controller_Action
{

	public function init() {
		parent::init();

		$module = $this->getRequest()->module;
		$controller = $this->getRequest()->controller;
		$action = $this->getRequest()->action;

		$items = array();
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse-resources-privileges"), "text" => 'BROWSE_RESOURCES/PRIVILEGES');
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add-resource"), "text" => 'ADD_RESOURCE');
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add-privilege"), "text" => 'ADD_PRIVILEGE');
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse-resources-privilege-groups"), "text" => 'BROWSE_RESOURCES/PRIVILEGE_GROUPS');
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add-privilege-group"), "text" => 'ADD_PRIVILEGE_GROUP');
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add-privilege-group-assembly"), "text" => 'ADD_PRIVILEGE_GROUP_ASSEMBLY');
                $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "view-json"), "text" => 'VIEW_JSON');
		$this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
	}

	public function indexAction()
	{
		$this->_helper->redirector("browse-resources-privileges");
	}

	public function browseResourcesPrivilegesAction()
	{
		$modulesDb = new Core_Model_Modules();
		$this->view->modules = $modulesDb->fetchAll(null, 'alias');
	}

	public function browseResourcesPrivilegeGroupsAction()
	{
		$modulesDb = new Core_Model_Modules();
		$this->view->modules = $modulesDb->fetchAll(null, 'alias');
	}



	public function addResourceAction()
	{

		$form = $this->_getAddEditResourceForm("edit");


		if ($this->getRequest()->isPost()) {
			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$resourcesDb = new Core_Model_AclResources();

				$resourcesData = array(
					'module_id' => $values['module_id'],
					'text' => $values['text'],
					'controller' => $values['controller']
				);

				$resource = $resourcesDb->createRow($resourcesData);

				$resource->save();

				$this->_helper->redirector('browse-resources-privileges', 'acl-admin', 'admin');
			} else {
				$form->populate($_POST);
			}
		}
		
		$this->view->form = $form;
	}


	public function deleteResourceAction()
	{
		$resourceId = (int)$this->_request->getParam('id');
		$resourcesDb = new Core_Model_AclResources();
		$resource = $resourcesDb->find($resourceId)->current();

		if (!$resource)
			return;

		$resource->delete();
		$this->_helper->redirector('browse-resources-privileges');

	}

	public function addPrivilegeAction()
	{

		$form = $this->_getAddEditPrivilegeForm("add");

		if ($this->getRequest()->isPost()) {
			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$privilegesDb = new Core_Model_AclPrivileges();

				$privilegeData = array(
					'acl_resource_id' => $values['acl_resource_id'],
					//'type' => $values['type'],
					'action' => $values['action'],
					'param1_key' => $values['param1_key'],
					'param1_value' => $values['param1_value'],
					'param2_key' => $values['param2_key'],
					'param2_value' => $values['param2_value'],
					'param3_key' => $values['param3_key'],
					'param3_value' => $values['param3_value']
				);

				$privilege = $privilegesDb->createRow($privilegeData);

				$privilege->save();

				$this->_helper->redirector('browse-resources-privileges');
			} else {
				$form->populate($_POST);
			}
		} else {
			$form->getElement('acl_resource_id')->setValue($this->_getParam("resource_id"));
		}

		$this->view->form = $form;
	}

	public function deletePrivilegeAction()
	{
		$privilegeId = (int)$this->_request->getParam('id');
		$privilegeDb = new Core_Model_AclPrivileges();
		$privilege = $privilegeDb->find($privilegeId)->current();

		if (!$privilege)
			return;

		$privilege->delete();

		$this->_helper->redirector('browse-resources-privileges');

	}

	public function editResourceAction() {
		$form = $this->_getAddEditResourceForm('edit');

		$resourceId = (int)$this->_request->getParam('id');
		$resourcesDb = new Core_Model_AclResources();
		$resource = $resourcesDb->find($resourceId)->current();

		if (!$resource)
			return;

		if ($this->getRequest()->isPost()) {

			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$resource->module_id = $values['module_id'];
				$resource->text = $values['text'];
				$resource->controller = $values['controller'];

				$resource->save();

				$this->_helper->redirector('browse-resources-privileges');
			} else {
				$form->populate($_POST);

			}
		} else {
			$form->populate($resource->toArray());
		}

		$this->view->form = $form;
	}

	public function editPrivilegeAction() {
		$form = $this->_getAddEditPrivilegeForm('edit');

		$privilegeId = (int)$this->_request->getParam('id');
		$privilegeDb = new Core_Model_AclPrivileges();
		$privilege = $privilegeDb->find($privilegeId)->current();

		if (!$privilege)
			return;

		if ($this->getRequest()->isPost()) {

			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$privilege->acl_resource_id = $values['acl_resource_id'];
				$privilege->action = $values['action'];
				$privilege->param1_key = $values['param1_key'];
				$privilege->param1_value = $values['param1_value'];
				$privilege->param2_key = $values['param2_key'];
				$privilege->param2_value = $values['param2_value'];
				$privilege->param3_key = $values['param3_key'];
				$privilege->param3_value = $values['param3_value'];

				$privilege->save();

				$this->_helper->redirector('browse-resources-privileges');
			} else {
				$form->populate($_POST);

			}
		} else {
			$form->populate($privilege->toArray());
		}

		$this->view->form = $form;
	}


	public function addPrivilegeGroupAction()
	{

		$form = $this->_getAddEditPrivilegeGroupForm("add");

		if ($this->getRequest()->isPost()) {
			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$groupDb = new Core_Model_AclPrivilegeGroups();

				$groupData = array(
					'acl_resource_id' => $values['acl_resource_id'],
					'text' => $values['text']
				);

				$group = $groupDb->createRow($groupData);

				$group->save();

				$this->_helper->redirector('browse-resources-privilege-groups');
			} else {
				$form->populate($_POST);

			}
		}

		$this->view->form = $form;
	}

	public function editPrivilegeGroupAction() {
		$form = $this->_getAddEditPrivilegeGroupForm('edit');

		$groupId = (int)$this->_request->getParam('id');
		$groupDb = new Core_Model_AclPrivilegeGroups();
		$group = $groupDb->find($groupId)->current();

		if (!$group)
			return;

		if ($this->getRequest()->isPost()) {

			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$group->acl_resource_id = $values['acl_resource_id'];
				$group->text = $values['text'];


				$group->save();

				$this->_helper->redirector('browse-resources-privilege-groups');
			} else {
				$form->populate($_POST);

			}
		} else {
			$form->populate($group->toArray());
		}

		$this->view->form = $form;
	}

	public function deletePrivilegeGroupAction() 
	{

		$groupId = (int)$this->_request->getParam('id');
		$groupDb = new Core_Model_AclPrivilegeGroups();
		$group = $groupDb->find($groupId)->current();

		if (!$group)
			return;
		
		$group->delete();
		
		$this->_helper->redirector('browse-resources-privilege-groups');

	}

	public function addPrivilegeGroupAssemblyAction()
	{

		$form = $this->_getAddEditPrivilegeGroupAssemblyForm("add");

		if ($this->getRequest()->isPost()) {
			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$assemblyDb = new Core_Model_AclPrivilegeGroupAssemblies();

				$assemblyData = array(
					'acl_privilege_group_id' => $values['acl_privilege_group_id'],
					'text' => $values['text']
				);

				$assembly = $assemblyDb->createRow($assemblyData);

				$assembly->save();
				$assemblyId = $assembly->acl_privilege_group_assembly_id;

				$assemblyPrivilegeDb = new Core_Model_AclPrivilegeGroupAssemblyPrivilege();

				$privilegeIds = $values['acl_privilege_ids'];

				foreach ($privilegeIds as $privilegeId) {
					$assemblyPrivilegeData = array(
						'acl_privilege_group_assembly_id' => $assemblyId,
						'acl_privilege_id' => $privilegeId
					);
					$assemblyPrivilege = $assemblyPrivilegeDb->createRow($assemblyPrivilegeData);
					$assemblyPrivilege->save();
				}


				$this->_helper->redirector('browse-resources-privilege-groups');
			} else {
				$form->populate($_POST);

			}
		}

		$this->view->form = $form;
	}

	public function deletePrivilegeGroupAssemblyAction()
	{

		$assemblyId = (int)$this->_request->getParam('id');
		$assemblyDb = new Core_Model_AclPrivilegeGroupAssemblies();
		$assembly = $assemblyDb->find($assemblyId)->current();

		if (!$assembly)
			return;

		$assembly->delete();
		$this->_helper->redirector('browse-resources-privilege-groups');

	}

	public function editPrivilegeGroupAssemblyAction() {
		$form = $this->_getAddEditPrivilegeGroupAssemblyForm('edit');

		$assemblyId = (int)$this->_request->getParam('id');
		$assemblyDb = new Core_Model_AclPrivilegeGroupAssemblies();
		$assembly = $assemblyDb->find($assemblyId)->current();

		if (!$assembly)
			return;

		if ($this->getRequest()->isPost()) {

			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$assembly->acl_privilege_group_id = $values['acl_privilege_group_id'];
				$assembly->text = $values['text'];

				$assembly->save();

				$assemblyPrivilegeDb = new Core_Model_AclPrivilegeGroupAssemblyPrivilege();

				$select = $assemblyPrivilegeDb->select()->where('acl_privilege_group_assembly_id = ?', $assemblyId);
				$assemblyPriviliges = $assemblyPrivilegeDb->fetchAll($select);
				$privilegeIds = $values['acl_privilege_ids'];
				foreach ($assemblyPriviliges as $assemblyPrivilege) {
					$key = array_search($assemblyPrivilege->acl_privilege_id, $privilegeIds);
					if ($key !== false) {
						unset($privilegeIds[$key]);
					} else {
						$assemblyPrivilege->delete();
					}
				}
				
				foreach ($privilegeIds as $privilegeId) {
					$assemblyPrivilegeData = array(
						'acl_privilege_group_assembly_id' => $assemblyId,
						'acl_privilege_id' => $privilegeId
					);
					$assemblyPrivilege = $assemblyPrivilegeDb->createRow($assemblyPrivilegeData);
					$assemblyPrivilege->save();
				}

				$this->_helper->redirector('browse-resources-privilege-groups');
			} else {
				$values = $_POST;
				
				$form->populate($values);

			}
		} else {
			$assemblyPrivileges = $assembly->findManyToManyRowset("Core_Model_AclPrivileges", "Core_Model_AclPrivilegeGroupAssemblyPrivilege", "PrivilegeGroupAssembly");
			$assemblyArray = $assembly->toArray();
			$assemblyArray['acl_privilege_ids'] = array();

			foreach($assemblyPrivileges as $assemblyPrivilege) {
				array_push($assemblyArray['acl_privilege_ids'], $assemblyPrivilege->acl_privilege_id);
			}
			$form->populate($assemblyArray);
		}

		$this->view->form = $form;
	}

        public function viewJsonAction() {
            	$this->_helper->layout->disableLayout();    //disable layout
		$this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

                $json = Core_Model_AclDbSync::generateJsonFromDb();
                
                echo Zend_Json::prettyPrint($json, array("indent" => " "));
        }

	private function _getAddEditResourceForm($action = 'add') {

		$form = new UniForm_Form();

		$modulesDb = new Core_Model_Modules();

		$modules = $modulesDb->fetchAll();

		$module_id = new UniForm_Form_Element_Select('module_id');
		$module_id->setLabel('Module')->setRequired();;
		
		foreach ($modules as $module) {
			$module_id->addMultiOption($module->module_id, $module->text);
		}



		$text = new UniForm_Form_Element_Text('text');
		$text->setLabel('Resource_text')
		->setAttrib('maxlength', '64')
		->addFilter('StringTrim')
		->setRequired();

		$controller = new UniForm_Form_Element_Text('controller');
		$controller->setLabel('Controller')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim')
			->setRequired();


		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$form->addElements(array($module_id, $text, $controller));
		$form->addDisplayGroup(array('module_id', 'text', 'controller'),'resource-group');

		$form->addElements(array($submit));

		return $form;

	}



	private function _getAddEditPrivilegeForm($action = 'add') {

		$form = new UniForm_Form();

		$resourcesDb = new Core_Model_AclResources();
		$modulesDb = new Core_Model_Modules();
		$modules = $modulesDb->fetchAll();
		$options = array();


		$acl_resource_id = new UniForm_Form_Element_Select('acl_resource_id');
		$acl_resource_id->setLabel('RESOURCE')->setRequired();

		foreach ($modules as $module) {
			$options = array();
			$resources = $resourcesDb->fetchAll($resourcesDb->select()->where('module_id = ?', $module->module_id));
			foreach ($resources as $res) {
				$options[$res->acl_resource_id] = $res->text;
			}

			$acl_resource_id->addMultiOption($module->text, $options);
		}

		
/*

		foreach ($resources as $resource) {
			$acl_resource_id->addMultiOption($resource->acl_resource_id, $resource->text);
		}*/

/*		$type = new UniForm_Form_Element_Select('type');
		$type->setLabel('Type')->setRequired();
		$type->addMultiOption(WebReport_Acl::PRIVILEGE_MODULE_CONTROLLER_ACTION, 'Module <=> Controller <=> Action');
		$type->addMultiOption(WebReport_Acl::PRIVILEGE_MODULE_CONTROLLER_ACTION_PARAM1, 'Module <=> Controller <=> Action <=> Param1');
		$type->addMultiOption(WebReport_Acl::PRIVILEGE_MODULE_CONTROLLER_ACTION_PARAM1_PARAM2, 'Module <=> Controller <=> Action <=> Param1 & Param2');
		$type->addMultiOption(WebReport_Acl::PRIVILEGE_MODULE_CONTROLLER_ACTION_PARAM1_PARAM2_PARAM3, 'Module <=> Controller <=> Action <=> Param1 & Param2 & Param3');
*/
		$action = new UniForm_Form_Element_Text('action');
		$action->setLabel('ACTION')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim')
			->setRequired();

		$param1_key = new UniForm_Form_Element_Text('param1_key');
		$param1_key->setLabel('PARAMETER_1_KEY')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim');

		$param1_value = new UniForm_Form_Element_Text('param1_value');
		$param1_value->setLabel('PARAMETER_1_VALUE')
			->setAttrib('maxlength', '128')
			->addFilter('StringTrim');

		$param2_key = new UniForm_Form_Element_Text('param2_key');
		$param2_key->setLabel('PARAMETER_2_KEY')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim');

		$param2_value = new UniForm_Form_Element_Text('param2_value');
		$param2_value->setLabel('PARAMETER_2_VALUE')
			->setAttrib('maxlength', '128')
			->addFilter('StringTrim');


		$param3_key = new UniForm_Form_Element_Text('param3_key');
		$param3_key->setLabel('PARAMETER_3_KEY')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim');

		$param3_value = new UniForm_Form_Element_Text('param3_value');
		$param3_value->setLabel('PARAMETER_3_VALUE')
			->setAttrib('maxlength', '128')
			->addFilter('StringTrim');

		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$form->addElements(array($acl_resource_id, $action));
		$form->addDisplayGroup(array('acl_resource_id', 'action'),'privilege-group');

		$form->addElements(array($param1_key, $param1_value, $param2_key, $param2_value, $param3_key, $param3_value));
		$form->addDisplayGroup(array('param1_key', 'param1_value', 'param2_key', 'param2_value', 'param3_key', 'param3_value'),'parameters-group',array('legend'=> 'PARAMETERS'));


		$form->addElements(array($submit));

		return $form;

	}




	private function _getAddEditPrivilegeGroupForm($action = 'add') {

		$form = new UniForm_Form();

		$resourcesDb = new Core_Model_AclResources();

		$resources = $resourcesDb->fetchAll();

		$acl_resource_id = new UniForm_Form_Element_Select('acl_resource_id');
		$acl_resource_id->setLabel('Resource')->setRequired();

		foreach ($resources as $resource) {
			$acl_resource_id->addMultiOption($resource->acl_resource_id, $resource->text);
		}

		$text = new UniForm_Form_Element_Text('text');
		$text->setLabel('Translation_index')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim')
			->setDescription("Translation_index_for_Privilege_Group")
			->setRequired();


		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$form->addElements(array($acl_resource_id, $text));
		$form->addDisplayGroup(array('acl_resource_id', 'text'),'privilege-group-group');

		$form->addElements(array($submit));

		return $form;

	}

	private function _getAddEditPrivilegeGroupAssemblyForm($action = 'add') {

		$form = new UniForm_Form();

		$groupsDb = new Core_Model_AclPrivilegeGroups();

		$groups = $groupsDb->fetchAll();

		$acl_privilege_group_id = new UniForm_Form_Element_Select('acl_privilege_group_id');
		$acl_privilege_group_id->setLabel('Privilege_group')->setRequired();

		foreach ($groups as $group) {
			$acl_privilege_group_id->addMultiOption($group->acl_privilege_group_id, $group->text);
		}

		$text = new UniForm_Form_Element_Text('text');
		$text->setLabel('Text')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim')
			->setDescription("Translation_index_for_Privilege_Group_Assembly")
			->setRequired();

		$acl_privilege_ids = new UniForm_Form_Element_MultiCheckbox('acl_privilege_ids');
		$acl_privilege_ids->setLabel('Privileges')->setRequired();

		$privilegesDb = new Core_Model_AclPrivileges();
		$select = $privilegesDb->select()->order('acl_resource_id');
		$privileges = $privilegesDb->fetchAll($select);

		foreach ($privileges as $privilege) {
			$privtext = $privilege->action;
			if ($privilege->param1_key) $privtext .= ' , ' . $privilege->param1_key . ': ' . $privilege->param1_value;
			if ($privilege->param2_key) $privtext .= ' , ' . $privilege->param2_key . ': ' . $privilege->param2_value;
			if ($privilege->param3_key) $privtext .= ' , ' . $privilege->param3_key . ': ' . $privilege->param3_value;
			$acl_privilege_ids->addMultiOption($privilege->acl_privilege_id, $privilege->findParentRow("Core_Model_AclResources","Resource")->controller . ': ' . $privtext);
		}

		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$form->addElements(array($acl_privilege_group_id, $text, $acl_privilege_ids));
		$form->addDisplayGroup(array('acl_privilege_group_id', 'text', 'acl_privilege_ids'),'privilege-group-assembly-group');

		$form->addElements(array($submit));

		return $form;

	}

	private function _translatableStrings()
	{
		$this->_translate->_("BROWSE_RESOURCES/PRIVILEGES");
		$this->_translate->_("ADD_RESOURCE");
		$this->_translate->_("ADD_PRIVILEGE");
		$this->_translate->_("BROWSE_RESOURCES/PRIVILEGE_GROUPS");
		$this->_translate->_("ADD_PRIVILEGE_GROUP");
		$this->_translate->_("ADD_PRIVILEGE_GROUP_ASSEMBLY");
                $this->_translate->_("VIEW_JSON");
	}


}
