<?php

include_once 'core/models/AclResources.php';
include_once 'core/models/AclPrivileges.php';

class Admin_AclController extends WebReport_Controller_Action
{

	public function init() {
		parent::init();

		$module = $this->getRequest()->module;
		$controller = $this->getRequest()->controller;
		$action = $this->getRequest()->action;

		$items = array();
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse-roles"), "text" => 'BROWSE_ROLES');
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add-role"), "text" => 'ADD_ROLE');
		$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
		$this->view->headTitle('ACCESS_CONTROL_LISTS', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
		$this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

	}

	public function browseRolesAction()
	{
	    $db = Zend_Registry::get('db');
		$rolesDb = new Core_Model_AclRoles();
		$companiesDb = new Core_Model_Companies();

//		$select = $rolesDb->select()->order('company_id')->order('name');

		$select = $db->select()->from(array("roles" => $rolesDb->info(Zend_Db_Table::NAME)), array("acl_role_id", "name", "description"))
			->join(array("co" => $companiesDb->info(Zend_Db_Table::NAME)), "roles.company_id = co.company_id", array("company" => "name"))
			->where("co.active = 1");
            

		if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
			$select->where('co.company_id = ?', Zend_Auth::getInstance()->getIdentity()->company_id);
		}
/*

		$identity = Zend_Auth::getInstance()->getIdentity();

		if (!$identity->multi_company) {
			$select->where('company_id = ?', $identity->company_id);
		}
*/
		$this->view->roles = $db->fetchAll($select);

		if ($this->_flashMessenger->hasMessages())
			$this->view->messages = $this->_flashMessenger->getMessages();


	}



	public function addRoleAction()
	{

		$this->view->headTitle('ADD_ROLE', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		$form = new Admin_Form_AddEditRole("add");

		if ($this->getRequest()->isPost()) {
			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$roles = new Core_Model_AclRoles();

				$roleData = array(
					'company_id' => $values['company_id'],
					'name' => $values['name'],
					'description' => $values['description']
				);

				$identity = Zend_Auth::getInstance()->getIdentity();
				if (!$identity->multi_company) {
					$roleData['company_id'] = $identity->company_id;
				}

				$role = $roles->createRow($roleData);

				$role->save();
				$this->_flashMessenger->addMessage($this->_translate->_('ROLE_ADDED.'));
				$editPrivs = $this->view->aclUrl(array("module" => "admin", "controller" => "acl", "action" => "edit-role-privileges"), null);
				if ($editPrivs) {
					$this->_helper->redirector->gotoSimple("edit-role-privileges", "acl", "admin", array("id" => $role->acl_role_id));
				} else {
					$this->_helper->redirector('browse-roles', 'acl', 'admin');
				}
			} else {
				$form->populate($_POST);

			}
		}

		$this->view->form = $form;
	}

	public function editRoleAction() {
		$form = new Admin_Form_AddEditRole('edit');

		$roleId = (int)$this->_request->getParam('id');
		$roles = new Core_Model_AclRoles();
		$role = $roles->find($roleId)->current();

		if (!$role)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_ROLE_WITH_SUCH_ID_EXISTS.'));

		if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $role->company_id)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_ROLE_WITH_SUCH_ID_EXISTS.'));

		$this->view->headTitle('EDIT_ROLE', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		if ($this->getRequest()->isPost()) {

			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$role->name = $values['name'];
				$role->description = $values['description'];

				$role->save();

				$this->_flashMessenger->addMessage($this->_translate->_('ROLE_UPDATED.'));
				$this->_helper->redirector('browse-roles','acl','admin');
			} else {
				$form->populate($_POST);

			}
		} else {
			$form->populate($role->toArray());
		}

		$this->view->form = $form;
	}



	public function editRolePrivilegesAction() {

		$form = new UniForm_Form();

		$roleId = (int)$this->_request->getParam('id');
		$roles = new Core_Model_AclRoles();
		$role = $roles->find($roleId)->current();

		if (!$role)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_ROLE_WITH_SUCH_ID_EXISTS.'));

		if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $role->company_id)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_ROLE_WITH_SUCH_ID_EXISTS.'));

		$this->view->headTitle('EDIT_ROLE_PRIVILEGES', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		$this->view->roleId = $roleId;

		$modules = new Core_Model_Modules();
		$this->view->modules = $modules->fetchAll(null, "alias");

		if ($this->getRequest()->isPost()) {
			$values = $_POST;

			$assemblyIds = array();

			// sort out assembly-ids
			foreach ($values as $key => $value) {
				if (substr($key, 0, 8) == 'assembly') {
					$id = intval(substr($key, 9));
					$assemblyIds[$id] = $value;
				}
			}

			$roleAssemblyDb = new Core_Model_AclRolePrivilegeGroupAssembly();

			foreach ($assemblyIds as $assemblyId => $value) {
				$select = $roleAssemblyDb->select();
				$select = $select->where('acl_role_id = ?', $roleId)->where('acl_privilege_group_assembly_id = ?', $assemblyId);
				$row = $roleAssemblyDb->fetchRow($select);
				if (!$row && $value == 1) {
					$roleAssemblyData = array(
						'acl_privilege_group_assembly_id' => $assemblyId,
						'acl_role_id' => $roleId
					);
					$row = $roleAssemblyDb->createRow($roleAssemblyData);
					$row->save();
				} elseif ($row && $value == 0) {
					$row->delete();
				}

			}

			$this->_helper->redirector('browse-roles', 'acl', 'admin');
		} else {
			$values = array();
			$groupsDb = new Core_Model_AclPrivilegeGroups();

			$assembliesDb = new Core_Model_AclPrivilegeGroupAssemblies();


			// @todo not so fast, solve with join or some cool shit
			$assemblies = $assembliesDb->fetchAll();
			foreach($assemblies as $assembly) {
				$group = $assembly->findParentRow("Core_Model_AclPrivilegeGroups");
				$groupAssembliesDb = new Core_Model_AclRolePrivilegeGroupAssembly();
				$grasSelect = $groupAssembliesDb->select();
				$roleAssembly = $assembly->findDependentRowset($groupAssembliesDb, null, $grasSelect->where("acl_role_id = ?", $roleId));
				$checked = (count($roleAssembly) == 0) ? '0' : '1';

				$values['assembly-' . $assembly->acl_privilege_group_assembly_id] = $checked;
				$groupKey = 'group-' . $group->acl_privilege_group_id;
				if (!array_key_exists($groupKey, $values) || $values[$groupKey] == '0')
					$values['group-' . $group->acl_privilege_group_id] = $checked;
			}

			// add groups that didn't have an assembly group
			$groupsDb = new Core_Model_AclPrivilegeGroups();
			$groups = $groupsDb->fetchAll();
			foreach($groups as $group) {
				if (!array_key_exists('group-' . $group->acl_privilege_group_id, $values)) {
					$values['group-' . $group->acl_privilege_group_id] = 0;
				}
			}

			$this->view->values = $values;
		}

	}

	public function deleteRoleAction()
	{
		$roleId = (int)$this->_request->getParam('id');
		$rolesDb = new Core_Model_AclRoles();
		$role = $rolesDb->find($roleId)->current();

		if (!$role)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_ROLE_WITH_SUCH_ID_EXISTS.'));

		$identity = Zend_Auth::getInstance()->getIdentity();

		if (!$identity->multi_company && $identity->company_id != $role->company_id)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_ROLE_WITH_SUCH_ID_EXISTS.'));

		$usersDb = new Core_Model_Users();
		$users = $usersDb->fetchAll($usersDb->select()->where('company_id = ?', $role->company_id)->where('acl_role_id = ?', $role->acl_role_id)->where('active = 1'));
		if (count($users) == 0) {
			// update all inactive user to random role
			$users = $usersDb->fetchAll($usersDb->select()->where('company_id = ?', $role->company_id)->where('acl_role_id = ?', $role->acl_role_id)->where('active = 0'));
			$usersNotRole = $usersDb->fetchAll($usersDb->select()->where('company_id = ?', $role->company_id)->where('acl_role_id <> ?', $role->acl_role_id)->where('active = 1'));
			$notRole = $usersNotRole->current();
			foreach ($users as $user) {
				$user->acl_role_id = $notRole->acl_role_id;
				$user->save();
			}
			$this->_flashMessenger->addMessage($this->_translate->_('ROLE_DELETED._INACTIVE_USERS_WITH_THIS_ROLE_HAVE_BEEN_UPDATED_TO_USE_A_RANDOM_EXISTING_ROLE.'));
			$role->delete();
		} else {
			$this->_flashMessenger->addMessage(sprintf($this->_translate->_('THERE_ARE_USERS_WHO_USES_THE_ROLE_"%1$s"._CHANGE_ROLES_FOR_THESE_USERS_BEFORE_DELETING_THE_ROLE.'), $role->name));
		}

		$this->_helper->redirector('browse-roles', 'acl', 'admin');

	}

	/**
	 * AJAX FUNCTIONS
	 */

	public function getRolesForCompanyAction()
	{
		$this->_helper->layout->disableLayout();    //disable layout
		$this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

		if (!$this->getRequest()->isPost()) {
			echo "[]";
			return;
		}

		$companyId = (int)$this->_getParam('companyId');
		$identity = Zend_Auth::getInstance()->getIdentity();

		if (!$identity->multi_company && $identity->company_id != $companyId) {
			echo "[]";
			return;
		}

		$aclRolesDb = new Core_Model_AclRoles();
		$aclRoles = $aclRolesDb->fetchAll($aclRolesDb->select()->where('company_id = ?', $companyId)->order("name"));
		$roles = array();
		foreach ($aclRoles as $role) {
			$roles[] = array("id" => $role->acl_role_id, "name" => $role->name);
		}
		echo Zend_Json::encode($roles);
	}


	private function _translatableStrings()
	{

		$this->_translate->_('ABOUT_PAGE');
		$this->_translate->_('HANDLE_ROLES');
		$this->_translate->_('BROWSE_ACTIVE');
		$this->_translate->_('BROWSE_INACTIVE');
		$this->_translate->_('BROWSE');
		$this->_translate->_('VIEW');

		$this->_translate->_('HANDLE_COMPANIES');
		$this->_translate->_('INACTIVATE');
		$this->_translate->_('ACTIVATE');
		$this->_translate->_('DASHBOARD');
		$this->_translate->_('VIEW_DASHBOARD');
		$this->_translate->_('HANDLE_FILTER_GROUPS');
		$this->_translate->_('HANDLE_FORMULAS');
		$this->_translate->_('BROWSE_LOGS');
		$this->_translate->_('DELETE_MESSAGES');
		$this->_translate->_('HANDLE_USERS');
		$this->_translate->_('HANDLE_VIRTUAL_CAMERAS');
		$this->_translate->_('TRUEVIEW_APPLICATIONS');

		$this->_translate->_('ACCESS_CONTROL_LISTS');
		$this->_translate->_('ADD_ROLE');
		$this->_translate->_('EDIT_ROLE');
		$this->_translate->_('EDIT_ROLE_PRIVILEGES');
		
		$this->_translate->_('IMPORT_.CNT_FILES');
		$this->_translate->_('SYSTEM_LOGO');
		


	}



}