<?php

class Admin_CompaniesController extends WebReport_Controller_Action
{

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        // Check if DM is selfhosted or hosted
        $config = Zend_Registry::get('config');
        $superapi_hash = $config->general->channelAdminHash;
        $is_hosted = strlen($superapi_hash)>5 ? 1 : 0; // This minimum length is defined in modules/admin/plugins/AuperApi.php, probably 20. This must ofc be less or equal.
        
        $identity = Zend_Auth::getInstance()->getIdentity();
        if ($identity) {   // No identity when using superapi...
            $items = array();
            if ($identity->multi_company) {

                $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => _T('BROWSE_COMPANIES'), "tooltip" => _T("BROWSE_COMPANIES_TOOLTIP"));

                // Do not show the add company option if we host, then this should be done in partner pages..
                if ($is_hosted === 0)
                    $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add"), "text" => _T('ADD_COMPANY'), "tooltip" => _T("ADD_COMPANY_TOOLTIP"));
                
                $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "inactive"), "text" => _T('BROWSE_INACTIVE_COMPANIES'), "tooltip" => _T("BROWSE_INACTIVE_COMPANIES_TOOLTIP"));
            
            } else {
                $companiesDb = new Core_Model_Companies();
                $companyId = $identity->company_id;
                $company = $companiesDb->find($companyId)->current();

                $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "view", "id" => $companyId), "text" => _T('VIEW_INFO'), "tooltip" => _T("COMPANY_VIEW_INFO_TOOLTIP"));
                $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "edit", "id" => $companyId), "text" => _T('EDIT'), "tooltip" => _T("COMPANY_EDIT_TOOLTIP"));
                $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "logo", "id" => $companyId), "text" => _T('LOGO'), "tooltip" => _T("COMPANY_LOGO_TOOLTIP"));
            }
            //$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
            $this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
            $this->view->headTitle('COMPANIES', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        }
    }


    public function browseAction()
    {

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
            throw new WebReport_Exception("PAGE_NOT_FOUND.");
        }

        $companiesDb = new Core_Model_Companies();
        $this->view->list = Admin_Model_CompanyListGenerator::CreateView($companiesDb->fetchAllActive());

        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();

    }


    public function inactiveAction()
    {

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
            throw new WebReport_Exception("PAGE_NOT_FOUND.");
        }

        $companiesDb = new Core_Model_Companies();
        $this->view->list = Admin_Model_InactiveCompanyListGenerator::CreateView($companiesDb->fetchAllInactive());

        $this->view->headTitle("INACTIVE", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    private function _addCompany($values, $user, $admin=false)
    {
      $feedback = [];
      $companies = new Core_Model_Companies();

      if ($admin) {
        $user = "0";
      }
      
      $config = Zend_Registry::get('config');
      if (isset($config->general->usersCanSetNbrOfChannels) && !$config->general->usersCanSetNbrOfChannels && !$admin) {
        if (isset($values['cameras_allowed']))
          $feedback[$this->_translate->_('YOU_DO_NOT_HAVE_PERMISSION_TO_CHANGE_NUMBER_OF_CHANNELS_FOR_COMPANY')]="ERROR";
        $values['cameras_allowed'] = "0";
      } else if ($values['cameras_allowed'] == '') {
        $values['cameras_allowed'] = Null;
      }
      
      $companyData = array(
        'name' => $values['name'],
        'owner' => 0,
        'url' => $values['url'],
        'email' => $values['email'],
        'cameras_allowed' => $values['cameras_allowed'],
        'distributor_contact' => $values['comment'],
        'distrobutor_name' => $values['distrobutor_name'],
        'integrator_name' => $values['integrator_name'],
        'integrator_contact' => $values['integrator_contact'],
        'end_customer_name' => $values['end_customer_name'],
        'end_customer_contact' => $values['end_customer_contact'],
        'next_invoice' => $values['next_invoice'],
        'comment' => $values['comment'],
        'phone1' => $values['phone1'],
        'phone2' => $values['phone2'],
        'fax' => $values['fax'],
        'address' => $values['address'],
        'city' => $values['city'],
        'state' => $values['state'],
        'zip' => $values['zip'],
        'week_start' => (int)$values['week_start'],
        'date_format' => $values['date_format'],
        'active' => 1,
        'created_by' => $user,
        'created_date' => new Zend_Db_Expr('NOW()'),
        'country' => $values['country']
      );

      if(array_key_exists('weather_enable', $values))
        $companyData['weather_enable'] = $values['weather_enable'];

      $company = $companies->createCompanyAndSave($companyData);

      $channelLogs = new Core_Model_CompanyChannelLogs();
      $entryData = array('company_id' => $company->company_id, 
        'user_id' => $user,
        'channels' => $company->cameras_allowed,
        'time' => date("Y-m-d H:i:s"));
      $entry = $channelLogs->createRow($entryData);
      $entry->save();

      /*
       * TODO:
       * This is done also in InstallationController userAndCompanyAction()
       * and very similar to Folder Creation in FoldersController addAction().
       * Make it utilize same code!
       */
      $folderName = $company->name . '-root';
      $dynamicSalt = CM_User::createSalt();
      $cameraGroupsDb = new Core_Model_CameraGroups();
      $cameraGroupsData = array(
        'name' => $folderName,
        'username' => $company->name . '-root-ccid',
        'password' => "",
        'password_salt' => $dynamicSalt,
        'description' => '',
        'email' => $values['email']
      );
      $cameraGroup = $cameraGroupsDb->createRow($cameraGroupsData);
      $cameraGroupId = $cameraGroup->save();

      $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
      $companyCameraGroupData = array(
        "company_id" => $company->company_id,
        "camera_group_id" => $cameraGroupId
      );
      $companyCameraGroup = $companyCameraGroupDb->createRow($companyCameraGroupData);
      $companyCameraGroup->save();

      $foldersDb = new Core_Model_Folders();
      $folder = $foldersDb->createRow(array(
        'company_id' => $company->company_id,
        'name' => $folderName,
        'description' => '',
        'parent_id' => NULL,
        'in_sync' => FALSE,
        'camera_group_id' => $cameraGroupId));

      $folder->save();

      $document = Core_Model_DocumentFactory::add($cameraGroupId, "camera_group", $folder);

      $feedback[$this->_translate->_('COMPANY_ADDED.')]="INFO";
      $feedback["id"]=$company->company_id;
      return $feedback;
    }

    public function superapiaddAction()
    {
      if (!isset($_POST) || !isset($_POST["admin_auth"])) {
        $this->_helper->redirector('add','companies','admin');
      } else {            
        $form = new Admin_Form_AddEditCompany("add");
        $config = Zend_Registry::get('config');
        $superapi = new Admin_Plugin_SuperApi();
        switch ($superapi->checkAuth($_POST["admin_auth"], $config)) {
        case Admin_Plugin_SuperApi::RET_ERROR:
          $this->_helper->json($superapi->getFullAnswer());
          break;
        case Admin_Plugin_SuperApi::RET_NOT_LOGGED_IN:
          $this->_helper->json($superapi->getFullAnswer());
          break;
        case Admin_Plugin_SuperApi::RET_LOGGED_IN:
          $fields = array_keys($form->getValues());
          if (!in_array('cameras-allowed', $fields))
            array_push($fields, 'cameras_allowed'); // This is only included if userCanSetNbrOfChannels is set.
          unset($_POST["admin_auth"]);
          foreach ($_POST as $key => $value) {
            if (!in_array($key, $fields)) {
              $superapi->answer(false, "Unknown parameter: " . $key);
              break 2;
            }
          }
          // Fill missing values as empty to avoid null-values in sql-query
          foreach ($fields as $key)
            if (!isset($_POST[$key]))
              $_POST[$key] = "";
          if ($form->isValid($_POST)){
            $fb = $this->_addCompany($_POST, 0, true);
            if (isset($fb["id"]))
              $superapi->answer(true,array("company_id"=>$fb["id"]));
            else
              $superapi->answer(false,$fb);
          } else {
            $msg = [];
            foreach($form->getErrors() as $field => $error) {
              if(empty($error)) continue;
              $msg[$field] = $error;
            }
            $superapi->answer(false,$msg);
          }
        }
        $this->_helper->json($superapi->getFullAnswer());
      }
    }

    public function addAction()
    {
        if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
            throw new WebReport_Exception("PAGE_NOT_FOUND.");
        }

        $this->view->headTitle('ADD', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $form = new Admin_Form_AddEditCompany("add");
        if ($this->getRequest()->isPost()) {
            if ($form->isValid($_POST)) {
              $userData = Zend_Auth::getInstance()->getIdentity();
              $fb = $this->_addCompany($form->getValues(), $userData->user_id);
              foreach ($fb as $msg => $type)
                if ($type === "ERROR" || $type === "INFO")
                  $this->_flashMessenger->addMessage($msg);
              $this->_helper->redirector('browse','companies','admin');
            } else {
                $form->populate($_POST);
            }
        }
        $this->view->form = $form;
    }

    private function _editCompany($company, $values, $user_id, $multi_company, $admin=false) {
      $feedback = [];
      if ($admin) {
        $user = "0";
      }

      $company->name = $values['name'];
      $company->owner = $values['owner'];
      $company->url = $values['url'];
      $company->email = $values['email'];
      $company->phone1 = $values['phone1'];
      $company->phone2 = $values['phone2'];
      $company->fax = $values['fax'];
      $company->address = $values['address'];
      $company->city = $values['city'];
      $company->state = $values['state'];
      $company->zip = $values['zip'];
      $company->country = $values['country'];
      $company->week_start = (int)$values['week_start'];
      $company->date_format = $values['date_format'];

      if (isset($values['cameras_allowed']) && $values['cameras_allowed'] != $company->cameras_allowed) {
        $config = Zend_Registry::get('config');
        if (!$admin && isset($config->general->usersCanSetNbrOfChannels) && !$config->general->usersCanSetNbrOfChannels) {
          $feedback[$this->_translate->_('YOU_DO_NOT_HAVE_PERMISSION_TO_CHANGE_NUMBER_OF_CHANNELS_FOR_COMPANY')]="ERROR";
          $values['cameras_allowed'] = $company->cameras_allowed;
        } else if ($values['cameras_allowed'] == '') {
          $values['cameras_allowed'] = Null;
        }
      } else {
        $values['cameras_allowed'] = $company->cameras_allowed;
      }

      if(($admin || $multi_company) && 
        $company->cameras_allowed != $values['cameras_allowed']) {
        $company->cameras_allowed = $values['cameras_allowed'];
        $channelLogs = new Core_Model_CompanyChannelLogs();
        $entryData = array('company_id' => $company->company_id, 
          'user_id' => $user_id,
          'channels' => $company->cameras_allowed,
          'time' => date("Y-m-d H:i:s"));
        $entry = $channelLogs->createRow($entryData);
        $entry->save();
      }
      if($multi_company && Core_Model_ConfigurationsList::getShowCustomCompanyInfo()){
        $company->distributor_name      = $values['distributor_name'];
        $company->distributor_contact    = $values['distributor_contact'];
        $company->integrator_name       = $values['integrator_name'];
        $company->integrator_contact     = $values['integrator_contact'];
        $company->end_customer_name     = $values['end_customer_name'];
        $company->end_customer_contact  = $values['end_customer_contact'];
        $company->next_invoice          = $values['next_invoice'];
        $company->comment               = $values['comment'];
      }
      if($multi_company && array_key_exists('weather_enable', $values))
        $company->weather_enable = $values['weather_enable'];

      $company->save();
      $feedback[$this->_translate->_('COMPANY_UPDATED.')]="INFO";
      $feedback["id"]=$company->company_id;
      return $feedback;
    }

    public function superapieditAction() {
      if (!isset($_POST) || !isset($_POST["admin_auth"])) {
        $this->_helper->redirector('edit','companies','admin');
      } else {            
        $superapi = new Admin_Plugin_SuperApi();
        $config = Zend_Registry::get('config');
        switch ($superapi->checkAuth($_POST["admin_auth"], $config)) {
        case Admin_Plugin_SuperApi::RET_ERROR:
          $this->_helper->json($superapi->getFullAnswer());
          break;
        case Admin_Plugin_SuperApi::RET_NOT_LOGGED_IN:
          $this->_helper->json($superapi->getFullAnswer());
          break;
        case Admin_Plugin_SuperApi::RET_LOGGED_IN:
          $companyId = $_POST["company_id"];
          if (!isset($companyId)) {
            $superapi->answer(false, array("company_id"=>"You have to specify company_id"));
            break;
          }
          $companies = new Core_Model_Companies();
          $company = $companies->find($companyId)->current();
          if (!$company) {
            $superapi->answer(false, "Company does not exist");
            break;
          }

          unset($_POST["admin_auth"]);
          foreach ($_POST as $key => $value) {
            if (!array_key_exists($key, $company->toArray())) {
              $superapi->answer(false, "Unknown parameter: " . $key);
              break;
            }
          }

          // Fill in missing values to make the validation ok
          foreach ($company->toArray() as $key => $value)
            if (!isset($_POST[$key]))
              $_POST[$key] = $value;
          // Assign current name for validation check
          $_POST["current_name"] = $company->name;

          $form = new Admin_Form_AddEditCompany("edit", $company);

          if ($form->isValid($_POST)){
            $fb = $this->_editCompany($company, $_POST, 0, 0, true);
            if (isset($fb["id"]))
              $superapi->answer(true,array("company_id"=>$fb["id"]));
            else
              $superapi->answer(false,$fb);
          } else {
            $msg = [];
            foreach($form->getErrors() as $field => $error) {
              if(empty($error)) continue;
              $msg[$field] = $error;
            }
            $superapi->answer(false,$msg);
          }
        }
        $this->_helper->json($superapi->getFullAnswer());
      }
    }

    public function editAction() {
      $companyId = (int)$this->_request->getParam('id');
      $identity = Zend_Auth::getInstance()->getIdentity();
      if (!$companyId) { $companyId = $identity->company_id; }
      if (!$identity->multi_company && $identity->company_id != $companyId) {
        throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COMPANY_WITH_SUCH_ID_EXISTS.'));
      }
      $companies = new Core_Model_Companies();
      $company = $companies->find($companyId)->current();
      if (!$company)
        throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COMPANY_WITH_SUCH_ID_EXISTS.'));
      $this->view->headTitle(sprintf($this->_translate->_('EDIT_"%1$s"'), $company->name), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

      $form = new Admin_Form_AddEditCompany("edit", $company);
      if ($this->getRequest()->isPost()) {
        if ($form->isValid($_POST)) {
          $values = $form->getValues();
          $fb = $this->_editCompany($company, $values, $identity->user_id, $identity->multi_company);
          foreach ($fb as $msg => $type)
            if ($type === "ERROR" || $type === "INFO")
              $this->_flashMessenger->addMessage($msg);
          if ($identity->multi_company) {
            $this->_helper->redirector('browse','companies','admin');
          } else {
            $this->_helper->redirector('view','companies','admin');
          }
        } else {
          $form->populate($_POST);
        }
      } else {
        $form->populate($company->toArray());
        $form->getElement('current_name')->setValue($company->name);
      }
      $this->view->form = $form;
    }


    public function logoAction() {
        $companyId = (int)$this->_request->getParam('id');

        $identity = Zend_Auth::getInstance()->getIdentity();

        if (!$companyId) { $companyId = $identity->company_id; }

        if (!$identity->multi_company && $identity->company_id != $companyId) {
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COMPANY_WITH_SUCH_ID_EXISTS.'));
        }

        $companies = new Core_Model_Companies();
        $company = $companies->find($companyId)->current();

        if (!$company)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COMPANY_WITH_SUCH_ID_EXISTS.'));

        $this->view->headTitle(sprintf($this->_translate->_('LOGO_FOR_"%1$s"'), $company->name), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);


        $form = new Admin_Form_Company_Logo($company);

        $config = Zend_Registry::get('config');

        $uploadDir = APPLICATION_PATH . "/../public/logos";

        if (!is_writable($uploadDir)) {
            $form->addError($this->_translate->_("THE_LOGO_DIRECTORY_IS_NOT_WRITABLE._FIX_THIS_TO_UPLOAD_LOGOS."));

            foreach ($form->getElements() as $element) {
                $form->removeElement($element->getName());
            }
        }

        if ($this->_request->isPost()) {
            $company->logo_enable = $_POST['logo_enable'];        
            $form->populate(array("logo_enable" => $_POST['logo_enable'])); 

            $upload = new Zend_File_Transfer_Adapter_Http();
            $upload//->addValidator('MimeType', true, 'image')
                ->addValidator('ImageSize', false,
                    array('minwidth' => 0,
                    'maxwidth' => 600,
                    'minheight' => 0,
                    'maxheight' => 600)
                );
            //->setDestination($uploadDir);
            $files = $upload->getFileInfo();


            foreach ($files as $file => $info) {

                if (!$upload->isUploaded($file)) {
                    continue;
                }

                $orgFileName = $upload->getFilename($file); 

                $element = $form->getElement($file);
                $extension = end(explode('.', $orgFileName));
                if (strlen($extension) > 9) {
                    $element->addError(sprintf($this->_translate->_('INVALID_FILE_EXTENSION_%1$s_FOR_FILE_%2$s.'), $extension, $upload->getFileName($file, false)));
                    continue;
                }

                $newFileName = '';
                do {
                    // random file name
                    $newFileName = $uploadDir . "/" . preg_replace("/([0-9])/e","chr((\\1+112))",rand(10000000,99999999)) . "." . $extension;
                } while (file_exists($newFileName));                

                $upload->addFilter('Rename',
                    array('target' => $newFileName,
                    'overwrite' => true), $file);

                if (!$upload->isValid($file)) {
                    foreach ($upload->getMessages() as $error) {
                        $element->addError($error);
                        $element->markAsError();
                        $dec = $element->getDecorator('HtmlTag');
                        $dec->setOption('class', $dec->getOption('class') . " error");
                    }
                    continue;
                }

                try {
                    $upload->receive($file);
                    $this->_flashMessenger->addMessage(sprintf($this->_translate->_('FILE_%1$s_IS_UPLOADED.'), basename($orgFileName)));
                    $company->logo_filename = basename($newFileName);
                    $form->getElement("logo_image")->setImage($this->view->baseUrl("public/logos/" . basename($newFileName)));
                } catch (Zend_File_Transfer_Exception $e) {
                    $form->addError($e->getMessage());
                }
            }
            $company->save();

            //    $this->_helper->redirector('logo');

        } else {
            $values = array();
            $values['logo_enable'] = $company->logo_enable; 
            $form->populate($values);   
        }

        if ($this->_flashMessenger->hasCurrentMessages())
            $this->view->messages = $this->_flashMessenger->getCurrentMessages();


        $this->view->form = $form;
    }


    public function viewAction() {
        $companyId = (int)$this->_request->getParam('id');

        $identity = Zend_Auth::getInstance()->getIdentity();
        if (!$companyId) { $companyId = $identity->company_id; }
        if (!$identity->multi_company && $identity->company_id != $companyId) {
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COMPANY_WITH_SUCH_ID_EXISTS.'));
        }

        $companies = new Core_Model_Companies();
        $company = $companies->find($companyId)->current();

        if (!$company)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COMPANY_WITH_SUCH_ID_EXISTS.'));

        $this->view->headTitle($company->name, 'PREPEND');

        $this->view->name = $company->name;
        $this->view->url = $company->url ? $company->url : '-';
        if ($company->owner) {
            $usersDb = new Core_Model_Users();
            $user = $usersDb->find($company->owner)->current();
            $this->view->owner = $user->forename . ' ' . $user->surname;
        } else {
            $this->view->owner = '-';
        }
        $this->view->channels = is_null($company->cameras_allowed) ? $this->_translate->_('UNLIMITED') : $company->cameras_allowed;
        if ($identity->multi_company && Core_Model_ConfigurationsList::getShowCustomCompanyInfo()) {
            $this->view->distributor_name = $company->distributor_name;
            $this->view->distributor_contact = $company->distributor_contact;
            $this->view->integrator_name = $company->integrator_name;
            $this->view->integrator_contact = $company->integrator_contact;
            $this->view->end_customer_name = $company->end_customer_name;
            $this->view->end_customer_contact = $company->end_customer_contact;
            $this->view->next_invoice = $company->next_invoice;
            $this->view->comment = $company->comment;
        }

        $logoUrl = false;
        if ($company->logo_enable && file_exists(APPLICATION_PATH . "/../public/logos/" . $company->logo_filename)) {
            $logoUrl = $company->logo_filename;
        }
        $this->view->logo = $logoUrl;
        $this->view->has_weather = Reports_Model_WeatherServiceAbstract::WeatherIsEnabled();
        $this->view->weather_enable = $company->weather_enable;

        $this->view->email = $company->email ? $company->email : '-';
        $this->view->phone1 = $company->phone1 ? $company->phone1 : '-';
        $this->view->phone2 = $company->phone2 ? $company->phone2 : '-';
        $this->view->fax = $company->fax ? $company->fax : '-';

        $this->view->address = $company->address ? $company->address : '-';
        $this->view->city = $company->city ? $company->city : '-';
        $this->view->state = $company->state ? $company->state : '-';
        $this->view->zip = $company->zip ? $company->zip : '-';

        $countries = new Talking_Countries();
        $country = $company->country;
        if ($country) {
            foreach ($countries->getContinentCountries() as $countries) {
                if (array_key_exists($country , $countries)) {
                    $country = $countries[$country];
                }
            }
        }
        $this->view->country = $country ? $country : '-';

        $companyChannelLogs = Core_Model_CompanyChannelLogs::getInstance();
        $this->view->channel_log = $companyChannelLogs->getLogsForCompany($company);
    }


    public function inactivateAction() {
      $identity = Zend_Auth::getInstance()->getIdentity();
      if (!$identity->multi_company) {
        throw new WebReport_Exception("PAGE_NOT_FOUND.");
      }

      $companyId = (int)$this->_request->getParam('id');
      if ($companyId == $identity->company_id) {
        $this->_flashMessenger->addMessage($this->_translate->_("YOU_CANT_DELETE_THE_COMPANY_YOU_BELONG_TO."));

      } else {
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($companyId)->current();
        if (!$company)
          return;

        $company->inactivate();

        $this->_flashMessenger->addMessage(sprintf($this->_translate->_('THE_COMPANY_"%1$s"_IS_INACTIVATED.'), $company->name));
      }
      $this->_helper->redirector('browse','companies','admin');
    }

    public function activateAction() {
        if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
            throw new WebReport_Exception("PAGE_NOT_FOUND.");
        }

        $companyId = (int)$this->_request->getParam('id');
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($companyId)->current();
        if (!$company)
            return;

        $company->activate();
        $this->_flashMessenger->addMessage(sprintf($this->_translate->_('THE_COMPANY_"%1$s"_IS_ACTIVATED.'), $company->name));
        $this->_helper->redirector('inactive','companies','admin');
    }

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {
        $this->_translate->_('GENERAL_INFORMATION');
        $this->_translate->_('COMPANY_NAME');
        $this->_translate->_('COMPANY_OWNER');
        $this->_translate->_('NO_COMPANY_OWNER');
        $this->_translate->_('URL');
        $this->_translate->_('CONTACT_INFORMATION');
        $this->_translate->_('E-MAIL');
        $this->_translate->_('PHONE');
        $this->_translate->_('PHONE_2');
        $this->_translate->_('FAX');
        $this->_translate->_('ADDRESS');
        $this->_translate->_('COUNTRY');
        $this->_translate->_('CITY');
        $this->_translate->_('STATE');
        $this->_translate->_('ZIP');
        $this->_translate->_('SELECT_A_COUNTRY.');
        $this->_translate->_('BROWSE_COMPANIES');
        $this->_translate->_('VIEW_INFO');
        $this->_translate->_('ADD_COMPANY');
        $this->_translate->_('VIEW_COMPANY');
        $this->_translate->_('BROWSE_INACTIVE_COMPANIES');

    }

    public function exportCompaniesInfoAction(){
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        if (!(Zend_Auth::getInstance()->getIdentity()->multi_company && Core_Model_ConfigurationsList::getShowCustomCompanyInfo())){
            throw new WebReport_Exception("PAGE_NOT_FOUND.");
        }


        header('Content-Description: File Transfer');
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename=companies-info.csv');
        header('Content-Transfer-Encoding: binary');
        header('Connection: Keep-Alive');
        header('Expires: 0');

        $companyChannelLogs = Core_Model_CompanyChannelLogs::getInstance();
        $companiesDb = Core_Model_Companies::getInstance();
        $select = $companiesDb->select()->order('name ASC');
        $companies = $companiesDb->fetchAll($select);

        echo "company name;next invoice;channels;distributor name;distributor contact;integrator name;integrator contact;end customer name;end customer contact;channels history;comment\n";
        foreach($companies as $c){
            $logs = $companyChannelLogs->getLogsForCompany($c);
            echo $c->name . ";" . $c->next_invoice . ";" . $c->cameras_allowed;
            echo ";" . $c->distributor_name . "; " . $c->distributor_contact;
            echo ";" . $c->integrator_name . "; " . $c->integrator_contact;
            echo ";" . $c->end_customer_name . "; " . $c->end_customer_contact . ";";
            foreach($logs as $i => $l){
                if($i != 0) echo " - ";
                echo substr($l->time, 0, -15) . substr($l->time, 5, -12) . substr($l->time,8,-9) . " (";
                echo is_null($l->channels) ? "unlimited" : $l->channels . " ch)";
            }
            echo ";" . $c->comment . "\n";
        }
    }
}
