<?php

class Admin_LoginController extends WebReport_Controller_Action
{

    public function init()
    {
        parent::init();
        $this->_helper->layout->setLayout('login');
    }

    public function indexAction()
    {

        $user = Zend_Auth::getInstance()->getIdentity();
        if ($user) {
            $this->_helper->_redirector("index", "index", "admin");
            exit;
        }

        $this->view->headTitle('LOGIN', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $form = new Admin_Form_Login();
        $form->addElement('hash', 'request_token', array('salt' => 'and-pepper'));

        $config = Zend_Registry::get('config');
        if(!$config->general->installationCompleted){
            $this->_helper->_redirector("user-and-company", "index", "installation");
        }
        $extraLogo = false;

        if (isset($config->logo) &&
                isset($config->logo->login_logo_enable) &&
                (bool) $config->logo->login_logo_enable &&
                isset($config->logo->login_logo) &&
                file_exists(APPLICATION_PATH . "/../public/logos/" . $config->logo->login_logo)) {
            $extraLogo = $config->logo->login_logo;
        }
        $this->view->logo = $extraLogo;

        if ($this->getRequest()->isPost()) {
            if ($form->isValid($_POST)) {
                $dbAdapter = Zend_Registry::get('db');
                $authAdapter = new Zend_Auth_Adapter_DbTable($dbAdapter);
                $authAdapter
                        ->setTableName('user')
                        ->setIdentityColumn(new Zend_Db_Expr('LOWER(user.email)'))
                        ->setCredentialColumn('password')
                        ->setCredentialTreatment("MD5(CONCAT('" . STATIC_SALT . "', ?, password_salt)) AND active = '1'");

                $values = $form->getValues();
                $email = strtolower($values['email']);
                $password = $values['password'];
                $authAdapter
                        ->setIdentity($email)
                        ->setCredential($password);

                $auth = Zend_Auth::getInstance();

                $result = $auth->authenticate($authAdapter);
                if ($result->isValid()) {
                    // user credential is right and user is not inactive
                    // before letting the user in, check if the company is active as well

                    $data = $authAdapter->getResultRowObject(null, 'password');

                    $users = new Core_Model_Users();

                    $user = $users->find($data->user_id)->current();

                    $companyId = $user->company_id;

                    $companies = new Core_Model_Companies();
                    $company = $companies->find($companyId)->current();
                    if ($company->active) {
                        $user->increaseLoginCount();
                        $user->setLastLogin();
                        $auth->getStorage()->write($data);

                        $this->_helper->loggers('user')->notice("User with e-mail '" . $this->view->escape($user->email) . "' and name '" . $this->view->escape($user->forename) . " " . $this->view->escape($user->surname) . "' logged in.");

                        $lastPg = new Zend_Session_Namespace('history');
                        if( isset ($lastPg->uri)) {
                            //remove base of uri...
                            $uri = substr($lastPg->uri, strlen( Zend_Controller_Front::getInstance()->getBaseUrl() ) );
                            $this->_redirect($uri);
                        }

                        if(!strlen($password)) {
                            $this->_flashMessenger->addMessage($this->_translate->_('YOUR_PASSWORD_WAS_BLANK._PLEASE_SELECT_A_NEW_PASSWORD.'));
                            $this->_helper->redirector('change-password', 'profile', 'admin', array('new' => true));
                        }

                        $this->_helper->redirector('index', 'index', 'admin');


                    } else {
                        $auth->clearIdentity();
                        $this->_helper->loggers('user')->notice("User with e-mail '" . $this->view->escape($email) . "' tried to login but access denied because user company is inactive.");
                        $form->setDescription($this->_translate->_('INVALID_CREDENTIALS_PROVIDED.'));
                    }
//				} else if (count($form->getErrors('request_token')) > 0) {
//					$form->addError('POSSIBLE_CSRF_ATTACK.');
//					$this->_helper->loggers('user')->warn("Possible CSRF attack from user with e-mail '" . $this->view->escape($email) . "'.");
                } else {

                    $this->_helper->loggers('user')->warn("User with e-mail '" . $this->view->escape($email) . "' tried to login but access denied.");
                    $form->addError('INVALID_CREDENTIALS_PROVIDED.');
                }
            } else {
                $form->populate($_POST);
            }
        }

        $this->view->form = $form;
    }

    public function logoutAction()
    {
        $user = Zend_Auth::getInstance()->getIdentity();
        $this->_helper->loggers(WebReport_Log::USER_LOG)->notice("User with e-mail '" . $this->view->escape($user->email) . "' and name '" . $this->view->escape($user->forename) . " " . $this->view->escape($user->surname) . "' logged out.");

        setcookie ("current_folder", "", time() - 3600, "/");
        Zend_Auth::getInstance()->clearIdentity();
        $this->_helper->redirector->gotoRoute(array("module" => "admin", "controller" => "login"), 'login', true);
    }

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings()
    {
        $this->_translate->_('LOGIN');
        $this->_translate->_('LOGOUT');
        $this->_translate->_('INVALID_CREDENTIALS_PROVIDED.');
    }

}
