<?php

class Admin_LogsController extends WebReport_Controller_Action
{

	public function init() {
		parent::init();

		$module = $this->getRequest()->module;
		$controller = $this->getRequest()->controller;
		$action = $this->getRequest()->action;

		$logs = new Core_Model_Logs();
		$logRows = $logs->fetchAllVisible();

		$items = array();
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => _T('MOD_ADMIN_LOG_SUBMENU_BROWSE_LOGS'), "tooltip" => _T("MOD_ADMIN_LOG_SUBMENU_BROWSE_LOGS_TOOLTIP"));

		foreach ($logRows as $log) {
			$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "messages", "log" => $log->alias),"tooltip" => _T("VIEW_LOG_ENTRIES_IN_THE_xxLOG_NAMExx"), "text" => $log->text, "class" => $log->alias);
		}

        $items = WebReport_Module_MenuItems_Abstract::translate($items);
        foreach ($items as &$item) {
            $item['tooltip'] = str_replace('xxLOG_NAMExx', $item['text'], $item['tooltip']);
        }
        $this->view->submenu = $items;
		$this->view->headTitle("LOGS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

	public function browseAction()
	{
		$logs = new Core_Model_Logs();
		$this->view->logs = $logs->fetchAllVisible();
	}

	public function clearMessagesAction()
	{

		$logAlias = (string)$this->_getParam('log');

		$logs = new Core_Model_Logs();
		$log = $logs->fetchRowWithAlias($logAlias);
        if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
            if($log->hidden) throw new WebReport_Exception("Page not found");
			$log->deleteMessages(Zend_Auth::getInstance()->getIdentity()->company_id);
		} else {
			$log->deleteMessages();
		}

		$this->_helper->redirector('browse');

	}

	public function messagesAction()
    {
        $logAlias = (string)$this->_getParam('log');
		$logs = new Core_Model_Logs();
		$log = $logs->fetchRowWithAlias($logAlias);
        
        if( $log->hidden == 1 ){
            $identity = Zend_Auth::getInstance()->getIdentity();
            if (!$identity->multi_company) {
                throw new WebReport_Exception("Page not found");
            }
        }
		
		$page = $this->_getParam('page');
		if (is_null($page)) $page = 1;
		else $page = (int)$page;

		$this->view->headTitle($this->_translate->_(sprintf('%1$s_LOG', strtoupper($log->alias))) . " " . "(" . sprintf($this->_translate->_('PAGE_%1$s'), $page) . ")", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		$this->view->logName = $log->text;
		$this->view->logAlias = $log->alias;
		$logId = $log->log_id;

		$db = Zend_Registry::get('db');

		$logMessages = new Core_Model_LogMessages();
		$companiesDb = new Core_Model_Companies();

		$select = $db->select()
			->from(array('m' => $logMessages->info(Zend_Db_Table::NAME)),
                    array('log_id', 'log_message_id', 'priority_id', 'message', 'company_id', 'date'))
			->joinLeft(array('c' => $companiesDb->info(Zend_Db_Table::NAME)),
                    'm.company_id = c.company_id', array('company' => 'name'))
			->where('log_id = ?', $logId)
			->order('date DESC');

		if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
			$select = $select->where('c.company_id = ?', Zend_Auth::getInstance()->getIdentity()->company_id);
		}


		$messagePaginator = new Zend_Paginator(new Zend_Paginator_Adapter_DbSelect($select));
		$messagePaginator->setCurrentPageNumber($this->_getParam('page'));
		$messagePaginator->setItemCountPerPage(40);

		$this->view->messagePaginator = $messagePaginator;
	}

	public function deleteMessageAction()
	{
		$page = (int)$this->_getParam('page');
		$log = $this->_getParam('log');
		$messageId = (int)$this->_getParam('id');
    
        if( $log->hidden == 1 ){
            $identity = Zend_Auth::getInstance()->getIdentity();
            if (!$identity->multi_company) {
                throw new WebReport_Exception("Page not found");
            }
        }
		
		$messages = new Core_Model_LogMessages();
		$message = $messages->find($messageId)->current();
		if (!$message) {
			throw new WebReport_Exception($this->_translate->_("NO_LOG_MESSAGE_WITH_SUCH_ID."));
		}

		$identity = Zend_Auth::getInstance()->getIdentity();
		if (!$identity->multi_company && $message->company_id != $identity->company_id) {
			throw new WebReport_Exception($this->_translate->_("NO_LOG_MESSAGE_WITH_SUCH_ID."));
		} else {
			$message->delete();
		}

		$this->_helper->redirector->gotoSimple("messages", "logs", "admin", array("log" => $log, "page" => $page));
	}

    public function addMessageAction()
    {
        $logger = $this->_helper->loggers($this->_getParam('logger'));

        $logger->notice($this->_getParam('message'));

        $this->_helper->Json->sendJson(array("status" => "ok"));
    }

	private function _translatableStrings()
	{
		$this->_translate->_('MOD_ADMIN_LOG_SUBMENU_BROWSE_LOGS');
		$this->_translate->_('USER_LOG');
		$this->_translate->_('COUNT_DATA_UPLOAD_LOG');
		$this->_translate->_('EVENT_LISTENER_LOG');
		$this->_translate->_('TRANSLATION_LOG');
		$this->_translate->_('MAIL_LOG');
        $this->_translate->_('SYNCHRONIZATION_LOG');
        $this->_translate->_('NO_COMPANY');
		$this->_translate->_('PAGE_%1$s');
		$this->_translate->_('SYNCHRONIZATION_LOG');
		$this->_translate->_('WEATHER_LOG');
        $this->_translate->_('MAINTENANCE_LOG');
	}

}
