<?php

class Admin_UsersController extends WebReport_Controller_Action
{
    protected $_user;

    public function init()
    {

        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $usersDb = new Core_Model_Users();
        $id = Zend_Auth::getInstance()->getIdentity();
        if (isset($id))
            $this->_user = $usersDb->find($id->user_id)->current();

        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => _T('BROWSE_USERS'), "tooltip" => _T("BROWSE_USERS_TOOLTIP"));
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add"), "text" => _T('ADD_USER'), "tooltip" => _T("ADD_USER_TOOLTIP"));
        //$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "inactive"), "text" => 'BROWSE_INACTIVE_USERS');

        //$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
        $this->view->headTitle("USERS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }


    public function browseAction()
    {
        $companiesDb = new Core_Model_Companies();
        $multi = $this->_user->multi_company;

        if ($multi) {
            $companies = $companiesDb->fetchAllActive();
        } else {
            $companies = $companiesDb->find($this->_user->company_id)->current();
        }
        $users = $companies->fetchAllActiveUsers();
        $this->view->list = Admin_Model_UserListGenerator::CreateView($users);

        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();

    }

    public function inactiveAction()
    {
        $companiesDb = new Core_Model_Companies();

        if ($this->_user->multi_company) {
            $companies = $companiesDb->fetchAllActive();
        } else {
            $companies = $companiesDb->find($this->_user->company_id)->current();
        }
        $users = $companies->fetchAllInactiveUsers();

        $this->view->headTitle("INACTIVE", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $this->view->list = Admin_Model_InactiveUserListGenerator::CreateView($users);

        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function showLegacyOption(){
        try {
            $version = new Zend_Db_Table('database_version');
            $s = $version->select();
            $s = $s->where("version = '2.4.1'");
            $q = $version->fetchAll($s);
            return count($q) > 0;
        } catch(Exception $e){
            throw $e;
            return false;
        }
    }

    private function _addUser($values, $created_by)
    {
        $users = new Core_Model_Users();
        $dynamicSalt = CM_User::createSalt();
        $userDataArray = array(
            'forename' => $values['forename'],
            'surname' => $values['surname'],
            'company_id' => $values['company_id'],
            'acl_role_id' => 1, //This should not be used anymore.
            'email' => $values['email'],
            'language' => $values['language'],
            'country' => $values['country'],
            'password' => md5(STATIC_SALT . $values['password'] . $dynamicSalt),
            'password_salt' => $dynamicSalt,
            'digest_helper_password' => md5($values['email'] . ':' . APPLICATION_REALM . ':' . $values['password']),
            'created_by' => $created_by,
            'created_date' => new Zend_Db_Expr('NOW()'),
            'administrator' => 1, //This refers to the now hidden folder manager, which once was called administrator.
            'multi_company' => $values['multi_company']
        );
        $user = $users->createRow($userDataArray);
        $user->save();
        return $user->user_id;
    }

    public function superapiaddAction() {
        // Check if user has the right to use superapi
        $config = Zend_Registry::get('config');
        $superapi = new Admin_Plugin_SuperApi();
        if (!isset($_POST) || !isset($_POST["admin_auth"])) {
            $this->_helper->redirector('add','users','admin');
        } else {            
            switch ($superapi->checkAuth($_POST["admin_auth"], $config)) {
            case Admin_Plugin_SuperApi::RET_ERROR:
                $this->_helper->json($superapi->getFullAnswer());
                break;
            case Admin_Plugin_SuperApi::RET_NOT_LOGGED_IN:
                $this->_helper->json($superapi->getFullAnswer());
                break;
            case Admin_Plugin_SuperApi::RET_LOGGED_IN:
                unset($_POST["admin_auth"]);
                // Create a fake form to list parameters and test if valid input
                $form = new Admin_Form_AddEditUser("add");
                // Make sure that the parameters submitted are valid parameters
                $fields = array_keys($form->getValues());
                array_push($fields, "multi_company");
                foreach ($_POST as $key => $value) {
                    if (!in_array($key, $fields)) {
                        $superapi->answer(false, "Unknown parameter: " . $key);
                        break 2;
                    }
                }
                // Set verify-password field to password
                $_POST["password_confirm"] = $_POST["password"];
                // Fill missing values as empty to avoid null-values in sql-query
                $cid = $_POST["company_id"];
                $cid_valid = isset($cid);
                $company = (new Core_Model_Companies())->getCompany($cid); 
                $cid_valid = $cid_valid && isset($company);
                $mc = $_POST["multi_company"];
                $mc_valid = isset($mc);
                $mc_valid = $mc_valid && ($mc==0 || $mc==1);
                foreach ($fields as $key)
                    if (!isset($_POST[$key]))
                        $_POST[$key] = "";
                $valid = $form->isValid($_POST);
                if ($valid && $cid_valid && $mc_valid){
                    // TODO: Verify that user 1 exists
                    $id = $this->_addUser($_POST, 1); // Created by id 1 - needs to have a api-user to fullfill constaraint
                    $superapi->answer(true, array("user_id"=>$id));
                } else {
                    $msg = [];
                    foreach($form->getErrors() as $field => $error) {
                        if(empty($error)) continue;
                        $msg[$field] = $error;
                    }
                    if (!isset($cid))
                        $msg["company_id"] = array("Must be specified");
                    elseif (!$cid_valid)
                        $msg["company_id"] = array("Not a valid company ID");
                    if (!isset($mc))
                        $msg["multi_company"] = array("Must be specified");
                    elseif (!$mc_valid)
                        $msg["multi_company"] = array("Must be 1 or 0");
                    $superapi->answer(false,$msg);
                }
            }
            $this->_helper->json($superapi->getFullAnswer());
            die();
        }
    }

    public function addAction()
    {
        $company_id = Zend_Auth::getInstance()->getIdentity()->company_id;

        $form = new Admin_Form_AddEditUser("add",null,$company_id, $this->showLegacyOption());
        $this->view->headTitle("ADD", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        if ($this->getRequest()->isPost()) {
            if ($form->isValid($_POST)) {
                // Get the values from the form
                $values = $form->getValues();
                // Get user identity
                $userData = Zend_Auth::getInstance()->getIdentity();
                // If user is not multi-company, then he can only create users in his own company.
                if(!$userData->multi_company) {
                    unset($values['multi_company']);
                    $values['company_id'] = $userData->company_id;
                }

                $this->_addUser($values, $userData->user_id);
                
                $this->_flashMessenger->addMessage($this->_translate->_('USER_ADDED.'));
                $this->_helper->redirector('browse','users','admin');
            } else {
                $form->populate($_POST);

            }
        }

        $this->view->form = $form;
    }

    public function editAction() {
        $userId = (int)$this->_request->getParam('id');
        $users = new Core_Model_Users();
        $user = $users->find($userId)->current();
        $k = $this->showLegacyOption();
        if (!$user)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $user->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        $form = new Admin_Form_AddEditUser("edit",null,$user->company_id, $this->showLegacyOption());

        $this->view->headTitle(sprintf($this->_translate->_('EDIT_"%1$s"'), $user->forename . ' ' . $user->surname), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        
        $identity = Zend_Auth::getInstance()->getIdentity();
        $multiDisable = ($identity->user_id == $userId || $userId == 1) && $identity->multi_company;
        if(($identity->user_id == $userId ||
          $userId == 1) && $identity->multi_company)
          $form->multi_company->setAttrib('disabled', 'disabled');

        if ($this->getRequest()->isPost()) {

            if ($form->isValid($_POST)) {
                $values = $form->getValues();

                $user->forename = $values['forename'];
                $user->surname = $values['surname'];
                $user->email = $values['email'];
                $user->acl_role_id = 1;
                $user->language = $values['language'];
                $user->country = $values['country'];
                $user->administrator = 1;

                $identity = Zend_Auth::getInstance()->getIdentity();
                if($identity->multi_company){ 
                    if(!$multiDisable)
                        $user->multi_company = $identity->multi_company ? $values['multi_company'] : 0;
                }
				
				$user->save();

                $this->_flashMessenger->addMessage($this->_translate->_('USER_UPDATED.'));

                if ($userId == Zend_Auth::getInstance()->getIdentity()->user_id) {
                    Zend_Auth::getInstance()->clearIdentity();
                }

                $this->_helper->redirector('browse','users','admin');
            } else {
                $form->populate($_POST);
		
            }
        } else {
            $form->populate($user->toArray());
            $form->getElement('current_email')->setValue($user->email);
        }
        if($userId == 1 && false){
            // If the superadmin is not longer admin
            // VERY BAD things will happen.
                        $form->getElement('acl_role_id')->setAttrib('disabled',true);
        }
        $this->view->form = $form;
    }

    public function changePasswordAction() {
        $form = new Admin_Form_ChangePassword();

        $userId = (int)$this->_request->getParam('id');
        $usersDb = new Core_Model_Users();
        $user = $usersDb->fetchRow($usersDb->select()->where('user_id = ?', $userId));
        $multi = Zend_Auth::getInstance()->getIdentity()->multi_company;
        if(!$multi && $user->multi_company)
            throw new WebReport_Exception($this->_translate->_("YOUR_PRIVILEGES_ARE_TOO_LOW_TO_CHANGE_PASSWORD_FOR_THIS_USER."));

        if (!$user)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        if (!$multi && Zend_Auth::getInstance()->getIdentity()->company_id != $user->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        if($this->_request->getParam('method') == 'reset') {
            $user->setPassword('');
            $user->save();

            $this->_flashMessenger->addMessage(sprintf($this->_translate->_('CHANGE_PASSWORD_FOR_"%1$s"'), $user->forename . ' ' . $user->surname));
            $this->_helper->redirector('browse', 'users', 'admin');
       
         }


        $this->view->headTitle(sprintf($this->_translate->_('CHANGE_PASSWORD_FOR_"%1$s"'), $user->forename . ' ' . $user->surname), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        if ($this->getRequest()->isPost()) {

            if ($form->isValid($_POST)) {
                $values = $form->getValues();

                $user->setPassword($values['password']);
                
                $user->save();
                $this->_flashMessenger->addMessage(sprintf($this->_translate->_('PASSWORD_IS_CHANGED_FOR_THE_USER_"%1$s".'), $user->getFullName()));
                $this->_helper->redirector('browse','users','admin');

            }
        }

        $this->view->form = $form;
    }


    public function inactivateAction() {
        $userId = (int)$this->_request->getParam('id');
        $users = new Core_Model_Users();
        $user = $users->find($userId)->current();

        if (!$user)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        $multi = Zend_Auth::getInstance()->getIdentity()->multi_company;
        if(!$multi && $user->multi_company)
            throw new WebReport_Exception($this->_translate->_("YOUR_PRIVILEGES_ARE_TOO_LOW_TO_DELETE_THIS_USER."));

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $user->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        $fullName = $user->getFullName();

        $user->inactivate();

        if ($userId == Zend_Auth::getInstance()->getIdentity()->user_id) {
            Zend_Auth::getInstance()->clearIdentity();
        }


        $this->_flashMessenger->addMessage($this->_translate->_(sprintf($this->_translate->_('THE_USER_"%1$s"_IS_DELETED.'), $fullName)));

        $this->_helper->redirector('browse','users','admin');
    }

    public function activateAction() {
        $userId = (int)$this->_request->getParam('id');
        $users = new Core_Model_Users();
        $user = $users->find($userId)->current();

        if (!$user)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $user->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_USER_WITH_SUCH_ID_EXISTS.'));

        $user->activate();
        $this->_flashMessenger->addMessage($this->_translate->_(sprintf($this->_translate->_('THE_USER_"%1$s"_IS_ACTIVATED.'), $user->getFullName())));
        $this->_helper->redirector('inactive','users','admin');
    }



    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {
        $this->_translate->_('GENERAL_INFORMATION');
        $this->_translate->_('FORENAME');
        $this->_translate->_('SURNAME');
        $this->_translate->_('COMPANY');
        $this->_translate->_('NO_COMPANY');
        $this->_translate->_('E-MAIL');
        $this->_translate->_('PASSWORD');
        $this->_translate->_('PASSWORD');
        $this->_translate->_('CONFIRM_PASSWORD');
        $this->_translate->_('LOCALE');
        $this->_translate->_('LANGUAGE');
        $this->_translate->_('ROLE');
        $this->_translate->_('SELECT_A_COMPANY.');
	$this->_translate->_('MULTI_COMPANY_USER');
	$this->_translate->_('A_MULTI_COMPANY_USER_HAS_GLOBAL_PRIVILEGES._USE_WITH_CAUTION.');
        $this->_translate->_("ENABLE_LEGACY_MODE");
        $this->_translate->_("THIS_ENABLES_DEPRECATED_FEATURES._AVOID_USING_THESE,_SINCE_THEY_WILL_BE_REMOVED_IN_FUTURE_WEB_REPORTS.");
        $this->_translate->_('BROWSE_USERS');
        $this->_translate->_('ADD_USER');
        $this->_translate->_('BROWSE_INACTIVE_USERS');
        $this->_translate->_('THE_PASSWORD_IS_TOO_WEAK._INCLUDE_SOME_SYMBOLS_AND_NUMBERS_AND_TRY_AGAIN.');
        $this->_translate->_('ADVANCED_OPTIONS');
    }

}
