<?php

class Admin_WarningsController extends WebReport_Controller_Action
{

	public function init()
	{
		parent::init();

		$module = $this->getRequest()->module;
		$controller = $this->getRequest()->controller;
		$action = $this->getRequest()->action;

		$items = array();
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => _T('BROWSE_WARNINGS'), "tooltip" => _T("BROWSE_WARNINGS_TOOLTIP"));

		//$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
		$this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
		$this->view->headTitle("WARNINGS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
	}


	public function browseAction()
	{
		$warningsDb = new Core_Model_Warnings();
        // cannot use warnings as view variable as its taken by the warnings in the top list
		// @todo fix this code to do only one db checkup
   	$identity = Zend_Auth::getInstance()->getIdentity();
		
		if ($identity->multi_company) {
			$camerasDb = new Core_Model_Cameras();
			$cameras = $camerasDb->fetchAll();
			$warnings = $warningsDb->fetchAll($warningsDb->select()->where("acknowledged = 0")->order("created_date DESC"));
			$c = array();
			foreach ($cameras as $camera) {
				$c[$camera->camera_id] = $camera;	
			}
			foreach ($warnings as $warning) {
				if ((int) $warning->camera_id > 0) {
					$warning->setCamera($c[$warning->camera_id]);
				}	
			}
		} else {
			$companiesDb = new Core_Model_Companies();
			$company = $companiesDb->find($identity->company_id)->current();
			$warnings = $warningsDb->fetchAllForCompany($company);
		}

		$this->view->warningsList = $warnings;

		if ($this->_flashMessenger->hasMessages())
			$this->view->messages = $this->_flashMessenger->getMessages();
		
	}	
	
	public function optionsAction()
	{
	
		$this->view->headTitle("OPTIONS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		$id = (int)$this->_getParam('id');

		if (!$id) {
			throw new WebReport_Exception($this->view->translate('NO_WARNING_WITH_SUCH_ID_EXISTS.'));
		}

		$warningsDb = new Core_Model_Warnings();
		$camerasDb = new Core_Model_Cameras();
		$cameraGroupsDb = new Core_Model_CameraGroups();
		$companyCameraGroupDb = new Core_Model_CompanyCameraGroup();

		$identity = Zend_Auth::getInstance()->getIdentity();

		$db = $camerasDb->getAdapter();
		$select = $db->select();
		$select->from(array("cams" => $camerasDb->info('name')), array("camera_id", "name", "ip", "application_id", "count_data_table_id", "report_warnings"))
				->join(array("cg" => $cameraGroupsDb->info('name')), "cams.camera_group_id = cg.camera_group_id", array("cameragroup" => "name"))
				->join(array("ccg" => $companyCameraGroupDb->info('name')), "ccg.camera_group_id = cg.camera_group_id", array())
				->join(array("w" => $warningsDb->info('name')), "w.camera_id = cams.camera_id", array())
				->where("w.warning_id = ?", $id);

		if (!$identity->multi_company) {
			$select = $select->where("ccg.company_id = ?", $identity->company_id);
		}

		$camera = $db->fetchRow($select);

		if (!$camera) {
			throw new WebReport_Exception($this->view->translate('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));
		}

		$camera = $camerasDb->find($camera->camera_id)->current();
		$this->view->cameraName = $camera->name;	
		$form = new Admin_Form_WarningOptions();

		if ($this->getRequest()->isPost()) {
			if ($form->isValid($_POST)) {
				$values = $form->getValues();

				$camera->report_warnings =  $values['report_warnings'];
				$camera->save();

				$this->_helper->redirector('browse','warnings','admin');
			} else {
				$form->populate($_POST);
			}
		} else {
			$form->populate(array("report_warnings" => $camera->report_warnings));	
		}

	
		$this->view->form = $form;	
	}


	public function acknowledgeAction()
	{
		if ($this->_request->isXmlHttpRequest()) {
			$this->_helper->layout()->disableLayout();  
			$this->_helper->viewRenderer->setNoRender();  	
		}	
		
   	    $identity = Zend_Auth::getInstance()->getIdentity();

		$warningsDb = new Core_Model_Warnings();
		
		$id = $this->_getParam('id', 0);		

		if ((int)$id <= 0 && $id != "all") {
			if ($this->_request->isXmlHttpRequest()) {
				echo Zend_Json::encode(array("success" => false));
				return;
			} else {
				$this->_flashMessenger->addMessage($this->_translate->_('AN_UNKNOWN_ERROR_OCCURED.'));
				$this->_helper->redirector('browse','warnings','admin');	
			}
		} 
		
		$warnings = array();				
		if ($identity->multi_company) {
			if ($id == "all") {
				$warnings = $warningsDb->fetchAll();
			} else {
				$warnings = $warningsDb->find($id);
			}
		} else {
			$companiesDb = new Core_Model_Companies();
			$company = $companiesDb->find($identity->company_id)->current();
			$ws = $warningsDb->fetchAllForCompany($company);
			if ($id == "all") {
				$warnings = $ws;	
			} else {
				foreach ($ws as $w) {
					if ($w->warning_id == $id) {
						$warnings[] = $w;	
						break;
					} 
				}	
			}
		}

		foreach ($warnings as $warning) {
			$warning->acknowledged = 1;
			$warning->save();	
		}

		if ($this->_request->isXmlHttpRequest()) {
			echo Zend_Json::encode(array("success" => true));
		} else {
			$this->_flashMessenger->addMessage($this->_translate->_('WARNING_ACKNOWLEDGED.'));
			$this->_helper->redirector('browse','warnings','admin');
		}
	}


	/**
	 * Function to catch translatable strings in Poedit.
	 *
	 * @return void
	 */
	private function _translatableStrings() {
		$this->_translate->_('BROWSE_WARNINGS');
		$this->_translate->_('ACKNOWLEDGE');
		$this->_translate->_('SAVE');
		$this->_translate->_('HANDLE');
		$this->_translate->_('ACKNOWLEDGE_ALL');
		$this->_translate->_('REPORT_WARNINGS_FOR_THIS_CAMERA');
		$this->_translate->_('UNCHECK_TO_DISABLE_REPORTING_OF_WARNINGS_FOR_THIS_CAMERA.');
		
	}
	
}
