<?php

class Admin_Form_AddCountDataSetStep2 extends UniForm_Form
{

	protected $_application;
	protected $_user;

	protected $_dataAvailable = false;

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($application, Core_Model_User $user, $options = null)
	{
		$this->_application = $application;
		$this->_user = $user;
		parent::__construct($options);
	}

	public function init()
	{
		parent::init();

		$application = $this->_application;

		$applicationsModel = new Core_Model_Applications();

		$camerasDb = new Core_Model_Cameras();
		$formulaSetsDb = new Core_Model_CountData_FormulaSets();
		$virtualCamerasDb = new Core_Model_VirtualCameras();

		$camera_id = new UniForm_Form_Element_MultiCheckbox('camera_id');
		$camera_id->setLabel('CAMERAS');

		$includeCamera = false;
		$cameras = $camerasDb->fetchAllVisibleNames($this->_user, $application);
		foreach ($cameras as $cameraId => $cameraName) {
			$camera_id->addMultiOption($cameraId, $cameraName . " (" . $application->getShortName() . ")");
			$includeCamera = true;
			$this->_dataAvailable = true;
		}
		if ($includeCamera) {
			$this->addElements(array($camera_id));
		}

		$include = false;
		$virtual_camera_id = new UniForm_Form_Element_MultiCheckbox('virtual_camera_id');
		$virtual_camera_id->setLabel('VIRTUAL_CAMERAS');
		$virtualCameras = $virtualCamerasDb->fetchAllVisibleNames($this->_user, $application);
		foreach ($virtualCameras as $cameraId => $cameraName) {
			$include = true;
			$virtual_camera_id->addMultiOption($cameraId, $cameraName . " (" . $application->getShortName() . ")");
		}

		if ($include) {
			$this->addElements(array($virtual_camera_id));
			$this->addDisplayGroup(array('camera_id', 'virtual_camera_id'), 'camera-group',array("legend" => "CAMERAS"));
		} else if ($includeCamera) {
			$this->addDisplayGroup(array('camera_id'), 'camera-group',array("legend" => "CAMERAS"));
		}

		$count_data_formula_set_id = new UniForm_Form_Element_MultiCheckbox('count_data_formula_set_id');
		$count_data_formula_set_id->setLabel('FORMULA_SETS');
		$include = false;
		$formulaSets = $formulaSetsDb->fetchAllVisibleNames($this->_user, $application);
		foreach ($formulaSets as $formulaSetId => $formulaSetName) {
			$include = true;
			$count_data_formula_set_id->addMultiOption($formulaSetId, $formulaSetName . " (" . $application->getShortName() . ")");
		}
		if ($include) {
			$this->addElements(array($count_data_formula_set_id));
			$this->addDisplayGroup(array('count_data_formula_set_id'), 'formula-set-group', array("legend" => "FORMULAS"));
		}


		$step = new UniForm_Form_Element_Hidden('step');
		$step->setValue(2);

		if ($this->dataAvailable()) {

			$submit = new UniForm_Form_Element_Submit('submit');
			$submit->setLabel('SUBMIT');
			$this->addElements(array($step));

			$this->addElements(array($submit));
		}
	}

	/**
	 * Returns true if there are any data available in the form.
	 *
	 * @return bool
	 */
	public function dataAvailable()
	{
		return $this->_dataAvailable;
	}

	/**
	 * Function to catch translatable strings in Poedit.
	 *
	 * @return void
	 */
	private function _translatableStrings() {
		$this->_translate->_('APPLICATION');
		$this->_translate->_('FILTER_GROUP');
		$this->_translate->_('FORMAT');
		$this->_translate->_('NO_COMPANY');
		$this->_translate->_('NO_FILTER_GROUP');
		$this->_translate->_('NO_APPLICATION');
		$this->_translate->_('CSV');
		$this->_translate->_('EXCEL');
		$this->_translate->_('HTML');
		$this->_translate->_('XML');
	}

}