<?php

class Admin_Form_AddEditCameraGroup extends UniForm_Form
{

	protected $_action;
	protected $_cameraGroup;

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param string $action Can be add or edit
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($action = "add", $cameraGroup = null, $options = null)
	{
		$this->_action = $action;
		$this->_cameraGroup = $cameraGroup;
		parent::__construct($options);
	}

	public function init()
	{
		parent::init();

		$this->addElementPrefixPath('CM_Validate_User', 'CM/Validate/User/', 'validate');
		$this->addElementPrefixPath('WebReport_Validate_User', 'WebReport/Validate/User/', 'validate');

		$action = $this->_action;

		$name = new UniForm_Form_Element_Text('name');
		$name->setLabel('CAMERA_GROUP_NAME')
			->setRequired()
			->addFilter('StringTrim');

		$this->addElement($name);

		if ($action == "add") {
			$usernameValidator = new Zend_Validate_Db_NoRecordExists('camera_group', 'username');
		} else { // edit
			if (!$this->_cameraGroup) {
				throw new WebReport_Exception("Camera group id must be set.");
			}
			$usernameValidator = new Zend_Validate_Db_NoRecordExists('camera_group', 'username', array('field' => 'camera_group_id', 'value' => $this->_cameraGroup->camera_group_id));
		}

		$username = new UniForm_Form_Element_Text('username');
		$username->setLabel('USER')
			->setRequired()
            ->setAttrib("autocomplete", "off")
			->addFilter('StringTrim')
			->addValidator($usernameValidator)
			->setErrorMessages(array("USERNAME_MUST_BE_UNIQUE_AND_NON_EMPTY."))
		        ->setDescription("USERNAME_AND_PASSWORD_DOES_NOT_NEED_TO_BE_THE_SAME_AS_YOUR_ACCOUNT_CREDENTIALS.");


		$password = new UniForm_Form_Element_Password('password');
		$password->setLabel('USER_PASSWORD')
			->addValidator('PasswordConfirmation')
			->setRequired();

		$password_confirm = new UniForm_Form_Element_Password('password_confirm');
		$password_confirm->setLabel('CONFIRM_PASSWORD')
			->setRequired();


		$this->addElements(array($username));
		$fieldsetElements = array('name', 'username');

		if ($action == 'add') {
			$this->addElements(array($password, $password_confirm));
			array_push($fieldsetElements, 'password','password_confirm');
		}

		array_push($fieldsetElements, 'description');
		$no_data_timeout = new UniForm_Form_Element_Text('no_data_timeout');
        $no_data_timeout->setLabel(_T("HOURS_WITHOUT_COUNT_DATA_ALLOWED"))
            ->addValidator("Int")
            ->addValidator(new Zend_Validate_GreaterThan(0))
            ->setDescription(_T("A_WARNING_WILL_BE_ISSUED_IF_YOUR_COUNTER_NO_LONGER_SENDS_NEW_COUNTING_DATA_TO_DataManager._THIS_SETTING_IS_THE_NUMBER_OF_HOURS_ALLOWED_TO_PASS_WITHOUT_SENDING_A_WARNING."))
            ->setValue("48");
		$no_connection_timeout = new UniForm_Form_Element_Text('no_connection_timeout');
        $no_connection_timeout->setLabel(_T("HOURS_WITHOUT_COMMUNICATION_ALLOWED"))
            ->addValidator("Int")
            ->addValidator(new Zend_Validate_GreaterThan(0))
            ->setDescription(_T("A_WARNING_WILL_BE_ISSUED_IF_YOUR_COUNTER_NO_LONGER_IS_ABLE_TO_CONTACT_DataManager._THIS_SETTING_IS_THE_NUMBER_OF_HOURS_ALLOWED_TO_PASS_WITHOUT_SENDING_A_WARNING."))
            ->setValue("8");
		$email = new UniForm_Form_Element_Text('email');
        $email->setLabel(_T("EMAIL_FOR_WARNINGS"))
            ->setDescription(_T("IF_YOU_WANT_WARNINGS_SENT_AS_EMAILS,_WRITE_THE RECEIVER_ADDRESS_HERE._THIS_REQUIRES_SMTP_TO_BE_CONFIGURED_BY_THE_SERVER_ADMINISTRATOR."));
        $this->addElement($no_data_timeout);
        $this->addElement($no_connection_timeout);
        $this->addElement($email);
        $fieldsetElements[] = 'no_connection_timeout';
        $fieldsetElements[] = 'no_data_timeout';
        $fieldsetElements[] = 'email';

		$description = new UniForm_Form_Element_Textarea('description');
		$description->setLabel('DESCRIPTION')
			->addFilter('StringTrim');

		$this->addElement($description);

		$this->addDisplayGroup($fieldsetElements, 'user-group');

		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$this->addElements(array($submit));
	}

}
