<?php

class Admin_Form_AddEditCompany extends UniForm_Form
{

	protected $_action;
	protected $_company;

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($action = "add", Core_Model_Company $company = null, $options = null)
	{
		$this->_action = $action;

		if ($company && !($company instanceof Core_Model_Company) ) {
			throw new WebReport_Exception('Parameter $company must be an instance of Core_Model_Company');
		}

		$this->_company = $company;

		parent::__construct($options);
	}

	public function init()
	{
		parent::init();
		$action = $this->_action;

		$this->addElementPrefixPath('WebReport_Validate_Company', 'WebReport/Validate/Company/', 'validate');

		$name = new UniForm_Form_Element_Text('name');
		$name->setLabel('COMPANY_NAME')
			->setAttrib('maxlength', '255')
			->addFilter('StringTrim')
			->addValidator('UniqueName')
			->setRequired();

		if ($action == 'edit') {
			$current_name = new UniForm_Form_Element_Hidden('current_name');
		} else {
			$current_name = null;
		}

		$url = new UniForm_Form_Element_Text('url');
		$url->setLabel('URL')
			->setAttrib('maxlength', '255');

		if ($action == 'edit') {
			$users = new Core_Model_Users();
			$companiesDb = new Core_Model_Companies();

			$company = $this->_company;
			$companyUsers = $company->fetchAllActiveUsers();

			$owner = new UniForm_Form_Element_Select('owner');
			$owner->setLabel('COMPANY_OWNER');
			$owner->addMultiOption(0, "NO_COMPANY_OWNER");
			foreach ($companyUsers as $user) {
				$owner->addMultiOption($user->user_id, $user->forename . ' ' . $user->surname . ' (' . $user->email . ')');
			}
		} else {
			$owner = null; // to prevent error msgs
		}

        $config = Zend_Registry::get('config');
        $setChannelsAllowed = (!isset($config->general->usersCanSetNbrOfChannels) || $config->general->usersCanSetNbrOfChannels);
        if(Zend_Auth::getInstance()->getIdentity()->multi_company && $setChannelsAllowed) {
            $cameras_allowed = new UniForm_Form_Element_Text('cameras_allowed');
            $cameras_allowed->setLabel('CHANNELS')
			    ->addValidator("Int")
			    ->addValidator(new Zend_Validate_GreaterThan(-1));
        }

        if(Zend_Auth::getInstance()->getIdentity()->multi_company &&
            Core_Model_ConfigurationsList::getShowCustomCompanyInfo()) {
            $distributor_name       = new UniForm_Form_Element_Text('distributor_name');
            $distributor_name       ->setLabel("DISTRIBUTOR_NAME")
                                    ->setAttrib("maxlength", 255);
            $distributor_contact    = new UniForm_Form_Element_Text('distributor_contact');
            $distributor_contact    ->setLabel("DISTRIBUTOR_CONTACT")
                                    ->setAttrib("maxlength", 255);
            $integrator_name        = new UniForm_Form_Element_Text('integrator_name');
            $integrator_name        ->setLabel("INTEGRATOR_NAME")
                                    ->setAttrib("maxlength", 255);
            $integrator_contact     = new UniForm_Form_Element_Text('integrator_contact');
            $integrator_contact     ->setLabel("INTEGRATOR_CONTACT")
                                    ->setAttrib("maxlength", 255);
            $end_customer_name      = new UniForm_Form_Element_Text('end_customer_name');
            $end_customer_name      ->setLabel("END_CUSTOMER_NAME")
                                    ->setAttrib("maxlength", 255);
            $end_customer_contact   = new UniForm_Form_Element_Text('end_customer_contact');
            $end_customer_contact   ->setLabel("END_CUSTOMER_CONTACT")
                                    ->setAttrib("maxlength", 255);
            $next_invoice           = new UniForm_Form_Element_Text('next_invoice');
            $next_invoice           ->setLabel("NEXT_INVOICE")
                                    ->setAttrib("maxlength", 255);
            $comment                = new UniForm_Form_Element_Textarea('comment');
            $comment                ->setLabel('COMMENTS')
                                    ->setAttrib('maxlength', '999');
        
            $custom_elements = array($distributor_name, $distributor_contact, $integrator_name, $integrator_contact, $end_customer_name, $end_customer_contact, $next_invoice, $comment);
            $custom_disp = array('distributor_name', 'distributor_contact', 'integrator_name', 'integrator_contact', 'end_customer_name', 'end_customer_contact', 'next_invoice', 'comment');
        }

        $email = new UniForm_Form_Element_Text('email');
		$email->setLabel('E-MAIL')
			->setRequired(true)
			->setAttrib('maxlength', '255')
			->addValidator('EmailAddress');

		$phone1 = new UniForm_Form_Element_Text('phone1');
		$phone1->setLabel('PHONE')
			->setAttrib('maxlength', '255');

		$phone2 = new UniForm_Form_Element_Text('phone2');
		$phone2->setLabel('PHONE_2')
			->setAttrib('maxlength', '255');

		$fax = new UniForm_Form_Element_Text('fax');
		$fax->setLabel('FAX')
			->setAttrib('maxlength', '255');

		$address = new UniForm_Form_Element_Textarea('address');
		$address->setLabel('ADDRESS')
			->setAttrib('rows', '2');

		$city = new UniForm_Form_Element_Text('city');
		$city->setLabel('CITY')
			->setAttrib('maxlength', '255');

		$state = new UniForm_Form_Element_Text('state');
		$state->setLabel('STATE')
			->setAttrib('maxlength', '255');

		$zip = new UniForm_Form_Element_Text('zip');
		$zip->setLabel('ZIP')
			->setAttrib('maxlength', '10');

        $country = new UniForm_Form_Element_Select('country');
        $country->setLabel('COUNTRY');

        $countries = new Core_Model_Countries();
        //$continents = $countries->getContinents();
        //$continentCountries = $countries->getContinentCountries();

		$country->addMultiOption("", "SELECT_A_COUNTRY.");
        foreach ($countries->getCountries() as $key => $value) {
			$country->addMultiOption($key, $value);
        }
/*
        $week_start = new UniForm_Form_Element_Select('week_start');
        $week_start->setLabel(_T('FIRST_DAY_OF_WEEK'))
            ->setDescription(_T('THIS_WILL_IMPACT_WHICH_DAYS_WILL_BE_SHOWN_FOR_A_WEEK_IN_CHARTS,_REPORTS_AND_EXPORTS.'))
            ->addMultiOption(0, _T("GUESS_FROM_COUNTRY_AND_LANGUAGE._(NOT_RECOMMENDED)"))
            ->addMultiOption(1, _T("MONDAY"))
            ->addMultiOption(7, _T("SUNDAY"))
            ->setValue(1);
        $date_format = new UniForm_Form_Element_Select('date_format');
        $date_format->setLabel(_T('DATE_FORMAT'))
            ->addMultiOption("auto", _T("GUESS_FROM_COUNTRY_AND_LANGUAGE._(NOT_RECOMMENDED)"))
            ->addMultiOption("dd/MM/yyyy", ("DD/MM/YYYY"))
            ->addMultiOption("yyyy/MM/dd", ("YYYY/MM/DD"))
            ->addMultiOption("yyyy/dd/MM", ("YYYY/DD/MM"))
            ->addMultiOption("MM/dd/yyyy", ("MM/DD/YYYY"))
            ->addMultiOption("yyyy-MM-dd", ("YYYY-MM-DD"))
            ->addMultiOption("dd-MM-yyyy", ("DD-MM-YYYY"))
            ->setValue("yyyy-MM-dd");
            */
        /*
        $time_format = new UniForm_Form_Element_Select('date_format');
        $time_format->setLabel(_T('TIME_FORMAT'))
                    ->addMultiOption(_T('AM/PM'))
                    ->addMultiOption(_T('24-HOUR_CLOCK'));
         */

		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$general_elements = array($name, $current_name, $url, $owner, $cameras_allowed);
		$general_disp = array('name','current_name','url','company_id', 'owner', 'cameras_allowed');

		$this->addElements($general_elements);
		$this->addDisplayGroup($general_disp, 'info-group', array('legend' => 'GENERAL_INFORMATION'));

        if(Zend_Auth::getInstance()->getIdentity() && Zend_Auth::getInstance()->getIdentity()->multi_company &&
            Core_Model_ConfigurationsList::getShowCustomCompanyInfo()) {
            $this->addElements($custom_elements);
            $this->addDisplayGroup($custom_disp, 'custom-group', array('legend' => 'CUSTOM_INFORMATION'));
        }

        $this->addElements(array($email, $phone1, $phone2, $fax));
		$this->addDisplayGroup(array('email','phone1','phone2','fax'), 'contact-group',array('legend' => 'CONTACT_INFORMATION'));

		$this->addElements(array($address, $city, $state, $zip, $country, $week_start, $date_format, $time_format));
		$this->addDisplayGroup(array('address','city','state','zip','country', 'week_start','date_format', 'time_format'), 'address-group',array('legend' => 'ADDRESS'));

		$this->addElements(array($submit));
	}

}

