<?php

class Admin_Form_AddEditRole extends UniForm_Form
{

	protected $_action;

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param string $action Can be add or edit
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($action = "add", $options = null)
	{
		$this->_action = $action;
		parent::__construct($options);
	}


	public function init()
	{
		parent::init();

		$identity = Zend_Auth::getInstance()->getIdentity();

		$name = new UniForm_Form_Element_Text('name');
		$name->setLabel('NAME')
			->setAttrib('maxlength', '64')
			->addFilter('StringTrim')
			->setRequired(true);

		if ($this->_action == "add") {

			if ($identity->multi_company) {

				$companiesDb = new Core_Model_Companies();
				$activeCompanies = $companiesDb->fetchAllActive();
				$company_id = new UniForm_Form_Element_Select('company_id');
				$company_id->setLabel('COMPANY')
					->setRequired()
					->addValidator(new Zend_Validate_GreaterThan(0)) // check if user belongs to company
					->setErrorMessages(array($this->getView()->translate("USER_MUST_BELONG_TO_A_COMPANY.")));

				$company_id->addMultiOption(0, "SELECT_A_COMPANY.");
				foreach ($activeCompanies as $company) {
					$company_id->addMultiOption($company->company_id, $company->name);
				}
			} else {
				$company_id = new UniForm_Form_Element_Hidden('company_id');
				$company_id->setValue(Zend_Auth::getInstance()->getIdentity()->company_id);
			}
		} else {
			$company_id = new UniForm_Form_Element_Hidden('company_id');
			$company_id->setValue(Zend_Auth::getInstance()->getIdentity()->company_id);
		}

		$description = new UniForm_Form_Element_Textarea('description');
		$description->setLabel('DESCRIPTION')
			->addFilter('StringTrim');

		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		

		$this->addElements(array($name, $company_id, $description));
		$this->addDisplayGroup(array('name', 'company_id', 'description'),'role-group');

		$this->addElements(array($submit));


	}

}
