<?php

class Admin_Form_AddEditUser extends UniForm_Form
{

  protected $_action;
  protected $_company_id;
  /**
   * Constructor
   *
   * @see Zend_Form
   * @param string $action Can be add or edit
   * @param mixed $options
   * @return void
   */
  public function __construct($action = "add", $options = null, $company_id = null)
  {
    $this->_company_id = $company_id;
    $this->_action = $action;
    parent::__construct($options);
  }

  public function init()
  {
    parent::init();

    $action = $this->_action;

    $this->addElementPrefixPath('CM_Validate_User', 'CM/Validate/User/', 'validate');
    $this->addElementPrefixPath('WebReport_Validate_User', 'WebReport/Validate/User/', 'validate');

    $forename = new UniForm_Form_Element_Text('forename');
    $forename->setLabel('FORENAME')
      ->setRequired()
      ->addFilter('StringTrim');

    $surname = new UniForm_Form_Element_Text('surname');
    $surname->setLabel('SURNAME')
      ->setRequired()
      ->addFilter('StringTrim');

    $identity = Zend_Auth::getInstance()->getIdentity();

    if(isset($identity) && $identity->multi_company) {
      $multi_company = new UniForm_Form_Element_Checkbox('multi_company');
      $multi_company->setLabel("MULTI_COMPANY_USER")
        ->setDescription("A_MULTI_COMPANY_USER_HAS_GLOBAL_PRIVILEGES._USE_WITH_CAUTION.");
    }

    if ($action == "add") {

      if (isset($identity) && $identity->multi_company) {

        $companiesDb = new Core_Model_Companies();
        $activeCompanies = $companiesDb->fetchAllActive();
        $company_id = new UniForm_Form_Element_Select('company_id');
        $company_id->setLabel('COMPANY')
          ->setRequired()
          ->addValidator(new Zend_Validate_GreaterThan(0)) // check if user belongs to company
          ->setErrorMessages(array($this->getView()->translate("USER_MUST_BELONG_TO_A_COMPANY.")));

        $company_id->addMultiOption(0, "NO_COMPANY");
        foreach ($activeCompanies as $company) {
          $company_id->addMultiOption($company->company_id, $company->name);
        }
      } else {
          $company_id = new UniForm_Form_Element_Hidden('company_id');
          $id = Zend_Auth::getInstance()->getIdentity();
          if (isset($id))
              $company_id->setValue($id->company_id);
      }

    }

    $email = new UniForm_Form_Element_Text('email');
    $email->setLabel('E-MAIL')
      ->setRequired()
      ->addValidator('EmailAddress')
      ->addValidator('UniqueEmailAddress');

    if ($action == 'edit') {
      $current_email = new UniForm_Form_Element_Hidden('current_email');
    } else {
      $current_email = null;
    }

    if ($action == 'add') {
      $password = new UniForm_Form_Element_Password('password');
      $password->setLabel('PASSWORD')
        ->addValidator('PasswordStrength',true)
        ->addValidator('PasswordConfirmation')
        ->setRequired();

      $password_confirm = new UniForm_Form_Element_Password('password_confirm');
      $password_confirm->setLabel('CONFIRM_PASSWORD')
        ->setRequired();
    } else {
      $password = null;
      $password_confirm = null;
    }

    $locale = Zend_Registry::get('Zend_Locale');

    /* Temporary commented until we support all languages again.
    $language = new UniForm_Form_Element_Select('language');
    $language->setLabel('LANGUAGE')
      ->setRequired();
    $availableLanguages = Zend_Registry::get('languages');
    $languages = array();
    foreach ($availableLanguages as $avLang) {
      $languages[$avLang] = $locale->getTranslation($avLang, 'Language', $locale);
    }

    asort($languages);

    foreach ($languages as $langId => $lang) {
      $language->addMultiOption($langId, $lang);
    }
    */

    //Remove following two row once we support all languages again.
    $language = new UniForm_Form_Element_Hidden('language');
    $language->setValue("en");


    $submit = new UniForm_Form_Element_Submit('submit');
    $submit->setLabel('SUBMIT');

    $this->addElements(array($forename, $surname, $email, $current_email));
    if (isset($company_id)) $this->addElement($company_id);
    if (isset($acl_role_id)) $this->addElement($acl_role_id);
    $this->addDisplayGroup(array('forename','surname','company_id', 'acl_role_id', 'email', 'current_email'), 'info-group', array('legend' => 'GENERAL_INFORMATION'));

    if ($action == 'add') {
      $this->addElements(array($password, $password_confirm));
      $this->addDisplayGroup(array('password','password_confirm'), 'password-group',array('legend' => 'PASSWORD'));
    }


    $country = new UniForm_Form_Element_Select('country');
    $country->setLabel('COUNTRY');

    $countries = new Core_Model_Countries();
    //    $continents = $countries->getContinents();
    //$continentCountries = $countries->getContinentCountries();

    $country->addMultiOption("", "SELECT_A_COUNTRY.");
    foreach ($countries->getCountries() as $key => $value) {
      $country->addMultiOption($key, $value);
    }

    $this->addElements(array($language, $country));
    $this->addDisplayGroup(array('language', 'country'), 'locale-group',array('legend' => 'LOCALE'));

    if(isset($identity) && $identity->multi_company) {
      $advanced_options = array();
      $this->addElement($multi_company);
      $advanced_options[] = 'multi_company';
      $this->addDisplayGroup($advanced_options, 'advanced-group',array('legend' => 'ADVANCED_OPTIONS'));
    }

    $this->addElements(array($submit));

  }

}
