<?php

class Admin_Form_AddFilterGroup extends UniForm_Form
{

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($options = null)
	{
		parent::__construct($options);
	}

	public function init()
	{
		parent::init();

		$action = $this->action;

		$step = new UniForm_Form_Element_Hidden('step');
		$step->setValue(1);

		$name = new UniForm_Form_Element_Text('name');
		$name->setLabel('NAME')
			->setRequired()
			->addFilter('StringTrim');

		if (Zend_Auth::getInstance()->getIdentity()->multi_company) {
			$companies = new Core_Model_Companies();
			$activeCompanies = $companies->fetchAllActive();
			$company_id = new UniForm_Form_Element_Select('company_id');
			$company_id->setRequired()
				->addValidator(new Zend_Validate_GreaterThan(0)) // check if user belongs to company
				->setErrorMessages(array($this->getView()->translate("FILTER_GROUP_MUST_BELONG_TO_A_COMPANY.")));
			$company_id->setLabel('COMPANY');
			$company_id->addMultiOption(0, "SELECT_A_COMPANY_FOR_THE_FILTER_GROUP.");
			foreach ($activeCompanies as $company) {
				$company_id->addMultiOption($company->company_id, $company->name);
			}

		} else {
			$company_id = new UniForm_Form_Element_Hidden('company_id');
			$company_id->setValue(Zend_Auth::getInstance()->getIdentity()->company_id);
		}

		$applicationsModel = new Core_Model_Applications();
		$applications = $applicationsModel->fetchAllWithCountData();

		$application_id = new UniForm_Form_Element_Select('application_id');
		$application_id->setLabel('TRUEVIEW_APPLICATION')
			->setRequired()
			->addValidator(new Zend_Validate_GreaterThan(0)) // check if user belongs to company
			->setErrorMessages(array($this->getView()->translate("YOU_MUST_CHOOSE_ONE_OF_THE_TRUEVIEW_APPLICATIONS.")));

		$application_id->addMultiOption(0, "SELECT_AN_APPLICATION_FOR_THE_FILTER_GROUP.");
		foreach ($applications as $app) {
			$application_id->addMultiOption($app->getId(), $app->getName());
		}

		$description = new UniForm_Form_Element_Textarea('description');
		$description->setLabel('DESCRIPTION');

		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('NEXT_STEP');

		$this->addElements(array($step,$name, $company_id, $application_id, $description));
		$this->addDisplayGroup(array('step','name','company_id','application_id','description'),'filter-info');

		$this->addElements(array($submit));

	}

}