<?php
 
class Admin_Plugin_WarningsLoader extends Zend_Controller_Plugin_Abstract
{

    public static function tryCreateWarning($warnType, $camera, $group) {
        $warningsDb = Core_Model_Warnings::getInstance();
        $warning = $warningsDb->fetchRow(
          $camera?
          $warningsDb->select()
          ->where("warning = ?", $warnType)
          ->where("camera_id = ?", $camera->camera_id)
          ->where("warning = ?", $warnType)
          ->where("acknowledged = 0"):
          $warningsDb->select()
          ->where("warning = ?", $warnType)
          ->where("acknowledged = 0")
        );
        if (!$warning) {
            $warning = $warningsDb->createRow();
            $warning->camera_id = $camera?$camera->camera_id:null;
            $warning->created_date = time();
            $warning->notification_sent = $camera?0:1;
            $warning->warning = $warnType;
            $warning->save();
        }
        if($warning->notification_sent == 0){
            $data_time = time() - strtotime($camera->last_date . " UTC");
            $contact_time = time() - strtotime($camera->last_contact .  " UTC");
            if($warnType == Core_Model_Warning::CAMERA_NO_DATA){
                $text = "The camera '" . $camera->name . "' has no new data. Latest data is " . floor($data_time/60/60) . " hours old.";
                $shortText = "No data from camera";
            } else if($warnType == Core_Model_Warning::CAMERA_NO_CONNECTION){
                $text = "The camera '" . $camera->name . "' has not contacted Web Report for " . floor($contact_time/60/60) . " hours.";
                $shortText = "No contact with camera";
            } else if($warnType == Core_Model_Warning::CAMERA_EXPIRES_SOON){
                $status = cm_get_camera_status($camera, true);
                if($status['trial']) {
                    $text ="The trial period for the camera '" . $camera->name . "' expires soon. You will need to activate it and register your Web Report to continue receiving data.";
                    $shortText = "The trial period for the camera is about to expire."; 
                } else {
                    $text = "The subscription for the camera '" . $camera->name . "' will expire in " . $camera->getDaysUntilExpire() . " days.";
                    $shortText = "The camera will soon expire.";
                }
            } else if ($warnType == Core_Model_Warning::CAMERA_NO_CHANNELS_LEFT){
                $text = "Camera '" . $camera->name - "' can not report its data because the number of cameras connected exceeds the number of channels allowed.";
                $shortText = "Number of cameras exceeds the allowed channels";
            } else {
                throw new Exception("Unexpected warning triggered.");
            }
            if(isset($group) && $group->email && strlen($group->email) > 3){
                $camera->sendWarningByEmail($shortText, $text);
                echo "Sending email to " . $group->email . " about camera " . $camera->name . ". ($shortText)\r\n";
            }
            $warning->notification_sent = 1;
            $warning->save();
        }
    }

    public function checkCameras($force = false){
        $config = Zend_Registry::get('config');

        $current = time();
        if ($force||$current - $config->warnings->lastCheck > $config->warnings->interval) {
            $warningsDb = Core_Model_Warnings::getInstance();
            $features = new Core_Model_Features();
            try {
                $writer = new Zend_Config_Writer_IniNice();
                $writer->setConfig($config)
                    ->setFilename(APPLICATION_PATH . '/config.ini');
                $config->warnings->lastCheck = time();
                $writer->write(); // throws exception if not ok

                $camerasDb = Core_Model_Cameras::getInstance();
                // We want to prevent large server from spending alot of time here,
                // so we limit it to 500 random cameras. Should not be a problem since
                // this is run every 10th minute. So it is fairly unlikely that we miss
                // a single camera for multiple hours.
                $random_cams = $camerasDb->select()
                    ->where('report_warnings = 1')
                    ->order('RAND()')
                    ->limit(500);
                $cameras = $camerasDb->fetchAll($random_cams);
                foreach ($cameras as $camera) {
                    if(!$camera->report_warnings || $camera->minutesSinceFirstContact() < 60)
                        continue;
                    
                    $data_time = time() - strtotime($camera->last_date. " UTC");
                    $contact_time = time() - strtotime($camera->last_contact . " UTC");
                    $group = $camera->findParentRow('Core_Model_CameraGroups');
                    if($group == NULL) continue;
                    $no_data = $data_time >= $group->no_data_timeout*3600;
                    $no_connection =  $contact_time >= $group->no_connection_timeout*3600;
                    $expireDays = $camera->getDaysUntilExpire();
                    if( $expireDays && (int)$expireDays == 6 && $features->dynamicChannels() ) {
                        self::tryCreateWarning(Core_Model_Warning::CAMERA_EXPIRES_SOON, $camera, $group);
                    }

                    if($no_data) {
                        self::tryCreateWarning(Core_Model_Warning::CAMERA_NO_DATA, $camera, $group);
                    }

                    if($no_connection) {
                        self::tryCreateWarning(Core_Model_Warning::CAMERA_NO_CONNECTION, $camera, $group);
                    }
                }

            } catch (Zend_Config_Exception $e) {
                $select = $warningsDb->select()->where("warning = ?", Core_Model_Warning::CONFIG_NOT_WRITABLE);
                $warning = $warningsDb->fetchAll($select);
                if (count($warning) > 0) {
                    $warning = $warning->current();
                    if ($warning->acknowledged == 1) {
                        $warning->acknowledged = 0;
                        $warning->created_date = time();
                        $warning->save();
                    }
                } else {
                    $warning = $warningsDb->createRow();
                    $warning->warning = Core_Model_Warning::CONFIG_NOT_WRITABLE;
                    $warning->created_date = time();
                    $warning->save();
                }
            }
        }
    }

	public function preDispatch(Zend_Controller_Request_Abstract $request)
	{
		//$this->checkCameras();
        //$this->keepPollAlive();

		$view = Zend_Controller_Action_HelperBroker::getStaticHelper('ViewRenderer')->view;
		if (!$view->url(array("module" => "admin", "controller" => "warnings", "action" => "browse"), null, 'default')) {
			$view->warnings = array();	
			return;
		}

       
		// @todo fix this code to do only one db checkup
       	$identity = Zend_Auth::getInstance()->getIdentity();
        $warningsDb = new Core_Model_Warnings();
		
		if ($identity->multi_company) {
			$camerasDb = new Core_Model_Cameras();
			$cameras = $camerasDb->fetchAll();
			$warnings = $warningsDb->fetchAll($warningsDb->select()->where("acknowledged = 0")->order("created_date DESC")->limit(10));
			$c = array();
			foreach ($cameras as $camera) {
				$c[$camera->camera_id] = $camera;	
			}
			foreach ($warnings as $warning) {
				if ((int)$warning->camera_id > 0) {
					$warning->setCamera($c[$warning->camera_id]);
				}	
			}
		} else {
			$companiesDb = new Core_Model_Companies();
			$company = $companiesDb->find($identity->company_id)->current();
			$warnings = $warningsDb->fetchAllForCompany($company);
		}

      $w = array();
      foreach ($warnings as $warning) {
      	$w[$warning->warning_id] = $warning;
      }

		$view->warnings = $w;
					
	}
}

