<?php
class Charts_ChartProducerController extends WebReport_Controller_Action {

    protected $_validDataTypes = array("application", "custom");
    protected $_user, $_group;

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();

        $this->_user = Core_Model_Users::getCurrentUser();

        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_PRODUCED_CHARTS');

        if($this->_group && $this->_group->canEdit($this->_user)) {
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "create", "folder" => $groupId, "mode" => "simple"), "text" => _T('CREATE_CHART'));
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "create", "folder" => $groupId, "mode" => "advanced"), "text" => _T('CREATE_CHART_(ADVANCED_MODE)'));
        }
        
        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

        $this->view->headTitle("CHART_CREATOR", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $this->view->type = $this->_getParam('type');
        $this->view->id = $this->_getParam('id');
    }

    public function browseAction()
    {
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
            $this->_user, $this->_group, $this->_helper, $this->_request,
            'Charts_Model_ChartDataList',
            'Charts_Model_ChartListGenerator');
        
        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function viewAction()
    {
        $chartId = (int)$this->_request->getParam('id');

        if($chartId) {
            $document = Core_Model_DocumentFactory::fetch($chartId, "chart");
            if(!$document->canView($this->_user)) {
                $this->_flashMessenger->addMessage($this->_translate->_(sprintf($this->_translate->_('THIS_CHART_DOES_NOT_EXIST,_OR_IS_NOT_VISIBLE_TO_YOU.'), $chart->title)));
                $this->_helper->redirector('browse','chart-producer','charts');
                return;
            }
            $chartdata = Charts_Model_ChartData::factoryFromDatabase($chartId);
        }

        $render = new Charts_Model_ChartRender_AmChart();
        $render->setChartData($chartdata);

        $this->view->chartId = $chartId;
        $this->view->exportForm = new Charts_Form_ExportChart('view');
        $this->view->title = $chartdata->getTitle();
        $this->view->description = $chartdata->getDescription();
        $this->view->render = $render;
    }

    public function deleteAction(){
        $chartId = (int)$this->_request->getParam('id');
        $chartDb = new Charts_Model_ChartDataList();
        $chart = $chartDb->find($chartId)->current();

        $document = Core_Model_DocumentFactory::fetch($chartId, "chart");

        if (!$chart || !$document->canEdit($this->_user))
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CHART_WITH_SUCH_ID_EXISTS.'));

        // make sure the chart is not used in a report.
        $cellMapper = new Reports_Model_Report_Page_CellMapper();
        $cells = $cellMapper->fetchAll();

        $do_delete = true;
        foreach ($cells as $cell){
            if ($cell instanceof Reports_Model_Report_Page_Cell_Datasource && $cell->getDataSourceId() == $chartId && $do_delete) {
                $this->_flashMessenger->addMessage($this->_translate->_('CHART_WAS_NOT_REMOVED_BECAUSE_ITS_USED_IN_A_REPORT.'));
                
                $do_delete = false;
            }
        }
        if($do_delete) {
            $title = $chart->title;
            Core_Model_DocumentFactory::remove($chart->chart_data_id, 'chart');
            $chart->delete();
            $this->_flashMessenger->addMessage($this->_translate->_(sprintf($this->_translate->_('THE_CHART_"%1$s"_HAS_BEEN_REMOVED.'),$title )));
        }

        $this->_helper->redirector('browse','chart-producer','charts', 
                                   array('folder' => $document->group_id));
    }

    public function saveAction(){
        if(!$this->getRequest()->isPost()){
            echo "[]";
            return;
        }
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering
        $goodFormat = $this->massagePostArrays($_POST);
        $chartData = Charts_Model_ChartData::factoryFromArray($goodFormat);
        $userDb = new Core_Model_Users();
        $companyDb = new Core_Model_Companies();
        $info = array();
        parse_str($_POST['info'],$info);
        $company = $companyDb->find($this->_group->company_id)->current();

        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector('browse','chart-producer','charts');
            return;
        }
        if($this->_request->getParam('saveAs') == 'true') {
            $chartDataId = $chartData->storeToDatabase($this->_user, $company, NULL, FALSE);
        } else {
            $chartDataId = $chartData->storeToDatabase($this->_user,$company,$_POST['chart_id'] > 0 ? $_POST['chart_id'] : NULL);
        }
        Core_Model_DocumentFactory::add($chartDataId, "chart", $this->_group);
    }

    public function copyAction(){
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        $chartId = (int)($this->_request->getParam('id'));
        $chart = Charts_Model_ChartData::factoryFromDatabase($chartId);
        $document = Core_Model_DocumentFactory::fetch($chartId, "chart");

        $chartDb = new Charts_Model_ChartDataList();

        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_(sprintf($this->_translate->_('THIS_CHART_DOES_NOT_EXIST,_OR_IS_NOT_VISIBLE_TO_YOU.'), $chart->title)));
            $this->_helper->redirector('browse','chart-producer','charts');
            return;
        }

        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector('browse','chart-producer','charts');
            return;
        }

        $user = Core_Model_Users::getUser($chart->getUserId());
        $company = Core_Model_Companies::getCompany($chart->getCompanyId());
        $chart->setTitle($chartDb->findNextCopyName($chartId));

        $chartDataId = $chart->storeToDatabase($user, $company, NULL, FALSE);
        $document = Core_Model_DocumentFactory::add($chartDataId, "chart", $this->_group);

        $this->_helper->redirector('browse','chart-producer','charts', 
                                   array('folder' => $document->group_id));
    }

    public function refreshViewAction(){
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering
        try{
            if (!$this->getRequest()->isPost()) {
                echo "[]";
                return;
            }

            Core_Model_User::SetDefaultDateFormat($this->_group->company_id);
            $goodFormat = $this->massagePostArrays($_POST);
            $chartdata = Charts_Model_ChartData::factoryFromArray($goodFormat);
            $render = new Charts_Model_ChartRender_AmChart();
            $render->useCache();

            $render->renderToOutput($chartdata, $this->getResponse());

        }catch(Exception $e){ error_log($e);}
        return;
    }

    public function massagePostArrays($a){

        $good = array();
        parse_str($a['global'], $good);
        if(array_key_exists('info',$a)){
            $info = array();
            parse_str($a['info'],$info);
            $good['description'] = $info['description'];
        }
        $cnt = intval($a['cnt']);
        $data_sources = array();
        for($i = 0; $i < $cnt;$i++){
            $tmp = array();
            parse_str($a["s".$i],$tmp);
            // Fix time period
            if($tmp['time_period_code'] == 'man'){
                $tmp['time_period_code'] = $tmp['time_period_manual'];
            }

            $tmp['color'] = trim($tmp['color']);
            $tmp['virtual_camera_ids'] = array();
            $tmp['virtual_camera_id_types'] = array();
            $tmp['camera_ids'] = array();
            $tmp['camera_id_types'] = array();
            $tmp['formula_ids'] = array();
            $tmp['formula_id_types'] = array();
            $tmp['data_set_ids'] = array();
            $tmp['data_set_id_types'] = array();
            $tmp['pos_data_ids'] = array();
            $tmp['pos_data_id_types'] = array();
            $tmp['data'] = array();
            if(!isset($tmp['chosen_data'])) break;
            foreach($tmp['chosen_data'] as $str){
                $split = explode(",",$str,4);
                $chosen = new stdClass();
                $chosen->id           = (int)$split[0];
                $chosen->dataTypeId   = $split[1];
                $chosen->identifier   = $split[2];
                
                $tmp['data'][] = $chosen;
            }
            if($tmp['title'] == $this->_translate->_('NEW_DATA_SOURCE_CLICK_HERE_TO_EDIT'))
                $tmp['title'] = $this->_translate->_('NEW_DATA_SOURCE');
            unset($tmp['chosen_data']);
            $data_sources[] = $tmp;
        }
        $good['data_sources'] = $data_sources;

        /*
           $good['data_aggregator'] = Charts_Model_DataAggregatorList::getAggregatorById(intval($good['data_aggregator']));
           if(!isset($good['data_aggregator'])) unset ($good['data_aggregator']);
         */
        return $good;
    }

    public function createAction()
    {
        $chartId = (int)$this->_request->getParam('id');
        $this->view->simple = $this->_request->getParam('mode') == 'simple';

        Core_Model_User::SetDefaultDateFormat($this->_group->company_id);
        if($chartId){
            $document = Core_Model_DocumentFactory::fetch($chartId, "chart");
            $chartDb = new Charts_Model_ChartDataList();
            $chart = $chartDb->find($chartId)->current();

            if(!$document->canEdit($this->_user)) {
                $this->_flashMessenger->addMessage($this->_translate->_(sprintf($this->_translate->_('THIS_CHART_DOES_NOT_EXIST,_OR_IS_NOT_VISIBLE_TO_YOU.'), $chart->title)));
                $this->_helper->redirector('browse','chart-producer','charts', array('folder' => $this->_group->group_id));
                return;
            }

            $chartdata = Charts_Model_ChartData::factoryFromDatabase($chartId);
            if(!$chartdata->isVisible($this->_user)) {
                $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_CHART_BECAUSE_IT_CONTAINS_CAMERAS_THAT_ARE_NOT_VISIBLE_TO_YOU.'));
                $this->_helper->redirector('browse','chart-producer','charts', array('folder' => $this->_group->group_id));
                return;
            }

            $company_id = $chart->company_id;
            $description = $chart->description;
        }else{
            $chartDb = new Charts_Model_ChartDataList();
            $chartdata = new Charts_Model_ChartData();
            $description = '';
        }
        // The first data source should always be a template for new data sources.
        // This is removed on the client side.
        $this->view->dataSourceForms = array();
        $this->view->dataSourceForms[] = Charts_Form_DataSource::factoryDefault();
        $pop = $chartdata->toArray();
        $this->view->saveForm = new Charts_Form_SaveChart();
        $this->view->saveForm->populate(array('description' => $description));

        $this->view->saveAsForm = new Charts_Form_SaveChart();
        $this->view->saveAsForm->populate(array('description' => $description));

        $this->view->exportForm = new Charts_Form_ExportChart('create');
        $this->view->globalSettingsForm = Charts_Form_GlobalSettings::factoryFromArray($pop);
        $this->view->chartId = $chartId;
        $this->view->yes = $this->_translate->_("Yes");
        $this->view->no = $this->_translate->_("No");
        $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();
        $this->view->dateFormat = strtolower($dateFormat);
        $this->view->dateFormat = str_replace("yyyy", "yy", $this->view->dateFormat);
        $this->view->dateFormat = $this->view->dateFormat ? $this->view->dateFormat : "yy-mm-dd";
        $now = new Zend_Date();
        $this->view->baseDate = $now->toString($dateFormat);

        if($this->view->simple && !$chartId) $this->view->globalSettingsForm->getElement('flags')->setValue("simple;");
        
        foreach($pop['data_sources'] as $source) {
            $this->view->dataSourceForms[] = Charts_Form_DataSource::factoryFromArray($source);
        }
    }

    public function exportAction() {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        $width = $this->_request->getParam('width');
        $height = $this->_request->getParam('height');
        //should handle to be posted to AND id/4/type/csv //for example
        $chartId = (int) $this->_request->getParam('id');
        error_log("TODO: Check rights for chart before exporting");

        $permalink = $this->_request->getParam('permalink') == 'true' ? true : false;

        if($permalink) {
            $permaDb = new Charts_Model_Permalinks();
            $format = $this->_request->getParam('format');
            $embed = $this->_request->getParam('embed') == "true";
            $filter = $permaDb->addChartPermalink($format, $chartId, $embed);

            $this->view->fullAddress = $filter->getApi($this->view);
            $this->render("api");

            return;
        }

        if (!$this->getRequest()->isPost() || $chartId ) {
            $type = $this->_request->getParam('format');

            try {
                $chartdata = Charts_Model_ChartData::factoryFromDatabase($chartId);
            } catch (WebReport_Exception $e) {
                echo "No such id\n";
                echo $e->getMessage();
                return;
            }
            try {
                if (!is_null($type) && $type != "") {
                    $render = Charts_Model_ChartRenders::getRender($type);
                } else {
                    $render = Charts_Model_ChartRenders::getRender();
                }
            } catch (WebReport_Exception $e) {
                echo "Invalid renderer\n";
                echo $e->getMessage();
                Zend_Debug::dump($e);
                return;
            }
        } else {
            $goodFormat = $this->massagePostArrays($_POST);

            $chartdata = Charts_Model_ChartData::factoryFromArray($goodFormat);
            Core_Model_User::SetDefaultDateFormat($this->_group->company_id);

            if (isset($_POST['format'])) {
                $render = Charts_Model_ChartRenders::getRender($_POST['format']);
            } else {
                $render = Charts_Model_ChartRenders::getRender();
            }
        }

        if (!is_null($width) && $width != "") {
            $render->setWidth((int) $width);
        }

        if (!is_null($height) && $height != "") {
            $render->setHeight((int) $height);
        }
        try {
            if ( !$permalink ) {
                $render->forceDownload();
            }
            $render->renderToOutput($chartdata, $this->getResponse());
        } catch (WebReport_Exception $e) {
            echo "Failed to render\n";
            echo $e->getMessage();
            Zend_Debug::dump($e);
            return;
        }
    }

    private function _stringsToTranslate()
    {
        _('CHART_WAS_NOT_REMOVED_BECAUSE_ITS_USED_IN_A_REPORT.');
        _('MOD_CHARTS_NAME');
        _('BROWSE_CHARTS');
        _('CHARTS');
        _('TOO_MANY_DATA_POINTS_CHOSEN._TRY_DECREASING_THE_TIME_RESOLUTION.');
        _('APPLICATION_CHART');
        _('DEFAULT_CHART_TYPE');
        _('CHART_NAME');
        _('FORMULAS');
        _('UPDATE');
        _('COMMON');
        _('VIEW_CHARTS');
        _('VIEW_TRUEVIEW_APPLICATIONS');
        _('A_PERMANENT_LINK_WAS_CREATED_FOR_THE_CHART.');
        _('THE_CHART_CAN_BE_REACHED_AT:');
        _('EXPORT_TO_IMAGE');
        _('LINE_CHART');
        _('LINE_CHARTS');
        _("CUSTOM_TIME_FILTER");
        _("PIE_CHART");
        _("DAY_USE");
        _("DAY_USE(RELATIVE)");
        _("WEEKDAYS");
        _("PIE_CHARTS");
        _("RESOLUTION");
        _("ROLES_ALLOWED_TO_SEE_THE_CHART");
        _("BROWSE_PRODUCED_CHARTS");
        _("CREATE_CHART");
        _("DAYS");
        _("DATA_SOURCES");
        _("HOUR_LIMITS");
        _("UNCHECK_THIS_TO_SHARE_THE_CHART_WITH_OTHERS_IN_YOUR_COMPANY.");
    	_("WEEK_USAGE");_("WEEK_USAGE(RELATIVE)");
    	_("MONTH_USAGE");_("MONTH_USAGE(RELATIVE)");
    	_("HOUR_USAGE");_("HOUR_USAGE(RELATIVE)");
        _("15_MINUTE_USAGE");
        _("DAY_OF_MONTH_USAGE");_("DAY_OF_MONTH_USAGE(RELATIVE)");
    }

}
