<?php
class Charts_TableCreatorController extends WebReport_Controller_Action {

    protected $_validDataTypes = array("application", "custom");
    protected $_user, $_group;

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();

        $this->_user = Core_Model_Users::getCurrentUser();

        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => _T('BROWSE_TABLES'));
        if($this->_group && $this->_group->canEdit($this->_user)) {
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", "folder" => $groupId, "mode" => "simple"), "text" => _T('CREATE_TABLE'));
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", "folder" => $groupId, "mode" => "advanced"), "text" => _T('CREATE_TABLE_(ADVANCED_MODE)'));
        }
        
        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

        $this->view->headTitle(_T("TABLE_CREATOR"), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $this->view->type = $this->_getParam('type');
        $this->view->id = $this->_getParam('id');
    }

    public function verifyAccess($action, $tableid, $redirect = true) {
        $tDb = Charts_Model_Tables::getInstance();
        $table = $tDb->find($tableid)->current();
        $document = Core_Model_DocumentFactory::fetch($tableid, "table");
        if ( $action != "view"  && $action != "edit") throw new Exception("Not a valid action: $action");

        $success = true;

        if ( $action == "view" && !$document->canView($this->_user)) $success = false;
        if ( $action == "edit" && !$document->canEdit($this->_user)) $success = false;

        if ( !$success && $redirect ) {
            $this->_flashMessenger->addMessage($this->_translate->_('THERE_IS_NO_TABLE_WITH_THAT_ID.'));
            return $this->_helper->redirector('browse','table-creator','charts');
        }
        return $success;
    }

    public function viewAction() {
        $tableId = (int)$this->_request->getParam('id');

        //Check access to table
        $allow = $this->verifyAccess("view", $tableId);
        if ( $allow !== true ) return $allow;

        $tDb = Charts_Model_Tables::getInstance();
        $table = $tDb->find($tableId)->current();

        $this->view->tableId = $tableId;
        $this->view->exportForm = new Charts_Form_ExportTableForm("view", $tableId);
        $tableExportUrl = $this->view->aclUrl(
            array("module" => "charts", "controller" => "table-creator", "action" => "export", "id" => $tableId), null, true
        );
        $this->view->exportForm->setAttrib("action", $tableExportUrl);
        $this->view->title = $table->title;
    }

    public function deleteAction() {
        $tableId = (int)$this->_request->getParam('id');

        $allow = $this->verifyAccess("edit", $tableId);
        if ( $allow !== true ) return $allow;

        $tDb = Charts_Model_Tables::getInstance();
        $table = $tDb->find($tableId)->current();
        $document = Core_Model_DocumentFactory::fetch($tableId, "table");

        $title = $table->title;

        // make sure the chart is not used in a report.
        $cellMapper = new Reports_Model_Report_Page_CellMapper();
        $cells = $cellMapper->fetchAll();

        $do_delete = true;
        foreach ($cells as $cell) {
            if ($cell instanceof Reports_Model_Report_Page_Cell_Table && $cell->getTableId() == $tableId && $do_delete) {
                $this->_flashMessenger->addMessage($this->_translate->_('TABLE_WAS_NOT_REMOVED_BECAUSE_ITS_USED_IN_A_REPORT.'));
                
                $do_delete = false;
                break;
            }
        }
        
        if($do_delete) {
            Core_Model_DocumentFactory::remove($tableId, 'table');
            $table->delete();
            $this->_flashMessenger->addMessage($this->_translate->_(sprintf($this->_translate->_('THE_TABLE_"%1$s"_HAS_BEEN_REMOVED.'),$title )));
        }

        $this->_helper->redirector('browse','table-creator','charts', 
                                   array('folder' => $document->group_id));
    }
    public function browseAction()
    {
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
            $this->_user, $this->_group, $this->_helper, $this->_request,
            'Charts_Model_Tables',
            'Charts_Model_TableListGenerator');
        
        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }


    public function saveAction() {
        if(!$this->getRequest()->isPost()){
            throw new WebReport_Exception("The request must be a post.");
        }
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector('browse','table-creator','charts');
            return;
        }

        $json = json_decode($this->_request->getParam("json"),true);
        $id = $this->_request->getParam("saveId");

        $table = Charts_Model_Table::fromArray($json);
        $tableId = $table->storeToDatabase($id == "false" ? null : $id);

        Core_Model_DocumentFactory::add($tableId, "table", $this->_group);
        header("Content-Type: application/json");
        echo json_encode(array(
            "success" => true,
            "msg"     => "Save complete"
        ));
    }

    public function addAction()
    {
        $this->view->translate = Zend_Registry::get("Zend_Translate");

        $this->view->simple = $this->_request->getParam('mode') == 'simple';
        $id = $this->_request->getParam("id");
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('THERE_IS_NO_TABLE_WITH_THAT_ID.'));
            $this->_helper->redirector('browse','table-creator','charts');
            return;
        }
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
            $this->_user, $this->_group, $this->_helper, $this->_request,
            'Charts_Model_Tables',
            'Charts_Model_TableListGenerator');
        if($id) {
            $tDb = Charts_Model_Tables::getInstance();
            $table = $tDb->find($id)->current();
            //Check access to table
            $allow = $this->verifyAccess("edit", $id);
            if ( $allow !== true ) return $allow;
            //Also check access to used sources
            if(!$table->sourcesVisible($this->_user)) {
                $this->_flashMessenger->addMessage($this->_translate->_('You cannot edit this table because it contains one or more sources that are not visible to you.'));
                return $this->_helper->redirector('browse','table-creator','charts', array('folder' => $this->_group->group_id));
            }
        } else {
            $table = Charts_Model_Table::CreateNew($this->view->translate, $this->view->simple);
        }
        if(!$table) {
            $this->_flashMessenger->addMessage($this->_translate->_('THERE_IS_NO_TABLE_WITH_THAT_ID.'));
            $this->_helper->redirector('browse','table-creator','charts');
            return;
        }
        $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();
        $now = new Zend_Date();
        $this->view->dateFormat = strtolower($dateFormat);
        $this->view->dateFormat = str_replace("yyyy", "yy", $this->view->dateFormat);
        $this->view->dateFormat = $this->view->dateFormat ? $this->view->dateFormat : "yy-mm-dd";
        $this->view->baseDate = $now->toString($dateFormat);
        $this->view->json = $table->toJson();
        $this->view->tableNameForm = new Charts_Form_TableName();
        $this->view->tableComparisonForm = new Charts_Form_TableComparison();
        $this->view->tableFilterForm = new Charts_Form_TableFilter();
        $this->view->generalForm = new Charts_Form_TableSettings();
        $this->view->compPeriodsForm = new Charts_Form_TableCompPeriodsSettings();
        $this->view->generalForm->populate($table->toNiceArray());
        $this->view->compPeriodsForm->populate($table->toNiceArray());
        $this->view->sectionForm = new Charts_Form_TableSectionSettings();
        $this->view->entryForm   = new Charts_Form_TableSectionEntrySettings();
        $this->view->exportForm   = new Charts_Form_ExportTableForm("add");

        $this->view->saveForm = new Charts_Form_SaveTable();
        $this->view->saveAsForm = new Charts_Form_SaveAsTable();
        $emptySection = Charts_Model_TableSection::CreateNew($this->view->translate);
        $emptyEntry   = Charts_Model_TableSectionEntry::CreateNew($this->view->translate);

        $this->view->emptySectionJson = json_encode($emptySection->toNiceArray());
        $this->view->emptyEntryJson = json_encode($emptyEntry->toNiceArray());
        
        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();

        $applications = new Core_Model_Applications();
        $this->view->applications = $applications->fetchAllWithCountData();
        $this->view->tableId = $id ? $id :  "false";
        $aggDb = Charts_Model_DataAggregatorList::getInstance();
        $agg = $aggDb->fetchAll($aggDb->select()->where("description = 'SHOW_USAGE_DISTRIBUTED_OVER_HOURS'"))->current();
        $this->view->hourUsageId = $agg->dataaggregator_id;

        $folder = new UniForm_Form_Element_Select('search_folder');
        $folder->setLabel(($this->_translate->_('SEARCH_IN_FOLDER')));
        $folder->addMultiOptions($this->getViewableFolders());
        $this->view->folderSelect = $folder;
        $this->view->currFolder = $this->_group->group_id;
    }

    private function getViewableFolders()
    {
        $groupDb = new Core_Model_Folders();
        if($this->_group != WebReport_View_Helper_GenericListView::ROOT_FOLDER)
            $folders = $groupDb->getRootGroups($this->_user, $this->_group->company_id);
        else
            $folders = array();
        
        $folderIds = array();
        foreach($groupDb->getViewableGroups($this->_user) as $group)
            $folderIds[] = $group->group_id;
        $viewable = array("all" => "0 " . $this->_translate->_("ALL"));
        if($this->_group != WebReport_View_Helper_GenericListView::ROOT_FOLDER){
            $this->getTree($folders, $viewable, 0, $folderIds);
        }
        return $viewable;
    }

    private function getTree($root, &$tree, $depth, $editableGroupIds = array())
    {
        if ($depth>10)
            return;
        foreach($root as $folder) {
            $tree[$folder->group_id] = $depth . " " . $folder->name; 
            $this->getTree($folder->getChildren(), $tree, $depth+1, $editableGroupIds);
        }
    }

    public function checkTableChoiceAction()
    {
        $sources = Charts_Model_SourceTypes::getInstance();
        $cams = $this->_getParam("cameras");
        $resp = $sources->canMerge(json_decode($cams,true));
        $this->_helper->Json->sendJson(array("resp" => $resp));
    }

    public function getLogicalPeriodsAction()
    {
        $aggId = $this->_getParam("aggId");
        $matches = Charts_Model_DataAggregatorList::getMatchingPeriodCodes($aggId);
        $this->_helper->Json->sendJson(array("resp" => json_encode($matches)));
    }

    public function exportAction() {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        $tableId = (int) $this->_request->getParam("id");
        if (!$tableId && !$_POST["json"]) {
            throw new WebReport_Exception("No table was specified.");
        }
        $permalink = $this->_request->getParam('permalink');
        $format = $this->_request->getParam('format');
        $disp = "attachment";
        if($permalink) {
            $disp = "inline";
            header('Content-type: application/json');
            $permaDb = new Charts_Model_Permalinks();
            $filter = $permaDb->addTablePermalink($format, $tableId, false);

            $this->view->fullAddress = $filter->getApi($this->view);
            echo json_encode(array("url" => $this->view->fullAddress));
            return;
        }
         
        if($tableId) {
            $tableDb = Charts_Model_Tables::getInstance();
            $table = $tableDb->find($tableId)->current();
            $allow = $this->verifyAccess("edit", $tableId);
            if ( $allow !== true ) return $allow;
        } else {
            $tableJson = $_POST["json"];
            $table = Charts_Model_Table::fromJson($tableJson);
        }
        error_log("TODO: Check rights for table before exporting");

        if( $format == "html" ) {
            $this->_helper->viewRenderer->setNoRender(false);
            $renderer = new Charts_Model_TableHtmlRenderer();
            $renderer->render($table, true);
        } else if ( $format == "csv") {
            $renderer = new Charts_Model_TableCsvRenderer();
            $renderer->render($table,true);
        }
        header("Content-Disposition: $disp; filename=\"export.$format\"");
    }

    public function searchAction()
    {
        $translate = Zend_Registry::get("Zend_Translate");

        $q = $this->_getParam("q");
        $folder_id = $this->_getParam("folder_id");

        $sourceTypes = Charts_Model_SourceTypes::getInstance();
        $sources = $sourceTypes->getSourcesFromString($this->_user, $q, $folder_id);
        foreach($sources as $i => $s){
            $sources[$i]['type']['title'] = $translate->translate($s['type']['title']);
        }

        $idToTitle = array();

        foreach($sourceTypes->_sourceTypes as $key => $value) {
            $idToTitle[$key] = $translate->translate($value->getGroupTitle());
        }

        $this->_helper->Json->sendJson(array(
            "sourceTypes" => $idToTitle, 
            "sources" => $sources));
    }

    public function copyAction(){
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        $tId = (int)($this->_request->getParam('id'));
        $allow = $this->verifyAccess("edit", $tId);
        if ( $allow !== true ) return $allow;
        $tDb = Charts_Model_Tables::getInstance();
        $table = $tDb->find($tId)->current();
        $document = Core_Model_DocumentFactory::fetch($tId, "table");

        $table->setTitle($tDb->findNextCopyName($tId));

        $tableId = $table->storeToDatabase(null);
        $document = Core_Model_DocumentFactory::add($tableId, "table", $this->_group);

        $this->_helper->redirector('browse','table-creator','charts', 
                                   array('folder' => $document->group_id));
    }

    public function renderAction() {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        $json = $this->_request->getParam("json");
        if ( $json ) {
            $table = Charts_Model_Table::fromJson($json);
        } else {
            $id = $this->_request->getParam("id");
            $tDb = Charts_Model_Tables::getInstance();
            $table = $tDb->find($id)->current();
            $allow = $this->verifyAccess("view", $id);
            if ( $allow !== true ) return $allow;
        }
        $table->UseCache();
        $format = $this->_request->getParam("format");

        if ( $format == "html" ) {
            $rdr = new Charts_Model_TableHtmlRenderer();
            $rdr->render($table, true);
        } else {
            throw new WebReport_Exception("Format not support.");
        }
    }
}
