<?php

class Charts_Form_AddCustomChart extends UniForm_Form
{

	protected $_action;

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param string $action Can be new or edit
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($action = "add", $options = null)
	{
		$this->_action = $action;
		parent::__construct($options);
	}

	public function init()
	{
		parent::init();

		$action = $this->_action;

		$name = new UniForm_Form_Element_Text('name');
		$name->setLabel('CHART_NAME')
			->setRequired()
			->addFilter('StringTrim');

		$company_id = new UniForm_Form_Element_Select("company_id");
		
		$companiesDb = new Core_Model_Companies();

		$identity = Zend_Auth::getInstance()->getIdentity();
		if (!$identity->multi_company) {
			$company = $companiesDb->find($identity->company_id);
			$company_id = new UniForm_Form_Element_Hidden('company_id');
			$company_id->setValue($identity->company_id);
		} else {

			$company_id = new UniForm_Form_Element_Select('company_id');
			$company_id->setLabel('COMPANY')
				->addMultiOption(0, "SELECT_A_COMPANY.")
				->setRequired();

			$companies = $companiesDb->fetchAll($companiesDb->select()->where('active = ?', 1)->order('name'));
			foreach ($companies as $company) {
				$company_id->addMultiOption($company->company_id, $company->name);
			}

		}


		$applicationsModel = new Core_Model_Applications();
		$applications = $applicationsModel->fetchAll();
		foreach ($applications as $application) {

		}

		if (!$identity->multi_company) {

			$countDataSetsDb = new Core_Model_CountData_Sets();
			$countDataSets = $countDataSetsDb->fetchAll($countDataSetsDb->select()->where('company_id = ?', $identity->company_id)->order('name'));

			$count_data_set_id = $countDataSets->getFormElement('count_data_set_id');

		} else {

			$count_data_set_id = new UniForm_Form_Element_Select("count_data_set_id");

			$count_data_set_id
				->setRegisterInArrayValidator(false)
				->addMultiOption(0, "SELECT_A_COMPANY.");
			$view = $this->getView();

			$view->headScript()->captureStart();  ?>
jQuery(function($) {
	$("#count_data_set_id").attr("disabled", true);
	var $setSelect = $("#count_data_set_id");

	$("#company_id").bind('change.count_data_set', function() {
		$("#company_id option[value=0]").remove();
		$.ajax({
			type : 'post',
			dataType : 'json',
			url : '<?php echo $view->url(array("module" => "count-data", "controller" => "count-data-sets", "action" => "get-sets-for-company"), 'default', true); ?>',
			data : { companyId : $(this).val() },
			success : function (data, textStatus) {
				$("option, optgroup", $setSelect).remove();
				if ($(data).length == 0) {
					$setSelect.attr("disabled", true);
					$(document.createElement('option')).val(0).text("<?= $view->translate("YOU_MUST_CREATE_A_COUNT_DATA_SET_BEFORE_ADDING_A_CUSTOM_CHART."); ?>").appendTo($setSelect);
					return;
				}
				$setSelect.attr("disabled", false);
				$.each(data, function(i, apps) {
					var $optGroup = $(document.createElement('optgroup')).attr("label", i);
					$.each(apps, function(i, val) {
					var $option = $(document.createElement('option')).val(val.id).text(val.name).appendTo($optGroup);
					//var $label = $(document.createElement('label')).attr("for", "count_data_set_id_" + val.id).addClass("inlineLabel");
					//$label.append($(document.createElement('input')).attr("type", "checkbox").attr("name", "count_data_set_id[]").val(val.id)).append(val.name);
					//console.log($multiField);
					//$multiField.append($label);
					});
					$optGroup.appendTo($setSelect);
				});

			}
		});
	});

	if (parseInt($("#company_id option[selected]").val()) > 0) {
		$("#company_id").trigger('change.count_data_set');
	}
});
<?php			$view->headScript()->captureEnd();
		}


		$count_data_set_id->setRequired()
			->setLabel("COUNT_DATA_SET");


		$acl_role_id = new UniForm_Form_Element_MultiCheckbox('acl_role_id');
		$acl_role_id->setLabel("ROLES_ALLOWED_TO_SEE_THE_CHART")
			->setRequired();

		if (!$identity->multi_company) {

			$aclRolesDb = new Core_Model_AclRoles();
			$select = $aclRolesDb->select();
			$select = $select->where('company_id = ?', $identity->company_id)->order('name');
			$aclRoles = $aclRolesDb->fetchAll($select);

			foreach ($aclRoles as $aclRole) {
				$acl_role_id->addMultiOption($aclRole->acl_role_id, $aclRole->name);
			}

		} else {

			$acl_role_id
				->setRegisterInArrayValidator(false)
				->addMultiOption(0, "SELECT_A_COMPANY.");
			$view = $this->getView();

			$view->headScript()->captureStart();  ?>
jQuery(function($) {
	$("#acl_role_id-0").attr("disabled", true);
	var $multiField = $(".ctrlHolder p.label").eq(1).next();

	$("#company_id").bind('change.acl_role', function() {
		$("#company_id option[value=0]").remove();
		$.ajax({
			type : 'post',
			dataType : 'json',
			url : '<?php echo $view->url(array("module" => "admin", "controller" => "acl", "action" => "get-roles-for-company"), 'default', true); ?>',
			data : { companyId : $(this).val() },
			success : function (data, textStatus) {
				$("label[for^=acl_role_id]").remove();
				if ($(data).length == 0) return;
				$.each(data, function(i, val) {
					var $label = $(document.createElement('label')).attr("for", "acl_role_id_" + val.id).addClass("inlineLabel");
					$label.append($(document.createElement('input')).attr("type", "checkbox").attr("name", "acl_role_id[]").val(val.id)).append(val.name);
					$multiField.append($label);
				});

			}
		});
	});

	if (parseInt($("#company_id").val()) > 0) {
		$("#company_id").trigger('change.acl_role');
	}
});
<?php			$view->headScript()->captureEnd();
		}


		$type = new UniForm_Form_Element_Radio('default_type');
		$type->setLabel('DEFAULT_CHART_TYPE');
		$type->addMultiOption('line','LINE_CHART');
		$type->addMultiOption('column','COLUMN_CHART');
//		$type->addMultiOption('pie','PIE_CHART');
		$type->setRequired();

		$description = new UniForm_Form_Element_Textarea('description');
		$description->setLabel('DESCRIPTION')
			->addFilter('StringTrim');


		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$this->addElements(array($name, $count_data_set_id, $type, $acl_role_id, $description, $company_id));

		$this->addDisplayGroup(array('name', 'company_id', 'count_data_set_id', 'default_type', 'acl_role_id', 'description'), 'chart-group');

		$this->addElements(array($submit));
	}

}
