<?php
class Charts_Form_DataSource extends UniForm_Form{

    static function factoryFromArray(Array $a = NULL){
        $form = new Charts_Form_DataSource();
        $form->populate($a);
        return $form;
    }

    static function factoryDefault(){
        $form = new Charts_Form_DataSource();
        return $form;
    }
    
    public function populate(array $a){
        $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();

        foreach(array('time_period_from','time_period_to') as $time)
            if(array_key_exists($time,$a) && !is_null($a[$time])){
                $a[$time] = $a[$time]->toString( $dateFormat ? $dateFormat : "yyyy-MM-dd");
            }
        UniForm_Form::populate($a);
        $chosen_data = $this->getElement('chosen_data');
        $sources = Charts_Model_SourceTypes::getInstance();
        $selected = array();
        foreach($a['data'] as $chosen) {
            $selected[] = $chosen->id . "," . $chosen->dataTypeId . "," . $chosen->identifier; 
        }
        $chosen_data->setValue($selected);
    }

    public function init(){
        parent::init();
        // Create elements
        $chosen_data = new UniForm_Form_Element_MultiSelect('chosen_data');
        $color = new UniForm_Form_Element_Hidden('color');
        $title = new UniForm_Form_Element_Hidden('title');
        $appearance = new UniForm_Form_Element_Select('appearance');
        $virtual_camera_ids = new UniForm_Form_Element_Hidden('virtual_camera_ids');
        $camera_ids = new UniForm_Form_Element_Hidden('camera_ids');
        $time_filter = new UniForm_Form_Element_Hidden('time_filter_code');
        $camera_list = new UniForm_Form_Element_Select('camera_list');
        $formula_list = new UniForm_Form_Element_Select('formula_list');
        $virtual_camera_list = new UniForm_Form_Element_Select('virtual_camera_list');
        $data_set_list = new UniForm_Form_Element_Select('data_set_list');
        //$this->fillLists($chosen_data,$camera_list,$formula_list,$virtual_camera_list,$data_set_list);
        $this->populateDataSources($chosen_data);
        $this->addElements(array($chosen_data));

        
        // Set Label
        $chosen_data->setLabel("DATA_SOURCES");
        $appearance ->setLabel("APPEARANCE")
                    ->addMultiOption('bar',_T('BAR'))
                    ->addMultiOption('line',_T('LINE'))
                    ->addMultiOption('default',_T('DEFAULT'))
                    ->setValue('default');
                    

        // Code Redundancy, same as in GlobalSettings
        $countDataSetsDb = new Core_Model_CountData_Sets();
        $countDataSets = $countDataSetsDb->fetchAll($countDataSetsDb->select()->order('name'));
        $data_set_id = $countDataSets->getFormElement('data_set_id', null, false, "SELECT_A_COUNT_DATA_SET.");
        
        $this->addElements(array($title,$color,$time_filter,$appearance));
        $this->addDisplayGroups(array(
           'first' => array(
               'elements' => array('chosen_data','appearance'),
           ),
        ));
        self::makeTimeFilterSubForm($this);
        self::makeTimeIntervalSubForm($this);
    }

    public function populateDataSources($chosen_data) {
        $usersDb = Core_Model_Users::getInstance();
        $user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();
        $sources = Charts_Model_SourceTypes::GetInstance();
        $valPack = function($meta) {
            $translate = Zend_Registry::get("Zend_Translate");
            return sprintf("%s (%s)",$meta['name'], $translate->translate($meta['type']['title']));
        };
        $idPack = function($meta = "huh") {
            return $meta['id'] . "," . $meta['type']['id'] . "," . $meta['sourcetype'];
        };
        $organizedSources = $sources->getSourcesByType($user,$valPack, $idPack);
        foreach($organizedSources as $type => $sources) {
            $chosen_data->addMultiOption($type, $sources);
        }
    }

    public static function addAllPeriods($form, $addSpecial = true) {
        //move this to factory somewhere
        $duration = array(
            "y" => "YEAR",
            "m" => "MONTH",
            "w" => "WEEK",
            "d" => "DAY");
        $period = array(
            "0" => "CURRENT_%dd",
            "-1" => "LAST_%dd",
            "-2" => "TWO_%ddS_AGO",
            "-3" => "THREE_%ddS_AGO",
            "-4" => "FOUR_%ddS_AGO"
            );
        foreach($duration as $t => $dur) {
            $group = array();
            foreach($period as $p => $format) {
                $group[$p . $t] = str_replace("%dd", $dur, $format);
            }
            if($t == 'd') {
                $group['-5d'] = _T("FIVE_DAYS_AGO");
                $group['-6d'] = _T("SIX_DAYS_AGO");
                $group['-7d'] = _T("SEVEN_DAYS_AGO");
            }
            if($t == 'y') {
                $group['-4y:1y'] = _T("FIVE_MOST_RECENT_YEARS");
            }
            
            $form->addMultiOptions(array($dur."S" => $group));
        }
        if ( $addSpecial) {
            $form->addMultiOptions(array("OTHER" => array(
                "man" =>"MANUAL_TIME_INTERVAL",
                "fix" => "FIXED_INTERVAL")));
        }
    }

    public static function makeTimeIntervalSubForm($form){
        $sub = new Zend_Form_SubForm();
        // Time period
        $time_period_code = new UniForm_Form_Element_Select('time_period_code');
        $time_period_from = new UniForm_Form_Element_Text('time_period_from');
        $time_period_to = new UniForm_Form_Element_Text('time_period_to');
        $time_period_manual = new UniForm_Form_Element_Text('time_period_manual');
        $time_period_to_title = new UniForm_Form_Element_Checkbox('time_period_to_title');

        $time_period_code->setLabel("TIME_PERIOD");
        $time_period_from->setLabel("FROM");
        $time_period_to->setLabel("TO");
        $time_period_manual->setLabel("TIME_CODE");
        $time_period_to_title->setLabel("Append time period to title in chart");

        self::addAllPeriods($time_period_code);


        $form->addElements(array( $time_period_code,$time_period_from,$time_period_to,$time_period_manual,$time_period_to_title));
        $form->addDisplayGroups(array(
            'timeinterval-code'             => array('elements' => array('time_period_code')),
            'timeinterval-period-to-title'  => array('elements' => array('time_period_to_title')),
            'timeinterval-fixed'            => array('elements' => array('time_period_from', 'time_period_to')),
            'timeinterval-manual'           => array('elements' => array('time_period_manual')),

        ));
    }

    public static function makeTimeFilterSubForm($form){
        $sub = new Zend_Form_SubForm('timefilterform');
        $sub = $form;
        $time_filter_type = new UniForm_Form_Element_Select('tool_timefilter_type');
        $time_filter_from = new UniForm_Form_Element_Select('timefilter_from');
        $time_filter_to = new UniForm_Form_Element_Select('timefilter_to');
        $time_filter_days = new UniForm_Form_Element_MultiCheckbox('time_filter_days');
        $time_filter_slider = new UniForm_Form_Element_Text('time_filter_slider');

        $time_filter_slider->setLabel('HOUR_LIMITS');
        $time_filter_type->setLabel("TIME_FILTER");
        $time_filter_days->setLabel("WEEK_DAY_FILTER");
        $sub->addElements(array($time_filter_type,$time_filter_days,$time_filter_slider));
        
        $days = str_split(Zend_Registry::get('Zend_Translate')->translate("MTWTFSS"),1);
        $timetype = array($time_filter_type->getId());
        $enames = array();
        $filtercode = array("monday","tuesday","wednesday","thursday","friday","saturday","sunday");
        for($i = 0; $i < 7; $i++){
            $name = 'timefilter_'.$i;
            $time_filter_days->addMultiOption($filtercode[$i], $days[$i]);
        }
        $interval = new UniForm_Form_Element_Text('timefilter_interval');
        $interval->setOptions(array("class" => "timefilter-element timefilter-interval"));
        $time_filter_type
           ->addMultiOption("none","NO_TIME_FILTER")
           ->addMultiOption("custom","CUSTOM_TIME_FILTER");

        $sub->addDisplayGroups(array(
           'timefilter-type' => array(
               'elements' => $timetype,
           ),
           'timefilter-boxes' => array(
               'elements' => array('time_filter_days','time_filter_slider'),
           ),
       ));
        return 'timefilterform';
    }

    public function translatableString(){
        _("CUSTOM_TIME_FILTER");_("ALL_TIME");_("OTHER");
        _("TWO_YEARS_AGO");_("TWO_MONTHS_AGO");_("TWO_DAYS_AGO");_("TWO_WEEKS_AGO");
        _("THREE_YEARS_AGO");_("THREE_MONTHS_AGO");_("THREE_DAYS_AGO");_("THREE_WEEKS_AGO");
        _("FOUR_YEARS_AGO");_("FOUR_MONTHS_AGO");_("FOUR_DAYS_AGO");_("FOUR_WEEKS_AGO");
        _("FIVE_YEARS_AGO");_("FIVE_MONTHS_AGO");_("FIVE_DAYS_AGO");_("FIVE_WEEKS_AGO");
        _("WEEK_DAY_FILTER");_('BAR_CHART');_('LINE_CHART');_('BEST_FIT');_('FEMALES');_('MALES');
        _("DISTRIBUTION"); 
    }
}
