<?php

abstract class Charts_Model_Chart_Abstract
{

    protected $_amChart;
    protected $_dataType;

    public function __construct($chartType = 'line')
    {
        $amChart = AmCharts_Chart::factory($chartType);
        $this->_amChart = $amChart;

        $settings = $amChart->getSettings();

        $settings->setPlotAreaMargins(array('top' => 10, 'left' => 60, 'right' => 60))
            ->setLegendAlign("center");

        $view = Zend_Controller_Action_HelperBroker::getStaticHelper('ViewRenderer')->view;
        $imageUrl = $view->aclUrl(array('type' => $this->_dataType), 'charts_toimage', false);

        if ($imageUrl) {
            $settings->exportAsImage = array();
            $settings->exportAsImage->file = $imageUrl;
            $settings->exportAsImage->color = "#CC0000";
            $settings->exportAsImage->alpha = 50;
        }

    }

    // Should be OK with raw count data also..
    public function setAggregatedData($descriptions, $values,$dataLabels, $graphOptions = array())
    {

        $amChart = $this->_amChart;
        $data = $amChart->getData();

        $series = array();
        $xLabels = array();

//        $types = $countData->getApplication()->getOutTypes();

        /* @todo Fix setter for translation instead */
        $translate = Zend_Registry::get('Zend_Translate');
        $i = -1;
        foreach ($values as $vals) {
            $vals = (array)$vals;
            $desc = 0;
            if(isset($vals['cnttime'])){
                $time = $vals['cnttime'];
                $desc = $time;
                unset($vals['cnttime']);
            }else{
                $i = $i + 1;
                $time = $i;
                $desc = $descriptions[$i];
            }
            $xLabels[$time] = $desc;
            $keys = array_keys($vals);
            foreach ($keys as $key) {
                $serie = null;
                if (isset($series[$key])) {
                    $serie = $series[$key];
                } else {
                    $serie = new AmCharts_Data_Serie($key);
                    $serie->setDescription($dataLabels[$key]);
                    if (isset($graphOptions[$key])) {
                        $serie->setOptions($graphOptions[$key]);
                    }
                    //$serie->setOptions(array("axis" => "right"));
                    $series[$key] = $serie;
                }

                if (!is_null($vals[$key])) {
                    $serie->addValue($time, sprintf("%01.2f", $vals[$key]));
                }

            }

        }

        $data->setXLabels($xLabels);

        foreach ($series as $key => $serie) {
            $data->addSerie("id" . $key, $serie);
        }
    }

    public function setData($descriptions, $values, $graphOptions = array())
    {

        $amChart = $this->_amChart;
        $data = $amChart->getData();

        $series = array();
        $xLabels = array();

//        $types = $countData->getApplication()->getOutTypes();

        /* @todo Fix setter for translation instead */
        $translate = Zend_Registry::get('Zend_Translate');

        foreach ($values as $vals) {
            $vals = (array)$vals;

            $time = $vals['cnttime'];
            unset($vals['cnttime']);
            $xLabels[$time] = $time;
            $keys = array_keys($vals);

            foreach ($keys as $key) {
                $serie = null;
                if (isset($series[$key])) {
                    $serie = $series[$key];
                } else {
                    $serie = new AmCharts_Data_Serie($key);
                    $serie->setDescription($descriptions[$key]);
                    if (isset($graphOptions[$key])) {
                        $serie->setOptions($graphOptions[$key]);
                    }
                    //$serie->setOptions(array("axis" => "right"));
                    $series[$key] = $serie;
                }

                if (!is_null($vals[$key])) {
                    $serie->addValue($time, sprintf("%01.2f", $vals[$key]));
                }

            }

        }

        $data->setXLabels($xLabels);

        foreach ($series as $key => $serie) {
            $data->addSerie("id" . $key, $serie);
        }
    }


    public function dataExists()
    {
        return $this->_amChart->dataExists();
    }

    public function getAmChartObject()
    {
        return $this->_amChart;
    }

    /**
     *
     * @return bool True if data exists, else false
     */
    public function render() {
        return $this->_amChart->render("chart");
    }

    public function renderData(){
        return $this->_amChart->renderData();
    }

}
