<?php
class Charts_Model_ChartData
{
    public    $_chartDataSource = NULL;
    protected $_title = NULL;

    protected $_timeFilterCode = NULL;
    protected $_timeFilter = NULL;

    protected $_aggregator = NULL;
    protected $_chartType = NULL;
    protected $_dataResolution = NULL;
    protected $_description = NULL;
    protected $_aggregator_id = NULL;
    protected $_flags = NULL;

    protected $_user_id;
    protected $_company_id;

    // statics

    static function factoryFromArray(array $a = NULL) {
        $c = get_called_class();
        $ret = new $c();
        $ret->populate($a);
        return $ret;
    }

    public function getCompanyId(){
        return $this->_company_id;
    }

    public function setToday($date) {
        foreach($this->_chartDataSource as $cds) {
            $cds->setToday($date);
        }
        return $this;
    }

    public function setCompanyId($id){
        $this->_company_id = $id;
    }

    public function getUserId(){
        return $this->_user_id;
    }

    public function setUserId($id){
        $this->_user_id = $id;
    }

    public function getAxes() {
        $axes = new Charts_Model_Axes();
        foreach($this->_chartDataSource as $i => $source) {
            foreach($dataTypeId=$this->_chartDataSource[$i]->_data as $data){
                $sourceAxes = $source->getAxes($data->dataTypeId);
                $axes->addAxes($sourceAxes);
            }
        }
        return $axes;
    }

    // normal getters and setters
    public function getAggregator()
    {
        return $this->_aggregator;
    }

    public function getAggregatorId()
    {
        return $this->_aggregator_id;
    }

    // normal getters and setters
    public function getAggregatorInstance()
    {
        if($this->_aggregator_id != NULL) return  Charts_Model_DataAggregatorList::getAggregatorById($this->_aggregator_id);
        return Charts_Model_DataAggregatorList::getAggregatorById($this->_aggregator);
    }

    public function setAggregator($aggregator)
    {
        $this->_aggregator = $aggregator;
        return $this;
    }

    public function setAggregatorId($id){
        $this->_aggregator_id = $id;
        return $this;
    }

    public function hasAggregator() {
        return !is_null($this->_aggregator) || !is_null($this->_aggregator_id);
    }

    public function getDescription(){
        return $this->_description;
    }

    public function setDescription($description){
        $this->_description = $description;
    }

    public function getTimeFilterCode() {
        return $this->_timeFilterCode;
    }

    public function setTimeFilterCode($timeCode) {
        $this->_timeFilterCode = $timeCode;
        return $this;
    }

    public function getTimeFilter() {
        if(!$this->_timeFilterCode) {
            throw new WebReport_Exception("No existing time code assigned to object");
        }

        if(!$this->_timeFilter) {
            $this->_timeFilter = Core_Model_TimeFilterFactory::makeFilter($this->_timeFilterCode);
        }

        return $this->_timeFilter;
    }

    //FIXME: alot of other stuff connected to this chart
    public function __construct() {
        // keep this thin/empty and have getters and setters
    }

    public function setTitle($title) {
        $this->_title = $title;
        return $this;
    }

    public function getTitle(){
        if($this->_title == NULL) return "No Title..";
        return $this->_title;
    }

    public function setChartType($chart_type) {
        $this->_chartType = $chart_type;
        return $this;
    }

    public function getChartType() {
        return $this->_chartType;
    }

    public function setDataResolution($dataResolution) {
        $this->_dataResolution = $dataResolution;
        return $this;
    }

    public function getDataResolution() {
        return $this->_dataResolution;
    }

    //specifc functions
    public function addDataSource(Charts_Model_ChartDataSource $data_source) {
        if(!$this->_chartDataSource)
            $this->_chartDataSource = array();

        $this->_chartDataSource[] = $data_source;

        return $this;
    }

    public function getDataSources() {
        if($this->getNumberOfSources() == 0)
            return array();
        return $this->_chartDataSource;
    }

    public function bogusData(){
        $this->_title = "Diagramtitel";
        $colors = array("FF0000","00FF00","0000FF","000000");
        $this->setDataResolution('ts1m');
        $this->setDataAggregatorFromDescription(array('aggregator' => 'Charts_Model_DataAggregator_HistSum', 
                                             'parameters' => '{"_mode": 4,"_relative": 0}'));
        $this->setTimeFilterCode("monday | friday");
        for($i = 0; $i <2; $i++){
            $d = new Charts_Model_ChartDataSource();
            $d->setTitle("Titel för datakälla " . $i)
                ->setColor($colors[$i])
                ->setTimeFilterCode("monday")
                ->setTimePeriodCode("0d");
            $this->_chartDataSource[] = $d;
        }
        return $this;
    }


    static function factoryFromDatabase($chart_data_id) {
        $c = get_called_class();
        $ret = new $c();

        $ret->loadFromDatabase($chart_data_id);
        return $ret;
    }

    public function setDataAggregatorFromDescription(array $a) {
        if(!array_key_exists("aggregator", $a) || !array_key_exists("parameters", $a) )
                throw new WebReport_Exception_ParameterError("Array in incorrect format");

        if(!is_null($a['aggregator'])) {
            $this->_aggregator = new $a['aggregator']();
            if(!is_null($a['parameters']) && strlen($a['parameters']) > 0)
                $this->_aggregator->setDefaultsJson(Zend_Json::decode($a['parameters']));
        }

        return $this;
    }

    public function getDataAggregatorDescription() {
        if(is_null($this->_aggregator))
            throw new WebReport_Exception_ParameterError("No aggregator set");

        return array(
            "aggregator" => get_class($this->_aggregator),
            "parameters" => $this->_aggregator->dumpJson());
    }

    public function getNumberOfSources(){
        if($this->_chartDataSource == NULL) return 0;
        return count($this->_chartDataSource);
    }

    public function storeToDatabase(Core_Model_User $user, Core_Model_Company $company=NULL, $chartDataId = NULL, $remove = TRUE) {
        $data = $this->toArray();
        $data['user_id'] = $user->user_id;
        $data['company_id'] = is_null($company) ? NULL : $data['company_id'] = $company->company_id;
        if(array_key_exists('data_aggregator', $data)) {
            $data['aggregator_object'] = $data['data_aggregator']['aggregator'];
            $data['aggregator_init'] = $data['data_aggregator']['parameters'];
            unset($data['data_aggregator']);
        } else {
            $data['aggregator_object'] = '';
            $data['aggregator_init'] = '';
        }

        $chartDataDb = new Charts_Model_ChartDataList();
        $chartDataRow = NULL;
        if(!is_null($chartDataId)) {
            $chartDataRow = $chartDataDb->find($chartDataId)->current();
            if(!$chartDataRow)
                throw new WebReport_Exception_ParameterError("Not valid id");
        } else {
           $chartDataRow = $chartDataDb->createRow();
        }
        if (array_key_exists('data_sources', $data)) {
            $dataSources = $data['data_sources'];
            unset($data['data_sources']);
        } else {
            $dataSources = array();
        }
        $chartDataRow->setFromArray($data);
        $chartDataId = $chartDataRow->save();
        // remove all the old stuff first
        $chartDataSourceDb = new Charts_Model_ChartDataSourceList();
        $chartDataSourceCameraDb = new Charts_Model_ChartDataSourceCameraList();
        $toRemove = $chartDataSourceDb->getAdapter()->quoteInto('chart_data_id = ?', $chartDataRow->chart_data_id);
        if($remove) $chartDataSourceDb->delete($toRemove);
        if(count($dataSources)>0) {
            foreach($dataSources as $dataSource) {
                foreach(array('time_period_from','time_period_to') as $time)
                    if(array_key_exists($time,$dataSource) && !is_null($dataSource[$time])){
                         $dataSource[$time] = $dataSource[$time]->get(Zend_Date::ISO_8601);
                    }
                $chartDataSourceRow = $chartDataSourceDb->createRow();
                $chartDataSourceRow->setFromArray($dataSource);
                $chartDataSourceRow->chart_data_id = $chartDataId;
                $chartDataSourceRow->save();

                foreach($dataSource['data'] as $chosen) {
                    $sourcetype = Charts_Model_SourceTypes::getSourceType($chosen->identifier);
                    $colname = $sourcetype->getChartDataSourcePickId();
                    $cameraRow = $chartDataSourceCameraDb->createRow();
                    $cameraRow->$colname = $chosen->id;
                    $cameraRow->type = $chosen->dataTypeId;
                    $cameraRow->chart_data_source_id = $chartDataSourceRow->chart_data_source_id;
                    $cameraRow->sourcetype = $sourcetype->getUniqueIdentifier();
                    $cameraRow->save();
                }
                
                /*
                foreach($dataSource['camera_ids'] as $index => $id) {
                    $cameraRow = $chartDataSourceCameraDb->createRow();
                    $cameraRow->chart_data_source_id = $chartDataSourceRow->chart_data_source_id;
                    $cameraRow->camera_id = $id;
                    $cameraRow->type = $dataSource['camera_id_types'][$index];
                    $cameraRow->save();
                }

                foreach($dataSource['pos_data_ids'] as $index => $id) {
                    $cameraRow = $chartDataSourceCameraDb->createRow();
                    $cameraRow->chart_data_source_id = $chartDataSourceRow->chart_data_source_id;
                    $cameraRow->pos_data_id = $id;
                    $cameraRow->type = $dataSource['pos_data_id_types'][$index];
                    $cameraRow->save();
                }

                foreach($dataSource['virtual_camera_ids'] as $index => $id) {
                    $cameraRow = $chartDataSourceCameraDb->createRow();
                    $cameraRow->chart_data_source_id = $chartDataSourceRow->chart_data_source_id;
                    $cameraRow->virtual_camera_id = $id;
                    $cameraRow->type = $dataSource['virtual_camera_id_types'][$index];
                    $cameraRow->save();
                }

                foreach($dataSource['data_set_ids'] as $index => $id) {
                    $cameraRow = $chartDataSourceCameraDb->createRow();
                    $cameraRow->chart_data_source_id = $chartDataSourceRow->chart_data_source_id;
                    $cameraRow->count_data_set_id = $id;
                    $cameraRow->type = $dataSource['data_set_id_types'][$index];
                    $cameraRow->save();
                }

                foreach($dataSource['formula_ids'] as $index => $id) {
                    $cameraRow = $chartDataSourceCameraDb->createRow();
                    $cameraRow->chart_data_source_id = $chartDataSourceRow->chart_data_source_id;
                    $cameraRow->count_data_formula_set_id = $id;
                    $cameraRow->type = $dataSource['formula_id_types'][$index];
                    $cameraRow->save();
                }
                */
            }
        }

        return $chartDataId;
    }

    public function loadFromDatabase($chartDataId) {
        /* @var $chartDataRow Zend_Db_Table_Row */
        /* @var $chartDataSourceRows Zend_Db_Table_Rowset */
        /* @var $chartDataSourceRow Zend_Db_Table_Row */
        /* @var $chartDataSourceCameraRows Zend_Db_Table_Rowset */
        /* @var $chartDataSourceCameraRow Zend_Db_Table_Row */
        $chartDataDb = new Charts_Model_ChartDataList();
        $chartDataRow = $chartDataDb->find($chartDataId)->current();
        if(is_null($chartDataRow)) {
            throw new WebReport_Exception("NO_SUCH_ID_EXISTS");
        }
        $dataSources = array();
        $chartDataSourceRows = $chartDataRow->findDependentRowset('Charts_Model_ChartDataSourceList');
        foreach($chartDataSourceRows as $chartDataSourceRow) {
            $chartDataSource = $chartDataSourceRow->toArray();
            $chartDataSource['camera_ids'] = array();
            $chartDataSource['camera_id_types'] = array();
            $chartDataSource['virtual_camera_ids'] = array();
            $chartDataSource['virtual_camera_id_types'] = array();
            $chartDataSource['data_set_ids'] = array();
            $chartDataSource['data_set_id_types'] = array();
            $chartDataSource['pos_data_ids'] = array();
            $chartDataSource['pos_data_id_types'] = array();
            $chartDataSource['formula_ids'] = array();
            $chartDataSource['formula_id_types'] = array();
            if ( isset($chartDataSource['time_period_code']) ) {
                $chartDataSource['time_period_from'] = null;
                $chartDataSource['time_period_to'] = null;
            }
            foreach(array('time_period_from','time_period_to') as $time){
                 if(!is_null($chartDataSource[$time])) $chartDataSource[$time] = new Zend_Date($chartDataSource[$time],Zend_Date::ISO_8601);
            }

            $chartDataSourceCameraRows = $chartDataSourceRow->findDependentRowset('Charts_Model_ChartDataSourceCameraList');
            $data = array();
            foreach($chartDataSourceCameraRows as $row) {
                if(is_null($row->sourcetype)) {
                    if(!is_null($row->camera_id)) $row->sourcetype = "PC";
                    if(!is_null($row->virtual_camera_id)) $row->sourcetype = "VC";
                    if(!is_null($row->count_data_formula_set_id)) $row->sourcetype = "F";
                    if(!is_null($row->parking_id)) $row->sourcetype = "P";
                    if(!is_null($row->count_data_set_id)) $row->sourcetype = "CDS";
                }
                $sourcetype = Charts_Model_SourceTypes::getSourceType($row->sourcetype);
                $column = $sourcetype->getChartDataSourcePickId();
                $chosen = new stdClass;
                $chosen->dataTypeId = $row->type;
                $chosen->id = $row->$column;
                $chosen->identifier = $sourcetype->getUniqueIdentifier();

                $chartDataSource['data'][]  = $chosen; 
                
                /*
                if(!is_null($chartDataSourceCameraRow->camera_id)) {
                   $chartDataSource['camera_ids'][] = $chartDataSourceCameraRow->camera_id;
                   $chartDataSource['camera_id_types'][] = $chartDataSourceCameraRow->type;
                } else if(!is_null($chartDataSourceCameraRow->virtual_camera_id)) {
                   $chartDataSource['virtual_camera_ids'][] = $chartDataSourceCameraRow->virtual_camera_id;
                   $chartDataSource['virtual_camera_id_types'][] = $chartDataSourceCameraRow->type;
                } else if(!is_null($chartDataSourceCameraRow->count_data_set_id)) {
                   $chartDataSource['data_set_ids'][] = $chartDataSourceCameraRow->count_data_set_id;
                   $chartDataSource['data_set_id_types'][] = $chartDataSourceCameraRow->type;
                } else if(!is_null($chartDataSourceCameraRow->count_data_formula_set_id)) {
                   $chartDataSource['formula_ids'][] = $chartDataSourceCameraRow->count_data_formula_set_id;
                   $chartDataSource['formula_id_types'][] = $chartDataSourceCameraRow->type;
                } else if(!is_null($chartDataSourceCameraRow->pos_data_id)) {
                   $chartDataSource['pos_data_ids'][] = $chartDataSourceCameraRow->pos_data_id;
                   $chartDataSource['pos_data_id_types'][] = $chartDataSourceCameraRow->type;
                }
                */
                
            }
            $dataSources[] = $chartDataSource;
        }

        $a = $chartDataRow->toArray();
        $a['data_aggregator'] = array(
            'aggregator' => $chartDataRow->aggregator_object,
            'parameters' => $chartDataRow->aggregator_init);

        unset($a['aggregator_object']);
        unset($a['aggregator_init']);


        $a['data_sources'] = $dataSources;
        $this->populate( $a );

        return $this;
    }

    // functions following a standard patterns
    public function toArray() {
        $data_sources = array();
        if($this->_chartDataSource)
            foreach($this->_chartDataSource as $data_source) {
                $data_sources[] = $data_source->toArray();
            }
        $ret = array();
        if($this->_title)
                $ret['title'] = $this->getTitle();

        if($this->_timeFilterCode)
                $ret['time_filter_code'] = $this->getTimeFilterCode();
        else
                $ret['time_filter_code'] = '';

        if($this->_dataResolution)
                $ret['data_resolution'] = $this->getDataResolution();

        if($this->_aggregator)
                $ret['data_aggregator'] = $this->getAggregator();

        if(!is_null($this->_aggregator_id))
                $ret['aggregator_id'] = $this->getAggregatorId();

        if($this->_description)
                $ret['description'] = $this->getDescription();

        if($this->_chartType)
                $ret['chart_type'] = $this->getChartType();
        
        $ret['flags'] = $this->_flags ? $this->_flags : "";

        $ret['data_sources'] = $data_sources;

        return $ret;
    }

    public function getDataTypes() {
        $a = array();
        foreach($this->_chartDataSource as $ds)
            foreach($ds->getDataTypes() as $b) {
              $a[$b] = 1;
            }
        return array_keys($a);
    }

    public function populate(array $a = NULL) {
        if(!$a)
            throw new Webreport_Exception("Populate needs an array");

        if(array_key_exists("title", $a))
            $this->setTitle ($a['title']);

        if(array_key_exists("time_filter_code", $a))
            $this->setTimeFilterCode ($a['time_filter_code']);

        if(array_key_exists("data_resolution", $a))
            $this->setDataResolution ($a['data_resolution']);

        if(array_key_exists("description", $a))
            $this->setDescription($a['description']);

        if(array_key_exists("data_aggregator", $a))
            $this->setAggregator($a['data_aggregator']);

        if(array_key_exists("aggregator_id", $a)){
            $this->setAggregatorId($a['aggregator_id']);
        }

        if(array_key_exists("company_id", $a)){
            $this->setCompanyId($a['company_id']);
        }
        $this->_flags = "";
        if(array_key_exists("flags", $a)){
            $this->_flags = $a['flags'];
        }

        if(array_key_exists("user_id", $a)){
            $this->setUserId($a['user_id']);
        }

        if(array_key_exists("data_sources",$a)) {
            $sources = array();

            foreach($a['data_sources'] as $data_source_array) {
                $data_source = new Charts_Model_ChartDataSource();
                $data_source->populate($data_source_array);
                $sources[] = $data_source;
            }

            $this->_chartDataSource = $sources;
        }

        if(array_key_exists("chart_type",$a)) {
            $this->_chartType = $a['chart_type'];
        }
        return $this;
    }
    
    /*
     * Returns true if $user can view all data sources in the chart.
     */
    public function isVisible($user) {

        $allowedSourceIds = array();
        $usedSourceIds = array();

        if ( $user->multi_company || $user->administrator ) return true;

        $allowedChartSourceTypes = Charts_Model_SourceTypes::getInstance();
        $allowedChartSources = $allowedChartSourceTypes->getSourcesByType($user);
        foreach ($allowedChartSources as $cs){
            foreach ($cs as $source){
                $allowedSourceIds[$source['sourcetype']][] = $source['id'];
            }
        }

        $usedSources = $this->toArray();
        $usedSources = $usedSources['data_sources'];
        foreach ($usedSources as $us) {
            foreach ($us['data'] as $sourceObject){
                $usedSourceIds[$sourceObject->identifier][] = $sourceObject->id;
            }
        }
        foreach($usedSourceIds as $key => $value){
            if(array_key_exists($key, $allowedSourceIds)){
                $intersect = array_intersect($value, $allowedSourceIds[$key]);
                if ($intersect != $value) {
                    return false;
                }
            } else {
                return false;
            }
        }
        return true;

       /* Commented by andersg 2015-01-10 in work for 3.4 release, but has propably not been working/used since sourcetypes were introduced.
        $camerasDb = new Core_Model_Cameras();
        $virtualCamerasDb = new Core_Model_VirtualCameras();
        $formulaSetsDb = new Core_Model_CountData_FormulaSets();
        $countDataSetsDb = new Core_Model_CountData_Sets();

        $cameraIds = array();
        $virtualCameraIds = array();
        $formulaSetIds = array();
        $countDataSetIds = array();

        foreach($this->getDataSources() as $ds) {
            $cameraIds = array_merge($cameraIds, $ds->getCameraIds());
            $virtualCameraIds = array_merge($virtualCameraIds, $ds->getVirtualCameraIds());
            $formulaSetIds = array_merge($formulaSetIds, $ds->getFormulaIds());
            $countDataSetIds = array_merge($countDataSetIds, $ds->getDataSetIds());
        }
        $cameraIds = array_unique($cameraIds);
        $virtualCameraIds = array_unique($virtualCameraIds);
        $formulaSetIds = array_unique($formulaSetIds);
        $countData = array_unique($countDataSetIds);        
        
        $allowedCameraIds = array_keys($camerasDb->fetchAllVisibleNames($user));
        $allowedVirtualCameraIds = array_keys($virtualCamerasDb->fetchAllVisibleNames($user));
        $allowedFormulaSetIds = array_keys($formulaSetsDb->fetchAllVisibleNames($user));
        $allowedCountDataSetIds = array_keys($countDataSetsDb->fetchAllVisibleNames($user));

        if(count($cameraIds) != count(array_intersect($cameraIds, $allowedCameraIds))) {
            return FALSE;
        }

       if(count($virtualCameraIds) != count(array_intersect($virtualCameraIds, $allowedVirtualCameraIds)))
            return FALSE;

       if(count($formulaSetIds) != count(array_intersect($formulaSetIds, $allowedFormulaSetIds)))
            return FALSE;

       if(count($countDataSetIds) != count(array_intersect($countDataSetIds, $allowedCountDataSetIds)))
            return FALSE;

        return TRUE;
    */
    }
}
