<?php

class Charts_Model_ChartDataList extends Core_Model_DocumentTableAbstract
{
    protected $_name = 'mod_charts_data';
    protected $_primary = 'chart_data_id';

    protected $_dependentTables = array('Charts_Model_ChartDataSourceList');

    protected $_referenceMap = array(
        'User' => array(
            'columns' => array('user_id'),
            'refTableClass' => 'Core_Model_Users',
            'refColumns' => array('user_id')
        ),
        'Company' => array(
            'columns' => array('company_id'),
            'refTableClass' => 'Core_Model_Companies',
            'refColumns' => array('company_id')
        )
    );
    public function __construct() {
        parent::__construct();
    }

    public function userCanSeeChart($chartDataId,$identity){
        $list = $this->fetchAllVisible($identity);

        foreach($list as $chart){
            if($chart['chart_data_id'] == $chartDataId){
                return true;
            }
        }
        return false;
    }

    public function fetchAllVisible($user) {
        $groupsDb = new Core_Model_Folders();
        $groups = $groupsDb->getViewableGroups($user);
       
        $charts = array();

        foreach($groups as $group) {
            $documents = $group->findDependentRowset('Core_Model_Documents');
            foreach($documents as $document) {
                $chart = $this->find($document->id)->current();
                if($chart)
                    $charts[] = $chart;
            }
        }

        return $charts;
    }

    public function fetchAllVisibleSorted($user) {
        $groupsDb = Core_Model_Folders::getInstance();
        $groups = $groupsDb->getViewableGroups($user); 

        $ret = array();
        $companiesDb = new Core_Model_Companies();

        foreach($groups as $group) {
            $select = $this->select()->
                setIntegrityCheck(FALSE)->
                from(array('c' => 'mod_charts_data'), array('*'))->
                join(array('d' => 'document'), 'c.chart_data_id = d.id', array())->
                where('d.type = ?', 'chart')->
                where('d.group_id = ?', $group->group_id)->
                order('c.title');

            $charts = $this->fetchAll($select);
            foreach($charts as $chart) {
                if($user->multi_company){
                    $ret[$companiesDb->find($group->company_id)->current()->name . ": " . $group->name][$chart->chart_data_id] = $chart->title;
                }else 
                    $ret[$group->name][$chart->chart_data_id] = $chart->title;
            }
        }

        return $ret;

        uksort($charts, "strcasecmp");
        $chartsSorted = array();
        foreach($charts as $k => $companyCharts){
            uasort($companyCharts, "strcasecmp");
            $chartsSorted[$k] = $companyCharts;
        }

        return $chartsSorted;
    }

    public function findNextCopyName($id){
        $chartDb = new Charts_Model_ChartDataList();
        $chart = Charts_Model_ChartData::factoryFromDatabase($id);

        $user = Core_Model_Users::getUser($chart->getUserId());
        $company = Core_Model_Companies::getCompany($chart->getCompanyId());

        $visible = $chartDb->fetchAllVisible($user);
        preg_match('/^.*\\(copy/',$chart->getTitle(), $matches);
        if(count($matches) > 0) $base = substr($matches[0],0,-strlen(' (copy'));
        else  $base = $chart->getTitle();

        for($i = 0; $i <= 20; $i++) {
            $tryTitle =  $base . " (copy" . ($i > 0 ? " $i)" : ")");
            foreach($visible as $k => $v){
                if($tryTitle == $v->title) break;
            }
            if($tryTitle == $v->title) continue;
            break;
        }
        if($i > 20) $tryTitle = $base . ' (copy 20+)';
        return $tryTitle;
    }

    /*
     * Return a list of Charts that belong to $group.
     */
    public function fetchDocuments($group) {
        // TODO: Write a nice JOIN.
        $documents = $group->getDocuments('chart');
        $chartsDb = new Charts_Model_ChartDataList();

        $charts = array();
        foreach($documents as $doc) {
            $chart = $chartsDb->find($doc->id)->current();
            if($chart)
                $charts[] = $chart;
        }

        return $charts;
    }
    
}
