<?php
class Charts_Model_ChartDataSource
{
    protected $_title = NULL;
    protected $_today = NULL;
    protected $_color = NULL;
    protected $_desc = NULL;
    protected $_appearance = NULL;
    protected $_timeFilterCode = NULL;
    protected $_timePeriodCode = NULL;
    protected $_timePeriodFrom = NULL;
    protected $_timePeriodTo = NULL;
    protected $_timePeriodToTitle = FALSE;

    //protected $_cameraIds = array();
    //protected $_cameraIdTypes = array();

    //protected $_virtualCameraIds = array();
    //protected $_virtualCameraIdTypes = array();

    //protected $_dataSetIds = array();
    //protected $_dataSetIdTypes = array();

    protected $_virtualId = NULL;
    public    $_data = array();

    protected $_posDataIds = array();
    protected $_posDataIdTypes = array();

    //protected $_formulaIds = array();
    //protected $_formulaIdTypes = array();

    const STANDARD_COLOR = "FF0000";
    const STANDARD_TITLE = "Dataset";
    const STANDARD_TIME_FILTER = NULL;

    public function __construct() {
        $this->_today = new Zend_Date();
    }

    public function setToday($date) {
        $this->_today = $date;
        return $this;
    }

    public function setVirtual($id) {
        $this->_virtualId = $id;
    }

    public function getDataTypes() {
        $axes = $this->getAxes()->getAxesInfo();
        return array_keys($axes);
    }

    public function getCurrency() {
        $jobDb = new Pos_Model_ImportJobs();
        foreach($this->_data as $d) {
            if(in_array($d->identifier, Charts_Model_SourceTypes::getPosIdentifiers(1))){
                return $jobDb->find($d->id)->current()->currency;
            }
        }
    }

    /* POS is unfortunately a special case of Count Data */
    public function splitPos() {
        $all = array();
        $choices = array();
        foreach($this->_data as $id => $chosen) {
            $source = Charts_Model_SourceTypes::getSourceType($chosen->identifier);
            if(method_exists($source, "splitSelection"))
                $split = $source->splitSelection($chosen);
            else
                $split = array($chosen);
            foreach($split as $splitid => $choice) {
                $group = "default";
                if(isset($choice->group)) $group = $choice->group;
                $choices[$group][] = $choice;
            }
        }
        $all = array();
        foreach($choices as $identifier => $choice) {
            $new = clone $this;
            $new->_data = $choice;
            if(count($choices) > 1) {
                $translate = Zend_Registry::get('Zend_Translate');
                $primaryIdentifier = $choice[0]->identifier;
                $source = Charts_Model_SourceTypes::getSourceType($primaryIdentifier);
                $primaryAxis = current($source->getAxes(NULL));
                $new->setDescription($translate->translate($primaryAxis->getTitle()));
            }
            $all[] = $new;
        }
        return $all;
    }

    public function hasGrouping() {
        foreach ($this->_data as $data) {
            if ( isset($data->group) ) return TRUE;
        }
        return FALSE;
    }

    public function splitSource() {
        $singelAxes = array();
        $singelAxesData = array();
        $all = $this->splitPos();
        foreach($all as $id => $data ) {
            if ( $data->hasGrouping()) {
                return $all;
            }
        }

        foreach($this->_data as $id => $data) {
            $dataId = Charts_Model_SourceTypes::getSourceType($data->identifier);
            if(method_exists($dataId, "splitSelection"))
                $splitData = $dataId->splitSelection($data);
            else
                $splitData = array($data);
            foreach($splitData as $singelData) {
                $dataType = Charts_Model_SourceTypes::getSourceType($singelData->identifier);
                $singelDataAxis = current($dataType->getAxes($singelData->dataTypeId));
                $singelAxes[] = $singelDataAxis;
                $singelAxesData[] = $singelData;
            }
        }

        //merge with own kind and sort out percentage
        $axes = array();
        $axesData = array();
        $percentageAxes = array();
        $percentageAxesData = array();
        foreach($singelAxes as $i=>$axis){
            if(isset($axis->tryMerge) && $axis->tryMerge == 1){
                $ind = $this->_array_search($axis,$percentageAxes);
                if($ind !== false){
                    $percentageAxesData[$ind][]=$singelAxesData[$i];
                }else{
                    $percentageAxesData[count($percentageAxes)][] = $singelAxesData[$i];
                    $percentageAxes[] = $axis;
                }               
            }else{
                $ind = $this->_array_search($axis,$axes);
                if($ind !== false){
                    $axesData[$ind][]=$singelAxesData[$i];
                }else{
                    $axesData[count($axes)][] = $singelAxesData[$i];
                    $axes[] = $axis;
                }
            }
        }
        //merge with other types
        foreach($percentageAxes as $ip => $ax){
            $nbrMerge = 0;
            foreach($axes as $i => $ax2){
                $mergeAxis = $ax->merge($ax2);
                if(!is_null($mergeAxis)){
                    $nbrMerge++;
                    $axes[$i]=$mergeAxis;
                    foreach($percentageAxesData[$ip] as $data)
                        $axesData[$i][]=$data;
                }        
            }
            if($nbrMerge == 0){
                $nbrAx = count($axes);
                $axes[$nbrAx]=$ax;
                $axesData[$nbrAx]=$percentageAxesData[$ip];
            }
        }
        //make it in to datasources
        $all = array();
        foreach($axes as $i => $ax) {
            $new = clone $this;
            $new->_data = $axesData[$i];
            if(count($axes) > 1) {
                $translate = Zend_Registry::get('Zend_Translate');
                $new->setDescription($translate->translate($ax->getTitle()));
                if(method_exists($ax,'getTitleShort'))
                    $new->setDescription($translate->translate($ax->getTitleShort()));
            }
            $all[] = $new;

        }
        return $all;
    }


    private function _array_search($obj, $array){
        foreach($array as $i => $element){
            if(get_class($element)==get_class($obj))
                return $i;
        }
        return false;
    }

    public function splitSourceold() {
        $axes = array();
        $sourceGroups = array();
        $nbrGroups = 0;
        foreach($this->_data as $id => $data) {
            $dataId = Charts_Model_SourceTypes::getSourceType($data->identifier);
            if(method_exists($dataId, "splitSelection"))
                $splitData = $dataId->splitSelection($data);
            else
                $splitData = array($data);
            foreach($splitData as $singelData) {
                $isPlaced = false;
                $dataType = Charts_Model_SourceTypes::getSourceType($singelData->identifier);
                $singelDataAxis = current($dataType->getAxes($singelData->dataTypeId));
                foreach($axes as $ind => $axis){
                    $mergeAxis = $singelDataAxis->merge($axis);
                    if(!is_null($mergeAxis)){
                        $sourceGroups[$ind][] = $singelData;
                        $axes[$ind] = $mergeAxis;
                        $isPlaced = true;
                    }
                }
                if(!$isPlaced){
                    $nbrGroups ++;
                    $sourceGroups['group' . $nbrGroups][] = $singelData;
                    $axes['group' . $nbrGroups] = $singelDataAxis;
                }
            }
        }
        $all = array();
        foreach($sourceGroups as $groupName => $group) {
            $new = clone $this;
            $new->_data = $group;
            if(count($sourceGroups) > 1) {
                $translate = Zend_Registry::get('Zend_Translate');
                $new->setDescription($translate->translate($axes[$groupName]->getTitle()));
            }
            $all[] = $new;
        }
        return $all;
    }

    public function virtualId(){
        return $this->_virtualId;
    }

    public function getAxes() {
        $axes = new Charts_Model_Axes();
        foreach($this->_data as $chosen) {
            $source = Charts_Model_SourceTypes::getSourceType($chosen->identifier);
            $axis = $source->getAxes($chosen->dataTypeId);
            $axes->addAxes($axis);
        }
        $axes->merge();
        return $axes;
    }

    public function grayColor($factor,$light = 1){
        $r = hexdec(substr($this->_color,0,2)) * $light;
        $g = hexdec(substr($this->_color,2,2)) * $light;
        $b = hexdec(substr($this->_color,4,2)) * $light;
        $r = $r > 255 ? 255 : $r;
        $g = $g > 255 ? 255 : $g;
        $b = $b > 255 ? 255 : $b;
        $avg = ($r+$g+$b)/3;
        $compf = 1 - $factor;
        $r = $r * $compf + $avg*$factor;
        $g = $g * $compf + $avg*$factor;
        $b = $b * $compf + $avg*$factor;
        $this->_color = "";
        $this->_color .= substr(dechex(256+$r),1,2);
        $this->_color .= substr(dechex(256+$g),1,2);
        $this->_color .= substr(dechex(256+$b),1,2);
    }
/*
    public function setCountDataFromPos(Core_Model_PosData $posData){
        $this->_cameraIdTypes = $posData->getCamerasTypes();
        $this->_cameraIds = $posData->getCameras();
    }
*/
    public function getTitle() {
        if(!$this->_title)
            return self::STANDARD_TITLE;
        return $this->_title;
    }

    public function getTitleFormated() {
        $dateStr = "";
        if (is_null($this->_timePeriodCode)) {
            $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();
            $dateStr = "[" . $this->_timePeriodFrom->toString($dateFormat) . " - " . $this->_timePeriodTo->toString($dateFormat) . "]";
            $this->_title = str_replace("%a", "", $this->_title);
        } else {
            $int = new CM_Interval($this->_today, $this->_timePeriodCode);
            $dateStr = $int->toReadable();
            $this->_title = str_replace("%a", $int->legacyToReadable(), $this->_title);
        }
        if ($this->_desc!=NULL && $this->_timePeriodToTitle) {
                $ret = $this->_title . " " . $dateStr . ", " . $this->_desc;
        } elseif ($this->_desc!=NULL && !$this->_timePeriodToTitle) {
                $ret = $this->_title . " " . $this->_desc;
        } elseif ($this->_desc==NULL && $this->_timePeriodToTitle) {
            $ret = $this->_title . " " . $dateStr;
        } else {
            $ret = $this->_title;
        }

        return $ret;
    }

    public function setTitle($title){
        $this->_title = $title;
        return $this;
    }

    public function setTimePeriodToTitle($timePeriodToTitle) {
        $this->_timePeriodToTitle = $timePeriodToTitle;
        return $this;
    }

    public function getTimePeriodToTitle() {
        return $this->_timePeriodToTitle;
    }

    public function setDescription($desc) {
        $this->_desc = $desc;
        return $this;
    }

    public function setAppearance($app){
        $this->_appearance = $app;
        return $this;
    }

    public function getAppearance(){
        return $this->_appearance;
    }

    // Lean is used to create a second color, with a brightness different.
    // If lean is 0, getColor will return the darkest of the two.
    public function getColor($lean = FALSE) {
        if(!$this->_color)
            return self::STANDARD_COLOR;
        if($lean === FALSE) return $this->_color;
        if($lean != 1 && $lean != 0) throw new Exception("Lean must be 1 or 0.");
        $col = hexdec($this->_color);
        $b = $col & 255;
        $g = ($col>>8) & 255;
        $r = ($col>>16) & 255;
        $bright = ($r+$g+$b) > 128*3 ? 1 : 0;
        if($bright == $lean) return $this->_color;
        if($bright) {
            $r = $r*3/6;
            $g = $g*3/6;
            $b = $b*3/6;
        } else {
            $r = ($r*2+3*255)/5;
            $g = ($g*2+3*255)/5;
            $b = ($b*2+3*255)/5;;
        }
        $r = ($r > 15 ? "" : "0") . dechex($r);
        $g = ($g > 15 ? "" : "0") . dechex($g);
        $b = ($b > 15 ? "" : "0") . dechex($b);
        return $r.$g.$b;
    }

    public function setColor($color) {
        $this->_color = $color;
        return $this;
    }

    public function getTimeFilterCode() {
        if(!$this->_timeFilterCode)
            return self::STANDARD_TIME_FILTER;
        return $this->_timeFilterCode;
    }

    public function setTimeFilterCode($timeFilterCode) {
        $this->_timeFilterCode = $timeFilterCode;
        return $this;
    }
/*
    public function getFormulaIdTypes(){
        return $this->_formulaIdTypes;
    }
*/
   public function setTimePeriodCode($timePeriodCode) {
        $ret = false;

        if($timePeriodCode == "all") {
            $this->_timePeriodTo = null;
            $this->_timePeriodFrom = null;
        }
        if($timePeriodCode == "fix" || $timePeriodCode == "" || $timePeriodCode == "all") {
            $timePeriodCode = null;
        }

        if($timePeriodCode) {
            try {
                $ret = CM_Interval::isValidCode($timePeriodCode);
            } catch(WebReport_Exception $e) {
                $ret = false;
            }
            if(!$ret) {
                throw new WebReport_Exception_ParameterError("Not a valid time period code");
            }
        }

        $this->_timePeriodCode = $timePeriodCode;

        return $this;
    }


    public function setTimePeriodFixed($from,$to) {
        try{
            $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();
            if ( !$dateFormat ) $dateFormat = "yyyy-MM-dd";

            if(is_string($from)) {
                $from = new Zend_Date($from, $dateFormat/*, Zend_Registry::get('Zend_Locale')*/);
            }
            if(is_string($to)) {
                $to = new Zend_Date($to, $dateFormat/*, Zend_Registry::get('Zend_Locale')*/);
            }
        } catch(Zend_Date_Exception $e){
            throw new WebReport_Exception_ParameterError("From or Too date is not of format: $dateFormat.");
        }
        if(get_class($to) != 'Zend_Date' || get_class($from) != 'Zend_Date') {
            throw new WebReport_Exception_ParameterError("In parameters must be string or Zend_Date");
        }
        $this->_timePeriodFrom = $from;
        $this->_timePeriodTo = $to;
        return $this;
    }
/*  Commented by andersg 2015-01-10 in work for 3.4 release, but has propably not been working/used since sourcetypes were introduced.
    public function getCameraIds() {
        return $this->_cameraIds;
    }

    public function getCameraIdTypes() {
        return $this->_cameraIdTypes;
    }


    public function setCameraIds(array $cameraIds, array $types=array()) {
        $this->_cameraIds = $cameraIds;
        $this->_cameraIdTypes = $types;
        return $this;
    }

    public function getVirtualCameraIds() {
        return $this->_virtualCameraIds;
    }

    public function getVirtualCameraIdTypes() {
        return $this->_virtualCameraIdTypes;
    }

    public function setVirtualCameraIds(array $virtual_camera_ids, array $types=array()) {
        $this->_virtualCameraIds = $virtual_camera_ids;
        $this->_virtualCameraIdTypes = $types;
        return $this;
    }

    public function getDataSetIds() {
        return $this->_dataSetIds;
    }

    public function getPosDataIds() {
        return $this->_posDataIds;
    }

    public function getFormulaIds() {
        return $this->_formulaIds;
    }
    
    public function getDataSetTypes() {
        return $this->_dataIdTypes;
    }

    public function getPosDataIdTypes() {
        return $this->_posDataIdTypes;
    }

    public function clearPosData(){
        $this->_posDataIds = array();
        $this->_posDataIdTypes = array();
        return $this;
    }

    public function clearCountData(){
        $this->_cameraIds = array();
        $this->_cameraIdTypes = array();

        $this->_virtualCameraIds = array();
        $this->_virtualCameraIdTypes = array();

        $this->_dataSetIds = array();
        $this->_dataSetIdTypes = array();

        $this->_formulaIds = array();
        $this->_formulaIdTypes = array();
        return $this;
    }

    public function setDataSetIds(array $data_set_ids, array $types = array()) {
        $this->_dataSetIds = $data_set_ids;
        $this->_dataSetIdTypes = $types;
        return $this;
    }

    public function setPosDataIds(array $pos_data_ids, array $types = array()) {
        $this->_posDataIds = $pos_data_ids;
        $this->_posDataIdTypes = $types;
        return $this;
    }

    public function setFormulaIds(array $formula_ids, array $types = array()) {
        $this->_formulaIds = $formula_ids;
        $this->_formulaIdTypes = $types;
        return $this;
    }


    public function getDataSetIdTypes() {
        return $this->_dataSetIdTypes;
    }
*/
    public function getTimePeriodCode() {
        return $this->_timePeriodCode;
    }

    public function hasTimePeriodLimits() {
        if(is_null($this->_timePeriodCode) &&
            (is_null($this->_timePeriodFrom) || is_null($this->_timePeriodTo)) ) {
                return false;
            }
        return true;
    }
    //end of getters and setters
    //custom objected
    public function getTimePeriodLimits() {
        if(!$this->hasTimePeriodLimits()) {
                throw new WebReport_Exception("Not enough time information specified");
            }

        $from = NULL;
        $to = NULL;

        // code has presidence
        if(!is_null($this->_timePeriodCode)) {
            $interval = new CM_Interval($this->_today, $this->_timePeriodCode);
            //throwa exception här
            $from = $interval->getStartDate();
            $to =  $interval->getStopDate();
        } else {
            $from = $this->_timePeriodFrom;
            $to = $this->_timePeriodTo;
        }

        return array("from"=> $from, "to" => $to);
    }

    public function getPosData(){
        $posData = new Core_Model_PosData();
        foreach($this->_posDataIds as $i => $id){
            $posData->addPosDataData($id,$this->_posDataIdTypes[$i], $this->_virtualId);
        }
        return $posData;
    }

    public function getCountData($type = NULL, $separate=False) {
        /* @var $countData Core_Model_CountData */
        $sources = Charts_Model_SourceTypes::getInstance();
        $countDatas = array();
        if(!$separate) $countDatas['all'] = new Core_Model_CountData();
        foreach($this->_data as $data) {
            $selections = Charts_Model_SourceTypes::getSelections($data->identifier, $data->dataTypeId);
            foreach($selections as $dataTypeId) {
                $source = Charts_Model_SourceTypes::getSourceType($data->identifier);
                if($source->isCountData($dataTypeId)) {
                    $key = !$separate ? 'all' : $dataTypeId;
                    if(!$type || $source->getCountType() == $type) {
                        if(!isset($countDatas[$key])) $countDatas[$key] = new Core_Model_CountData();
                        $source->addCountData($data->id, $dataTypeId, $countDatas[$key]);
                    }
                } else {
                  error_log($dataTypeId."@".$data->identifier . " is not count data...");
                }
            }
        }
        if(!$separate) return $countDatas['all'];
        return $countDatas;
    }

   public function toArray() {
        $ret = array(
             "data" => $this->_data,
             "title"=> $this->getTitle(),
             "appearance"=> $this->getAppearance(),
             "color"=> $this->getColor(),
             "time_filter_code" => $this->getTimeFilterCode(),
             "time_period_code" => $this->getTimePeriodCode(),
             "time_period_manual" => $this->getTimePeriodCode(),
             "time_period_from" => $this->_timePeriodFrom,
             "time_period_to" => $this->_timePeriodTo,
             "time_period_to_title" => $this->getTimePeriodToTitle(),
             /*,
             "camera_ids" => $this->getCameraIds(),
             "camera_id_types" => $this->getCameraIdTypes(),
             "virtual_camera_ids" => $this->getVirtualCameraIds(),
             "virtual_camera_id_types" => $this->getVirtualCameraIdTypes(),
             "data_set_ids" => $this->getDataSetIds(),
             "data_set_id_types" => $this->getDataSetIdTypes(),
             "pos_data_ids" => $this->getPosDataIds(),
             "pos_data_id_types" => $this->getPosDataIdTypes(),
             "formula_ids" => $this->getFormulaIds(),
             "formula_id_types" => $this->getFormulaIdTypes()
*/
            );
        return $ret;
    }

    public function populate(array $a=NULL) {
        if(!$a)
            throw new Webreport_Exception("Populate needs an array");

        if(array_key_exists("title" , $a))
            $this->setTitle($a['title']);

        if(array_key_exists("color", $a))
            $this->setColor($a['color']);

        if(array_key_exists("appearance",$a))
            $this->setAppearance($a['appearance']);

        if(array_key_exists("time_filter_code", $a))
            $this->setTimeFilterCode($a['time_filter_code']);
        try{
            if(array_key_exists("time_period_from", $a) && array_key_exists("time_period_to",$a)){
                if(strlen($a['time_period_from']) > 4 && strlen($a['time_period_to']) > 4){
                    $this->setTimePeriodFixed($a['time_period_from'], $a['time_period_to']);
                    $this->setTimePeriodCode(null);
                }
            }
        }catch(WebReport_Exception_ParameterError $e){
            if(!array_key_exists("time_period_code", $a)){
                throw new WebReport_Exception("No time interval in form.");
            }
        }

        if(array_key_exists("time_period_code", $a))
            $this->setTimePeriodCode($a['time_period_code']);

        if(array_key_exists("time_period_to_title", $a))
            $this->setTimePeriodToTitle($a['time_period_to_title']);

        if(array_key_exists("data",$a)) {
          $this->_data = $a['data'];
        }

        return $this;
    }

    public function getChosen() {
        return $this->_data;
    }
}

