<?php

abstract class Charts_Model_ChartRender_Abstract
{
    const DATATYPE_COUNT_DATA = 10;
    protected $_code = ""; // unique code for this format
    protected $_content_type = "text/html"; //mime content type for this format
    protected $_file_extension = ".json";
    protected $_extra_headers = NULL;
    protected $_force_download = false;

    //used during rendering
    protected $_title = NULL; // set title of the whole shabang
    public    $_dataLabels = NULL; //array of the x-axis labels;
    public    $_dataLabelsRaw = NULL; //array of the x-axis labels, not translated;
    public    $_resolution = NULL;  //time resolution on x-axis;
    public    $_dataSourceTitles = NULL; //label for the data-set;
    public    $_dataSourceColors = NULL; //colors that are used for the sources
    public $_dataSourceData = NULL; // array of arrays containing the data;
    public $_dataSourceType = NULL; // type of data using enum DataType.
    public    $_dataSourceAppearance = NULL; // type of data using enum DataType.
    public    $_numberOfDataSources = NULL;
    public    $_dataLength = NULL;
    protected $_multipleAxes = NULL;
    protected $_use_cache = FALSE;

    protected $_appearance = NULL;

    protected $_height = 400;
    protected $_width = 800;
    protected $_force_wanted_resolution = false;
    protected $_add_file_type_to_header = true;

    protected $_latestEndUserError = NULL;
    protected $_generateDate = NULL;

    public function setGenerateDate($date){
        $this->_generateDate = $date;
    }

    public function forceDownload($force = true) {
        $this->_force_download = $force;
    }

    public function getStatus(){
        //Check for three axes => can not display chart but can export data as csv
        $translate = Zend_Registry::get('Zend_Translate');
        if(count($this->_axes)>2){
            $status =   $translate->_("YOU_HAVE_SELECTED_TOO_MANY_DATA_UNITS,_THE_CHART_CAN_NOT_BE_DISPLAYED._DO_YOU_WANT_TO_REMOVE_YOUR_LAST_SELECTION?");
            return $status;
        }else if(isset($this->_dataSourceStackId) && max($this->_dataSourceStackId) > 4){
            $status =   $translate->_("IT_IS_NOT_POSSIBLE_TO_HAVE_MORE_THAN_THREE_STACKED_BARS._DO_YOU_WANT_TO_REMOVE_YOUR_LAST_SELECTION?");
            return $status;
        }
    }

    public function useCache($use = TRUE) {
        $this->_use_cache = $use;
    }


    public function getWidth() {
        return $this->_width;
    }

    public function setWidth($width) {
        $this->_width = $width;
        return $this;
    }

    public function getHeight() {
        return $this->_height;
    }

    public function setHeight($height) {
        $this->_height = $height;
        return $this;
    }

    public function getCode() {
        return $this->_code;
    }

    public function getMimeType() {
        return $this->_content_type;
    }

    public function getFileExtension() {
        return $this->_file_extension;
    }

    public function rightUnitIsPrefix() {
        if($this->_rightUnit == "%") return FALSE;
        if($this->_rightUnit && mb_strlen($this->_rightUnit,"UTF-8") == 1) return TRUE;
        return FALSE;
    }

    public function getRightUnit(){
        if($this->_rightUnit == "%") return "%";
        if(!$this->rightUnitIsPrefix()) return " " . $this->_rightUnit;
        return $this->_rightUnit;
    }

    public function getLongUnit(){
        $translate = Zend_Registry::get('Zend_Translate');
        if($this->_unit == null) return $translate->_("PERSONS");
        else if($this->_unit == "%") return $translate->_("PERCENTAGE");
        else return $this->_unit;
    }

    public function getEndUserError(){
        $translate = Zend_Registry::get('Zend_Translate');
        return $translate->_($this->_latestEndUserError);
    }

    abstract function _render();

    private function obj2array($objectList) {
        // Convert list of objects to list of ints
        $list = Array();
        foreach($objectList as $key => $obj) {
            $sum = 0;
            foreach((array)$obj as $k => $v) {
                if ( $k == "fields" ) continue;
                if ( strpos($k, "_exists") !== FALSE) continue;
                $sum += $v;
            }
            $list[$key] = $sum;
        }
        return $list;
   }

    private function fill_gaps($data, $agg, $cache, $times) {
        $labels = $agg->getXLabelsRaw();
        $final = array_fill(0,count($times),0);
        $nbrAdds = array_fill(0,count($times),0);
        $map = array();
        foreach($times as $i => $t) {
            $map[$t->toString('yyyyMMddHHmmss')] = $i;
        }
        $res = $agg->getResolution();
        foreach($data as $t => $d)  {
            //$date = new Zend_Date($labels[$t], "yyyy-MM-dd HH:mm:ss");
            //$date = Core_Model_CountData::truncateDate($date, $res);
            //$mapkey = $date->toString('yyyyMMddHHmmss');
            $mapkey = Core_Model_CountData::truncateDateString($labels[$t], $res); 
            if(!isset($map[$mapkey])) {
                error_log("Skipping mapkey $mapkey, was not found in ". print_r($map,true));
                continue;
            }
            $id = $map[$mapkey];

            $final[$id] = $final[$id] + current((array)$d);
            $nbrAdds[$id] ++;
        }

        if($agg->getIsAverage()){
            foreach($final as $i => $bin){
                if($nbrAdds[$i] > 0){
                    $final[$i] = round($final[$i]/$nbrAdds[$i],1);
                }
            }
        }
    
        return $final;
    }

    protected function findGoodLimits($min, $max) {
        $dif = $max-$min;
        $min = max(0,$min-1-round($dif*0.05));
        $max += 1+round($dif*0.05);
        return array($min,$max);
    }

    public function _renderCommon(Charts_Model_ChartData $chartData, $perma = false, $forceAverage = false) {
        $translate = Zend_Registry::get('Zend_Translate');
        if($chartData == null){
            throw new Exception ("Can't render ChartData which is null.");
            r_log("Can't render ChartData which is null.");
        }
        if(!$perma) $cache = Zend_Registry::get('timestamp-cache');
        else $cache = NULL;

        $this->_latestEndUserError = NULL;
        /* @var $chartDataSource Charts_Model_ChartDataSource */
        /* @var $agg Charts_Model_DataAggregatorAbstract */
        $this->_title = $chartData->getTitle();
        $this->_dataLabels = NULL; // populated on first agg
        $this->_dataLabelsRaw = NULL; // populated on first agg
        $this->_currency = NULL;
        $axes = $chartData->getAxes();
        $this->_axes = $axes->getAxesInfo();
        $this->_axisUnits = $axes->getUnits();
        $this->_axisTypes = $axes->getAxesDrawTypes();
        $this->_chartType = $chartData->getChartType();
        $this->_singleAxis = True;

        $this->_dataSourceColors  = array();
        $this->_dataSourceData    = array();
        $this->_dataSourceTitles  = array();
        $this->_simpleTitles      = array();
        $this->_unit              = array();
        $this->_dataSourceDataSum = array();
        $this->_dataSourceType    = array();
        $this->_dataSourceAppearance = array();
        $this->_minValue = array();
        $this->_maxValue = array();
        $this->_numberOfDataSources = 0;
        $this->_dataLength = 0;
        if ( $this->_generateDate == NULL ) $this->_generateDate = new Zend_Date();

        $timeFilterCode = $chartData->getTimeFilterCode();

        if($chartData->hasAggregator()) {
            $aggregator = $chartData->getAggregatorInstance();
            $aggregator->setWantedResolution($chartData->getDataResolution(), $this->_force_wanted_resolution);
        } else {
            throw new WebReport_Exception("Not implmented when setting no agg");
        }
        $this->_unit = $aggregator->getDisplayUnit();
        $this->_resolution = $aggregator->getMode();
        $sync_time = false;
        $rawLabels = array();
        if($aggregator instanceof Charts_Model_DataAggregator_Unit) {
            $sync_time = true;
        }
        $sync_time = $aggregator instanceof Charts_Model_DataAggregator_Unit;
        $is_hour = false;
        $rawLabels = array();

        $highHour = 0;
        $lowHour = 24;

        if($aggregator instanceof Charts_Model_DataAggregator_Unit) {
            $sync_time = true;
        }
        
        if($aggregator->getMode() == Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_15MIN)
            $lowHour*=4;
        $chartDataSources = $chartData->getDataSources();
        if(count($chartDataSources) == 0) return true;
        $posPostProcess = array();
        $chartData->setToday($this->_generateDate);
        if($sync_time) {
            $this->_createAllLabels($chartDataSources, $chartData, $aggregator);
            $times = $this->_times;
        }
        $sources = array();
        foreach($chartDataSources as $id => $ds) {
            $splut = $ds->splitSource();
            foreach($splut as $splat) {
                $sources[] = $splat;
                $this->_singleAxis = $this->_singleAxis && count($splat->getAxes()) <= 1;
            }
        }
        $stacks = 0;
        foreach($sources as $id => $chartDataSource) {
            //generate data
            $agg = clone $aggregator;
            $agg->useCache($this->_use_cache);
            $aggRight = null;
            $cdSet = $chartDataSource->getCountData('count');
            $units = $chartDataSource->getAxes()->getUnits();
            $noCountData = $cdSet->isEmpty();

            $ratioSets = $chartDataSource->getCountData('ratio', True);
            $scaledSets = $chartDataSource->getCountData('divbycount:1');
            $factor100 = $scaledSets->isEmpty();
            if($factor100) $scaledSets = $chartDataSource->getCountData('divbycount:100');
            $noRatioData = count($ratioSets) == 0;
            $hasScaledData = !$scaledSets->isEmpty();
            

            $posSet = $chartDataSource->getPosData();
            $dataTypeString = 'SHOULD_NOT_SHOW_123';#$chartDataSource->getUnit(); // PEOPLE
            $dataType = NULL;

            $ratio = NULL;
            /*if(count($dataTypes = $chartDataSource->getDataTypes()) > 2){
                $this->_needsSplitting = $id;    
                throw new WebReport_Exception("Too many data types chosen on one datasource.");
            }
            foreach($dataTypes as $k => $v) {
                if($dataType == NULL) $dataType = $v;
                $appearance = $this->_axisTypes[$v];
                break;
            }*/
            $dataTypes = $chartDataSource->getDataTypes();
            $dataType = $dataTypes[0];
            $appearance = $this->_axisTypes[$dataType];

            if($chartDataSource->getAppearance() != "default"){
                $appearance = $chartDataSource->getAppearance();
            }
            $this->_axisTypes[reset($dataTypes)] = $appearance;
            if($cdSet->isEmpty()) $type = $this->_appearance;
            $cdSet->setSumDataSeries(True);

            if($chartDataSource->hasTimePeriodLimits()) {
                $timePeriodLimits = $chartDataSource->getTimePeriodLimits();
                $agg->setPeriod($timePeriodLimits['from'], $timePeriodLimits['to']);
            }
            $dataSourceTimeCodeFilter = $chartDataSource->getTimeFilterCode();
            if(isset($dataSourceTimeCodeFilter)){
                $agg->setTimeFilter($dataSourceTimeCodeFilter);
            }
            else if(isset($timeFilterCode)){
                $agg->setTimeFilter($timeFilterCode);
            }
            $aggBase = clone $agg;
            if($posSet->hasPosData()){
                $agg->setPosData($posSet);
                //$agg->setCountData($posSet->getCountData());
                $dataType = $posSet->getPosType();
                $dataTypeString = Core_Model_PosData::GetTypeDescription($dataType);
                $appearance = $appearance == 'line' ? 'bar' : 'line';
            } else {
                $agg->setCountData($cdSet);
            }
            $isAverage = $forceAverage;
            $stupid = $chartDataSource->getAxes();
            foreach($stupid->getAxes() as $ax){
                if($ax->isAverage()) $isAverage = true;
            }
            $agg->setIsAverage($isAverage);
            $data = $agg->renderData();
            if($sync_time) $data = $this->fill_gaps($data, $agg, $cache,$times);
            if(is_null($this->_dataLabels)) {
                $this->_dataLabelsRaw = array();
                $this->_dataLabels = $agg->getXLabel(NULL, NULL, $this->_dataLabelsRaw);
            } else {
            }
            if($sync_time) {
                $rawLabels[] = $agg->getXLabelsRaw();
            }
            $dataFixed = array();
            $sum = 0;
            if(!$sync_time) {
                $dataFixed = $this->obj2array($data);
            } else $dataFixed = $data;

            $translate = Zend_Registry::get("Zend_Translate");
            $totOrAvgString = $translate->_("TOTAL");
            if($isAverage){
                $totOrAvgString = $translate->_("AVERAGE");
                $div = $agg->getNbrBinsWithData() == 0 ? 1 : $agg->getNbrBinsWithData();
                $sum = round(array_sum($dataFixed)/$div,1);
            } else {
                $sum = array_sum($dataFixed);
            }
            
            if(get_class($agg) == 'Charts_Model_DataAggregator_HistSum') {
                if($agg->isRelative()){
                    $sum = 100;
                    $totOrAvgString = $translate->_("TOTAL");
                }
            }
            $title = $chartDataSource->getTitleFormated();
            $finalSources = array();
            if($hasScaledData) {
                $period = $agg->getPeriod();
                $aggBase->setPeriod($period['from'], $period['to']);
                $aggBase->setTimeFilter($timeFilterCode);
                $scaleagg = clone $aggBase;
                $scaleagg->setCountData($scaledSets);
                $generated_data = $scaleagg->renderData();
                $generated_data = $this->obj2array($generated_data);
                if($sync_time) $generated_data = $this->fill_gaps($generated_data, $scaleagg, $cache, $times);
                $scaledData = $generated_data;
            }
            if(!$noRatioData) {
                $ratdata = array();
                $period = $agg->getPeriod();
                foreach($ratioSets as $rtype =>$set) {
                    $aggBase->setPeriod($period['from'], $period['to']);
                    $aggBase->setTimeFilter($timeFilterCode);
                    $ratagg = clone $aggBase;
                    $ratagg->setCountData($set);
                    $generated_data = $ratagg->renderData();
                    $generated_data = $this->obj2array($generated_data);
                    if($sync_time) $generated_data = $this->fill_gaps($generated_data, $ratagg, $cache, $times);

                    $ratdata[$rtype] =  $generated_data;
                }
                $weighted = array();
                $ratsum = array();
                foreach($ratdata as $ratType => $datalist) {
                    foreach($datalist as $k => $v) {
                        if(!isset($ratsum[$k])) $ratsum[$k] = 0;
                        if(is_array($v) && count($v) > 1) throw new Exception("Bad format from data aggregator");
                        $ratsum[$k] = $ratsum[$k] + (is_array($v) ? current($v) : $v);
                    }
                }
                $sums = array();
                foreach($ratdata as $ratType => $ratVal) {
                    $sum = 0;
                    $dividor = 0;
                    $data = array();
                    foreach($dataFixed as $i => $point) {
                        if($noCountData && !$noRatioData) $point = 100;
                        if(!isset($ratVal[$i])) {
                            $sum += $point;
                            $data[] = round(100/count($ratdata));
                            continue;
                        }
                        if(is_object($ratVal[$i])) {
                            throw new Exception("Expecting integer, not object.");
                        }
                        if($ratVal[$i] == 0) {
                            $weightedValue = $noCountData ? 0 : round($point/count($ratdata));
                        } else {
                            $weightedValue = round($point *$ratVal[$i] / $ratsum[$i]);
                        }
                        $data[] =  $weightedValue;
                        $sum += $weightedValue;
                        $dividor += 1;
                    }
                    if($isAverage && $dividor > 0) $sum = round($sum/$dividor);
                    $sums[$ratType] = $sum;
                    $weighted[$ratType] = $data;
                }
                $stacks = $stacks + 1;
                foreach($weighted as $k => $data) {
                    $category = $translate->_(strToUpper($k));
                    $fixed_title = sprintf("%s (%s: %.1f, %s)",$title, $totOrAvgString,$sums[$k],$category);
                    $simple_title = sprintf("%s (%s)",$title, $category);
                    
                    if ($k=="male" || $k=="female"){
                        $gender = $translate->_(strToUpper($k) . "S");     
                        $fixed_title = sprintf("%s, %s (%s, %.1f)",$title, $gender, $totOrAvgString ,$sums[$k]);
                        $simple_title = sprintf("%s, %s",$title, $gender);
                    }

                    if($chartDataSource->getAppearance() == "default"){  //override default decision if stacked
                        $appearance = 'bar';
                    $this->_axisTypes[reset($dataTypes)] = $appearance;
                    }
                    $finalSources[] = array('_dataSourceSum'       => $sums[$k],
                                            '_dataSourceData'      => $data,
                                            '_dataSourceType'      => $dataType,
                                            '_dataSourceAppearance'=> $appearance,
                                            '_dataSourceStackId'   => $stacks,
                                            '_simpleTitles'        => $simple_title,
                                            '_dataSourceColors'    => $chartDataSource->getColor(1-count($finalSources)),
                                            '_dataSourceTitles'    => $fixed_title);
                    $key = $dataType;
                }
                $max = 0;
                $min = 0;
                $wkeys = array_keys($weighted);
                foreach(range(0,count($data)-1) as $i) {
                    $sum = 0;
                    foreach($wkeys as $wkey) {
                        $sum = $sum + $weighted[$wkey][$i];
                    }
                    $max = max($sum,$max);
                }
            } else 
            {
                if($hasScaledData) {
                    $factor = $factor100 ? 100 : 1;
                    $totOrAvgString = $translate->_("AVERAGE");
                    $sum = 0;
                    $dividor = 0;
                    foreach($dataFixed as $i => &$__data) {
                        if($__data == 0) continue;
                        if($factor100) $__data = round(100*$scaledData[$i] / $__data);
                        else           $__data = round(100*$scaledData[$i] / $__data / 100);
                        $sum += $__data;
                        $dividor += 1;
                    }
                    $sum = $dividor > 0 ? $sum/$dividor : 0;
                }
                $title = sprintf("%s (%s: %.1f)",$title, $totOrAvgString ,$sum);
                $finalSources[] = array('_dataSourceSum'       => $sum,
                                        '_dataSourceData'      => $dataFixed,
                                        '_dataSourceStackId'   => 0,
                                        '_dataSourceType'      => $dataType,
                                        '_dataSourceAppearance'=> $appearance,
                                        '_dataSourceColors'    => $chartDataSource->getColor(),
                                        '_simpleTitles'        => $title,
                                        '_dataSourceTitles'    => $title);
                $key = $dataType;
                $max = max($dataFixed);
                $min = min($dataFixed);
            }
            if(isset($this->_minValue[$key])) $this->_minValue[$key] = min(array($this->_minValue[$key],$min));
            else $this->_minValue[$key] = $min;
            if(isset($this->_maxValue[$key])) $this->_maxValue[$key] = max(array($this->_maxValue[$key],$max));
            else $this->_maxValue[$key] = $max;
            // Add all values in finalSource to the array list with the
            // same name as the key.
            foreach($finalSources as $finalSource) {
                foreach($finalSource as $k => $v) {
                    if(!isset($this->$k)) $this->$k = array();
                    $a = $this->$k;
                    $a[] = $v;
                    $this->$k = $a;
                }
            }
            if(get_class($agg) == 'Charts_Model_DataAggregator_HistSum') {
                if($agg->isRelative()) $sum = 100;
            }
            $this->_dataLength = count($dataFixed);

            if($agg->getMode() == Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_HOUR) {
                $is_hour = true;
                $hours = $agg->getTimeFilterHours();
                $lowHour = min($lowHour, $hours[0]);
                $highHour = max($highHour, $hours[1]);
            }
            if($agg->getMode() == Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_15MIN) {
                ## REVIEW: Does not work
                $is_hour = true;
                $hours = $agg->getTimeFilterHours();
                $lowHour = min($hours[0], $lowHour);
                $highHour = max($highHour, $hours[1]+3);         
            }

            if($posSet->hasPosData() && $chartDataSource->virtualId() <= 1){
                $type = $this->_dataSourceType;
                $data = $this->_dataSourceData;
                $i = count($this->_dataSourceData)-1;
                $sum = 0;
                if($type[$i] == Core_Model_PosData::TYPE_CONVERSION_RATE){
                    $posPostProcess[] = array('i' => $i, 'type' => $type[$i], 'removeNextDataSource' => FALSE);
                } else if($type[$i] == Core_Model_PosData::TYPE_AVERAGE_BASKET_VALUE) {
                    $posPostProcess[] = array('i' => $i, 'type' => $type[$i], 'removeDataSource' => TRUE);
                }
            }
        }

        $data = $this->_dataSourceData;
        $obsolete = array();
        $this->_dataSourceData = array_values($this->_dataSourceData);
        $this->_dataSourceTitles = array_values($this->_dataSourceTitles);
        $this->_dataSourceColors = array_values($this->_dataSourceColors);
        $this->_dataSourceAppearance = array_values($this->_dataSourceAppearance);
        $this->_dataSourceType = array_values($this->_dataSourceType);
        $this->_dataSourceSum = $this->_dataLabels && count($this->_dataLabels) > 0 ? array_values($this->_dataSourceSum) : array();
        $this->_dataLabels = $this->_dataLabels && count($this->_dataLabels) > 0 ?  array_values($this->_dataLabels) : array();
        $this->_dataLabelsRaw = $this->_dataLabelsRaw && count($this->_dataLabelsRaw) > 0 ?  array_values($this->_dataLabelsRaw) : array();

        /*
         * remove filtered values on edges for hour charts.
         */
        if($is_hour) {
            $size = $this->_dataLength;
            for($i = 0; $i < $lowHour; $i++) {
                for($j = 0; $j < count($this->_dataSourceData); $j++)
                    array_shift($this->_dataSourceData[$j]);

                array_shift($this->_dataLabels);
                array_shift($this->_dataLabelsRaw);
                $this->_dataLength--;
            }
            for($i = $highHour + 1; $i < $size; $i++) {
                for($j = 0; $j < count($this->_dataSourceData); $j++)
                    array_pop($this->_dataSourceData[$j]);

                array_pop($this->_dataLabels);
                array_pop($this->_dataLabelsRaw);
                $this->_dataLength--;
            }
        }

        $this->_numberOfDataSources = count($this->_dataSourceData);
        return True;
    }


    function _createAllLabels($chartDataSources, $chartData, $aggregator) {
        $times = array();
        $globalLast = NULL;
        $globalFirst = NULL;

        foreach($chartDataSources as $id => $chartDataSource) {
            if($chartDataSource->hasTimePeriodLimits()) {
                $res = $chartData->getDataResolution();
                $timePeriodLimits = $chartDataSource->getTimePeriodLimits();
                $from = $timePeriodLimits['from'];#->get(Zend_Date::TIMESTAMP);
                $to = $timePeriodLimits['to'];#->get(Zend_Date::TIMESTAMP);
                $fromTrunc = Core_Model_CountData::truncateDate($from,$res);
                $toTrunc = Core_Model_CountData::truncateDate($to,$res);
                if(!$globalFirst || $globalFirst->isLater($fromTrunc)) {
                    $globalFirst = $fromTrunc;
                }

                if(!$globalLast || $globalLast->isEarlier($toTrunc)) {
                    $globalLast = $toTrunc;
                }
            }
        }
        $t = clone $globalFirst;
        $last = Core_Model_CountData::getNextTime($globalLast, $res);
        while($t < $last) {
            $times[] = $t;
            $told = $t;
            $t = Core_Model_CountData::getNextTime($t, $res);
        }
        $this->_dataLabels = $aggregator->getXLabel($times, $res, $this->_dataLabelsRaw);
        $this->_times = $times;
    }

    public function renderToFile(Charts_Model_ChartData $chartData, $fn_base) {
        $fn = $fn_base . $this->getFileExtension();
        if(is_null($this->_title))
            $this->_renderCommon($chartData);
        $render = $this->_render();

        if(file_put_contents($fn, $render) === FALSE) {
            throw new WebReport_Exception("Could not write to file $fn");
        }
        return $fn;
    }

    protected function _fillTemplate($template, $template_extra = null) {
        $view = new Zend_View();
        // bit of a hack to hardcode the path but its cause the its standalone layout engine here.

        $view->setBasePath(APPLICATION_PATH . '/modules/charts/views/render');
        //$view->setLayout($template);

        $view->title = $this->_title;
        $view->dataLength = $this->_dataLength;
        $view->numberOfDataSources = $this->_numberOfDataSources;

        $view->dataLabels = $this->_dataLabels;

        $view->dataColors = $this->_dataSourceColors;
        $view->dataTitles = $this->_dataSourceTitles;
        $view->data = $this->_dataSourceData;

        $view->appearance = $this->_appearance;
        $view->extra = $template_extra;

        return $view->render($template . '.phtml');
    }

    public function renderToOutput(Charts_Model_ChartData $data, $response=NULL, $perma=false)
    {
        if (function_exists('mb_internal_encoding')) {
            $oldEncoding=mb_internal_encoding();
            mb_internal_encoding('latin1');
        }

        if($this->_add_file_type_to_header){
            if ( $this->_force_download ) {
                header('Content-Disposition: attachment; filename="export' . $this->getFileExtension() . '"');
            } else {
                header('Content-Disposition: inline; filename="export' . $this->getFileExtension() . '"');
            }
        }
        $this->_renderCommon($data, $perma);

        if(is_null($response)) {
            echo $this->_render();
        } else {
            $response->setHeader('Content-Type', $this->_content_type);

            $response->appendBody($this->_render());
            if(!is_null($this->_extra_headers)) {
                foreach($this->_extra_headers as $header => $value) {
                    $response->setHeader($header, $value);
                }
            }
        }


        if (function_exists('mb_internal_encoding'))
            mb_internal_encoding($oldEncoding);
    }

    // just a little hack, sorry
    private $_chartdata = NULL;
    public function setChartData(Charts_Model_ChartData $data)
    {
        $this->_chartdata = $data;
    }
    public function toString() {

        if(is_null($this->_chartdata))  {
            throw new WebReport_Error("setChartData() must be used before tostring");
        }

        $this->_renderCommon($this->_chartdata);
        return $this->_render();
    }
}
