<?php
class Charts_Model_ChartRender_AmChart extends Charts_Model_ChartRender_Abstract{

    protected $_code = "am-chart"; // unique code for this format
    protected $_content_type = "text/plain"; //mime content type for this format
    protected $_file_extension = ".json";

    private $_translateAppearance = array(
        "bar" => "column",
        "pie" => "pie",
        "line" => "line",
        "table" => "table",
    );
    function _darken($darken) {
        $v = hexdec($darken);
        $R = floor(($v >> 16 & 255)*0.93);
        $G = floor(($v >>  8 & 255)*0.93);
        $B = floor(($v >>  0 & 255)*0.93);
        $R = ($R<15?"0":"").dechex($R);
        $G = ($G<15?"0":"").dechex($G);
        $B = ($B<15?"0":"").dechex($B);
        return $R.$G.$B;
    }

    function _render(){
        $data = array();
        $total = array();
        $graphs = array();
        $settings = array();
        $valueAxes = array();
        $categoryAxis = array();
        $customSettings = array();
        $translate = Zend_Registry::get('Zend_Translate');
        $points = $this->_dataLength * $this->_numberOfDataSources;
        $axes = array();
        $axesTypes = array();
        $pos = 'left';
        foreach($this->_axes as $title => $params) {
            $limits = $this->findGoodLimits($this->_minValue[$title],$this->_maxValue[$title]);
            $unit = '#';
            if (isset($params['unit'])) $unit = $params['unit'];
            $shared = array(
                'position' => $pos,
                'maximum'       => $limits[1],
                'minimum'       => $limits[0],
                'stackType'     => 'regular',
                'lineAlpha'     => 0,
                'type'          => $title,
                'axisalpha'     => 0.1,
                'title'         => $translate->_($title),
                'balloonTitle'  => $translate->_($title)
            );
            if($this->_axisUnits[$title] == 'time') {
                $shared['duration'] = 'mm';
                $shared['durationUnits'] = array("DD" => "d. ", "hh" => "h ", "mm" => "min", "ss" => "");
            }
            $axes[$title . "##1"] = array(
                'gridAlpha'     => count($axes) == 0 ? 0.03 : 0/*$this->_multipleaxes ? 0 : 0.15*/,
                'labelsEnabled' => True,
                'totalText'     => '[[total]]',) + $shared;
            $axes[$title . "##2"] = array(
                'gridAlpha'     => 0/*$this->_multipleaxes ? 0 : 0.15*/,
                'labelsEnabled' => False,
                'totalText'     => '[[total]]',
                'title' => ""
                ) + $shared;
            $axes[$title . "##3"] = array(
                'gridAlpha'     => 0/*$this->_multipleaxes ? 0 : 0.15*/,
                'labelsEnabled' => False,
                'totalText'     => '[[total]]',
                "title" => ""
                ) + $shared;
            $pos = "right";
        }
        $stackIds = array();
      
        $numberOfStackedOnAxis = Array(); 
        for($i=0; $i < $this->_numberOfDataSources; $i++) {
            //valid properties are
            //http://www.amcharts.com/docs/v.2/bundle/javascript_reference/amgraph
            $stackId = $this->_dataSourceStackId[$i];
            $gid = 'k'.$i;
            $graph = array(
              "title" => $this->_dataSourceTitles[$i],
              "type" => isset($this->_dataSourceAppearance[$i]) ? $this->_translateAppearance[$this->_dataSourceAppearance[$i]] : "default",
              "lineColor" => "#".$this->_dataSourceColors[$i],
              "fillColors" => array("#".$this->_darken($this->_dataSourceColors[$i]),"#".$this->_dataSourceColors[$i]),
              "bulletColor" => "#".$this->_dataSourceColors[$i],
              'labelText'  => $this->_dataSourceType[$i] == $this->_dataSourceType[0] ? "[[value]]" : "",
              "valueField" => "f" . $i,
              "descriptionField" => "ts"
            );
            if(count($this->_axes) > 1) {
                $graph['markerType'] = $graph['type'] == 'line' ? 'line' : 'square';
            }

            if($graph['type'] == "column") {
               $graph['fillAlphas'] = $this->_singleAxis ? 0.7 : 0.6;
               $graph['lineAlpha'] = $this->_singleAxis  ? 1.0 : 0.7;
               $graph['bullet'] = 'none';
            } else {
               $graph['lineAlpha'] = 0.9;
               $graph['fillAlphas'] = 0;
               $graph['bullet'] = 'circle';
            }
            $graph['stack'] = $stackId;
            $graph['gid'] = $gid;
            if($stackId == 0) {
                $stackId = 1;
                $graph['stackable'] = False;
            }
        
            $stackInd = 1;
            if($this->_dataSourceStackId[$i] > 0){ //stacked
                if(!isset($numberOfStackedOnAxis[$this->_dataSourceType[$i]]))
                    $numberOfStackedOnAxis[$this->_dataSourceType[$i]]=0;
                $numberOfStackedOnAxis[$this->_dataSourceType[$i]] ++;
                $stackInd = round($numberOfStackedOnAxis[$this->_dataSourceType[$i]]/2);
            }
            $graph['valueAxis'] = $this->_dataSourceType[$i] . "##" . $stackInd;
            if(isset($axes[$graph['valueAxis']])) $axis = $axes[$graph['valueAxis']];
            if($this->_numberOfDataSources == 1) {
                $graph['balloonText'] = $axis['balloonTitle'] . ":" ." [[value]]";
            } else {
                $graph['balloonText'] = $axis['balloonTitle'] . ":" ." [[value]]";
            }
            if($graph['type'] == 'line') {
                $graph['stackable'] = False;
            }

            $graph['lineThickness'] = $graph['type'] == 'line' ? 1.5 : 1;
            $graphs[$gid] = $graph; //fixme something smarter
            $total[$gid] = $this->_dataSourceSum[$i];

        }
        # Swap order of stacks in really smart and awesome way.
        # Needs swaping beacause females should be printed above males in the stacked bar
        uasort($graphs, function($a, $b) { 
            return $a['stack'] == $b['stack'] ? $a['gid'] < $b['gid'] : $a['gid'] > $b['gid'];
        });


        for($j=0; $j < $this->_dataLength; $j++) {
            $row = array(
                "ts" => $this->_dataLabels[$j]
            );
            for($i=0; $i < $this->_numberOfDataSources; $i++) {
                $row["f" . $i] = $this->_dataSourceData[$i][$j];
            }
            $data[] = $row;
        }

        //$valueAxes['unit'] = $this->getUnit();
        //$valueAxes['axes'] = $axes;

        $valueAxes['usePrefixes'] = true;
        $valueAxes['integersOnly'] = true;

        $categoryAxis['autoGridCount'] = $this->_dataLength < 13;
        $categoryAxis['gridCount'] = 12;
        $categoryAxis['gridAlpha'] = 0.1;
        
        $customSettings['formatNumbers'] = $points > 14;
        $customSettings['hideNumbers'] = true;
        $customSettings['thousandSeparator'] = $this->getThousandSeparator();
        $customSettings['chartScroll'] = false;

        $settings['panEventsEnabled'] = true;
        $customSettings['chartCursor'] = array(
            'cursorPosition' => 'mouse',
            'oneBalloonOnly' => $this->_multipleAxes && $this->_numberOfDataSources >2
        );

        $error = $this->getStatus();

        //$error = $this->getEndUserError();
        $ret = array(
            "data" => $data,
            "graphs" => $graphs,
            "showTotal" => true,
            "settings" => $settings,
            "valueAxes" => $axes,
            "categoryAxis" => $categoryAxis,
            "customSettings" => $customSettings,
            "hasError" => !is_null($error),
            "error" => $error,
        );
        return Zend_Json::encode($ret);
    }

    function getThousandSeparator(){
        $locale = Zend_Registry::get('Zend_Locale');
        $num = Zend_Locale_Format::toNumber("1111", array('locale' => $locale));
        $num = str_replace('1','',$num);
        if(strlen($num) > 1) return " ";
        return  $num;
    }
}
