<?php
include_once("pChart/class/pData.class.php");
include_once("pChart/class/pDraw.class.php");
include_once("pChart/class/pImage.class.php");
function AlmostAmChartDurationFormat($value) {
    //$trans = Zend_Registry::get('Zend_Translate');
    $m = $value % 60;
    $h = ($value-$m) / 60;
    if($h < 1) return $m."m";
    if($h > 9) return $h . "h";
    return $h."h ".$m."m";
}
class Charts_Model_ChartRender_Png extends Charts_Model_ChartRender_Abstract {
    const _font = "/fonts/verdana.ttf";//"/fonts/Lucida_Grande_Bold.ttf";

    const LABEL_ROTATION_LIMIT_30 = 7;
    const LABEL_ROTATION_LIMIT_90 = 1400;

    protected $_code = "png";
    protected $_content_type = "image/png";
    protected $_file_extension = ".png";
    protected $_surface;
    protected $_data;
    protected $_basedir;
    protected $_format;
    protected $_translate;
    protected $_legendFontSize = 7;
    protected $_antialias = true;
    public $_origo;
    public $_imageHeight;
    public $_drawLegend = true;
    public $_includeTable = false;

    private $_appearanceConvert = array(
            "bar" => "column",
            "line" => "line",
            "pie" => "pie"
            );

    function __construct(){
        $this->_force_wanted_resolution = true;
        $this->antialias = true;
    }


    function _createChart($fn, $width, $height) {
        $data    = new pData();
        $surface = new pImage($width, $height, $data);

        $this->_setup($data,$surface,$width,$height);
        $this->_setupArea();
        $this->_setupTitle();
        if($this->_drawLegend)
            $this->_setupAbscissa();
        $this->_prepareAxes();
        $this->_setupData();
        $this->_setupAxes();

        $this->_setupColors();
        $this->_drawData();
        if($this->_drawLegend)
            $this->_drawLegend();    //is done later in the pdf renderer

        $surface->render($fn);
    }

    function _setupColors() {
        $this->_colors = array();
        foreach($this->_dataSourceColors as $index => $color) {
            if(strlen($color) == 3) $color = $color[0].$color[0]
                .$color[1].$color[1]
                    .$color[2].$color[2];
            $color = hexdec($color);
            $this->_colors[$index] = array(
                    "B" => $color >>  0 & 255,
                    "G" => $color >>  8 & 255,
                    "R" => $color >> 16 & 255,
                    "Alpha" => count($this->_axisIDs) > 1 ? "60" : "80"
                    );
            if($this->_dataSourceAppearance[$index] == 'line' && $this->_totalBars > 0)
                $this->_colors[$index]["Alpha"] = "100";
            $this->_data->setPalette("data$index", $this->_colors[$index]);
        }
    }

    function _drawLegend() {
        foreach($this->_dataSourceAppearance as $index => $appearance)
            $this->_data->setSerieDrawable("data$index");
        $this->_setFontSize($this->_legendFontSize);
        $format = array("BoxHeight" => $this->_legendFontSize*1.6,
                "BoxWidth" => $this->_legendFontSize*1.5,
                "Alpha"    => 0);
        $size = $this->_surface->getLegendSize($format);
        $this->_surface->drawLegend(40+$this->_width/50, $this->_height-$size['Height'], $format);
    }

    function _addStackedHeights(){
        //add stacked bars height
        for($index=count($this->_dataSourceAppearance)-2; $index>=0; $index--) {
            if(($this->_dataSourceAppearance[$index] == 'bar') &&
            ($this->_dataSourceStackId[$index] > 0) &&
            ($this->_dataSourceStackId[$index] == $this->_dataSourceStackId[$index+1])){
                $i=1;
                foreach($this->_data->Data['Series']["data$index"]['Data'] as $j => &$d){
                    $index2 = $index+1;
                    $d += $this->_data->Data['Series']["data$index2"]['Data'][$j];
                }
            }
        }
    }

    function _drawFirstLayer($params){
        $drawnBarsStackId = array();
        foreach($this->_dataSourceAppearance as $index => $appearance) {
            if($appearance == 'bar'){
                if($this->_dataSourceStackId[$index] == 0){
                    $this->_data->setSerieDrawable("data$index",TRUE);
                }elseif(!in_array($this->_dataSourceStackId[$index], $drawnBarsStackId)){
                    $drawnBarsStackId[] = $this->_dataSourceStackId[$index];
                    $this->_data->setSerieDrawable("data$index",TRUE);
                }else{
                    $this->_data->setSerieDrawable("data$index",FALSE);
                }
            }else{
                $this->_data->setSerieDrawable("data$index",FALSE);
            }
        }
        $this->_surface->drawBarChart($params);

        //put all singel bars height to 0
        foreach($this->_dataSourceAppearance as $index => $appearance)
            if($appearance == 'bar' && $this->_dataSourceStackId[$index] == 0)
                foreach($this->_data->Data['Series']["data$index"]['Data'] as &$d)
                    $d=0;
    }

    function _drawSecondLayer($params){
        $drawnBarsStackId = array();
        foreach($this->_dataSourceAppearance as $index => $appearance) {
            if($appearance == 'bar'){
                if($this->_dataSourceStackId[$index] == 0){
                    $this->_data->setSerieDrawable("data$index",TRUE);
                }elseif($index>0 && $this->_dataSourceStackId[$index]==$this->_dataSourceStackId[$index-1] && !in_array($this->_dataSourceStackId[$index],$drawnBarsStackId)){
                    $this->_data->setSerieDrawable("data$index",TRUE);
                }else{
                    $this->_data->setSerieDrawable("data$index",FALSE);
                }
            }else{
                $this->_data->setSerieDrawable("data$index",FALSE);
            }
        }
        $this->_surface->drawBarChart($params);
    }

    function _drawData() {
        $fsize = max(floor(3+min(10,$this->_width / max(1,$this->_totalBars/6))),10);
        $this->_surface->setShadow(FALSE);
        $this->_setFontSize($fsize);
        $params = array("DisplayColor"  => DISPLAY_AUTO,
                "Gradient"      => TRUE,
                "GradientAlpha" => $this->_containsStacked ? 4 : 15,
                );

        $params['DisplayValues'] = true;
        if($this->_includeTable || $this->_containsStacked || $this->_totalBars > 32) 
            $params['DisplayValues']=false;

        $params['Rounded'] = FALSE;
        $params['Interleave'] = 0.4;
        $params['Surrounding'] = -25;

        
        // Draw bars
        if($this->_containsStacked){
            $this->_addStackedHeights();
            $this->_drawFirstLayer($params);
            $this->_drawSecondLayer($params);
        }else{
            foreach($this->_dataSourceAppearance as $index => $appearance) {
                if($appearance == 'bar')     $this->_data->setSerieDrawable("data$index",TRUE);
                else                        $this->_data->setSerieDrawable("data$index",FALSE);
            }
            $this->_surface->drawBarChart($params);
        }

    
        // Draw lines
        $params['DisplayValues'] = 0;
        $lineWeight = floor($this->_width/400) / 3;
        foreach($this->_dataSourceAppearance as $index => $appearance) {
            $this->_data->setSerieWeight("data$index",$lineWeight);
            if($appearance == 'line') $this->_data->setSerieDrawable("data$index",TRUE);
            else                     $this->_data->setSerieDrawable("data$index",FALSE);
        }
#        $params['PlotSize'] = 1+floor($lineWeight+0.5);
        $this->_surface->setShadow(TRUE);
#       $this->_surface->drawPlotChart($params);
        $this->_surface->setShadow(FALSE);
        $this->_surface->drawLineChart($params);
#     $params['PlotBorder'] = TRUE;
#      $params['BorderAlpha'] = 100;
}

function _setupAbscissa() {


    $this->_data->AddPoints($this->_dataLabels, 1337);
    $this->_data->setSerieDescription(1337, $this->_translate->_("PERIOD"));
    $this->_data->setAbscissa(1337);

    $this->_scaleParams['LabelRotation'] = 0;
    if(count($this->_dataLabels) > self::LABEL_ROTATION_LIMIT_30) $this->_scaleParams['LabelRotation'] = 30;
    if(count($this->_dataLabels) > self::LABEL_ROTATION_LIMIT_90) $this->_scaleParams['LabelRotation'] = 90;

    // When the width is 1400 pixels, don't allow more than 40 labels.
    $this->_scaleParams['LabelSkip'] = floor(count($this->_dataLabels) / (40*$this->_width/1400.0));
    
    if($this->_drawLegend == false){
        $this->_dataLabels = 0;
        $this->_scaleParams['LabelRotation'] = 90;
    }

}

function _prepareAxes() {
   /* if(count(array_unique($this->_axisTypes)) == 1)
        $axisId = array(current($this->_axisTypes) => 0);
    else
        $axisId = array("bar" => 0, "line" => 1);
    $this->_axisAppearanceIDs = $axisId;
    $this->_axisIDs = array();
    foreach($this->_axisTypes as $k => $v) {
        $this->_axisIDs[$k] = $axisId[$v];
    }
  */
    $i = 0;
    foreach($this->_axisTypes as $k => $v) {
        $this->_axisIDs[$k] = $i;
        $i++;
    }
}

function _setupAxes() {

    $nbrTicks;
    $axisIndex = 0;
    foreach($this->_axisTypes as $k => $v) {
        $includeAppearanceInAxisTitle = true;
        $sourcesOnAxis = array_keys($this->_dataSourceType,$k);
        $sourcesOnOtherAxes = array_diff(array_keys($this->_dataSourceType), $sourcesOnAxis);
        foreach($sourcesOnAxis as $sourceIndex){
            if($this->_dataSourceAppearance[$sourceIndex] != $v)
                $includeAppearanceInAxisTitle = false;
        }
        foreach($sourcesOnOtherAxes as $sourceIndex){
            if($this->_dataSourceAppearance[$sourceIndex] == $v)
                $includeAppearanceInAxisTitle = false;
        }
        $this->_data->setAxisDisplay($axisIndex, AXIS_FORMAT_METRIC);
        if($axisIndex > 0)
            $this->_data->setAxisPosition($axisIndex, AXIS_POSITION_RIGHT);
        $limits = $this->findGoodLimits($this->_minValue[$k], $this->_maxValue[$k]);//*(1+$this->_tallFactor));
        $axisTitle =  $this->_translate->_($k);
        if(count($this->_axisTypes) > 1 && $includeAppearanceInAxisTitle) {
            $axisTitle .= strtolower(" (" . $this->_translate->_(strtoupper($v)) . ")");
        }
        $this->_data->setAxisName($axisIndex,$axisTitle);
        if($this->_axisUnits[$k] == 'time') {
            $this->_data->setAxisDisplay($axisIndex, AXIS_FORMAT_CUSTOM, "AlmostAmChartDurationFormat");
            //$this->_data->setAxisName($axisIndex, ' ');
            //try to make axes look nice for time
            if($axisIndex==0){
                $time   = ($limits[1]- $limits[0]);
                $factor = 1;
                foreach(array(24*4*60, 24*60, 4*60, 2*60, 60) as $base){
                    if($time > $base){
                        $factor = $base/4;
                        break;
                    }
                }
                $evenLimits = $this->_makeLimitsEven($limits, $factor);
                $nbrTicks   = $evenLimits[1]/$factor;
            }else{
                $minFactor = $limits[1]/$nbrTicks;
                $factor    = 60 * ceil($minFactor/60); 
                foreach(array(5, 10, 15, 30, 60) as $base){
                    if($minFactor < $base){
                        $factor = $base;
                        break;
                    }
                }                
                $evenLimits = $this->_makeLimitsEven($limits, $factor, $nbrTicks);
            }
        }else{
            //Try to make axes look nice 
            if($axisIndex==0){
                $factor = floor(log10($limits[1]));
                $factor = pow(10,$factor);
                if($limits[1]/$factor < 4 && $limits[1]/$factor > 2){
                    $factor = $factor/2;
                }elseif($limits[1]/$factor < 2){
                    $factor = $factor/5;
                }
                $evenLimits = $this->_makeLimitsEven($limits, $factor);
                $nbrTicks   = $evenLimits[1]/$factor; 

            }else{
                $minFactor = ceil($limits[1]/$nbrTicks);
                $power  = pow(10, strlen($minFactor)-1); 
                $factor = ceil($minFactor/$power);
                if($factor > 5 && $factor < 10){
                    $factor = 10;
                }
                $factor *= $power;
                $evenLimits = $this->_makeLimitsEven($limits, $factor, $nbrTicks);
            }
        }
        $this->_scaleParams['Factors'][$axisIndex] =  $factor;
        $this->_scaleParams['ManualScale'][$axisIndex] = array("Min" => $evenLimits[0] , "Max" => $evenLimits[1] );   
        $axisIndex ++;
    }

    $this->_scaleParams['GridTicks'] = 0;
    $this->_scaleParams['Mode'] = SCALE_MODE_MANUAL;# min($this->_minValue) < 0 ? SCALE_MODE_ADDALL : SCALE_MODE_ADDALL_START0;
    $this->_scaleParams['GridR'] = 0;
    $this->_scaleParams['GridG'] = 0;
    $this->_scaleParams['GridB'] = 0;
    $this->_scaleParams['GridAlpha'] = 7;
    $this->_scaleParams['DrawYLines'] = array(0);
    $this->_setFontSize(min(5 + min($this->_width, $this->_height)/120,12));
    $this->_surface->drawScale($this->_scaleParams);
}

function _setupData() {
    $this->_dataSourceIndexByAppearance = array();
    $dataPoints = 0;
    foreach($this->_dataSourceData as $index => &$data) {
        $type = $this->_dataSourceAppearance[$index];
        foreach($data as $i => &$d) {
            $dataPoints += 1;
            if ($d != 0) continue;
            if($type == 'bar') {
                $d = VOID;
                continue;
            }
            $left = !isset($data[$i-1]) || ($data[$i-1] == 0 || $data[$i-1] == VOID);
            $right= !isset($data[$i+1]) || ($data[$i+1] == 0 || $data[$i+1] == VOID);
            if($left && $right) $d = VOID;
        }
        $name = "data$index";
        $sourceType = $this->_dataSourceType[$index];
        $axisName = $sourceType ; // Multiple stacks not supported "0" <= strval($this->_dataSourceStackId[$index]);
        $axisId = $this->_axisIDs[$axisName];
        $title = $this->_dataSourceTitles[$index];
        $data = $this->_dataSourceData[$index];
        $this->_data->AddPoints($data, $name);
        $this->_data->setSerieDescription("data$index", $title );
        $this->_data->setSerieOnAxis($name, $axisId);
    }
    if ( $dataPoints > 50 ) $this->_surface->Antialias = False;
}

function _setupArea() {
    $legendHeight = $this->_numberOfDataSources * ($this->_legendFontSize+5);  //10 above text, 5 in between, 11 textheight, 10 below text
    $extraLabelSpace = 0;      
    if(count($this->_dataLabels) > self::LABEL_ROTATION_LIMIT_90){
        $extraLabelSpace = 100;
    } else if(count($this->_dataLabels) > self::LABEL_ROTATION_LIMIT_30){
        $extraLabelSpace = 40;
    }

    $dx = 100;
    $dy = 50;
    $dx2 = $this->_width - $dx;
    $dy2 = $this->_height - 30;
    if($this->_drawLegend)
        $dy2 -= $extraLabelSpace + $legendHeight + 20;

    $this->_origo = Array('x' => $dx/$this->_width, 'y' => ($dy2)/$this->_height);
#$this->_surface->drawFilledRectangle(0,0,$this->_width,$this->_height,array("R"=>250, "B" => 240, "G" => 250));

    $color = array("StartR"  =>255, "StartB"  => 255, "StartG" => 255,
            "EndR"    =>251, "EndB"    => 251, "EndG"   => 248
            );

    $this->_surface->drawGradientArea($dx,$dy,$dx2,$dy2, DIRECTION_VERTICAL, $color);

    $dash = array("Alpha" =>0, "R"=>255, "G"=>255, "B"=>255, 
            "Dash"=>1, "DashR"=>0, "DashG"=>0, "DashB"=>0,
            "BorderR" =>  0,  "BorderG" =>  0, "BorderB" => 0, "BorderAlpha" => 15);
    $this->_surface->drawFilledRectangle($dx,$dy,$dx2,$dy2,$dash);

    $this->_surface->setGraphArea($dx,$dy,$dx2,$dy2);
}

function _setupTitle() {
    $this->_setFontSize(18);
    $textSettings = array('Align' => TEXT_ALIGN_MIDDLEMIDDLE);
    $this->_surface->drawText($this->_width/2, 18, $this->_title, $textSettings);
}


function _setup($data, $surface, $width, $height) {
    $this->_surface = $surface;
    $this->_tallFactor = max(($height -$width) / $width,0);
    $this->_data = $data;
    $this->_width = $width;
    $this->_height = $height;
    $this->_translate = Zend_Registry::get('Zend_Translate');
    $this->_format = "png";
    $this->_basedir = APPLICATION_PATH;
    $this->_basedir .= "/../library/php/pChart";
    $this->_containsStacked = array_sum($this->_dataSourceStackId) > 0;
    $this->_surface->Antialias = $this->_antialias;
    $this->_scaleParams = array();
    $this->_valuesShown = count(current($this->_dataSourceData)) * count($this->_dataSourceData);
    $this->_totalBars = 0;
    foreach($this->_dataSourceAppearance as $index => $type) {
        if($type == 'bar') $this->_totalBars += count($this->_dataSourceData[$index]);
    }

    $this->_valuesShown = count(current($this->_dataSourceData)) * count($this->_dataSourceData);
}

function _setFontSize($size) {
    $this->_surface->setFontProperties(array("FontName"=>$this->_basedir.self::_font,"FontSize"=>$size));
}

function _render() {
    $fn = APPLICATION_PATH . "/../data/session/" . uniqid() . ".png";

    $width = $this->getWidth();
    $height = $this->getHeight();

    $this->_createChart($fn, $width, $height);

    if(!file_exists($fn)){
        return file_get_contents(APPLICATION_PATH . "/../public/library/images/errors/data-session-access.png");
    }
    $ret = file_get_contents($fn);
    unlink($fn);
    return $ret;
}

function _makeLimitsEven($limits, $factor, $ticks = false){
    if($ticks){
        $retMin = floor($limits[0]/$factor);
        $retMax = $ticks*$factor; 
    }else{
        $retMin = floor($limits[0]/$factor);
        $retMax = 0;
        while($retMax < ($limits[1]-0.0001)){
            $retMax += $factor;
        }
        $retMax += $factor;
    }
    if($limits[0] == 0)
        $retMin = 0;

    return Array($retMin, $retMax);
}

}

