<?php

class Charts_Model_DataAggregator_HistSum extends Charts_Model_DataAggregatorAbstract
{

    private $_relative = FALSE;
    private $_sundayFirst = NULL;
    protected $_cumulative = false;

    public function  __construct() {
        $this->_mode = 1;
        parent::__construct();
        $this->_defaults_to_store = array("_mode", "_relative");
        if($this->_relative) $this->_display_unit = '%';
        $locale = Zend_Registry::get('Zend_Locale');
        $weekList = Zend_Locale_Data::getList($locale,'week');
        //$this->_sundayFirst = $weekList['firstDay'] == 'sun';
        $this->_sundayFirst = $weekList['weekendEnd'] != 'sun';
        if($weekstart = Core_Model_User::GetCurrentUsersStartOfWeek()) {
            $this->_sundayFirst = $weekstart == "sun";
        }
        $this->_trim = $this->_mode == self::MODE_HIST_OVER_HOUR;
    }

    public function isRelative(){
        return $this->_relative;
    }

    public function getNbrBinsWithData() {
        if ( !isset($this->_agg) ) {
            return 0;
        }
        $bins = 0;
        foreach($this->_agg as $data) {
            if ( $data->fields ) $bins += 1;
        }
        if ( $bins == 0 ) return count($this->_agg);
        return $bins;
    }

    public function setHistogramMode($timeMode, $relative=FALSE)
    {
        $this->_mode = $timeMode;
        $this->_relative = $relative;

        $this->_trim = $this->_mode == self::MODE_HIST_OVER_HOUR;
        if($this->_relative) $this->_display_unit = '%';
    }

    public function getResolution(){
        return $this->_get_resolution();
    }

    public function getMode() {
        return $this->_mode;
    }

    private function _get_resolution() {
        switch ($this->_mode) {
            case self::MODE_HIST_OVER_15MIN:
                return 'ts15min';
                break;
            case self::MODE_HIST_OVER_HOUR:
                return 'ts1h';
                break;
            case self::MODE_HIST_OVER_WEEK:
                return 'ts1d';
                break;
            case self::MODE_HIST_OVER_MONTH:
                return 'ts1d';
                break;
            case self::MODE_HIST_OVER_DAY:
                return 'ts1h';
                break;
            case self::MODE_HIST_OVER_DAYS_IN_MONTH:
                return 'ts1d';
                break;
            case self::MODE_HIST_OVER_QUARTER:
                return 'ts15m';
                break;
            default:
                throw new WebReport_Exception("Not supported" . $this->_mode);
        }
    }

    private function _get_bin($ts) {
        $date = localtime(strtotime($ts),true);
        //$ts_date = $date->toArray();
        $bin = -1;
        switch ($this->_mode) {
            case self::MODE_HIST_OVER_15MIN:
                //$bin = $ts_date['hour'];
                $bin = intval($date['tm_hour']*4 + $date['tm_min'] / 15);
                break;
            case self::MODE_HIST_OVER_HOUR:
                //$bin = $ts_date['hour'];
                $bin = $date['tm_hour'];
                break;
            case self::MODE_HIST_OVER_WEEK:
                //$bin = $ts_date['weekday'];a
                $bin = $date['tm_wday'];
                if(!$this->_sundayFirst){
                    $bin = $bin == 0 ? 6 : $bin - 1;
                }
                break;
            case self::MODE_HIST_OVER_MONTH:
                //$bin = $ts_date['month'];
                $bin = $date['tm_mon'];
                break;
            case self::MODE_HIST_OVER_DAY:
                //$hour = $ts_date['hour'];
                $hour = $date['tm_hour'];
                if($hour>= 9 && $hour< 16)  $bin = 1; //MIDDAY
                if($hour>=16 && $hour< 20) $bin = 2;  //AFTERNOON
                if($hour>=20 || $hour< 6) $bin = 3;  //OFF_HOURS
                if($hour>=6 && $hour< 9 )  $bin = 0; // MORNING
                break;
            case self::MODE_HIST_OVER_DAYS_IN_MONTH:
                $bin = $date['tm_mday'] -1;
                break;
            case self::MODE_HIST_OVER_QUARTER:
                $bin = $date['tm_min'] / 15 + $date['tm_hour'] * 4;
                break;
            default:
                throw new WebReport_Exception("Not supported" . $this->_mode);
        }

        if($bin==-1)
            throw new Webreport_Exception("runtime error");

        return $bin;
    }

	private function _init_return_array($first_element=null) {
		$arrlen = null;
		$begin=0;
		switch ($this->_mode) {
			case self::MODE_HIST_OVER_15MIN:
				$arrlen = 4*24;
				break;
            case self::MODE_HIST_OVER_HOUR:
				$arrlen = 24;
				break;
			case self::MODE_HIST_OVER_WEEK:
		    	$arrlen = 7;
				break;
			case self::MODE_HIST_OVER_MONTH:
				$arrlen = 12;
				break;
			case self::MODE_HIST_OVER_DAY:
				$arrlen = 4;
				break;
			case self::MODE_HIST_OVER_DAYS_IN_MONTH:
				$arrlen = 31;
				break;
			default:
				throw new WebReport_Exception("Not supported" . $this->_mode);
		}
		$baseObj = new stdClass();
		if($first_element != null){
			foreach ($first_element as $key => $value) {
				if($key == 'cnttime') {
					continue;
				}
				$baseObj->$key = 0;
			}
		}

		$ret = array();

		for($i=$begin;$i<$arrlen+$begin;++$i) {
			$ret[$i] = clone($baseObj);
		}

		return $ret;
	}

    protected function _renderData()
    {
        $data = $this->getRawData($this->_get_resolution(),$this->_timeFilter);
        if(count($data) == 0) {
           return $this->_init_return_array(); //TODO: warn better there is no data
        }
        $agg = $this->_init_return_array($data[0]);
        $sum = null;
        if($this->_relative) {
           $sum = new stdClass();
           foreach($data[0] as $key => &$value) {
                if($key == 'cnttime')
                    continue;
                $sum->$key = 0;
           }
        }
        $nbrAdds = array();
        foreach ($data as &$cur) {
            $bin = $this->_get_bin($cur->cnttime);
            foreach($cur as $key => &$value) {
                $meta = FALSE;
                if($key == 'cnttime') continue;
                if($key == 'fields') $meta = TRUE;
                if(strpos($key, "_exists") !== FALSE) $meta = TRUE;
                if(!is_null($value)) {
                    $agg[$bin]->$key += $value;
                    if(!isset($nbrAdds[$bin])) $nbrAdds[$bin] = 0;
                    if ($meta == FALSE) {
                        $nbrAdds[$bin] ++;
                    }
                    if(!is_null($sum)) {
                        $sum->$key += $value;
                    }
                }
            }
        }
        for($i = 1; $this->_cumulative && $i < count($agg);$i++){
            foreach($agg[$i] as $key => $value){
                $agg[$i]->$key += $agg[$i-1]->$key;
            }
            foreach($agg[$i] as $key => $value){
                $agg[$i]->$key = max(0,$value);
            }
        }

        if($this->_relative) {
            foreach($agg as &$e)
               foreach($sum as $key => &$value)
		   if($value > 0.00001 || $value < -0.00001){
                       $e->$key = round((float) ($e->$key)/$value*100);
		   }
        }
        if($this->_isAverage && !$this->_relative) {
            foreach ($agg as $bin => $k) {
                if(isset($nbrAdds[$bin]) && $nbrAdds[$bin]>0) $agg[$bin]->sum = round($agg[$bin]->sum/$nbrAdds[$bin],1);
            }
        }

        $this->_agg = $agg;
        return $agg;
    }

    protected function _getXLabel($lang, $times = NULL, $resolution = NULL, &$raw = false)
    {

        $ret = null;
        switch ($this->_mode) {
            case Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_HOUR:
                $ret = array('00:00', '01:00', '02:00', '03:00', '04:00',
                    '05:00', '06:00', '07:00', '08:00', '09:00', '10:00',
                    '11:00', '12:00', '13:00', '14:00', '15:00', '16:00',
                    '17:00', '18:00', '19:00', '20:00', '21:00', '22:00',
                    '23:00');
                
                if ( $raw !== false ) $raw = $ret;
                break;
            case Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_15MIN:
                // No, I used python.
                $ret = array('00:00', '00:15', '00:30', '00:45', '01:00', '01:15', '01:30', '01:45', '02:00', '02:15', '02:30', '02:45', 
                             '03:00', '03:15', '03:30', '03:45', '04:00', '04:15', '04:30', '04:45', '05:00', '05:15', '05:30', '05:45',
                             '06:00', '06:15', '06:30', '06:45', '07:00', '07:15', '07:30', '07:45', '08:00', '08:15', '08:30', '08:45',
                             '09:00', '09:15', '09:30', '09:45', '10:00', '10:15', '10:30', '10:45', '11:00', '11:15', '11:30', '11:45', 
                             '12:00', '12:15', '12:30', '12:45', '13:00', '13:15', '13:30', '13:45', '14:00', '14:15', '14:30', '14:45', 
                             '15:00', '15:15', '15:30', '15:45', '16:00', '16:15', '16:30', '16:45', '17:00', '17:15', '17:30', '17:45',
                             '18:00', '18:15', '18:30', '18:45', '19:00', '19:15', '19:30', '19:45', '20:00', '20:15', '20:30', '20:45',
                             '21:00', '21:15', '21:30', '21:45', '22:00', '22:15', '22:30', '22:45', '23:00', '23:15', '23:30', '23:45');
                if ( $raw !== false ) $raw = $ret;
                break;
            case Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_WEEK:
                $locale = Zend_Registry::get('Zend_Locale');
                $ret = $locale::getTranslationList('days', $lang);
                $ret = array_values($ret['format']['wide']);
                if(!$this->_sundayFirst){
                     $ret[] = $ret[0];
                     unset($ret[0]);
                     $ret = array_values($ret);
                     if ( $raw !== false ) $raw = array("SUNDAY", "MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY","SATURDAY");
                } else {
                     if ( $raw !== false ) $raw = array("MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY","SATURDAY", "SUNDAY");
                }
                break;
            case Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_MONTH:
                $ret = array_values(Zend_Locale::getTranslationList('month', $lang));
                if ( $raw !== false ) 
                    $raw = array("JANUARY","FEBRUARY", "MARCH", "APRIL", "MAY", "JUNE",
                                 "JULY", "AUGUST", "SEPTEMBER", "OCTOBER", "NOVEMBER", "DECEMBER");
                break;
            case Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_DAY:
                $ret = array('MORNING', 'MIDDAY', 'AFTERNOON', 'OFF_HOURS');
                $translate = Zend_Registry::get('Zend_Translate');
                $raw = array();
                foreach($ret as $k => $v){
                    $ret[$k] = $translate->_($v);
                    $raw[] = $v;
                }
                break;
            case Charts_Model_DataAggregator_HistSum::MODE_HIST_OVER_DAYS_IN_MONTH:
                $ret = range(1,31);
                if( $raw !== false ) $raw = range(1,31);
                break;
            default:
                throw new WebReport_Exception("Not supported" . $this->_mode);
        }
        return $ret;
    }

        
    protected function _getXLabelRaw() {
        return array_keys($this->_agg);
    }

    protected function _getDataLabels($lang)
    {
        $ret = $this->_count_data_set->getDescriptionsForLastFetchedData();
        unset($ret['cnttime']);
        return $ret;
    }

    private function _stringsToTranslate() {
        $this->_translate->_('MORNING');
        $this->_translate->_('MIDDAY');
        $this->_translate->_('AFTERNOON');
        $this->_translate->_('OFF_HOURS');
    }

    public function setDefaultsJson($defaults) {
        // for each value in default, assign to self
        foreach ($defaults as $v => $value) {
            $this->$v = $value;
        }
        if($this->_relative) $this->_display_unit = '%';
        return True;
    }

    public function dumpJson() {
        $enc = array();
        foreach($this->_defaults_to_store as $k) {
            $enc[$k] = $this->$k;
        }

        return Zend_Json::encode($enc);
    }
}
