<?php

class Charts_Model_DataAggregator_TotalSum extends Charts_Model_DataAggregatorAbstract
{
    private $_relative = FALSE;

    public function  __construct() {
        parent::__construct();
        $this->_defaults_to_store = array("_mode", "_relative");

        if($this->_relative) $this->_display_unit = '%';
    }

    public function setHistogramMode($timeMode, $relative=FALSE)
    {
        $this->_mode = $timeMode;
        $this->_relative = $relative;

        if($this->_relative) $this->_display_unit = '%';
    }

    public function getResolution(){
        return $this->_get_resolution();
    }

    public function getNbrBinsWithData() {
        return 1;
    }

    private function _get_resolution() {
        return 'ts1y';
    }

    private function _get_bin($ts) {
        return 0;
    }

    private function _init_return_array($first_element=null) {
        $begin=0;
        $baseObj = new stdClass();

        $ret = array();

        $ret[0] = clone($baseObj);

        return $ret;
    }

    protected function _renderData()
    {
        $data = $this->getRawData($this->_get_resolution(),$this->_timeFilter);
        if(count($data) == 0) {
           return $this->_init_return_array(); //TODO: warn better there is no data
        }

        $agg = $this->_init_return_array($data[0]);
        $sum = null;

        if($this->_relative) {
           $sum = new stdClass();
           foreach($data[0] as $key => &$value) {
                if($key == 'cnttime')
                    continue;
                $sum->$key = 0;
           }
        }
        foreach ($data as &$cur) {
            $bin = 0;
            foreach($cur as $key => &$value) {
                if($key == 'cnttime') continue;
                if(!is_null($value)) {
                    $agg[$bin]->$key += round($value,1);
                    if(!is_null($sum)) {
                        $sum->$key += $value;
                    }
                }
            }

        }

        if($this->_relative) {
            foreach($agg as &$e)
               foreach($sum as $key => &$value)
           if($value > 0.00001 || $value < -0.00001){
                       $e->$key = round((float) ($e->$key)/$value*100);
           }
        }

        $this->_agg = $agg;

        return $agg;
    }

    protected function _getXLabel($lang, $times = NULL, $resolution = NULL, &$raw = false)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        return array($translate->_('TOTAL'));
    }

    protected function _getXLabelRaw() {
        return array_keys($this->_agg);
    }

    protected function _getDataLabels($lang)
    {
        $ret = $this->_count_data_set->getDescriptionsForLastFetchedData();
        unset($ret['cnttime']);
        return $ret;
    }

    private function _stringsToTranslate() {
        $this->_translate->_('TOTAL_SUM_OF_DATA_SOURCES');
    }

    public function setDefaultsJson($defaults) {
        // for each value in default, assign to self
        foreach ($defaults as $v => $value) {
            $this->$v = $value;
        }
        if($this->_relative) $this->_display_unit = '%';
        return True;
    }

    public function dumpJson() {
        $enc = array();
        foreach($this->_defaults_to_store as $k) {
            $enc[$k] = $this->$k;
        }

        return Zend_Json::encode($enc);
    }
}
