<?php

class Charts_Model_DataAggregator_Unit extends Charts_Model_DataAggregatorAbstract
{

    private $_xlabels = null;
    private $_resolution = null;
    public $nbrBins = null;

    public function  __construct() {
        parent::__construct();
        $this->_defaults_to_store = array();
    }

    private function _get_resolution() {
        if(!is_null($this->_count_data_set)) {// backwards compability 
            if(!$this->_database_inited) {
                $this->_count_data_set->initDatabaseSelect();
                $this->_database_inited = true;
            }
            if($this->_force_wanted_resolution) return $this->_wanted_resolution;
            return $this->_count_data_set->getBestResolution($this->_wanted_resolution, $this->_from_date, $this->_to_date);
        }
        if(!is_null($this->_wanted_resolution))
                return $this->_wanted_resolution;
        //protected $_from_date = null;
        //protected $_to_date = null;
        if(!isset($this->_to_date) || !isset($this->_from_date)){
            throw new WebReport_Exception_ParameterError("to_date and from_date must be set");

        }
        $sub = new Zend_Date($this->_to_date);
        $sub->sub($this->_from_date);
        if($sub->getYear()->getTimeStamp()/(24*60*60*365) > 4) {
            return 'ts1q';
        }
        if((int)$sub->getYear()->getTimeStamp()/(24*60*60*365) > 0) {
            return 'ts1m';
        }
        if((int)$sub->getMonth()->getTimeStamp()/(24*60*60*30) > 0) {
            return 'ts1w';
        }
        if((int)$sub->getDay()->getTimeStamp()/(24*60*60) > 15) {
            return 'ts1d';
        }
        if((int)$sub->getDay()->getTimeStamp()/(24*60*60) > 0) {
            return 'ts1h';
        }

        return 'ts30min';
    }

    public function getNbrBinsWithData() {
        $bins = 0;
        foreach($this->_agg as $data) {
            if ( $data->fields ) $bins += 1;
        }
        return $bins;
    }

    public function getResolution(){
        return $this->_get_resolution();
    }

    protected function _renderData()
    {
        $this->_resolution = $this->_get_resolution();
        $data = $this->getRawData($this->_resolution,$this->_timeFilter);
        $agg = array();
        $this->_xlabels = array();
        $sum = array();
        foreach ($data as &$cur) {
            $base = new stdClass();
            foreach($cur as $key => &$value) {
                if($key == 'cnttime') continue;
                if(!is_null($value)) {
                    if(!$this->_cumulative){
                        $base->$key = $value;
                    } else {
                        if(!array_key_exists($key,$sum)) $sum[$key] = $value;
                        else $sum[$key] += $value;
                        $base->$key = $sum[$key];
                    }
                }else {
                    $base->$key = 0;
                }
            }

            $agg[] = $base;
            $this->_xlabels[] = $cur->cnttime;
        }
        $this->nbrBins = count($agg);
        $this->_agg = $agg;
        return $agg;
    }


    protected function _getXLabel($lang, $times = NULL, $resolution = NULL, &$raw = false)
    {
        $format = Zend_Date::ISO_8601;
        if(is_null($resolution))
            $resolution = $this->_resolution;
        switch ($resolution) {
            case "ts1min":
            case "ts5min":
            case "ts10min":
            case "ts15min":
            case "ts30min":
                // full time stamp
                break;
            case "ts1h":
            case "ts6h":
            case "ts12h":
                $format = Zend_Date::DATE_MEDIUM;
                //hour only
                break;
            case "ts1d":
                $format = Core_Model_User::GetCurrentUsersDateFormat();
                if(!$format) $format = Zend_Date::DATE_SHORT;
                //date
                break;
            case "ts1w":
            case "ts2w":
                $format = "Y-ww";
                break;
            case "ts1m":
                $format = Zend_Date::YEAR . "-" . Zend_Date::MONTH_NAME_SHORT;
                break;
            case "ts1q":
                //todo fixme, quarters is not supported
                $format =  Zend_Date::MONTH_NAME . " " . Zend_Date::YEAR;
                break;
            case "ts1y":
                $format = Zend_Date::YEAR;
                //yearly
                break;
        }
        $res = array();
        
        if(is_null($times))
            $times = $this->_xlabels;
        foreach($times as $key => $value) {
            if(is_string($value))
                $d= new Zend_Date($value, Zend_Date::ISO_8601);
            else
                $d = clone $value;
            if(Zend_Registry::get('Zend_Locale') == 'en_US' && !(strpos($format,"w") === false)){
                $d->addDay(1);
            }
            $res[$key] = $d->toString($format);
        }
        return $res;
    }

    protected function _getXLabelRaw() {
        return $this->_xlabels;
    }

    protected function _getDataLabels($lang)
    {
        $ret = $this->_count_data_set->getDescriptionsForLastFetchedData();
        unset($ret['cnttime']);
        return $ret;
    }

    private function _stringsToTranslate() {

    }

    public function setDefaultsJson($defaults) {
        // for each value in default, assign to self
        foreach ($defaults as $v => $value) {
            $this->$v = $value;
        }
        return True;
    }

    public function dumpJson() {
        $enc = array();
        foreach($this->_defaults_to_store as $k) {
            $enc[$k] = $this->$k;
        }

        return Zend_Json::encode($enc);
    }
}
