<?php
abstract class Charts_Model_DataAggregatorAbstract
{
    const MODE_HIST_UNIT = 0;
    const MODE_HIST_OVER_HOUR = 1;
    const MODE_HIST_OVER_WEEK = 2;
    const MODE_HIST_OVER_MONTH = 3;
    const MODE_HIST_OVER_DAY = 4;
    const MODE_HIST_OVER_DAYS_IN_MONTH = 5;
    const MODE_HIST_OVER_15MIN = 6;
    protected $_data_set = null;
    protected $_count_data_set = null;
    protected $_pos_data = null;
    protected $_ready = FALSE;
    protected $_from_date = null;
    protected $_to_date = null;
    protected $_defaults_to_store = null;
    protected $_display_unit = '';
    protected $_timeFilter = null;
    protected $_wanted_resolution = null;
    protected $_force_wanted_resolution = false;
    protected $_trim = false;
    protected $_trim_limit = 8;
    protected $_trim_left  = 0;
    protected $_trim_right = 0;
    protected $_cumulative = false;
    protected $_mode = self::MODE_HIST_UNIT;
    protected $_isAverage = false;
    protected $_use_cache = FALSE;

    protected $_database_inited = false;

    abstract protected function _renderData();
    abstract public function getResolution();
    abstract protected function _getXLabel($lang, $times, $resolution, &$raw);
    abstract protected function _getXLabelRaw();
    //abstract protected function _getYLabel($lang);
    abstract protected function _getDataLabels($lang);
    abstract protected function getNbrBinsWithData();

    public function __construct() {
        //load settings for this class based on some id
    }

    public function useCache($use = TRUE) {
        $this->_use_cache = $use;
    }

    public function getDisplayUnit()
    {
        return $this->_display_unit;
    }

    public function setIsAverage($isAverage)
    {
        $this->_isAverage = $isAverage;
    }

    public function getIsAverage(){
        return $this->_isAverage;
    }

    public function getMode() {
        return $this->_mode;
    }

    public function setWantedResolution($resolution, $force = false){
        $this->_wanted_resolution = $resolution;
        $this->_force_wanted_resolution = $force;
    }

    private function checkReady() {
        if($this->_data_set || $this->_count_data_set || $this->_pos_data) {
            $this->_ready = TRUE;
        } else {
            $this->_ready = FALSE;
        }
        return $this->_ready;
    }

    public function setTimeFilter($timeFilter){
        if (is_string($timeFilter)
                || get_class($timeFilter) == "Core_Model_CountData_TimeFilter"
                || is_null($timeFilter))
            $this->_timeFilter = $timeFilter;
        return $this;
    }

    protected function getRawData($resolution = 'ts1h', $timeFilter = null, $countData = null) {
        if($this->_use_cache) {
            $cache = Zend_Registry::get('cache');
            $key = "ChartsModelDataAggregatorAbstract".md5(print_r($this,true)).round(time()/60*10);
            $result = $cache->load($key);
            if($result) {
                return $result;
            }
        }
        if(is_null($this->_count_data_set) && is_null($this->_data_set)){
            if(is_null($this->_pos_data)){
                throw new WebReport_Exception_ParameterError("NO_COUNT_DATA_SET");
            }
            return $this->_pos_data->getData($resolution, $this->_from_date,$this->_to_date, $timeFilter);
        }
        //very confusing stuff. these names need to be proper!!!!!!!
        if(is_null($this->_count_data_set)) {
            $this->_count_data_set = $this->_data_set->getCountData();
        }
        if(!$this->_database_inited) {
            $this->_count_data_set->initDatabaseSelect();
            $this->_database_inited = true;
        }
        $count_data = $this->_count_data_set;
        $data = $count_data->getData($resolution, $firstDate = $this->_from_date, $lastDate = $this->_to_date, $timeFilter);
        if($this->_use_cache) {
            $cache->save($data, $key);
        }
        return $data;
    }

    public function setData(Core_Model_CountData_Set $count_data_set = null,$overide=FALSE) {
        if($this->_count_data_set != null && ! $overide) {
            throw new WebReport_Exception('COUNT_DATA_SET_WAS_SET_ALREADY');
        }

        $this->_data_set = $count_data_set;
        return $this;
    }

    public function setCountData(Core_Model_CountData $count_data, $override=false) {
        if($this->_count_data_set && !$override) {
            throw new WebReport_Exception('COUNT_DATA_WAS_SET_ALREADY');
        }

        $this->_count_data_set = $count_data;
        return $this;
    }

    public function setPosData(Core_Model_PosData $posData){
        $this->_pos_data = $posData;
        return $this;
    }

    public function setPeriodFromCode($now, $code) {
        $int = new CM_Interval($now, $code);
        $this->setPeriod($int->getStartDate(), $int->getStopDate());
    }

    public function setPeriod($from=null, $to=null) {
       $this->_from_date = $from;
       $this->_to_date = $to;
       return TRUE;
    }

    public function getPeriod(){
        return array("from" => $this->_from_date, "to" => $this->_to_date);
    }

    public function getXLabel($times = NULL,$resolution = NULL, &$raw = false) {
        $locale = Zend_Registry::get('Zend_Locale');
        $lbl = $this->_getXLabel($locale->getLanguage(), $times, $resolution, $raw);
        if(!$this->_trim || true) return $lbl;
        $cnt = count($lbl);
        for($i = 0; $i < $this->_trim_left;$i++){
            unset($lbl[$i]);
            if ($raw !== false) unset($raw[$i]);
        }
        for($i = 0; $i < $this->_trim_right ;$i++){
            unset($lbl[$cnt - $i - 1]);
            if ($raw !== false) unset($raw[$cnt - $i - 1]);
        }
        $raw = array_values($raw);
        return array_values($lbl);
    }

    public function getXLabelsRaw() {
        $lbls = $this->_getXLabelRaw();
        return array_values($lbls);
    }

    /*public function getYLabel() {
        return $this->_getYLabel(Zend_Auth::getInstance()->getIdentity()->language);
    }*/

    public function getDataLabels($array = true) {
        $lbls = $this->_getDataLabels(Zend_Auth::getInstance()->getIdentity()->language);

        if($array)
            return array_values($lbls);
    
        return $lbls;
    }

    public function renderData() {
        if(!$this->checkReady())
            return array();

        //$this-a>_data_set->initDatabaseSelect();
        $data = $this->_renderData();
        $data = array_values($data);
        return $data;
    }

    public function renderCleanData() {
        if(!$this->checkReady())
            return array();

        //$this-a>_data_set->initDatabaseSelect();
        $key = NULL;
        $data = $this->_renderData();
        foreach($data as &$d) {
            if ( $key == NULL) {
                $key = key(get_object_vars($d));
            }
            $d = isset($d->$key) ? $d->$key : 0;
        }
        return $data;
    }

    // needs to be abstract due to privacy restrictions
    public abstract function setDefaultsJson($defaults);
    public abstract function dumpJson();
    // should contain something like below
    /*
    public function setDefaultsJson($defaults) {
        // for each value in default, assign to self
        foreach ($defaults as $v) {
            $this->$v = $defaults->$v;
        }
        return True;
    }*/

    /*
    public function dumpJson() {
        $enc = array();
        foreach($this->_defaults_to_store as $k) {
            $enc[$k] = $this->$k;
        }

        return Zend_Json::encode($enc);
    }*/
    public function getTimeFilterHours() {
        if($this->_timeFilter != '') {
            preg_match('/hour\s*>=\s*(\d\d|\d)/', $this->_timeFilter, $lowHour);
            preg_match('/hour\s*<=\s*(\d\d|\d)/', $this->_timeFilter, $highHour);
            
            $lowHour = $lowHour[1];
            $highHour = $highHour[1];
        } else {
            $lowHour = 0;
            $highHour = 23;
        }
        if($this->_mode == self::MODE_HIST_OVER_15MIN) {
            $lowHour = $lowHour * 4;
            $highHour = $highHour*4;
        }
        
        return array($lowHour, $highHour);
    }

    public function getTimeFilterDays(){
        $days = array("monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday");
        $retdays = array();
        foreach($days as $day)
            if(strpos($this->_timeFilter,$day) !== false)
                $retdays[] = $day;
        return $retdays;
    }

    public function getNbrBins(){
        $nbrBins = 1;
        switch($this->getMode()){
            case 0:
                $res = $this->getResolution();
                if($res == 'ts1w' || $res == 'ts1m' || $res == 'ts1y'){
                    $nbrBins = count($this->getXLabel());
                }else if($res == 'ts1d'){
                    $nbrBins = $this->nbrBins ? $this->nbrBins : 1; 
                }
                break;
            case self::MODE_HIST_OVER_HOUR:
                $hours = $this->getTimeFilterHours();
                $nbrBins = ($hours[1] - $hours[0]) + 1;
                break;
            case self::MODE_HIST_OVER_15MIN:
                $hours = $this->getTimeFilterHours();
                $nbrBins = ($hours[1] - $hours[0]) + 1;
                break;
            case self::MODE_HIST_OVER_WEEK:
                $nbrBins = count($this->getTimeFilterDays());
                break; 
        }
        return $nbrBins;
    }
}
