<?php

class Charts_Model_DataAggregatorList extends Talking_Db_Table
{
    protected $_name = 'mod_charts_dataaggregator';
    protected $_primary = 'dataaggregator_id';
    public    $show_relative_aggregators = False;

    public function getAggregator($id) {
           $row = $this->find($id)->current();
           if(!isset($row)) return null; // not valid id
           $settings = Zend_Json::decode($row->initobject);
           $obj = new $row->obj;
           if (!method_exists($obj,"setDefaultsJson")) {
                throw new WebReport_Exception();
           }
           $obj->setDefaultsJson($settings);
           $obj->_id = $id;
           return $obj;
    }

    public static function getMatchingPeriodCodes($id){
        $aggDb = new Charts_Model_DataAggregatorList();
        $usage = $aggDb->find($id)->current()->name;
        switch($usage){
            case "HOUR_USAGE":
                $ret = array('0d', '-1d', '-2d', '-3d', '-4d', '-5d', '-6d', '-7d');
                break;
            case "WEEK_USAGE":
                $ret = array('0w', '-1w', '-2w', '-3w', '-4w');
                break;
            case "DAY_OF_MONTH_USAGE":
                $ret = array('0m', '-1m', '-2m', '-3m', '-4m');
                break;
            case "MONTH_USAGE":
                $ret = array('0y', '-1y', '-2y', '-3y', '-4y');
                break;
        }
        return $ret;
    }

    public static function getUsageById($id){
        $db = new Charts_Model_DataAggregatorList();
        $res = $db->fetchAll($db->select()->where("dataaggregator_id = ?", $id))->current();
        return $res->name;
    }

    public static function getAggregatorById($id){
        if($id == 0 || $id == null) return new Charts_Model_DataAggregator_Unit();
        $agg = new Charts_Model_DataAggregatorList();
        $res =  $agg->getAggregator($id);
 //       if($res == null ) return new Charts_Model_DataAggregator_Unit();
        return $res;
    }

    public static function getAggregatorID(Charts_Model_DataAggregatorAbstract $aggregator=null) {

        if(isset($aggregator) && isset($aggregator->_id))
            return $aggregator->_id;
        return -1; // unknown and does not strictly require a database id to be valid
    }

    public function getFormElement($name = 'data_aggregator', $label = null, $isRequired = false, $nullValueString = null, $filter = array())
    {
        $data_aggregator = $this;
        $rows = $data_aggregator->fetchAll($data_aggregator->select());
	$data_aggregator_id = new UniForm_Form_Element_Select($name);
	$data_aggregator_id->setLabel($label);
	$data_aggregator_id->isRequired($isRequired);
	if(!is_null($nullValueString)){
	    $data_aggregator_id->addMultiOption(0,$nullValueString);
    }
    $show_relative_aggregators = Core_Model_ConfigurationsList::getShowRelative();

    $aggregator_options = array('DAY_USE'        => false,                      //DAY_USE (hidden)
                     'DAY_USE(RELATIVE)'         => $show_relative_aggregators, //DAY_USE_(RELATIVE) (hidden)
                     'WEEK_USAGE'                => true,                       //DAY_OF_WEEK
                     'MONTH_USAGE'               => true,                       //MONTH
                     'HOUR_USAGE'                => true,                       //HOUR
                     'TOTAL_SUM_OF_DATA_SOURCES' => true,                       //TOTAL_SUM_OF_DATA_SOURCES
                     'WEEK_USAGE(RELATIVE)'      => $show_relative_aggregators, //DAY_OF_WEEK_(RELATIVE) (hidden)
                     'MONTH_USAGE(RELATIVE)'     => $show_relative_aggregators, //MONTH_(RELATIVE) (hidden)
                     'HOUR_USAGE(RELATIVE)'      => $show_relative_aggregators, //HOUR_(RELATIVE) (hidden)
                     '15_MINUTE_USAGE'           => true,                       //15_MINUTES
                     'DAY_OF_MONTH_USAGE'        => true);                      //DAY_OF_MONTH
    foreach($rows as $row) {
        if ( in_array( $row->name, $filter)) continue;
        if(!isset($aggregator_options[$row->name]) || $aggregator_options[$row->name])
            $data_aggregator_id->addMultiOption($row->dataaggregator_id, $row->name);
    }

	return $data_aggregator_id;

    }

}
