<?php
class Charts_Model_SourceTypes {
  protected static $_allTypes = NULL;
  public static function getInstance() {
    if (self::$_allTypes) return self::$_allTypes;
    $all = new Charts_Model_SourceTypes();
    // Fetch these automatically
    $all->addSourceType(new Charts_Model_SourceType_Sales());
    $all->addSourceType(new Charts_Model_SourceType_Transactions());
    $all->addSourceType(new Charts_Model_SourceType_ConversionRate());
    $all->addSourceType(new Charts_Model_SourceType_AveragePurchase());
    $all->addSourceType(new Charts_Model_SourceType_CountDataSet());
    $all->addSourceType(new Charts_Model_SourceType_Parking());
    $all->addSourceType(new Charts_Model_SourceType_Queue());
    $all->addSourceType(new Charts_Model_SourceType_Gender());
    $all->addSourceType(new Charts_Model_SourceType_PeopleCounter());
    $all->addSourceType(new Charts_Model_SourceType_Occupancy());
    $all->addSourceType(new Charts_Model_SourceType_Formula());
    $all->addSourceType(new Charts_Model_SourceType_VirtualCamera());
    self::$_allTypes = $all;
    return $all;
  }

  public static function CreateChosen($dataTypeId, $id, $uniqueIdentifier) {
      $chosen = new stdClass;
      $chosen->dataTypeId = $dataTypeId;
      $chosen->id = $id;
      $chosen->identifier = $uniqueIdentifier;
      return $chosen;
  }

  public static function getSourceType($identifier) {
    $types = self::getInstance();
    return $types->getTypeByIdentifier($identifier);
  }

  public static function getSelections($identifier, $dataTypeId) {
    $types = self::getInstance();
    $type = $types->getTypeByIdentifier($identifier);
    if(method_exists($type, "expandSelections")) {
        return $type->expandSelections($dataTypeId);
    }
    return array($dataTypeId);
  }

  public function getTypeByIdentifier($identifier) {
      return $this->_sourceTypes[$identifier];
  }

  public function canMerge($cams){
      $axis = null;
      foreach($cams as $c){
          $source = $this->getTypeByIdentifier($c[4]);
          $axArray = $source->getAxes($c[2]);
          if(in_array($c[4],self::getPosIdentifiers()))
              unset($axArray[1]);
          foreach($axArray as $a){
              if($axis == null) $axis = $a;
              else{
                  if($a->merge($axis) == null) return false;
                  else $axis = $a->merge($axis);
              }
          } 
      }
      return true;
  }

  public function __construct() {
    $this->_sourceTypes = array();
  }

  public function addCountData($id, $type, $sourceTypeIdentifier, $countData) {
    if(!$this->isCountData($type, $sourceTypeIdentifier))
      throw new WebReport_Exception("Can't add type $type for source $sourceTypeIdentifier, it is not Count Data.");
    $this->_sourceTypes[$sourceTypeIdentifier]->addCountData($id, $type, $countData);
  }

  public function isCountData($type, $sourceTypeIdentifier) {
    return $this->_sourceTypes[$sourceTypeIdentifier]->isCountData($type);
  }

  public function addSourceType($sourceType) {
    $this->_sourceTypes[$sourceType->getUniqueIdentifier()] = $sourceType;
  }

  public static function getPosIdentifiers($onlyMoney = null){
      if($onlyMoney) return array("AP","S");
      else return array("TA","CR","AP","S");
  }

  public function getSourcesFromString($user, $string, $folder_id = null) {
    if(strlen($string) < 1 && is_null($folder_id))
        return array();

    $documentsDb = Core_Model_Documents::getInstance();
    $camerasDb = Core_Model_Cameras::getInstance();

    $filter = array("count_data_set", "virtual_camera", "formula_set", "importjob");
    $documents = $documentsDb->searchDocuments($string, 100, $user, $folder_id, $filter);
    $cameras = $camerasDb->searchCameras($string, 100, $user, $folder_id);

    $sources = array();
    $tvpc = new Core_Model_Application_PeopleCounter();
    $tvp = new Core_Model_Application_PeopleCounter();


    foreach($documents as $document) {
        $id = $document->id;
        $name = $document->name;
        $sourceTypes = array();
        $app_id = -2;

        switch($document->type) {
        case "count_data_set":
            $sourceTypes = array($this->getSourceType("CDS"));
            $db = Core_Model_CountData_Sets::getInstance();
            $app_id = $db->find($id)->current()->application_id;
            break;
        case "virtual_camera":
            $sourceTypes = array($this->getSourceType("VC"));
            $db = Core_Model_VirtualCameras::getInstance();
            $app_id = $db->find($id)->current()->application_id;
            break;
        case "formula_set":
            $sourceTypes = array($this->getSourceType("F"));
            $db = Core_Model_CountData_FormulaSets::getInstance();
            $app_id = $db->find($id)->current()->application_id;
            break;
        case "importjob":
            $sourceTypes = array();
            $app_id = -1;
            foreach(self::getPosIdentifiers() as $_id)
                $sourceTypes[] = $this->getSourceType($_id);
            break;
        }
        $keep = false;
        if ( $app_id == -1 ) $keep = true;
        if ( $app_id == $tvpc->getId() ) $keep = true;
        if ( $app_id == $tvp->getId() ) $keep = true;
        if ( $keep == false ) continue;
        foreach($sourceTypes as $sourceType) {
            $sources = array_merge($sources, $sourceType->toSources($id, $name));
        }
    }

    foreach($cameras as $camera) {
        $id = $camera->id;
        $name = $camera->name;
        $sourceTypes = array();

        switch($camera->application_id) {
        case 1:
            $sourceTypes = array($this->getSourceType("PC"));
            break;
        case 3:
            $sourceTypes = array($this->getSourceType("P"));
            break;
        case 7:
            $sourceTypes = array($this->getSourceType("Q"));
            break;
        case 8:
            $sourceTypes = array($this->getSourceType("G"));
            break;
        case 11:
            $sourceTypes = array($this->getSourceType("OCC"));
            break;
        }

        foreach($sourceTypes as $sourceType) {
            $sources = array_merge($sources, $sourceType->toSources($id, $name));
        }
    }

    return $sources;
  }

  public function getSourcesByType($user, $valPack = null, $idPack = null) {
    $list = array();
    foreach($this->_sourceTypes as $sourceType) {
      $sources = array();
      if(isset($list[$sourceType->getGroupTitle()])) $sources = $list[$sourceType->getGroupTitle()];
      foreach($sourceType->getAllSources($user, $valPack, $idPack) as $id => $source) {
        $sources[$id] = $source;
      }
      $list[$sourceType->getGroupTitle()] = $sources;
    }
    return $list;
  }
}
