<?

class Charts_Model_Table extends Zend_Db_Table_Row_Abstract {
    public $_sections = NULL;
    public $now = NULL;
    public $use_cache = NULL;

    public function __construct($configOrTitle = array(), $description = "") {
        if(is_string($configOrTitle)) {
            throw new Exception("Not support anymore");
        }
        parent::__construct($configOrTitle);
        $this->_sections = array();


        if(is_array($configOrTitle) and count($configOrTitle) > 0) {
            $arr = $this->toArray();
            if (array_key_exists("table_id", $arr) and $arr["table_id"]) {
                $tsDb = Charts_Model_TableSections::getInstance();
                $this->_sections = $tsDb->fetchAll("table_id = ". $arr["table_id"]);
                $this->meta_data = json_decode($arr['meta_data']);
            }
        } else {
            throw new Exception("This is also not supported anymore");
        }
    }

    public function useCache($use = TRUE) {
        $this->use_cache = $use;
    }

    public function getTimeFilterCode() {
        return sprintf("hour >= %d & hour <= %d", $this->hour_from, $this->hour_to);
    }

    public function setTitle($title){
        $this->title = $title;
    }

    public function getNow() {
        if ($this->now) return $this->now;
        return new Zend_Date();
    }

    /*
        Create a new, non-empty, Table for end-user usage.
    */
    public static function CreateNew($translate = false, $simple_mode = false) {
        $aggDb = Charts_Model_DataAggregatorList::getInstance();
        $agg = $aggDb->fetchAll($aggDb->select()->where("description = 'SHOW_USAGE_DISTRIBUTED_OVER_MONTH'"))->current();
        $tDb = Charts_Model_Tables::getInstance();
        $table = $tDb->createRow();
        $table->title = $translate ? $translate->translate(_T("A_NEW_TABLE")) : "";
        $table->color_hex = "#bfbfbf";
        $table->mode = $agg->dataaggregator_id;
        $table->description = "";
        $table->now = NULL;
        $table->hour_from = 8;
        $table->hour_to = 20;
        $table->show_tot = 1;
        $table->show_avg = 0;
        $table->comp_tot = "never";
        $table->comp_avg = "never";
        $table->meta_data = array("simple_mode" => $simple_mode ? true : false);
        return $table;
    }

    public static function demo() {
        $tbl = new Charts_Model_Table();
        $tbl->title = "This is an example table";
        return $tbl;
    }

    public static function fromJson($json) {
        return self::fromArray(json_decode($json, true));
    }

    public static function fromArray($array) {
        $tbl = Charts_Model_Table::CreateNew();
        $tbl->title = $array["title"];
        $tbl->color_hex = $array["color_hex"];
        $tbl->mode = $array["mode"];
        $tbl->description = $array["description"];
        $tbl->hour_from = $array["hour_from"];
        $tbl->hour_to = $array["hour_to"];
        $tbl->show_tot        = $array['show_tot'];
        $tbl->show_avg        = $array['show_avg'];
        $tbl->comp_tot        = $array['comp_tot'];
        $tbl->comp_avg        = $array['comp_avg'];
        $tbl->meta_data       = $array['meta_data'];
        if(array_key_exists("sections", $array)) {
            foreach($array["sections"] as $section) {
                $s = Charts_Model_TableSection::fromArray($section);
                $tbl->addSection($s);
            }
        }
        return $tbl;
    }

    public function toJson() {
        return json_encode($this->toNiceArray());
    }

    public static function loadFromDatabase($id) {
        $tDb  = Charts_Model_Tables::getInstance();
        $tsDb = Charts_Model_TableSections::getInstance();

        $table = $tDb->find($id)->current();
        return $table;

        if (!$table) return false;
        // What I really want to do:
        //$sections = $table->findDependentRowset("Charts_Model_TableSection");
        $sections = $tsDb->fetchAll($tsDb->select(array("table_section_id"))->where("table_id = ?", $id));
        $table->_sections = array();
        foreach($sections as $sectionRow) {
            $table->sections[] = Charts_Model_TableSection::loadFromDatabase($sectionRow);
        }
        return $table;
    }

    public function storeToDatabase($id = null) {
        $data = $this->toNiceArray();
        $tDb  = Charts_Model_Tables::getInstance();

        if(!is_null($id)) {
            $tableRow = $tDb->find($id)->current();
            if(!$tableRow) {
                throw new WebReport_Exception_ParameterError("Not a valid id for the table.");
            }
        } else {
            $tableRow = $tDb->createRow();
        }
        $data['meta_data'] = json_encode($data['meta_data']);
        $tableRow->setFromArray($data);
        $tableId = $tableRow->save();

        Charts_Model_TableSections::removeSectionsForTable($tableId);
        foreach($this->_sections as $section) {
           $section->storeToDatabase($tableId);
        }
        return $tableId;
    }

    public function toNiceArray() {
        $list = array(
            "description"    => $this->description,
            "sections"       => array(),
            "title"          => $this->title,
            "color_hex"      => $this->color_hex,
            "mode"           => $this->mode,
            "hour_from"      => $this->hour_from,
            "hour_to"        => $this->hour_to,
            "show_tot"       => $this->show_tot,
            "show_avg"       => $this->show_avg,
            "comp_tot"       => $this->comp_tot,
            "comp_avg"       => $this->comp_avg,
            "meta_data"      => $this->meta_data,
        );
        foreach($this->_sections as $s) $list["sections"][] = $s->toNiceArray();
        return $list;
    }

    public function getColors() {
        $colors = array();
        $colors['picked']       = $this->color_hex;
        $colors['pickedHover']  = $this->interpolate($this->color_hex,"000000",0.1); 
        $colors['odd']          = $this->interpolate($this->color_hex,"FFFFFF",0.5); 
        $colors['oddHover']     = $this->interpolate($colors['odd'],"000000",0.1); 
        $colors['even']         = $this->interpolate($this->color_hex,"FFFFFF",0.8); 
        $colors['evenHover']    = $this->interpolate($colors['even'],"000000",0.1); 
        return $colors;
    }

    public function interpolate($hex1,$hex2,$scale) {
        $hex1   = substr($hex1,1);
        if(strlen($hex1) == 3) {
            $r1 = hexdec(substr($hex1,0,1).substr($hex1,0,1));
            $g1 = hexdec(substr($hex1,1,1).substr($hex1,1,1));
            $b1 = hexdec(substr($hex1,2,1).substr($hex1,2,1));
        } else {
            $r1 = hexdec(substr($hex1,0,2));
            $g1 = hexdec(substr($hex1,2,2));
            $b1 = hexdec(substr($hex1,4,2));
        }
        if(strlen($hex2) == 3) {
            $r2 = hexdec(substr($hex2,0,1).substr($hex2,0,1));
            $g2 = hexdec(substr($hex2,1,1).substr($hex2,1,1));
            $b2 = hexdec(substr($hex2,2,1).substr($hex2,2,1));
        } else {
            $r2 = hexdec(substr($hex2,0,2));
            $g2 = hexdec(substr($hex2,2,2));
            $b2 = hexdec(substr($hex2,4,2));
        }
        $channel=dechex($r1*(1-$scale)+$r2*$scale);
        if(strlen($channel)<2){
            $channel = "0" . "$channel";
        }
        $ret=$channel;

        $channel=dechex($g1*(1-$scale)+$g2*$scale);
        if(strlen($channel)<2){
            $channel = "0" . "$channel";
        }
        $ret=$ret . $channel; 
        $channel=dechex($b1*(1-$scale)+$b2*$scale);
        if(strlen($channel)<2){
            $channel = "0" . "$channel";
        }
        $ret=$ret . $channel;
        return "#".$ret;
    }


    public function addSection($section) {
        $this->_sections[] = $section;
        return $this;
    }

    public function renderArray($translate, $generateDate = false) {
        if ( $generateDate === false ) {
            $generateDate = new Zend_Date();
        }
        $tlist = array("sections" => array());
        $tlist['colorsHex'] = $this->getColors();

        foreach($this->_sections as $section) {
            $tlist["sections"][] = $section->renderArray($this, $translate, $generateDate, $this->use_cache);
        }

        $timesTitle = "";
        $usage = Charts_Model_DataAggregatorList::getUsageById($this->mode);
        $aggregator = Charts_Model_DataAggregatorList::getAggregatorById($this->mode);
        $xlabels = $aggregator->getXLabel();
        switch ($usage) {
            case 'WEEK_USAGE':
                $timesTitle = $translate->translate(_T("DAY"));
                break;
            case 'MONTH_USAGE':
                $timesTitle = $translate->translate(_T("MONTH"));
                break;
            case 'HOUR_USAGE':
                $timesTitle = $translate->translate(_T("HOUR"));
                $xlabels = array_slice($xlabels,$this->hour_from, $this->hour_to - $this->hour_from + 1);
                break;
            case 'DAY_OF_MONTH_USAGE':
                $period = $this->_sections[0]->period;
                if(strpos($period, "m") == false){
                    $timesTitle = $translate->translate(_T("DAY"));
                }else{
                    $i = new CM_Interval(new Zend_Date(), $period);
                    $timesTitle = $i->toReadable();
                    foreach($this->_sections as $sec){
                        if($sec->period != $period)
                            $timesTitle = $translate->translate(_T("DAY"));
                    }
                }
                break;
            default:
                $timesTitle = $translate->translate(_T("TIME"));
        }

        $tot = $translate->translate(_T("TOTAL"));
        $avg = $translate->translate(_T("AVERAGE"));
        if(count($xlabels)>12){
            $tot = $translate->translate(_T("TOT [short for total]"));
            $avg = $translate->translate(_T("AVG [short for average]"));
        }

        if($this->show_tot) $xlabels[] = $tot;
        if($this->show_avg) $xlabels[] = $avg;
        if($this->comp_tot != "never"){
            $i = new CM_Interval(new Zend_Date(), $this->comp_tot);
            array_splice($xlabels, count($xlabels) - $this->show_avg, 0,  $tot . " " . $i->toReadable());
        }
        if($this->comp_avg != "never"){
            $i = new CM_Interval(new Zend_Date(), $this->comp_avg);
            $xlabels[] = $avg . " " . $i->toReadable();
        }
        $tlist["xlabels"] = $xlabels;
        $tlist["times_title"] = $timesTitle;
        return $tlist;
    }

    public function getSection($id) {
        return $this->_sections[$id];
    }

    public function sourcesVisible($user) {
        $allowedSourceIds = array();
        $usedSourceIds = array();

        if ( $user->multi_company || $user->administrator ) return true;

        $allowedTableSourceTypes = Charts_Model_SourceTypes::getInstance();
        $allowedTableSources = $allowedTableSourceTypes->getSourcesByType($user);
        foreach ($allowedTableSources as $ts) {
            foreach ($ts as $source) {
                $allowedSourceIds[$source['sourcetype']][] = $source['id'];
            }
        }

        $usedSources = $this->toNiceArray();
        $usedSources = $usedSources['sections'];

        foreach ($usedSources as $us) {
            foreach ($us['entries'] as $entry) {
                foreach ($entry['selections'] as $source) {
                    $usedSourceIds[$source[4]][] = $source[1];
                }
            }
        }

        r_log($usedSourceIds, "used Source Ids: ");
        r_log($allowedSourceIds, "allowed Source Ids: ");
        foreach ($usedSourceIds as $key => $value) {
            if (array_key_exists($key, $allowedSourceIds)) {
                $intersect = array_intersect($value, $allowedSourceIds[$key]);
                if ($intersect != $value) {
                    return false;
                }
            } else {
                return false;
            }
        }
        return true;
    }
}
