<?php
class Charts_Model_TablePdfRenderer {

    protected $_font;
    protected $_title;
    protected $_sections;
    protected $_mainColor;
    protected $_secondaryColor;

    protected $_generateDate;
    protected $_weather;
    protected $_page;
    protected $_frame;
    private $_baseTextSize  = NULL;
    private $_limitedTextSize  = NULL;
    private $_entryCount    = NULL;
    protected $_table;

    function __construct($table){
        $this->_table=$table;

    }

    public function prepareRender(){
        $translate = Zend_Registry::get("Zend_Translate");
        $tableData = $this->_table->renderArray($translate,$this->_generateDate);
        $this->_times = new Charts_Model_TableTimesPdfRenderer($tableData['xlabels'], $tableData['times_title']);
        $this->_setSections($tableData['sections']);
        $this->_colorsHex = $tableData['colorsHex'];
        return $this;
    }

    private function _setSections($secsData){
        $secs = array();
        foreach($secsData as $secData){
            $section = new Charts_Model_TableSectionPdfRenderer($secData);
            $secs[] = $section;      
        }
        $this->_sections = $secs;
    }
    
    public function setFont($font){
        $this->_font = $font;
        $this->_times->setFont($font);
        foreach($this->_sections as $sec)
            $sec->setFont($font);
        return $this;
    }

    public function setPage($page){
        $this->_page = $page;
        $this->_times->setPage($page);
        foreach($this->_sections as $sec)
            $sec->setPage($page);
        return $this;
    }

    public function setGenerateDate($date){
        $this->_generateDate = $date;
        return $this;
    }

    public function setWeather($hasWeather){
        $this->_weather = $hasWeather;
        return $this;
    }

    public function setFrame($frame){
        $this->_frame = $frame;
        return $this;
    }

    public function limitTextSize($size){
        $this->_limitedTextSize = $size; 
    }

    public function render(){
        $this->calculateProps();
        $y  = $this->_times->drawTimes($this);
        foreach($this->_sections as $sec)
            $y = $sec->drawSection($y, $this);
    }


    public function getVerticalMargin($textSize = null) {
        if(!$textSize) return $this->getBaseTextSize()*0.45;
        else return $textSize*0.45;
    }

    public function getHorizontalMargin($textSize = null) {
        if(!$textSize) return $this->getBaseTextSize()*0.5;
        else return $textSize*0.5;
    }

    public function getLineWidth() {
        return $this->getBaseTextSize()/15.;
    }

    public function getBaseTextSize() {
        if ( $this->_baseTextSize == NULL )
            throw new WebReport_Exception("No base text size was set.");
        if ( !is_null($this->_limitedTextSize) )
            return $this->_limitedTextSize;
        return $this->_baseTextSize;
    }

    public function getSectionTextSize($textSize = null){
        if (!$textSize) $textSize = $this->getBaseTextSize();
        return round($textSize * 1.25);
    }

    public function getSectionHeight($textSize = null) {
        $textSize = $this->getSectionTextSize($textSize);
        return $textSize + 2*$this->getVerticalMargin($textSize)*1.25;
    }

    public function getEntryHeight($textSize = null) {
        if (!$textSize) $textSize = $this->getBaseTextSize();
        return $textSize + 2*$this->getVerticalMargin($textSize);
    }   

    public function getEntryCellWidth($textSize = null) {
        if (!$textSize) $textSize = $this->getBaseTextSize();
        return ($this->getCanvasWidth() - $this->getTitlesWidth() - $textSize/2)/count($this->_times->_times);
    }

    public function getCanvasWidth() {
        return $this->_frame->getWidth()-(10/2.6); 
    }

    public function getCanvasHeight() {
        return $this->_frame->getHeight()-(10/2.6);
    }

    public function getTableHeight() {
        $eHeight = $this->getEntryHeight();
        $sHeight = $this->getSectionHeight();
        $height = count($this->_sections)*$sHeight + ($this->_entryCount+1)*$eHeight;
        if($this->_times->isTwoRows())
            $height += $this->getBaseTextSize() + $this->getVerticalMargin();
        return $height;
    }

    public function getCanvasMarginX() {
        return (($this->_frame->getX2()-$this->_frame->getX1()) - $this->getCanvasWidth())/2;
    }

    public function getCanvasMarginY() {
        return 10;
        return (($this->_frame->getY2()-$this->_frame->getY1()) - $this->getCanvasHeight())/20;
    }

    public function getStartX() {
        return $this->_frame->getX1() + $this->getCanvasMarginX();
    }

    public function getStartY() {
        return $this->_frame->getY2();
    }

    public function getEndX() {
        return $this->getStartX()+$this->getCanvasWidth();
    }

    public function getEndY() {
        return $this->_frame->getY2()-$this->getTableHeight();
    }

    public function getTitlesWidth() {
        return Reports_Pdf_Drawer::get_text_width($this->_getLongestRowTitle(),$this->getBaseTextSize(), $this->_font) + $this->getHorizontalMargin()*2 + $this->getBaseTextSize();
    }

    public function getBlackZendColor() {
        return new Zend_Pdf_Color_Rgb(0,0,0);
    }

    public function getZendColor() {
        return new Zend_Pdf_Color_Html($this->_colorsHex['picked']);
    }

    public function getEvenZendColor() {
        return new Zend_Pdf_Color_Html($this->_colorsHex['even']);
    }

    public function getOddZendColor() {
        return new Zend_Pdf_Color_Html($this->_colorsHex['odd']);
    }

    public function getLightGrayZendColor() {
        return new Zend_Pdf_Color_Rgb(0.95, 0.95, 0.95);
    }


    public function calculateProps(){
        $longestRowTitle = $this->_getLongestRowTitle();
        $longestEntry    = $this->_times->getLongestString();
        $nbrSections     = count($this->_sections);
        $nbrTimes        = count($this->_times->_times);
        $nbrEntries      = 0;
        foreach($this->_sections as $sec){
            $nbrEntries += count($sec->_entries);
            $str  = $sec->getLongestEntryString();
            $strW = Reports_Pdf_Drawer::get_text_width($str, 10, $this->_font);
            $longestW = Reports_Pdf_Drawer::get_text_width($longestEntry, 10, $this->_font);
            if($strW > $longestW)
                $longestEntry = $str;
        }

        $textSize = 10;
        $textFit  = false;
        while(!$textFit && $textSize > 1){
            $titlesWidth = Reports_Pdf_Drawer::get_text_width($longestRowTitle,$textSize,$this->_font);
            $titlesCellWidth = $titlesWidth + $this->getHorizontalMargin($textSize) * 2;
            $titlesFit = $titlesWidth<$titlesCellWidth && $titlesCellWidth<($this->getCanvasWidth()/5);
    
            $entryCellWidth  = ($this->getCanvasWidth()-$titlesWidth)/$nbrTimes;
            $entryStrLength = Reports_Pdf_Drawer::get_text_width($longestEntry,$textSize,$this->_font);
            $entryFit = (($entryStrLength + 2*$this->getHorizontalMargin($textSize)) < $entryCellWidth);

            $sHeight = $this->getSectionHeight($textSize);
            $eHeight = $this->getEntryHeight($textSize);
            $heightFit = ($this->getCanvasHeight() > $nbrSections*$sHeight + (1+$nbrEntries)*$eHeight);

            if($titlesFit && $entryFit && $heightFit)
                $textFit = true;
            else
                $textSize -= 1;
        }
        
        $this->_entryCount = $nbrEntries;
        $this->_baseTextSize = $textSize;
    }
    
    protected function _getLongestRowTitle(){
        $longestRowTitle = $this->_times->_timesTitle;
        foreach($this->_sections as $sec){
            foreach($sec->_entries as $entry){
                $longestLength = Reports_Pdf_Drawer::get_text_width($longestRowTitle, 10, $this->_font);
                $thisLength    = Reports_Pdf_Drawer::get_text_width($entry->_title, 10, $this->_font);
                if($thisLength > $longestLength) {
                    $longestRowTitle = $entry->_title;
                }
            }
        }
        return $longestRowTitle;
    }
}
