<?php
class Charts_Model_TableRender_Pdf {

    const WANTED_FONTSIZE = 9;
    const SMALLEST_FONTSIZE = 6;
    const WANTED_NUMBERSIZE = 7;
    const SMALLEST_NUMBERSIZE = 3;
    const MARGIN = 1.5;

    protected $_font;
    protected $_title;
    protected $_sections;

    protected $_generateDate;
    protected $_weather;
    protected $_page;
    protected $_frame;

    function __construct($tableData){
        $this->_times = new Charts_Model_TableTimesRender_Pdf($tableData['xlabels']);
        $this->_setSections($tableData['sections']);
    }

    private function _setSections($secsData){
        $secs = array();
        foreach($secsData as $secData){
            $section = new Charts_Model_TableSectionRender_Pdf($secData);
            $secs[] = $section;      
        }
        $this->_sections = $secs;
    }
    
    public function setFont($font){
        $this->_font = $font;
        $this->_times->setFont($font);
        foreach($this->_sections as $sec)
            $sec->setFont($font);
        return $this;
    }

    public function setPage($page){
        $this->_page = $page;
        $this->_times->setPage($page);
        foreach($this->_sections as $sec)
            $sec->setPage($page);
        return $this;
    }

    public function setGenerateDate($date){
        $this->_generateDate = $date;
        return $this;
    }

    public function setWeather($hasWeather){
        $this->_weather = $hasWeather;
        return $this;
    }

    public function setFrame($frame){
        $this->_frame = $frame;
        return $this;
    }

    public function render(){
        $props = $this->_calculateProps();
        $yPos  = $this->_times->drawTimes($props);
        foreach($this->_sections as $sec){
            $sec->setProps($props);
            $yPos = $sec->drawSection($yPos);
        }
    }

    private function _calculateProps(){
        $props = array();
        $props['width']     = $this->_frame->getWidth()-(200/2.6); 
        $props['times']     = $this->_times->_times;
        $props['nbrTimes']  = count($props['times']);
        $props['namesWidth']= Reports_Pdf_Drawer::get_text_width($this->_getLongestRowName(), self::WANTED_FONTSIZE, $this->_font) + 3;
        $props['cellWidth'] = ($props['width']-$props['namesWidth'])/$props['nbrTimes'];
        $props['y2']        = $this->_frame->getY1();

        $nbrSections        = count($this->_sections);
        $nbrEntries         = 0;
        foreach($this->_sections as $sec)
            $nbrEntries    += count($sec->_entries);

        $textSize = $this->_makeLabelsFit($props['cellWidth'], $props['times']);

        //cut labels if text got too small
        if($textSize < self::SMALLEST_FONTSIZE){
            $this->_cutLabels();
            $textSize = $this->_makeLabelsFit($this->_cellWidth);
        }

        //skip labels if text is still too small
        $props['skippedLabels'] = 0;
        $count = 0;
        while($textSize < self::SMALLEST_FONTSIZE && count($props['times'])>2){
            $count++;
            $props['skippedLabels'] = pow(2,$count) - 1;
            $props = $this->_skipEveryOtherLabel($props);
            $textSize = $this->_makeLabelsFit($props['cellWidthAfterSkipping']);
        }

        $props['textSize']          = $textSize; 
        $props['margin']            = self::MARGIN; 
        $props['lineWidth']         = $textSize/12;
        $props['sectionHeight']     = $props['textSize'] + 3*self::MARGIN + $props['lineWidth'];

        //check if wheter can be matched with labels
        // $this->_setWeatherHeight();

        //calculate font size for numbers inside table
        $props['numberSize']    = $this->_makeNumbersFit($props);
        $props['entryHeight']   = $props['numberSize'] + 2*self::MARGIN + $props['lineWidth'];
        $props['height']        = ($nbrSections+1) * $props['sectionHeight'] + $nbrEntries * $props['entryHeight'];
        $props['x1']            = $this->_frame->getX1() + (($this->_frame->getX2()-$this->_frame->getX1()) - $props['width'])/2;
        $props['y1']            = $this->_frame->getY2() - (($this->_frame->getY2()-$this->_frame->getY1()) - $props['height'])/2;
        $props['x2']            = $props['x1'] + $props['width'];

        return $props; 
    }

    protected function _getLongestRowName(){
        $longestRowName     = "DAY_OF_X";
        foreach($this->_sections as $sec){
            foreach($sec->_entries as $entry){
                $longestLength = Reports_Pdf_Drawer::get_text_width($longestRowName, self::WANTED_FONTSIZE, $this->_font);
                $thisLength    = Reports_Pdf_Drawer::get_text_width($entry->_title, self::WANTED_FONTSIZE, $this->_font);
                if($thisLength > $longestLength)
                    $longestRowName = $entry->_title;
            }
        }
        return $longestRowName;
    }

    protected function _makeLabelsFit($width, $labels){
        $longestLabel       = $labels[0];
        $textSize           = self::WANTED_FONTSIZE;
        foreach($labels as $label){
            $labelLength = Reports_Pdf_Drawer::get_text_width($label, $textSize, $this->_font);
            $longestLabelLength = Reports_Pdf_Drawer::get_text_width($longestLabel, $textSize, $this->_font);
            if($labelLength > $longestLabelLength)
                $longestLabel = $label;
        }
        while(Reports_Pdf_Drawer::get_text_width($longestLabel, $textSize, $this->_font) > $width){
            $textSize--;
        }
        return $textSize;
    }

    protected function _cutLabels(){
        if(!preg_match('/\d/', $props['times'][0])){
            foreach($props['times'] as $k => $label){
                if(strlen($label)>4)
                    $props['times'][$k]=substr($label,0,3);
            }
        }
    }

    protected function _skipEveryOtherLabel($props){
        $labels = Array();
        for($k = 0; $k < count($props['times']); $k += 2)
            $labels[$k/2] = $props['times'][$k];
        $this->_times->_times = $labels;
        $gainedWidth = ($props['skippedLabels'] * $props['cellWidth'])*(2/5)*2;
        $props['cellWidthAfterSkipping'] = $gainedWidth + $props['cellWidth'];
        $props['cellSpace'] = ($props['skippedLabels'] * $props['cellWidth'])*(1/5);
        return $props;
    }

    protected function _makeNumbersFit($props){
        $numberSize    = self::WANTED_NUMBERSIZE;
        $longestNumber = 0;
        foreach($this->_sections as $sec){
            foreach($sec->_entries as $entry){
                foreach($entry->_data as $number){
                    $numberLength = Reports_Pdf_Drawer::get_text_width($number,$numberSize, $this->_font);
                    $longestNumberLength = Reports_Pdf_Drawer::get_text_width($longestNumber, $numberSize, $this->_font);
                    if($numberLength > $longestNumberLength)
                        $longestNumber = $number;
                }
            }        
        }
        while(Reports_Pdf_Drawer::get_text_width($longestNumber, $numberSize, $this->_font) > $props['cellWidth']){
            $numberSize -= 1;
        }

        if($numberSize < self::SMALLEST_NUMBERSIZE){
            $numberSize = self::SMALLEST_NUMBERSIZE;
            foreach($this->_sections as $s => $sec){
                foreach($sec->_entries as $e => $entry){
                    foreach($entry->_data as $n => $number){
                        $pot = 0;
                        $numberWidth = Reports_Pdf_Drawer::get_text_width($number, $numberSize, $this->_font);
                        while($numberWidth > $props['cellWidth'] & $number>800){
                            $number = round($number/1000);
                            $pot++;
                        }
                    }if($pot == 1){
                        $this->_sections[$s]->_entries[$e]->_data[$n] = $number . 'k';
                    }else if($pot == 2){
                        $this->_sections[$s]->_entries[$e]->_data[$n] = $number . 'M';
                    }else{
                        $this->_sections[$s]->_entries[$e]->_data[$n] = '∞';
                    }
                }
            }
        }
        return $numberSize;
    }
}
