<?php

class Core_Model_AccuracyPoller {
    public static function MaybeRunAccuracyCalculation(){
        $timestamp = time();
        $config = Zend_Registry::get('config');
        //r_log(gmdate("H:i:s", $timestamp), "Running Accuray Poller, servertime GMT: ");
        $warningsDb = Core_Model_Warnings::getInstance();

        //Set true if it should run every doIteration.
        $debug_skip_hour_check = false;
        //Set true if existing count errors for current date and sections should be overwritten.
        $debug_force_recalculation = false;

        if (isset($config->accuracy->lastCalculation)) {
            if ($debug_skip_hour_check === true || $timestamp - intval($config->accuracy->lastCalculation) >= 3600) {
                /*
                 *Should the following ini write only be done after actually calculating something?
                 *That would meen a check would initially be done every 10 minutes until something is calculated.
                 *Pros: It would likely find an better time for calculating accuracy each hour.
                        (Now the time of the very first poll will define the hourly check time.)
                 *Cons: Unneccesary checks will be done initially.
                 *      If there are cameras that are slow to report compared to other these slow cameras'
                 *      sections could end up not being done at the calculation time making the calculations
                 *      for those one hour "late".
                 */
                try {
                    $writer = new Zend_Config_Writer_IniNice();
                    $writer->setConfig($config)
                        ->setFilename(APPLICATION_PATH . '/config.ini');
                    $config->accuracy->lastCalculation = $timestamp;
                    $writer->write(); // throws exception if not ok
                } catch (Zend_Config_Exception $e) {
                    $select = $warningsDb->select()->where("warning = ?", Core_Model_Warning::CONFIG_NOT_WRITABLE);
                    $warning = $warningsDb->fetchAll($select);
                    if (count($warning) > 0) {
                        $warning = $warning->current();
                        if ($warning->acknowledged == 1) {
                            $warning->acknowledged = 0;
                            $warning->created_date = time();
                            $warning->save();
                        }
                    } else {
                        $warning = $warningsDb->createRow();
                        $warning->warning = Core_Model_Warning::CONFIG_NOT_WRITABLE;
                        $warning->created_date = time();
                        $warning->save();
                    }
                }
                $accuraciesDb = new Core_Model_CameraSet_CameraSetType1SectionAccuracies();
                $accuracies_array = $accuraciesDb->calculateAccuracies($debug_force_recalculation);
                if (isset($accuracies_array)) {
                    //r_log(gmdate("H:i:s", time()), "Finished calculating accuarcies @ servertime GMT");
                    //r_log("==================================================================");
                    /*
                     *TODO: Check if warnings needs to be set and set them.
                     */
                } else {
                    //r_log(gmdate("H:i:s", time()), "No accuracies were to be calculated @ servertime GMT");
                    //r_log("==================================================================");
                }
            } else {
                //r_log("Less than an hour since last accuracy calculation.");
                //r_log("==================================================================");
            }
        }
    }
}
