<?php

include_once 'AclRole.php';

class Core_Model_AclRoles extends Talking_Db_Table
{
    protected $_name = 'acl_role';
    protected $_primary = 'acl_role_id';
    protected $_rowClass = 'Core_Model_AclRole';

    protected $_dependentTables = array('Core_Model_Users');

    protected $_referenceMap    = array(
    	'Company' => array(
    		'columns'			=> array('company_id'),
    		'refTableClass'		=> 'Core_Model_Companies',
    		'refColumns'		=> array('company_id')
    	)
    );

    public function setUpRolesForAllCompanies($logger){
        if($logger == null) $logger = new Zend_Log();
        $companiesDb = new Core_Model_Companies();
        $companies = $companiesDb->fetchAll();
        $roles = new Core_Model_AclRoles();
        
        $logger->notice("Fetching all standard roles..");

        $admins = $this->getRoles("Administrator");
        $creators = $this->getRoles("Creator");
        $viewers = $this->getRoles("Viewer");
        $adminResources = $this->getAllResources();
        $creatorResources = $this->getCreatorResources();
        $viewerResources = $this->getViewerResources();
        $works = array(
            array("role" => "ADMINISTRATORS", "roles" => $admins, "resources" => $adminResources),
            array("role" => "CREATORS", "roles" => $creators, "resources" => $creatorResources),
            array("role" => "VIEWERS", "roles" => $viewers, "resources" => $viewerResources));
        $logger->notice("Found ".count($viewers)." viewers, ".count($creators)." creators and ".count($admins)." administrators.");
        foreach($works as $work){
            $total = 0; $added = 0;
            $logger->notice("Giving ".$work["role"]." rights to ".count($work["resources"])." resources. (This may take a while)");
            foreach($work["roles"] as $acl_role){
                foreach($work["resources"] as $resource){
                    // Very slow but who cares..
                    if($this->addResourceIfMissing($resource, $acl_role)) $added++;
                    $total++;
                }
            }
            $logger->notice("Needed to add ".$added." of ".$total." resources.");
        }
    }

    public function getRoles($name){
        $aclRolesDb = new Core_Model_AclRoles();
        $s = $aclRolesDb->select()->where("name = '".$name."'");
        return $aclRolesDb->fetchAll($s);
    }

    public function getCurrentResources(){
        $hashmap = array();
        $resourceDb = new Core_Model_AclRolePrivilegeGroupAssembly();
        foreach($resourceDb->fetchAll() as $r){
            $key = $r->acl_role_id."-".$r->acl_role_privilege_group_assembly_id;
            $hashmap[$key] = true;
        }
        return $hashmap;
    }
    
    public function addResourceIfMissing($res, $role){
        //throw new Exception(print_r($res,true));
        $resourceDb = new Core_Model_AclRolePrivilegeGroupAssembly();
        $s = $resourceDb->select()
            ->where("acl_role_id = ?",$role->acl_role_id)
            ->where("acl_privilege_group_assembly_id = ?",$res->acl_privilege_group_assembly_id)->limit(1);
        $row = $resourceDb->fetchRow($s);
        if($row != null) return false;
        //return true; // Make it quick
        $resource = $resourceDb->createRow(array(
            "acl_role_id" => $role->acl_role_id,
            "acl_privilege_group_assembly_id" => $res->acl_privilege_group_assembly_id));
        $resource->save();
        
        return true;
    }
    
    public function getAllResources(){
        $aclPrivilegeGroupAssembliesDb = new Core_Model_AclPrivilegeGroupAssemblies();
        return $aclPrivilegeGroupAssembliesDb->fetchAll();
    }

    public function getResourcesByPrivilegeGroupText($rights){
    	$aclPrivilegeGroupAssembliesDb = new Core_Model_AclPrivilegeGroupAssemblies();
        
    	$db = Zend_Registry::get('db');
    	$select = $aclPrivilegeGroupAssembliesDb->select();
    	$modulesDb = new Core_Model_Modules();
    	$aclResourcesDb = new Core_Model_AclResources();
    	$aclPrivilegeGroupsDb = new Core_Model_AclPrivilegeGroups();
        //$select->setIntegrityCheck(false); 
        //$select->join(array("pg" => $aclPrivilegeGroupsDb->info('name'),$aclPrivilegeGroupAssembliesDb->info('name').".acl_privilege_group_id = pg.acl_privilege_group_id"));

        $select = $db->select();
        $select->from(array('pg' => $aclPrivilegeGroupsDb->info('name')));
        //$select->setIntegrityCheck(false); 
        $select->join(array("pga" => $aclPrivilegeGroupAssembliesDb->info('name')),"pg.acl_privilege_group_id = pga.acl_privilege_group_id");

        foreach($rights as $right){
            $select->orWhere("pg.text = '".$right."'");
        }
        return $db->fetchAll($select);
    }

    public function getCreatorResources(){
        $rights = array(
            /* Overview */
                "ABOUT_PAGE","BROWSE_MODULES","SHOW_PROFILE","CHANGE_PASSWORD",
            /* Count data */
                "BROWSE_FILTER_GROUPS","HANDLE_FILTER_GROUPS","VIEW_DASHBOARD","BROWSE_FORMULA_SETS",
                "HANDLE_FORMULAS", "BROWSE_VIRTUAL_CAMERAS","HANDLE_VIRTUAL_CAMERAS","BROWSE_COUNT_DATA_SETS", "HANDLE_COUNT_DATA_SETS",
            /* Reports */
                "BROWSE_REPORTS","HANDLE_REPORTS",
            /* Charts */
                "CUSTOM_CHARTS","TRUEVIEW_APPLICATIONS","BROWSE_CHARTS",
            /* Export data */
                "EXPORT_DATA", 
            /* Subscriptions */
                "BROWSE_SUBSCRIPTIONS", "HANDLE_SUBSCRIPTIONS", "SEND_SUBSCRIPTION",
            /* Other */
                "HEATMAPS","TIME_FILTERS");
        return $this->getResourcesByPrivilegeGroupText($rights);
    }

    public function getViewerResources(){
        $rights = array(
            /* Overview */
                "ABOUT_PAGE","SHOW_PROFILE","CHANGE_PASSWORD",
            /* Count data */
                
            /* Reports */
                "BROWSE_REPORTS",
            /* Charts */
                "BROWSE_CHARTS",
            /* Export data */
                "EXPORT_DATA", 
            /* Subscriptions */
                
            /* Other */
                "HEATMAPS");
        return $this->getResourcesByPrivilegeGroupText($rights);
    }

    public function createRolesWithStandardPrivilegesForCompany($companyId) {
    	$companyId = (int)$companyId;

    	$companyDb = new Core_Model_Companies();
    	if (count($companyDb->find($companyId)) != 1) {
    		throw new WebReport_Exception("No company with id $companyId is available.");
    	}

    	$adminData = array(
    		'company_id' => $companyId,
    		'name' => 'Administrator'
    	);

    	$admin = $this->createRow($adminData);
    	$admin->save();
    	$adminId = $admin->acl_role_id;

    	$aclPrivilegeGroupAssembliesDb = new Core_Model_AclPrivilegeGroupAssemblies();
    	// all rights for the admin role
    	$adminAssemblies = $aclPrivilegeGroupAssembliesDb->fetchAll();

    	$roleGroupAssembliesDb = new Core_Model_AclRolePrivilegeGroupAssembly();

    	foreach ($adminAssemblies as $adminAssembly) {
    		$rowData = array(
    			'acl_privilege_group_assembly_id' => $adminAssembly->acl_privilege_group_assembly_id,
    			'acl_role_id' => $adminId
    		);
    		$row = $roleGroupAssembliesDb->createRow($rowData);
    		$row->save();
    	}


    	$creatorData = array(
    		'company_id' => $companyId,
    		'name' => 'Creator'
    	);

    	$creator = $this->createRow($creatorData);
    	$creator->save();
    	$creatorId = $creator->acl_role_id;

    	$db = Zend_Registry::get('db');
    	$select = $db->select();
    	$modulesDb = new Core_Model_Modules();
    	$aclResourcesDb = new Core_Model_AclResources();
    	$aclPrivilegeGroupsDb = new Core_Model_AclPrivilegeGroups();

    	$select->from(array('m' => $modulesDb->info('name')), array('module_alias' => 'alias'))
    		->join(array("aclr" => $aclResourcesDb->info('name')), "m.module_id = aclr.module_id", array("resource_text" => "text"))
    		->join(array("aclpg" => $aclPrivilegeGroupsDb->info('name')), "aclr.acl_resource_id = aclpg.acl_resource_id", array("privilege_group_text" => "text"))
    		->join(array("aclpga" => $aclPrivilegeGroupAssembliesDb->info('name')), "aclpg.acl_privilege_group_id = aclpga.acl_privilege_group_id", array("privilege_group_assembly_text" => "text", "acl_privilege_group_assembly_id"))
    		->orWhere("m.alias = 'mod_admin' AND (aclr.controller = 'formulas' OR aclr.controller = 'virtual-cameras' OR aclr.controller = 'filter-groups' OR aclr.controller = 'index' OR aclr.controller = 'about' OR aclr.controller = 'profile')")
    		->orWhere("m.alias = 'mod_charts'")
    		->orWhere("m.alias = 'mod_reports'")
    		->orWhere("m.alias = 'mod_count-data'");
    	$creatorAssemblies = $db->fetchAll($select);


    	foreach ($creatorAssemblies as $assembly) {
    		$rowData = array(
    			'acl_privilege_group_assembly_id' => $assembly->acl_privilege_group_assembly_id,
    			'acl_role_id' => $creatorId
    		);
    		$row = $roleGroupAssembliesDb->createRow($rowData);
    		$row->save();
    	}

    	$select = $db->select();
    	$select->from(array('m' => $modulesDb->info('name')), array('module_alias' => 'alias'))
    		->join(array("aclr" => $aclResourcesDb->info('name')), "m.module_id = aclr.module_id", array("resource_text" => "text"))
    		->join(array("aclpg" => $aclPrivilegeGroupsDb->info('name')), "aclr.acl_resource_id = aclpg.acl_resource_id", array("privilege_group_text" => "text"))
    		->join(array("aclpga" => $aclPrivilegeGroupAssembliesDb->info('name')), "aclpg.acl_privilege_group_id = aclpga.acl_privilege_group_id", array("privilege_group_assembly_text" => "text", "acl_privilege_group_assembly_id"))
    		->orWhere("m.alias = 'mod_admin' AND (aclr.controller = 'index' OR aclr.controller = 'about' OR aclr.controller = 'profile')")
    		->orWhere("m.alias = 'mod_charts' AND aclr.controller = 'index' AND (aclpga.text = 'VIEW' OR aclpga.text = 'BROWSE_CHARTS')")
    		->orWhere("m.alias = 'mod_charts' AND aclr.controller = 'chart-producer' AND (aclpga.text = 'View' )")
    		->orWhere("m.alias = 'mod_reports' AND aclr.controller = 'index' AND  (aclpga.text = 'PREVIEW' OR aclpga.text = 'BROWSE' OR aclpga.text = 'export'  )")
    		->orWhere("m.alias = 'mod_reports' AND aclr.controller = 'export'");

    	$viewerData = array(
    		'company_id' => $companyId,
    		'name' => 'Viewer'
    	);

    	$viewer = $this->createRow($viewerData);
    	$viewer->save();
    	$viewerId = $viewer->acl_role_id;


    	$viewerAssemblies = $db->fetchAll($select);
    	foreach ($viewerAssemblies as $assembly) {
    		$rowData = array(
    			'acl_privilege_group_assembly_id' => $assembly->acl_privilege_group_assembly_id,
    			'acl_role_id' => $viewerId
    		);
    		$row = $roleGroupAssembliesDb->createRow($rowData);
    		$row->save();
    	}
    }
}
