<?php

class Core_Model_Api_Method_Installation_MigrateCompany extends WebReport_Api_Method_Abstract
{
    public function userIsAllowed(Core_Model_User $user)
    {
        if ( $user->multi_company ) {
            return true;
        }
    }

    public function needsAuthentication() {
        return true;
    }

    private function disable_ob()
    {
        // Turn off output buffering
        ini_set('output_buffering', 'off');
        // Turn off PHP output compression
        ini_set('zlib.output_compression', false);
        // Implicitly flush the buffer(s)
        ini_set('implicit_flush', true);
        ob_implicit_flush(true);
        // Clear, and turn off output buffering
        while (ob_get_level() > 0) {
            // Get the curent level
            $level = ob_get_level();
            // End the buffering
            ob_end_clean();
            // If the current level has not changed, abort
            if (ob_get_level() == $level) break;
        }
        // Disable apache output buffering/compression
        if (function_exists('apache_setenv')) {
            apache_setenv('no-gzip', '1');
            apache_setenv('dont-vary', '1');
        }
    }

    public function call(array $params)
    {
        $this->disable_ob();
        /*
        const EMERG   = 0;  // Emergency: system is unusable
        const ALERT   = 1;  // Alert: action must be taken immediately
        const CRIT    = 2;  // Critical: critical conditions
        const ERR     = 3;  // Error: error conditions
        const WARN    = 4;  // Warning: warning conditions
        const NOTICE  = 5;  // Notice: normal but significant condition
        const INFO    = 6;  // Informational: informational messages
        const DEBUG   = 7;  // Debug: debug messages
        */
        header("Content-Type: text/plain");
        $params["format"] = WebReport_Api::FORMAT_PRETTY_JSON;
        $log_level = @strtoupper($params["params"]["log_level"]);
        if ($log_level=="") {
            $log_level="DEBUG";
        }
        $writer = new Zend_Log_Writer_Stream("php://stderr");
        $output_writer = new Zend_Log_Writer_Stream("php://output");
        $file_writer = new Zend_Log_Writer_Stream(APPLICATION_PATH . "/../data/session/migration_result.log");

        $this->_logger = new Zend_Log($writer);
        $this->_results_logger = new Zend_Log($output_writer);
        $this->_file_logger = new Zend_Log($file_writer);
        $filter = new Zend_Log_Filter_Priority(constant("Zend_Log::$log_level"));
        $this->_logger->addFilter($filter);
        $this->_logger->info("API Input Params: " . print_r($params["params"], true));
        $webreport_db_user = $params["params"]["dbuser"];
        $webreport_db_pass = $params["params"]["dbpass"];
        $webreport_db_name = $params["params"]["database"];
        $webreport_db_host = $params["params"]["source_host"];
        if (isset($params["params"]["db_charset"])) {
            $webreport_db_charset = $params["params"]["db_charset"];
        } else {
            $webreport_db_charset = "latin1";
        }
        if (isset($params["params"]["company_name"])) {
            $this->_company_name = $params["params"]["company_name"];
        }
        if (isset($params["params"]["company_id"])) {
            $this->_source_company_id = $params["params"]["company_id"];
        }
        if (isset($params["params"]["write_to_db"])) {
            $this->_write_to_db = intval($params["params"]["write_to_db"]);
        } else {
            $this->_write_to_db = 0;
        }
        if (isset($params["params"]["ignore_zeros"])) {
            $this->_ignore_zeros = intval($params["params"]["ignore_zeros"]);
        } else {
            $this->_ignore_zeros = 1;
        }
        $this->_errors = array();

        $this->_manual_handling = $this->_errors = $this->_updated_cameras = array();
        if ($this->_write_to_db != 1) {
            $this->_logger->notice("\n\n ==== Dry run, nothing will actually be written to target DB ====
==== To enable writing to target DB add write_to_db=1 to api call. ==== \n");
            $this->_results_logger->notice("\n\n ==== Dry run, nothing will actually be written to target DB ====
==== To enable writing to target DB add write_to_db=1 to api call. ==== \n");
        }
        if ($this->_ignore_zeros != 1) {
            $this->_ignore_zeros == 0;
        } else {
            $this->_logger->notice("\n\n ==== Speeding things up by ignoring all 15 minute periods with only zeros! ====");
        }

        if (isset($this->_source_name) && isset($this->_source_company_id)) {
            $this->_logger->err(
                "Can't have both company_id and company_name parameter specified. Use only one. - Aborting"
            );
            $this->_errors["initialization"][] = "Bad input parameters - Can't have both company_id and company_name specified. Use only one.";
            $this->_errors["count"]++;
            return $this->printResult();
        }

        if (!isset($webreport_db_user) || !isset($webreport_db_pass) ||
            !isset($webreport_db_name) || !isset($webreport_db_host) ||
            (!isset($this->_company_name) && !isset($this->_source_company_id))) {
            $this->_logger->err(
                "One or more parameters missing, minimum needed is dbuser, dbpass, database, source_host and company_name OR company_id - Aborting"
            );
            $this->_errors["initialization"][] = "Bad input parameters - One or more needed parameters missing";
            $this->_errors["count"]++;
            return $this->printResult();
        }

        $webreport_db_params = array(
                    "host" => $webreport_db_host,
                    "dbname" => $webreport_db_name,
                    "username" => $webreport_db_user,
                    "password" => $webreport_db_pass,
                    "charset" => $webreport_db_charset
            );

        $sourceTables = $this->prepareMigration($webreport_db_params);
        if (isset($sourceTables)) {
            if (!$this->migrateCompanyAndAdmins($sourceTables["admins"])
                || !$this->migrateFolders($sourceTables["folders"])
                || !$this->migrateCameras($sourceTables["cameras"])
                || !$this->migratePermaLinks($sourceTables["permalinks"])
                ) {
                //Migration step failed.
                return $this->printResult();
            }
        } else {
            //Preparations failed.
            return $this->printResult();
        }
        //All OK.
        return $this->printResult();
    }

    private function prepareMigration($webreport_db_params)
    {
        $this->_logger->notice("\n\n ==== Starting preparations! @ servertime " . date("H:i:s", time()) . "====\n");
        $this->_logger->info("Setting up resources.");
        $this->_dataManagerDB = Zend_Registry::get("db");
        $autoloader = Zend_Loader_Autoloader::getInstance();
        $resourceLoader = new Zend_Loader_Autoloader_Resource(array(
            'basePath'  => APPLICATION_PATH . "/modules/core",
            'namespace' => ucfirst("Core")
        ));
        $resourceLoader->addResourceType('model', 'models/', 'Model');
        $this->_logger->info("** Local resources loaded.");

        $this->_webreportDB = Zend_Db::factory("PDO_MYSQL", $webreport_db_params);
        $this->_errors["count"] = 0;

        try {
            $this->_dataManagerDB->getConnection();
        } catch (Exception $e) {
            $this->_logger->debug($e);
            $this->_logger->crit("** Could not connect to target DB - Aborting");
            $this->_errors["initialization"][] = "Could not connect to target DB";
            $this->_errors["count"]++;
            return null;
        }

        try {
            $this->_webreportDB->getConnection();
        } catch (Exception $e) {
            $this->_logger->debug($e);
            $this->_logger->crit("** Could not connect to source DB - Aborting");
            $this->_errors["initialization"][] = "Could not connect to source DB";
            $this->_errors["count"]++;
            return null;
        }

        $this->_logger->info("Gathering information from source DB");
        $companyDb = new Core_Model_Companies($this->_webreportDB);
        if (isset($this->_source_company_id)) {
            $select = $companyDb->select();
            $select->where("company_id = ?", $this->_source_company_id);
            $this->_company = $companyDb->fetchAll($select)->current();
            $this->_company_name = $this->_company->name;
        } else {
            $this->_company = $companyDb->fetchWithName($this->_company_name);
        }
        if (!$this->_company) {
            $this->_logger->crit("** Company not found in source DB - Aborting");
            $this->_errors["initialization"][] = "Specified company was not found in source DB.";
            $this->_errors["count"]++;
            return null;
        }
        $this->_logger->notice("Company named: " . $this->_company_name . " found.");
        $this->_results_logger->notice("Company named: " . $this->_company_name . " found.");
        $this->_skipped = array("cameras" => array(), "admins" => array(),
            "folders" => array(), "company" => array(),
            "camera_groups" => array(), "permalinks" => array());
        $this->_company_id = $this->_company->company_id;
        $foldersDb = new Core_Model_Folders($this->_webreportDB);
        $folders = $foldersDb->fetchAllByCompanyId($this->_company_id);
        $camerasDb = new Core_Model_Cameras($this->_webreportDB);
        $cameras = $camerasDb->getCameras(null, $this->_company);
        $permalinkDb = new Reports_Model_Permalinks($this->_webreportDB);
        $permalinks['reports'] = $permalinkDb->fetchAllForCompany($this->_company);
        $chartPermalinkDb = new Charts_Model_Permalinks($this->_webreportDB);
        $allChartPermalinks = $chartPermalinkDb->fetchAll();
        //Change default DB and registry 'db' to source db temporarily for CountDataSetList class to work.
        $targetDbTemp = Zend_Registry::get('db');
        Zend_Registry::set('db', $this->_webreportDB);
        Zend_Db_Table_Abstract::setDefaultAdapter($this->_webreportDB);
        $permalinks['charts'] = array();
        foreach ($allChartPermalinks as $chartPermalink) {
            if ($chartPermalink->format == "table-html" || $chartPermalink->format == "table-csv") {
                $tableDataDb = new Charts_Model_Tables($this->_webreportDB);
                $tableData = $tableDataDb->fetchAll(
                    $tableDataDb->select()
                        ->where('table_id = ?', $chartPermalink->chart_id)
                )->current();
                if (!isset($tableData)) {
                    r_log("table data for table permalink missing. Table most likely deleted. Skipping.");
                } else {
                    $documentDb = new Core_Model_Documents($this->_webreportDB);
                    $document = $documentDb->fetchAll(
                        $documentDb->select()
                            ->where('id = ?', $chartPermalink->chart_id)
                            ->where('type = ?', "table")
                    )->current();
                    if ($document->getGroup()->company_id == $this->_company_id) {
                        $permalinks['charts'][] = array(
                            'permalink' => $chartPermalink,
                            'data' => $tableData
                            );
                    }
                }
            } else {
                $chartDataDb = new Charts_Model_ChartDataList($this->_webreportDB);
                $chartData = $chartDataDb->fetchAll(
                    $chartDataDb->select()
                        ->where('chart_data_id = ?', $chartPermalink->chart_id)
                )->current();
                if (!isset($chartData)) {
                    r_log("chart data for chart permalink missing. Chart most likely deleted. Skipping.");
                } elseif ($chartData->company_id == $this->_company_id) {
                    $permalinks['charts'][] = array(
                        'permalink' => $chartPermalink,
                        'data' => $chartData
                        );
                }
            }
        }
        Zend_Db_Table_Abstract::setDefaultAdapter($targetDbTemp);
        Zend_Registry::set('db', $targetDbTemp);
        $userDb = new Core_Model_Users($this->_webreportDB);
        $admins = $userDb->fetchAll(
            $userDb->select()
                ->where('company_id = ?', $this->_company_id)
                ->where('active = ?', 1)
                ->where('administrator = ?', 1)
        );
        $users = $userDb->fetchAll(
            $userDb->select()
                ->where('company_id = ?', $this->_company_id)
                ->where('administrator = ?', 0)
                ->where('multi_company = ?', 0)
        );
        $this->_logger->info("** Found " . sizeof($folders) . " company folders.");
        $this->_logger->info("** Found " . sizeof($cameras) . " company cameras.");
        $this->_logger->info(
            "** Found " . (sizeof($permalinks['charts']) + sizeof($permalinks['reports'])) . " company permalinks. (".
            sizeof($permalinks['charts']) . " based on charts/tables, " . sizeof($permalinks['reports']) . " based on data export)."
        );
        $this->_logger->info("** Found " . sizeof($admins) . " company administrators.");
        $this->_logger->notice("** Found " . sizeof($users) . " company users that are NOT administrators THESE WILL NOT BE MIGRATED!");
        $this->_logger->notice("\n\n==== Starting Migration process! @ servertime " . date("H:i:s", time()). "====\n");
        $this->_id_mappings = array("admins" => array(), "folders" => array());
        $sourceTables = array(
            "admins" => $admins,
            "folders" => $folders,
            "cameras" => $cameras,
            "permalinks" => $permalinks
            );
        return $sourceTables;
    }

    private function migrateCompanyAndAdmins($admins)
    {
        $this->_logger->info("Migrating company and administrators...");
        $this->_results_logger->info("Migrating company and administrators...");
        $targetUserDb = new Core_Model_Users();
        $targetCompanyDb = new Core_Model_Companies();
        $company_exists = $targetCompanyDb->fetchAll(
            $targetCompanyDb->select()
                ->where("name = ?", $this->_company->name)
        )->current();
        if (isset($company_exists)) {
            $this->_logger->notice("** Company already exists in target DB - skipping.");
            $this->_skipped["company"][] = $this->_company->toArray();
            $this->_id_mappings["company"][$this->_company_id] = $company_exists->company_id;
        } else {
            $companyData = $this->_company->toArray();
            unset($companyData["company_id"]);
            $companyData["owner"] = 0;
            if ($this->_write_to_db == 1) {
                $new_company = $targetCompanyDb->createRow($companyData);
                $new_company_id = $new_company->save();
            } else {
                $new_company_id = -1;
            }
            $this->_id_mappings["company"][$this->_company_id] = $new_company_id;
        }
        foreach ($admins as $admin) {
            $user_exists = $targetUserDb->fetchAll(
                $targetUserDb->select()
                    ->where("email = ?", $admin->email)
            )->current();
            if (isset($user_exists)) {
                $this->_logger->notice("** User already exists in target DB. Forname: " . $admin->forename . " - skipping.");
                $this->_skipped["admins"][] = $admin->toArray();
                $this->_id_mappings["admins"][$admin->user_id] = $user_exists->user_id;
                continue;
            }
            $new_admin_data = $admin->toArray();
            unset($new_admin_data["user_id"]);
            /*
             *Migrated user will not have created_by as it created problems with not
             *knowing new user_id's before they had to be set with foreign contraint.
             */
            unset($new_admin_data["created_by"]);
            unset($new_admin_data["password_expire"]); //Datamangager doesn't support this yet.
            /*
             * Force set all users to english
             * REMOVE THIS AS SOON AS WE SUPPORT THE PREVIOUS WEB REPORT LANGUAGES AGAIN
             */
            $new_admin_data["language"] = "en";
            $new_admin_data["company_id"] = $this->_id_mappings["company"][$this->_company_id];
            if ($this->_write_to_db == 1) {
                $new_admin = $targetUserDb->createRow($new_admin_data);
                $new_admin_id = $new_admin->save();
            } else {
                $new_admin_id = -1;
            }
            $this->_id_mappings["admins"][$admin->user_id] = $new_admin_id;
        }
        $this->_logger->info("Migrating company and administrators... Done.");
        return true;
    }

    private function migrateFolders($folders)
    {
        $this->_logger->info("Migrating folders...");
        $this->_results_logger->info("Migrating folders...");
        $cameraGroupDb = new Core_Model_CameraGroups($this->_webreportDB);
        $targetCameraGroupDb = new Core_Model_CameraGroups();
        $targetCompanyCameraGroupDb = new Core_Model_CompanyCameraGroup();
        $targetFolderDb = new Core_Model_Folders();
        $documentFactory = new Core_Model_DocumentFactory();
        $count_added_camera_groups = $count_added_folders = 0;
        $folder_count = 0;
        $nbr_of_folders = count($folders);
        foreach ($folders as $folder) {
            $folder_count++;
            $this->_logger->debug("** Migrating folder no: " . $folder_count . " of " . $nbr_of_folders);
            $newFolderData = $folder->toArray();
            unset($newFolderData["group_id"]);
            $newFolderData["company_id"] = $this->_id_mappings["company"][$folder->company_id];
            if ($folder->parent_id == null) {
                $newFolderData["parent_id"] = null;
            } elseif (is_array($this->_id_mappings["folders"][$folder->parent_id])) {
                $parent_id = 0;
                $max_cameras = -1;
                foreach ($this->_id_mappings["folders"][$folder->parent_id] as $key => $value) {
                    if ($value["nbr_of_cameras"] >= $max_cameras) {
                        $max_cameras = $value["nbr_of_cameras"];
                        $parent_id = $value["id"];
                    }
                }
                $newFolderData["parent_id"] = $parent_id;
            } else {
                $this->_logger->err("**** Parent folder absent for '" . $folder->name .  "'. This should not happen!");
                $this->_errors["parent_folder_absent"][] = $folder->toArray();
                $this->_errors["count"]++;
                continue;
            }

            $camera_group_documents = $folder->getDocuments("camera_group");
            $nbr_of_camera_groups = count($camera_group_documents);
            if ($nbr_of_camera_groups>1) {
                $this->_logger->notice("**** Folder with the name '" . $folder->name . "' holds more than one camera group. A folder will be created for each camera group.
                    Folder created with the camera group containing the highest number of cameras will become parent to previous subfolder.");
            } elseif ($nbr_of_camera_groups==0) {
                $folder_exists = $targetFolderDb->fetchAll(
                    $targetFolderDb->select()
                        ->where("name = ?", $folder->name)
                        ->where("camera_conn_enabled = ?", 0)
                )->current();
                if (isset($folder_exists)) {
                    $this->_logger->debug("**** Folder with the same name without camera connection enabled (no camera group) exists in target DB - skipping.");
                    $this->_skipped["folders"][] = $folder->toArray();
                    $new_folder_id = $folder_exists->group_id;
                } else {
                    $this->_logger->debug("**** Creating a folder with camera connection disabled named '" . $newFolderData["name"] . "'.");
                    $dynamicSalt = CM_User::createSalt();
                    $cameraGroupsData = array(
                        'name' => $folder->name, //Will get same name as folder.
                        'username' => "",
                        'password' => md5(STATIC_SALT . "" . $dynamicSalt),
                        'password_salt' => $dynamicSalt,
                        'description' => $folder->description, //Will get same description as folder.
                    );
                    $newCameraGroup = $targetCameraGroupDb->createRow($cameraGroupsData);
                    $count_added_folders++;
                    if ($this->_write_to_db == 1) {
                        $newFolderData["camera_group_id"] = $newCameraGroup->save();
                        $companyCameraGroupData = array(
                            'company_id' => $this->_id_mappings["company"][$this->_company_id],
                            'camera_group_id' => $newFolderData["camera_group_id"]
                        );
                        $companyCameraGroup = $targetCompanyCameraGroupDb->createRow($companyCameraGroupData);
                        $companyCameraGroup->save();
                        $this->_logger->debug("**** Write folder without camera group");
                        $new_folder = $targetFolderDb->createRow($newFolderData);
                        $new_folder_id = $new_folder->save();
                        $documentFactory->add(
                            $newFolderData["camera_group_id"],
                            "camera_group",
                            $new_folder_id
                        );
                    } else {
                        $new_folder_id = -1; //Dummy data to not trip checks on dry_runs
                        $this->_logger->debug("**** Dry run 'write' folder without camera group");
                    }
                }
                $this->_id_mappings["folders"][$folder->group_id][] = array(
                    "id" => $new_folder_id,
                    "nbr_of_cameras" => -1
                );
            }
            foreach ($camera_group_documents as $camera_group_document) {
                $this->_logger->debug("**** Looking for camera group with id=" . $camera_group_document->id);
                $nbr_of_cameras = 1; //initial value, is set correctly if folder is split (and used to determine "main" camera group.)
                $camera_group = $cameraGroupDb->fetchAll(
                    $cameraGroupDb->select()->where("camera_group_id = ?", $camera_group_document->id)
                )->current();
                $this->_logger->debug("**** Camera group with id " .$camera_group_document->id .
                                      " had username '" . $camera_group->username . "'.");
                $camera_group_exists = $targetCameraGroupDb->fetchAll(
                    $targetCameraGroupDb->select()
                        ->where("username = ?", $camera_group->username)
                )->current();
                if (isset($camera_group_exists)) {
                    $company_conflict = $targetCompanyCameraGroupDb->select()
                        ->where("camera_group_id = ?", $camera_group_exists->camera_group_id)
                        ->where("company_id != ?", $this->_id_mappings["company"][$this->_company_id])
                        ->query()->fetchAll();
                    if ( count($company_conflict) > 0 ) {
                        $this->_logger->warn("**** Camera group '" . $camera_group->name .  "' already exists at a different company. This is bad! This must be handled manually - skipping");
                        $this->_manual_handling["duplicate_camera_group"][] = $camera_group->toArray();
                        $this->_manual_handling["count"]++;
                        continue;
                    } else {
                        $this->_logger->debug("**** Camera group with connection_id/username = " . $camera_group_exists->username . " already exists in target DB with id=" . $camera_group_exists->camera_group_id . " - skipping.");
                        $this->_skipped["camera_groups"][] = $camera_group->toArray();
                        $this->_id_mappings["camera_groups"][$camera_group->camera_group_id] = $camera_group_exists->camera_group_id;
                    }
                } else {
                    $cameras = $camera_group->getCameras();
                    $nbr_of_cameras = count($cameras);
                    $newCameraGroupData = $camera_group->toArray();
                    unset($newCameraGroupData["camera_group_id"]);

                    $count_added_camera_groups++;
                    if ($this->_write_to_db == 1) {
                        $this->_logger->debug("**** Writing camera group.");
                        $new_camera_group = $targetCameraGroupDb->createRow($newCameraGroupData);
                        $new_cg_id = $new_camera_group->save();
                        $companyCameraGroupData = array(
                            'company_id' => $this->_id_mappings["company"][$this->_company_id],
                            'camera_group_id' => $new_cg_id
                        );
                        $companyCameraGroup = $targetCompanyCameraGroupDb->createRow($companyCameraGroupData);
                        $companyCameraGroup->save();
                        $this->_id_mappings["camera_groups"][$camera_group->camera_group_id] = $new_cg_id;
                    } else {
                        //Dummy data to let script continue dry run.
                        $this->_id_mappings["camera_groups"][$camera_group->camera_group_id] = -1;
                    }
                }
                if ($nbr_of_camera_groups>1) {
                    $new_folder_name = $camera_group->name;
                } else {
                    $new_folder_name = $folder->name;
                }
                $this->_logger->debug("**** Creating a folder with camera group named '" . $new_folder_name . "'.");
                $folder_exists = $targetFolderDb->fetchAll(
                    $targetFolderDb->select()
                        ->where("name = ?", $new_folder_name)
                        ->where("camera_group_id = ?", $this->_id_mappings["camera_groups"][$camera_group->camera_group_id])
                )->current();
                if (isset($folder_exists)) {
                    $this->_logger->debug("**** Folder with the name '"
                        . $new_folder_name . "'' and camera_group="
                        . $folder_exists->camera_group_id . " already exists in target DB - skipping.");
                    $this->_skipped["folders"][] = $folder->toArray();
                    $new_folder_id = $folder_exists->group_id;
                } else {
                    $newFolderData["camera_group_id"] = $this->_id_mappings["camera_groups"][$camera_group->camera_group_id];
                    $newFolderData["camera_conn_enabled"] = 1;
                    $newFolderData["name"] = $new_folder_name;
                    $count_added_folders++;
                    if ($this->_write_to_db == 1) {
                        $this->_logger->debug("**** Write folder with camera group");
                        $new_folder = $targetFolderDb->createRow($newFolderData);
                        $new_folder_id = $new_folder->save();
                        $documentFactory->add(
                            $this->_id_mappings["camera_groups"][$camera_group->camera_group_id],
                            "camera_group",
                            $new_folder_id
                        );
                    } else {
                        $this->_logger->debug("**** Dry run 'write' folder with camera group");
                        $new_folder_id = -1; //Dummy data for dry run.
                    }
                }
                $this->_id_mappings["folders"][$folder->group_id][] = array(
                            "id" => $new_folder_id,
                            "nbr_of_cameras" => $nbr_of_cameras
                            );
            }
        }
        $this->_logger->notice("Added " . $count_added_camera_groups . " camera groups to target database.");
        $this->_logger->notice("Skipped " . count($this->_skipped["camera_groups"]) . " camera groups due to already being in target database.");
        $this->_logger->notice("Added " . $count_added_folders . " folders to target database.");
        $this->_logger->notice("Skipped " . count($this->_skipped["folders"]) . " folders due to already being in target database.");
        $this->_logger->info("Migrating folders... Done.");

        return true;
    }

    private function migrateCameras($cameras)
    {
        $this->_logger->info("Migrating cameras (this might take minutes or hours depening on number of cameras and timespan of data)");
        $this->_results_logger->info("Migrating cameras (this might take minutes or hours depening on number of cameras and timespan of data)");
        $targetCameraDb = new Core_Model_Cameras();
        $targetCameraGroupDb = new Core_Model_CameraGroups();
        $sourceCameraDb = new Core_Model_Cameras($this->_webreportDB);
        $applicationDB = new Core_Model_Applications();
        $count_added_cameras = $count_updated_data = 0;
        $camera_count = 0;
        $nbr_of_cameras = count($cameras);
        foreach ($cameras as $camera) {
            $countData = new Core_Model_CountData($this->_webreportDB);
            $camera_count++;
            $this->_logger->info("Migrating camera " . $camera_count . " of " . $nbr_of_cameras . ".");
            $camera_row = $sourceCameraDb->fetchAll(
                $sourceCameraDb->select()
                    ->where("camera_id = ?", $camera["camera_id"])
            )->current();
            if (!isset($this->_id_mappings["camera_groups"][$camera_row->camera_group_id])) {
                $this->_logger->crit("Skipping " . $camera_row->name . ". No map to camera group found. " .
                    "Could be caused by folder connection identifier collision.");
                $this->_skipped["cameras"][] = $camera_row->toArray();
                continue;
            }
            $newCameraData = $camera_row->toArray();
            unset($newCameraData["camera_id"]);
            // More robust if first_date and last_date is set from data import.
            unset($newCameraData["first_date"]);
            unset($newCameraData["last_date"]);
            $camera_exists = $targetCameraDb->fetchAll(
                $targetCameraDb->select()
                ->where("serial = ?", $camera_row->serial)
                ->where("name = ?", $camera_row->name)
                ->where("camera_group_id = ?", $this->_id_mappings["camera_groups"][$camera_row->camera_group_id])
            )->current();
            $first_date = new Zend_Date($camera_row->first_date, 'YYYY-MM-DD HH:mm:ss');
            $last_date = new Zend_Date($camera_row->last_date, 'YYYY-MM-DD HH:mm:ss');
            $data_written = $fake_write = $skip_data = $skip_camera = false;
            if ($camera_exists) {
                $skip_camera = true;
                $this->_logger->debug("**** Camera (". $camera_row->name .") with the serial '" . $camera_row->serial . "'' already exists in target DB");
                $last_date_existing = new Zend_Date($camera_exists->last_date, 'YYYY-MM-DD HH:mm:ss');
                if ($last_date_existing->getTimestamp() < $last_date->getTimestamp()) {
                    $this->_logger->debug("**** It has new data. Will update");
                    $this->_updated_cameras[] = $camera_row->toArray();
                    $count_updated_data++;
                    $first_date = $last_date_existing;
                } else {
                    $this->_logger->debug("**** It does not have new data - skipping.");
                    $this->_skipped["cameras"][] = $camera_row->toArray();
                    $skip_data = true;
                }
                $new_camera_id = $camera_exists->camera_id;
            }
            $newCameraData["camera_group_id"] = $this->_id_mappings["camera_groups"][$camera_row->camera_group_id];
            if (!$skip_camera) {
                $count_added_cameras++;
                if ($this->_write_to_db == 1) {
                    $new_camera = $targetCameraDb->createRow($newCameraData);
                    $new_camera_id = $new_camera->save();
                    $this->_logger->debug("**** Camera written");
                } else {
                    $this->_logger->debug("**** Dry run 'write' camera");
                    $new_camera_id = -1; //Dummy data for dry run.
                }
            }
            $this->_logger->debug("**** Camera (". $camera_row->name .") with previous id " . $camera_row->camera_id . " processed. New id: ". $new_camera_id);
            $this->_id_mappings["cameras"][$camera_row->camera_id] = $new_camera_id;
            if (!$skip_data) {
                $this->_logger->debug("**** Starting to write camera data");
                $targetDbTemp = Zend_Registry::get('db');
                //Change default DB and registry 'db' to source db temporarily for countData class to work.
                Zend_Registry::set('db', $this->_webreportDB);
                Zend_Db_Table_Abstract::setDefaultAdapter($this->_webreportDB);
                $countData->addCamera($camera_row);
                $countData->initDatabaseSelect(true);
                $this->_logger->debug("**** Getting camera data from: ". $first_date->toString(). " to: " . $last_date->toString());
                $cameraData = $countData->getData("ts15min", $first_date, $last_date);
                Zend_Db_Table_Abstract::setDefaultAdapter($targetDbTemp);
                Zend_Registry::set('db', $targetDbTemp);
                $camera_group = $targetCameraGroupDb->fetchAll(
                    $targetCameraGroupDb->select()
                        ->where("camera_group_id = ?", $this->_id_mappings["camera_groups"][$camera_row->camera_group_id])
                )->current();
                $app = $applicationDB->findWithId($camera_row->application_id);
                $id = $camera_row->camera_id;
                $map = $app->getCntMapping();
                if ($map == null || count($map) == 0) {
                    $this->_logger->warn("**** Camera must be handled manually - unsupported application by Migration");
                    $this->_manual_handling["unsupported_application"][] = $camera_row->toArray();
                    $this->_manual_handling["count"]++;
                    continue;
                }
                $stats = array();
                foreach ($cameraData as $cd) {
                    foreach ($map as $value_label => $index) {
                        if (isset($cd["c$id" . "_" . $value_label])) {
                            $stats[$index] = $cd["c$id" . "_" . $value_label];
                        } else {
                            $this->_logger->error("**** Expected field not found. Cannot build new indata.");
                            $this->_errors["field_missing"][] = array("camera" => $camera_row->toArray(), "field" => $value_label);
                            return false;
                        }
                    }
                    $data = (object) array(
                        "username" => null,
                        "password" => null,
                        "version" => '3', //Doesn't matter now as long as it's >=3.
                        "cameraName" => $camera_row->name,
                        "serial" => $camera_row->serial,
                        "regcode" => $camera_row->regcode,
                        "fields" => "10", //Doesn't matter now as long as it's >=10.
                        "stats" => $stats,
                        "nbrOfTypes" => count($map),
                        "cnttime" => $cd['cnttime']
                    );
                    if ($this->_write_to_db == 1) {
                        $countData->addDataForSeparatedString($data, null, true, $camera_group, true, !$this->_ignore_zeros);
                        $data_written = true;
                    } else {
                        $fake_write = true;
                    }
                }
                if (!isset($new_camera) && $this->_write_to_db) { //If not write_to_db==1 getCamera will fail stopping script.
                    $new_camera = $targetCameraDb->getCamera($this->_id_mappings["cameras"][$camera_row->camera_id]);
                }
                if ($this->_write_to_db) {
                    $new_camera->last_date = $camera_row->last_date;
                    $new_camera->save();
                }
            }
            if ($data_written) {
                $this->_logger->debug("**** Wrote camera data to DB");
            } elseif ($fake_write) {
                $this->_logger->debug("**** Dry run 'wrote' camera data to DB");
            }
        }
        $this->_logger->notice("Added " . $count_added_cameras . " cameras to target database.");
        $this->_logger->notice("Updated data to " . $count_updated_data . " cameras already existing in target database");
        $this->_logger->notice("Skipped " . count($this->_skipped["cameras"]) . " cameras due to already being in target database.");
        $this->_logger->info("Migrating cameras... Done.");
        return true;
    }

    private function migratePermaLinks($permalinks)
    {
        $this->_logger->info("Migrating permalinks...");
        $this->_results_logger->info("Migrating permalinks...");
        $this->migrateChartPermaLinks($permalinks['charts']);
        $this->migrateReportPermaLinks($permalinks['reports']);
        $this->_logger->info("Migrating permalinks - Done.");
        return true;
    }

    private function migrateChartPermaLinks($chart_permalinks)
    {
        $targetChartPermalinkDb = new Charts_Model_Permalinks();
        $targetChartDataDb = new Charts_Model_ChartDataList();
        $targetChartDataSourceDb = new Charts_Model_ChartDataSourceList();
        $targetChartDataSourceCameraDb = new Charts_Model_ChartDataSourceCameraList();
        $chartDataSourceDb = new Charts_Model_ChartDataSourceList($this->_webreportDB);
        $chartDataSourceCameraDb = new Charts_Model_ChartDataSourceCameraList($this->_webreportDB);
        $documentDb = new Core_Model_Documents($this->_webreportDB);
        $documentFactory = new Core_Model_DocumentFactory();
        $chart_permalink_count=0;
        foreach ($chart_permalinks as $permalink) {
            $chart_permalink_count++;
            $this->_logger->info("Migrating chart/table permalink " . $chart_permalink_count . " of " . count($chart_permalinks) . ".");
            if (isset($permalink['data']->chart_data_id)) {
                $this->_logger->debug("***** Beginning migration of permalink prerequisites (permalink based on chart)");
                $id_mapping_group_for_permalink = 'chart_data';
                //ChartData
                $chartData = $permalink['data'];
                $document = $documentDb->fetchAll(
                    $documentDb->select()
                        ->where('type = ?', 'chart')
                        ->where('id = ?', $chartData->chart_data_id)
                )->current();
                $keys_to_ignore = array('chart_data_id', 'user_id');
                $values_to_replace = array(
                    "company_id" => $this->_id_mappings["company"][$chartData->company_id]
                );
                $select = $this->createSelectWithoutNulls($chartData, $targetChartDataDb, $keys_to_ignore, $values_to_replace);
                $chartData_exists = $targetChartDataDb->fetchAll($select)->current();
                if (isset($chartData_exists)) {
                    $this->_logger->debug("****** Chart data for current permalink already exists in target DB - skipping.");
                    $new_chart_data_id = $chartData_exists->chart_data_id;
                } else {
                    $newChartDataData = $this->createNewData($chartData, $keys_to_ignore, $values_to_replace);
                    if ($this->_write_to_db == 1) {
                        $this->_logger->debug("****** Write permalink prerequisite (Chart data)");
                        $new_chart_data = $targetChartDataDb->createRow($newChartDataData);
                        $new_chart_data_id = $new_chart_data->save();
                    } else {
                        $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Chart data)");
                        $new_chart_data_id = -1; //Dummy data for dry run.
                    }
                }
                $this->_id_mappings["chart_data"][$chartData->chart_data_id] = $new_chart_data_id;

                //ChartDataSources
                $chartDataSources = $chartDataSourceDb->fetchAll(
                    $chartDataSourceDb->select()
                        ->where("chart_data_id = ?", $chartData->chart_data_id)
                );
                foreach ($chartDataSources as $chartDataSource) {
                    $this->_logger->debug("****** Add a chart data source");
                    $keys_to_ignore = array('chart_data_source_id');
                    $values_to_replace = array(
                        "chart_data_id" => $new_chart_data_id
                    );
                    $select = $this->createSelectWithoutNulls($chartDataSource, $targetChartDataSourceDb, $keys_to_ignore, $values_to_replace);
                    $chartDataSource_exists = $targetChartDataSourceDb->fetchAll($select)->current();
                    if (isset($chartDataSource_exists)) {
                        $this->_logger->debug("****** Chart data source for current permalink already exists in target DB - skipping.");
                        $new_chart_data_source_id = $chartDataSource_exists->chart_data_source_id;
                    } else {
                        $newChartDataSourceData = $this->createNewData($chartDataSource, $keys_to_ignore, $values_to_replace);
                        if ($this->_write_to_db == 1) {
                            $this->_logger->debug("****** Write permalink prerequisite (Chart data source)");
                            $new_chart_data_source = $targetChartDataSourceDb->createRow($newChartDataSourceData);
                            $new_chart_data_source_id = $new_chart_data_source->save();
                        } else {
                            $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Chart data source)");
                            $new_chart_data_source_id = -1; //Dummy data for dry run.
                        }
                    }
                    $this->_id_mappings["chart_data_source"][$chartDataSource->chart_data_source_id] = $new_chart_data_source_id;

                    $chartDataSourceCameras = $chartDataSourceCameraDb->fetchAll(
                        $chartDataSourceCameraDb->select()
                            ->where("chart_data_source_id = ?", $chartDataSource->chart_data_source_id)
                    );
                    foreach ($chartDataSourceCameras as $chartDataSourceCamera) {
                        $this->_logger->debug("**** Adding another data source.");
                        $additional_id_that_matters = $additional_id_mapping_group = $id_that_matters = $id_mapping_group = null;
                        if (!is_null($chartDataSourceCamera->camera_id)) {
                            $id_that_matters = "camera_id";
                            $id_mapping_group = "cameras";
                        } elseif (!is_null($chartDataSourceCamera->virtual_camera_id)) {
                            $id_that_matters = "virtual_camera_id";
                            $id_mapping_group = "virtual_cameras";
                            $this->migrateVirtualCamera($chartDataSourceCamera->virtual_camera_id);
                        } elseif (!is_null($chartDataSourceCamera->count_data_set_id)) {
                            $id_that_matters = "count_data_set_id";
                            $id_mapping_group = "count_data_sets";
                            $this->migrateCountDataSet($chartDataSourceCamera->count_data_set_id);
                        } elseif (!is_null($chartDataSourceCamera->count_data_formula_set_id)) {
                            $id_that_matters = "count_data_formula_set_id";
                            $id_mapping_group = "formula_sets";
                            $additional_id_that_matters = "type";
                            $additional_id_mapping_group = "formula_set_formulas";
                            $this->migrateFormulaSet($chartDataSourceCamera->count_data_formula_set_id);
                        } elseif (!is_null($chartDataSourceCamera->queue_id)) {
                            $id_that_matters = "queue_id";
                            $id_mapping_group = "cameras";
                        } elseif (!is_null($chartDataSourceCamera->parking_id)) {
                            $id_that_matters = "parking_id";
                            $id_mapping_group = "cameras";
                        } elseif (!is_null($chartDataSourceCamera->occupancy_id)) {
                            $id_that_matters = "occupancy_id";
                            $id_mapping_group = "cameras";
                        } elseif (!is_null($chartDataSourceCamera->pos_id)) {
                            $this->_logger->warn("**** Permalink based on pos data found. It's data source cannot be migrated but it already has been written to target DB! - This permalink must be manually deleted on migration completion");
                            $this->_manual_handling["permalink_based_on_pos"][] = $permalink;
                            $this->_manual_handling["count"]++;
                            continue;
                        } else {
                            r_log("No non null column!");
                            throw new Exception("No non null column in chartDataSourceCamera!");
                        }
                        $keys_to_ignore = array('chart_data_source_camera_id');
                        $values_to_replace = array(
                            $id_that_matters => $this->_id_mappings[$id_mapping_group][$chartDataSourceCamera->$id_that_matters],
                            "chart_data_source_id" => $new_chart_data_source_id
                        );
                        if (isset($additional_id_that_matters)) {
                            $values_to_replace[$additional_id_that_matters] = $this->_id_mappings[$additional_id_mapping_group][$chartDataSourceCamera->$additional_id_that_matters];
                        }
                        $select = $this->createSelectWithoutNulls($chartDataSourceCamera, $targetChartDataSourceCameraDb, $keys_to_ignore, $values_to_replace);
                        $chartDataSourceCamera_exists = $targetChartDataSourceCameraDb->fetchAll($select)->current();
                        if (isset($chartDataSourceCamera_exists)) {
                            $this->_logger->debug("****** Chart data source camera for current permalink already exists in target DB - skipping.");
                            $new_chart_data_source_camera_id = $chartDataSourceCamera_exists->chart_data_source_camera_id;
                        } else {
                            $newChartDataSourceCameraData = $this->createNewData($chartDataSourceCamera, $keys_to_ignore, $values_to_replace);
                            if ($this->_write_to_db == 1) {
                                $this->_logger->debug("****** Write permalink prerequisite (Chart data source camera)");
                                $new_chart_data_source_camera = $targetChartDataSourceCameraDb->createRow($newChartDataSourceCameraData);
                                $new_chart_data_source_camera_id = $new_chart_data_source_camera->save();
                            } else {
                                $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Chart data source camera)");
                                $new_chart_data_source_camera_id = -1; //Dummy data for dry run.
                            }
                        }
                        $this->_id_mappings["chart_data_source_camera"][$chartDataSourceCamera->chart_data_source_camera_id] = $new_chart_data_source_camera_id;
                    }
                }
            } else {
                $targetTableDb = new Charts_Model_Tables();
                $targetTableSectionDb = new Charts_Model_TableSections();
                $targetTableSectionEntryDb = new Charts_Model_TableSectionEntries();
                $id_mapping_group_for_permalink = 'tables';
                $this->_logger->debug("***** Beginning migration of permalink prerequisites (permalink based on table)");
                //Table
                $table = $permalink['data'];
                $document = $documentDb->fetchAll(
                    $documentDb->select()
                        ->where('type = ?', 'table')
                        ->where('id = ?', $table->table_id)
                )->current();
                $keys_to_ignore = array('table_id');
                $select = $this->createSelectWithoutNulls($table, $targetTableDb, $keys_to_ignore);
                $table_exists = $targetTableDb->fetchAll($select)->current();
                if (isset($table_exists)) {
                    $this->_logger->debug("****** Table for current permalink already exists in target DB - skipping.");
                    $new_table_id = $table_exists->table_id;
                } else {
                    $newTableData = $this->createNewData($table, $keys_to_ignore);
                    if ($this->_write_to_db == 1) {
                        $this->_logger->debug("****** Write permalink prerequisite (Table)");
                        $new_table = $targetTableDb->createRow($newTableData);
                        $new_table_id = $new_table->save();
                    } else {
                        $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Table)");
                        $new_table_id = -1; //Dummy data for dry run.
                    }
                }
                $this->_id_mappings["tables"][$table->table_id] = $new_table_id;


                //TableSection

                //Charts_Model_TableSections wont work without switching default db.
                $targetDbTemp = Zend_Registry::get('db');
                Zend_Registry::set('db', $this->_webreportDB);
                Zend_Db_Table_Abstract::setDefaultAdapter($this->_webreportDB);
                $tableSectionDb = new Charts_Model_TableSections();
                $tableSections = $tableSectionDb->fetchAll(
                    $tableSectionDb->select()
                       ->where("table_id = ?", $table->table_id)
                );
                Zend_Db_Table_Abstract::setDefaultAdapter($targetDbTemp);
                Zend_Registry::set('db', $targetDbTemp);
                foreach ($tableSections as $tableSection) {
                    $keys_to_ignore = array('table_section_id');
                    $values_to_replace = array('table_id' => $new_table_id);
                    $select = $this->createSelectWithoutNulls($tableSection, $targetTableSectionDb, $keys_to_ignore, $values_to_replace);
                    $tableSection_exists = $targetTableSectionDb->fetchAll($select)->current();
                    if (isset($tableSection_exists)) {
                        $this->_logger->debug("****** Table section for current permalink already exists in target DB - skipping.");
                        $new_table_section_id = $tableSection_exists->table_section_id;
                    } else {
                        $newTableSectionData = $this->createNewData($tableSection, $keys_to_ignore, $values_to_replace);
                        if ($this->_write_to_db == 1) {
                            $this->_logger->debug("****** Write permalink prerequisite (Table section)");
                            $new_table_section = $targetTableSectionDb->createRow($newTableSectionData);
                            $new_table_section_id = $new_table_section->save();
                        } else {
                            $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Table section)");
                            $new_table_section_id = -1; //Dummy data for dry run.
                        }
                    }
                    $this->_id_mappings["table_sections"][$tableSection->table_section_id] = $new_table_section_id;

                    //Charts_Model_TableSectionEntries wont work without switching default db.
                    $targetDbTemp = Zend_Registry::get('db');
                    Zend_Registry::set('db', $this->_webreportDB);
                    Zend_Db_Table_Abstract::setDefaultAdapter($this->_webreportDB);
                    $tableSectionEntryDb = new Charts_Model_TableSectionEntries();
                    $tableSectionEntries = $tableSectionEntryDb->fetchAll(
                        $tableSectionEntryDb->select()
                            ->where("table_section_id = ?", $tableSection->table_section_id)
                    );
                    Zend_Db_Table_Abstract::setDefaultAdapter($targetDbTemp);
                    Zend_Registry::set('db', $targetDbTemp);
                    foreach ($tableSectionEntries as $tableSectionEntry) {
                        $keys_to_ignore = array('table_section_entry_id');
                        $values_to_replace = array('table_section_id' => $new_table_section_id);
                        $select = $this->createSelectWithoutNulls($tableSectionEntry, $targetTableSectionEntryDb, $keys_to_ignore, $values_to_replace);
                        $tableSectionEntry_exists = $targetTableSectionEntryDb->fetchAll($select)->current();
                        if (isset($tableSectionEntry_exists)) {
                            $this->_logger->debug("****** Chart table section entry for current permalink already exists in target DB - skipping.");
                            $new_table_section_entry_id = $tableSectionEntry_exists->table_section_entry_id;
                        } else {
                            $newTableSectionEntryData = $this->createNewData($tableSectionEntry, $keys_to_ignore, $values_to_replace);

                            if ($this->_write_to_db == 1) {
                                $this->_logger->debug("****** Write permalink prerequisite (Table section entry)");
                                $new_table_section_entry = $targetTableSectionEntryDb->createRow($newTableSectionEntryData);
                                $new_table_section_entry_id = $new_table_section_entry->save();
                            } else {
                                $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Table section entry)");
                                $new_table_section_entry_id = -1; //Dummy data for dry run.
                            }
                        }
                        $this->_id_mappings["table_section_entries"][$tableSectionEntry->table_section_entry_id] = $new_table_section_entry_id;
                    }
                }
            }
            $documentData = array(
                $this->_id_mappings[$id_mapping_group_for_permalink][$document->id],
                $document->type,
                $this->_id_mappings['folders'][$document->group_id][0]['id']
            );
            $document_exists = $documentFactory->fetch($documentData[0], $documentData[1]);
            if (!$document_exists) {
                if ($this->_write_to_db == 1) {
                    $this->_logger->debug("****** Adding document: " . print_r($documentData, true));
                    $documentFactory->add(
                        $documentData[0],
                        $documentData[1],
                        $documentData[2]
                    );
                } else {
                    $this->_logger->debug("****** Dry run, should have added document: " . print_r($documentData, true));
                }
            }
            $permalinkData = $permalink['permalink'];
            $keys_to_ignore = array('permalink_id');
            $values_to_replace = array(
                'legacy_id' => $permalinkData->permalink_id,
                'chart_id' => $this->_id_mappings[$id_mapping_group_for_permalink][$permalinkData->chart_id]
            );
            $select = $this->createSelectWithoutNulls($permalinkData, $targetChartPermalinkDb, $keys_to_ignore, $values_to_replace);
            $permalink_exists = $targetChartPermalinkDb->fetchAll($select)->current();
            if (isset($permalink_exists)) {
                $this->_logger->debug("**** Chart permalink already exists in target DB - skipping.");
                $this->_skipped["permalinks"][] = $permalinkData->toArray();
                $new_permalink_id = $permalink_exists->permalink_id;
            } else {
                $newPermalinkData = $this->createNewData($permalinkData, $keys_to_ignore, $values_to_replace);
                if ($this->_write_to_db == 1) {
                    $this->_logger->debug("**** Write chart permalink");
                    $new_permalink = $targetChartPermalinkDb->createRow($newPermalinkData);
                    $new_permalink_id = $new_permalink->save();
                } else {
                    $this->_logger->debug("**** Dry run 'write' chart permalink");
                    $new_permalink_id = -1; //Dummy data for dry run.
                }
            }
            $this->_id_mappings["chart_permalinks"][$permalinkData->permalink_id] = $new_permalink_id;
        }
    }

    private function modifyValueBasedOnKey($key, $value)
    {
        if ($key == "formula") {
            preg_match('~c(.*?)_~', $value, $output);
            if (isset($output[1])) {
                $old_id = (int) $output[1];
                $replacement_id = $this->_id_mappings['cameras'][$old_id];
                $value = str_replace($old_id, $replacement_id, $value);
            }
        }
        if ($key == "selections_json") {
            $selections = json_decode($value);
            foreach ($selections as &$selection) {
                if ($selection[4] == "CDS") {
                    if (!isset($this->_id_mappings['count_data_sets'][$selection[1]])) {
                        $this->migrateCountDataSet($selection[1]);
                    }
                    $selection[1] = $this->_id_mappings['count_data_sets'][$selection[1]];
                } elseif ($selection[4] == "PC" || $selection[4] == "Q" || $selection[4] == "OCC" || $selection[4] == "P") {
                    $selection[1] = $this->_id_mappings['cameras'][$selection[1]];
                } elseif ($selection[4] == "F") {
                    if (!isset($this->_id_mappings['formula_sets'][$selection[1]])) {
                        $this->migrateFormulaSet($selection[1]);
                    }
                    $selection[1] = $this->_id_mappings['formula_sets'][$selection[1]];
                    $selection[2] = $this->_id_mappings['formula_set_formulas'][$selection[2]];
                } elseif ($selection[4] == "VC") {
                    if (!isset($this->_id_mappings['virtual_cameras'][$selection[1]])) {
                        $this->migrateVirtualCamera($selection[1]);
                    }
                    $selection[1] = $this->_id_mappings['virtual_cameras'][$selection[1]];
                } else {
                    r_log("Unknown table section entry type, must abort.");
                    throw new Exception("Unknown table section entry type, cannot set correct new id.");
                }
            }
            $value = json_encode($selections);
        }
        return $value;
    }

    private function createNewData($data_row, array $keys_to_ignore = null, array $values_to_replace = null)
    {
        $return_array = $data_row->toArray();
        if (isset($keys_to_ignore)) {
            foreach ($keys_to_ignore as $key) {
                unset($return_array[$key]);
            }
        }
        if (isset($values_to_replace)) {
            foreach ($values_to_replace as $key => $value) {
                $return_array[$key] = $value;
            }
        }
        foreach ($return_array as $key => $value) {
            $return_array[$key] = $this->modifyValueBasedOnKey($key, $value);
            if (is_object($value)) { //För trubbigt?
                $return_array[$key] = json_encode($value);
            }
        }
        return $return_array;
    }

    private function createSelectWithoutNulls($row, $db_table, array $key_ignores = null, array $values_to_replace = null)
    {
        $select = $db_table->select();
        $row_array = $row->toArray();
        foreach ($row_array as $key => $value) {
            if (is_object($value)) {
                $value = json_encode($value);
            }
            if (!is_null($value) && !in_array($key, $key_ignores)) {
                $value = $this->modifyValueBasedOnKey($key, $value);
                if (isset($values_to_replace[$key])) {
                    $value = $values_to_replace[$key];
                }
                $select->where($key . ' = ?', $value);
            }
        }
        return $select;

    }
    private function migrateVirtualCamera($virtualCameraId)
    {
        $targetVirtualCameraDb = new Core_Model_VirtualCameras();
        $targetVirtualCameraFormulaDb = new Core_Model_VirtualCameraFormula();
        $virtualCameraFormulaDb = new Core_Model_VirtualCameraFormula($this->_webreportDB);
        $virtualCameraDb = new Core_Model_VirtualCameras($this->_webreportDB);
        $virtualCamera = $virtualCameraDb->fetchAll(
            $virtualCameraDb->select()
                ->where('virtual_camera_id = ?', $virtualCameraId)
        )->current();
        $keys_to_ignore = array('virtual_camera_id','created_by');
        $values_to_replace = array(
            'company_id' => $this->_id_mappings["company"][$virtualCamera->company_id],
        );
        $select = $this->createSelectWithoutNulls($virtualCamera, $targetVirtualCameraDb, $keys_to_ignore, $values_to_replace);
        $virtualCamera_exists = $targetVirtualCameraDb->fetchAll($select)->current();
        if (isset($virtualCamera_exists)) {
            $this->_logger->debug("****** Virtual camera for current permalink already exists in target DB - skipping.");
            $new_virtual_camera_id = $virtualCamera_exists->virtual_camera_id;
        } else {
            $newVirtualCameraData = $this->createNewData($virtualCamera, $keys_to_ignore, $values_to_replace);
            if ($this->_write_to_db == 1) {
                $this->_logger->debug("****** Write virtual camera");
                $new_virtual_camera = $targetVirtualCameraDb->createRow($newVirtualCameraData);
                $new_virtual_camera_id = $new_virtual_camera->save();
            } else {
                $this->_logger->debug("****** Dry run 'write' virtual camera");
                $new_virtual_camera_id = -1; //Dummy data for dry run.
            }
        }
        $this->_id_mappings["virtual_cameras"][$virtualCamera->virtual_camera_id] = $new_virtual_camera_id;
        $virtualCameraFormulas = $virtualCameraFormulaDb->fetchAll(
            $virtualCameraFormulaDb->select()
                ->where("virtual_camera_id = ?", $new_virtual_camera_id)
        );
        foreach ($virtualCameraFormulas as $virtualCameraFormula) {
            $keys_to_ignore = array('virtual_camera_formula_id');
            $values_to_replace = array(
                'virtual_camera_id' => $this->_id_mappings["virtual_cameras"][$virtualCameraFormula->virtual_camera_id],
            );
            $select = $this->createSelectWithoutNulls($virtualCameraFormula, $targetVirtualCameraFormulaDb, $keys_to_ignore, $values_to_replace);
            $virtualCameraFormula_exists = $targetVirtualCameraFormulaDb->fetchAll($select)->current();
            if (isset($virtualCameraFormula_exists)) {
                $this->_logger->debug("****** Formula set formula for current permalink already exists in target DB - skipping.");
                $new_virtual_camera_formula_id = $virtualCameraFormula_exists->virtual_camera_formula_id;
            } else {
                $newFormulaSetData = $this->createNewData($virtualCameraFormula, $keys_to_ignore, $values_to_replace);
                if ($this->_write_to_db == 1) {
                    $this->_logger->debug("****** Write virtual camera formula");
                    $new_virtual_camera_formula = $targetVirtualCameraFormulaDb->createRow($newFormulaSetData);
                    $new_virtual_camera_formula_id = $new_virtual_camera_formula->save();
                } else {
                    $this->_logger->debug("****** Dry run 'write' virtual camera formula");
                    $new_virtual_camera_formula_id = -1; //Dummy data for dry run.
                }
            }
            $this->_id_mappings["virtual_camera_formulas"][$virtualCameraFormula->virtual_camera_id] = $new_virtual_camera_formula_id;
        }
        return $new_virtual_camera_id;
    }

    private function migrateFormulaSet($formulaSetId)
    {
        $targetFormulaSetDb = new Core_Model_CountData_FormulaSets();
        $targetFormulaSetFormulaDb = new Core_Model_CountData_FormulaSetFormulas();
        $formulaSetDb = new Core_Model_CountData_FormulaSets($this->_webreportDB);
        $formulaSetFormulaDb = new Core_Model_CountData_FormulaSetFormulas($this->_webreportDB);
        $formulaSet = $formulaSetDb->fetchAll(
            $formulaSetDb->select()
                ->where('count_data_formula_set_id = ?', $formulaSetId)
        )->current();
        $keys_to_ignore = array('count_data_formula_set_id','created_by');
        $values_to_replace = array(
            'company_id' => $this->_id_mappings["company"][$formulaSet->company_id],
        );
        $select = $this->createSelectWithoutNulls($formulaSet, $targetFormulaSetDb, $keys_to_ignore, $values_to_replace);
        $formulaSet_exists = $targetFormulaSetDb->fetchAll($select)->current();
        if (isset($formulaSet_exists)) {
            $this->_logger->debug("****** Formula set for current permalink already exists in target DB - skipping.");
            $new_count_data_formula_set_id = $formulaSet_exists->count_data_formula_set_id;
        } else {
            $newFormulaSetData = $this->createNewData($formulaSet, $keys_to_ignore, $values_to_replace);
            if ($this->_write_to_db == 1) {
                $this->_logger->debug("****** Write formula set");
                $new_formula_set = $targetFormulaSetDb->createRow($newFormulaSetData);
                $new_count_data_formula_set_id = $new_formula_set->save();
            } else {
                $this->_logger->debug("****** Dry run 'write' formula set");
                $new_count_data_formula_set_id = -1; //Dummy data for dry run.
            }
        }
        $this->_id_mappings["formula_sets"][$formulaSet->count_data_formula_set_id] = $new_count_data_formula_set_id;
        //FormulaSet Formulas
        $formulaSetFormulas = $formulaSetFormulaDb->fetchAll(
            $formulaSetFormulaDb->select()
                ->where("count_data_formula_set_id = ?", $formulaSet->count_data_formula_set_id)
        );
        foreach ($formulaSetFormulas as $formulaSetFormula) {
            $keys_to_ignore = array('count_data_formula_set_formula_id');
            $values_to_replace = array(
                'count_data_formula_set_id' => $this->_id_mappings["formula_sets"][$formulaSetFormula->count_data_formula_set_id],
            );
            $select = $this->createSelectWithoutNulls($formulaSetFormula, $targetFormulaSetFormulaDb, $keys_to_ignore, $values_to_replace);
            $formulaSetFormula_exists = $targetFormulaSetFormulaDb->fetchAll($select)->current();
            if (isset($formulaSetFormula_exists)) {
                $this->_logger->debug("****** Formula set formula for current permalink already exists in target DB - skipping.");
                $new_count_data_formula_set_formula_id = $formulaSetFormula_exists->count_data_formula_set_formula_id;
            } else {
                $newFormulaSetData = $this->createNewData($formulaSetFormula, $keys_to_ignore, $values_to_replace);
                if ($this->_write_to_db == 1) {
                    $this->_logger->debug("****** Write formula set formula");
                    $new_formula_set_formula = $targetFormulaSetFormulaDb->createRow($newFormulaSetData);
                    $new_count_data_formula_set_formula_id = $new_formula_set_formula->save();
                } else {
                    $this->_logger->debug("****** Dry run 'write' formula set formula");
                    $new_count_data_formula_set_formula_id = -1; //Dummy data for dry run.
                }
            }
            $this->_id_mappings["formula_set_formulas"][$formulaSetFormula->count_data_formula_set_formula_id] = $new_count_data_formula_set_formula_id;
        }
        return $new_count_data_formula_set_id;
    }

    //Migrates a specific countDataSet from source db to target db and returns the new id.
    private function migrateCountDataSet($countDataSetId)
    {
        $targetCountDataSetDb = new Core_Model_CountData_Sets();
        $targetCountDataSetsCamerasDb = new Core_Model_CountData_Set_Cameras();
        $countDataSetDb = new Core_Model_CountData_Sets($this->_webreportDB);
        $countDataSetCamerasDb = new Core_Model_CountData_Set_Cameras($this->_webreportDB);
        $countDataSet = $countDataSetDb->fetchAll(
            $countDataSetDb->select()
                ->where("count_data_set_id = ?", $countDataSetId)
        )->current();
        if (!isset($countDataSet)) {
            return false;
        }
        if ($countDataSet->company_id != $this->_company_id) {
            $this->_logger->notice("******* Company had permalink that refers to CountDataSet owned by another company. This must be handled manually! Skipping permalink!");
            $this->_manual_handling["permalink_countdataset_owned_by_other_company"][] = $countDataSet->toArray();
            $this->_manual_handling["count"]++;
            return false;
        }
        $countDataSet_exists = $targetCountDataSetDb->fetchAll(
            $targetCountDataSetDb->select()
            ->where("name = ?", $countDataSet->name)
            ->where("created_date = ?", $countDataSet->created_date)
            ->where("company_id = ?", $this->_id_mappings["company"][$countDataSet->company_id])
        )->current();
        if (isset($countDataSet_exists)) {
            $this->_logger->debug("****** Count data set already exists in target DB - skipping.");
            $new_count_data_set_id = $countDataSet_exists->count_data_set_id;
        } else {
            $newCountDataSetData = $countDataSet->toArray();
            unset($newCountDataSetData["count_data_set_id"]);
            $newCountDataSetData["company_id"] = $this->_id_mappings["company"][$countDataSet->company_id];
            unset($newCountDataSetData["created_by"]);

            if ($this->_write_to_db == 1) {
                $this->_logger->debug("****** Write permalink prerequisite (Count data set)");
                $new_count_data_set = $targetCountDataSetDb->createRow($newCountDataSetData);
                $new_count_data_set_id = $new_count_data_set->save();
            } else {
                $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Count data set)");
                $new_count_data_set_id = -1; //Dummy data for dry run.
            }
        }
        $this->_id_mappings["count_data_sets"][$countDataSet->count_data_set_id] = $new_count_data_set_id;

        //CountDataSet Cameras
        $countDataSetCameras = $countDataSetCamerasDb->fetchAll(
            $countDataSetCamerasDb->select()
                ->where("count_data_set_id = ?", $countDataSetId)
        );

        foreach ($countDataSetCameras as $countDataSetCamera) {
            $countDataSetCamera_exists = $targetCountDataSetsCamerasDb->fetchAll(
                $targetCountDataSetsCamerasDb->select()
                ->where("count_data_set_id = ?", $this->_id_mappings["count_data_sets"][$countDataSetCamera->count_data_set_id])
                ->where("camera_id = ?", $this->_id_mappings["cameras"][$countDataSetCamera->camera_id])
            )->current();
            if (isset($countDataSetCamera_exists)) {
                $this->_logger->debug("****** Count data set camera for same camera_set_id and created_date already exists in target DB - skipping.");
                //$this->_skipped["count_data_set_cameras"][] = $countDataSetCamera->toArray();
                $new_count_data_set_camera_id = $countDataSetCamera_exists->count_data_set_id;
            } else {
                $newCountDataSetCameraData = $countDataSetCamera->toArray();
                $newCountDataSetCameraData["count_data_set_id"] = $this->_id_mappings["count_data_sets"][$countDataSetCamera->count_data_set_id];
                $newCountDataSetCameraData["camera_id"] = $this->_id_mappings["cameras"][$countDataSetCamera->camera_id];
                unset($newCountDataSetCameraData["count_data_set_camera_id"]);
                if ($this->_write_to_db == 1) {
                    $this->_logger->debug("****** Write permalink prerequisite (Count data set camera)");
                    $new_count_data_set_camera = $targetCountDataSetsCamerasDb->createRow($newCountDataSetCameraData);
                    $new_count_data_set_camera_id = $new_count_data_set_camera->save();
                } else {
                    $this->_logger->debug("****** Dry run 'write' permalink prerequisite (Count data set camera)");
                    $new_count_data_set_camera_id = -1; //Dummy data for dry run.
                }
            }
            $this->_id_mappings["count_data_set_cameras"][$countDataSetCamera->count_data_set_id] = $new_count_data_set_camera_id;
        }
        return $new_count_data_set_id;
    }

    private function migrateReportPermaLinks($permalinks)
    {

        $targetPermalinkDb = new Reports_Model_Permalinks();
        $report_permalink_count = 0;
        foreach ($permalinks as $permalink) {
            $report_permalink_count++;
            $this->_logger->info("Migrating report permalink " . $report_permalink_count . " of " . count($permalinks) . ".");
            $this->_logger->debug("***** Beginning migration of permalink prerequisites (permalink based on data export)");
            $decoded_permalink_data = json_decode($permalink->permalink_data);
            $new_count_data_set_id = $this->migrateCountDataSet($decoded_permalink_data->countDataSetId);
            if (!$new_count_data_set_id) {
                $this->_logger->notice("****** Permalink source data (Count data set) was not found or had source data in another company. - Skipping");
                continue;
            }
            $new_decoded_permalink_data = $decoded_permalink_data;
            $new_decoded_permalink_data->countDataSetId = $new_count_data_set_id;
            $new_decoded_permalink_data->legacyId = $permalink->permalink_id;
            $new_encoded_permalink_data = json_encode($new_decoded_permalink_data);
            $permalink_exists = $targetPermalinkDb->fetchAll(
                $targetPermalinkDb->select()
                ->where("permalink_data = ?", $new_encoded_permalink_data)
                ->where("company_id = ?", $this->_id_mappings["company"][$permalink->company_id])
            )->current();
            if (isset($permalink_exists)) {
                $this->_logger->debug("****** Permalink for company with the exact same permalink_data exists in target DB - skipping.");
                $this->_skipped["permalinks"][] = $permalink->toArray();
                $new_permalink_id = $permalink_exists->permalink_id;
            } else {
                $newPermalinkData = $permalink->toArray();
                unset($newPermalinkData["permalink_id"]);
                $newPermalinkData["permalink_data"] = $new_encoded_permalink_data;
                $newPermalinkData["company_id"] = $this->_id_mappings["company"][$permalink->company_id];
                if ($this->_write_to_db == 1) {
                    $this->_logger->debug("****** Write permalink");
                    $new_permalink = $targetPermalinkDb->createRow($newPermalinkData);
                    $new_permalink_id = $new_permalink->save();
                } else {
                    $this->_logger->debug("****** Dry run 'write' permalink");
                    $new_permalink_id = -1; //Dummy data for dry run.
                }
            }
            $this->_id_mappings["report_permalinks"][$permalink->permalink_id] = $new_permalink_id;
        }
    }

    private function printResult()
    {
        $nbr_of_errors = $nbr_of_issues = $nbr_of_skipped=0;
        if (!empty($this->_errors)) {
            $nbr_of_errors = $this->_errors["count"];
        }
        if (!empty($this->_manual_handling)) {
            $nbr_of_issues = $this->_manual_handling["count"];
        }
        $skipped = false;
        if (!empty($this->_skipped)) {
            $nbr_of_skipped = count($this->_skipped["admins"])
             + count($this->_skipped["cameras"])
             + count($this->_skipped["camera_groups"])
             + count($this->_skipped["folders"])
             + count($this->_skipped["company"])
             + count($this->_skipped["permalinks"]);
        }
        $return_string = "Migration script finished with "
            . $nbr_of_errors . " errors and "
            . $nbr_of_issues . " withstanding issues and "
            . $nbr_of_skipped . " skipped items @servertime "
            . date("H:i:s", time());
        if ($nbr_of_errors!=0) {
             $return_array["errors"] = $this->_errors;
        }
        if ($nbr_of_issues!=0) {
             $return_array["withstanding_issues"] = $this->_manual_handling;
        }
        if (!empty($this->_skipped)) {
            $return_array["skipped"] = $this->_skipped;
        }
        if (!empty($this->_updated_cameras)) {
            $return_array["updated_cameras"] = $this->_updated_cameras;
        }
        $this->_logger->notice("\n\n ==== Outputting summary to stdout and also writing it to: " . APPLICATION_PATH ."/data/session/migration_result.log ==== \n");
        $this->_results_logger->notice("\n\n ==== " . $return_string . "==== \n");
        $this->_results_logger->notice(print_r($return_array, true));
        $this->_file_logger->notice("\n\n ==== " . $return_string . "==== \n");
        $this->_file_logger->notice(print_r($return_array, true));
    }
}
