<?php

class Core_Model_Application_Bicycle extends Core_Model_Application_CountDataAbstract
{
    const PEDESTRIANS_IN = 'pin';
    const PEDESTRIANS_OUT = 'pout';
    const BICYCLES_IN = 'bin';
    const BICYCLES_OUT = 'bout';

    public function init()
    {
        parent::init();

        $typePedestriansIn = "COUNT_TYPE_PEDESTRIANS_IN";
        $typePedestriansOut = "COUNT_TYPE_PEDESTRIANS_OUT";
        $typeBicyclesIn = "COUNT_TYPE_BICYCLES_IN";
        $typeBicyclesOut = "COUNT_TYPE_BICYCLES_OUT";
        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');
            $typePedestriansIn = $translate->_("COUNT_TYPE_PEDESTRIANS_IN");
            $typePedestriansOut = $translate->_("COUNT_TYPE_PEDESTRIANS_OUT");
            $typeBicyclesIn = $translate->_("COUNT_TYPE_BICYCLES_IN");
            $typeBicyclesOut = $translate->_("COUNT_TYPE_BICYCLES_OUT");
        }

        $this->setId(4);
        $this->setName("TrueView Bicycle");
        $this->setShortName("Bicycle");
        $this->setAbbreviatedName("B");

        $this->addOutType(self::PEDESTRIANS_IN, $typePedestriansIn);
        $this->addOutType(self::PEDESTRIANS_OUT, $typePedestriansOut);
        $this->addOutType(self::BICYCLES_IN, $typeBicyclesIn);
        $this->addOutType(self::BICYCLES_OUT, $typeBicyclesOut);

        $this->addInType(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN);
        $this->addInType(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT);
        $this->addInType(Core_Model_CountData::TYPE_BICYCLES_GOING_IN);
        $this->addInType(Core_Model_CountData::TYPE_BICYCLES_GOING_OUT);
    }

    public function getCntMapping() {
        return array(
            'bin'   => Core_Model_CountData::TYPE_BICYCLES_GOING_IN, 
            'bout'   => Core_Model_CountData::TYPE_BICYCLES_GOING_OUT, 
            'pin'   => Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN,
            'pout'   => Core_Model_CountData::TYPE_PEDESTRIAN_GOING_OUT
        );
    }

    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }


        $type = -1;
        if ($outType == self::PEDESTRIANS_IN) {
            $type = Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN;
        } else if ($outType == self::PEDESTRIANS_OUT) {
            $type = Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT;
        } else if ($outType == self::BICYCLES_IN) {
            $type = Core_Model_CountData::TYPE_BICYCLES_GOING_IN;
        } else if ($outType == self::BICYCLES_OUT) {
            $type = Core_Model_CountData::TYPE_BICYCLES_GOING_OUT;
        } else {
            throw new WebReport_Exception("Not valid outtype");
        }

        $ret = "SUM(";
        if ($coalesce) { $ret .= "COALESCE("; }
        $ret .= "c" . $cameraId . "_" . $type;
        if ($coalesce) { $ret .= ",0)"; }
        $ret .= ")";

        return $ret;
    }

}

