<?php

class Core_Model_Application_Gender extends Core_Model_Application_CountDataAbstract
{
    const MALE = 'male';
    const FEMALE = 'female';

    public function init()
    {
        parent::init();

        $male = _T("MALE");
        $female = _T("FEMALE");
        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');
            $female = $translate->_($female);
            $male = $translate->_($male);
        }

        $this->addOutType(self::MALE,$male);
        $this->addOutType(self::FEMALE,$female);

        $this->addInType(Core_Model_CountData::TYPE_GENDER_MALE_DETECTION);
        $this->addInType(Core_Model_CountData::TYPE_GENDER_FEMALE_DETECTION);

        $this->setId(8);
        $this->setName("TrueView Gender");
        $this->setShortName("Gender");
        $this->setAbbreviatedName("G");
    }

    public function getCntMapping() {
        return array(
            'male'   => Core_Model_CountData::TYPE_GENDER_MALE_DETECTION, 
            'female' => Core_Model_CountData::TYPE_GENDER_FEMALE_DETECTION
        );
    }


    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }


        $type = -1;
        if ($outType == self::MALE) {
            $type = Core_Model_CountData::TYPE_GENDER_MALE_DETECTION;
        } else if ($outType == self::FEMALE) {
            $type = Core_Model_CountData::TYPE_GENDER_FEMALE_DETECTION;
        } else {
            throw new WebReport_Exception("Not valid gender type");
        }

        $ret = "SUM(";
        if ($coalesce) { $ret .= "COALESCE("; }
        $ret .= "c" . $cameraId . "_" . $type;
        if ($coalesce) { $ret .= ",0)"; }
        $ret .= ")";

        return $ret;
    }
}

