<?php

class Core_Model_Application_Occupancy extends Core_Model_Application_CountDataAbstract
{
    const IN = 'in';
    const OUT = 'out';
    const OCCUPANCY = 'occ';
    const AVERAGE_VISIT_TIME = 'avgt';

    public function init()
    {
        parent::init();

        $occ = _T("COUNT_TYPE_OCCUPANCY");
        $avgt = _T("COUNT_TYPE_AVERAGE_VISIT_TIME");
        $total_in = _T("COUNT_TYPE_TOTAL_IN");
        $total_out = _T("COUNT_TYPE_TOTAL_OUT");
        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');
            $occ = $translate->_($occ);
            $avgt = $translate->_($avgt);
            $total_in = $translate->_($total_in);
            $total_out = $translate->_($total_out);
        }

        //$trans = Core_Model_Application_PeopleCounter::getInOutText();

        $this->addOutType(self::OCCUPANCY,$occ);
        $this->addOutType(self::AVERAGE_VISIT_TIME,$avgt);
        $this->addOutType(self::IN, $total_in);
        $this->addOutType(self::OUT, $total_out);

        $this->addInType(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN);
        $this->addInType(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT);
        $this->addInType(Core_Model_CountData::TYPE_OCCUPANCY);
        $this->addInType(Core_Model_CountData::TYPE_AVERAGE_VISIT_TIME);

        $this->setId(11);
        $this->setName("AXIS Occupancy Estimator");
        $this->setShortName("AXIS Occupancy Estimator");
        $this->setAbbreviatedName("OCC");
    }

    public function getCntMapping() {
        return array(
            self::IN   => Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN,
            self::OUT  => Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT,
            self::OCCUPANCY  => Core_Model_CountData::TYPE_OCCUPANCY, 
            self::AVERAGE_VISIT_TIME => Core_Model_CountData::TYPE_AVERAGE_VISIT_TIME
        );
    }


    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }


        $type = -1;
        if ($outType == self::OCCUPANCY) {
            $type = Core_Model_CountData::TYPE_OCCUPANCY;
        } else if ($outType == self::AVERAGE_VISIT_TIME) {
            $type = Core_Model_CountData::TYPE_AVERAGE_VISIT_TIME;
        } else if ($outType == self::IN) {
            $type = Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN;
        } else if ($outType == self::OUT) {
            $type = Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT;
        } else {
            throw new WebReport_Exception("Not valid occupancy type");
        }
        if ( $type == Core_Model_CountData::TYPE_OCCUPANCY || $type == Core_Model_CountData::TYPE_AVERAGE_VISIT_TIME) {
            $ret = "AVG(";
            if ($coalesce) { $ret .= "COALESCE("; }
            $ret .= "c" . $cameraId . "_" . $type;
            if ($coalesce) { $ret .= ",0)"; }
            $ret .= ")";
        } else {
            $ret = "SUM(IFNULL(";
            if ($coalesce) { $ret .= "COALESCE("; }
            $ret .= "c" . $cameraId . "_" . $type;
            if ($coalesce) { $ret .= ",0)"; }
            $ret .= ",0))";
        }

        return $ret;
    }

}

