<?php

class Core_Model_Application_Parking extends Core_Model_Application_InOutAbstract
{
    const IN = 'cars_in';
    const OUT = 'cars_out';

    public function init()
    {
        parent::init();

        $typeIn = "COUNT_TYPE_CARS_IN";
        $typeOut = "COUNT_TYPE_CARS_OUT";
        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');
            $typeIn = $translate->_("COUNT_TYPE_CARS_IN");
            $typeOut = $translate->_("COUNT_TYPE_CARS_OUT");
        }

        $this->setId(3);
        $this->setName("TrueView Parking");
        $this->setShortName("Parking");
        $this->setAbbreviatedName("P");

        $this->addOutType(self::IN, $typeIn);
        $this->addOutType(self::OUT, $typeOut);

        $this->addInType(Core_Model_CountData::TYPE_CARS_GOING_IN);
        $this->addInType(Core_Model_CountData::TYPE_CARS_GOING_OUT);
    }

    public function getCntMapping() {
        return array(self::IN => 36, self::OUT => 37);
    }

    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }

        $type = -1;
        if ($outType == self::IN) {
            $type = Core_Model_CountData::TYPE_CARS_GOING_IN;
        } else if ($outType == self::OUT) {
            $type = Core_Model_CountData::TYPE_CARS_GOING_OUT;
        } else {
            throw new WebReport_Exception("Not valid outtype");
        }

        $ret = "SUM(";
        if ($coalesce) { $ret .= "COALESCE("; }
        $ret .= "c" . $cameraId . "_" . $type;
        if ($coalesce) { $ret .= ",0)"; }
        $ret .= ")";

        return $ret;
    }

}

