<?php

class Core_Model_Application_PeopleCounter extends Core_Model_Application_InOutAbstract
{
    const IN = 'in';
    const OUT = 'out';

    public static function getInOutText() {
        $typeIn = _T("COUNT_TYPE_IN");
        $typeOut = _T("COUNT_TYPE_OUT");
        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');
            $typeIn = $translate->_("COUNT_TYPE_IN");
            $typeOut = $translate->_("COUNT_TYPE_OUT");
        }
        return array("in" => $typeIn, "out" => $typeOut);
    }

    public function init()
    {
        parent::init();

        $trans = self::getInOutText();

        $this->setId(1);
        $this->setName("AXIS People Counter");
        $this->setShortName("AXIS People Counter");
        $this->setAbbreviatedName("PC");

        $this->addOutType(self::IN, $trans["in"]);
        $this->addOutType(self::OUT, $trans["out"]);

        $this->addInType(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN);
        $this->addInType(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT);
    }

    public function getCntMapping() {
        return array('in' => 3, 'out' => 4);
    }

    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }

        $type = -1;
        if ($outType == self::IN) {
            $type = Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN;
        } else if ($outType == self::OUT) {
            $type = Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT;
        } else {
            throw new WebReport_Exception("Not valid outtype");
        }

        $ret = "SUM(IFNULL(";
        if ($coalesce) { $ret .= "COALESCE("; }
        $ret .= "c" . $cameraId . "_" . $type;
        if ($coalesce) { $ret .= ",0)"; }
        $ret .= ",0))";

        return $ret;

    }
}

