<?php

class Core_Model_Application_Pos extends Core_Model_Application_CountDataAbstract
{
    const SALE = 'sale';
    const TRANSACTION = 'transaction';

    public function init()
    {
        parent::init();

        $sales = _T("SALES");
        $transactions = _T("TRANSACTIONS");
        $name = "Point of Sale";
        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');
            $sales = $translate->_($sales);
            $transactions = $translate->_($transactions);
            $name = $translate->_("POINT_OF_SALE");
        } else {
            $sales = strtolower($sales);
            $transactions = strtolower($transactions);
        }

        $this->addOutType(self::SALE, $sales);
        $this->addOutType(self::TRANSACTION,$transactions);

        $this->addInType(Core_Model_CountData::TYPE_POS_SALE);
        $this->addInType(Core_Model_CountData::TYPE_POS_TRANSACTION);

        $this->setId(9);
        $this->setName($name);
        $this->setShortName("POS");
        $this->setAbbreviatedName("PS");
    }

    public function getCntMapping() {
        return array(
            'sale' => Core_Model_CountData::TYPE_POS_SALE,
            'trans' => Core_Model_CountData::TYPE_POS_TRANSACTION,
            'conv' => Core_Model_CountData::TYPE_POS_CONVERSION_RATE,
            'basket' => Core_Model_CountData::TYPE_POS_AVERAGE_BASKET_VALUE
        );
    }

    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }


        $type = -1;
        if ($outType == self::SALE) {
            $type = Core_Model_CountData::TYPE_POS_SALE;
        } else if ($outType == self::TRANSACTION) {
            $type = Core_Model_CountData::TYPE_POS_TRANSACTION;
        } else {
            throw new WebReport_Exception("Type '$outType' is not valid POS type");
        }

        $ret = "SUM(IFNULL(";
        if ($coalesce) { $ret .= "COALESCE("; }
        $ret .= "c" . $cameraId . "_" . $type;
        if ($coalesce) { $ret .= ",0)"; }
        $ret .= ",0))";

        return $ret;
    }
}

