<?php

abstract class Core_Model_Application_SalesDataAbstract extends Core_Model_Application_Abstract implements Core_Model_Application_CountDataInterface
{

    protected $_outTypes;
    protected $_inTypes;
    protected $_graphOptions = array();

    public function addOutType($id, $text, $graphOptions = null)
    {
        $this->_outTypes[$id] = $text;
        $this->_graphOptions[$id] = $graphOptions;
    }

    public function addInType($type)
    {
        $this->_inTypes[$type] = $type;
    }

    public function getGraphOptions($id)
    {
        if (isset($this->_graphOptions[$id])) {
            return $this->_graphOptions[$id];
        } else {
            return array();
        }
    }

    public function getInTypes()
    {
        return $this->_inTypes;
    }

    public function getOutTypes()
    {
        return $this->_outTypes;
    }

    public function getSqlQueryForInTypes($camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }

        $ret = array();

        foreach( $this->_inTypes as $type ) {
            $ret[] = 'c' . $cameraId . '_' . $type;
        }

        return $ret;

    }

    public function getRawData($cameras) {
        /* @var $db Zend_Db_Adapter_Pdo_Mysqdl() */
        $db = Zend_Registry::get('db');
        $select_table = NULL;
        $join_table = array();
        $columns = array();
        $count_data_table = new Core_Model_CountData_Tables();
        $start_time = INF;
        $end_time = 0;

        foreach($cameras as $camera) {
            $row = $count_data_table->find($camera->count_data_table_id)->current();
            $name = $row->table;

            if($start_time > $camera->first_date)
                    $start_time = $camera->first_date;

            if($end_time < $camera->last_date)
                    $end_time = $camera->last_date;

            if(is_null($select_table))
                $select_table = $name;

            if(!isset($join_table[$name]) && $name != $select_table) {
                $join_table[] = $name;
            }

            $cols = $this->getSqlQueryForInTypes($camera);

            foreach($cols as $col) {
                $columns[$col] = $name . '.' . $col;
            }
        }
       
        
        $select = $db->select()
                ->from($select_table, array_merge(array('cnttime'),$columns));
        foreach($join_table as $table)
            $select->join($table, $table . '.cnttime = ' . $select_table . '.cnttime');
        //$select->where('DATE_FORMAT(cnttime, "%Y-%m-%d") >= ?', $start_time->toString("yyyy-MM-dd"));
        //$select->where('DATE_FORMAT(cnttime, "%Y-%m-%d") <= ?', $end_time->toString("yyyy-MM-dd"));

        return $db->fetchAll($select);
    }
}

