<?php

class Core_Model_Application_Queue extends Core_Model_Application_CountDataAbstract
{
    const HIGH_FLOW = 'high';
    const MID_FLOW = 'mid';
    const LOW_FLOW = 'low';
    const PEOPLE = "people";

    public function init()
    {
        parent::init();

        $highflow = _T("COUNT_TYPE_HIGH");
        $midflow = _T("COUNT_TYPE_MID");
        $lowflow = _T("COUNT_TYPE_LOW");
        $people = _T("COUNT_TYPE_PEOPLE_IN_QUEUE");
        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');
            $highflow = $translate->_($highflow);
            $midflow = $translate->_($midflow);
            $lowflow = $translate->_($lowflow);
            $people = $translate->_($people);
        }

        $this->addOutType(self::HIGH_FLOW,$highflow);
        $this->addOutType(self::MID_FLOW,$midflow);
        $this->addOutType(self::LOW_FLOW,$lowflow);
        $this->addOutType(self::PEOPLE,$people);

        $this->addInType(Core_Model_CountData::TYPE_QUEUE_HIGH_FLOW);
        $this->addInType(Core_Model_CountData::TYPE_QUEUE_MID_FLOW);
        $this->addInType(Core_Model_CountData::TYPE_QUEUE_LOW_FLOW);
        $this->addInType(Core_Model_CountData::TYPE_QUEUE_PEOPLE);

        $this->setId(7);
        $this->setName("TrueView Queue");
        $this->setShortName("Queue");
        $this->setAbbreviatedName("QM");
    }

    public function getCntMapping() {
        return array(
            'high'   => Core_Model_CountData::TYPE_QUEUE_HIGH_FLOW, 
            'mid'   => Core_Model_CountData::TYPE_QUEUE_MID_FLOW, 
            'low'   => Core_Model_CountData::TYPE_QUEUE_LOW_FLOW,
            'people' => Core_Model_CountData::TYPE_QUEUE_PEOPLE
        );
    }

    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }


        $type = -1;
        $sumOrAvg = "SUM(";
        if ($outType == self::HIGH_FLOW) {
            $type = Core_Model_CountData::TYPE_QUEUE_HIGH_FLOW;
        } else if ($outType == self::MID_FLOW) {
            $type = Core_Model_CountData::TYPE_QUEUE_MID_FLOW;
        } else if ($outType == self::LOW_FLOW) {
            $type = Core_Model_CountData::TYPE_QUEUE_LOW_FLOW;
        } else if ($outType == self::PEOPLE) {
            $type = Core_Model_CountData::TYPE_QUEUE_PEOPLE;
            $sumOrAvg = "AVG(";
        } else {
            throw new WebReport_Exception("Not valid queue type");
        }

        $ret = $sumOrAvg;
        if ($coalesce) { $ret .= "COALESCE("; }
        $ret .= "c" . $cameraId . "_" . $type;
        if ($coalesce) { $ret .= ",0)"; }
        $ret .= ")";

        return $ret;
    }

}

