<?php

class Core_Model_Application_Queue2 extends Core_Model_Application_CountDataAbstract
{
    const QUEUE2_PEOPLE_IN_QUEUE = 'people_in_queue';
    const QUEUE2_UPPER_LIMIT = 'queue_upper_limit';
    const QUEUE2_LOWER_LIMIT = 'queue_lower_limit';

    public function init()
    {
        parent::init();

        $people_in_queue =  _T('people_in_queue');
        $upper_limit = _T('queue_high_limit');
        $lower_limit = _T('queue_medium_limit');

        if (Zend_Registry::isRegistered('Zend_Translate')) {
            $translate = Zend_Registry::get('Zend_Translate');

            $people_in_queue = $translate->_($people_in_queue);
            $upper_limit = $translate->_($upper_limit);
            $lower_limit = $translate->_($lower_limit);
        }
        $this->addOutType(self::QUEUE2_PEOPLE_IN_QUEUE, $people_in_queue);
        $this->addOutType(self::QUEUE2_UPPER_LIMIT, $upper_limit);
        $this->addOutType(self::QUEUE2_LOWER_LIMIT, $lower_limit);

        $this->addInType(Core_Model_CountData::TYPE_QUEUE2_PEOPLE_IN_QUEUE);
        $this->addInType(Core_Model_CountData::TYPE_QUEUE2_UPPER_LIMIT);
        $this->addInType(Core_Model_CountData::TYPE_QUEUE2_LOWER_LIMIT);

        $this->setId(13);
        $this->setName("AXIS Queue Monitor");
        $this->setShortName("AXIS Queue Monitor");
        $this->setAbbreviatedName("QM2");
    }

    public function getCntMapping()
    {
            return array(
                self::QUEUE2_PEOPLE_IN_QUEUE => Core_Model_CountData::TYPE_QUEUE2_PEOPLE_IN_QUEUE,
                self::QUEUE2_UPPER_LIMIT => Core_Model_CountData::TYPE_QUEUE2_UPPER_LIMIT,
                self::QUEUE2_LOWER_LIMIT => Core_Model_CountData::TYPE_QUEUE2_LOWER_LIMIT,
            );
    }

    public function getTimeResolutions()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $return = array();
        $return["ts1min"] = $translate->_("COUNT_TIME_RESOLUTION_MINUTE");
        $return["ts5min"] = $translate->_("COUNT_TIME_RESOLUTION_FIVE_MINUTES");
        $return["ts15min"] = $translate->_("COUNT_TIME_RESOLUTION_FIFTEEN_MINUTES");
        $return["ts30min"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_HOUR");
        $return["ts1h"] = $translate->_("COUNT_TIME_RESOLUTION_HOUR");
        $return["ts6h"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_DAY");
        $return["ts12h"] = $translate->_("COUNT_TIME_RESOLUTION_HALF_DAY");
        $return["ts1d"] = $translate->_("COUNT_TIME_RESOLUTION_DAY");
        $return["ts1w"] = $translate->_("COUNT_TIME_RESOLUTION_WEEK");
        $return["ts2w"] = $translate->_("COUNT_TIME_RESOLUTION_TWO_WEEKS");
        $return["ts1m"] = $translate->_("COUNT_TIME_RESOLUTION_MONTH");
        $return["ts1q"] = $translate->_("COUNT_TIME_RESOLUTION_QUARTER_OF_YEAR");
        $return["ts1y"] = $translate->_("COUNT_TIME_RESOLUTION_YEAR");

        return $return;
    }

    public static function letAppModifyData($data)
    {
        if ($data['protocol']==1) {
            $ret = array();
            foreach ($data['data'] as &$entry) {
                array_push($entry, $data['queue_upper_limit']);
                array_push($entry, $data['queue_lower_limit']);
            }
            array_push($data['data_struct'], 'queue_upper_limit');
            array_push($data['data_struct'], 'queue_lower_limit');
            return $data;
        } else {
            throw new WebReport_Exception("Unknown protocol version in Queue2::letAppModifyData");
        }
    }

    public function getSqlQueryForCntOutType($outType, $camera, $coalesce = false)
    {
        if ($camera instanceof Core_Model_Camera) {
            $cameraId = $camera->camera_id;
        } else {
            $cameraId = $camera;
        }

        $type = -1;
        $types = $this->getCntMapping();

        if(isset($types[$outType])){
            $type = $types[$outType];
        } else {
            throw new WebReport_Exception("Not valid outtype");
        }

        $ret = "ROUND(AVG(IFNULL(";
        if ($coalesce) { $ret .= "COALESCE("; }
        $ret .= "c" . $cameraId . "_" . $type;
        if ($coalesce) { $ret .= ",0)"; }
        $ret .= ",0)), " . ($outType===self::QUEUE2_PEOPLE_IN_QUEUE?"1":"0") . ")";

        return $ret;
    }

    public static function aggregateData($data){
        if (count($data['data'])) {
            new DemographicsAggregateData($data);
        }

    }

    public static function areaName($camera_name, $queue_area) {
    	return $queue_area . " [" . $camera_name ."]";
    }

}



