<?php

class Core_Model_Applications
{
    static $_instance = NULL;
    public static function getInstance() {
      if(self::$_instance == NULL) self::$_instance = new Core_Model_Applications();
      return self::$_instance;
    }
    protected $_data = array();

    public static function getWithId($id) {
        $apps = self::getInstance();
        return $apps->findWithId($id);
    }

    public function __construct()
    {

        //$config = Zend_Registry::get('config');
        #$availableApplications = $config->general->applications->toArray();
        $availableApplications = array(1,2,3,4,5,7,8,9,11,12,13,14);

        $apps = array('PeopleCounter', 'Parking', 'Bicycle', 'Heatmap', 'Queue', 'Gender', 'Pos', 'Occupancy', 'Demographics','Queue2', 'StereoPeopleCounter');
        foreach ($apps as $appName) {
            $className = "Core_Model_Application_" . $appName;
            $app = new $className();
            if (in_array($app->getId(), $availableApplications))
                $this->addApplication($app);
        }
    }

    public function addApplication(Core_Model_Application_Abstract $app)
    {
        $this->_data[$app->getId()] = $app;
    }

    public function fetchAll()
    {
        return $this->_data;
    }

    public function fetchAllWithCountData()
    {
        $d = array();
        foreach ($this->_data as $app) {
            if ($app instanceof Core_Model_Application_CountDataAbstract) {
                $d[$app->getId()] = $app;
            }
        }
        return $d;
    }

    public function fetchAllWithEventData()
    {
        $d = array();
        foreach ($this->_data as $app) {
            if ($app instanceof Core_Model_Application_AsEventDataInterface){
                $app = $app->asEventDataApplication();
            }

            if ($app instanceof Core_Model_Application_EventDataAbstract) {
                $d[$app->getId()] = $app;
            } 
        }
        return $d;
    }

    public function fetchAllThatHasVisibleDataInDb($type = "CountData", $user = null)
    {
        if (strtolower($type) == "countdata") {
            $all_apps = $this->fetchAllWithCountData();
        } elseif (strtolower($type) == "eventdata") {
            $all_apps = $this->fetchAllWithEventData();
        } else {
            throw new WebReport_Exception("Unknown datatype.");
        }
        if (!isset($user)) {
            $user = Zend_Auth::getInstance()->getIdentity();
        }
        $cameraDb = Core_Model_Cameras::getInstance();
        if (!$user->multi_company) {
            $companyDb = Core_Model_Companies::getInstance();
            $company = $companyDb->getCompany($user->company_id);
            $cameras = $cameraDb->getCameras(null, $company);
        } else {
            $cameras = $cameraDb->fetchAll();
        }
        $application_ids = array();
        foreach ($cameras as $camera) {
            $application_ids[] = $camera->application_id;
        }
        $applications_to_return = array_unique($application_ids);
        $ret = array();
        foreach ($all_apps as $app) {
            $appId = $app->getId();
            if (in_array($appId, $applications_to_return)) {
                $ret[$appId] = $app;
            }
        }
        return $ret;
    }

    public function fetchAllWithInOut()
    {
        $d = array();
        foreach ($this->_data as $app) {
            if ($app instanceof Core_Model_Application_InOutAbstract) {
                $d[$app->getId()] = $app;
            }
        }
        return $d;
    }

    public function findWithId($id)
    {
        if (isset($this->_data[$id]))
            return $this->_data[$id];
        else
            return null;
    }

    public function findWithShortName($shortName)
    {

        $return = null;
        foreach ($this->_data as $app) {
            if ($app->getShortName() == $shortName)
                return $app;
        }
        return $return;
    }

    public static function guessApplication($types = array())
    {
        $app = null;
        // @todo Better check of application with types and shit
        if (count($types) == 2) {
            if (
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN, $types) &&
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT, $types)
            ) {
                $app = new Core_Model_Application_PeopleCounter();
            } else if (
                    in_array(Core_Model_CountData::TYPE_CARS_GOING_IN, $types) &&
                    in_array(Core_Model_CountData::TYPE_CARS_GOING_OUT, $types)
            ) {
                $app = new Core_Model_Application_Parking();
            } else if (
                    in_array(Core_Model_CountData::TYPE_GENDER_MALE_DETECTION, $types) &&
                    in_array(Core_Model_CountData::TYPE_GENDER_FEMALE_DETECTION, $types) 
            ) {
                $app = new Core_Model_Application_Gender();
            } 
        } else if (count($types) == 4) {
            if (
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN, $types) &&
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT, $types) &&
                    in_array(Core_Model_CountData::TYPE_BICYCLES_GOING_IN, $types) &&
                    in_array(Core_Model_CountData::TYPE_BICYCLES_GOING_OUT, $types)
            ) {
                $app = new Core_Model_Application_Bicycle();
            } else if(in_array(Core_Model_CountData::TYPE_QUEUE_HIGH_FLOW, $types) &&
                     in_array(Core_Model_CountData::TYPE_QUEUE_MID_FLOW, $types) &&
                     in_array(Core_Model_CountData::TYPE_QUEUE_LOW_FLOW, $types) &&
                     in_array(Core_Model_CountData::TYPE_QUEUE_PEOPLE, $types))
            {
                $app = new Core_Model_Application_Queue();
            } else if (
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN, $types) &&
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT, $types) &&
                    in_array(Core_Model_CountData::TYPE_OCCUPANCY, $types) &&
                    in_array(Core_Model_CountData::TYPE_AVERAGE_VISIT_TIME, $types) 
            ) {
                $app = new Core_Model_Application_Occupancy();
            } else { // check for zone conter types
                throw new WebReport_Exception("Could not find application with these characteristics.");
            }
        } else if (count($types) == 3) {
            if (
                    in_array(Core_Model_CountData::TYPE_QUEUE_HIGH_FLOW, $types) &&
                    in_array(Core_Model_CountData::TYPE_QUEUE_MID_FLOW, $types) &&
                    in_array(Core_Model_CountData::TYPE_QUEUE_LOW_FLOW, $types))
            {
                $app = new Core_Model_Application_Queue();
            } else if ( 
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_IN, $types) &&
                    in_array(Core_Model_CountData::TYPE_PEDESTRIANS_GOING_OUT, $types) &&
                    in_array(Core_Model_CountData::TYPE_OCCUPANCY, $types) ) {
                $app = new Core_Model_Application_Occupancy();
            }
        }
        if(!$app && (in_array(Core_Model_CountData::TYPE_POS_SALE, $types) || in_array(Core_Model_CountData::TYPE_POS_TRANSACTION, $types)))
           $app = new Core_Model_Application_Pos();

        return $app;
    }

    public function getSqlQueryForCntOutType($outType, $camera)
    {
        /**
         * existing fields is associative array with in types as keys and table join names as values
         */
        foreach ($this->_data as $application) {
            try {
                $ret = $application->getSqlQueryForCntOutType($outType, $camera);
                return $ret;
            } catch (WebReport_Exception $e) {
                
            }
        }

        throw new WebReport_Exception("Not valid outtype");
    }

}
