<?php

include_once(APPLICATION_PATH . '/abc.php');
include_once('Pest/PestJSON.php');

class Core_Model_Camera extends Zend_Db_Table_Row_Abstract
{
    protected $_countDataTableName = null;

    public function minutesSinceContact(){
        if($this->last_contact == null) return false;
        return (time() - strtotime($this->last_contact . " UTC"))/60;
    }

    public function minutesSinceFirstContact(){
        if($this->first_contact == null) return PHP_INT_MAX;
        return (time() - date(strtotime($this->first_contact)))/60;
    }

    public function getConnectionLevel(){
        $timeout = $this->getCameraGroup()->no_connection_timeout * 60;

        $min = $this->minutesSinceContact();
        
        if($min !== 0 && !$min) return "contact_unknown";
        if($min < 30) return "contact_2";
        if($min < $timeout) return "contact_1";
        return "contact_0";
    }

    protected function _postUpdate() {
        if($this->signature and $this->serial) {
           $data = json_decode($this->signature);
           if($data->data->serial != $this->serial) {
               $this->signature = NULL;
               $this->save();
               file_put_contents(APPLICATION_PATH . "/../public/tmp/has_tentative_cams", "1");
           }
        }
    }
    /**
     * Pre insert logic. Alter count data table with a new column for this camera
     */
    protected function _insert()
    {
        $oldTimeLimit = (int) ini_get('max_execution_time');
        set_time_limit(300);
        $mutex = new Core_Model_Mutex();
        $mutex->init(Core_Model_Mutex::MUTEX_INSERT_CAMERA);
        //$semid=sem_get(1);
        if($mutex->acquire() == FALSE)
            throw new WebReport_Exception("Unable to require semaphore.");

        $db = $this->getTable()->getAdapter();
        $applications = new Core_Model_Applications();
        $application = $applications->findWithId($this->application_id);
        if ($application == null)
            throw new WebReport_Exception("Tried to create camera with invalid application id.");

        if (!($application instanceof Core_Model_Application_CountDataAbstract)) {
            return;
        }
        $countDataTablesDb = new Core_Model_CountData_Tables();
        $table = $countDataTablesDb->getTableForInserting($application);
        if (!$table) {
            $countDataTables = $countDataTablesDb->fetchAll($countDataTablesDb->select()->where('application_id = ?', $application->getId())->order("table"));
            $nbrOfCountDataTables = count($countDataTables);
            $newTableName = false;
            $i = 1;

            // find table name that do not already exist
            do {
                $newTableName = "count_data_" . strtolower($application->getAbbreviatedName()) . ($nbrOfCountDataTables + $i);

                $row = $countDataTablesDb->fetchRow($countDataTablesDb->select()->where($db->quoteIdentifier('table') . ' = ?', $newTableName));

                if ($row) {
                    $newTableName = false;
                    $i++;
                }
            } while (!$newTableName);
            $table = $countDataTablesDb->fetchNew();
            $table->table = $newTableName;
            $table->camera_count = 0;
            $table->application_id = $application->getId();

            $db->query("CREATE TABLE IF NOT EXISTS " . $db->quoteIdentifier($newTableName) . " (
                `" . $newTableName . "_id` int(11) NOT NULL auto_increment,
                `cnttime` datetime NOT NULL,
                `ts1min` int(11) NOT NULL,
                `ts5min` int(11) NOT NULL,
                `ts10min` int(11) NOT NULL,
                `ts15min` int(11) NOT NULL,
                `ts30min` int(11) NOT NULL,
                `ts1h` int(11) NOT NULL,
                `ts6h` int(11) NOT NULL,
                `ts12h` int(11) NOT NULL,
                `ts1d` int(11) NOT NULL,
                `ts1w` int(11) NOT NULL,
                `ts1w_m` int(11) NOT NULL,
                `ts2w` int(11) NOT NULL,
                `ts2w_m` int(11) NOT NULL,
                `ts1m` int(11) NOT NULL,
                `ts1q` int(11) NOT NULL,
                `ts1y` int(11) NOT NULL,
                `last_update` TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
                PRIMARY KEY  (`" . $newTableName . "_id`),
            KEY `time` (`cnttime`),
            KEY `ts5min` (`ts5min`),
            KEY `ts10min` (`ts10min`),
            KEY `ts15min` (`ts15min`),
            KEY `ts1h` (`ts1h`),
            KEY `ts6h` (`ts6h`),
            KEY `ts1d` (`ts1d`),
            KEY `ts1w` (`ts1w`),
            KEY `ts1w_m` (`ts1w_m`),
            KEY `ts30min` (`ts30min`),
            KEY `ts2w` (`ts2w`),
            KEY `ts2w_m` (`ts2w_m`),
            KEY `ts1m` (`ts1m`),
            KEY `ts1q` (`ts1q`),
            KEY `ts1y` (`ts1y`),
            KEY `ts1min` (`ts1min`),
            KEY `ts12h` (`ts12h`)
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_bin;");
            $timeGroups = array("cnttime", "ts1min", "ts5min", "ts10min", "ts15min", "ts30min", "ts1h", "ts6h", "ts12h", "ts1d", "ts1w","ts1w_m", "ts2w", "ts2w_m", "ts1m", "ts1q", "ts1y");

            if ($nbrOfCountDataTables > 0) {
                $tableWithTimes = $countDataTables->current();
                $db->query("INSERT INTO " . $db->quoteIdentifier($newTableName) . " (" . implode(", ", $timeGroups) . ")" .
                    " SELECT " . implode(", ", $timeGroups) . " FROM " . $tableWithTimes->table . ";");
            }
        }
        $table->camera_count = $table->camera_count + 1;
        $table->save();

        $this->count_data_table_id = $table->count_data_table_id;
        $this->last_contact = new Zend_Db_Expr("UTC_TIMESTAMP()");
        $this->_countDataTableName = $table->table;
        if($this->application_id == 11) {
            $this->report_warnings = 0;
        }
        if($mutex->release()==FALSE)
            throw new WebReport_Exception("Unable to release semaphore.");
        set_time_limit($oldTimeLimit);
    }

    public function isCountData() {
        $applications = new Core_Model_Applications();
        $application = $applications->findWithId($this->application_id);
        return $application instanceof Core_Model_Application_CountDataAbstract;
    }

    public function hasEventData() {
        $applications = new Core_Model_Applications();
        $application = $applications->findWithId($this->application_id);
        return $application instanceof Core_Model_Application_AsEventDataInterface;
    }


    public function sendWarningByEmail($shortText, $text, $dry = false){
        $cameraGroup = $this->findParentRow('Core_Model_CameraGroups');
        $notification = new Core_Model_EmailNotification();
        $notification->addRecipient($cameraGroup->email, ";");
        $notification->setTitle("DataManager Notification: ". $shortText);
        $notification->setText($text);
        $hours = floor($this->minutesSinceContact() / 60);
        $notification->setInfo(array("Name" => $this->name,
            "IP" => ($this->ip . " (might be a gateway, not camera)"),
            "Company" => implode(', ', $cameraGroup->getCompanyNames()),
            "Camera Group" => $cameraGroup->name,
            "Last date for which there exists data" => $this->last_date . " (" . date_default_timezone_get() . ")",
            "Last date for which there was any contact " => $this->last_contact . " (" . date_default_timezone_get() . ")",
            "Time since last contact" =>  $hours . " hour" . ($hours > 1 ? "s" : "")
        ));
        if ( $dry == false ) {
            $notification->send("camera group '" . $cameraGroup->name . "'");
        }
        return $notification;
    }

    protected function _postInsert()
    {
        $applications = new Core_Model_Applications();
        $application = $applications->findWithId($this->application_id);

        // TODO: remove this if it doesn't break anything.
        if (!($application instanceof Core_Model_Application_CountDataAbstract)) {
            $this->_refresh();
            $this->first_contact_hash = cm_hash_camera($this);
            $this->save();

            return;        
        }

        /* @var $db Zend_Db */
        $db = $this->getTable()->getAdapter();
        $tableName = $this->getCountDataTableName();
        $oldTimeLimit = (int) ini_get('max_execution_time');
        $inTypes = array();
        if(method_exists($application, "getInTypes")) {
            $inTypes = $application->getInTypes();
        }

        $sql = "ALTER TABLE " . $db->quoteIdentifier($tableName);
        foreach ($inTypes as $typeName => $typeId) {
            $sql .= " ADD " . $db->quoteIdentifier("c" . $this->camera_id . "_" . $typeName) . " INT(6), ";
        }
        $sql = substr($sql, 0, -2); // remove trailing comma.

        $mutex = new Core_Model_Mutex();
        $mutex->init(Core_Model_Mutex::MUTEX_INSERT_CAMERA);
        set_time_limit(600);
        if($mutex->acquire() == FALSE)
            throw new WebReport_Exception("Unable to require semaphore.");
        $db->query($sql);
        if($mutex->release()==FALSE)
            throw new WebReport_Exception("Unable to release semaphore.");


        $this->_refresh();
        $this->first_contact_hash = cm_hash_camera($this);
        $this->save();

        set_time_limit($oldTimeLimit);
    }

    public function getCountDataTableName()
    {
        return is_null($this->_countDataTableName) ? $this->table : $this->_countDataTableName;
    }

    /**
     * Pre-delete logic to be applied to row.
     *
     * @return void
     */
    protected function _delete()
    {
        $applications = new Core_Model_Applications();
        $application = $applications->findWithId($this->application_id);
        if (!($application instanceof Core_Model_Application_CountDataAbstract)) {
            return;
//            throw new WebReport_Exception("No such application.");
        }

        $cameraId = $this->camera_id;

        $identity = Zend_Auth::getInstance()->getIdentity();

        $formulaSetFormulasDb = new Core_Model_CountData_FormulaSetFormulas();
        $formulas = $formulaSetFormulasDb->fetchAll();

        foreach ($formulas as $formula) {
            $f = new WebReport_Formula($formula->formula);
            $cameras = $f->getCameraIdsAndOutTypesInvolved();
            $cameraIds = array_keys($cameras);
            if (in_array($cameraId, $cameraIds)) {
                $formula->delete();
            }
        }

        $virtualCamerasDb = new Core_Model_VirtualCameras();
        $virtualCameraFormulaDb = new Core_Model_VirtualCameraFormula();

        $formulas = $virtualCameraFormulaDb->fetchAll();

        foreach ($formulas as $formula) {
            $f = new WebReport_Formula($formula->formula);
            $cameras = $f->getCameraIdsAndOutTypesInvolved();

            $cameraIds = array_keys($cameras);
            if (in_array($cameraId, $cameraIds)) {
                $virtualCamera = $virtualCamerasDb->find($formula->virtual_camera_id)->current();
                if ($virtualCamera)
                    $virtualCamera->delete();
            }
        }

        $countDataTablesDb = new Core_Model_CountData_Tables();
        $table = $countDataTablesDb->find($this->count_data_table_id)->current();
        $table->camera_count = $table->camera_count - 1;
        $table->save();

        $applications = new Core_Model_Applications();
        $db = Zend_Registry::get('db');
        $sql = "ALTER TABLE " . $db->quoteIdentifier($table->table) . " ";
        $i = 0;
        $application = $applications->findWithId($this->application_id);
        $inTypes = $application->getInTypes();
        foreach ($inTypes as $type) {
            $i++;
            $sql .= " DROP " . $db->quoteIdentifier("c" . $this->camera_id . "_" . $type);
            if ($i < count($inTypes))
                $sql .= ", ";
            else
                $sql .= ";";
        }
        try {
            $db->query($sql);
        } catch(Exception $e) {
        }

        /*
         * Let's not drop the table based on camera_count,
         * it is not to be trusted. Also notice that the preg_match inside
         * only has 3-4, not [0-9]+. So it can remove non-TVPC data if unlucky.
         *
        if ($table->camera_count <= 0) {
            $columnNames = array_keys($db->describeTable($table->table));
            foreach ($columnNames as $c) {
                if(preg_match("/c[0-9]+_[3-4]/", $c) == 1){
                    return;
                }
            }
            $table->delete();
        }
         */
        
        $logger = Zend_Registry::get('loggers')->getLogger('user');
        $logger->notice($identity->forename . " " . $identity->surname . ' deleted a camera with id ' . $this->camera_id);
    }

    public function hasCountDataApplication()
    {
        $applications = new Core_Model_Applications();
        $app = $applications->findWithId($this->application_id);
        return ($app instanceof Core_Model_Application_CountDataAbstract);
    }

    public function getCameraGroup()
    {
        if(Zend_Registry::isRegistered('cg-'.$this->camera_group_id)){
            return Zend_Registry::get('cg-'.$this->camera_group_id);
        } else {
            $cg = $this->findParentRow('Core_Model_CameraGroups');
            Zend_Registry::set('cg-'.$this->camera_group_id, $cg);
            return $cg;
        }
    }

    public function getChannelStatus()
    {
        return cm_get_camera_status($this, TRUE);
    }

    public function getDaysUntilExpire() {
        $status = $this->getChannelStatus();
        $expires = $status['expires'];
        if(!$expires){
            return $status['trial_days'];
        }
        return floor(($expires - time())/60/60/24); 
    }

    public function hasValidCertificate()
    {
        $status = cm_get_camera_status($this, TRUE);
        return ($status['status'] == Abc::VALID_SIGNATURE);
    }

    /*
     * Returns some general data for the camera.
     */
    public function getMetaData()
    {
        $document = Core_Model_DocumentFactory::fetch($this->camera_group_id, "camera_group");

        $cameraGroupsDb = new Core_Model_CameraGroups();
        $cg = $cameraGroupsDb->find($this->camera_group_id)->current();
        $cgInfo = array("id" => $cg->camera_group_id, "name" => $cg->name, "description" => $cg->description);

        $folder = $document->getGroup();
        $folderInfo = array("id" => $folder->group_id, "name" => $folder->name, "description" => $folder->description, "path" => $folder->getPath());

        $company = $folder->getCompany();
        $companyInfo = array("id" => $company->company_id, "name" => $company->name, "url" => $company->url, "email" => $company->email, "address" => $company->address, "city" => $company->city, "state" => $company->state, "zip" => $company->zip, "country" => $company->country);

        return array("ip" => $this->ip, "name" => $this->name, "serial" => $this->serial, "registration_code" => $this->regcode, "folder" => $folderInfo, "company" => $companyInfo, "camera_group" => $cgInfo);
    }

    public function getExpiration() {
        if($this->signature) {
            $sig = json_decode($this->signature);
            $expiration = new Zend_Date($sig->data->expires, 'yyyy-MM');
        } else 
            $expiration = NULL;
        return $expiration;
    }


    /*
     * Return an array
     */
    public function getCertificate()
    {
        $first_contact = new Zend_Date($this->first_contact, 'yyyy-MM-dd');

        return array(
            'name' => $this->name,
            'first_contact' => $first_contact->toString('yyyy-MM'),
            'misc' => json_encode($this->getMetaData())
        );
    }

    /*
     * Fetch certificate for camera.
     */
    public function synchronizeChannel()
    {
        set_time_limit(30);
        if(!$this->sync_me)
            return NULL;

        $config = Zend_Registry::get('config');

        $pest = new Pest($config->cogadmin->endpoint);
        $pest->curl_opts[CURLOPT_TIMEOUT] = 3;

        $key = $config->cogadmin->key;
        $serial = $this->serial;

        try {
            $certificate = json_decode($pest->put('/web_reports/' . $key . '/certificates/' . $serial,
                $this->getCertificate()
            ));
        } catch(Pest_Exception $e) {
            throw new WebReport_Exception($this->name . ' did not synchronize since it failed fetching certificate. ' . $e->getMessage());
        }


        /*$last_contact = new Zend_Date($this->last_contact);
        $now = new Zend_Date();
        if(is_null($this->last_contact) || $now->getTimestamp() - $last_contact->getTimestamp() >  7*24*60*60){
            throw new WebReport_Exception($this->name . " did not synchronize since last contact date was over a week ago.");
        }*/
     
        $this->signature = json_encode($certificate->_msg);

        $this->save();

        return $certificate;
    }

    public function __toString()
    {
        return $this->camera_id;
    }

    public function updateDataLastUTCTimestamp($utc_timestamp)
    {
        $this->data_last_utc_timestamp = $utc_timestamp;
        $this->save();
    }

    public function updateLocalTimes($new_first_date, $new_last_date)
    {
        $this->last_contact = new Zend_Db_Expr("UTC_TIMESTAMP()");
        if (isset($new_first_date)) {
            $this->first_date = $this->first_date ? min($this->first_date, $new_first_date) : $new_first_date;
            // Never use an old date
            if ($this->first_date < "1986-01-01 00:00:00") {
                $this->first_date = $new_first_date;
            }
        }
        if (isset($new_last_date)) {
            $this->last_date = $this->last_date ? max($this->last_date, $new_last_date) : $new_last_date;
        }
        $this->save();
    }
}
