<?php

include_once 'Camera.php';

class Core_Model_Cameras extends Talking_Db_Table
{

    protected $_name = 'camera';
    protected $_primary = 'camera_id';
    protected $_rowClass = 'Core_Model_Camera';

    protected $_dependentTables = array('Core_Model_CameraGroups');

    protected $_referenceMap    = array(
        'CameraGroup' => array(
            'columns'           => array('camera_group_id'),
            'refTableClass'     => 'Core_Model_CameraGroups',
            'refColumns'        => array('camera_group_id')
        )
    );

    // $ids is an array
    public function getCameras(Core_Model_Application_Abstract $application = null, Core_Model_Company $company = null, $ids = array(), $limitToCountDataApps = false)
    {
        $db = $this->getAdapter();

        $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
        $cameraGroupsDb = new Core_Model_CameraGroups();

        $select = $db->select()->from(array("cams" => $this->info('name')), array("camera_id", "name", "ip", "application_id", "first_date", "last_date"))
            ->join(array("cg" => $cameraGroupsDb->info('name')), "cams.camera_group_id = cg.camera_group_id", array("group" => "name"))
            ->join(array("ccg" => $companyCameraGroupDb->info('name')), "cg.camera_group_id = ccg.camera_group_id", array());

        if ($company) {
            $select = $select->where('ccg.company_id = ?', $company->company_id);
        }
        if ($application) {
            $select = $select->where('cams.application_id = ?', $application->getId());
        }
        $select = $select->group("cams.camera_id")->order("cams.application_id")
            ->order("cams.name");
        foreach ($ids as $id) {
            $select->where('cams.camera_id = ?', $id);
        }
        $applications = new Core_Model_Applications();
        $cameras = $db->fetchAll($select);
        $filtered = array();
        if ($limitToCountDataApps) {
            foreach ($cameras as $c) {
                $app = $applications->findWithId($c->application_id);

                if ($app instanceof Core_Model_Application_CountDataAbstract) {
                    $filtered[] = $c;
                }
            }
        } else {
            $filtered = $cameras;
        }
        return $filtered;
    }

    public function hasCameras(Core_Model_Application_Abstract $application = null, Core_Model_Company $company = null, $ids = array())
    {
        $db = $this->getAdapter();

        $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
        $cameraGroupsDb = new Core_Model_CameraGroups();

        $select = $db->select()->from(array("cams" => $this->info('name')), array("COUNT(*) > 0 as hasCams"))
            ->join(array("cg" => $cameraGroupsDb->info('name')), "cams.camera_group_id = cg.camera_group_id", array("group" => "name"))
            ->join(array("ccg" => $companyCameraGroupDb->info('name')), "cg.camera_group_id = ccg.camera_group_id", array());

        if ($company) {
            $select = $select->where('ccg.company_id = ?', $company->company_id);
        }
        if ($application) {
            $select = $select->where('cams.application_id = ?', $application->getId());
        }
        foreach ($ids as $id) {
            $select->where('cams.camera_id = ?', $id);
        }
        $cameras = current($db->fetchAll($select));
        return $cameras->hasCams;
    }

    public function getCamera($id)
    {
        if (!$id) {
            throw new WebReport_Exception('NO_CAMERA_WITH_SUCH_ID_EXISTS.');
        }

        $identity = Zend_Auth::getInstance()->getIdentity();
        if (!isset($identity) || $identity->multi_company) {
            $camera = $this->find($id)->current();
        } else {
            $cameraGroupsDb = new Core_Model_CameraGroups();
            $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
            $db = $this->getAdapter();
            $select = $db->select();
            $select->from(array("cams" => $this->info('name')), array("*"))
                ->join(array("cg" => $cameraGroupsDb->info('name')), "cams.camera_group_id = cg.camera_group_id", array())
                ->join(array("ccg" => $companyCameraGroupDb->info('name')), "ccg.camera_group_id = cg.camera_group_id", array())
                ->where("cams.camera_id = ?", $id)
                ->where("ccg.company_id = ?", $identity->company_id);
            $camera = $db->fetchRow($select);
        }

        if (!$camera) {
            throw new WebReport_Exception('NO_CAMERA_WITH_SUCH_ID_EXISTS.');
        }

        return $camera;
    }

    /**
     * Returns camera names
     *
     * @param <type> $application
     * @param <type> $company
     * @return array Associative array with camera ids as keys and names as value
     */
    public function getCameraNames(Core_Model_Application_Abstract $application = null, Core_Model_Company $company = null)
    {
        $ret = array();
        $cameras = $this->getCameras($application, $company);
        foreach ($cameras as $cam) {
            $ret[$cam->camera_id] = $cam->name;
        }

        return $ret;
    }

    /**
     * Splits string and fetch the last time data was saved for that camera.
     *
     * @param string $data separated string, if $separator is ":": <user>:<pass>:<version>:<cameraname>:<serial>
     * @param string $separator Which string seperates fields in data
     * @param boolean $event_data specifies if search for data should be done in event_data table rather
     *        than count_data table. Also returns date in unixtime if true.
     * @return string|null returns null if camera don't exist in database
     * @throws WebReport_Exception if user/pass to table was wrong or if $string is on wrong format
     */
    public static function getLastSavedTimeForSeperatedString($data, $updateLastContact = false, $separator = "##", $event_data = false)
    {
        if(is_string($data)) {
            $data = explode($separator, $data);
        }

        if(is_object($data)) {
            $username   = $data->username;
            $pass       = $data->pass;
            $version    = $data->version;
            $cameraName = $data->cameraName;
            $serial     = $data->serial;
            $fields     = intval($data->fields);
        } else if(is_array($data)){
            $username   = $data[0];
            $pass       = $data[1];
            $version    = $data[2];
            $cameraName = $data[3];
            $serial     = $data[4];
            $fields     = count($data);
        } else {
            throw new Exception("Only array, string and stdObject are accepted");
        }
        if ($version == '3' && $fields != 5) {
            throw new WebReport_Exception("Invalid data. Not in the correct format.");
        }

        if (intval($version) < 3) {
            throw new WebReport_Exception("The version is $version. Should be 3.");
        }

        $countDataTablesDb = new Core_Model_CountData_Tables();
        $cameraGroupsDb = new Core_Model_CameraGroups();
        $cameraGroup = $cameraGroupsDb->fetchRow($cameraGroupsDb->select()->where("username = ?", $username));

        if (!$cameraGroup) {
            throw new WebReport_Exception("No camera group with username '$username'.");
        } elseif (md5(STATIC_SALT . $pass . $cameraGroup->password_salt) != $cameraGroup->password) {
            throw new WebReport_Exception('Invalid username for camera group "' . $cameraGroup->name . '" (user ' . $username . ').');
        }

        //Just a thought...
        //If the database is busy this call might fail and $max would not be set
        //This would trigger the camera to start resending data all over
        try
        {
            $camerasDb = new Core_Model_Cameras();

            $select = $camerasDb->select()->setIntegrityCheck(false)
                            ->from($camerasDb)
                            ->where('name = ?', $cameraName)
                            ->where('camera_group_id = ?', $cameraGroup->camera_group_id);
            $camera = $camerasDb->fetchRow($select);
            if (!$event_data) {
                $max = $camera ? $camera->last_date : false;
            } else {
                $max = $camera ? $camera->data_last_utc_timestamp : false;
            }
        } catch(Exception $e) {
            throw new WebReport_Exception('Database is busy or other database error');
        }

        if($camera) {
            Core_Model_Cameras::updateLastCount(intval($camera->camera_id));
        }

        if (!$max) {
            if ($event_data) {
                $date = 0;
            } else {
                $date = date("Y", time() - 31536000) . "0101000000";
            }
            return $date;
        }

        $latestCount = $max;

        $date = '';
        if ($latestCount) {
            if (!$event_data) {
                $latestDate = new Zend_Date();
                $latestDate->set($latestCount);
                $latestDate->setDate($latestCount, "yyyyMMddHHmmss");
                // Convert time from TVWR time to TVPC time (i.e. date identifier is end of interval, not start)
                // This should really be done on the camera instead.
                if($camera->application_id == 1) $latestDate->add('00:15:00',Zend_Date::TIMES);
                if($camera->application_id == 11) $latestDate->add('00:15:00',Zend_Date::TIMES);
                
                //$latestDate = new Zend_Date($latestCount, "yyyyMMddHHmmss"); // a little more simple version?
                
                if ( ($latestDate->get(Zend_Date::MINUTE) % 15) != 0 || $latestDate->get(Zend_Date::SECOND) != 0) {
                    $latestDate->sub($latestDate->get(Zend_Date::MINUTE) % 15, Zend_Date::MINUTE);
                    $latestDate->set(0, Zend_Date::SECOND);
                }
                $date = $latestDate->toString("yyyyMMddHHmmss");

                //This is probably unecessary as it's done above, commenting. AG 2015-02-09
                //Core_Model_Cameras::updateLastCount(intval($camera->camera_id));
            } else {
                $date = $latestCount;
            }
        } else {
            $date = null;
        }

        return $date;
    }

    public static function getSerialFromData($data) {
        return substr($data[4],0,12);
    }

    public static function getRegCodeFromData($data) {
        $keys = explode("::",$data[4]);
        return count($keys) > 1 ? $keys[1] : "?";
    }

    public static function getStatsFromData($data) {
        $res = array();
        if(!isset($data[6])) return $res;
        $n = intval($data[6]);
        for($i = 0; $i < $n; $i++) {
            if($i > 10){
                 throw new WebReport_Exception("Way too many fields in indata!");
            }
            $key = $data[7+$i*2];
            $value = $data[7+$i*2+1];
            $key = is_numeric($key) ? intval($key) : $key;
            $res[$key] = intval($value);
        }
        return $res;
    }

    public static function updateLastCount($id)
    {
        $date = new Zend_Db_Expr("UTC_TIMESTAMP()");
        $db = new Core_Model_Cameras();
        $db->update(
            array('last_contact' => $date),
            array('camera_id = ?'=>$id)
        );
    }

    public static function addImage($username, $pass, $cameraName, $serial, $time, $jpeg)
    {
        $cameraGroupsDb = new Core_Model_CameraGroups();
        $cameraGroup = $cameraGroupsDb->fetchRow($cameraGroupsDb->select()->where("username = ?", $username));

        if (!$cameraGroup) {
            throw new WebReport_Exception("No camera group with username '$username'.");
        } elseif (md5(STATIC_SALT . $pass . $cameraGroup->password_salt) != $cameraGroup->password) {
            throw new WebReport_Exception('Invalid username for camera group "' . $cameraGroup->name . '" (user ' . $username . ').');
        }

        if (strtotime($time) === false) {
            throw new WebReport_Exception("Invalid date and time sent, expecting yyyy-mm-dd HH:MM:SS");
        }

        $camerasDb = new Core_Model_Cameras();

        $select = $camerasDb->select()
            ->where('name = ?', $cameraName)
            ->where('camera_group_id = ?', $cameraGroup->camera_group_id)
            ->order('last_date DESC')
            ->limit(1, 0);

        $camera = $camerasDb->fetchRow($select);
        if (!$camera) {
            throw new WebReport_Exception("No camera found with serial " . $serial . " in camera group with user " . $username);
        }

        $uploadDir = realpath(APPLICATION_PATH . "/../public/resources/camera");
        if(count($uploadDir) == 0) {
            throw new WebReport_Exception('path does not exist:' . APPLICATION_PATH . "/../public/resources/camera");
        }
        $fullPath = '';
        $filename = '';
        do {
            // random file name
            $filename = preg_replace("/([0-9])/e", "chr((\\1+112))", rand(10000000000, 99999999999)) . ".jpg";
            $fullPath = $uploadDir . "/" . $filename;
        } while (file_exists($fullPath));

        $fh = fopen($fullPath, 'w');
        if ($fh === FALSE) {
            throw new WebReport_Exception('Could not fopen file: ' . $fullPath . ' for camera with id: ' . $camera->camera_id);
        }

        $ret = fwrite($fh, $jpeg);
        if ($ret === FALSE) {
            throw new WebReport_Exception('Could not write file: ' . $fullPath . ' for camera with id: ' . $camera->camera_id);
        }

        fclose($fh);

        $time = date('Y-m-d H:i:s', strtotime($time));

        $imagesDb = new Core_Model_CameraImages();
        $imageData = array(
            "camera_id" => $camera->camera_id,
            "time" => $time,
            "filename" => $filename,
        );

        $image = $imagesDb->createRow($imageData);
        $image->save();
    }

    public function fetchAllEditable($user)
    {
        $groupsDb = new Core_Model_Folders();
        $groups = $groupsDb->getEditableGroups($user);

        $groupIds = array(0);
        foreach($groups as $group)
            $groupIds[] = $group->group_id;

        $select = $this->select();
        $select->setIntegrityCheck(false);
        $select->from('camera')
            ->join('camera_group', 'camera_group.camera_group_id = camera.camera_group_id',array())
            ->join('document', 'document.id = camera_group.camera_group_id',array())
            ->where('document.type = ?', 'camera_group')
            ->where('document.group_id IN (?)', $groupIds);

        return $this->fetchAll($select);;
    }

    public function fetchAllVisible($user, $application = null)
    {
        $groupsDb = new Core_Model_Folders();
        $groups = $groupsDb->getViewableGroups($user);

        $groupIds = array(0);
        foreach($groups as $group)
            $groupIds[] = $group->group_id;
        if($application !== null && !is_int($application))
            $application = $application->getId();
        $select = $this->select();
        $select->setIntegrityCheck(false);
        $select->from('camera')
            ->join('camera_group', 'camera_group.camera_group_id = camera.camera_group_id',array())
            ->join('document', 'document.id = camera_group.camera_group_id',array())
            ->where('document.type = ?', 'camera_group')
            ->where('document.group_id IN (?)', $groupIds);
        if($application) $select = $select->where('camera.application_id = (?)', $application);
        return $this->fetchAll($select);
    }

    public function fetchAllVisibleNames($user, $application = NULL)
    {
        $cameras = $this->fetchAllVisible($user);

        $ret = array();
        foreach($cameras as $cam) {
            if(!is_null($application)) {
                if($cam->application_id == $application->getId())
                    $ret[$cam->camera_id] = $cam->name;
            }
            else
                $ret[$cam->camera_id] = $cam->name;
        }
        return $ret;
    }

    public function fetchAllEditableNames($user, $application = NULL)
    {
        $cameras = $this->fetchAllEditable($user);

        $ret = array();
        foreach($cameras as $cam) {
            if(!is_null($application)) {
                if($cam->application_id == $application->getId())
                    $ret[$cam->camera_id] = $cam->name;
            }
            else
                $ret[$cam->camera_id] = $cam->name;
        }
        return $ret;  
    }

    public function fetchActiveCameras()
    {
        return $this->fetchAll($this->select()->where('sync_me = TRUE'));
    }


    public function synchronizeCameras() {

        $logger = Zend_Registry::get('loggers')->getLogger('synchronization');
        $cameras = $this->fetchActiveCameras();
        $synced = 0;
        $skipped = 0;
        $newCameras = 0;        
        foreach($cameras as $camera) {
            try{
                $isNew = ($camera->signature == NULL);
                $cert = $camera->synchronizeChannel();
                if($isNew) 
                    $newCameras++;
                $synced++;
            }catch(WebReport_Exception $e){
                $logger->err($e->getMessage());
                $skipped++;
            }
        }
        if($skipped>0){
            $logger->err("Synchronized " . $synced . " cameras, " . $newCameras . " for the first time. Skipped " . $skipped . " cameras");
        }else{
            $logger->notice("Synchronized " . $synced . " cameras, " . $newCameras . " for the first time.");
        }
        return array('status', 'ok', 'synced' => $synced, 'newCameras' => $newCameras, 'skipped' => $skipped);
    } 

    public function searchCameras($string, $limit=10, $user=NULL, $folder_id=NULL)
    {
        if(strlen($string)<1 && is_null($folder_id))
            return array();
        if(is_null($user))
            $user = Zend_Auth::getInstance()->getIdentity();
        $groupsDb = new Core_Model_Folders();
        $groups = $groupsDb->getViewableGroups($user);

        $groupIds = array(0);
        foreach($groups as $group)
            $groupIds[] = $group->group_id;

        $likes = array();
        foreach(explode(' ', $string) as $word) {
            $word = trim($word);

            if(strlen($word))
                $likes[] = "(UPPER(camera.name) LIKE UPPER('%$word%'))";
        }
        $like = implode(' AND ', $likes);

        $db = $this->getAdapter();
        $select = $this->select()->SetIntegrityCheck(false)
               ->from("camera",array('name as cameraname', "camera_id", "application_id"))
               ->join('camera_group','camera.camera_group_id = camera_group.camera_group_id' )
               ->join('document','camera_group.camera_group_id = document.id')
               ->join('group','group.group_id = document.group_id')
               ->join('company','group.company_id = company.company_id')
               ->where("type='camera_group'")
               ->where("company.active = 1")
               ->where('document.group_id IN (?)', $groupIds)
               ->limit($limit);
        if(count($likes))
            $select->where($like);

        if(!is_null($folder_id) && $folder_id != "all")
            $select->where("group.group_id = ?", $folder_id);

        $res=array();
        foreach($this->fetchAll($select) as $item) {
            $ar = array(
                'type' => 'camera',
                'group_id' => $item->group_id,
                'name' => $item->cameraname,
                'id' => $item->camera_id,
                'application_id' => $item->application_id,
                'extra' => array('id' => $item->camera_group_id)
            );
            $res[] = (object)$ar;
        }
        return $res;
    }
    public function nbrOfCameras(){
        $select = $this->select()
                       ->from($this, array("unique" => "COUNT(DISTINCT serial)"));
        return $this->fetchRow($select)->unique;
    }
    private function verify_fields_in_array($required_fields, $data){
        foreach($required_fields as $key){
            if(!isset($data[$key])){
                error_log("Missing argument '$key' in " . debug_backtrace()[1]['function']);
                throw new Exception("Missing required argument '$key'");
            }
        }
    }

    public function createCamera($data, $cameraGroup = null)
    {
        $req_fields = array('name', 'application_id', 'serial', 'regcode', 'time');
        if (!isset($cameraGroup)) {
            $req_fields[] = 'username';
            $req_fields[] = 'password';
        }
        $this->verify_fields_in_array(
            $req_fields,
            $data
        );
        $logger = Zend_Registry::get('loggers')->getLogger(WebReport_Log::COUNT_DATA_UPLOAD);
        $logger->notice("Creating new camera '" . $data['name'] . "'");
        error_log("Creating new camera '" . $data['name'] . "'");

        $features = new Core_Model_Features();
        $nbrOfCameras = $this->nbrOfCameras();
        $allowed = $features->nbrOfCamerasAllowed();
        if ($nbrOfCameras >= $allowed && $allowed > 0) {
            $warnType = Core_Model_Warning::CAMERA_NO_CHANNELS_LEFT;
            Admin_Plugin_WarningsLoader::tryCreateWarning($warnType, null, null);
            throw new WebReport_Exception("Too many cameras connected.");
        }

        if (array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)) {
            $IParray=array_values(array_filter(explode(',',$_SERVER['HTTP_X_FORWARDED_FOR'])));
            $CAM_IP=reset($IParray);
        } else if (array_key_exists('REMOTE_ADDR', $_SERVER)) {
            $CAM_IP=$_SERVER["REMOTE_ADDR"];
        } else {
            $CAM_IP="";
        }
        $IP_clean = filter_var($CAM_IP, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4);
        if (!strlen($IP_clean))
            $IP_clean = "Unknown";
        $data['ip'] = $IP_clean;
        if (!isset($cameraGroup)) {
            $cameraGroupsDb = new Core_Model_CameraGroups();
            $cameraGroup = $cameraGroupsDb->fetchCameraGroup(
                $data['username'],
                $data['password']
            );
        }
        $data['camera_group_id'] = $cameraGroup->camera_group_id;
        $data['first_date'] = date('Y-m-d H:i:s', $data['time']);

        $start = time();
        $camera = $this->createRow($data);
        $camera->save(); // creates new count data table if necessary as well
        $end = time();
        $tableName = $camera->getCountDataTableName();
        set_time_limit(600); // Prevent mutex->acquire() from timing out easily
        $time = $end - $start;
        if ($time > 60) {
            $logger->warn('Added camera "' . $data['name'] . '" in ' . $time . " seconds");
        } else {
            $logger->notice('Added camera "' . $data['name'] . '" in ' . $time . " seconds");
        }
        return $camera;

    }

    public function fetchCamera($data, $cameraGroup = null)
    {
        $req_fields = array('name');
        if (!isset($cameraGroup)) {
            $req_fields[] = 'username';
            $req_fields[] = 'password';
        }
        $this->verify_fields_in_array(
            $req_fields,
            $data
        );
        if (!isset($cameraGroup)) {
            $cameraGroupsDb = new Core_Model_CameraGroups();
            $cameraGroup = $cameraGroupsDb->fetchCameraGroup(
                $data['username'],
                $data['password']
            );
        }
        $select = $this->select()
                       ->setIntegrityCheck(false)
                       ->from($this, "*")
                       ->where("camera_group_id = ?",
                           empty($cameraGroup->camera_group_id) ?
                                "NULL" : $cameraGroup->camera_group_id)
                       ->where("name = ?", $data['name'])
                       ->order("last_date DESC")
                       ->limit(1);


        return $this->fetchRow($select);
    }



    public function fetchOrCreateCamera($data, $cameraGroup = null)
    {
        $req_fields = array('name', 'application_id', 'serial', 'regcode', 'time');
        if (!isset($cameraGroup)) {
            $req_fields[] = 'username';
            $req_fields[] = 'password';
        }
        $this->verify_fields_in_array(
            $req_fields,
            $data
        );

        $camera = $this->fetchCamera($data, $cameraGroup);
        if (!$camera) {
            $camera = $this->createCamera($data, $cameraGroup);
        }
        return $camera;
    }

}

