<?php

include_once 'Company.php';
include_once 'CompaniesRowset.php';

class Core_Model_Companies extends Talking_Db_Table
{
	protected $_name = 'company';
	protected $_primary = 'company_id';
	protected $_rowClass = 'Core_Model_Company';
	protected $_rowsetClass = 'Core_Model_CompaniesRowset';

	protected $_dependentTables = array('Users','LogMessages','AclRoles', 'Core_Model_Folders');

	protected $_referenceMap    = array(
		'Owner' => array(
			'columns'           => array('owner'),
			'refTableClass'     => 'Core_Model_Users',
			'refColumns'        => array('user_id')
		),
			'CreatedBy' => array(
			'columns'           => array('created_by'),
			'refTableClass'     => 'Core_Model_Users',
			'refColumns'        => array('user_id')
		),
			'CreatedBy' => array(
			'columns'           => array('company_id'),
			'refTableClass'     => 'Core_Model_CountData_TimeFilters',
			'refColumns'        => array('company_id')
		)
	);

    public static function getCompany($id){
        $db = new Core_Model_Companies();
        return $db->find($id)->current();
    }


	public function inactivate($companyIds) {
		$companies = $this->find($companyIds);
		if (count($companies) == 0) {
			return false;
		}

		foreach ($companies as $company) {
			if ($company)
			$company->inactivate();
		}
		return true;
	}



	public function activate($companyIds)
	{
		$companies = $this->find($companyIds);
		if (count($companies) == 0) {
			return false;
		}

		foreach ($companies as $company) {
			if ($company)
			$company->activate();
		}
		return true;
	}


	public function fetchAllActive()
	{
		$select = $this->select()->where('active = ?', 1)->order('UPPER(name)');
		return $this->fetchAll($select);
	}

	public function fetchAllInactive()
	{
		$select = $this->select()->where('active = ?', 0)->order('UPPER(name)');
		return $this->fetchAll($select);
	}

	public function fetchWithName($name)
	{
		$select = $this->select()->where('name = ?', $name);
		return $this->fetchRow($select);
	}

	public function createCompanyAndSave($companyData)
	{
		$company = $this->createRow($companyData);
		$company->save();

		$companyId = $company->company_id;

		$aclRolesDb = new Core_Model_AclRoles();
		$aclRolesDb->createRolesWithStandardPrivilegesForCompany($companyId);

		$modulesDb = new Core_Model_Modules();
		$moduleCompanyDb = new Core_Model_ModuleCompany();
		$modules = $modulesDb->fetchAll();
		foreach ($modules as $module) {
			$row = $moduleCompanyDb->createRow(array(
				'module_id' => $module->module_id,
				'company_id' => $companyId
			));
			$row->save();
		}

		return $company;
	}
        
}
