<?php

include_once 'CountData/FormulaSetFormulas.php';

class Core_Model_CountData {

    const TYPE_PEDESTRIANS = 0;
    const TYPE_PEDESTRIANS_GOING_IN = 3;
    const TYPE_PEDESTRIANS_GOING_OUT = 4;
    const TYPE_NUMBER_OF_RUNS_OF_DETECTOR_IN_THE_CURRENT_TIME_INTERVAL = 7;
    const TYPE_PEDESTRIANS_WITHIN_SPECIFIED_AREA = 8;
    const TYPE_AVERAGE_LOITER_TIME_IN_SPECIFIED_AREA_IN_SECONDS = 9;
    const TYPE_BICYCLES_GOING_IN = 34;
    const TYPE_BICYCLES_GOING_OUT = 35;
    const TYPE_CARS_GOING_IN = 36;
    const TYPE_CARS_GOING_OUT = 37;
    const TYPE_QUEUE_HIGH_FLOW = 300;
    const TYPE_QUEUE_MID_FLOW = 301;
    const TYPE_QUEUE_LOW_FLOW = 302;
    const TYPE_QUEUE_PEOPLE = 310;
    const TYPE_QUEUE2_PEOPLE_IN_QUEUE = 320;
    const TYPE_QUEUE2_UPPER_LIMIT = 321;
    const TYPE_QUEUE2_LOWER_LIMIT = 322;
    const TYPE_GENDER_MALE_DETECTION = 46;
    const TYPE_GENDER_FEMALE_DETECTION = 47;
    const TYPE_OCCUPANCY = 64;
    const TYPE_AVERAGE_VISIT_TIME = 65;

    const TYPE_POS_SALE = 500;
    const TYPE_POS_TRANSACTION = 510;
    const TYPE_POS_CONVERSION_RATE = 520;
    const TYPE_POS_AVERAGE_BASKET_VALUE = 511;
    const TYPE_POS = 599;
    const TYPE_DEMOGRAPHICS_MALE_0_TO_14     = 601;
    const TYPE_DEMOGRAPHICS_MALE_15_TO_24    = 602;
    const TYPE_DEMOGRAPHICS_MALE_25_TO_34    = 603;
    const TYPE_DEMOGRAPHICS_MALE_35_TO_44    = 604;
    const TYPE_DEMOGRAPHICS_MALE_45_TO_54    = 605;
    const TYPE_DEMOGRAPHICS_MALE_55_TO_64    = 606;
    const TYPE_DEMOGRAPHICS_MALE_65_TO_74    = 607;
    const TYPE_DEMOGRAPHICS_MALE_75_TO_INF   = 608;
    const TYPE_DEMOGRAPHICS_FEMALE_0_TO_14     = 651;
    const TYPE_DEMOGRAPHICS_FEMALE_15_TO_24    = 652;
    const TYPE_DEMOGRAPHICS_FEMALE_25_TO_34    = 653;
    const TYPE_DEMOGRAPHICS_FEMALE_35_TO_44    = 654;
    const TYPE_DEMOGRAPHICS_FEMALE_45_TO_54    = 655;
    const TYPE_DEMOGRAPHICS_FEMALE_55_TO_64    = 656;
    const TYPE_DEMOGRAPHICS_FEMALE_65_TO_74    = 657;
    const TYPE_DEMOGRAPHICS_FEMALE_75_TO_INF   = 658;



    // Number of thousands of data points recommended for one getData()
    // Around 10-16 seem to be the maximum limit if the allowed memory is at 32MB.
	// Trying 32 for 128
    const MAX_ALLOWED_LOAD = 32;

    protected $_formulaSets = array();
    protected $_formulaSetsTypes = array();
    protected $_filterGroups = array();
    protected $_virtualCameras = array();
    protected $_virtualCamerasTypes = array();
    protected $_cameras = array();
    protected $_camerasTypes = array();
    protected $_applications = array();
    protected $_applicationsCompanies = array();
    protected $_lastFetchedKeys = array();
    protected $_graphOptionsForLastFetchedKeys = array();
    protected $_selectAfterInit = null;
    protected $_cameraIdsInvolvedInSelectAfterInit = array();
    protected $_countDataTablesToSelectFromAfterInit = array();
    protected $_firstDate = null;
    protected $_lastDate = null;
    protected $_applicationsUsed = array();
    protected $_countDataTables = array();
    protected $_time_mask_start = null;
    protected $_time_mask_stop = null;
    protected $_sumDataSeries = False;
    protected $_noFields = False;

    public function getFormulaSets() {
        return $this->_formulaSets;
    }

    public function isEmpty() {
        return count($this->_cameras+
                     $this->_formulaSets+
                     $this->_virtualCameras
                     ) == 0;
    }

    public function hasFields() {
        return !$this->_noFields;
    }

    public function getFormulaSetTypes() {
        return $this->_formulaSetsTypes;
    }

    public function getVirtualCameras() {
        return $this->_virtualCameras;
    }

    public function getVirtualCameraTypes() {
        return $this->_virtualCamerasTypes;
    }

    public function getFilterGroups() {
        return $this->_filterGroups;
    }

    public function getCameras() {
        return $this->_cameras;
    }

    public function getCamerasTypes() {
        return $this->_camerasTypes;
    }

    public function getSelect() {
        return $this->_selectAfterInit;
    }

    public function addCountData(Core_Model_CountData $merge, $type = NULL,
            $mergeCameras = True, $mergeVirtualCameras = True,
            $mergeFormulas = False, $mergeFilterGroups = True,
            $mergeApplications = False) {
        if(!is_null($type) && !is_array($type)) {
            $type = array($type);
        }

        if ($mergeCameras) {
            $cameras = $merge->getCameras();
            $cameraTypes = $merge->getCamerasTypes();

            foreach ($cameras as $index => $camera) {

                // check if this camera needs to be added
                $addCamera = True;
                foreach ($this->_cameras as $cameraIndex => $setCamera) {
                    if ($setCamera->camera_id == $camera->camera_id) {
                        $addCamera = False;
                        $cameraType = is_null($type) ? $cameraTypes[$index] : $type;
                        $this->_camerasTypes[$cameraIndex] = array_unique(array_merge($this->_camerasTypes[$cameraIndex] ,$cameraType));
                        break;
                    }
                }
                if ($addCamera) {
                    $this->addCamera($camera, !is_null($type) ? $type : $cameraTypes[$index]);
                } else {
                }
            }
        }


        if ($mergeVirtualCameras) {
            $virtualCameras = $merge->getVirtualCameras();
            $virtualCameraTypes = $merge->getVirtualCameraTypes();

            foreach ($virtualCameras as $index => $virtualCamera) {
                $addVirtualCamera = True;
                foreach ($this->_virtualCameras as $virtualCameraIndex => $setVirtualCamera) {
                    if ($virtualCamera->virtual_camera_id == $virtualCamera->virtual_camera_id) {
                        $addVirtualCamera = False;
                        $virtualCameraType = is_null($type) ? $virtualCameraTypes[$index] : $type;
                        $this->_virtualCamerasTypes[$virtualCameraIndex] =
                                array_unique($this->_virtualCamerasTypes[$virtualCameraIndex] + $virtualCameraType);
                        break;
                    }
                }
                if ($addVirtualCamera)
                    $this->addVirtualCamera($virtualCamera, !is_null($type) ? $type : $virtualCameraTypes[$index]);
            }
        }

        if ($mergeFormulas) {
            $formulas = $merge->getFormulaSets();
            $formulaTypes = $merge->getFormulaSetTypes();

            foreach ($formulas as $index => $formula) {
                $this->addFormulaSet($formula, $formulaTypes[$index]);
            }
        }

        if ($mergeFilterGroups)
            $this->addFilterGroups($merge->getFilterGroups());

        if ($mergeApplications)
            throw new WebReport_Error("NOT_IMPLEMENTED only needed when there are other applications and that is not supported.");

        return $this;
    }

    public function setSumDataSeries($bool) {
        $this->_sumDataSeries = $bool;
        return $this;
    }

    public function addFormulaSets($formulaSets) {
        foreach ($formulaSets as $formulaSet) {
            $this->addFormulaSet($formulaSet);
        }
        return $this;
    }

    public function addFormulaSet(Core_Model_CountData_FormulaSet $formulaSet, $types = array()) {
        $this->_formulaSets[] = $formulaSet;
        $this->_formulaSetsTypes[] = $types;
        return $this;
    }

    public function addVirtualCameras($virtualCameras, $types = array()) {
        foreach ($virtualCameras as $camera) {
            $this->addVirtualCamera($camera, $types);
        }
    }

    public function addVirtualCamera($virtualCamera, $types = array()) {
        $this->_virtualCameras[] = $virtualCamera;
        $this->_virtualCamerasTypes[] = $types;
        return $this;
    }

    public function addFilterGroups($filterGroups) {
        foreach ($filterGroups as $group) {
            $this->addFilterGroup($group);
        }
        return $this;
    }

    public function addFilterGroup(Core_Model_CountData_FilterGroup $filterGroup) {
        $this->_filterGroups[] = $filterGroup;
        return $this;
    }

    public function addCameras($cameras, $types = array()) {
        foreach ($cameras as $camera) {
            $this->addCamera($camera, $types);
        }
        return $this;
    }

    public function addCamera($camera, $types = array()) {
        $this->_cameras[] = $camera;
        $this->_camerasTypes[] = $types;
        return $this;
    }

    public function addApplication(Core_Model_Application_Abstract $application, Core_Model_Company $company) {
        $camerasDb = new Core_Model_Cameras();
        throw new WebReport_Exception("Not yet implemented");
        $this->_applications[] = $application;
        $this->_applicationsCompanies[] = $company;
        return $this;
    }

    public function getNumberOfSources() {
        return count($this->_cameras) + count($this->_filterGroups) + count($this->_virtualCameras) + count($this->_formulaSets);
    }

    public function initDatabaseSelect($skip_meta_data = false) {
        $db = Zend_Registry::get('db');
        $select = $db->select();

        $camerasDb = new Core_Model_Cameras();
        $applicationsModel = new Core_Model_Applications();
        $cameraGroupsDb = new Core_Model_CameraGroups();
        $companiesDb = new Core_Model_Companies();
        $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
        $virtualCamerasDb = new Core_Model_VirtualCameras();
        $virtualCamerasFormulasDb = new Core_Model_VirtualCameraFormula();

        $fieldsToSelect = array();
        $fieldsToNullCheck = array();
        $countDataTablesIds = array();

        $cameraIdsInvolved = array();


        foreach ($this->_filterGroups as $filterGroup) {
            $application = $applicationsModel->findWithId($filterGroup->application_id);
            $this->_applicationsUsed[$application->getId()] = $application;

            $inTypes = $application->getInTypes();
            $outTypes = $application->getOutTypes();

            $company = $companiesDb->find($filterGroup->company_id)->current();
            $cameras = $camerasDb->getCameraNames($application, $company);
            $allCameraIds = array_keys($cameras);

            $filters = $filterGroup->getFilters();


            foreach ($filters as $filter) {
                $typesToInclude = array();
                $typesToExclude = array();
		$removeThese = array();
                foreach ($filter->getParts() as $part) {
                    if ($part instanceof Core_Model_CountData_Filter_Part_CameraName) {
                        $cameraName = current($part->getCurrentConditionValue());
                        $condition = $part->getCurrentCondition();

                        switch ($condition) {
                            case Core_Model_CountData_Filter_Part_CameraName::CONDITION_CONTAINS:
                                foreach ($cameras as $cameraId => $camera)
                                    if (!(strpos($camera, $cameraName) !== false))
					$removeThese[] = $cameraId;
                                break;
                            case Core_Model_CountData_Filter_Part_CameraName::CONDITION_DOES_NOT_CONTAIN:
                                foreach ($cameras as $cameraId => $camera)
                                    if (strpos($camera, $cameraName) !== false)
					    $removeThese[] = $cameraId;
                                break;
                            case Core_Model_CountData_Filter_Part_CameraName::CONDITION_IS:
                                foreach ($cameras as $cameraId => $camera)
                                    if ($camera != $cameraName)
					$removeThese[] = $cameraId;
                                break;
                            case Core_Model_CountData_Filter_Part_CameraName::CONDITION_IS_NOT:
                                foreach ($cameras as $cameraId => $camera)
                                    if ($camera == $cameraName)
			                $removeThese[] = $cameraId;
                                break;
                        }
                    } else if ($part instanceof Core_Model_CountData_Filter_Part_CameraId) {
			throw new Exception("Core_Model_CountData_Filter_Part_CameraId is not implemented.");
			/*
                        $cameraId = current($part->getCurrentConditionValue());
                        $condition = $part->getCurrentCondition();

                        switch ($condition) {
                            case Core_Model_CountData_Filter_Part_CameraId::CONDITION_IS:
                                if (isset($cameras[$cameraId])) {
                                    $cameraIdsToInclude[] = $cameraId;
                                }
                                break;
                            case Core_Model_CountData_Filter_Part_CameraId::CONDITION_IS_NOT:
                                if (isset($cameras[$cameraId])) {
				    $removeThese[] = $cameraId;
                                }
                                break;
                        }

			 */
                    } else if ($part instanceof Core_Model_CountData_Filter_Part_CountType) {
                        $countType = current($part->getCurrentConditionValue());
                        $condition = $part->getCurrentCondition();

                        switch ($condition) {
                            case Core_Model_CountData_Filter_Part_CountType::CONDITION_IS:
                                $typesToInclude[] = $countType;
                                break;
                            case Core_Model_CountData_Filter_Part_CountType::CONDITION_IS_NOT:
                                $typesToExclude[] = $countType;
                                break;
                        }
                    }
                }
		array_unique($removeThese);

		$cameraIds = array_diff(array_keys($cameras),$removeThese);

                $types = array_keys($outTypes);

                if (empty($typesToInclude) && empty($typesToExclude)) {
                    $types = $types;
                } elseif (empty($typesToExclude)) {
                    $types = $typesToInclude;
                } elseif (empty($typesToInclude)) {
                    $types = array_diff($types, $typesToExclude);
                }

                foreach ($cameraIds as $cameraId) {
                    foreach ($types as $cntType) {
                        $sql = $application->getSqlQueryForCntOutType($cntType, $cameraId);
                        $fieldsToSelect["c" . $cameraId . "_" . $cntType] = "COALESCE($sql,0)";
                        $fieldsToNullCheck["c" . $cameraId . "_" . $cntType."_exists"] = "IF(ISNULL($sql),0,1)";
                        $cameraIdsInvolved[] = $cameraId;
                    }
                }
            }
        }
        foreach ($this->_cameras as $arrayIndex => $cam) {
            $cameraId = $cam->camera_id;
            if(is_null($cameraId)){
                r_log("Warning: trying to get camera that is null");
                continue;
            }
	        $camera = $camerasDb->find($cameraId)->current();
            $application = $applicationsModel->findWithId($camera->application_id);
            $this->_applicationsUsed[$application->getId()] = $application;
            $inTypes = $application->getInTypes();

            $outTypes = array();
            if (empty($this->_camerasTypes[$arrayIndex])) {
                $outTypes = array_keys($application->getOutTypes());
            } else {
                $outTypes = $this->_camerasTypes[$arrayIndex];
            }
            $cameraId = $camera->camera_id;
            foreach ($outTypes as $outType) {
                $sql = $application->getSqlQueryForCntOutType($outType, $camera);
                $fieldsToSelect["c" . $cameraId . "_" . $outType] = "COALESCE($sql,0)";
                $fieldsToNullCheck["c" . $cameraId . "_" . $outType."_exists"] = "IF(ISNULL($sql),0,1)";
            }

            $cameraIdsInvolved[] = $camera->camera_id;
        }

        foreach ($this->_virtualCameras as $arrayIndex => $virtualCamera) {
            $application = $applicationsModel->findWithId($virtualCamera->application_id);

            $virtualCameraSelect = $virtualCamerasFormulasDb->select()->where('virtual_camera_id = ?', $virtualCamera->virtual_camera_id);

            if (empty($this->_virtualCamerasTypes[$arrayIndex])) {
                $typesToSelect = array_keys($application->getOutTypes());
            } else {
                $typesToSelect = $this->_virtualCamerasTypes[$arrayIndex];
                $arr = array();
                foreach ($typesToSelect as $t) {
                    $arr[] = 'count_out_type = ' . $virtualCamerasDb->getAdapter()->quote($t);
                }
                $virtualCameraSelect->where("(" . implode(" OR ", $arr) . ")");
            }

            $formulas = $virtualCamerasFormulasDb->fetchAll($virtualCameraSelect);
            $inTypes = array();

            $inTypes = $application->getInTypes();
            $this->_applicationsUsed[$application->getId()] = $application;

            foreach ($formulas as $formula) {
                $cameras = array();
                $outTypes = array();

                $f = new WebReport_Formula($formula->formula);
                $cams = $f->getCameraIdsAndOutTypesInvolved();
                foreach ($cams as $cameraId => $types) {
                    if (!isset($cameras[$cameraId])) {
                        $cameras[$cameraId] = $types;
                    } else {
                        foreach ($types as $type) {
                            if (!in_array($type, $cameras[$cameraId])) {
                                $cameras[$cameraId][] = $type;
                            }
                        }
                    }
                }

                $fieldsToReplace = array();
                $fieldsToReplaceNullCheck = array();

                foreach ($cameras as $cameraId => $out) {
                    foreach ($out as $o) {
                        $sql = $application->getSqlQueryForCntOutType($o, $cameraId, true);
                        $fieldsToReplace["c" . $cameraId . "_" . $o] = "COALESCE($sql, 0)";
                        $fieldsToReplaceNullCheck["c" . $cameraId . "_" . $o] = "IF(ISNULL($sql),0,1)";
                        $cameraIdsInvolved[] = $cameraId;
                    }
                }

                $printableFormula = $formula->formula;
                $fieldsToSelect["v" . $virtualCamera->virtual_camera_id . "_" . $formula->count_out_type] = str_replace(array_keys($fieldsToReplace), array_values($fieldsToReplace), $printableFormula);
                $fieldsToSelectNullCheck["v" . $virtualCamera->virtual_camera_id . "_" . $formula->count_out_type . "_exists"] = str_replace(array_keys($fieldsToReplaceNullCheck), array_values($fieldsToReplaceNullCheck), $printableFormula);
            }
        }

        foreach ($this->_formulaSets as $arrayIndex => $formulaSet) {
            $cameras = array();
            $inTypes = array();
            $outTypes = array();

            $application = $applicationsModel->findWithId($formulaSet->application_id);

            $inTypes = $application->getInTypes();
            $this->_applicationsUsed[$application->getId()] = $application;

            $formulas = $formulaSet->findDependentRowset('Core_Model_CountData_FormulaSetFormulas');
            foreach ($formulas as $formula) {
                if(count($this->_formulaSetsTypes[$arrayIndex]) > 0
                        && !in_array($formula->count_data_formula_set_formula_id, $this->_formulaSetsTypes)) {
                    continue;
                }

                $cameras = array();
                $outTypes = array();

                $f = new WebReport_Formula($formula->formula);
                $cams = $f->getCameraIdsAndOutTypesInvolved();
                foreach ($cams as $cameraId => $types) {
                    if (!isset($cameras[$cameraId])) {
                        $cameras[$cameraId] = $types;
                    } else {
                        foreach ($types as $type) {
                            if (!in_array($type, $cameras[$cameraId])) {
                                $cameras[$cameraId][] = $type;
                            }
                        }
                    }
                }

                $fieldsToReplace = array();
                $fieldsToReplaceNullCheck = array();

                foreach ($cameras as $cameraId => $out) {
                    foreach ($out as $o) {
                        $sql = $application->getSqlQueryForCntOutType($o, $cameraId, true);
                        $fieldsToReplace["c" . $cameraId . "_" . $o] = "COALESCE($sql, 0)";
                        $fieldsToReplaceNullCheck["c" . $cameraId . "_" . $o] = "IF(ISNULL($sql),0,1)";
                        $cameraIdsInvolved[] = $cameraId;
                    }
                }

                $printableFormula = $formula->formula;
                $fieldsToSelect["f" . $formula->count_data_formula_set_formula_id] = str_replace(array_keys($fieldsToReplace), array_values($fieldsToReplace), $printableFormula);
                $fieldsToSelectNullCheck["f" . $formula->count_data_formula_set_formula_id . "_exists"] = str_replace(array_keys($fieldsToReplaceNullCheck), array_values($fieldsToReplaceNullCheck), $printableFormula);
            }
        }


        $this->_noFields = empty($fieldsToSelect);
        if (empty($fieldsToSelect)) {
            return;
            throw new WebReport_Exception_NoFieldsToSelect("No fields to select.");
        }
        $cameraIdsInvolved = array_unique($cameraIdsInvolved);
        $cameras = $camerasDb->find(array_values($cameraIdsInvolved));

        $countDataTableIds = array();
        foreach ($cameras as $camera) {
            $countDataTableIds[] = $camera->count_data_table_id;
        }

        $countDataTableIds = array_unique($countDataTableIds);
        $countDataTablesDb = new Core_Model_CountData_Tables();
        $countDataTables = $countDataTablesDb->find($countDataTableIds);
        $countDataTables = $countDataTablesDb->find($countDataTableIds);

        $select = $db->select();
        $count = 0;
        $firstDataTable = null;
        if ($this->_sumDataSeries) {
            //$sum = 'SUM( ' . str_replace('SUM','',implode('+',array_values($fieldsToSelect))) . ')';
            $sum =  implode('+', array_values($fieldsToSelect));
            $check =  implode('+', array_values($fieldsToNullCheck));
            unset($fieldsToSelect);
            $fieldsToSelect = array('sum' => $sum);
            $fieldsToNullCheck = array('fields' => $check);
        }
        if ( $skip_meta_data == true ) {
            $fieldsToNullCheck = array();
        }

        $this->_countDataTables = $countDataTables;

        foreach ($countDataTables as $dataTable) {
            if ($count == 0) {
                $fields = array_merge(array("cnttime" => "min(". $dataTable->table .  ".cnttime)"), $fieldsToSelect,$fieldsToNullCheck);
                $select->from($dataTable->table, $fields);
                $firstDataTable = $dataTable;
            } else {
                $select->join($dataTable->table, $dataTable->table . ".cnttime = " . $firstDataTable->table . ".cnttime", array(NULL));
            }
            $count++;
        }

        $this->_lastFetchedKeys = array_values(array_merge(array("cnttime"), array_keys($fieldsToSelect)));

        $select->order("cnttime");

        $this->_cameraIdsInvolvedInSelectAfterInit = $cameraIdsInvolved;
        $this->_selectAfterInit = $select;
    }

    public static function acceptLoad($load) {
        return $load < Core_Model_CountData::MAX_ALLOWED_LOAD;
    }

    // @todo Implement usage of time filter
    public static function estimateQueryLoad($timeResolution = "ts1h", Zend_Date $firstDate = NULL, Zend_Date $lastDate = NULL, $timeFilter = null, $numberOfSources){
	$diff = new Zend_Date($lastDate);
	$diff = $diff->sub($firstDate);
	$days = $diff->toValue() / 60 / 60 / 24;

	$minutes = $days * 24*60;

	$datapoints = $minutes * $numberOfSources;
	$load = 0;

        switch ($timeResolution) {
            case "ts1min":
                $load = $datapoints / 1.0;
                break;
            case "ts5min":
                $load = $datapoints / 5.0;
                break;
            case "ts10min":
                $load = $datapoints / 10.0;
                break;
            case "ts15min":
                $load = $datapoints / 15.0;
                break;
            case "ts30min":
                $load = $datapoints / 30.0;
                break;
            case "ts1h":
                $load = $datapoints / 60.0;
                break;
            case "ts6h":
                $load = $datapoints / 60.0 / 6.0;
                break;
            case "ts12h":
                $load = $datapoints / 60.0 / 12.0;
                break;
            case "ts1d":
                $load = $datapoints / 60.0 / 24.0;
                break;
            case "ts1w":
                $load = $datapoints / 60.0 / 24.0 / 7.0;
                break;
            case "ts2w":
                $load = $datapoints / 60.0 / 24.0 / 14.0;
                break;
            case "ts1m":
                $load = $datapoints / 60.0 / 24.0 / 31.0;
                break;
            case "ts1q":
                $load = $datapoints / 60.0 / 24.0 / 31.0 / 4.0;
                break;
            case "ts1y":
                $load = $datapoints / 60.0 / 24.0 / 31.0 / 365.0;
                break;
        }
        return $load / 1000.0;
    }

    public function canGetData($timeResolution = "ts1h", Zend_Date $firstDate = null, Zend_Date $lastDate = null, $timeFilter = null) {
        $sources = $this->getNumberOfSources();

        $load = Core_Model_CountData::estimateQueryLoad($timeResolution, $firstDate, $lastDate, $timeFilter, $sources);
        return Core_Model_CountData::acceptLoad($load);
    }

    public function getBestResolution($wantedResolution = "ts1h", Zend_Date $firstDate = null, $lastDate = null) {
        return $wantedResolution;
        if (is_null($firstDate))
            $firstDate = $this->getFirstDate();
        if (is_null($lastDate))
            $lastDate = $this->getLastDate();

         $all = array(
            "ts1min" => 0,
            "ts5min" => 1,
            "ts10min" => 2,
            "ts15min" => 3,
            "ts30min" => 4,
            "ts1h" => 5,
            "ts6h" => 6,
            "ts12h" => 7,
            "ts1d" => 8,
            "ts1w" => 9,
            "ts2w" => 10,
            "ts1m" => 11,
            "ts1q" => 12,
            "ts1y" => 13);

        $sources = $this->getNumberOfSources();
        if (!array_key_exists($wantedResolution, $all)) {
            $wantedResolution = 'ts1min';
        }

        $ret = $wantedResolution;

        foreach ($all as $key => $value) {
            if ($all[$wantedResolution] > $value)
                continue;
            $ret = $key;
            if ($this->estimateQueryLoad($ret, $firstDate, $lastDate, NULL, $sources) * 1000 < 300)
                break;
        }

        return $ret;
    }

    public static function getNextTime(Zend_Date $date, $timeResolution = "ts1m") {
        /* @var $ret Zend_Date */
        $ret = clone $date;

        switch ($timeResolution) {
            case "ts1min":
                $ret->addMinute(1);
                break;
            case "ts5min":
                $ret->addMinute(5);
                break;
            case "ts10min":
                $ret->addMinute(10);
                break;
            case "ts15min":
                $ret->addMinute(15);
                break;
            case "ts30min":
                $ret->addMinute(30);
                break;
            case "ts1h":
                $ret = Core_Model_DateHelper::addHour($ret,1);
                break;
            case "ts6h":
                $ret = Core_Model_DateHelper::addHour($ret,6);
                break;
            case "ts12h":
                $ret = Core_Model_DateHelper::addHour($ret,12);
                break;
            case "ts1d":
                $ret->addDay(1);
                break;
            case "ts1w":
                $ret->addWeek(1);
                break;
            case "ts2w":
                $ret->addWeek(2);
                break;
            case "ts1m":
                $ret->addMonth(1);
                break;
            case "ts1q":
                $ret->addMonth(4);
                break;
            case "ts1y":
                $ret->addYear(1);
                break;
            default:
                throw new WebReport_Exception_ParameterError("Invalid time resolution $timeResolution");
                break;
        }

        return $ret;
    }

    public static function truncateDate(Zend_Date $date, $timeResolution = "ts1min") {
        /* @var $ret_date Zend_Date */
        $ret = clone $date;
        $arr = $ret->toArray();
        switch ($timeResolution) {
            case "ts1min":
                $ret->setSecond(0);
                break;
            case "ts5min":
                $ret->setSecond(0);
                $min = (int) $arr['minute' /* could not use constant since documentation is wrong*/];
                $min = $min - $min % 5;
                $ret->setMinute($min);
                break;
            case "ts10min":
                $ret->setSecond(0);
                $min = (int) $arr['minute'];
                $min = $min - $min % 10;
                $ret->setMinute($min);
                break;
            case "ts15min":
                $ret->setSecond(0);
                $min = (int) $arr['minute'];
                $min = $min - $min % 15;
                $ret->setMinute($min);
                break;
            case "ts30min":
                $ret->setSecond(0);
                $min = (int) $arr['minute'];
                $min = $min - $min % 30;
                $ret->setMinute($min);
                break;
            case "ts1h":
                $ret->setSecond(0)
                    ->setMinute(0);
                break;
            case "ts6h":
                $ret->setSecond(0)
                    ->setMinute(0);
                $hour = (int) $arr['hour'];
                $hour = $hour - $hour % 6;
                $ret = Core_Model_DateHelper::setHour($ret,$hour);
                break;
            case "ts12h":
                $ret->setSecond(0)
                    ->setMinute(0);
                $hour = (int) $arr['hour'];
                $hour = $hour - $hour % 12;
                $ret = Core_Model_DateHelper::setHour($ret,$hour);
                break;
            case "ts1d":
                $ret->setSecond(0)
                    ->setMinute(0);
                $ret = Core_Model_DateHelper::setHour($ret,0);
                break;
            case "ts1w":
                $ret->setSecond(0)
                    ->setMinute(0);
                $ret = Core_Model_DateHelper::setHour($ret,0);
                $day = $ret->get(Zend_Date::WEEKDAY_DIGIT);
                // For WEEKDAY_DIGIT Sunday = 0
                if(CM_Interval::firstDayOfWeek() == "monday") $day = ($day-1+7) % 7;
                $ret->subDay($day);
                break;
            case "ts2w":
                $ret->setSecond(0)
                    ->setMinute(0);
                $ret = Core_Model_DateHelper::setHour($ret,0);
                $day = $ret->get(Zend_Date::WEEKDAY_DIGIT);
                // For WEEKDAY_DIGIT Sunday = 0
                if(CM_Interval::firstDayOfWeek() == "monday") $day = ($day-1+7) % 7;
                $ret->subDay($day);
                if($ret->get(Zend_Date::WEEK) % 2)
                    $ret->subDay(7);
                break;
            case "ts1m":
                $ret = Core_Model_DateHelper::setHour($ret,0);
                $ret->setSecond(0)
                    ->setMinute(0)
                    ->setDay(1);
                break;
            case "ts1q":
                $m = (int) $arr['month'];
                $m = $m - ($m-1) % 3;
                $ret = Core_Model_DateHelper::setHour($ret,0);
                $ret->setSecond(0)
                    ->setMinute(0)
                    ->setDay(1)
                    ->setMonth($m);
                break;
            case "ts1y":
                $ret = Core_Model_DateHelper::setHour($ret,0);
                $ret->setSecond(0)
                    ->setMinute(0)
                    ->setDay(1)
                    ->setMonth(1);
                break;
            default:
                throw new WebReport_Exception_ParameterError("Invalid time resolution $timeResolution");
                break;
        }

        return $ret;
    }

    public function getData($timeResolution = "ts1m", Zend_Date $firstDate = null, Zend_Date $lastDate = null, $timeFilter = null, $row_limit=null, $high_resolution=false)
    {
        if($this->_noFields){
            r_log("No fields were selected when generating data in CountData::getData!");
            return array();
        }
        $select = $this->_selectAfterInit;
        if (!$firstDate) {
            $firstDate = $this->getFirstDate();
        }


        if (!$lastDate) {
            $lastDate = $this->getLastDate();
        }

        foreach($this->_countDataTables as $dataTable) {
            if ($high_resolution) {
                $select->where('DATE_FORMAT('.$dataTable->table.'.cnttime, "%Y-%m-%d %H:%i:%s") >= ?', $firstDate->toString("yyyy-MM-dd HH:mm:ss"));
                $select->where('DATE_FORMAT('.$dataTable->table.'.cnttime, "%Y-%m-%d %H:%i:%s") <= ?', $lastDate->toString("yyyy-MM-dd HH:mm:ss"));
            } else {
                $select->where('DATE_FORMAT('.$dataTable->table.'.cnttime, "%Y-%m-%d") >= ?', $firstDate->toString("yyyy-MM-dd"));
                $select->where('DATE_FORMAT('.$dataTable->table.'.cnttime, "%Y-%m-%d") <= ?', $lastDate->toString("yyyy-MM-dd"));
            }

        }

        if(is_string($timeFilter)){
            if(strlen($timeFilter) > 0)
                $select->where(Core_Model_TimeFilterFactory::makeFilter($timeFilter,$this->_countDataTables[0]->table));
        } else if (get_class($timeFilter) == "Core_Model_CountData_TimeFilter") {
            $timeFilter->modifySelect($select);
        } else if (!is_null($timeFilter)) {
            throw new WebReport_Exception("NOT_A_VALID_TIME_FILTER");
        }

        $db = Zend_Registry::get('db');
        $select = $this->addTimeResolution($timeResolution, $select);
        if (isset($row_limit)) {
            $select->limit(round($row_limit/2)); //Divide by two as every countData app has at least two fields (that generate a row each). We're still going to fetch more than neccesary.
        }
        $data = $db->fetchAll($select);
        return $data;
    }

    /**
     * this method relies on getDescriptionsForLastFetchedData that must be called before this method does
     */
    public function getGraphOptionsForLastFetchedData() {
        return $this->_graphOptionsForLastFetchedKeys;
    }

    public static function truncateDateFaster($date, $timeResolution = "ts1min", $cache = null) {
        $y = substr($date,0,4);
        $m = substr($date,5,2);
        $d = substr($date,8,2);
        $key = implode('_',array($y,$m,$d,$timeResolution));
        if($cache && $val = $cache->load($key)) return $val;
        $ret = self::truncateDate(new Zend_Date($date,Zend_Date::ISO_8601),$timeResolution);
        $stamp =  $ret->getTimestamp();
        if($cache) $cache->save($stamp,$key);
        return $stamp;
    }

/*
    Use this if you don't want/need the awfully slow Zend_Date.

    Assumes in format: yyyy-MM-dd HH:mm:ss
    Assumes out format: yyyyMMddHHmmss

    This function will fall back to truncateDateFaster for high resolutions.
*/
    public static function truncateDateString($string, $res) {
        $y  = substr($string, 0, 4);
        $M = substr($string, 5, 2);
        $d  = substr($string, 8, 2);
        $h  = substr($string, 11, 2);
        $m  = substr($string, 14, 2);
        $s  = substr($string, 17, 2);
        if ( $res == "ts15min" ) {
            $m  = (int)$m;
            $m = $m < 15 ? "00" : ($m - $m%15);
            $s = "00";
        } else if( $res == "ts1h" ) {
            $m = "00";
            $s = "00";
        } else if( $res == "ts1d" ) {
            $h = "00";
            $m = "00";
            $s = "00";
        } else {
            $date = self::truncateDate(new Zend_Date($string, "yyyy-MM-dd HH:mm:ss"), $res);
            return $date->toString("yyyyMMddHHmmss");
        }
        $date = "$y$M$d$h$m$s";
        return $date;
    }

    public function getDescriptionsForLastFetchedData()
    {
        $translate = Zend_Registry::get('Zend_Translate');

        $shorts = $this->_lastFetchedKeys;
        $index = array_search("cnttime", $shorts);
        $ret = array();
        if ($index !== false) {
            $ret = array('cnttime' => $translate->_('TIME'));
            unset($shorts[$index]);
        }

        if ($this->_sumDataSeries) {
            $ret['sum'] = 'Sum';
            return $ret;
        }

        $camerasDb = new Core_Model_Cameras();
        $virtualCamerasDb = new Core_Model_VirtualCameras();
        $formulasDb = new Core_Model_CountData_FormulaSetFormulas();
        $cameraIds = array();
        $virtualCameraIds = array();
        $formulaIds = array();
        $applicationsModel = new Core_Model_Applications();

        foreach ($shorts as $sh) {
            $splits = preg_split("/_/", $sh);
            $cid = $splits[0];
            $id = (int) substr($cid, 1);

            if ($sh{0} == 'c') { // camera
                $cameraIds[] = $id;
            } else if ($sh{0} == 'v') { // virtual camera
                $virtualCameraIds[] = $id;
            } else if ($sh{0} == 'f') {
                $formulaIds[] = $id;
            } else {
                throw new WebReport_Exception("Invalid data type");
            }
        }

        $select = $camerasDb->select();
        foreach ($cameraIds as $id) {
            $select->orWhere('camera_id = ?', $id);
        }
        $cameras = $camerasDb->fetchAll($select);
        $cameraPairs = array();
        $cameraApplication = array();
        foreach ($cameras as $camera) {
            $cameraPairs[$camera->camera_id] = $camera->name;
            $cameraApplication[$camera->camera_id] = $camera->application_id;
        }

        $select = $formulasDb->select();
        foreach ($formulaIds as $id) {
            $select->orWhere('count_data_formula_set_formula_id = ?', $id);
        }
        $formulas = $formulasDb->fetchAll($select);
        $formulaPairs = array();
        foreach ($formulas as $formula) {
            $formulaPairs[$formula->count_data_formula_set_formula_id] = $formula;
        }

        $select = $virtualCamerasDb->select();
        foreach ($virtualCameraIds as $id) {
            $select->orWhere('virtual_camera_id = ?', $id);
        }
        $virtualCameras = $virtualCamerasDb->fetchAll($select);
        $virtualCameraPairs = array();
        $virtualCameraApplication = array();
        foreach ($virtualCameras as $camera) {
            $virtualCameraPairs[$camera->virtual_camera_id] = $camera->name;
            $virtualCameraApplication[$camera->virtual_camera_id] = $camera->application_id;
        }

        foreach ($shorts as $sh) {
            $splits = preg_split("/_/", $sh);
            $cid = $splits[0];
            $type = substr($sh, strlen($cid) + 1);
            $id = (int) substr($cid, 1);

            if ($sh{0} == 'c') { // camera
                $application = $applicationsModel->findWithId($cameraApplication[$id]);
                $this->_graphOptionsForLastFetchedKeys[$sh] = $application->getGraphOptions($type);
                $transindex = 'COUNT_TYPE_OUT_' . strtoupper($type);
                $ret[$sh] = $cameraPairs[$id] . ' (' . $translate->_($transindex) . ')';
            } else if ($sh{0} == 'v') { // virtual camera
                $application = $applicationsModel->findWithId($virtualCameraApplication[$id]);
                $this->_graphOptionsForLastFetchedKeys[$sh] = $application->getGraphOptions($type);
                $transindex = 'COUNT_TYPE_OUT_' . strtoupper($type);
                $ret[$sh] = $virtualCameraPairs[$id] . ' (' . $translate->_($transindex) . ')';
            } else if ($sh{0} == 'f') {
                $application = $applicationsModel->findWithId($formulaPairs[$id]->findParentRow('Core_Model_CountData_FormulaSets')->application_id);
                $ret[$sh] = $formulaPairs[$id]->name . " (" . $application->getAbbreviatedName() . ")";
            } else {
                throw new WebReport_Exception("Invalid data type");
            }
        }

        return $ret;
    }

    public function getDescriptionsForLastFetchedData2()
    {
        //New function to allow old function to work exactly the same for legacy permalinks.
        $translate = Zend_Registry::get('Zend_Translate');

        $shorts = $this->_lastFetchedKeys;
        $index = array_search("cnttime", $shorts);
        $ret = array();
        if ($index !== false) {
            $ret = array('cnttime' => "Time");
            unset($shorts[$index]);
        }

        if ($this->_sumDataSeries) {
            $ret['sum'] = 'Sum';
            return $ret;
        }

        $camerasDb = new Core_Model_Cameras();
        $cameraIds = array();
        $applicationsModel = new Core_Model_Applications();

        foreach ($shorts as $sh) {
            $splits = preg_split("/_/", $sh);
            $cid = $splits[0];
            $id = (int) substr($cid, 1);

            if ($sh{0} == 'c') { // camera
                $cameraIds[] = $id;
            } else {
                throw new WebReport_Exception("Invalid data type");
            }
        }

        $select = $camerasDb->select();
        foreach ($cameraIds as $id) {
            $select->orWhere('camera_id = ?', $id);
        }
        $cameras = $camerasDb->fetchAll($select);
        $cameraPairs = array();
        $cameraApplication = array();
        foreach ($cameras as $camera) {
            $cameraPairs[$camera->camera_id] = $camera->name;
            $cameraApplication[$camera->camera_id] = $camera->application_id;
        }

        foreach ($shorts as $sh) {
            $splits = preg_split("/_/", $sh);
            $cid = $splits[0];
            $type = substr($sh, strlen($cid) + 1);
            $id = (int) substr($cid, 1);
            if ($sh{0} == 'c') { // camera
                $application = $applicationsModel->findWithId($cameraApplication[$id]);
                $this->_graphOptionsForLastFetchedKeys[$sh] = $application->getGraphOptions($type);
                //$transindex = 'COUNT_TYPE_OUT_' . strtoupper($type);
                $ret[$sh] = $cameraPairs[$id] . ' (' . $type . ')';
            } else {
                throw new WebReport_Exception("Invalid data type");
            }
        }
        return $ret;
    }

    protected function _initFirstAndLastDate() {

        $db = Zend_Registry::get('db');

        $cameraIdsInvolved = $this->_cameraIdsInvolvedInSelectAfterInit;

        $camerasDb = new Core_Model_Cameras();

        $select = $camerasDb->select()->setIntegrityCheck(false)
                        ->from($camerasDb, array("last_date" => "MAX(last_date)", "first_date" => "MIN(first_date)"));
        foreach ($cameraIdsInvolved as $camId) {
            $select->orWhere('camera_id = ?', $camId);
        }

        $timeRow = $camerasDb->fetchRow($select);

        $firstDate = new Zend_Date();
        $lastDate = new Zend_Date();
        if (isset($timeRow->first_date) && $timeRow->first_date!="") {
            $firstDate->setDate($timeRow->first_date, "yyyy-MM-DD hh:mm:ss");
        }
        if (isset($timeRow->last_date) && $timeRow->last_date!="") {
            $lastDate->setDate($timeRow->last_date, "yyyy-MM-DD hh:mm:ss");
        }
        $this->_firstDate = $firstDate;
        $this->_lastDate = $lastDate;
    }

    public function getFirstDate() {
        if (is_null($this->_firstDate)) {
            $this->_initFirstAndLastDate();
        }
        return $this->_firstDate;
    }

    public function getLastDate() {
        if (is_null($this->_lastDate)) {
            $this->_initFirstAndLastDate();
        }
        return $this->_lastDate;
    }

    /**
     * Fix time resolution for select
     *
     * @param int $timeResolution Time resolution in seconds
     * @param Zend_Db_Select $select
     * @return Zend_Db_Select Select that calculates data with time resolution as well
     */
    public function addTimeResolution($timeResolution, $select)
    {
        $firstTable = array_keys($select->getPart(Zend_Db_Select::FROM));
        if(count($firstTable)>0) {
            $firstTable = $firstTable[0] . ".";
        } else {
            $firstTable = "";
        }

        if(CM_Interval::firstDayOfWeek() == "monday" && ($timeResolution == "ts1w" || $timeResolution == "ts2w")) {
            $timeResolution = $timeResolution . '_m';
        }

        switch ($timeResolution) {
            case "ts1min":
            case "ts5min":
            case "ts10min":
            case "ts15min":
            case "ts30min":
            case "ts1h":
            case "ts6h":
            case "ts12h":
            case "ts1d":
            case "ts1w":
            case "ts1w_m":
            case "ts2w":
            case "ts2w_m":
            case "ts1m":
            case "ts1q":
            case "ts1y":
                $select->group($firstTable . $timeResolution);
                break;
            default:
                throw new Exception("$timeResolution not supported");
        }
        return $select;
    }

    public function getTimeResolutions() {
        $resolutions = array();
        foreach ($this->_applicationsUsed as $application) {
            if (empty($resolutions)) {
                $resolutions = $application->getTimeResolutions();
            } else {
                $resolutions = array_intersect_assoc($resolutions, $application->getTimeResolutions());
            }
        }
        return $resolutions;
    }

    /**
     * Inserta data in $data to db.
     *
     * @param array $data
     * @return void
     * @throws WebReport_Exception if user/pass to table was wrong or if $data is on wrong format or anything else unexpected happens
     */

    public static function addDataForDecodedJSON($objects, $protocol)
    {
        # Mutex handling is done in Core_Model_Protocols_JSON::addData
        $camera = $objects['camera'];
        $app = $objects['app'];
        $data = $objects['modified_data'];
        $entries = $data['data'];
        $struct = $data['data_struct'];
        $protocol::updateTimestamps($camera, $data["utc_offset"], $data["utc_time_start"], null);
        if (!empty($entries)) {
            $countDataTablesDb = Core_Model_CountData_Tables::getInstance();
            $table = $countDataTablesDb->find($camera->count_data_table_id)->current();
            foreach ($entries as $entry) {
                $key_entry = array();
                foreach ($struct as $index => $field) {
                    $key_entry[$field] = $entry[$index];
                }
                $stats = array();
                foreach ($app->getCntMapping() as $key => $stat) {
                    $stats[$stat] = $key_entry[$key];
                }
                if (isset($key_entry['cnttime'])) {
                    $timestamp = $key_entry['cnttime'];
                } elseif (isset($key_entry['unixtime'])) {
                    $timestamp = $key_entry['unixtime'];
                }
                # Shift timestamp from end of interval to beginning of interval
                if ($app instanceof Core_Model_Application_Queue2 || 
                  $app instanceof Core_Model_Application_StereoPeopleCounter)
                  $timestamp -= 60;
                $cnttime = $protocol::createLocalCntTimeFromTimestampAndOffset($timestamp, $camera->utc_offset);
                $tableName = $table->table;
                $db = Zend_Registry::get('db');
                $select = $db->select()
                                ->from($tableName, array($tableName . "_id"))
                                ->where("cnttime = ?", $cnttime);
                $row_id = $db->fetchOne($select);
                if (!$row_id) {
                    self::assureCountDataRowExists(null, $cnttime, $camera, $stats);
                } else {
                    self::updateCountDataRow($row_id, $camera, $stats);
                }
                $protocol::updateTimestamps($camera, $data["utc_offset"], null, $timestamp);
            }
        }
        $protocol::updateOpeningHours($data, $camera);
        if (isset($new_row_id) && $new_row_id > 0) {
            return true;
        } else {
            return false;
        }
    }

    private static function updateCountDataRow($row_id, $camera, $stats)
    {
        // update $row->count_data_table_id;
        $countDataTablesDb = Core_Model_CountData_Tables::getInstance();
        $table = $countDataTablesDb->find($camera->count_data_table_id)->current();
        $tableName = $table->table;
        $cameraId = $camera->camera_id;
        $types = array_keys($stats);
        $nbrOfTypes = sizeof($types);
        $db = Zend_Registry::get('db');
        $sql = "UPDATE " . $db->quoteIdentifier($tableName) . " SET ";

        foreach ($types as $i => $key) {
            $value = $stats[$key];
            $qval = $value === null ? "NULL" : $db->quote($value);
            $sql .= $db->quoteIdentifier("c" . $cameraId . "_" . $key) . " = " . $qval;
            if ($i < $nbrOfTypes - 1) {
                $sql .= ", ";
            }
        }
        $sql .= " WHERE " . $db->quoteIdentifier($tableName . "_id") . " = " . $db->quote((int) $row_id);
        $db->query($sql);
    }

    /**
     * Splits string and inserts data in the string.
     *
     * @param string $data  separated string, if $separator is ":": <user>:<pass>:<version>:<cameraname>:<serial>:<time>:<nbrOfTypes>:<type1>:<data1>[:...]
     * @param string $separator Which string seperates $data fields
     * @return void
     * @throws WebReport_Exception if user/pass to table was wrong or if $string is on wrong format or anything else unexpected happens
     */
    public static function addDataForSeparatedString($data, $separator = "##", $ignore_mutex = false, $cameraGroup = false, $isImport = false, $importWithZeros = false, $appId = NULL)
    {
        if(is_string($data)) {
            $data = explode($separator, $data);
        }

        if(is_object($data)) {
            if (isset($data->username)) {
                $username   = $data->username;
            }
            if (isset($data->pass)) {
                $pass       = $data->pass;
            }
            $version    = $data->version;
            $cameraName = $data->cameraName;
            $serial     = $data->serial;
            $regcode    = $data->regcode;
            $fields     = intval($data->fields);
            $stats      = $data->stats;
            $cnttime    = $data->cnttime;
            $nbrOfTypes = $data->nbrOfTypes;
        } else if(is_array($data)){
            $username   = $data[0];
            $pass       = $data[1];
            $version    = $data[2];
            $cameraName = $data[3];
            $serial     = Core_Model_Cameras::getSerialFromData($data);
            $stats      = Core_Model_Cameras::getStatsFromData($data);
            $regcode    = Core_Model_Cameras::getRegCodeFromData($data);
            $cnttime    = $data[5];
            $nbrOfTypes = intval($data[6]);
            $fields     = count($data);
            $cntmp = (new Zend_Date($cnttime, "yyyyMMddHHmmss"));
            $cnttime = $cntmp
                //->sub('00:15:00',Zend_Date::TIMES)
                ->toString('yyyyMMddHHmmss');
        } else {
            throw new Exception("Only array, string and stdObject are accepted");
        }

        if ($fields <= 9) {
            throw new WebReport_Exception("Invalid data. Not in the correct format.");
            r_log("Invalid data. Not in the right format");
        }
        if (!$isImport) {
            $cntmp = (new Zend_Date($cnttime, "yyyyMMddHHmmss"));
            $cnttime = $cntmp
                ->sub('00:15:00',Zend_Date::TIMES)
                ->toString('yyyyMMddHHmmss');
        }
        if($isImport && !$importWithZeros) {
            $noData = TRUE;
            foreach($stats as $id => $val) {
                if(strval($val) != '0') $noData = FALSE;
            }
            if($noData) {
                return;
            } 
        }

        if (intval($version) < 3) {
            throw new WebReport_Exception("The version is $version. Should be atleast 3.");
        }

        $types = array_keys($stats);
        if(!$ignore_mutex) {
            $mutex = new Core_Model_Mutex();
            $mutex->init(Core_Model_Mutex::MUTEX_MODIFY_CAMERA_TABLES);
        }

        if (strtotime($cnttime) === false) {
            throw new WebReport_Exception("Invalid date and time sent, expecting yyyy-mm-dd HH:MM:SS");
        }
        if(isset($username) && $username == self::TYPE_POS) {
            $cameraGroup = (object)array(
                "camera_group_id" => NULL
            );
        }
        if(!$cameraGroup) {
            $cameraGroupsDb = new Core_Model_CameraGroups();
            $cameraGroup = $cameraGroupsDb->fetchCameraGroup($username, $pass);
        }

        //will this work as expected? http://www.phpeveryday.com/articles/Zend-Framework-Session-Locking-and-Unlocking-Namespace-P575.html
        //removed locking using namespaces, maybe need to add it later
        $camerasDb = Core_Model_Cameras::getInstance();
        $countDataTablesDb = Core_Model_CountData_Tables::getInstance();

        $select = $camerasDb->select()->setIntegrityCheck(false)->from($camerasDb, '*');
        $select
                ->join(array("cdt" => $countDataTablesDb->info('name')), 'cdt.count_data_table_id = camera.count_data_table_id', array("table"))
        		->where('name = ?', $cameraName)
                ->order("last_date DESC")
                ->limit(1);
        if($cameraGroup->camera_group_id !== NULL)
            $select->where('camera_group_id = ?', $cameraGroup->camera_group_id);
        else
            $select->where('camera_group_id is NULL');

        if(!$ignore_mutex && $mutex->acquire() == FALSE) {
                throw new WebReport_Exception("Unable to require semaphore.");
        }
        $camera = $camerasDb->fetchRow($select);

        $db = Zend_Registry::get('db');

        $dbTime = date('Y-m-d H:i:s', strtotime($cnttime));

        if (!$camera) {
            if (is_null($appId)) {
              $application = Core_Model_Applications::guessApplication($types);
            } else {
              $appDb = new Core_Model_Applications();
              $application = $appDb->findWithId($appId);
            }
            if (!$application) {
                error_log("No application");
                throw new Exception("Not valid application.");
            }
            $camera = $camerasDb->createCamera(array(
                "username" => $username,
                "password" => $pass,
                "name" => $cameraName,
                "serial" => $serial,
                "time" => strtotime($cnttime),
                "regcode" => $regcode,
                "application_id" => $application->getId()
            ), $cameraGroup);
        } else {
            if(!$isImport) {
                // In version 2.1.0 the ip field to camera table was added
                // insert IP for the cameras that doesn't have an IP and are already inserted
                // in the table
                if (array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)) {
                    $IParray=array_values(array_filter(explode(',',$_SERVER['HTTP_X_FORWARDED_FOR'])));
                    $CAM_IP=reset($IParray);
                } else if (array_key_exists('REMOTE_ADDR', $_SERVER)) {
                    $CAM_IP=$_SERVER["REMOTE_ADDR"];
                } else {
                    $CAM_IP="";
                }
                $IP_clean = filter_var($CAM_IP, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4);
                if (!strlen($IP_clean))
                    $IP_clean = "Unknown";
                if (strlen($camera->regcode< 4) || $camera->ip == "" || $camera->ip != $IP_clean || $camera->serial != $serial) {
                    // $camera is read only due to setIntegrityCheck(true) above
                    $c = $camerasDb->find($camera->camera_id)->current();
                    $c->ip = $IP_clean;
                
                    $c->serial = substr($serial,0,12);
                    $keys = explode("::",$serial);
                    if(count($keys) > 1){
                        $c->regcode = $keys[1];
                    } else if($camera->regcode == ""){
                        $c->regcode = "?";
                    }
                    $c->save();
                }
            }
            set_time_limit(300); // Prevent mutex->acquire() from timing out easily
        }
        $tableName = $camera->getCountDataTableName();
        $cameraId = $camera->camera_id;

        
        $select = $db->select()
                        ->from($tableName, array($tableName . "_id"))
                        ->where("cnttime = ?", $dbTime);

        $id = $db->fetchOne($select);
        if (!$id) {
            self::assureCountDataRowExists(null, $cnttime, $camera, $stats);
        } else {
            self::updateCountDataRow($id, $camera, $stats);
            // old update $row->count_data_table_id;
            /*$sql = "UPDATE " . $db->quoteIdentifier($tableName) . " SET ";

            foreach($types as $i => $key) {
                $value = $stats[$key];
                // old if (!isset($data[7 + ($i * 2)]) || !isset($data[7 + ($i * 2)]))
                // old    throw new WebReport_Exception("Format was incorrect, too stressfull?");
                $qval = $value === NULL ? "NULL" : $db->quote($value);
                $sql .= $db->quoteIdentifier("c" . $cameraId . "_" . $key) . " = " . $qval;
                if ($i < $nbrOfTypes - 1)
                    $sql .= ", ";
            }*/
            /* Old
            for ($i = 0; $i < $nbrOfTypes; $i++) {
                if (!isset($data[7 + ($i * 2)]) || !isset($data[7 + ($i * 2)]))
                    throw new WebReport_Exception("Format was incorrect, too stressfull?");
                $sql .= $db->quoteIdentifier("c" . $cameraId . "_" . $data[7 + ($i * 2)]) . " = " . $db->quote((int) $data[7 + ($i * 2 + 1)]);
                if ($i < $nbrOfTypes - 1)
                    $sql .= ", ";
            }
            */
            /*$sql .= " WHERE " . $db->quoteIdentifier($tableName . "_id") . " = " . $db->quote((int) $id);
            $db->query($sql);*/
        }

        if(!$ignore_mutex && $mutex->release()==FALSE) {
            throw new WebReport_Exception("Unable to release semaphore.");
        }

        $data = array('last_contact' => new Zend_Db_Expr("UTC_TIMESTAMP()"));
        $data["first_date"] = $camera->first_date ? min($camera->first_date, $dbTime) : $dbTime;
        $data["last_date"] = $camera->last_date ? max($camera->last_date, $dbTime) : $dbTime;
        // Never use an old date
        if ( $data["first_date"] < "1986-01-01 00:00:00" ) $data["first_date"] = $dbTime;
        $db->update($camerasDb->info('name'), $data, 'camera_id = ' . $db->quote($cameraId));
        $db->update($camerasDb->info('name'), array("last_date" => $dbTime, 'last_contact' => new Zend_Db_Expr("UTC_TIMESTAMP()")), 'camera_id = ' . $db->quote($cameraId));

        Core_Model_CountData::_calculateOpeningHours($version,$cameraId,$cnttime,$types,$stats);

        return array(
            "id" => $camera ? $camera->camera_id : FALSE
        );
    }

    public function recalculateOpeningHours($camera)
    {
        $this->initDatabaseSelect(true);
        $countData = $this->getData("ts15min");
        $applicationDB = Core_Model_Applications::getInstance();
        $app = $applicationDB->findWithId($camera->application_id);
        $id = $camera->camera_id;
        $openingHoursDb = new Core_Model_OpeningHours();
        if ($camera->application_id != 12) { //Make sure it's not Demographics
            $current_rows = $openingHoursDb->fetchAll($openingHoursDb->select()->where("camera_id = ?", $id));
            foreach ($current_rows as $row) {
                $row->delete();
            }
            $map = $app->getCntMapping();
            foreach ($countData as $cd) {
                $cd = (array) $cd;
                $version = 3; //As long as it's <5 it will calculate.
                $stats = array();
                foreach ($map as $value_label => $index) {
                    if (isset($cd["c$id" . "_" . $value_label])) {
                        $stats[$index] = $cd["c$id" . "_" . $value_label];
                    } else {
                        r_log("Expected field not found. Cannot build data for recalculation.");
                        return false;
                    }
                }
                $types = array_keys($stats);
                $cnttime = $cd['cnttime'];
                Core_Model_CountData::_calculateOpeningHours($version, $id, $cnttime, $types, $stats);
            }
        }
        return true;
    }

    /*
    * Calculate opening hours and add these to the camera_opening_hours database table.
    */
    protected static function _calculateOpeningHours($version, $cameraId, $cnttime, $types, $stats)
    {
        //r_log("======= Begin calculate opening hours in CountData: ====== ");
        $updateOpeningHours = false;
        $connProtocol = intval($version);
        if($connProtocol>=5){
            //If connProtocol is higher or equal to 5 this function should not be used. Opening hours should be set from setting in camera instead.
            return;
        } else {
            //Legacy data
            $openingHoursDb = new Core_Model_OpeningHours();
            $timeIntervalStart = strtotime($cnttime);
            $timeIntervalEnd = strtotime("+14 minutes 59 seconds", $timeIntervalStart);
            $dataDate = date('Y-m-d', $timeIntervalStart);
            $openingHours = $openingHoursDb->fetchLatestOpeningHours($cameraId, $dataDate)->current();
            $dataTimeClose = date('H:i:s', $timeIntervalEnd);
            $dataTimeOpen = date('H:i:s', $timeIntervalStart);
            /*r_log($dataDate);
            r_log($dataTimeOpen, "time period start: ");
            r_log($dataTimeClose, "time period end: ");
            r_log($stats);*/
            foreach ($types as $i => $key) {
                //If key is TYPE_QUEUE_LOW_FLOW then the value when counting is shut off is 15. Otherwise 0.
                if ((intval($key)==self::TYPE_QUEUE_LOW_FLOW ? 15 : 0) != $stats[$key]) {
                    $updateOpeningHours = true;
                }
            }
            if (!$openingHours) {
                //r_log("No opening hours found for current date and camera. Add an empty row.");
                $openingHours = $openingHoursDb->createRow(array(
                                                        'camera_id' => $cameraId,
                                                        'date' => $dataDate,
                                                        'conn_protocol' => $connProtocol));
                $openingHours->save();
            }
            if ($updateOpeningHours) {
                if (!$openingHours->opens) {
                    //r_log("No opens value found - setting it");
                    $openingHours->opens = $dataTimeOpen;
                }
                if (!$openingHours->closes) {
                    //r_log("No closes value found - setting it");
                    $openingHours->closes = $dataTimeClose;
                }
                $timediff = intval(substr($dataTimeClose, 0, 2))-intval(substr($openingHours->closes, 0, 2));
                //r_log($timediff, "diff in hours from last entry: ");
                if ($timediff > 2 && $openingHours->opens && $openingHours->closes) {
                    //r_log("An opening and closing hour exists for today and receiving data again after 3 or more hours of no values. Assuming two opening times and creating new row.");
                    $openingHours = $openingHoursDb->createRow(array(
                                                        'camera_id' => $cameraId,
                                                        'date' => $dataDate,
                                                        'conn_protocol' => $connProtocol,
                                                        'opens' => $dataTimeOpen,
                                                        'closes' => $dataTimeClose));
                } else {
                    //r_log("Updating closes value");
                    $openingHours->closes = $dataTimeClose;
                }
                //r_log($openingHours->toArray());
                $openingHours->save();
            } else {
                //r_log("Only zero/standard values, do not update anything");
            }
        }
    }

    public static function getGroupTimes($cnttime, $debug = false)
    {
        /* ================================================== */

        /* Since the date function uses the local time zone,  
           we have to compensate with an offset. 
           This compensation should only be done on the unix
           timestamp we use in date, not the ones with modulo
           magic.                                              */

        /* =================================================== */

        $times = array();
        $timeInstance = new DateTime($cnttime, new DateTimeZone("Etc/GMT0"));
       
        $currentTimeZone = new DateTime();
        $time = (int) ($timeInstance->getTimestamp());
        $ftime = (int) ($timeInstance->getTimestamp() - $currentTimeZone->getOffset());
        $d = $timeInstance;

        $times['ts1min'] = $time - $time % (1 * 60);
        $times['ts5min'] = $time - $time % (5 * 60);
        $times['ts10min'] = $time - $time % (10 * 60);
        $times['ts15min'] = $time - $time % (15 * 60);
        $times['ts30min'] = $time - $time % (30 * 60);
        $times['ts1h'] = $time - $time % (60 * 60);
        $times['ts6h'] = $time - $time % (6 * 60 * 60);
        $times['ts12h'] = $time - $time % (12 * 60 * 60);
        $times['ts1d'] = (int) $d->format("Yz");
        $times['ts1w'] = (int) $d->format("oW")
                        + ($d->format("w") == 0 ? 1 : 0 ); // Sunday
        $times['ts1w_m'] = (int) $d->format("oW");
        $times['ts2w'] = $times['ts1w'] & ~1;
        $times['ts2w_m'] = $times['ts1w_m'] & ~1;
        $times['ts1m'] = ((int) $d->format("Y"))*100+ (int)($d->format("m"));#(int) (date("Y", $ftime) * 100 + date("m", $ftime));
        $times['ts1q'] = (int) $d->format("Y") . (int)(floor($d->format("m")) / 3.1 + 1);# (date("Y", $ftime) . (int) floor(date('m', $ftime) / 3.1) + 1);
        $times['ts1y'] = (int) $d->format("Y");#("Y", $ftime);
        if($debug) {
            $times['time'] = $time;
            $times['ftime'] = $ftime;
            $times['ts1y_wrong'] = (int) date("Y", $time);
            $times['hour_diff'] = ($time-$ftime) / 3600;
            $times['tz'] = date("e");
        }
        return $times;
    }

    /*
     * Make sure a row exists in all count data tables with a certain
     * application id. This function is not thread-safe, so make sure you have
     * created an environment where no one else tries to create these rows.
     *
     * If you want to add data at the same time, specify $camera and $stats.
     * If so, $application_id is ignored, and the app ID from $camera is used.
     *
     * The values in $stats are only added if the row does not exist in the
     * count data table where it should be added.
     *
     */
    public static function assureCountDataRowExists($application_id, $cnttime, $camera = false, $stats = array()) {
        if ( $camera ) {
            $application_id = $camera->application_id;
        }
        $db = Zend_Registry::get('db');
        $dbTime = date('Y-m-d H:i:s', strtotime($cnttime));
        $countDataTablesDb = Core_Model_CountData_Tables::getInstance();

        $times = self::getGroupTimes($dbTime);
        $insertTables = $db->fetchAll(
            $db->select()
            ->from($countDataTablesDb->info("name"), array("count_data_table_id", "table"))
            ->where("application_id = ?", $application_id));

        foreach ($insertTables as $insertTable) {
            $addValues = $camera && $insertTable->count_data_table_id == $camera->count_data_table_id;
            $tableName = $insertTable->table;
            $exists = $db->query("SELECT cnttime from `$tableName` where cnttime=".$db->quote($cnttime)."")->fetchAll();

            if ( count($exists) ) {
                continue;
            }
            $sql = "INSERT INTO " . $db->quoteIdentifier($tableName) . " (cnttime, ";
            if ( $addValues ) {
                foreach($stats as $key => $value) {
                    $sql .= "c".$camera->camera_id."_$key" . ", ";
                }
            }
            $sql .= "ts1min, ts5min, ts10min, ts15min, ts30min, ts1h, ts6h, ts12h, ts1d, ts1w, ts1w_m, ts2w, ts2w_m, ts1m, ts1q, ts1y) VALUES ";
            $sql .= "(" . $db->quote($dbTime) . ",";
            if ( $addValues ) {
                foreach($stats as $key => $value) {
                    $qval = $value === NULL ? "NULL" : $db->quote($value);
                    $sql .= $qval . ", ";
                }
            } else {
            }
            $sql .= $db->quote($times['ts1min']) . ", ";
            $sql .= $db->quote($times['ts5min']) . ", ";
            $sql .= $db->quote($times['ts10min']) . ", ";
            $sql .= $db->quote($times['ts15min']) . ", ";
            $sql .= $db->quote($times['ts30min']) . ", ";
            $sql .= $db->quote($times['ts1h']) . ", ";
            $sql .= $db->quote($times['ts6h']) . ", ";
            $sql .= $db->quote($times['ts12h']) . ", ";
            $sql .= $db->quote($times['ts1d']) . ", ";
            $sql .= $db->quote($times['ts1w']) . ", ";
            $sql .= $db->quote($times['ts1w_m']) . ", ";
            $sql .= $db->quote($times['ts2w']) . ", ";
            $sql .= $db->quote($times['ts2w_m']) . ", ";
            $sql .= $db->quote($times['ts1m']) . ", ";
            $sql .= $db->quote($times['ts1q']) . ", ";
            $sql .= $db->quote($times['ts1y']) . ")";
            try {
                $db->query($sql);
            } catch(Exception $e) {
                error_log("Failed to create row in $tableName");
                error_log($e->getMessage());
                throw $e;
            }
        }
    }
}
