<?php

abstract class Core_Model_CountData_Filter_Part_Abstract
{

	protected $_id;
	protected $_application;
	protected $_description;
	protected $_conditions = array();
	protected $_currentCondition = null;
	protected $_currentConditionData = null;


	/**
	 * Setter for filter description (i.e. Count, Camera name etc)
	 * 
	 * @param string $description
	 */
	public function setDescription($description)
	{
		$this->_description = $description;
	}

	/**
	 * Getter for description.
	 *
	 * @return string
	 */
	public function getDescription()
	{
		return $this->_description;
	}

	/**
	 * Returns conditions for filter
	 *
	 * @param bool $omitKeys If true the condition values is excluded, numeric keys instead
	 * @return array Array with conditions
	 */
	public function getConditions($omitKeys = false, $firstElementChain = false, $company = null, $application = null)
	{
		if ($omitKeys) {
			$return = array_values($this->_conditions);
			if ($firstElementChain) {
				$return[0]['elements'] = $this->getConditionElements($return[0]['id'], $company, $application);
			}
			return $return;
		} else {
			$return = $this->_conditions;
			if ($firstElementChain) {
				$first = current($return);
				$first['elements'] =  $this->getConditionElements($return[0]['id'], $company, $application);
			}
			return $return;
		}


	}

	public function getId()
	{
		return $this->_id;
	}

	public function getCondition($conditionId = null)
	{
		if ($conditionId != null && !isset($this->_conditions[$conditionId])) {
				throw new WebReport_Exception("Condition id doesn't exist.");
		} else if ($this->_currentCondition != null) {
			$conditionId = $this->_currentCondition;
		} else {
			return null;
		}

		if (!array_key_exists($conditionId, $this->_conditions)) {
			throw new WebReport_Exception("Condition with id $conditionId don't exists.");
		}

		return $this->_conditions[$conditionId];

	}

	public function getCurrentCondition()
	{
		$cond = $this->getCondition();
		return $cond["id"];
	}

	public function getCurrentConditionValue() {
		return $this->_currentConditionData;
	}

	// @todo set conditions with conditionId
	public function setCondition($conditionId, $conditionData = null)
	{
		$this->_currentCondition = $conditionId;
		
		if ($conditionData != null) {
			if (!is_array($conditionData)) {
				$conditionData = array($conditionData);
			}

			$this->_currentConditionData = $conditionData;
		}

	}

	abstract function getConditionElements($conditionId = null, $company = null, $application = null);

	public function setApplication(Core_Model_Application_Abstract $application)
	{
		$this->_application = $application;
	}

	public function getApplication()
	{
		return $this->_application;
	}



}