<?php

class Core_Model_CountData_Filter_Part_Count extends Core_Model_CountData_Filter_Part_Abstract
{

	protected $_id = "filterpartcount";

	const CONDITION_IS_LESS_THAN = "is less than";
	const CONDITION_IS_EQUAL_TO = "is equal to";
	const CONDITION_IS_GREATER_THAN = "is greater than";

	public function __construct()
	{
		$view = $this->getView();
		$this->setDescription($view->translate("COUNT_DATA_FILTER_PART_DESCRIPTION_COUNT"));
		$this->_conditions[self::CONDITION_IS_LESS_THAN] = array("id" => self::CONDITION_IS_LESS_THAN, "description" => $view->translate("COUNT_FILTER_CONDITION_IS_LESS_THAN"));
		$this->_conditions[self::CONDITION_IS_EQUAL_TO] = array("id" => self::CONDITION_IS_EQUAL_TO, "description" => $view->translate("COUNT_FILTER_CONDITION_IS_EQUAL_TO"));
		$this->_conditions[self::CONDITION_IS_GREATER_THAN] = array("id" => self::CONDITION_IS_GREATER_THAN, "description" => $view->translate("COUNT_FILTER_CONDITION_IS_GREATER_THAN"));
	}

	public function getConditionElements($conditionId = null, $companyId = null, $applicationId = null) {
		if ($conditionId == null) {
			$conditionId = $this->_currentCondition;
		}

		$elements = array();

		$filterGroupNamespace = new Zend_Session_Namespace('WebReport_FilterGroup');

		if ($companyId == null) {
			$companyId = $filterGroupNameSpace->company_id;
		}
		if ($applicationId == null) {
			$applicationId = $filterGroupNameSpace->application_id;
		}

		$countData = new Core_Model_CountData($companyId, $applicationId);

		switch ($conditionId) {
			case self::CONDITION_IS_LESS_THAN:
			case self::CONDITION_IS_EQUAL_TO:
			case self::CONDITION_IS_GREATER_THAN:
				$times = $countData->getTimeResolutions();
				$timesArr = array();
				foreach ($times as $k => $t) {
					$timesArr[] = array("value" => $k, "text" => $t);
				}
				$elements[] = array("type" => "input", "name" => "value");
				$elements[] = array("type" => "select", "data" => $timesArr);
				break;
		}

		return $elements;

	}

	public function select($select) {
		parent::select($select);

		switch ($this->_currentCondition) {
			case self::CONDITION_IS_LESS_THAN:
				$select = $select->where('cnt < ?', $this->_currentConditionData[0]);
				break;
			case self::CONDITION_IS_EQUAL_TO:
				$select = $select->where('cnt = ?', $this->_currentConditionData[0]);
				break;
			case self::CONDITION_IS_GREATER_THAN:
				$select = $select->where('cnt > ?', $this->_currentConditionData[0]);
				break;
			}

		return $select;

	}

	private function _translatableStrings()
	{
	}

}