<?php

class Core_Model_CountData_FilterGroup extends Zend_Db_Table_Row_Abstract
{

	protected $_filters;
	protected $_countData;
	protected $_fetchedData;

	/**
	 * Fetches all belonging filter parts to an associative array,
	 * $this->_filters should look something like this:
	 *
	 * [0] => array([0] => Core_CountDataFilter_XXXX, [1] => Core_CountDataFilter_XXXX)
	 * [1] => ....
	 */
	public function init() {
		parent::init();

		if (isset($this->filters) && !is_null($this->filters)) {
			$filters = unserialize($this->filters);
			$filters = new Core_Model_CountData_Filters();
			
			$filtersArray = unserialize($this->filters);
			$i = 0;
			foreach ($filtersArray as $filterSer) {
				$filter = new Core_Model_CountData_Filter();
				foreach ($filterSer as $part) {
					$fp = Core_Model_CountData_Filter_Parts::factory($part['filterPartId']);
					$fp->setCondition($part['conditionId'], $part['values']);
					$filter->addPart($fp);
				}
				$filters->addFilter($filter);
			}
			$this->setFilters($filters);
		}
	}

	/**
	 * Setter for filter groups filters
	 */
	public function setFilters(Core_Model_CountData_Filters $filters)
	{
		$this->_filters = $filters;
	}

	/**
	 * Getter for filter groups filter.
	 *
	 * @return Core_Model_CountData_Filters
	 */
	public function getFilters()
	{
		return $this->_filters;
	}

	/**
	 * Get the application that belongs to the filter
	 *
	 * @return Core_Model_Application_Abstract Application that belongs to filter
	 */
	public function getApplication()
	{
		$applicationId = $this->application_id;
		$applicationsModel = new Core_Model_Applications();
		$application = $applicationsModel->findWithId($applicationId);
		return $application;
	}
}
