<?php

class Core_Model_Countries
{

	protected $_continents;
	protected $_countries = array();
	protected $_continentCountries;
	protected $_real_countries;

	public function __construct()
	{
		$this->init();
	}

	public function init() 
	{
		$locale = Zend_Registry::get('Zend_Locale');

		$territoryToRegion = ($locale->getTranslationList('TerritoryToRegion'));
		asort($territoryToRegion, SORT_LOCALE_STRING);
		$territory = $locale->getTranslationList('Territory', $locale, 1);
		$countries = $locale->getTranslationList('Territory', $locale, 2);

		// remove obsolete contries
		//unset($countries['SU'], $countries['ZZ'], $countries['VD'], $countries['DD']);

		asort($countries, SORT_LOCALE_STRING);

		$continents = array(
			150 => $territory[150], // europe
			"003" => $territory["003"], // north america
			"005" => $territory["005"], // south america
			142 => $territory[142], // asia
			"053" => $territory["053"], // australia and NZ
			"002" => $territory["002"] // africa
		);
		
		$this->_continents = $continents;

		$continentCountries = array();

		foreach ($continents as $key => $continent) {
			$continentCountries[(string)$key] = array();
		}


		foreach ($countries as $iso => $cntry) {
			$continentIso = $iso;
			$ready = false;
			do {
				if (array_key_exists($continentIso, $territoryToRegion) === true) {
					$check = explode(' ', $territoryToRegion[$continentIso]);
					foreach ($check as $c) {
						$continentIso = $c;
						if (array_key_exists($continentIso, $continents)) {
							$ready = true;
							break;
						}
					}
				} else {
					$ready = true;
					$continentIso = false;
				}
			} while (!$ready);
			if ($continentIso) {
				$continentCountries[$continentIso][$iso] = $cntry;
			}

		}
		
		$this->_continentCountries = $continentCountries;

		$cs = array_values($continentCountries);
		foreach ($cs as $c) {
			$this->_countries = $this->_countries + $c;
		}

		$rc = $countries;

		unset($rc['AQ'],
			  $rc['BQ'],
			  $rc['IO'],
			  $rc['CT'],
			  $rc['CC'],
			  $rc['CG'],
			  $rc['NQ'],
			  $rc['DD'],
			  $rc['TF'],
			  $rc['FQ'],
			  $rc['AN'],
			  $rc['NT'],
			  $rc['VD'],
			  $rc['YD'],
			  $rc['UM'],
			  $rc['PU'],
			  $rc['VI'],
			  $rc['SU'],
			  $rc['ZZ']);


		$this->_real_countries = $rc;
	}

    public function __call($method, array $params)
	{
		$is = substr($method, 0, 2);

		if ($is != 'is') return false;

		$country = substr($method, 2, 2);

		$in = substr($method, 4, 2);

		if ($in != 'In') return false;

		$continent = substr($method, 6);

		$continentIndex = array_search($continent, $this->_continents);
		if ($continentIndex === false && $continent != 'EU') return false;


		if ($continent == 'EU') {
			$euCountries = array('AT', 'BE', 'BG', 'CY', 'CZ', 
								 'DK', 'EE', 'FI', 'FR', 'DE', 
								 'GR', 'HU', 'IE', 'IT', 'LV', 
								 'LT', 'LU', 'MT', 'NL', 'PL',
								 'PT', 'RO', 'SK', 'SI', 'ES',
								 'SE', 'GB');
			return in_array($country, $euCountries);
		} elseif (array_key_exists($country, $this->_continentCountries[$continentIndex])) return true;
			
		return false;
	}

	
	public function getContinents() 
	{
		return $this->_continents;
	}

	public function getCountries()
	{
	  return $this->_real_countries;
	}
	
	public function getContinentCountries()
	{
		return $this->_continentCountries;
	}

	public function __get($value) {
		if (array_key_exists($value, $this->_countries)) return $this->_countries[$value];
		else return false;
	}
	
}
