<?php

class Core_Model_Documents extends Talking_Db_Table
{
    protected $_name = 'document';
	protected $_primary = 'document_id';
	protected $_rowClass = 'Core_Model_Document';

	protected $_dependentTables = array('Core_Model_Folders');

	protected $_referenceMap    = array(
		'Group' => array(
			'columns'           => array('group_id'),
			'refTableClass'     => 'Core_Model_Folders',
			'refColumns'        => array('group_id')
            )
        );

    // Also add in controllers/FoldersControllers.php
    protected $_types = array(
        'camera_group' => array(
            'table' => 'camera_group',
            'id' => 'camera_group_id',
            'name' => 'name'
        ),
        'count_data_set' => array(
            'table' => 'count_data_set',
            'id' => 'count_data_set_id',
            'name' => 'name'
        ),
        'virtual_camera' => array(
            'table' => 'virtual_camera',
            'id' => 'virtual_camera_id',
            'name' => 'name'
        ),
        'formula_set' => array(
            'table' => 'count_data_formula_set',
            'id' => 'count_data_formula_set_id',
            'name' => 'name'
        ),
        'chart' => array(
            'table' => 'mod_charts_data',
            'id' => 'chart_data_id',
            'name' => 'title'
        ),
        'table' => array(
            'table' => 'table',
            'id' => 'table_id',
            'name' => 'title'
        ),
        'report' => array(
            'table' => 'mod_reports_report',
            'id' => 'report_id',
            'name' => 'name'
        ),
        'importjob' => array(
            'table' => 'pos_data',
            'id' => 'pos_data_id',
            'name' => 'name'
        ),
        'subscription' => array(
            'table' => 'mod_reports_subscription',
            'id' => 'subscription_id',
            'name' => 'name'
        )
    );

    public function searchDocuments($string, $limit=10, $user=NULL, $folder_id = NULL, $types = NULL) {
        if(strlen($string)<1 && is_null($folder_id))
            return array();

        $selects = array();

        $db = $this->getAdapter();

        if(is_null($user))
            $user = Zend_Auth::getInstance()->getIdentity();
        
        if($user->multi_company){
            $subquery = $this->select()
                ->from(array('d' => $this->_name), array('*'))
                ->join(array('g' => 'group'), 'd.group_id = g.group_id', array());
            if(!is_null($folder_id) && $folder_id != "all")
                $subquery->where('d.group_id = ?', $folder_id);
        }else if($user->administrator){
            $subquery = $this->select()->
                from(array('d' => $this->_name), array('*'))->
                join(array('g' => 'group'), 'd.group_id = g.group_id', array())->
                where('g.company_id = ?', $user->company_id);
            if(!is_null($folder_id) && $folder_id != "all") {
                $subquery->where('d.group_id = ?', $folder_id);
            }
        }else{
            $subquery = $db->select()->
                from(array('d' => $this->_name), array('document_id', 'group_id', 'type', 'id'))->
                join(array('g' => 'group'), 'd.group_id = g.group_id', array())->
                join(array('s' => 'group_user_sync'), 's.group_id = d.group_id', array())->
                where('user_id = ?', $user->user_id);
            if(!is_null($folder_id) && $folder_id != "all")
                $subquery->where('d.group_id = ?', $folder_id);
        }
        $subquery->join(array('c' => 'company'), 'c.company_id = g.company_id', array());
        $subquery->where('c.active = 1');
        $subquery = $subquery->__toString();

        foreach($this->_types as $type => $params) {
            if($types != NULL && !in_array($type,$types))
                continue;
            $table = $params['table'];
            $id = $params['id'];
            $name = $params['name'];
            $doc = $this->_name;

            $likes = array();
            foreach(explode(' ', $string) as $word) {
                $word = trim($word);

                if(strlen($word))
                   $likes[] = "(UPPER(`$name`) LIKE UPPER('%$word%'))";
            }
            $like = "";
            if(count($likes))
                $like = "AND (" . implode(' AND ', $likes) . ")";

            $selects[] = "SELECT `$name` as name, `$doc`.* FROM ($subquery)  AS `$doc` JOIN `$table` ON `$doc`.type = '$type' WHERE (`$id`=id $like)";

        }
        $select = $db->select()->union($selects)->limit($limit);

        return $select->query()->fetchAll();
    }
}
