<?php

class Core_Model_DummyData
{

    public static function adjustStatsForPrettyData($stats, $weekday, $week, $flow_factor)
    {
        //Relative differences between weekdays approximatley based on one year of Panduro Data.
        if ($weekday == 3) {
            //Wednesday
            $weekday_weight = 0.95;
        } elseif ($weekday == 4) {
            //Thursday
            $weekday_weight = 0.92;
        } elseif ($weekday == 5) {
            //Friday
            $weekday_weight = 1.10;
        } elseif ($weekday == 6) {
            //Saturday
            $weekday_weight = 1.35;
        } elseif ($weekday == 0) {
            //Sunday
            $weekday_weight = 1.2;
        } else {
            $weekday_weight = 1;
        }

        if ($week == 1) {
            //Still some holidays
            $week_weight = 1.1;
        } elseif ($week == 7 || $week == 8) {
            //Vacation
            $week_weight = 1.05;
        } elseif ($week == 13 || $week == 14) {
            //Easter
            $week_weight = 1.15;
        } elseif ($week > 14 && $week < 25) {
            //Spring
            $week_weight = 0.90;
        } elseif ($week > 24 && $week < 35) {
            //Summer (nice weather)
            $week_weight = 0.80;
        } elseif ($week == 43 || $week == 44) {
            //Last week of October, Halloween, black friday.
            $week_weight = 1.3;
        } elseif ($week == 45) {
            //Christmas ramp up starting.
            $week_weight = 1.1;
        } elseif ($week == 46) {
            $week_weight = 1.2;
        } elseif ($week == 47) {
            $week_weight = 1.3;
        } elseif ($week == 48) {
            $week_weight = 1.5;
        } elseif ($week == 49) {
            $week_weight = 1.7;
        } elseif ($week == 50) {
            $week_weight = 1.8;
        } elseif ($week == 51) {
            $week_weight = 2.2;
        } elseif ($week == 52) {
            $week_weight = 1.4;
        } elseif ($week == 53) {
            $week_weight = 1.2;
        } else {
            $week_weight = 1;
        }
        $combined_weight = $weekday_weight * $week_weight * $flow_factor;
        foreach ($stats as $stat => $value) {
            if ($value == 0 && $combined_weight>=1.5) {
                $stats[$stat] = 1;
            } else {
                $stats[$stat] = (int) round($value * $combined_weight);
            }
        }
        return $stats;
    }

    public static function getDataForTimeAndDay($product_id, $t, $openingHours, $flow_factor)
    {
        /*
         * Product id's:
         * 1: People Counter
         * 3: Parking
         * 7: Queue
         * 11: Occupancy
         */
        $weekday = $t->get(Zend_Date::WEEKDAY_DIGIT);
        $week = $t->get(Zend_Date::WEEK);
        $time = strtotime($t->get(Zend_Date::TIME_MEDIUM));
        $ret = array();
        if (count($openingHours) == 1) {
            $oh = $openingHours[0];
        } else {
            $oh = $openingHours[$weekday];
        }
        if ($time > $oh["open"] && $time <= $oh["close"]) {
            //NOTE THAT THE PARTS OF THE DAY BELOW ARE NOT IN ORDER!
            if ($time < $oh["open"]+$oh["whole_15_min_periods"]/5*60*15) {
                //r_log("morning");
                //1 (of 5) parts of opening interval - MORNING
                if ($product_id == 1) {
                    $ret = array(
                        3 => rand(0, 1),
                        4 => rand(0, 1)
                    );
                } elseif ($product_id == 3) {
                    $ret = array(
                        36 => rand(0, 10),
                        37 => rand(0, 2)
                    );
                } elseif ($product_id == 7) {
                    $high = 0;
                    $mid = rand(0, 5);
                    $low = 15 - $high - $mid;
                    $ret = array(
                        300 => $low,
                        301 => $mid,
                        302 => $high,
                    );
                } elseif ($product_id == 11) {
                    $ret = array(
                        64 => rand(3, 10),
                        65 => rand(10, 15)
                    );
                }
            } elseif ($time < $oh["open"]+$oh["whole_15_min_periods"]*2/5*60*15) {
                //r_log("before noon");
                //2 (of 5) parts of opening interval - LATE MORNING
                if ($product_id == 1) {
                    $ret = array(
                        3 => rand(0, 3),
                        4 => rand(0, 3)
                    );
                } elseif ($product_id == 3) {
                    $ret = array(
                        36 => rand(3, 10),
                        37 => rand(0, 8)
                    );
                } elseif ($product_id == 7) {
                    $high = rand(0, 2);
                    $mid = rand(0, 7);
                    $low = 15 - $high - $mid;
                    $ret = array(
                        300 => $low,
                        301 => $mid,
                        302 => $high,
                    );
                } elseif ($product_id == 11) {
                    $ret = array(
                        64 => rand(5, 15),
                        65 => rand(7, 15)
                    );
                }
            } elseif ($time > $oh["close"]-$oh["whole_15_min_periods"]/5*60*15) {
                //r_log("late afternoon");
                //5 (of 5) fifth of opening interval - LATE AFTERNOON
                if ($product_id == 1) {
                    $ret = array(
                        3 => rand(1, 4),
                        4 => rand(1, 4)
                    );
                } elseif ($product_id == 3) {
                    $ret = array(
                        36 => rand(2, 10),
                        37 => rand(5, 20)
                    );
                } elseif ($product_id == 7) {
                    $high = rand(0, 3);
                    $mid = rand(2, 10);
                    $low = 15 - $high - $mid;
                    $ret = array(
                        300 => $low,
                        301 => $mid,
                        302 => $high,
                    );
                } elseif ($product_id == 11) {
                    $ret = array(
                        64 => rand(10, 25),
                        65 => rand(5, 15)
                    );
                }
            } elseif ($time > $oh["close"]-$oh["whole_15_min_periods"]*2/5*60*15) {
                //r_log("early afternoon");
                //4 (of 5) parts of opening interval - EARLY AFTERNOON
                if ($product_id == 1) {
                    $ret = array(
                        3 => rand(1, 6),
                        4 => rand(1, 6)
                    );
                } elseif ($product_id == 3) {
                    $ret = array(
                        36 => rand(1, 4),
                        37 => rand(1, 8)
                    );
                } elseif ($product_id == 7) {
                    $high = rand(0, 1);
                    $mid = rand(0, 6);
                    $low = 15 - $high - $mid;
                    $ret = array(
                        300 => $low,
                        301 => $mid,
                        302 => $high,
                    );
                } elseif ($product_id == 11) {
                    $ret = array(
                        64 => rand(5, 10),
                        65 => rand(10, 20)
                    );
                }
            } else {
                //r_log("noon");
                //middle fifth of opening interval - NOON
                if ($product_id == 1) {
                    $ret = array(
                        3 => rand(2, 8),
                        4 => rand(2, 8)
                    );
                } elseif ($product_id == 3) {
                    $ret = array(
                        36 => rand(5, 15),
                        37 => rand(5, 15)
                    );
                } elseif ($product_id == 7) {
                    $high = rand(0, 5);
                    $mid = rand(4, 10);
                    $low = 15 - $high - $mid;
                    $ret = array(
                        300 => $low,
                        301 => $mid,
                        302 => $high,
                    );
                } elseif ($product_id == 11) {
                    $ret = array(
                        64 => rand(25, 30),
                        65 => rand(3, 15)
                    );
                }
            }
        }
        if (empty($ret)) {
            if ($product_id == 1) {
                $ret = array(
                    3 => 0,
                    4 => 0
                );
            } elseif ($product_id == 3) {
                $ret = array(
                    36 => 0,
                    37 => 0
                );
            } elseif ($product_id == 7) {
                $ret = array(
                    300 => 0,
                    301 => 0,
                    302 => 15,
                    310 => 0
                );
            } elseif ($product_id == 11) {
                $ret = array(
                    3 => 0,
                    4 => 0,
                    64 => 0,
                    65 => 0
                );
            }
        } else {
            $ret = self::adjustStatsForPrettyData($ret, $weekday, $week, $flow_factor);
            if ($product_id == 7) {
                $ret[310] = round(($ret[302]*0.6+$ret[301]*0.2)*rand(80, 120)/100);
            } elseif ($product_id == 11) {
                $ppl = round($ret[64]/(1+$ret[65]));
                $ret = array(
                    3 => rand(0, $ppl*2),
                    4 => rand(0, $ppl*2),
                    64 => $ret[64],
                    65 => $ret[65] == 0 ? null : $ret[65] );
            }
        }
        return $ret;
    }
}
