<?php

class Core_Model_EventData {

protected $_application;
protected $_cameras = array();

    private function makeSureSameApp($app)
    {
        if (isset($this->_application)) {
            $prev_id=$this->_application->getId();
            $new_id=$app->getId();
            if ($prev_id != $new_id) {
                throw new WebReport_Exception("Cannot set up EventData class for more than one application type at the same time.");
            }
        } elseif ($app instanceof Core_Model_Application_EventDataAbstract) {
            $this->_application = $app;
        } elseif ($app instanceof Core_Model_Application_AsEventDataInterface) {
            $this->_application = $app->asEventDataApplication();
        } else {
            throw new WebReport_Exception("Cannot set up EventData class with non EventData application.");
        }
    }

    public function addCameras($camera)
    {
        $appDb = Core_Model_Applications::getInstance();
        if (is_array($camera)) {
            foreach ($camera as $c) {
                $cameras[$c->camera_id] = $c;
                $app = $appDb->getWithId($c->application_id);
                $this->makeSureSameApp($app);
            }
        } elseif ($camera instanceof Core_Model_Camera) {
            $cameras[$camera->camera_id] = $camera;
            $app = $appDb->getWithId($camera->application_id);
            $this->makeSureSameApp($app);
        } else {
            throw new WebReport_Exception("Input to Core_Model_EventData->addCameras must be an array of cameras or a camera.");
        }
        $this->_cameras += $cameras;
    }

    public function getDescriptionsForLastFetchedData()
    {
        if (isset($this->_lastFetchedData)) {
            $translate = Zend_Registry::get('Zend_Translate');
            $return['cnttime'] = $translate->_("Time");
            $app = $this->_application;
            $outTypes = $app->getOutTypes();
            $return['event_id_header'] = $app->getIdColumn() . " (" . $outTypes[$app->getIdColumn()] .")";
            foreach ($this->_lastFetchedData as $key => $value) {
                $return[$key] = $value;
            }
            return $return;
        } else {
            throw new WebReport_Exception("getDescriptionsForLastFetchedData() can not be called before getData().");
        }
    }

    public static function fromLocalDatetimeToUTCTimestamp($date, $utc_offset)
    {
        if ($date instanceof Zend_Date) {
            $datestr_wo_timezone = $date->toString("YYYY-MM-ddTHH:mm:ss");
        } else {
            $datestr_wo_timezone = $date;
        }
        /*Next row creates "Fake" UTC datetime, it's really localtime with UTC
         set so that it doesn't change the timestamp when creating timestamp from datetime."*/
        $datetime = new DateTime($datestr_wo_timezone, new DateTimeZone("UTC"));
        /* Next row creates the real utc_timestamp by subtracting the stored offset from the localtime timestamp*/
        $utc_timestamp = $datetime->format('U.u') - $utc_offset;
        return $utc_timestamp;
    }

    private function getUTCTimestampsPerCamera($firstDate, $lastDate, $override_firstdate = null, $override_lastdate = null)
    {
        $utc_timestamps = array();
        foreach ($this->_cameras as $camera) {
            if (!isset($override_firstdate)) {
                $utc_timestamps[$camera->camera_id]['from'] =
                    self::fromLocalDatetimeToUTCTimestamp($firstDate, $camera->utc_offset);
            } else {
                $utc_timestamps[$camera->camera_id]['from'] = $override_firstdate;
            }
            if (!isset($override_lastdate)) {
                $utc_timestamps[$camera->camera_id]['to'] =
                    self::fromLocalDatetimeToUTCTimestamp($lastDate, $camera->utc_offset);
            } else {
                $utc_timestamps[$camera->camera_id]['to'] = $override_lastdate;
            }
        }
        return $utc_timestamps;
    }

    public function getData($firstDate = null, $lastDate = null, $getRawRows = false, $rowLimit = null)
    {
        if (empty($this->_cameras)) {
            return null;
        }
        $cameras = $this->_cameras;
        $dbTable = $this->_application->getEventDataTable();
        $db = Zend_Registry::get('db');
        $override_firstdate = $override_lastdate = null;
        if ($firstDate == null) {
            $sql = "SELECT MIN(t_start) AS t_start FROM " . $dbTable->info($dbTable::NAME);
            $override_firstdate = $db->fetchRow($sql);
            $override_firstdate = $override_firstdate->t_start;

        }
        if ($lastDate == null) {
            $sql = "SELECT MAX(t_end) AS t_end FROM " . $dbTable->info($dbTable::NAME);
            $override_lastdate = $db->fetchRow($sql);
            $override_lastdate = $override_lastdate->t_end;
        }
        $utc_timestamps = $this->getUTCTimestampsPerCamera($firstDate, $lastDate, $override_firstdate, $override_lastdate);
        $cam_ids = array_keys($cameras);
        $sql_base = "SELECT * FROM " . $dbTable->info($dbTable::NAME) . " WHERE ";
        $sql = "";
        $params_array = array();
        $i = 0;
        foreach ($cam_ids as $cam_id) {
            if ($i!=0) {
                $sql = $sql . " OR ";
            }
            $sql = $sql . "( camera_id = :cam_id_" . $cam_id . " AND t_start >= :from_" . $cam_id . " AND t_end <= :to_" . $cam_id . " )";
            $params_array += array(
                    "cam_id_" . $cam_id => $cam_id,
                    "from_" . $cam_id => $utc_timestamps[$cam_id]['from'],
                    "to_" . $cam_id => $utc_timestamps[$cam_id]['to']
                );
            $i++;
        }
        $sql = $sql_base . $sql;
        $outTypes = $this->_application->getOutTypes();
        $typeCount = count($outTypes)-1; //One outype is id row, which is printed as column instead of row.
        if (isset($rowLimit)) {
            $sql = $sql . " LIMIT " . round($rowLimit/$typeCount); //Divide by count of outtypes as each outtype generate an output row in export.
        }
        $rows = $db->fetchAll($sql, $params_array);
        if ($getRawRows) {
            return $rows;
        }
        $formatted_output = array();
        $last_fetched_data = array();
        foreach ($rows as $rowkey => $row) {
            $camera = $cameras[$row->camera_id];
            $event_time_col = $this->_application->getEventEndTimeColumn();
            $datetime = Core_Model_Protocol_JSON::safelyCreateDateTime($row->$event_time_col + $camera->utc_offset);
            $formatted_output[$rowkey]['cnttime'] = $datetime->format('Y-m-d H:i:s'); //Could possible add .u for microseconds in output.
            foreach ($row as $key => $value) {
                if (array_key_exists($key, $outTypes)) {
                    $field_str = "c" . $row->camera_id . "_" . $key;
                    $formatted_output[$rowkey][$field_str] = $value;
                    if (!isset($last_fetched_data[$field_str])) {
                        $last_fetched_data[$field_str] = $camera->name . " (" . $outTypes[$key]. ")";
                    }
                }
            }
        }
        $this->_lastFetchedData = $last_fetched_data;
        return $formatted_output;
    }



    private static function insertEntry($entry, $data, $camera_id, $app, $eventDb)
    {
        $row = array('camera_id' => $camera_id);
        foreach ($data['data_struct'] as $index => $field) {
            $row[$field] = $entry[$index];
        }
        $new_row = $eventDb->createRow($row);
        try {
            return $new_row->save();
        } catch (Exception $e) {
            r_log($e->getMessage());
            throw new Exception($e->getMessage());
        }
    }

    public static function addData($objects, $protocol)
    {
        $camera = $objects['camera'];
        $app = $objects['app'];
        $data = $objects['modified_data'];
        $entries = $data['data'];
        if (!empty($entries)) {
            $eventDb = $objects['eventDb'];
            foreach ($entries as $entry) {
                $new_row_id = self::insertEntry($entry, $data, $camera->camera_id, $app, $eventDb);
            }
        }
        $protocol::updateOpeningHours($data, $camera);
        $protocol::updateTimestamps($camera, $data["utc_offset"], $data["utc_time_start"], $data["utc_time_end"]);
        if (isset($new_row_id) && $new_row_id > 0) {
            return true;
        } else {
            return false;
        }
    }

    //Adds data from exported file, returns camera id as that is needed for further import of opening hours.
    public static function addImportData($data, $app, $group, $import_info)
    {
        $cache = Zend_Registry::get('cache');
        $status = $import_info['status'];
        $i = $status['cur'];
        $unique = $import_info['unique'];
        $objects = Core_Model_Protocol_JSON::prepareDataAndFetchObjects($data, $app, $group);
        $camera = $objects['camera'];
        $app = $objects['app'];
        $modified_data = $objects['modified_data'];
        $mod_entries = $modified_data['data'];
        if (!empty($mod_entries)) {
            $eventDb = $objects['eventDb'];
            foreach ($mod_entries as $entry) {
                $new_row_id = self::insertEntry($entry, $modified_data, $camera->camera_id, $app, $eventDb);
                session_write_close();
                $i = $i+1;
                $status['cur'] = $i;
                $cache->save(json_encode($status), "importstatus_$unique");
            }
        }
        //Opening Hours should not be updated here. It's done later in import process after this functions has returned.
        Core_Model_Protocol_JSON::updateTimestamps($modified_data, $camera);
        if (isset($new_row_id) && $new_row_id > 0) {
            return $camera->camera_id;
        } else {
            return null;
        }
    }

    public static function userHasVisibleEventData($user)
    {
        $applicationDb = Core_Model_Applications::getInstance();
        $eventDataApps = $applicationDb->fetchAllThatHasVisibleDataInDb('eventData', $user);
        if (count($eventDataApps)!=0) {
            return true;
        } else {
            return false;
        }

    }
}
