<?php

abstract class Core_Model_Generic_AbstractListView {
    abstract protected function getHeaders();
    protected $_isMultiCompany;
    protected $_data;
    protected $_translate;
    protected $_group;
    protected $_user;
    protected $_movable;

    public function __construct() {
        $usersDb = new Core_Model_Users();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        $this->_isMultiCompany = $this->_user &&  $this->_user->multi_company;
        $this->_translate = Zend_Registry::isRegistered("Zend_Translate") ? Zend_Registry::get("Zend_Translate") : NULL;
        $this->_movable = true;
    }

    public function translatedNoDataInformation(){
        return $this->translate($this->noDataInformation());
    }

    public function translate($string){
        return $this->_translate->_($string);
    }

    public function movableItems($yes = NULL){
        if($yes == NULL) return $this->_movable;
        $this->_movable = $yes;
    }

    abstract protected function _dataRowToTableRow($row);
    abstract protected function _dataRowToRowId($row);
    abstract public function noDataInformation();
    abstract public function getActions($row);

    public function isEmpty(){
        return !$this->_data || count($this->_data) == 0;
    }

    public abstract function removeText();


    public static function CreateCellWithClass($text, $classes){
        return array('text' => $text, 'classes' => $classes);
    }

    public static function CreateCellWithClassAndTitle($text, $classes, $title){
        return array('text' => $text, 'classes' => $classes, 'title' => $title);
    }

    public static function GiveLoaderCookie($req){
        setcookie($req->getParam('loadercookie'), '1', time() + 60, '/');
    }

    //Note that for the $showLoader parameter (true/false) below to work it must also be one of the selected classes in
    //GenericListViewGenerator on line 83. This is due to a workaround to get wanted dialog behaviour in addition to overlays.
    public static function CreateAction($translation, $aclAction, $class = null, $folder = null, $canBeHidden = false, $showLoader = false, $module = null, $controller = null, $ext_url = null){
        if(!isset($controller)){
            $controller = Zend_Controller_Front::getInstance()->getRequest()->getControllerName();
        }
        if(!isset($module)){
            $module = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();
        }
        if($class == null) $class = $aclAction;
        if(isset($ext_url)) {
            $action['ext_url'] = $ext_url;
        } elseif (isset($aclAction)) {
            $action['url'] = array('module' => $module, 'controller' => $controller, 'action' => $aclAction);
        } else {
            $action['unclickable'] = true;
        }
        $action['translation'] = $translation;
        $action['class'] = $class;
        if(!is_null($folder))
            $action['acl']['folder'] = $folder;
        if($showLoader){
            $id = "loadercookie".uniqid();
            $action['url'] ['loadercookie'] = $id;
            $action['loader'] = $id;
        } else {
            $action['loader'] = "";
        }

        return $action;
    }

    public function getOrderedData(){
        $data = array();
        if(!empty($this->_data))
           foreach($this->_data as $i => $item){
               $id = $this->_dataRowToRowId($item);
               $data[$id] = &$this->_dataRowToTableRow($item);
               $data[$id]['_source'] = $item;
           }

        return $data;
    }

    public static function CreateView($data, $group = NULL){
        $listType = get_called_class();
        $list = new $listType();
        $list->setData($data);

        if(!is_null($group))
            $list->setGroup($group);

        return $list;
    }

    public static function CreateViews($data, $group = NULL){
        $listType = get_called_class();
        $lists = array();
        foreach($data as $key => $d){
            $list = new $listType();
            $list->setData($d);
            if(!is_null($group))
                $list->setGroup($group);
            $lists[$key] = $list;
        }

        $lists['__type'] = $listType;
        //Need to keep track of group for folder view even if lists contain no data.
        $lists['__group'] = $group;
        return $lists;
    }

    public function setData($data){
        $this->_data = $data;
        return $this;
    }

    public function getData(){
        return $this->_data;
    }

    public function setGroup($group){
        $this->_group = $group;
        return $this;
    }

    public function getGroup(){
        return $this->_group;
    }

    public function forceMultiCompany($value){
        $this->_isMultiCompany = $value;
    }
}
