<?php

class Core_Model_Module extends Zend_Db_Table_Row_Abstract
{

    /**
     * Returns index for translatable string, that is mod_module_name
     *
     * @return string
     */
    public function getTranslateNameIndex()
    {
        return strtoupper($this->alias . '_name');
    }

    /**
     * Deletes mod_ from module alias
     *
     * @return string
     */
    public function getCleanAlias()
    {
        $delete = "mod_";
        $alias = $this->alias;
        $pos = strpos($alias, $delete);
        if ($pos === false)
            return false;

        return substr($alias, $pos + strlen($delete));
    }

    /**
     * Returns module path
     *
     * @return string
     */
    public function getPath()
    {
        $cleanAlias = $this->getCleanAlias();
        return APPLICATION_PATH . '/modules/' . $cleanAlias;
    }

    /**
     * Returns modules settings directory path
     *
     * @return string
     */
    public function getSettingsPath()
    {
        return $this->getPath() . '/settings';
    }

    /**
     * Returns modules routes ini file path
     *
     * @return string
     */
    public function getRoutesSettingPath()
    {
        return $this->getSettingsPath() . '/routes.ini';
    }

    /**
     * Returns array with modules menu item if defined. Null otherwise.
     *
     * @return array|null
     */
    public function getMenuItems()
    {

        $className = Talking_Toolbox_String::toCamel(str_replace("-", "_", ucfirst($this->getCleanAlias())));
        $className = $className . '_Model_MenuItems';
        $menuItems = new $className();

        return $menuItems->getAll();
    }

    /**
     * Returns api class.
     *
     * @return ??
     */
    public function getApiResource()
    {
        $className = ucfirst($this->getCleanAlias()) . '_Model_Api_Resource';

        if (Zend_Loader_Autoloader::autoload($className)) {
            return new $className();
        } else {
            return null;
        }
    }

}
