<?php

//include_once 'CountData/FormulaSetFormulas.php';

class Core_Model_PosData
{
    const TYPE_SALES = 500;
    const TYPE_TRANSACTIONS = 510;
    const TYPE_CONVERSION_RATE = 520;
    const TYPE_AVERAGE_BASKET_VALUE = 511;

    // Number of thousands of data points recommended for one getData()
    // Around 10-16 seem to be the maximum limit if the allowed memory is at 32MB.
    // Trying 32 for 128
    const MAX_ALLOWED_LOAD = 32;

    protected $_posData = array();
    protected $_posDataTypes = array();
    protected $_realPosDataTypes = array();
    protected $_formulaSets = array();
    protected $_formulaSetsTypes = array();
    protected $_filterGroups = array();
    protected $_virtualCameras = array();
    protected $_virtualCamerasTypes = array();
    protected $_cameras = array();
    protected $_camerasTypes = array();
    protected $_applications = array();
    protected $_applicationsCompanies = array();
    protected $_lastFetchedKeys = array();
    protected $_graphOptionsForLastFetchedKeys = array();
    protected $_selectAfterInit = null;
    protected $_cameraIdsInvolvedInSelectAfterInit = array();
    protected $_countDataTablesToSelectFromAfterInit = array();
    protected $_firstDate = null;
    protected $_lastDate = null;
    protected $_applicationsUsed = array();
    protected $_arbitraryTable = null;
    protected $_unit = "";

    protected $_time_mask_start = null;
    protected $_time_mask_stop = null;

    protected $_sumDataSeries = False;
    protected $_noFields = False;

    public function getFormulaSets() {
        return $this->_formulaSets;
    }

    public function hasFields(){
        return !$this->_noFields;
    }

    public function getFormulaSetTypes() {
        return $this->_formulaSetsTypes;
    }

    public function getVirtualCameras() {
        return $this->_virtualCameras;
    }

    public function getVirtualCameraTypes() {
        return $this->_virtualCamerasTypes;
    }

    public function getFilterGroups() {
        return $this->_filterGroups;
    }

    public function getCameras() {
        return $this->_cameras;
    }

    public function getCamerasTypes() {
        return $this->_camerasTypes;
    }

    public function getCountData() {
        $cd = new Core_Model_CountData();
        foreach($this->_cameras as $i => $cam){
            $cd->addCamera($cam , $this->_camerasTypes[$i]);
        }
        $cd->setSumDataSeries(true);
        return $cd;
    }

    public function getPosType(){
        return $this->_realPosDataTypes[0];
    }

    public static function GetTypeDescription($id){
        if($id ==self::TYPE_SALES) return _T("SALES");
        if($id ==self::TYPE_TRANSACTIONS) return _T("TRANSACTIONS");
        if($id ==self::TYPE_CONVERSION_RATE) return _T("CONVERSION_RATE");
        if($id ==self::TYPE_AVERAGE_BASKET_VALUE) return _T("AVERAGE_TRANSACTION_VALUE");
        return _T("UNKNOWN_TITLE");
    }

    public function getUnit() {
        if(count($this->_posData) == 0) return "";
        $jobDb = Pos_Model_ImportJobs::getInstance();
        $job = $jobDb->find($this->_posData[0])->current();
        $type = $this->_realPosDataTypes[0];
        $translate = Zend_Registry::get('Zend_Translate');
        if($type == self::TYPE_SALES) {
            return $job->currency;
        } else if($type == self::TYPE_TRANSACTIONS) {
            return "";
        } else if($type == self::TYPE_CONVERSION_RATE) {
            return "%"; // $job->currency ."/". $translate->_("PERSON");
        }
        
        return $job->currency;
    }

/*
    public function addPosData(Core_Model_CountData $merge, $type = NULL,
            $mergeCameras = True, $mergeVirtualCameras = True,
            $mergeFormulas = False, $mergeFilterGroups = True,
            $mergeApplications = False) {
        if(!is_null($type) && !is_array($type)) {
            $type = array($type);
        }

        if ($mergeCameras) {
            $cameras = $merge->getCameras();
            $cameraTypes = $merge->getCamerasTypes();

            foreach ($cameras as $index => $camera) {

                // check if this camera needs to be added
                $addCamera = True;
                foreach ($this->_cameras as $cameraIndex => $setCamera) {
                    if ($setCamera->camera_id == $camera->camera_id) {
                        $addCamera = False;
                        $cameraType = is_null($type) ? $cameraTypes[$index] : $type;
                        $this->_camerasTypes[$cameraIndex] = array_unique(array_merge($this->_camerasTypes[$cameraIndex] ,$cameraType));
                        break;
                    }
                }
                if ($addCamera) {
                    $this->addCamera($camera, !is_null($type) ? $type : $cameraTypes[$index]);
                } else {
                }
            }
        }


        if ($mergeVirtualCameras) {
            $virtualCameras = $merge->getVirtualCameras();
            $virtualCameraTypes = $merge->getVirtualCameraTypes();

            foreach ($virtualCameras as $index => $virtualCamera) {
                $addVirtualCamera = True;
                foreach ($this->_virtualCameras as $virtualCameraIndex => $setVirtualCamera) {
                    if ($virtualCamera->virtual_camera_id == $virtualCamera->virtual_camera_id) {
                        $addVirtualCamera = False;
                        $virtualCameraType = is_null($type) ? $virtualCameraTypes[$index] : $type;
                        $this->_virtualCamerasTypes[$virtualCameraIndex] =
                                array_unique($this->_virtualCamerasTypes[$virtualCameraIndex] + $virtualCameraType);
                        break;
                    }
                }
                if ($addVirtualCamera)
                    $this->addVirtualCamera($virtualCamera, !is_null($type) ? $type : $virtualCameraTypes[$index]);
            }
        }

        if ($mergeFormulas) {
            $formulas = $merge->getFormulaSets();
            $formulaTypes = $merge->getFormulaSetTypes();

            foreach ($formulas as $index => $formula) {
                $this->addFormulaSet($formula, $formulaTypes[$index]);
            }
        }

        if ($mergeFilterGroups)
            $this->addFilterGroups($merge->getFilterGroups());

        if ($mergeApplications)
            throw new WebReport_Error("NOT_IMPLEMENTED only needed when there are other applications and that is not supported.");

        return $this;
            }


 */
    public function addPosDataData($posData, $type = NULL, $posDataIndex = NULL) {
        if(!$type) throw new Exception("No type provided");
        $posDb = new Pos_Model_ImportJobs();
        $pos = $posDb->fetchOne($posData);
        if(!$pos) {
            error_log("Coult not find your pos data :(");
            return false;
        }
        $this->addCameras($pos->getCameras(),'in');
        $this->_posData[] = (int)$posData;
        if($type == self::TYPE_SALES || $type == self::TYPE_TRANSACTIONS){
            $this->_posDataTypes[] = $type;
            $this->_realPosDataTypes[] = $type;
        } else if ($type == self::TYPE_CONVERSION_RATE) {
            $this->_posDataTypes[] = self::TYPE_TRANSACTIONS;
            $this->_realPosDataTypes[] = $type;
        } else if ($type == self::TYPE_AVERAGE_BASKET_VALUE) {
            if($posDataIndex == 1) $this->_posDataTypes[] = self::TYPE_SALES;
            else if($posDataIndex == 2) $this->_posDataTypes[] = self::TYPE_TRANSACTIONS;
            else {
                $this->_posDataTypes[] = "VIRTUAL POS DATA TYPE";
            }
            $this->_realPosDataTypes[] = $type;
        } else {
            throw new Exception("Type is not a valid POS type: $type");
        }
    }

    public function hasPosData(){
        return count($this->_posData) > 0;
    }

    public function setSumDataSeries($bool) {
        $this->_sumDataSeries = $bool;
        return $this;
    }

    public function addFormulaSets($formulaSets)
    {
        foreach ($formulaSets as $formulaSet) {
            $this->addFormulaSet($formulaSet);
        }
        return $this;
    }

    public function addFormulaSet(Core_Model_CountData_FormulaSet $formulaSet, $types = array())
    {
        $this->_formulaSets[] = $formulaSet;
        $this->_formulaSetsTypes[] = $types;
        return $this;
    }

    public function addVirtualCameras($virtualCameras, $types = array())
    {
        foreach ($virtualCameras as $camera) {
            $this->addVirtualCamera($camera, $types);
        }
    }

    public function addVirtualCamera($virtualCamera, $types = array())
    {
        $this->_virtualCameras[] = $virtualCamera;
        $this->_virtualCamerasTypes[] = $types;
        return $this;
    }

    public function addFilterGroups($filterGroups)
    {
        foreach ($filterGroups as $group) {
            $this->addFilterGroup($group);
        }
        return $this;
    }

    public function addFilterGroup(Core_Model_CountData_FilterGroup $filterGroup)
    {
        $this->_filterGroups[] = $filterGroup;
        return $this;
    }

    public function addCameras($cameras, $types = array())
    {
        foreach ($cameras as $camera) {
            $this->addCamera($camera, $types);
        }
        return $this;
    }

    public function addCamera($camera, $types = array())
    {
        $this->_cameras[] = $camera;
        $this->_camerasTypes[] = $types;
        return $this;
    }

    public function addApplication(Core_Model_Application_Abstract $application, Core_Model_Company $company)
    {
        $camerasDb = new Core_Model_Cameras();
        throw new WebReport_Exception("Not yet implemented");
        $this->_applications[] = $application;
        $this->_applicationsCompanies[] = $company;
        return $this;
    }

    public function getNumberOfSources(){
        return count($this->_cameras) + count($this->_filterGroups)  + count($this->_virtualCameras) + count($this->_formulaSets);
    }

    public function initDatabaseSelect()
    {
        $db = Zend_Registry::get('db');
        $select = $db->select();

        $camerasDb = new Core_Model_Cameras();
        $cameraGroupsDb = new Core_Model_CameraGroups();
        $companiesDb = new Core_Model_Companies();

        $fieldsToSelect = array();
        $countDataTablesIds = array();

        $cameraIdsInvolved = array();

        foreach ($this->_cameras as $arrayIndex => $cam) {
            $cameraId = $cam->camera_id;
            $camera = $camerasDb->find($cameraId)->current();
            $application = $applicationsModel->findWithId($camera->application_id);
            $this->_applicationsUsed[$application->getId()] = $application;
            $inTypes = $application->getInTypes();

            $outTypes = array();
            if (empty($this->_camerasTypes[$arrayIndex])) {
                $outTypes = array_keys($application->getOutTypes());
            } else {
                $outTypes = $this->_camerasTypes[$arrayIndex];
            }
            $cameraId = $camera->camera_id;
            foreach ($outTypes as $outType) {
                $fieldsToSelect["c" . $cameraId . "_" . $outType] = "(" . $application->getSqlQueryForCntOutType($outType, $camera) . ")";
            }

            $cameraIdsInvolved[] = $camera->camera_id;
        }

        $this->_noFields = empty($fieldsToSelect);
        if (empty($fieldsToSelect)) {
            return;
            throw new WebReport_Exception_NoFieldsToSelect("No fields to select.");
        }
        $cameraIdsInvolved = array_unique($cameraIdsInvolved);
        $cameras = $camerasDb->find(array_values($cameraIdsInvolved));

        $countDataTableIds = array();
        foreach ($cameras as $camera) {
            $countDataTableIds[] = $camera->count_data_table_id;
        }

        $countDataTableIds = array_unique($countDataTableIds);
        $countDataTablesDb = new Core_Model_CountData_Tables();
        $countDataTables = $countDataTablesDb->find($countDataTableIds);
        $countDataTables = $countDataTablesDb->find($countDataTableIds);

        $select = $db->select();
        $count = 0;
        $firstDataTable = null;

        if($this->_sumDataSeries) {
            //$sum = 'SUM( ' . str_replace('SUM','',implode('+',array_values($fieldsToSelect))) . ')';
            $sum =  implode('+', array_values($fieldsToSelect));
            unset($fieldsToSelect);
            $fieldsToSelect = array('sum' => $sum);
        }

        foreach ($countDataTables as $dataTable) {
            $this->_arbitraryTable = $dataTable->table;
            if ($count == 0) {
                $select->from($dataTable->table, array_merge(array("cnttime" => "(". $dataTable->table .  ".cnttime)"), $fieldsToSelect));
                $firstDataTable = $dataTable;
            } else {
                $select->join($dataTable->table, $dataTable->table . ".cnttime = " . $firstDataTable->table . ".cnttime", array(NULL));
            }
            $count++;
        }


        $this->_lastFetchedKeys = array_values(array_merge(array("cnttime"), array_keys($fieldsToSelect)));

        $select->order("cnttime");

        $this->_cameraIdsInvolvedInSelectAfterInit = $cameraIdsInvolved;
        $this->_selectAfterInit = $select;
    }

    public static function getNextTime(Zend_Date $date, $timeResolution = "ts1m") {
        /* @var $ret Zend_Date */
        $ret = clone $date;

        switch ($timeResolution) {
        case "ts1min":
            $ret->addMinute(1);
            break;
        case "ts5min":
            $ret->addMinute(5);
            break;
        case "ts10min":
            $ret->addMinute(10);
            break;
        case "ts15min":
            $ret->addMinute(15);
            break;
        case "ts30min":
            $ret->addMinute(30);
            break;
        case "ts1h":
            $ret->addHour(1);
            break;
        case "ts6h":
            $ret->addHour(6);
            break;
        case "ts12h":
            $ret->addHour(12);
            break;
        case "ts1d":
            $ret->addDay(1);
            break;
        case "ts1w":
            $ret->addWeek(1);
            break;
        case "ts2w":
            $ret->addWeek(2);
            break;
        case "ts1m":
            $ret->addMonth(1);
            break;
        case "ts1q":
            $ret->addMonth(4);
            break;
        case "ts1y":
            $ret->addYear(1);
            break;
        default:
            throw new WebReport_Exception_ParameterError("Invalid time resolution $timeResolution");
            break;
        }

        return $ret;
    }

    public static function truncateDate(Zend_Date $date, $timeResolution = "ts1min") {
        /* @var $ret_date Zend_Date */
        $ret = clone $date;
        $arr = $ret->toArray();

        switch ($timeResolution) {
        case "ts1min":
            $ret->setSecond(0);
            break;
        case "ts5min":
            $ret->setSecond(0);
            $min = (int) $arr['minute' /* could not use constant since documentation is wrong*/];
            $min = $min - $min % 5;
            $ret->setMinute($min);
            break;
        case "ts10min":
            $ret->setSecond(0);
            $min = (int) $arr['minute'];
            $min = $min - $min % 10;
            $ret->setMinute($min);
            break;
        case "ts15min":
            $ret->setSecond(0);
            $min = (int) $arr['minute'];
            $min = $min - $min % 15;
            $ret->setMinute($min);
            break;
        case "ts30min":
            $ret->setSecond(0);
            $min = (int) $arr['minute'];
            $min = $min - $min % 30;
            $ret->setMinute($min);
            break;
        case "ts1h":
            $ret->setSecond(0)
                ->setMinute(0);
            break;
        case "ts6h":
            $ret->setSecond(0)
                ->setMinute(0);
            $hour = (int) $arr['hour'];
            $hour = $hour - $hour % 6;
            $ret->setHour($hour);
            break;
        case "ts12h":
            $ret->setSecond(0)
                ->setMinute(0);
            $hour = (int) $arr['hour'];
            $hour = $hour - $hour % 12;
            $ret->setHour($hour);
            break;
        case "ts1d":
            $ret->setSecond(0)
                ->setMinute(0)
                ->setHour(0);
            break;
        case "ts1w":
            $ret->setSecond(0)
                ->setMinute(0)
                ->setHour(0);
            $day = $ret->get(Zend_Date::WEEKDAY_DIGIT);
            // For WEEKDAY_DIGIT Sunday = 0
            if(CM_Interval::firstDayOfWeek() == "monday") $day = ($day-1+7) % 7;
            $ret->subDay($day);
            break;
        case "ts2w":
            $ret->setSecond(0)
                ->setMinute(0)
                ->setHour(0);
            $day = $ret->get(Zend_Date::WEEKDAY_DIGIT);
            // For WEEKDAY_DIGIT Sunday = 0
            if(CM_Interval::firstDayOfWeek() == "monday") $day = ($day-1+7) % 7;
            $ret->subDay($day);
            if($ret->get(Zend_Date::WEEK) % 2)
                $ret->subDay(7);
            break;
        case "ts1m":
            $ret->setSecond(0)
                ->setMinute(0)
                ->setHour(0)
                ->setDay(1);
            break;
        case "ts1q":
            $m = (int) $arr['month'];
            $m = $m - ($m-1) % 3;
            $ret->setSecond(0)
                ->setMinute(0)
                ->setHour(0)
                ->setDay(1)
                ->setMonth($m);
            break;
        case "ts1y":
            $ret->setSecond(0)
                ->setMinute(0)
                ->setHour(0)
                ->setDay(1)
                ->setMonth(1);
            break;
        default:
            throw new WebReport_Exception_ParameterError("Invalid time resolution $timeResolution");
            break;
        }

        return $ret;
    }

    public function getData($timeResolution = "ts1m", Zend_Date $firstDate = null, Zend_Date $lastDate = null, $timeFilter = null)
    {
        if (!$firstDate) {
            $firstDate = $this->getFirstDate();
        }

        if (!$lastDate) {
            $lastDate = $this->getLastDate();
        }

        $db = Zend_Registry::get('db');
        $select = $db->select();
        $posDb = new Pos_Model_ImportJobs();
        $fieldsToSelect = array();
        $usePostProcess = false;
        foreach($this->_posData as $kid => $vid){
            $id = $vid;
            $cnt = $this->_posDataTypes[$kid];
            $realcnt = $this->_realPosDataTypes[$kid];
            if($realcnt == self::TYPE_CONVERSION_RATE){
                //$cnt = self::TYPE_SALES;
                $fieldsToSelect[] = "COALESCE(p$id"."_".self::TYPE_TRANSACTIONS.",0)";
            } else if($realcnt == self::TYPE_AVERAGE_BASKET_VALUE) {
                $fieldsToSelect[] = "COALESCE(p$id"."_".$cnt.",0)";
            } else {
                $fieldsToSelect[] = "COALESCE(p$id"."_$cnt,0)";
            }
        }
        $sum =  implode('+', array_values($fieldsToSelect));
        $select = $db->select();
        $oneTable = 'pos_data_pos1'; // When using multiple pos tables, this must be used.
        $fieldsToSelect = array('sum' => "sum($sum)");
        $select->from('pos_data_pos1', array_merge(array("cnttime" => "(". $oneTable .  ".cnttime)"), $fieldsToSelect));
        unset($fieldsToSelect);
/*
        foreach ($countDataTables as $dataTable) {
            $this->_arbitraryTable = $dataTable->table;
            if ($count == 0) {
                $firstDataTable = $dataTable;
            } else {
                $select->join($dataTable->table, $dataTable->table . ".cnttime = " . $firstDataTable->table . ".cnttime", array(NULL));
            }
            $count++;
        }
 */

        $select = $this->addTimeResolution($timeResolution, $select);
        $select->where('DATE_FORMAT(pos_data_pos1.cnttime, "%Y-%m-%d") >= ?', $firstDate->toString("yyyy-MM-dd"));
        $select->where('DATE_FORMAT(pos_data_pos1.cnttime, "%Y-%m-%d") <= ?', $lastDate->toString("yyyy-MM-dd"));

        if(is_string($timeFilter)){
            if(strlen($timeFilter) > 0){
                $select->where(Core_Model_TimeFilterFactory::makeFilter($timeFilter, 'pos_data_pos1'));
            }
        } else if (get_class($timeFilter) == "Core_Model_CountData_TimeFilter") {
            $timeFilter->modifySelect($select);
        } else if(!is_null($timeFilter)){
            throw new WebReport_Exception("NOT_A_VALID_TIME_FILTER");
        }
        $data = $db->fetchAll($select);
        return $data;
    }

    /**
     * this method relies on getDescriptionsForLastFetchedData that must be called before this method does
     */
    public function getGraphOptionsForLastFetchedData()
    {
        return $this->_graphOptionsForLastFetchedKeys;
    }

    public function getDescriptionsForLastFetchedData()
    {
        $translate = Zend_Registry::get('Zend_Translate');

        $shorts = $this->_lastFetchedKeys;
        $index = array_search("cnttime", $shorts);
        $ret = array();
        if ($index !== false) {
            $ret = array('cnttime' => $translate->_('TIME'));
            unset($shorts[$index]);
        }

        if($this->_sumDataSeries) {
            $ret['sum'] = 'Sum';
            return $ret;
        }

        $camerasDb = new Core_Model_Cameras();
        $virtualCamerasDb = new Core_Model_VirtualCameras();
        $formulasDb = new Core_Model_CountData_FormulaSetFormulas();
        $cameraIds = array();
        $virtualCameraIds = array();
        $formulaIds = array();
        $applicationsModel = new Core_Model_Applications();

        foreach ($shorts as $sh) {
            $splits = preg_split("/_/", $sh);
            $cid = $splits[0];
            $id = (int) substr($cid, 1);

            if ($sh{0} == 'c') { // camera
                $cameraIds[] = $id;
            } else if ($sh{0} == 'v') { // virtual camera
                $virtualCameraIds[] = $id;
            } else if ($sh{0} == 'f') {
                $formulaIds[] = $id;
            } else {
                throw new WebReport_Exception("Invalid data type");
            }
        }

        $select = $camerasDb->select();
        foreach ($cameraIds as $id) {
            $select->orWhere('camera_id = ?', $id);
        }
        $cameras = $camerasDb->fetchAll($select);
        $cameraPairs = array();
        $cameraApplication = array();
        foreach ($cameras as $camera) {
            $cameraPairs[$camera->camera_id] = $camera->name;
            $cameraApplication[$camera->camera_id] = $camera->application_id;
        }

        $select = $formulasDb->select();
        foreach ($formulaIds as $id) {
            $select->orWhere('count_data_formula_set_formula_id = ?', $id);
        }
        $formulas = $formulasDb->fetchAll($select);
        $formulaPairs = array();
        foreach ($formulas as $formula) {
            $formulaPairs[$formula->count_data_formula_set_formula_id] = $formula;
        }

        $select = $virtualCamerasDb->select();
        foreach ($virtualCameraIds as $id) {
            $select->orWhere('virtual_camera_id = ?', $id);
        }
        $virtualCameras = $virtualCamerasDb->fetchAll($select);
        $virtualCameraPairs = array();
        $virtualCameraApplication = array();
        foreach ($virtualCameras as $camera) {
            $virtualCameraPairs[$camera->virtual_camera_id] = $camera->name;
            $virtualCameraApplication[$camera->virtual_camera_id] = $camera->application_id;
        }

        foreach ($shorts as $sh) {
            $splits = preg_split("/_/", $sh);
            $cid = $splits[0];
            $type = substr($sh, strlen($cid) + 1);
            $id = (int) substr($cid, 1);

            if ($sh{0} == 'c') { // camera
                $application = $applicationsModel->findWithId($cameraApplication[$id]);
                $this->_graphOptionsForLastFetchedKeys[$sh] = $application->getGraphOptions($type);
                $transindex = 'COUNT_TYPE_OUT_' . strtoupper($type);
                $ret[$sh] = $cameraPairs[$id] . ' (' . $translate->_($transindex) . ')';
            } else if ($sh{0} == 'v') { // virtual camera
                $application = $applicationsModel->findWithId($virtualCameraApplication[$id]);
                $this->_graphOptionsForLastFetchedKeys[$sh] = $application->getGraphOptions($type);
                $transindex = 'COUNT_TYPE_OUT_' . strtoupper($type);
                $ret[$sh] = $virtualCameraPairs[$id] . ' (' . $translate->_($transindex) . ')';
            } else if ($sh{0} == 'f') {
                $application = $applicationsModel->findWithId($formulaPairs[$id]->findParentRow('Core_Model_CountData_FormulaSets')->application_id);
                $ret[$sh] = $formulaPairs[$id]->name . " (" . $application->getAbbreviatedName() . ")";
            } else {
                throw new WebReport_Exception("Invalid data type");
            }
        }

        return $ret;
    }

    protected function _initFirstAndLastDate()
    {

        $db = Zend_Registry::get('db');

        $cameraIdsInvolved = $this->_cameraIdsInvolvedInSelectAfterInit;

        $camerasDb = new Core_Model_Cameras();

        $select = $camerasDb->select()->setIntegrityCheck(false)
            ->from($camerasDb, array("last_date" => "MAX(last_date)", "first_date" => "MIN(first_date)"));
        foreach ($cameraIdsInvolved as $camId) {
            $select->orWhere('camera_id = ?', $camId);
        }

        $timeRow = $camerasDb->fetchRow($select);

        $firstDate = new Zend_Date(new Zend_Locale('en_IE'));
        $lastDate = new Zend_Date(new Zend_Locale('en_IE'));
        $firstDate->setDate($timeRow->first_date, "yyyy-MM-DD hh:mm:ss");
        $lastDate->setDate($timeRow->last_date, "yyyy-MM-DD hh:mm:ss");

        $this->_firstDate = $firstDate;
        $this->_lastDate = $lastDate;
    }

    public function getFirstDate()
    {
        if (is_null($this->_firstDate)) {
            $this->_initFirstAndLastDate();
        }
        return $this->_firstDate;
    }

    public function getLastDate()
    {
        if (is_null($this->_lastDate)) {
            $this->_initFirstAndLastDate();
        }
        return $this->_lastDate;
    }

    /**
     * Fix time resolution for select
     *
     * @param int $timeResolution Time resolution in seconds
     * @param Zend_Db_Select $select
     * @return Zend_Db_Select Select that calculates data with time resolution as well
     */
    public function addTimeResolution($timeResolution, $select)
    {
        $firstTable = array_keys($select->getPart($select::FROM));
        if(count($firstTable)>0) {
            $firstTable = $firstTable[0] . ".";
        } else {
            $firstTable = "";
        }

        if(CM_Interval::firstDayOfWeek() == "monday" && ($timeResolution == "ts1w" || $timeResolution == "ts2w")) {
            $timeResolution = $timeResolution . '_m';
        }

        switch ($timeResolution) {
        case "ts1min":
        case "ts5min":
        case "ts10min":
        case "ts15min":
        case "ts30min":
        case "ts1h":
        case "ts6h":
        case "ts12h":
        case "ts1d":
        case "ts1w":
        case "ts1w_m":
        case "ts2w":
        case "ts2w_m":
        case "ts1m":
        case "ts1q":
        case "ts1y":
            $select->group($firstTable . $timeResolution);
            break;
        default:
            throw new Exception("$timeResolution not supported");
        }
        return $select;

    }

    public function getTimeResolutions()
    {
        $resolutions = array();
        foreach ($this->_applicationsUsed as $application) {
            if (empty($resolutions)) {
                $resolutions = $application->getTimeResolutions();
            } else {
                $resolutions = array_intersect_assoc($resolutions, $application->getTimeResolutions());
            }
        }

        return $resolutions;
    }

    public static function cleanValue($v){
        $v = str_replace(' ','',$v);
        $v = str_replace('.','',$v);
        $pos = strpos($v,',');
        if($pos != FALSE){
            $v = substr($v, 0,$pos);
        }
        return (int)$v;
    }

    public static function addData($posId,$cnttime, $data)
    {
        if ($data == null)
            return;
        $posDb = new Pos_Model_ImportJobs();
        $posData = $posDb->find($posId)->current();
        if (strtotime($cnttime) == false) {
            throw new WebReport_Exception("Invalid date and time sent, expecting yyyy-mm-dd HH:MM:SS but got " . ($cnttime ? $cnttime : 'null') . ".");
        }

        $db = Zend_Registry::get('db');
        $dbTime = date('Y-m-d H:i:s', strtotime($cnttime));

        $tableName = "pos_data_pos1";
        //Add column to table if it don't exist
        foreach($data as $k => $v){
            $column_name= "p" . $posId . "_" . $k;
            $select = $db->select()
                ->from("information_schema.columns", array("TABLE_NAME"))
                ->where("table_name = ?", $tableName)
                ->where("column_name = ?", $column_name);
            $col = $db->fetchOne($select);
            if(!$col) {
                $query = "ALTER TABLE " . $tableName . " ADD " . $column_name . " INT(6)";
                $db->query($query);
            }
        }

        //See if item exist or else update
        $select = $db->select()
            ->from($tableName, array($tableName . "_id"))
            ->where("cnttime = ?", $dbTime);

        $id = $db->fetchOne($select);        

        if (!$id) {
            $time = strtotime($cnttime);
 
            $times = Core_Model_CountData::getGroupTimes($dbTime);

            $sql = "INSERT INTO " . $tableName . " (cnttime, ";
            foreach($data as $k => $v){

                $sql .= "p" . $posId . "_" . $k . ", ";
            }
            $sql .= "ts1min, ts5min, ts10min, ts15min, ts30min, ts1h, ts6h, ts12h, ts1d, ts1w, ts1w_m, ts2w, ts2w_m, ts1m, ts1q, ts1y) VALUES ";
            $sql .= "(" . $db->quote($dbTime) . ",";
            foreach($data as $k => $v){
                $clean = self::cleanValue($v);
                $sql .= $db->quote($clean) . ", ";
            }
            $sql .= $db->quote($times['ts1min']) . ", ";
            $sql .= $db->quote($times['ts5min']) . ", ";
            $sql .= $db->quote($times['ts10min']) . ", ";
            $sql .= $db->quote($times['ts15min']) . ", ";
            $sql .= $db->quote($times['ts30min']) . ", ";
            $sql .= $db->quote($times['ts1h']) . ", ";
            $sql .= $db->quote($times['ts6h']) . ", ";
            $sql .= $db->quote($times['ts12h']) . ", ";
            $sql .= $db->quote($times['ts1d']) . ", ";
            $sql .= $db->quote($times['ts1w']) . ", ";
            $sql .= $db->quote($times['ts1w_m']) . ", ";
            $sql .= $db->quote($times['ts2w']) . ", ";
            $sql .= $db->quote($times['ts2w_m']) . ", ";
            $sql .= $db->quote($times['ts1m']) . ", ";
            $sql .= $db->quote($times['ts1q']) . ", ";
            $sql .= $db->quote($times['ts1y']) . ")";
            $db->query($sql);
        } else {
            // update $row->count_data_table_id;
            $sqlComma = "UPDATE " . $db->quoteIdentifier($tableName) . " SET ";
            $sql = null;
            foreach($data as $k => $v){
                $clean = self::cleanValue($v);
                $sql = $sqlComma . $db->quoteIdentifier("p" . $posId . "_" . $k) . " = " . $db->quote($clean);
                $sqlComma = $sql . ", ";
            }
            $sql .= " WHERE " . $db->quoteIdentifier($tableName . "_id") . " = " . $db->quote((int) $id);

            $db->query($sql);
        }
    }
    /**
     * Splits string and inserts data in the string.
     *
     * @param string $data  separated string, if $separator is ":": <user>:<pass>:<version>:<cameraname>:<serial>:<time>:<nbrOfTypes>:<type1>:<data1>[:...]
     * @param string $separator Which string seperates $data fields
     * @return void
     * @throws WebReport_Exception if user/pass to table was wrong or if $string is on wrong format or anything else unexpected happens
     */
    public static function addDataForSeparatedString($data, $separator = "##")
    {
        //$data = explode($separator, $data);
        if (count($data) <= 4) {
            throw new WebReport_Exception("Invalid data. Not in the correct format.");
        }
        $camera = null;
        $posId = $data[0];
        $cnttime = $data[1];
        $nbrOfTypes = $data[2];
        if (strtotime($cnttime) == false) {
            throw new WebReport_Exception("Invalid date and time sent, expecting yyyy-mm-dd HH:MM:SS but got " . ($cnttime ? $cnttime : 'null') . ".");
        }

        $db = Zend_Registry::get('db');

        $dbTime = date('Y-m-d H:i:s', strtotime($cnttime));

        if (!$camera) {
            $types = array();
            for ($i = 0; $i < $nbrOfTypes; $i++) {
                $types[] = $data[3 + ($i * 2)];
            }

            $application = new Core_Model_Application_Pos();
            //$application = $applications->findWithId($chart->application_id);
            //$application = Core_Model_Pos::findWithId(6);

            if (!$application) {
                throw new Exception("Not valid application.");
            }
        }

        $tableName = "pos_data_pos1";

        //Add column to table if it don't exist
        for ($i = 0; $i < $nbrOfTypes; $i++) {
            $column_name= "p" . $posId . "_" . $data[3 + ($i * 2)];
            $select = $db->select()
                ->from("information_schema.columns", array("TABLE_NAME"))
                ->where("table_name = ?", $tableName)
                ->where("column_name = ?", $column_name);
            $col = $db->fetchOne($select);
            if(!$col) {
                $query = "ALTER TABLE " . $tableName . " ADD " . $column_name . " INT(6)";
                $db->query($query);
            }
        }

        //See if item exist or else update
        $select = $db->select()
            ->from($tableName, array($tableName . "_id"))
            ->where("cnttime = ?", $dbTime);

        $id = $db->fetchOne($select);        

        //insert data
        if (!$id) {
            $time = strtotime($cnttime);

            $times = Core_Model_CountData::getGroupTimes($time);
            //$insertTables = $db->fetchCol($db->select()->from($countDataTablesDb->info("name"), array("table")));

            $sql = "INSERT INTO " . $tableName . " (cnttime, ";
            for ($i = 0; $i < $nbrOfTypes; $i++) {
                $sql .= "p" . $posId . "_" . $data[3 + ($i * 2)] . ", ";
            }
            $sql .= "ts1min, ts5min, ts10min, ts15min, ts30min, ts1h, ts6h, ts12h, ts1d, ts1w, ts1w_m, ts2w, ts2w_m, ts1m, ts1q, ts1y) VALUES ";
            $sql .= "(" . $db->quote($dbTime) . ",";
            for ($i = 0; $i < $nbrOfTypes; $i++) {
                $sql .= $db->quote((int) $data[3 + ($i * 2 + 1)]) . ", ";
            }
            $sql .= $db->quote($times['ts1min']) . ", ";
            $sql .= $db->quote($times['ts5min']) . ", ";
            $sql .= $db->quote($times['ts10min']) . ", ";
            $sql .= $db->quote($times['ts15min']) . ", ";
            $sql .= $db->quote($times['ts30min']) . ", ";
            $sql .= $db->quote($times['ts1h']) . ", ";
            $sql .= $db->quote($times['ts6h']) . ", ";
            $sql .= $db->quote($times['ts12h']) . ", ";
            $sql .= $db->quote($times['ts1d']) . ", ";
            $sql .= $db->quote($times['ts1w']) . ", ";
            $sql .= $db->quote($times['ts1w_m']) . ", ";
            $sql .= $db->quote($times['ts2w']) . ", ";
            $sql .= $db->quote($times['ts2w_m']) . ", ";
            $sql .= $db->quote($times['ts1m']) . ", ";
            $sql .= $db->quote($times['ts1q']) . ", ";
            $sql .= $db->quote($times['ts1y']) . ")";
            $db->query($sql);

        } else {
            // update $row->count_data_table_id;
            $sql = "UPDATE " . $db->quoteIdentifier($tableName) . " SET ";
            for ($i = 0; $i < $nbrOfTypes; $i++) {
                if(!isset($data[3 + ($i * 2)]) ||!isset($data[3 + ($i * 2)])) throw new WebReport_Exception("Format was incorrect, too stressfull?");
                $sql .= $db->quoteIdentifier("p" . $posId . "_" . $data[3 + ($i * 2)]) . " = " . $db->quote((int) $data[3 + ($i * 2 + 1)]);
                if ($i < $nbrOfTypes - 1)
                    $sql .= ", ";
            }
            $sql .= " WHERE " . $db->quoteIdentifier($tableName . "_id") . " = " . $db->quote((int) $id);

            $db->query($sql);
        }

        //END FIX

        //$db->update($camerasDb->info('name'), array("last_date" => $dbTime), 'camera_id = ' . $db->quote($cameraId));
    }

    public static function getGroupTimes($timeInstance)
    {
        $times = array();
        $time = (int) $timeInstance;

        $times['ts1min'] = $time - $time % (1 * 60);
        $times['ts5min'] = $time - $time % (5 * 60);
        $times['ts10min'] = $time - $time % (10 * 60);
        $times['ts15min'] = $time - $time % (15 * 60);
        $times['ts30min'] = $time - $time % (30 * 60);
        $times['ts1h'] = $time - $time % (60 * 60);
        $times['ts6h'] = $time - $time % (6 * 60 * 60);
        $times['ts12h'] = $time - $time % (12 * 60 * 60);
        $times['ts1d'] = (int) date("Yz", $time);
        $times['ts1w'] =(int) date("oW", $time
            + (date("w",$time) == 0 ? 24*60*60*7: 0 )); // Sunday
        $times['ts1w_m'] = (int) date("oW", $time);
        $times['ts2w'] = $times['ts1w'] & ~1;
        $times['ts2w_m'] = $times['ts1w_m'] & ~1;
        $times['ts1m'] = (int) (date("Y", $time) * 100 + date("m", $time));
        $times['ts1q'] = (int) (date("Y", $time) . (int) floor(date('m', $time) / 3.1) + 1);
        $times['ts1y'] = (int) date("Y", $time);
        return $times;
    }

            }
