<?

include_once('Pest/PestJSON.php');

class Core_Model_Registration {
    protected $_pest, $_endpoint, $_identifier;

    public function __construct()
    {
        $config = Zend_Registry::get('config');
        if ( isset($config->cogadmin) ) {
            $this->_identifier = $config->cogadmin->identifier;
            $this->_key = $config->cogadmin->key;

            $this->_pest = new Pest($config->cogadmin->endpoint);
            $this->_pest->curl_opts[CURLOPT_TIMEOUT] = 3;
        } else {
            $this->_identifier = "";
            $this->_key = "";

            $this->_pest = new Pest("");
            $this->_pest->curl_opts[CURLOPT_TIMEOUT] = 3;
        }
    }

    /*
     * Check if Web Report is able to connect to the registration server.
     */
    public function isOnline()
    {
        $pest = $this->_pest;

        try {
            $pest->get('/');
        }
        catch(Pest_Exception $e) {
            return FALSE;
        }

        return TRUE;
    }

    /*
     * Checks if Web Report is registered.
     */
    public function isRegistered() 
    {
        if(!self::isOnline() && strlen($this->_key) == 40)
            return true;

        $pest = $this->_pest;

        if($this->_key) {
            try {
                $thing = $pest->get('/web_reports/' . $this->_key);
            }
            catch(Pest_Exception $e) {
                return FALSE;
            }
            return TRUE;
        }

        return FALSE;
    }

    /*
     * Verify public key.
     */
    public function keyIsValid()
    {
        $pest = $this->_pest;

        if(!self::isRegistered())
            return FALSE;

        try {
            $thing = $pest->get('/web_reports/' . $this->_key . '/test_signature');
        } catch(Pest_Exception $e) {
            return FALSE;
        }

        return cm_verify_key($thing);
    }

    /*
     * Register the Web Report
     */
    public function register($api_key)
    {
        $oldKey = $this->_key;

        $this->_key = $api_key;

        $writer = new Core_Model_ConfigWriter(true);
        $writer->config->cogadmin->key = $api_key;
        $writer->save();

        if(!self::isOnline() && strlen($api_key) == 40)
            return TRUE;

        if($this->keyIsValid())
            return TRUE;

        $writer->config->cogadmin->key = $oldKey;
        $writer->save();

        return FALSE;
    }
}
