<?php

class Core_Model_TimeFilterFactory {


    private static $validOperators = array(
// These entries must be sorted by the length of the first value!
        array(">=", ">="),
        array("<=", "<="),
        array("<>", "<>"),
        array("!=", "<>"),
        array("<", "<"),
        array("=", "="),
        array(">", ">"));
    
    private static $validVariables = array(
        array("year", "%Y"),
        array("month", "%m"),
        array("hour", "%k"),
        array("minute", "%i"),
        array("day", "%e"));
    
    private static $validFlags = array(
// Complex
        array("weekfrom", ""),
        array("monthlength", ""),
        // Weekdays
        array("monday", 'DATE_FORMAT(maintime, "%w") = 1'),
        array("tuesday", 'DATE_FORMAT(maintime, "%w") = 2'),
        array("wednesday", 'DATE_FORMAT(maintime, "%w") = 3'),
        array("thursday", 'DATE_FORMAT(maintime, "%w") = 4'),
        array("friday", 'DATE_FORMAT(maintime, "%w") = 5'),
        array("saturday", 'DATE_FORMAT(maintime, "%w") = 6'),
        array("sunday", 'DATE_FORMAT(maintime, "%w") = 0'),
        // Parts of week
        array("weekend", '(DATE_FORMAT(maintime, "%w") > 4 OR DATE_FORMAT(maintime, "%w") = 0)'),
        array("workday", '(DATE_FORMAT(maintime, "%w") <= 4 OR DATE_FORMAT(maintime, "%w") > 0)')
    );
    
    private static $sqlCondition = array("&" => "AND", "|" => "OR");

    private static function getAllMappings() {
        return array(Core_Model_TimeFilterFactory::$validOperators,
        Core_Model_TimeFilterFactory::$validFlags,
        Core_Model_TimeFilterFactory::$validVariables,
        Core_Model_TimeFilterFactory::$sqlCondition);
    }

//fixme  constructor
//fixme set filterCode

    public static function makeFilter($filterCode, $table = null) {
        $tree = Core_Model_TimeFilterFactory::interpretFilter(str_replace(" ", "", $filterCode));
        $where = Core_Model_TimeFilterFactory::makeWhere($tree);
        if($table) return str_replace("maintime", "`$table`.cnttime",$where);
        else return str_replace("maintime", "cnttime",$where);
    }

    public static function isAllowed($eq) {
        foreach (Core_Model_TimeFilterFactory::getAllMappings() as $allowedSymbols) {
            foreach ($allowedSymbols as $op) {
                $eq = str_replace($op[0], "", $eq);
            }
        }

        foreach (array("|","&") as $op) {
            $eq = str_replace($op, "", $eq);
        }
        foreach (array("(",")","[","]") as $op) {
            $eq = str_replace($op, "", $eq);
        }
        foreach (array("0", "1", "2", "3", "4", "5", "6", "7", "8", "9") as $op) {
            $eq = str_replace($op, "", $eq);
        }

        $eq = str_replace(" ", "", $eq);

        return strlen($eq) == 0;
    }

    private static function makeOperand($op) {
        if (is_numeric($op)) {
            return $op;
        } else {
            foreach (Core_Model_TimeFilterFactory::$validVariables as $var) {
                if ($var[0] == $op) {
                    return 'DATE_FORMAT(maintime, "' . $var[1] . '")';
                }
            }
        }
        throw new WebReport_Exception("The value or variable '" . $op . "' was not recognized.");
    }

    private static function makeCondition($cond) {
        if ($cond == "") {
            throw new WebReport_Exception("Could not create time filter from zero-length code.", null, null);
        }
        foreach (Core_Model_TimeFilterFactory::$validOperators as $op) {
            $pos = strpos($cond, $op[0]);
            if ($pos !== false) {
                $x = substr($cond, 0, $pos);
                $y = substr($cond, $pos + strlen($op[0]), strlen($cond) - $pos);
                if (is_numeric($x) && is_numeric($y) || $x == $y) {
                    throw new WebReport_Exception("The condition " . $cond . " is static, thus not allowed.");
                }
                return Core_Model_TimeFilterFactory::makeOperand($x) . " "
                . $op[1] . " "
                . Core_Model_TimeFilterFactory::makeOperand($y);
            }
        }
// Hard cases
        if (substr($cond, 0, 8) == "weekfrom") {
            $day = substr($cond, 9, strlen($cond) - 8 - 2);
            if (!is_numeric($day) || $cond != "weekfrom[" . $day . "]") {
                throw new WebReport_Exception("Could not interpret '" . $cond . "'. Is '");
            }
            if((int)$day < 1 || (int)$day > 28 ){
                throw new WebReport_Exception("The number ".$day." is not a valid day of the month");
            }
            $ret = sprintf('(' .
                    '(DATEDIFF(DATE_FORMAT(DATE_ADD(maintime,INTERVAL -1 MONTH),"%%Y-%%m-%s"),maintime)' .
                    'BETWEEN -6 AND 0)' .
                    ' OR ' .
                    '(DATEDIFF(DATE_FORMAT(maintime,"%%Y-%%m-%s"),maintime) ' .
                    'BETWEEN -6 AND 0))', $day, $day);
            return $ret;
        } 

        foreach (Core_Model_TimeFilterFactory::$validFlags as $op) {
            $pos = strpos($cond, $op[0]);
            if ($pos !== false) {
                return $op[1];
            }
        }

        throw new WebReport_Exception("Could not create boolean condition from '".$cond."'.", null, null);
    }

    public static function canConstruct($eq) {
        try {
            Core_Model_TimeFilterFactory::makeFilter($eq);
        } catch (WebReport_Exception $e) {
            return false;
        }
        return true;
    }

    private static function makeWhere($eq) {
        if (is_string($eq[0])) {
            if (Core_Model_TimeFilterFactory::isAllowed($eq[0])) {
                return Core_Model_TimeFilterFactory::makeCondition($eq[0]);
            } else {
                throw new WebReport_Exception("The expression '" . $eq[0] . "' is not a valid flag or condition.", null, null);
            }
        }
        $res = "";
        foreach ($eq as $p) {
            $exp = $p["exp"];
            $op = $p["op"];
            $res = $res . "(" . Core_Model_TimeFilterFactory::makeWhere($exp) . ")" . $p["op"];
        }
        return $res;
    }

    private static function interpretFilter($code, $depth = "", $all = null) {
        if(strlen($code) == 0){
            throw new WebReport_Exception("Could not create time filter from zero-length code.", null, null);
        }
        $lst = array();
        $depth .= "  ";
        if (is_null($all)) {
            $all = $code;
        }

// Split the expressions by AND, OR and paranthesis 
        $pardepth = 0;
        $haspar = false;
        $start = 0;
        for ($i = 0; $i < strlen($code); $i++) {
            $cur = $code[$i];
            if ($cur == "(") {
                $pardepth++;
                $haspar = true;
            } else if ($cur == ")") {
                $pardepth--;
            }
            if ($pardepth > 0) {
                continue;
            } else if ($pardepth < 0) {
                throw new WebReport_Exception("Too many ending parentheses.", null, null);
            }
            if ($cur == "&" || $cur == "|") {
                // This check is not needed, but will produce better exception message.
                
                if($i == $start){
                    throw new WebReport_Exception("Bad expression before ".$cur.".", null, null);
                }
                $exp = Core_Model_TimeFilterFactory::interpretFilter(substr($code, $start, $i - $start), $depth, $all);
                $lst[] = array("exp" => $exp, "op" => "\n" . $depth . " " . Core_Model_TimeFilterFactory::$sqlCondition[$cur] . " ");
                $start = $i + 1;
            }
        }
        if (count($lst) == 0) {
            if ($haspar) {
                return Core_Model_TimeFilterFactory::interpretFilter(substr($code, 1, strlen($code) - 2), $depth, $all);
            } 
            return array($code);
        } else {
            $exp = Core_Model_TimeFilterFactory::interpretFilter(substr($code, $start, $i - $start), $depth, $all);
            $lst[] = array("exp" => $exp, "op" => "");
            return $lst;
        }
    }

    public function modifySelect(Zend_Db_Select $select) {
        $select->where($this->_where);
    }

}
