<?php

class Core_Model_User extends Zend_Db_Table_Row_Abstract
{
    const WeekStartKey = 'User::DefaultWeekStart';
    const DateFormatKey = 'User::DefaultDateFormat';
    public function increaseLoginCount() {
        $this->logincount++;
        $this->save();
    }

    public function activate() {
        $this->active = 1;
        $this->save();
    }

    public static function SetDefaultDateFormat($date_format_or_company_id = FALSE) {
        if ( is_numeric($date_format_or_company_id ) ) {
            $date_format_or_company_id  = (int)$date_format_or_company_id;
        }
        if ( is_string($date_format_or_company_id) || $date_format_or_company_id  === FALSE ) {
            Zend_Registry::set(self::DateFormatKey, $date_format_or_company_id);
        } else if ( is_int($date_format_or_company_id) ) {
            $compDb = Core_Model_Companies::GetInstance();
            $company = $compDb->find($date_format_or_company_id)->current();
            if ( $company->date_format != "auto" ) {
                Zend_Registry::set(self::DateFormatKey, $company->date_format);
            } else {
                Zend_Registry::set(self::DateFormatKey, FALSE);
            }
        } else {
            throw new Exception("Could not update date format. Only accepts FALSE, string or int.");
        }
    }

    public static function GetCurrentUsersDateFormat($company_id = FALSE) {
        if (Zend_Registry::isRegistered(self::DateFormatKey)) {
            $format = Zend_Registry::get(self::DateFormatKey);
            if($format && $format != "auto") return $format;
        }
        $identity = Zend_Auth::getInstance()->getIdentity();
        $companyDb = Core_Model_Companies::getInstance();
        if ( $company_id === FALSE ) {
            if ( !$identity ) return false;//"yyyy-MM-dd";
            $company_id = $identity->company_id;
        }
        $company = $companyDb->find($company_id)->current();
        if($company->date_format != "auto") {
            return $company->date_format;
        }

        /*
         * Previously we returned false here, and expected the caller to handle unknown formats.
         * Let's not do that anymore.  We adapt to US dates, and just use yyyy-MM-dd for the rest.
         * */
        $locale = Zend_Registry::get("Zend_Locale");
        if ( $locale ) {
            if ( $locale == "en_US" ) {
                return "MM/dd/yyyy";
            }
        }

        return "yyyy-MM-dd";
    }

    public static function SetDefaultStartOfWeek($week_start = 0) {
        Zend_Registry::set(self::WeekStartKey,$week_start);
    }

    public static function GetCurrentUsersStartOfWeek() {
        if(Zend_Registry::isRegistered(self::WeekStartKey)) {
            $start = Zend_Registry::get(self::WeekStartKey);
            if($start == 1) return "mon";
            if($start == 7) return "sun";
        }
        $identity = Zend_Auth::getInstance()->getIdentity();
        $companyDb = Core_Model_Companies::getInstance();
        if($identity) {
            $company = $companyDb->find($identity->company_id)->current();
            if($company->week_start == 1) return "mon";
            if($company->week_start == 7) return "sun";
        }
        return FALSE;
    }

    public function inactivate() {
        $this->active = 0;
        $this->email = null;
        $this->forename = 'Removed';
        $this->surname = 'User';
        $groupsUsersSyncDb = new Core_Model_FoldersUsersSync();
        $user = $groupsUsersSyncDb->getAdapter()->quoteInto('user_id = ?',$this->user_id);
        $groupsUsersSyncDb->delete($user);
        $groupsUsersDb = new Core_Model_FoldersUsers();
        $user = $groupsUsersDb->getAdapter()->quoteInto('user_id = ?',$this->user_id);
        $groupsUsersDb->delete($user);
        
        $this->save();
    }

    public function setLanguage($language)
    {
        $this->language = (string)$language;
        $this->save();
    }


    public function getFullName()
    {
        $name = '';
        if (trim($this->forename) != '') $name = $this->forename;

        if ($name != '' && trim($this->surname) != '') $name .= ' ' . $this->surname;
        if ($name == '' && trim($this->surname) != '') $name .= $this->surname;

        return $name;
    }

    public function getRoleDescription()
    {
        /*$db = new Core_Model_AclRoles();
        $role = $db->fetchRow('acl_role_id = ' . $this->acl_role_id);
        return $role->name;*/
        return null;
    }


    public function toggleActivate()
    {
        $this->active = ($this->active+1)%2;
        $this->save();
    }

    public function setLastLogin()
    {
        $this->last_login = new Zend_Db_Expr('UTC_TIMESTAMP()');
        $this->save();
    }

    public function getLocale()
    {
        $l = $this->language;
        if ($this->country) {
            $l = $l . "_" . strtoupper($this->country);
        }
        return $l;

    }

    public function setPassword($password)
    {
        $dynamicSalt = CM_User::createSalt();
        $this->password = md5(STATIC_SALT . $password . $dynamicSalt);
        $this->password_salt = $dynamicSalt;
        $this->digest_helper_password = md5($this->email . ':' . APPLICATION_REALM . ':' . $password);
    }

    public function setApiKey($apikey)
    {
        $this->apikey = $apikey;
    }

    public function setApiKeyEnable($flag)
    {
        $this->enableapikey=$flag;
    }

    public function __toString()
    {
        return $this->email;
    }
}
