<?php

class Core_Model_Warning extends Zend_Db_Table_Row_Abstract
{
    const CAMERA_TIME_1970 = 'camera_time_1970';
    const CAMERA_NO_DATA = 'camera_no_data';
    const CAMERA_NO_CONNECTION = 'camera_no_connection';
    const CONFIG_NOT_WRITABLE = 'config-not-writable';
    const CAMERA_EXPIRES_SOON = 'camera-expires-soon';
    const CAMERA_NO_CHANNELS_LEFT = 'camera_no_channels_left';

    protected $_camera;

    public function getText($show_html = true)
    {
        $t = Zend_Registry::get('Zend_Translate');
        $view = Zend_Controller_Action_HelperBroker::getStaticHelper('ViewRenderer')->view;

        $camera = $this->getCamera();
        $text = "";
        $url = "";
        if ($camera) {
            $url = $view->url(array("module" => "sources", "controller" => "data-storage-management", "action" => "view-camera", "id" => $camera->camera_id), null, 'default');
        }
        switch ($this->warning) {
        case self::CAMERA_TIME_1970:
            $text = sprintf($t->_('IT_LOOKS_LIKE_THE_TIME_IN_CAMERA_%s_IS_WRONG.'), ($show_html ? '<a class="cameraWarningsHref" title="' . $camera->name .'" "href="' . $url . '">' . $camera->name . '</a>' : $camera->name));
            break;
        case self::CAMERA_NO_DATA:
            $text = sprintf($t->_('%s_HAS_EXCEEDED_YOUR_TIME_LIMIT_FOR_NOT_REPORTING_DATA_TO_DataManager.'), ($show_html ? '<a class="cameraWarningsHref" title="' . $camera->name .'" "href="' . $url . '">' . $camera->name . '</a>' : $camera->name));
            break;
        case self::CAMERA_NO_CONNECTION:
            $text = sprintf($t->_('%s_HAS_EXCEEDED_YOUR_TIME_LIMIT_FOR_NOT_CONNECTING_TO_DataManager.'), ($show_html ? '<a class="cameraWarningsHref" title="' . $camera->name .'" "href="' . $url . '">' . $camera->name . '</a>' : $camera->name));
            break;
        case self::CONFIG_NOT_WRITABLE:
            $text = $t->_('CONFIG_FILE_IS_NOT_WRITABLE._IT_MUST_BE_WRITABLE_FOR_SAVING_WARNINGS.');
            break;
        case self::CAMERA_EXPIRES_SOON:
            $text = sprintf($t->_(_T("THE_CAMERA_'%s'_WILL_EXPIRE_SOON._MAKE_SURE_IT_IS_ACTIVATED_AND_SYNCHRONIZE_YOUR_DataManager_TO_KEEP_GETTING_DATA._IF_YOU_DON'T_WANT_THIS_WARNING_ANYMORE,_DEACTIVATE_THE_CAMERA.")), $camera->name);
            break;
        case self::CAMERA_NO_CHANNELS_LEFT:
            $text = $t->_('THE_CAMERA_CAN_NOT_REPORT_DATA_BECAUSE_THE_NUMBER_OF_CONNECTED_CAMERAS_EXCEEDS_THE_ALLOWED_NUMBER_OF_CHANNELS.');
            break;
        default:
            $text = $t->_("UNKNOWN_WARNING.");
            break;
        }
        return $text;
    }

    public function getTextRaw()
    {
        $t = Zend_Registry::get('Zend_Translate');

        $camera = $this->getCamera();
        $text = "";

        switch($this->warning) {
        case self::CAMERA_TIME_1970:
            $text = $t->_('TIME_ON_CAMERA_IS_WRONG.');
            break;
        case self::CAMERA_NO_DATA:
            $text = $t->_('THE_CAMERA_DATA_IS_NOT_UP_TO_DATE.');
            break;
        case self::CAMERA_NO_CONNECTION:
            $text = $t->_('THE_CAMERA_IS_CURRENTLY_NOT_CONNECTING_TO_DataManager');
            break;
        case self::CONFIG_NOT_WRITABLE:
            $text = $t->_('CONFIG_FILE_IS_NOT_WRITABLE._IT_MUST_BE_WRITABLE_FOR_SAVING_WARNINGS.');
            break;
        case self::CAMERA_EXPIRES_SOON:
            $text = $t->_(_T("THE_CAMERA_WILL_EXPIRE_SOON."));
            break;
        case self::CAMERA_NO_CHANNELS_LEFT:
            $text = $t->_('CAMERAS_EXCEEDS_NUMBER_OF_CHANNELS_ALLOWED.');
        default:
            $text = $t->_("UNKNOWN_WARNING");
            break;
        }

        return array($camera->name, $text, $this->created_date);
    }

    public function setCamera($camera)
    {
        $this->_camera = $camera;
    }

    public function getCamera()
    {
        return $this->_camera;
    }

}
