<?php

class CountData_CountDataSetsController extends WebReport_Controller_Action
{
    protected $_group, $_user;

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();

        $usersDb = new Core_Model_Users();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_COUNT_DATA_SETS');

        if($this->_group && $this->_group->canEdit($this->_user))
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", "folder" => $groupId), "text" => 'ADD_COUNT_DATA_SET');
        
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "merge"), "text" => 'MERGE_COUNT_DATA');

        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
        $this->view->headTitle('COUNT_DATA_SETS', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }


    public function browseAction()
    {
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
            $this->_user, $this->_group, $this->_helper, $this->_request,
            'Core_Model_CountData_Sets',
            'CountData_Model_CountDataSetListGenerator');

        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function addAction()
    {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector("browse", "count-data-sets", "count-data", array("folder" => $this->_group->group_id));
            return;
        }

        if (!$this->getRequest()->isPost()) {
            $this->view->title = $this->_translate->_('ADD_COUNT_DATA_SET') . ": " . sprintf($this->_translate->_('STEP_%1$s'), 1);
            $form = new Admin_Form_AddCountDataSetStep1();
            $tvpcApp = new Core_Model_Application_PeopleCounter();
            $tvpApp = new Core_Model_Application_Parking();
            $this->view->tvpc_id = $tvpcApp->getId();
            $this->view->tvp_id = $tvpApp->getId();
        } else {
            $this->view->title = $this->_translate->_('ADD_COUNT_DATA_SET') . ": " . sprintf($this->_translate->_('STEP_%1$s'), 1);
            $values = $_POST;
            if ($values['step'] == 1) {
                $form = new Admin_Form_AddCountDataSetStep1();
                if ($form->isValid($values)) {
                    Zend_Session::namespaceUnset('WebReport_CountData_Set') ;
                    $countDataSetNamespace = new Zend_Session_Namespace('WebReport_CountData_Set');
                    $countDataSetNamespace->application_id = $values['application_id'];
                    $countDataSetNamespace->name = $values['name'];
                    $countDataSetNamespace->description = $values['description'];
                    
                    unset($form);

                    $applicationsModel = new Core_Model_Applications();
                    $application = $applicationsModel->findWithId($countDataSetNamespace->application_id);

                    $form = new Admin_Form_AddCountDataSetStep2($application, $this->_user);
                    $this->view->dataAvailable = $form->dataAvailable();
                    $this->view->title = $this->_translate->_('ADD_COUNT_DATA_SET') . ": " . sprintf($this->_translate->_('STEP_%1$s'), 2);

                } else {
                    $form->populate($values);
                }
            } else { // step 2
                $this->view->title = $this->_translate->_('ADD_COUNT_DATA_SET') . ": " . sprintf($this->_translate->_('STEP_%1$s'), 2);
                $values = $_POST;

                $countDataSetNamespace = new Zend_Session_Namespace('WebReport_CountData_Set');

                $applicationsModel = new Core_Model_Applications();
                $application = $applicationsModel->findWithId($countDataSetNamespace->application_id);

                $form = new Admin_Form_AddCountDataSetStep2($application, $this->_user);

                $cameraIds = array();
                if (isset($values['camera_id'])) { $cameraIds = $values['camera_id']; }
                $virtualCameraIds = array();
                if (isset($values['virtual_camera_id'])) { $virtualCameraIds = $values['virtual_camera_id']; }
                $formulaSetIds = array();
                if (isset($values['count_data_formula_set_id'])) { $formulaSetIds = $values['count_data_formula_set_id']; }

                if (empty($cameraIds) && empty($virtualCameraIds) && empty($formulaSetIds)) {
                    $form->addError('NO_DATA_IS_SELECTED.');
                    $this->view->form = $form;
                    return;
                }

                $companiesDb = new Core_Model_Companies();
                $camerasDb = new Core_Model_Cameras();
                $virtualCamerasDb = new Core_Model_VirtualCameras();
                $formulaSetsDb = new Core_Model_CountData_FormulaSets();
                $applicationsModel = new Core_Model_Applications();

                $countDataDb = new Core_Model_CountData();

                $company = $companiesDb->find($countDataSetNamespace->company_id)->current();

                $allowedCameraIds = array_keys($camerasDb->fetchAllVisibleNames($this->_user, $application));
                if (!empty($cameraIds)) {
                    $cameras = $camerasDb->find($cameraIds);
                    foreach ($cameras as $camera) 
                        if (in_array($camera->camera_id, $allowedCameraIds)) 
                            $countDataDb->addCamera($camera);
                }
                
                $allowedVirtualCameraIds = array_keys($virtualCamerasDb->fetchAllVisibleNames($this->_user, $application));
                if (!empty($virtualCameraIds)) {
                    $virtualCameras = $virtualCamerasDb->find($virtualCameraIds);
                    foreach ($virtualCameras as $virtualCamera) 
                        if(in_array($virtualCamera->virtual_camera_id, $allowedVirtualCameraIds))
                            $countDataDb->addVirtualCamera($virtualCamera);
                }

                $allowedFormulaSetIds = array_keys($formulaSetsDb->fetchAllVisibleNames($this->_user, $application));
                if (!empty($formulaSetIds)) {
                    $formulaSets = $formulaSetsDb->find($formulaSetIds);
                    foreach ($formulaSets as $formulaSet)
                        if(in_array($formulaSet->count_data_formula_set_id, $allowedFormulaSetIds))
                            $countDataDb->addFormulaSet($formulaSet);
                }

                if (empty($cameraIds) && empty($virtualCameraIds) && empty($formulaSetIds)) {
                    $form->addError('NO_DATA_IS_SELECTED.');
                    $this->view->form = $form;
                    return;
                }

                try {
                    $countDataDb->initDatabaseSelect();
                } catch (WebReport_Exception_TooManyJoins $e) {
                    $form->addError("TOO_MUCH_DATA_SELECTED._DESELECT_A_FEW_CAMERAS_AND_TRY_AGAIN.");
                    $form->populate($values);
                    $this->view->form = $form;
                    return;
                }

                $countDataSetNamespace = new Zend_Session_Namespace('WebReport_CountData_Set');

                $applicationId = $countDataSetNamespace->application_id;
                $name = $countDataSetNamespace->name;
                $companyId = $countDataSetNamespace->company_id;
                $description = $countDataSetNamespace->description;
    
                $setsDb = new Core_Model_CountData_Sets();
                $setData = array(
                    'application_id' => $applicationId,
                    'name' => $name,
                    'company_id' => $this->_group->company_id,
                    'description' => $description,
                    'created_by' => $this->_user->user_id,
                    'created_date' => new Zend_Db_Expr("NOW()")
                    );
                $set = $setsDb->createRow($setData);
                $set->save();
                Core_Model_DocumentFactory::add($set->count_data_set_id, "count_data_set", $this->_group);

                $setId = $set->count_data_set_id;
                
                $cdsCamerasDb = new Core_Model_CountData_Set_Cameras();
                foreach ($cameraIds as $cameraId) {
                    $row = $cdsCamerasDb->createRow(array(
                                                        'count_data_set_id' => $setId,
                                                        'camera_id' => $cameraId
                                                        ));
                    $row->save();
                }

                $cdsFormulaSetsDb = new Core_Model_CountData_Set_FormulaSets();
                foreach ($formulaSetIds as $formulaSetId) {
                    $row = $cdsFormulaSetsDb->createRow(array(
                                                            'count_data_set_id' => $setId,
                                                            'count_data_formula_set_id' => $formulaSetId
                                                            ));
                    $row->save();
                }

                $cdsVirtualCamerasDb = new Core_Model_CountData_Set_VirtualCameras();
                foreach ($virtualCameraIds as $virtualCameraId) {
                    $row = $cdsVirtualCamerasDb->createRow(array(
                                                               'count_data_set_id' => $setId,
                                                               'virtual_camera_id' => $virtualCameraId
                                                               ));
                    $row->save();
                }


                Zend_Session::namespaceUnset("WebReport_CountData_Set");
                $this->_helper->_redirector("browse", "count-data-sets", "count-data", array('folder' => $this->_group->group_id));

                exit;

            }
        }

        $this->view->form = $form;
    }

    public function editAction()
    {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_COUNT_DATA_SET.'));
            $this->_helper->redirector("browse", "count-data-sets", "count-data", array("folder" => $this->_group->group_id));
            return;
        }

        $id = (int)$this->_getParam("id");
        if (!$id) {
            throw new WebReport_Exception_ParameterError("CHART_NOT_FOUND");
        }

        $this->view->title = $this->_translate->_("EDIT");
        $this->view->headTitle("EDIT", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $setsDb = new Core_Model_CountData_Sets();
        $set = $setsDb->find($id)->current();

        if (!$set || (!$this->_user->multi_company && $this->_user->company_id != $set->company_id)) {
            throw new WebReport_Exception_ParameterError($this->_translate->_("COUNT_DATA_SET_NOT_FOUND."));
        }

        $applicationsModel = new Core_Model_Applications();
        $application = $applicationsModel->findWithId($set->application_id);

        $form = new Admin_Form_EditCountDataSet($application, $this->_user);

        $camerasDb = new Core_Model_Cameras();
        $virtualCamerasDb = new Core_Model_VirtualCameras();
        $formulaSetsDb = new Core_Model_CountData_FormulaSets();

        if ($this->getRequest()->isPost()) {

            if ($form->isValid($_POST)) {
                $values = $form->getValues();

                $countDataDb = new Core_Model_CountData();

                $cameraIds = array();
                if (isset($values['camera_id'])) { $cameraIds = $values['camera_id']; }
                $virtualCameraIds = array();
                if (isset($values['virtual_camera_id'])) { $virtualCameraIds = $values['virtual_camera_id']; }
                $formulaSetIds = array();
                if (isset($values['count_data_formula_set_id'])) { $formulaSetIds = $values['count_data_formula_set_id']; }        
                
                if (empty($cameraIds) && empty($virtualCameraIds) && empty($formulaSetIds)) {
                    $form->addError('NO_DATA_IS_SELECTED.');
                    $form->populate($values);
                    $this->view->form = $form;
                    return;
                }

                $allowedCameraIds = array_keys($camerasDb->fetchAllVisibleNames($this->_user, $application));
                if (!empty($cameraIds)) {
                    $cameras = $camerasDb->find($cameraIds);
                    foreach ($cameras as $camera)
                        if(in_array($camera->camera_id, $allowedCameraIds))
                            $countDataDb->addCamera($camera);
                }

                $allowedVirtualCameraIds = array_keys($virtualCamerasDb->fetchAllVisibleNames($this->_user, $application));
                if (!empty($virtualCameraIds)) {
                    $virtualCameras = $virtualCamerasDb->find($virtualCameraIds);
                    foreach ($virtualCameras as $virtualCamera)
                        if(in_array($camera->camera_id, $allowedVirtualCameraIds))
                            $countDataDb->addVirtualCamera($virtualCamera);
                }

                $allowedFormulaSetIds = array_keys($formulaSetsDb->fetchAllVisibleNames($this->_user, $application));
                if (!empty($formulaSetIds)) {
                    $formulaSets = $formulaSetsDb->find($formulaSetIds);
                    foreach ($formulaSets as $formulaSet)
                        if(in_array($formulaSet->count_data_formula_set_id, $allowedFormulaSetIds))
                            $countDataDb->addFormulaSet($formulaSet);
                }

                try {
                    $countDataDb->initDatabaseSelect();
                } catch (WebReport_Exception_TooManyJoins $e) {
                    $form->addError("TOO_MUCH_DATA_SELECTED._DESELECT_A_FEW_CAMERAS_AND_TRY_AGAIN.");
                    $form->populate($values);
                    $this->view->form = $form;
                    return;
                }


                $set->name = $values['name'];
                $set->description = $values['description'];

                $set->save();
                $setId = $set->count_data_set_id;

                $ccCamerasDb = new Core_Model_CountData_Set_Cameras();
                $ccCamerasDb->delete($ccCamerasDb->getAdapter()->quoteInto('count_data_set_id = ?', $setId));
                foreach ($cameraIds as $cameraId) {
                    $row = $ccCamerasDb->createRow(array(
                                                       'count_data_set_id' => $setId,
                                                       'camera_id' => $cameraId
                                                       ));
                    $row->save();
                }

                $ccFormulaSetsDb = new Core_Model_CountData_Set_FormulaSets();
                $ccFormulaSetsDb->delete($ccFormulaSetsDb->getAdapter()->quoteInto('count_data_set_id = ?', $setId));
                foreach ($formulaSetIds as $formulaSetId) {
                    $row = $ccFormulaSetsDb->createRow(array(
                                                           'count_data_set_id' => $setId,
                                                           'count_data_formula_set_id' => $formulaSetId
                                                           ));
                    $row->save();
                }

                $ccVirtualCamerasDb = new Core_Model_CountData_Set_VirtualCameras();
                $ccVirtualCamerasDb->delete($ccVirtualCamerasDb->getAdapter()->quoteInto('count_data_set_id = ?', $setId));
                foreach ($virtualCameraIds as $virtualCameraId) {
                    $row = $ccVirtualCamerasDb->createRow(array(
                                                              'count_data_set_id' => $setId,
                                                              'virtual_camera_id' => $virtualCameraId
                                                              ));
                    $row->save();
                }

                $this->_flashMessenger->addMessage($this->_translate->_('COUNT_DATA_SET_UPDATED.'));
                $this->_helper->redirector('browse','count-data-sets','count-data', array('folder' => $this->_group->group_id));
            } else {
                $form->populate($_POST);
            }
        } else {
            $pop = array();
            $pop += $set->toArray();

            $cameraRows = $set->findManyToManyRowset('Core_Model_Cameras', 'Core_Model_CountData_Set_Cameras');
            foreach ($cameraRows as $row) { $pop['camera_id'][] = $row->camera_id; }
            $virtualCameraRows = $set->findManyToManyRowset('Core_Model_VirtualCameras', 'Core_Model_CountData_Set_VirtualCameras');
            foreach ($virtualCameraRows as $row) { $pop['virtual_camera_id'][] = $row->virtual_camera_id; }            
            $formulaSetRows = $set->findManyToManyRowset('Core_Model_CountData_FormulaSets', 'Core_Model_CountData_Set_FormulaSets');
            foreach ($formulaSetRows as $row) { $pop['count_data_formula_set_id'][] = $row->count_data_formula_set_id; }

            /*
             * check if cameras, virtual cameras and formula sets are all viewable by editing user.
             */
            $allowedCameraIds = array_keys($camerasDb->fetchAllVisibleNames($this->_user, $application));
            $allowedVirtualCameraIds = array_keys($virtualCamerasDb->fetchAllVisibleNames($this->_user, $application));
            $allowedFormulaSetIds = array_keys($formulaSetsDb->fetchAllVisibleNames($this->_user, $application));

            $canEdit = true;

            if(count(($inBoth = array_intersect($pop['camera_id'], $allowedCameraIds))) != count($pop['camera_id'])) {
                $canEdit = false;
                
                foreach(array_diff($pop['camera_id'], $inBoth) as $camId) {
                    $camName = $camerasDb->find($camId)->current()->name;
                    $this->_flashMessenger->addMessage($this->_translate->_('CAMERA_NOT_VISIBLE_TO_YOU:_') . $camName);
                }
            }

            if(count(($inBoth = array_intersect($pop['virtual_camera_id'], $allowedVirtualCameraIds))) != count($pop['virtual_camera_id'])) {
                $canEdit = false;
                
                foreach(array_diff($pop['virtual_camera_id'], $inBoth) as $camId) {
                    $camName = $virtualCamerasDb->find($camId)->current()->name;
                    $this->_flashMessenger->addMessage($this->_translate->_('VIRTUAL_CAMERA_NOT_VISIBLE_TO_YOU:_') . $camName);
                }
            }

            if(count(($inBoth = array_intersect($pop['count_data_formula_set_id'], $allowedFormulaSetIds))) != count($pop['count_data_formula_set_id'])) {
                $canEdit = false;
                
                foreach(array_diff($pop['count_data_formula_set_id'], $inBoth) as $camId) {
                    $camName = $formulaSetsDb->find($camId)->current()->name;
                    $this->_flashMessenger->addMessage($this->_translate->_('FORMULA_SET_NOT_VISIBLE_TO_YOU:_') . $camName);
                }
            }

            if(!$canEdit) {
                $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_COUNT_DATA_SET_BECAUSE_IT_CONTAINS_CAMERAS_THAT_ARE_NOT_VISIBLE_TO_YOU.'));
                $this->_helper->redirector('browse','count-data-sets','count-data', array('folder' => $this->_group->group_id));
            }

            $form->populate($pop);
        }

        $this->view->form = $form;
    }


    public function deleteAction()
    {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_REMOVE_THIS_COUNT_DATA_SET.'));
            $this->_helper->redirector("browse", "count-data-sets", "count-data", array("folder" => $this->_group->group_id));
            return;
        }

        $setId = (int)$this->_getParam("id");

        $setsDb = new Core_Model_CountData_Sets();
        $set = $setsDb->find($setId)->current();
        if (!$set)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COUNT_DATA_SET_WITH_SUCH_ID_EXISTS.'));

        if (!$this->_user->multi_company && $this->_user->company_id != $set->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_COUNT_DATA_SET_WITH_SUCH_ID_EXISTS.'));

        //see if it is used in a chart
        $db = Zend_Registry::get('db');
        $select =  $db->select()->from("mod_charts_data_source_camera")->where("count_data_set_id = ?", $setId);
        $cdsInCharts = $db->fetchAll($select);
        if(count($cdsInCharts) > 0 ){
            $this->_flashMessenger->addMessage($this->_translate->_('Count data set could not be removed since it is used in a chart'));
            $this->_helper->redirector('browse','count-data-sets','count-data', array('folder' => $this->_group->group_id));
        }
 
        //see if it is used in a table
        $db = Zend_Registry::get('db');
        $select =  $db->select()->from("table_section_entry");
        $entries = $db->fetchAll($select);
        foreach($entries as $e){
            $selections = json_decode($e->selections_json);
            foreach($selections as $s){
                if($s[4] == "CDS"){
                    if($s[1] == $setId){
                        //found in table!
                        $this->_flashMessenger->addMessage($this->_translate->_('Count data set could not be removed since it is used in a table'));
                        $this->_helper->redirector('browse','count-data-sets','count-data', array('folder' => $this->_group->group_id));
                    }
                }
            }
        }

        //$set->removeDependencies(); Did not find any meaning with this
        $this->_flashMessenger->addMessage($this->_translate->_('COUNT_DATA_SET_DELETED.'));

        Core_Model_DocumentFactory::remove($set->count_data_set_id, "count_data_set");
        $set->delete();

        $this->_helper->redirector('browse','count-data-sets','count-data', array('folder' => $this->_group->group_id));
    }


    public function viewAction()
    {

        $id = (int)$this->_getParam("id");
        if (!$id) {
            throw new WebReport_Exception_ParameterError("CHART_NOT_FOUND");
        }

        $this->view->headTitle("VIEW", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $setsDb = new Core_Model_CountData_Sets();
        $set = $setsDb->find($id)->current();

        if (!$set || (!$this->_user->multi_company && $this->_user->company_id != $set->company_id)) {
            throw new WebReport_Exception_ParameterError($this->_translate->_("COUNT_DATA_SET_NOT_FOUND."));
        }

        $this->view->headTitle($set->name, Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $applicationsModel = new Core_Model_Applications();
        $application = $applicationsModel->findWithId($set->application_id);
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($set->company_id)->current();

        $this->view->name = $set->name;
        $this->view->company = $company->name;
        $this->view->application = $application->getName();

        $this->view->cameras = $set->findManyToManyRowset('Core_Model_Cameras', 'Core_Model_CountData_Set_Cameras');
        $this->view->virtualCameras = $set->findManyToManyRowset('Core_Model_VirtualCameras', 'Core_Model_CountData_Set_VirtualCameras');
        $this->view->formulaSets = $set->findManyToManyRowset('Core_Model_CountData_FormulaSets', 'Core_Model_CountData_Set_FormulaSets');
    }

    /**
     * Ajax functions
     */

    public function getSetsForCompanyAction() {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        if (!$this->getRequest()->isPost()) {
            echo "[]";
            return;
        }

        $companyId = $this->_getParam('companyId');

        if (!$this->_user->multi_company && $this->_user->company_id != $companyId) {
            echo "[]";
            return;
        }

        $setsDb = new Core_Model_CountData_Sets();
        
        $sets = $setsDb->fetchAll($setsDb->select()->where('company_id = ?', $companyId)->order('application_id')->order("name"));
        $ret = array();
        $applicationsModel = new Core_Model_Applications();
        foreach ($sets as $set) {
            $ret[$applicationsModel->findWithId($set->application_id)->getName()][] = array("id" => $set->count_data_set_id, "name" => $set->name);
        }
        echo Zend_Json::encode($ret);

    }

    public function getTimeResolutionsForSetAction() {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        if (!$this->getRequest()->isPost()) {
            echo "[]";
            return;
        }

        $setId = (int)$this->_getParam('countDataSetId');
        $setsDb = new Core_Model_CountData_Sets();
        
        $set = $setsDb->find($setId)->current();
        if (!$set) {
            echo "[]";
            return;
        }        
        $ret = array();
        $applicationsModel = new Core_Model_Applications();
                
        $app = $applicationsModel->findWithId($set->application_id);
        if (!$app) {
            echo "[]";
            return;
        }
        
        $ret = array();
        foreach ($app->getTimeResolutions() as $k => $v) {
            $ret[$k] = $v;
        }
        
        echo Zend_Json::encode($ret);
    }


    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {
        $this->_translate->_("BROWSE_COUNT_DATA_SETS");
        $this->_translate->_("HANDLE_COUNT_DATA_SETS");

    }



}
