<?php

class CountData_FiltergroupsController extends WebReport_Controller_Action
{

	public function init()
	{
		parent::init();

		$module = $this->getRequest()->module;
		$controller = $this->getRequest()->controller;
		$action = $this->getRequest()->action;

		$items = array();
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_FILTER_GROUPS');
		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add"), "text" => 'ADD_FILTER_GROUP');

		$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
		$this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
		$this->view->headTitle("FILTER_GROUPS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
	}

	public function browseAction()
	{

		$groups = new Core_Model_CountData_FilterGroups();
		if (Zend_Auth::getInstance()->getIdentity()->multi_company) {
			$this->view->groups = $groups->fetchAll();
		} else {
			$this->view->groups = $groups->fetchAll($groups->select()->where('company_id = ?', Zend_Auth::getInstance()->getIdentity()->company_id));
		}

		if ($this->_flashMessenger->hasMessages())
			$this->view->messages = $this->_flashMessenger->getMessages();

	}
  

	public function deleteAction() {
		$groupId = (int)$this->_getParam("id");

		$groupsDb = new Core_Model_CountData_FilterGroups();
		$group = $groupsDb->find($groupId)->current();
		if (!$group) 
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FILTER_GROUP_WITH_SUCH_ID_EXISTS.'));

		if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $group->company_id)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FILTER_GROUP_WITH_SUCH_ID_EXISTS.'));



		$this->_flashMessenger->addMessage($this->_translate->_('FILTER_GROUP_DELETED.'));
		$group->delete();
		$this->_helper->redirector('browse','filter-groups','count-data');
	}

  
	public function addAction()
	{

		$this->view->headTitle("ADD", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		if (!$this->getRequest()->isPost()) {
			$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'),1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

			$form = new Admin_Form_AddFilterGroup();
			$this->view->form = $form;
			$this->render("add-step1");
		} else { // request is post
			if ($_POST['step'] == 1) {

				$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'),1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
				$form = new Admin_Form_AddFilterGroup();

				if ($form->isValid($_POST)) {
					$values = $form->getValues();

					$filterGroupNamespace = new Zend_Session_Namespace('WebReport_FilterGroup');
					$filterGroupNamespace->application_id = $values['application_id'];
					$filterGroupNamespace->name = $values['name'];
					$filterGroupNamespace->company_id = $values['company_id'];
					$filterGroupNamespace->description = $values['description'];

					$identity = Zend_Auth::getInstance()->getIdentity();
					if (!$identity->multi_company) {
						$filterGroupNamespace->company_id = $identity->company_id;
					}

					$applicationsModel = new Core_Model_Applications();
					$companyDb = new Core_Model_Companies();
					
					$application = $applicationsModel->findWithId($filterGroupNamespace->application_id);
					$company = $companyDb->find($filterGroupNamespace->company_id)->current();

					$this->view->applicationName = $application->getShortName();
					$this->view->filterGroupName = $filterGroupNamespace->name;
					$this->view->company = $company->name;

					$this->view->headScript()->appendFile($this->view->baseUrl('/modules/admin/javascript/jquery/plugins/jquery.countDataFilters.js'));

					$this->view->headScript()->captureStart(); ?>
$(document).ready(function() {
	$("#fieldset-filters").countDataFilters({
		descriptionsUrl : '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'filter-groups', 'action' => 'get-filter-parts-descriptions'),null,true); ?>',
		conditionsUrl : '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'filter-groups', 'action' => 'get-filter-part-conditions'),null,true); ?>',
		conditionElementsUrl : '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'filter-groups', 'action' => 'get-filter-part-condition-elements-data'),null,true); ?>'
	});
    $(".resetButton").click(function () {
		window.location = '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'filter-groups', 'action' => 'browse')); ?>';
	});
}); <?php			$this->view->headScript()->captureEnd();

					$this->render("add-step2");

				} else {

					$form->populate($_POST);
					$this->view->form = $form;
					$this->render("add-step1");

				}

			} else { // step == 2
				$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'),2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

				$filterGroupNamespace = new Zend_Session_Namespace('WebReport_FilterGroup');
				$applicationId = $filterGroupNamespace->application_id;
				$companyId = $filterGroupNamespace->company_id;
				$name = $filterGroupNamespace->name;
				$description = $filterGroupNamespace->description;

				$values = $_POST;


				$userData = Zend_Auth::getInstance()->getIdentity();

				$filterGroupsDb = new Core_Model_CountData_FilterGroups();
				$filterGroupData = array(
					'name' => $name,
					'description' => $description,
					'application_id' => $applicationId,
					'company_id' => $companyId,
					'filters' => serialize($values['filters']),
					'created_by' => $userData->user_id,
					'created_date' => new Zend_Db_Expr('NOW()')
				);
				$filterGroup = $filterGroupsDb->createRow($filterGroupData);
				$filterGroup->save();

				unset($filterGroupNamespace);

				$this->_flashMessenger->addMessage($this->_translate->_('FILTER_GROUP_ADDED.'));
				$this->_helper->redirector('browse','filter-groups','count-data');

				
			}
		}
	}

	public function editAction()
	{

		$form = new Admin_Form_EditFilterGroup();
     
		$groupId = (int)$this->_request->getParam('id');
		$groups = new Core_Model_CountData_FilterGroups();
		$group = $groups->find($groupId)->current();

		if (!$group)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FILTER_GROUP_WITH_SUCH_ID_EXISTS.'));

		if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $group->company_id)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FILTER_GROUP_WITH_SUCH_ID_EXISTS.'));

		$this->view->headTitle(sprintf($this->_translate->_('EDIT_"%1$s"'), $group->name), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		$this->view->headScript()->appendFile($this->view->baseUrl('/modules/admin/javascript/jquery/plugins/jquery.countDataFilters.js'));

		if ($this->getRequest()->isPost()) {

			if ($form->isValid($_POST)) {
				$values = $_POST;
				$group->name = $values['name'];
				$group->description = $values['description'];
				$group->filters = serialize($values['filters']);
				$group->save();

				$this->_flashMessenger->addMessage($this->_translate->_('FILTER_GROUP_UPDATED.'));
				$this->_helper->redirector('browse','filter-groups','count-data');

			}

		} else {

			$applicationsModel = new Core_Model_Applications();
			$application = $applicationsModel->findWithId($group->application_id);
			$form->getElement("application")->setValue($application->getName());

			$companiesDb = new Core_Model_Companies();
			$company = $companiesDb->find($group->company_id)->current();
			$form->getElement("company")->setValue($company->name);

			$this->view->headScript()->captureStart(); ?>
$(document).ready(function() {
	$("#fieldset-filters").countDataFilters({
		descriptionsUrl : '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'filter-groups', 'action' => 'get-filter-parts-descriptions'),null,true); ?>',
		conditionsUrl : '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'filter-groups', 'action' => 'get-filter-part-conditions'),null,true); ?>',
		conditionElementsUrl : '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'filter-groups', 'action' => 'get-filter-part-condition-elements-data'),null,true); ?>',
		companyId : <?= $group->company_id; ?>,
		applicationId : <?= $group->application_id; ?>,
		filters : <?= Zend_Json::encode(unserialize($group->filters)); ?>
	});
});
<?php		$this->view->headScript()->captureEnd();

			$form->populate($group->toArray());
		}
      
		$this->view->form = $form;
	}
  
	public function viewAction() {
		$filterGroupId = (int)$this->_request->getParam('id');

		$filterGroups = new Core_Model_CountData_FilterGroups();
		$filterGroup = $filterGroups->find($filterGroupId)->current();

		if (!$filterGroup)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FILTER_GROUP_WITH_SUCH_ID_EXISTS.'));

		if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $filterGroup->company_id)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FILTER_GROUP_WITH_SUCH_ID_EXISTS.'));


		$this->view->headTitle($filterGroup->name, Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		$this->view->filterGroup = $filterGroup;

		//Zend_Debug::dump(unserialize($filterGroup->filters));
		//die();
		//$this->view->groupFilters = $groupFilters;
		//$this->view->filters = CountDataFilters::getFilters();

}

 
  
  
	/*
	 *  AJAX Functions
	 */

	public function getFilterPartsDescriptionsAction()
	{
		$this->_helper->layout->disableLayout();    //disable layout
		$this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

		if (!$this->getRequest()->isPost()) {
			echo "[]";
			return;
		}

		$filterGroupNamespace = new Zend_Session_Namespace('WebReport_FilterGroup');

		$companyId = $_POST['companyId'];
		if ($companyId <= 0) $companyId = $filterGroupNamespace->company_id;

		$applicationId = $_POST['applicationId'];
		if ($applicationId <= 0) $applicationId = $filterGroupNamespace->application_id;

		$applicationsModel = new Core_Model_Applications();
		$companiesDb = new Core_Model_Companies();
		$application = $applicationsModel->findWithId($applicationId);
		$company = $companiesDb->find($companyId)->current();

		$descriptions = Core_Model_CountData_Filter_Parts::getDescriptions(true, $company, $application);

		$json = Zend_Json::encode($descriptions);
		echo $json;
		unset($json);
		unset($descriptions);
	}

	public function getFilterPartConditionsAction()
	{
  
		$this->_helper->layout->disableLayout();    //disable layout
		$this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

		if (!$this->getRequest()->isPost()) {
			echo "[]";
			return;
		}

		$filterGroupNamespace = new Zend_Session_Namespace('WebReport_FilterGroup');

		$companyId = (int)$_POST['companyId'];
		if ($companyId <= 0) $companyId = $filterGroupNamespace->company_id;
		$applicationId = (int)$_POST['applicationId'];
		if ($applicationId <= 0) $applicationId = $filterGroupNamespace->application_id;

		$applicationsModel = new Core_Model_Applications();
		$companiesDb = new Core_Model_Companies();
		$application = $applicationsModel->findWithId($applicationId);
		$company = $companiesDb->find($companyId)->current();

		$filterPartId = $_POST['filterId'];

		$conditions = Core_Model_CountData_Filter_Parts::getConditions($filterPartId, true, true, $company, $application);

		$json = Zend_Json::encode($conditions);
		echo $json;
		unset($conditions);
		unset($json);
	}
  
	public function getFilterPartConditionElementsDataAction()
	{
  
		$this->_helper->layout->disableLayout();    //disable layout
		$this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering
  
		if (!$this->getRequest()->isPost()) {
			echo "[]";
			return;
		}

		$filterGroupNamespace = new Zend_Session_Namespace('WebReport_FilterGroup');
		$companyId = $_POST['companyId'];
		if ($companyId <= 0) $companyId = $filterGroupNamespace->company_id;
		$applicationId = $_POST['applicationId'];
		if ($applicationId <= 0) $applicationId = $filterGroupNamespace->application_id;

		$applicationsModel = new Core_Model_Applications();
		$companiesDb = new Core_Model_Companies();
		$application = $applicationsModel->findWithId($applicationId);
		$company = $companiesDb->find($companyId)->current();
		
		$filterPartId = $_POST['filterId'];
		$conditionId = $_POST['conditionId'];
		
		$filter = Core_Model_CountData_Filter_Parts::getPart($filterPartId);
		
		$conditionTypes = $filter->getConditionElements($conditionId, $company, $application);

		$json = Zend_Json::encode($conditionTypes);
		echo $json;
		unset($conditionTypes, $filter, $json);
	}

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */  
	private function _translatableStrings() {
		$this->_translate->_('FILTER_GROUP_NAME');
		$this->_translate->_('COMPANY');
		$this->_translate->_('DESCRIPTION');

		$this->_translate->_('BROWSE_FILTER_GROUPS');
		$this->_translate->_('ADD_FILTER_GROUP');
		$this->_translate->_('FILTER_GROUP_COMPANY');
		
		$this->_translate->_('SELECT_A_COMPANY_FOR_THE_FILTER_GROUP.');
		$this->_translate->_('YOU_MUST_CHOOSE_ONE_OF_THE_TRUEVIEW_APPLICATIONS.');
		$this->_translate->_('SELECT_AN_APPLICATION_FOR_THE_FILTER_GROUP.');
		$this->_translate->_('FILTER_GROUP_DESCRIPTION');

	}

}