<?php

class CountData_FormulasController extends WebReport_Controller_Action
{
    protected $_user, $_group;

	public function init()
	{
		parent::init();

		$module = $this->getRequest()->module;
		$controller = $this->getRequest()->controller;
		$action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();

        $usersDb = new Core_Model_Users();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

		$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_FORMULA_SETS');
	
        if($this->_group && $this->_group->canEdit($this->_user))
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", 'folder' => $groupId), "text" => 'ADD_FORMULA_SET');

		$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
		$this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
		$this->view->headTitle('FORMULA_SETS', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
	}


	public function browseAction()
	{
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
            $this->_user, $this->_group, $this->_helper, $this->_request,
            'Core_Model_CountData_FormulaSets',
            'CountData_Model_FormulaSetListGenerator');
        
        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
	}

	public function addAction()
	{
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector('browse','formulas','count-data', array('folder' => $this->_group->group_id));
            return;
        }

		$this->view->headTitle("ADD", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

		if (!$this->getRequest()->isPost()) {
			$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'),1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

			$form = new Admin_Form_AddFormulaSet();
			$this->view->form = $form;
			$this->render("add-step1");

		} else { // request is post
			if ($_POST['step'] == 1) {

				$form = new Admin_Form_AddFormulaSet();

				if ($form->isValid($_POST)) {
					$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
					$values = $form->getValues();

					$formulaNamespace = new Zend_Session_Namespace('WebReport_Formula');
					$formulaNamespace->application_id = $values['application_id'];
					$formulaNamespace->name = $values['name'];
					$formulaNamespace->description = $values['description'];

					$applicationsModel = new Core_Model_Applications();

					$application = $applicationsModel->findWithId($formulaNamespace->application_id);
					
					$this->view->applicationName = $application->getShortName();
					$this->view->formulaName = $formulaNamespace->name;
					$this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/jquery/plugins/jquery.livequery.js'));
					$this->view->headScript()->appendFile($this->view->baseUrl('/modules/admin/javascript/jquery/plugins/jquery.formulaEditor.js'));

					$camerasDb = new Core_Model_Cameras();

					$this->view->headScript()->captureStart(); ?>
$(document).ready(function() {
	$(".resetButton").click(function () {
		window.location = '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'formulas', 'action' => 'browse')); ?>';
	});

	$("#formula").formulaEditor({
		cameras : <?= Zend_Json::encode($camerasDb->fetchAllVisibleNames($this->_user, $application)); ?>,
		types : <?= Zend_Json::encode($application->getOutTypes()); ?>,
		form : $("form.uniForm"),
		missedElementsString : "<?= $this->view->escape($this->_translate->_("SOME_ELEMENTS_ARE_MISSING,_THEY_ARE_MARKED_RED_IN_THE_FORMULA.")); ?>",
		okString : "<?= $this->view->escape($this->_translate->_("OK.")); ?>",
		errorString : "<?= $this->view->escape($this->_translate->_("ERROR")); ?>",
		addMore : true,
		nameInputs : true,
		addNewFormulaString : "<?= $this->view->escape($this->_translate->_("ADD_NEW_FORMULA")); ?>",
		clearFormulaString : "<?= $this->view->escape($this->_translate->_("CLEAR_FORMULA")); ?>",
		deleteFormulaString : "<?= $this->view->escape($this->_translate->_("DELETE_FORMULA")); ?>",
		formulaNameString : "<?= $this->view->escape($this->_translate->_("FORMULA_NAME")); ?>"

	});
}); <?php			$this->view->headScript()->captureEnd();
					$this->render("add-step2");

				} else {
					$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
					$form->populate($_POST);
					$this->view->form = $form;
					$this->render("add-step1");
				}
			} else { // step == 2
				$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

				$formulaNamespace = new Zend_Session_Namespace('WebReport_Formula');
				$applicationId = $formulaNamespace->application_id;
				$name = $formulaNamespace->name;
				$description = $formulaNamespace->description;

				$values = $_POST;

				$userData = Zend_Auth::getInstance()->getIdentity();

				$formulaSetsDb = new Core_Model_CountData_FormulaSets();
				$formulaSetData = array(
					'name' => $name,
					'description' => $description,
					'application_id' => $applicationId,
					'company_id' => $this->_group->company_id,
					'created_by' => $userData->user_id,
					'created_date' => new Zend_Db_Expr('NOW()')
				);
				$formulaSet = $formulaSetsDb->createRow($formulaSetData);
				$formulaSet->save();
                
                Core_Model_DocumentFactory::add($formulaSet->count_data_formula_set_id, "formula_set", $this->_group);

				$formulasDb = new Core_Model_CountData_FormulaSetFormulas();

				$camerasDb = new Core_Model_Cameras();
				$applicationsModel = new Core_Model_Applications();
				$application = $applicationsModel->findWithId($applicationId);
				$allowedCameraIds = array_keys($camerasDb->fetchAllVisibleNames($this->_user, $application));

				foreach ($values['formulas'] as $index => $formFormula) {
					// first check if company is allowed to add camera
					$f = new WebReport_Formula($formFormula);
					$cameraIds = array_keys($f->getCameraIdsAndOutTypesInvolved());

					foreach ($cameraIds as $camId) {
						if (!in_array($camId, $allowedCameraIds)) {
							continue 2;
						}
					}


					$formula = $formulasDb->createRow(array(
						'count_data_formula_set_id' => $formulaSet->count_data_formula_set_id,
						'formula' => $formFormula,
						'name' => $values['formulaNames'][$index]
					));
					$formula->save();
				}

				$this->_flashMessenger->addMessage($this->_translate->_('FORMULA_SET_ADDED.'));

				unset($formulaNamespace);

				
				$this->_helper->redirector('browse','formulas','count-data', array('folder' => $this->_group->group_id));
				exit;
			}
		}
	}

	public function viewAction()
	{
		$formulaSetId = (int)$this->_getParam("id");
		$formulaSetsDb = new Core_Model_CountData_FormulaSets();
		$companiesDb = new Core_Model_Companies();
		$db = $formulaSetsDb->getAdapter();
		$select = $db->select()->from(array("f" => $formulaSetsDb->info(Zend_Db_Table::NAME)), array("name", "description", "application_id"))
			->join(array("c" => $companiesDb->info(Zend_Db_Table::NAME)), "f.company_id = c.company_id", array("company" => "name"))
			->where('count_data_formula_set_id = ?', $formulaSetId);

		$identity = Zend_Auth::getInstance()->getIdentity();
		if (!$identity->multi_company) { $select->where('c.company_id = ?', $identity->company_id); }

		$formulaSet = $db->fetchRow($select);
		if (!$formulaSet) {
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FORMULA_SET_WITH_SUCH_ID_EXISTS.'));
		}

		$applicationsModel = new Core_Model_Applications();

		$this->view->name = $formulaSet->name;
		$this->view->application = $applicationsModel->findWithId($formulaSet->application_id)->getName();
		$this->view->company = $formulaSet->company;
		$this->view->description = $formulaSet->description;

		$formulaSetFormulasDb = new Core_Model_CountData_FormulaSetFormulas();
		$this->view->formulas = $formulaSetFormulasDb->fetchAll($formulaSetFormulasDb->select()->where('count_data_formula_set_id = ?', $formulaSetId));
		

	}

	public function editAction()
	{
		$formulaSetId = (int)$this->_getParam("id");
       
        $document = Core_Model_DocumentFactory::fetch($formulaSetId, "formula_set");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_FORMULA_SET.'));
            $this->_helper->redirector('browse','formulas','count-data', array('folder' => $this->_group->group_id));
            return;
        }

		$formulaSetsDb = new Core_Model_CountData_FormulaSets();
		$select = $formulaSetsDb->select()->where('count_data_formula_set_id = ?', $formulaSetId);
		$formulaSet = $formulaSetsDb->fetchRow($select);

		if (!$formulaSet) {
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FORMULA_SET_WITH_SUCH_ID_EXISTS.'));
		}
       
        $applicationsModel = new Core_Model_Applications();

        $application = $applicationsModel->findWithId($formulaSet->application_id);

        $this->view->applicationName = $application->getShortName();
        $this->view->formulaName = $formulaSet->name;
        $this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/jquery/plugins/jquery.livequery.js'));
        $this->view->headScript()->appendFile($this->view->baseUrl('/modules/admin/javascript/jquery/plugins/jquery.formulaEditor.js'));

        $formulasDb = new Core_Model_CountData_FormulaSetFormulas();
        $formulas = $formulasDb->fetchAll($formulasDb->select()->where('count_data_formula_set_id = ?', $formulaSet->count_data_formula_set_id));
        
        $camerasDb = new Core_Model_Cameras();
        $cameranames = $camerasDb->fetchAllVisibleNames($this->_user, $application);
        $cameraIds = array_keys($cameranames);

        $formCameraIds = array();
        foreach($formulas as $f) {
            $wrf = new WebReport_Formula($f->formula);
            $formCameraIds = array_merge($formCameraIds, array_keys($wrf->getCameraIdsAndOutTypesInvolved()));
        }
        $formCameraIds = array_unique($formCameraIds);

        if(count(($inBoth = array_intersect($formCameraIds, $cameraIds))) != count($formCameraIds)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_FORMULA_SET_BECAUSE_IT_CONTAINS_CAMERAS_THAT_ARE_NOT_VISIBLE_TO_YOU.'));

            foreach(array_diff($formCameraIds, $inBoth) as $camId) {
                $camName = $camerasDb->find($camId)->current()->name;
                $this->_flashMessenger->addMessage($this->_translate->_('NOT_VISIBLE_TO_YOU:_') . $camName);
            }

            
            $this->_helper->redirector('browse','formulas','count-data', array("folder" => $this->_group->group_id));
            return;
        }


		$form = new Admin_Form_EditFormulaSet();
        $this->view->headScript()->captureStart();
        ?>
$(function() {
  <?php foreach ($formulas as $f): ?>
  newdiv = $('<div id="formula" class="formula formulaHolder ui-dialog-titlebar ui-widget-header ui-corner-all ui-helper-clearfix"/>');
  $("form.uniForm").children().last().before(newdiv);
  
    <?php
        $wrf = new WebReport_Formula($f->formula);
        $nodes = split(" ", $f->formula);
        echo 'newdiv.append( $(\'<span class="operation"\>&nbsp;</span>\').hide());';
        echo "\n";
        foreach ($nodes as $node) {
            $node = trim($node);
            switch ($node) {
                case '':
                    echo 'newdiv.append( $(\'<span class="operation"\>&nbsp;</span>\').hide());';
                    echo "\n";
                    break;
                case '+':
                case '-':
                case '*':
                case '/':
                case '(':
                case ')':
                    echo 'newdiv.append(\'<span class="operation"\>'.$node.'</span>\');';
                    echo "\n";
                    break;
                default:
                    $readable = $wrf->toReadable_notags($node);
                    $cname = $wrf->toReadableNameOnly($node);
                    $camid = array_keys($cameranames, $cname);
                    $out_type = preg_split('/_/', $node);
                    echo 'newdiv.append( $(\'<span class="element"\>'.$readable.'</span>\').data("info", {cameraId:'.$camid[0].', type :"'.strtolower( $out_type[1] ).'"}));';
                    echo "\n";
                    break;
                }
        }
   ?>
  
<?php endforeach; ?>


  $(".resetButton").click(function () {
		window.location = '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'formulas', 'action' => 'browse')); ?>';
  });

  $(".uniForm .formula").formulaEditor({
        cameras : <?= Zend_Json::encode($camerasDb->fetchAllVisibleNames($this->_user, $application)); ?>,
		types : <?= Zend_Json::encode($application->getOutTypes()); ?>,
		form : $("form.uniForm"),
		missedElementsString : "<?= $this->view->escape($this->_translate->_("SOME_ELEMENTS_ARE_MISSING,_THEY_ARE_MARKED_RED_IN_THE_FORMULA.")); ?>",
		okString : "<?= $this->view->escape($this->_translate->_("OK.")); ?>",
		errorString : "<?= $this->view->escape($this->_translate->_("ERROR")); ?>",
		addMore : true,
		nameInputs : true,
        createFormulaNameEdit : false,
        createEmptyElement : false,
		addNewFormulaString : "<?= $this->view->escape($this->_translate->_("ADD_NEW_FORMULA")); ?>",
		clearFormulaString : "<?= $this->view->escape($this->_translate->_("CLEAR_FORMULA")); ?>",
		deleteFormulaString : "<?= $this->view->escape($this->_translate->_("DELETE_FORMULA")); ?>",
		formulaNameString : "<?= $this->view->escape($this->_translate->_("FORMULA_NAME")); ?>"
  });

<?php foreach ($formulas as $id=>$f): ?>
  $(".uniForm .formula").eq(<?php echo $id;?>).next('input').val("<?php echo $f->name;?>").removeClass('empty formula-name-dim');
<?php endforeach; ?>
});
    <?php
    $this->view->headScript()->captureEnd();
		if ($this->getRequest()->isPost()) {
			$values = $_POST;
			if ($form->isValid($values)) {

				$formulaSet->name = $values['name'];
				$formulaSet->description = $values['description'];
                $formulaSet->save();
                
                if(count($values['formulas']) != count($values['formulaNames']) )
                    throw new WebReport_Exception_ParameterError ($this->_translate->_('MISSMATCH_OF_FORMULAS_AND_FORMULA_NAMES.'));

                $where = $formulasDb->getAdapter()->quoteInto('count_data_formula_set_id = ?', $formulaSet->count_data_formula_set_id);
                $formulasDb->delete($where);
                
                foreach ($values['formulas'] as $key => $f) {
                    $formula = $formulasDb->createRow(array(
						'count_data_formula_set_id' => $formulaSet->count_data_formula_set_id,
						'formula' => $f,
						'name' => $values['formulaNames'][$key]
					));
					$formula->save();
                }
                
				$this->_flashMessenger->addMessage($this->_translate->_('FORMULA_SET_UPDATED.'));

				$this->_helper->redirector('browse','formulas','count-data', array('folder' => $this->_group->group_id));
			} else {
				$form->populate($values);
			}

		} else {
			$form->populate($formulaSet->toArray());
		}

		$this->view->form = $form;
		
	}

	public function deleteAction()
	{
		$formulaSetId = (int)$this->_getParam("id");

        $document = Core_Model_DocumentFactory::fetch($formulaSetId, "formula_set");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_FORMULA_SET.'));
            $this->_helper->redirector('browse','formulas','count-data', array('folder' => $this->_group->group_id));
            return;
        }

		$formulaSetsDb = new Core_Model_CountData_FormulaSets();
		$formulaSet = $formulaSetsDb->find($formulaSetId)->current();
		if (!$formulaSet)
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_FORMULA_SET_WITH_SUCH_ID_EXISTS.'));

		$this->_flashMessenger->addMessage($this->_translate->_('FORMULA_SET_DELETED.'));

        Core_Model_DocumentFactory::remove($formulaSetId, "formula_set");
		$formulaSet->delete();
		$this->_helper->redirector('browse','formulas','count-data', array('folder' => $this->_group->group_id));
	}

	protected function _stringsToTranslate()
	{
		$this->_translate->_('MOD_ADMIN_MENU_FORMULAS');
		$this->_translate->_('FORMULA_NAME');
		$this->_translate->_('FORMULA_SETS');
		$this->_translate->_('SELECT_A_COMPANY_FOR_THE_FORMULA.');
		$this->_translate->_('SELECT_AN_APPLICATION_FOR_THE_FORMULA.');
		$this->_translate->_('ADD_FORMULA_SET');
		$this->_translate->_('BROWSE_FORMULA_SETS');
	}

}
