<?php

class CountData_TimeFilterController extends WebReport_Controller_Action {

    public function init() {

        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_TIME_FILTERS');
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add"), "text" => 'ADD_TIME_FILTER');
        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
        $this->view->headTitle("TIME_FILTERS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function browseAction() {
        $timeFilterDb = new Core_Model_CountData_TimeFilters();
        $identity = Zend_Auth::getInstance()->getIdentity();
        $companiesDb = new Core_Model_Companies();
        $company = null;

        if (!$identity->multi_company) {
            $companiesDb = new Core_Model_Companies();
            $company = $companiesDb->find($identity->company_id)->current();
        }

        $this->view->filters = $timeFilterDb->fetchAllForCompany($company);

        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function addAction() {

        $this->view->headTitle("ADD", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $form = new CountData_Form_AddEditTimeFilter("add");

        if ($this->getRequest()->isPost()) {
            if ($form->isValid($_POST)) {
                $values = $form->getValues();

                $filterData = Zend_Auth::getInstance()->getIdentity();

                $filters = new Core_Model_CountData_TimeFilters();

                $filterDataArray = array(
                    'filter_name' => $values['filter_name'],
                    'filter_code' => $values['filter_code'],
                    'company_id' => $values['company_id'],
                    'created_by' => $filterData->user_id,
                    'date_last_modified' => new Zend_Db_Expr('NOW()')
                );

                if (!$filterData->multi_company) {
                    $filterDataArray['company_id'] = $filterData->company_id;
                }

                $filter = $filters->createRow($filterDataArray);
                $filter->save();
                $this->_flashMessenger->addMessage($this->_translate->_('TIME_FILTER_ADDED.'));
                $this->_helper->redirector('browse', 'time-filter', 'count-data');
            } else {
                $form->populate($_POST);
            }
        }

        $this->view->form = $form;
    }

    public function editAction() {
        $form = new CountData_Form_AddEditTimeFilter("edit");

        $filterId = (int) $this->_request->getParam('id');
        $filters = new Core_Model_CountData_TimeFilters();
        $filter = $filters->find($filterId)->current();

        if (!$filter)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_TIME_FILTER_WITH_SUCH_ID_EXISTS.'));

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $filter->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_TIME_FILTER_WITH_SUCH_ID_EXISTS.'));


        $this->view->headTitle(sprintf($this->_translate->_('EDIT_"%1$s"'), $filter->filter_name), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);


        if ($this->getRequest()->isPost()) {

            if ($form->isValid($_POST)) {
                $values = $form->getValues();
/*
 *
                $userDataArray = array(
                    'filter_name' => $values['filtername'],
                    'filter_code' => $values['filtercode'],
                    'company_id' => $values['company_id'],
                    'created_by' => $userData->user_id,
                    'date_last_modified' => new Zend_Db_Expr('NOW()')
                );
 */
                $filter->filter_name = $values['filter_name'];
                $filter->filter_code = $values['filter_code'];
                $filter-> date_last_modified = new Zend_Db_Expr('NOW()');
                $filter->save();
                $this->_flashMessenger->addMessage($this->_translate->_('TIME_FILTER_UPDATED.'));

                /*
                if ($filterId == Zend_Auth::getInstance()->getIdentity()->filter_id) {
                    Zend_Auth::getInstance()->clearIdentity();
                }
                 *
                 */

                $this->_helper->redirector('browse', 'time-filter','count-data');
            } else {
                $form->populate($_POST);
            }
        } else {
            $form->populate($filter->toArray());
            //$form->getElement('current_email')->setValue($filter->email);
        }

        $this->view->form = $form;
    }

    public function removeAction() {

        $filterId = (int) $this->_request->getParam('id');
        $filters = new Core_Model_CountData_TimeFilters();
        $filter = $filters->find($filterId)->current();

        if (!$filter)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_TIME_FILTER_WITH_SUCH_ID_EXISTS.'));

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $filter->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_TIME_FILTER_WITH_SUCH_ID_EXISTS.'));


        $filter->delete();

        $this->view->headTitle(sprintf($this->_translate->_('EDIT_"%1$s"'), $filter->filter_name), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $this->_flashMessenger->addMessage($this->_translate->_('TIME_FILTER_REMOVED.'));

        $this->_helper->redirector('browse', 'time-filter','count-data');
    }

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {
        $this->_translate->_('FILTER_NAME');
        $this->_translate->_('ADD_TIME_FILTER');
        $this->_translate->_('BROWSE_TIME_FILTERS');
        $this->_translate->_('TIME_FILTER_ADDED');
        $this->_translate->_('NO_DEFINED_TIME_FILTERS');
        $this->_translate->_('DELETE_TIME_FILTER');
        $this->_translate->_('EDIT_TIME_FILTER');
        $this->_translate->_('TIME_FILTERS');
        $this->_translate->_('TIME_FILTER_NAME');
        $this->_translate->_('TIME_FILTER_CODE');
        $this->_translate->_('NO_TIME_FILTER_WITH_SUCH_ID_EXISTS');
        $this->_translate->_('TIME_FILTER_UPDATED');
        $this->_translate->_('WEEKEND_AFTER_PAYDAY_(25TH)');
        $this->_translate->_('FIRST_WEEKEND_IN_MONTH');
        $this->_translate->_('NINE_TO_FIVE');
        $this->_translate->_('WEEKENDS');
        $this->_translate->_('WORKING_DAYS');
        $this->_translate->_('FIRST_FRIDAY_OF_THE_MONTH');
    }
}
