<?php

class CountData_VirtualCamerasController extends WebReport_Controller_Action
{
    protected $_group, $_user;

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();

        $usersDb = new Core_Model_Users();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_VIRTUAL_CAMERAS');

        if($this->_group && $this->_group->canEdit($this->_user))
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", "folder" => $groupId), "text" => 'ADD_VIRTUAL_CAMERA');
        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
        $this->view->headTitle('VIRTUAL_CAMERAS', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }


    public function browseAction()
    {
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
            $this->_user, $this->_group, $this->_helper, $this->_request,
            'Core_Model_VirtualCameras',
            'CountData_Model_VirtualCameraListGenerator');
        
        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }


    public function addAction()
    {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector('browse','virtual-cameras','count-data', array('folder' => $this->_group->group_id));
            return;
        }

        $this->view->headTitle("ADD", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        if (!$this->getRequest()->isPost()) {
            $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'),1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

            $form = new Admin_Form_AddVirtualCamera();
            $this->view->form = $form;
            $this->render("add-step1");

        } else { // request is post
            if ($_POST['step'] == 1) {

                $form = new Admin_Form_AddVirtualCamera();

                if ($form->isValid($_POST)) {
                    $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
                    $values = $form->getValues();

                    $virtualCameraNamespace = new Zend_Session_Namespace('WebReport_VirtualCamera');
                    $virtualCameraNamespace->application_id = $values['application_id'];
                    $virtualCameraNamespace->name = $values['name'];
                    $virtualCameraNamespace->description = $values['description'];

                    $applicationsModel = new Core_Model_Applications();
                    $application = $applicationsModel->findWithId($virtualCameraNamespace->application_id);

                    $this->view->applicationName = $application->getShortName();
                    $this->view->formulaName = $virtualCameraNamespace->name;
                    $this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/jquery/plugins/jquery.livequery.js'));
                    $this->view->headScript()->appendFile($this->view->baseUrl('/modules/admin/javascript/jquery/plugins/jquery.formulaEditor.js'));
                    $this->view->types = $application->getOutTypes();
                    
                    $camerasDb = new Core_Model_Cameras();

                    $this->view->headScript()->captureStart(); ?>
$(document).ready(function() {
    $(".resetButton").click(function () {
        window.location = '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'virtual-cameras', 'action' => 'browse')); ?>';
    });

    $(".uniForm .formula").formulaEditor({
        cameras : <?= Zend_Json::encode($camerasDb->fetchAllVisibleNames($this->_user, $application)); ?>,
        types : <?= Zend_Json::encode($application->getOutTypes()); ?>,
        form : $("form.uniForm"),
        missedElementsString : "<?= $this->view->escape($this->_translate->_("SOME_ELEMENTS_ARE_MISSING,_THEY_ARE_MARKED_RED_IN_THE_FORMULA.")); ?>",
        okString : "<?= $this->view->escape($this->_translate->_("OK.")); ?>",
        errorString : "<?= $this->view->escape($this->_translate->_("ERROR")); ?>"
    });
}); <?php            $this->view->headScript()->captureEnd();
                    $this->render("add-step2");

                } else {
                    $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
                    $form->populate($_POST);
                    $this->view->form = $form;
                    $this->render("add-step1");
                }
            } else { // step == 2
                $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

                $virtualCameraNamespace = new Zend_Session_Namespace('WebReport_VirtualCamera');
                $applicationId = $virtualCameraNamespace->application_id;
                $companyId = $virtualCameraNamespace->company_id;
                $name = $virtualCameraNamespace->name;
                $description = $virtualCameraNamespace->description;

                $values = $_POST;


                $userData = Zend_Auth::getInstance()->getIdentity();

                $virtualCamerasDb = new Core_Model_VirtualCameras();
                $virtualCameraData = array(
                    'name' => $name,
                    'description' => $description,
                    'application_id' => $applicationId,
                    'company_id' => $this->_group->company_id,
                    'created_by' => $userData->user_id,
                    'created_date' => new Zend_Db_Expr('NOW()')
                );
                $virtualCamera = $virtualCamerasDb->createRow($virtualCameraData);
                $virtualCamera->save();
                
                Core_Model_DocumentFactory::add($virtualCamera->virtual_camera_id, "virtual_camera", $this->_group);

                $virtualCameraFormulaDb = new Core_Model_VirtualCameraFormula();

                $companiesDb = new Core_Model_Companies();
                $applicationsModel = new Core_Model_Applications();
                $camerasDb = new Core_Model_Cameras();
                $application = $applicationsModel->findWithId($applicationId);
                $company = $companiesDb->find($companyId)->current();
                $allowedCameraIds = array_keys($camerasDb->fetchAllVisibleNames($this->_user, $application));


                $rowsToSave = array();

                foreach ($values['formulas'] as $type => $formula) {

                    $f = new WebReport_Formula($values['formulas'][$type]);

                    $cameraIds = array_keys($f->getCameraIdsAndOutTypesInvolved());

                    foreach ($cameraIds as $camId) {
                        if (!in_array($camId, $allowedCameraIds)) {
                            $virtualCamera->delete();
                            $this->_flashMessenger->addMessage($this->_translate->_('AT_LEAST_ONE_INVALID_CAMERA_IN_THE_FORMULAS._THE_VIRTUAL_CAMERA_IS_NOT_ADDED.'));
                            unset($virtualCameraNamespace);
                            $this->_helper->redirector('browse','virtual-cameras','count-data', array('folder' => $this->_group->group_id));

                        }
                    }

                    $rowData = array(
                        'count_out_type' => $type,
                        'virtual_camera_id' => $virtualCamera->virtual_camera_id,
                        'formula' => $values['formulas'][$type]
                    );
                    $row = $virtualCameraFormulaDb->createRow($rowData);
                    $rowsToSave[] = $row;
                }

                foreach ($rowsToSave as $saveRow) {
                    $saveRow->save();
                }

                //$virtualCamera->parseFormulaAndSave($values['formulas'][0]);

                $this->_flashMessenger->addMessage($this->_translate->_('VIRTUAL_CAMERA_ADDED.'));

                unset($virtualCameraNamespace);

                $this->_helper->redirector('browse','virtual-cameras','count-data', array('folder' => $this->_group->group_id));
                exit;
            }
        }
    }

    public function editAction()
    {
        $cameraId = (int)$this->_getParam("id");
        
        $document = Core_Model_DocumentFactory::fetch($cameraId, "virtual_camera");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_VIRTUAL_CAMERA.'));
            $this->_helper->redirector('browse','virtual-cameras','count-data', array('folder' => $this->_group->group_id));
            return;
        }

        $camerasDb = new Core_Model_Cameras();
        
        $vcamerasDb = new Core_Model_VirtualCameras();
        $camera = $vcamerasDb->find($cameraId)->current();

        $applicationsModel = new Core_Model_Applications();
        $application = $applicationsModel->findWithId($camera->application_id);
        
        $this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/jquery/plugins/jquery.livequery.js'));
        $this->view->headScript()->appendFile($this->view->baseUrl('/modules/admin/javascript/jquery/plugins/jquery.formulaEditor.js'));
        
        //populate with stored virtual camera...
        $virtualCameraFormulaDb = new Core_Model_VirtualCameraFormula();
        $formulas = $virtualCameraFormulaDb->fetchAll($virtualCameraFormulaDb->select()->where('virtual_camera_id = ?', $cameraId));

        $cameranames = $camerasDb->fetchAllVisibleNames($this->_user, $application);
        $cameraIds = array_keys($cameranames);

        $formCameraIds = array();
        foreach($formulas as $f) {
            $wrf = new WebReport_Formula($f->formula);
            $formCameraIds = array_merge($formCameraIds, array_keys($wrf->getCameraIdsAndOutTypesInvolved()));
        }
        $formCameraIds = array_unique($formCameraIds);

        if(count(($inBoth = array_intersect($formCameraIds, $cameraIds))) != count($formCameraIds)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_VIRTUAL_CAMERA_BECAUSE_IT_CONTAINS_CAMERAS_THAT_ARE_NOT_VISIBLE_TO_YOU.'));

            foreach(array_diff($formCameraIds, $inBoth) as $camId) {
                $camName = $camerasDb->find($camId)->current()->name;
                $this->_flashMessenger->addMessage($this->_translate->_('NOT_VISIBLE_TO_YOU:_') . $camName);
            }

            
            $this->_helper->redirector('browse','virtual-cameras','count-data', array("folder" => $this->_group->group_id));
            return;
        }

        $this->view->headScript()->captureStart();
        ?>
$(function() {
  <?php foreach ($application->getOutTypes() as $t): ?>
  newdiv = "<div id=\"formulaHolder\" class=\"formula formulaHolder ui-dialog-titlebar ui-widget-header ui-corner-all ui-helper-clearfix\" type=\"<?php echo $t; ?>\"></div>";
  $("form").children().last().before(newdiv);
  <?php endforeach; ?>

  $(".resetButton").click(function () {
        window.location = '<?= $this->view->url(array('module' => 'count-data', 'controller' => 'virtual-cameras', 'action' => 'browse')); ?>';
  });
    $(".uniForm .formula").formulaEditor({
        cameras : <?= Zend_Json::encode($cameranames); ?>,
        types : <?= Zend_Json::encode($application->getOutTypes()); ?>,
        form : $("form.uniForm"),
        missedElementsString : "<?= $this->view->escape($this->_translate->_("SOME_ELEMENTS_ARE_MISSING,_THEY_ARE_MARKED_RED_IN_THE_FORMULA.")); ?>",
        okString : "<?= $this->view->escape($this->_translate->_("OK.")); ?>",
        errorString : "<?= $this->view->escape($this->_translate->_("ERROR")); ?>"
    });
    
    <?php
    //parse the formulastring
    // TODO: Refactor and move to formula class
    $elbase = "\$el.append(\$(document.createElement('span'))";
    foreach ($application->getOutTypes() as $raw => $t) {
        echo "\$el = \$(\".formulaHolder[type=\\\"$t\\\"]\");\n";
        echo "\$el.children().remove();\n";
        echo $elbase.".addClass('operation').html('&nbsp;').hide());\n";
        foreach ($formulas as $f) {
            if( strtolower( $f['count_out_type'] ) == strtolower( $raw ) )  {
                $wrf = new WebReport_Formula($f->formula);
                $nodes = explode(" ", $f->formula);
                
                foreach ($nodes as $node) {
                    $node = trim($node);
                    switch ($node) {
                        case '':
                            echo $elbase.".addClass('operation').html('&nbsp;').hide());\n";
                            break;
                        
                        case '+':
                        case '-':
                        case '*':
                        case '/':
                        case '(':
                        case ')':
                            echo $elbase.".addClass('operation').html('$node'));\n";
                            break;
                        default:
                            $readable = $wrf->toReadable_notags($node);
                            $cname = $wrf->toReadableNameOnly($node);
                            $camid = array_keys($cameranames, $cname);
                            $out_type = preg_split('/_/', $node);
                            
                            echo $elbase.".addClass('element').html('$readable').data('info', { cameraId : $camid[0], type :\"".strtolower( $out_type[1] )."\"}));\n";
                            break;
                    }
                    
                }
                 
                 
            }
        } //foreach formulas
        
    } //foreach outtypes
    ?>
});

<?php
        $this->view->headScript()->captureEnd();
        
        $form = new Admin_Form_EditVirtualCamera();
        $this->view->application = $application;
        if ($this->getRequest()->isPost()) {
            $values = $_POST;
            //Zend_Debug::dump($values);

            if ($form->isValid($values)) {
                $camera->name = $values['name'];
                $camera->description = $values['description'];
                $camera->save();
                
                foreach ($values['formulas'] as $type => $formula) {
                    $updselect = $virtualCameraFormulaDb->select()->where('virtual_camera_id = ?', $cameraId)
                                                                    ->where('count_out_type = ?', strtolower($type));

                    $formularow = $virtualCameraFormulaDb->fetchRow($updselect);
                    if(!$formularow)
                        throw new WebReport_Exception_ParameterError($this->_translate->_('NO_VIRTUAL_CAMERA_WITH_SUCH_ID_EXISTS.'));
                    $formularow->formula = $formula;
                    $formularow->save();
                    if(strtolower( $f['count_out_type'] ) == strtolower( $type )) {
                        $f['formula'] = $formula;
                    }
                }
                
                $this->_flashMessenger->addMessage($this->_translate->_('VIRTUAL_CAMERA_UPDATED.'));
                $this->_helper->redirector('browse', 'virtual-cameras', 'count-data', array('folder' => $this->_group->group_id));

            } else {
                $form->populate($values);
                $this->view->form = $form;
            }
        } else {
            
            $form->populate($camera->toArray());
            $this->view->form = $form;
        }

        

    }

    public function viewAction()
    {
        $cameraId = (int)$this->_getParam("id");

        $document = Core_Model_DocumentFactory::fetch($cameraId, "virtual_camera");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_VIEW_THIS_VIRTUAL_CAMERA.'));
            $this->_helper->redirector('browse','virtual-cameras','count-data', array('folder' => $this->_group->group_id));
            return;
        }

        $camerasDb = new Core_Model_VirtualCameras();
        $companiesDb = new Core_Model_Companies();

        $db = $camerasDb->getAdapter();
        $select = $db->select()->from(array("vc" => $camerasDb->info(Zend_Db_Table::NAME)), array("name", "virtual_camera_id", "application_id", "description"))
            ->join(array("co" => $companiesDb->info(Zend_Db_Table::NAME)), "vc.company_id = co.company_id", array("company" => "name"))
            ->where('virtual_camera_id = ?', $cameraId);

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company) {
            $select->where('co.company_id = ?', Zend_Auth::getInstance()->getIdentity()->company_id);
        }

        $camera = $db->fetchRow($select);
        if (!$camera) {
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_VIRTUAL_CAMERA_WITH_SUCH_ID_EXISTS.'));
        }

        $applicationsModel = new Core_Model_Applications();

        $application = $applicationsModel->findWithId($camera->application_id);
        
        $this->view->name = $camera->name;
        $this->view->application = $application->getName();
        $this->view->company = $camera->company;
        $this->view->description = $camera->description;

        $this->view->types = $application->getOutTypes();

        $types = $application->getOutTypes();


        $typeFormulas = array();

        $cameraFormulaDb = new Core_Model_VirtualCameraFormula();
        $select = $cameraFormulaDb->select()->where('virtual_camera_id = ?', $cameraId);
        $t = array_keys($types);
        $formulas = $cameraFormulaDb->fetchAll($select);
        
        foreach ($formulas as $formula) {
            $f = new WebReport_Formula($formula->formula);
            $types[$formula->count_out_type] = $f->toReadable();
        }
        $this->view->formulas = $types;



    }

    public function deleteAction()
    {
        $cameraId = (int)$this->_getParam("id");

        $camerasDb = new Core_Model_VirtualCameras();
        $camera = $camerasDb->find($cameraId)->current();
        if (!$camera)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_VIRTUAL_CAMERA_WITH_SUCH_ID_EXISTS.'));

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $camera->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_VIRTUAL_CAMERA_WITH_SUCH_ID_EXISTS.'));

        $this->_flashMessenger->addMessage($this->_translate->_('VIRTUAL_CAMERA_DELETED.'));
        
        Core_Model_DocumentFactory::remove($camera->virtual_camera_id, "virtual_camera");
        $camera->delete();
        $this->_helper->redirector('browse','virtual-cameras','count-data', array('folder' => $this->_group->group_id));
    }


    protected function _stringsToTranslate()
    {
        $this->_translate->_('MOD_ADMIN_MENU_VIRTUAL_CAMERAS');
        $this->_translate->_('SELECT_A_COMPANY_FOR_THE_VIRTUAL_CAMERA.');
        $this->_translate->_('SELECT_AN_APPLICATION_FOR_THE_VIRTUAL_CAMERA.');
        $this->_translate->_('BROWSE_VIRTUAL_CAMERAS');
    }

}
