<?php

class CountData_Form_AddEditTimeFilter extends UniForm_Form
{
	protected $_action;

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param string $action Can be add or edit
	 * @param mixed $options
	 * @return void
	 */
	public function     __construct($action = "add", $options = null)
	{
		$this->_action = $action;
		parent::__construct($options);
	}

	public function init()
	{
		parent::init();

		$action = $this->_action;

		$this->addElementPrefixPath('CountData_Forms_Validate', 'count-data/forms/Validate/', 'validate');

		$filtername = new UniForm_Form_Element_Text('filter_name');
		$filtername->setLabel('FILTER_NAME')
			->setRequired()
			->addFilter('StringTrim');

		$filtercode = new UniForm_Form_Element_Text('filter_code');
		$filtercode->setLabel('FILTER_CODE')
			->setRequired()
			->addFilter('StringTrim')
                        ->addValidator('TimeFilter');

		$identity = Zend_Auth::getInstance()->getIdentity();
                $company_id = null;
		if ($action == "add") {

			if ($identity->multi_company) {

				$companiesDb = new Core_Model_Companies();
				$activeCompanies = $companiesDb->fetchAllActive();
				$company_id = new UniForm_Form_Element_Select('company_id');
				$company_id->setLabel('COMPANY')
					->setRequired()
					->addValidator(new Zend_Validate_GreaterThan(0)) // check if user belongs to company
					->setErrorMessages(array($this->getView()->translate("USER_MUST_BELONG_TO_A_COMPANY.")));

				$company_id->addMultiOption(0, "NO_COMPANY");
				foreach ($activeCompanies as $company) {
					$company_id->addMultiOption($company->company_id, $company->name);
				}
			} else {
				$company_id = new UniForm_Form_Element_Hidden('company_id');
				$company_id->setValue(Zend_Auth::getInstance()->getIdentity()->company_id);
			}
		}

		$acl_role_id = new UniForm_Form_Element_Select('acl_role_id');
		$acl_role_id->setLabel('ROLE')
		->setRequired()
		->setRegisterInArrayValidator(false);

		if (!$identity->multi_company || $action == "edit") {

			$aclRolesDb = new Core_Model_AclRoles();
			$select = $aclRolesDb->select();
			$select = $select->where('company_id = ?', $identity->company_id)->order('name');
			$aclRoles = $aclRolesDb->fetchAll($select);

			foreach ($aclRoles as $aclRole) {
				$acl_role_id->addMultiOption($aclRole->acl_role_id, $aclRole->name);
			}
		} else {

			$acl_role_id->addMultiOption(0, "SELECT_A_COMPANY.");
			$view = $this->getView();
			
			$view->headScript()->captureStart();  ?>
jQuery(function($) {
	$("#company_id").change(function() {
		$.ajax({
			type : 'post',
			dataType : 'json',
			url : '<?php echo $view->url(array("module" => "admin", "controller" => "acl", "action" => "get-roles-for-company")); ?>',
			data : { companyId : $(this).val() },
			success : function (data, textStatus) {

				if ($(data).length == 0) return;
				$("#acl_role_id option").remove();
				$.each(data, function(i, val) {
					var $option = $(document.createElement('option'));
					$option.val(val.id).text(val.name);
					$("#acl_role_id").append($option);
				});

			}
		});
	}).trigger('change');
});
<?php			$view->headScript()->captureEnd();
		}


		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel('SUBMIT');

		$this->addElements(array($filtername,$filtercode));
                if(!is_null($company_id)){
                    $this->addElements(array($company_id));
                }
                /*
		if (isset($company_id)) $this->addElement($company_id);
		if (isset($acl_role_id)) $this->addElement($acl_role_id);
                 * 
                 */
		$this->addDisplayGroup(array('filter_name','filter_code','company_id'), 'info-group', array('legend' => 'GENERAL_INFORMATION'));
/*
		if ($action == 'add') {
			$this->addElements(array($password, $password_confirm));
			$this->addDisplayGroup(array('password','password_confirm'), 'password-group',array('legend' => 'PASSWORD'));
		}
*/

                /*
        $country = new UniForm_Form_Element_Select('country');
        $country->setLabel('COUNTRY');

        $countries = new Core_Model_Countries();
        $continents = $countries->getContinents();
        $continentCountries = $countries->getContinentCountries();

		$country->addMultiOption("", "SELECT_A_COUNTRY.");
        foreach ($continentCountries as $key => $value) {
			$country->addMultiOption($continents[$key], $value);
        }



		$this->addElements(array($language, $country));
		$this->addDisplayGroup(array('language', 'country'), 'locale-group',array('legend' => 'LOCALE'));

*/
		$this->addElements(array($submit));
	}

}