<?php

class Export_ExportController extends WebReport_Controller_Action
{
   protected $_user;
   protected $_idParam;
   protected $_hasEventData;

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $usersDb = Core_Model_Users::getInstance();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        $items = array();
        $items[] = array("urlOptions" => array(
            'module' => $module, "controller" => 'index',
            "action" => "index"),
            "text" => _T('LIST_STORED_EXPORTS'),
            'tooltip' => _T('LIST_STORED_EXPORTS_TOOLTIP')
        );
        $items[] = array("urlOptions" => array(
            'module' => $module, "controller" => 'export',
            "action" => "create-export"),
            "text" => _T('CREATE_EXPORT'),
            'tooltip' => _T('CREATE_EXPORT_TOOLTIP')
        );

        $this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
        $this->view->module = $module;
        $this->view->headTitle("CREATE_EXPORT", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $idParam = (int) $this->_request->getParam('id');
        $this->_idParam = $idParam;

        $this->_hasEventData = Core_Model_EventData::userHasVisibleEventData($this->_user);
        if (!$this->_hasEventData) {
            $this->_hasEventData = 0; //View can't handle it only being false, must output 0 for javascript check.
        }
    }

    private function prepareRepopulation($values)
    {
        $sources = explode(",", $values["chosen_sources"]);
        return $extra_populate_data = array(
            'sources' => $sources,
            'timeResolution' => $values["time_resolution"]
        );
        //More logic needed here once sets are introduced.
        return $extra_populate_data;
    }

    private function saveExport($values, $export = null)
    {
        if (!isset($export)) {
            $create_new = true;
        } else {
            $create_new = false;
        }
        $exportsDb = Export_Model_Exports::getInstance();
        $exportData = $exportsDb->createRowDataFromPost($values, $export);
        if ($create_new) {
            $export = $exportsDb->createRow($exportData);
        } else {
            $export->setFromArray($exportData);
        }
        $exportId = $export->save();

        //Use this when source sets is used.
        /*$storeDb = Core_Model_CameraSet_CameraSetType1s::getInstance();
        //TODO: Handle incorrect/empty location entries.
        $location_array = $this->locationArrayFromString($values["location"]);
        $storeData = array(
                'camera_set_id' => $cameraSetId,
                'location_city' => $location_array["city"],
                'location_region' => $location_array["region"],
                'location_country' => $location_array["country"],
                'location_coordinates' => $location_array["coordinates"],
                'sales_region_id' => $sales_region_id,
                'day_start_offset' => $values["day_start_offset"],
                );
        if ($create_new) {
            $storeMetadata = $storeDb->createRow($storeData);
        } else {
            $storeMetadata = $cameraSet->getSetMetadataRow();
            $storeMetadata->setFromArray($storeData);
        }
        $storeMetadata->save();*/

        $exportCamerasDb = Export_Model_ExportCameras::getInstance();
        //Add store cameras (and perhaps remove on edit)
        $cameraDb = Core_Model_Cameras::getInstance();

        $camera_to_be_deleted = false;
        $existingExportCameras = $export->getExportCameraRows();
        if ($existingExportCameras != null) {
            $camera_to_be_deleted = true; //Might not need to delete, but check must be run.
        }
        if (isset($values['chosen_sources']) && $values['chosen_sources'] != "") {
            $exportCameraIds = explode(',', $values['chosen_sources']);
            foreach ($exportCameraIds as $cameraId) {
                $camera = $cameraDb->getCamera($cameraId);
                if (!$camera) {
                    throw new WebReport_Exception("Could not get camera with id: $cameraId for storing of export");
                } else {
                    if ($camera_to_be_deleted) {
                        $cameras_to_save[] = $camera;
                    }
                    $existing_camera = $exportCamerasDb->getRowByCameraAndExportId($camera->camera_id, $exportId);
                    if (!isset($existing_camera)) {
                        $exportCameraData = array(
                            'camera_id' => $camera->camera_id,
                            'export_id' => $exportId
                            );
                        $exportCamera = $exportCamerasDb->createRow($exportCameraData);
                        $exportCamera->save();
                    }
                }
            }
        }
        if ($camera_to_be_deleted) {
            foreach ($existingExportCameras as $existingExportCamera) {
                $delete = true;
                $existing_id = $existingExportCamera->camera_id;
                if (isset($cameras_to_save)) {
                    foreach ($cameras_to_save as $camera_to_save) {
                        if ($camera_to_save->camera_id == $existing_id) {
                            $delete = false;
                        }
                    }
                }
                if ($delete) {
                    $existingExportCamera->delete();
                }
            }
        }
    }

    public function fetchApplicationsForExportAction()
    {
        //This is used from view as a POST AJAX call.
        if ($this->_request->isPost()) {
            $this->_helper->layout->disableLayout();    //disable layout
            $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering
            $export_type = $_POST['export_type'];
            if ($export_type == Export_Model_Exports::COUNT_DATA_EXPORT) {
                $app_type = "countdata";
            } elseif ($export_type == Export_Model_Exports::EVENT_DATA_EXPORT) {
                $app_type = "eventdata";
            }
            $applicationsDb = Core_Model_Applications::getInstance();
            $available_apps = $applicationsDb->fetchAllThatHasVisibleDataInDb($app_type);
            foreach ($available_apps as $application) {
                $applications[$application->getId()] = $application->getName();
            }
            echo json_encode($applications);
        }
    }

    public function fetchAvailableDataForViewAction()
    {
        //This is used from view as a POST AJAX call.
        if ($this->_request->isPost()) {
            $this->_helper->layout->disableLayout();    //disable layout
            $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering
            $company_id = $_POST['company_id'];
            $application_id = $_POST['application_id'];
            $newAvailableData = $this->fetchAvailableDataForUser($this->_user, $company_id, $application_id);
            $cameras = $newAvailableData["cameras"];
            $cameraDb = Core_Model_Cameras::getInstance();
            $applicationsDb = Core_Model_Applications::getInstance();
            $return_array['cameras'] = array();
            foreach ($cameras as $camera) {
                $metaData = $camera->getMetaData();
                $return_array['cameras'][]=array(
                    "camera_id" => $camera->camera_id,
                    "camera_name" => $camera->name,
                    "application" => $applicationsDb->findWithId($camera->application_id)->getName(),
                    "path" => $metaData["folder"]["path"]
                     );
            }
            $return_array['firstDate'] = $newAvailableData['firstDate']->toString("yyyy-MM-dd");
            $return_array['lastDate'] = $newAvailableData['lastDate']->toString("yyyy-MM-dd");
            $return_array['firstDateSelected'] = $newAvailableData['firstDateSelected']->toString("yyyy-MM-dd");
            $return_array['lastDateSelected'] = $newAvailableData['lastDateSelected']->toString("yyyy-MM-dd");
            $return_array['timeResolutions'] = $newAvailableData['timeResolutions'];

            $return_json = json_encode($return_array);
            echo $return_json;
        }
    }

    private function fetchAvailableDataForUser($user, $limit_by_company_id = null, $limit_by_application_id = null)
    {
        $cameraDb = Core_Model_Cameras::getInstance();
        $allAvailableCameras = array();
        if ($user->multi_company && !isset($limit_by_company_id) && !isset($limit_by_application_id)) {
            $allAvailableCameras = $cameraDb->fetchAll();
        } else {
            $companyDb = Core_Model_Companies::getInstance();
            $applicationsDb = Core_Model_Applications::getInstance();
            if (!$user->multi_company) {
                $company = $companyDb->getCompany($user->company_id);
            } elseif (isset($limit_by_company_id)) {
                $company = $companyDb->getCompany($limit_by_company_id);
            } else {
                $company = null;
            }
            if (isset($limit_by_application_id)) {
                $application = $applicationsDb->getWithId($limit_by_application_id);
            } else {
                $application = null;
            }
            $cameras_array = $cameraDb->getCameras($application, $company);
            foreach ($cameras_array as $camera) {
                $select = $cameraDb->select();
                $camera_row = $cameraDb->fetchAll($select->where("camera_id = ?", $camera->camera_id))->current();
                $allAvailableCameras[] = $camera_row;
            }
        }
        $countDataTotal = new Core_Model_CountData();
        foreach ($allAvailableCameras as $camera) {
            $countDataTotal->addCamera($camera);
        }

        $countDataTotal->initDatabaseSelect();
        $firstDate = $countDataTotal->getFirstDate();
        $lastDate = $countDataTotal->getLastDate();
        $sevenDaysBeforeLast = clone $lastDate;
        $sevenDaysBeforeLast->sub(7, Zend_Date::DAY);
        if ($sevenDaysBeforeLast->compare($firstDate) > 0) {
            $firstDateSelected = $sevenDaysBeforeLast;
        } else {
            $firstDateSelected = $firstDate;
        }
        /*if ($this->_hasEventData) {
            $firstDateEventData = Core_Model_EventData::dbFirstDate();
            $lastDateEventData = Core_Model_EventData::dbLastDate();
            $ret["firstDateEventData"] = $firstDateEventData;
            $ret["lastDateEventData"] = $lastDateEventData;
        }*/
        $ret["timeResolutions"] = $countDataTotal->getTimeResolutions();
        $ret["firstDate"] = $firstDate;
        $ret["lastDate"] = $lastDate;
        $ret["firstDateSelected"] = $firstDateSelected;
        $ret["lastDateSelected"] = $lastDate;
        $ret["cameras"] = $allAvailableCameras;
        return $ret;
    }

    public function createExportAction()
    {
        if ($this->_flashMessenger->hasMessages()) {
            $this->view->messages = $this->_flashMessenger->getMessages();
        }
        $this->view->title = $this->_translate->_("CREATE_EXPORT");
        $this->view->baseUrl = $this->_request->getBaseUrl();
        $this->view->hasEventData = $this->_hasEventData;
        $this->_helper->viewRenderer->setRender('create-edit-export');
        $form = new Export_Form_CreateEditExport('create', $this->_hasEventData);
        if ($this->_request->isPost()) {
            $values = $_POST;
            if ($form->isValid($values)) {
                $this->saveExport($values);
                $this->_flashMessenger->addMessage($this->_translate->_('EXPORT_STORED_SUCCESSFULLY.'));
                $this->_helper->redirector('index', 'index', 'export');

            } else {
                $extra_populate_data = $this->prepareRepopulation($values);
                $this->view->extra_populate_data = $extra_populate_data;
                $form->populate($_POST);
            }
        }
        $this->view->form = $form;
    }

    public function validTimeCodeAjaxAction()
    {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        try {
            $valid = CM_Interval::isValidCode($_POST['code']);
        } catch (Exception $e) {
            echo Zend_Json::encode(array("valid"=>false, "message" => $e->getMessage()));
            return;
        }
        echo Zend_Json::encode(array("valid" => $valid, "message" => "success"));
    }

    public function editExportAction()
    {
        if ($this->_flashMessenger->hasMessages()) {
            $this->view->messages = $this->_flashMessenger->getMessages();
        }
        $this->view->title = $this->_translate->_("EDIT_EXPORT");
        $this->view->baseUrl = $this->_request->getBaseUrl();
        $this->view->hasEventData = $this->_hasEventData;
        $this->_helper->viewRenderer->setRender('create-edit-export');
        $form = new Export_Form_CreateEditExport('edit', $this->_hasEventData);
        $exportDB = Export_Model_Exports::getInstance();
        $export = $exportDB->getRowById($this->_idParam);
        if ($this->_request->isPost()) {
            $values = $_POST;
            if ($form->isValid($values)) {
                $this->saveExport($values, $export);
                $this->_flashMessenger->addMessage($this->_translate->_('EXPORT_UPDATED_SUCCESSFULLY.'));
                $this->_helper->redirector('index', 'index', 'export');

            } else {
                $extra_populate_data = $this->prepareRepopulation($values);
                $this->view->extra_populate_data = $extra_populate_data;
                $form->populate($_POST);
            }
        } else {
            $exportSources = $export->getDataSources();
            $sources_populate_data = array();
            foreach ($exportSources as $es) {
                $sources_populate_data[] = $es->camera_id;
            }
            if (isset($export->time_resolution)) {
                $time_resolution = $export->time_resolution;
            }
            $data = array('name' => $export->name,
                          'company_id' => $export->company_id,
                          'time_interval_type' => $export->time_interval_type,
                          'export_type' => $export->export_type,
                          'end-time' => $export->fix_interval_end,
                          'output-format' => $export->output_format,
                          'aggregate' => $export->aggregator_id
            );
            //Everything that is loaded dynamically also must be populated dynamically:
            $extra_populate_data = array(
                'sources' => $sources_populate_data,
                'timeResolution' => $time_resolution,
                'application_id' => $export->application_id,
                'first_date_selected' => $export->fix_interval_start,
                'last_date_selected' => $export->fix_interval_end
            );
            $this->view->extra_populate_data = $extra_populate_data;
            $form->populate($data);
        }
        $this->view->form = $form;
    }

    public function generatePreviewAction()
    {
        try {
            if ($this->_request->isPost()) {
                $this->_helper->layout->disableLayout();    //disable layout
                $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering
                $exportsDb = Export_Model_Exports::getInstance();
                $exportData = $exportsDb->createRowDataFromPost($_POST);
                $export = $exportsDb->createRow($exportData);
                $cameras_for_preview = array();
                if (isset($_POST['chosen_sources']) && $_POST['chosen_sources']!="") {
                    $cameraDb = Core_Model_Cameras::getInstance();
                    $camerasIds = explode(',', $_POST['chosen_sources']);
                    foreach ($camerasIds as $cameraId) {
                        $camera_as_stdclass = $cameraDb->getCamera($cameraId);
                        if (isset($camera_as_stdclass)) {
                            $camera = $cameraDb->fetchRow($cameraDb->select()->where("camera_id = ?", $camera_as_stdclass->camera_id));
                            $cameras_for_preview[] = $camera;
                        } else {
                            throw new WebReport_Exception("Could not get camera with id: $cameraId for export preview!");
                        }
                    }
                    $row_limit = 30;
                    echo $export->render($cameras_for_preview, $row_limit);
                } else {
                    echo "No data sources selected.";
                }
            }
        } catch (Exception $e) {
            r_log("Something went wrong when preparing preview");
            r_log($e->getTrace(), "Stacktrace: ");
            r_log($e->getMessage());
            echo "Something went wrong when preparing preview. Please contact support if the problem persists.";
        }
    }

    public function deleteExportAction()
    {
        $exportsDb = Export_Model_Exports::getInstance();
        $export = $exportsDb->getRowById($this->_idParam);
        $export->delete();
        $this->_flashMessenger->addMessage($this->_translate->_('EXPORT_REMOVED_SUCCESSFULLY.'));
        $this->_helper->redirector('index', 'index', 'export');
    }
}
