<?php

class Export_Form_CreateEditExport extends UniForm_Form
{

    protected $_hasEventData;
    protected $_action;
    /**
    * Constructor
    *
    * @see Zend_Form
    * @param mixed $options
    * @return void
    */
    public function __construct($action, $hasEventData)
    {
        $this->_hasEventData = $hasEventData;
        $this->_action = $action;
        parent::__construct();
    }

    public function isValid($values)
    {
        $validation_result = parent::isValid($values);
        /*if ($values["nbr_of_sections"] != 0) {
            for ($i=1; $i<=$values["nbr_of_sections"]; $i++) {
                if ($values["section_name_" . $i] == "") {
                    $validation_result = false;
                }
            }
        }*/
        return $validation_result;
    }

    public function init()
    {
        parent::init();

        $this->addElementPrefixPath('WebReport_Validate_DataExport', 'WebReport/Validate/DataExport', 'validate');

        $action = $this->_action;

        $name = new UniForm_Form_Element_Text('name');
        $name->setLabel('NAME')
          ->setRequired()
          ->addFilter('StringTrim');

        $identity = Zend_Auth::getInstance()->getIdentity();

        if ($identity->multi_company) {
            $companiesDb = new Core_Model_Companies();
            $activeCompanies = $companiesDb->fetchAllActive();
            $company_id = new UniForm_Form_Element_Select('company_id');
            $company_id->setLabel('COMPANY')
              ->addValidator(new Zend_Validate_GreaterThan(0)) // check if user belongs to company
              ->setErrorMessages(array($this->getView()->translate("USER_MUST_BELONG_TO_A_COMPANY.")));

            foreach ($activeCompanies as $company) {
                $company_id->addMultiOption($company->company_id, $company->name);
            }
        } else {
            $company_id = new UniForm_Form_Element_Hidden('company_id');
            $company_id->setValue(Zend_Auth::getInstance()->getIdentity()->company_id);
        }

/*
 * Export specific form elements
 */
        if ($this->_hasEventData) {
            $export_type = new UniForm_Form_Element_Select('export_type');
            $export_type->setLabel('EXPORT_TYPE')
                        ->setDescription('CREATE_EXPORT_EXPORT_TYPE_DESCRIPTION')
                        ->addMultiOptions(
                            array(
                                Export_Model_Exports::COUNT_DATA_EXPORT=>Export_Model_Exports::getTranslatedNameOfType(Export_Model_Exports::COUNT_DATA_EXPORT),
                                Export_Model_Exports::EVENT_DATA_EXPORT=>Export_Model_Exports::getTranslatedNameOfType(Export_Model_Exports::EVENT_DATA_EXPORT)
                            )
                        )
                        ->setValue(0);
        } else {
            $export_type = new UniForm_Form_Element_Hidden('export_type');
            $export_type->SetValue(Export_Model_Exports::COUNT_DATA_EXPORT);
        }

        //if ($this->_hasEventData) {
            $applicationsModel = new Core_Model_Applications();
            //$eventDataApplications = $applicationsModel->fetchAllThatHasVisibleDataInDb("EventData");
            $application_id = new UniForm_Form_Element_Select('application_id');
            //$applications = $applicationsModel->fetchAllThatHasVisibleDataInDb();
            $application_id->setLabel("APPLICATION")
                            ->setRegisterInArrayValidator(false); //As we add the select options dynamically we have to disable validator.
            //foreach ($eventDataApplications as $application) {
            /*foreach ($applications as $application) {
                $application_id->addMultiOption($application->getId(), $application->getName());
            }*/
        //}
        //$this->view->timeResolutions = $countDataDb->getTimeResolutions();
        $timeIntervals = Core_Model_Factory_TimeInterval::makeSelections();
        foreach ($timeIntervals as $key => &$value) {
            $value = $this->getView()->translate($value);
        }
        $time_interval_type = new UniForm_Form_Element_Select('time_interval_type');
        $time_interval_type->setLabel("TIME_INTERVAL")
                            ->addMultiOptions($timeIntervals);

        $manual_time_interval = new UniForm_Form_Element_Text('manual_time_interval');
        $manual_time_interval->setLabel("MANUAL_TIME_INTERVAL");

        //Time interval start and end is created in view due to being multiField that is not supported out of the box from the form.

        $time_resolution = new UniForm_Form_Element_Select('time_resolution');
        $time_resolution->setLabel("TIME_RESOLUTION")
                        ->setRegisterInArrayValidator(false); //As we add the select options dynamically we have to disable validator.

        $aggregate = new UniForm_Form_Element_Checkbox('aggregate');
        $aggregate->setLabel('SUM_SOURCES');
        /*$output_format = new UniForm_Form_Element_Select('output_format');
        $output_format->setLabel("OUTPUT_FORMAT")
                ->addMultiOption('csv', 'CSV')
                //->addMultiOption('xml', 'XML')
                ->addMultiOption('excel', 'EXCEL');*/
        //Remove this and replace with above when more output formats are supported.
        $output_format = new UniForm_Form_Element_Hidden('output_format');
        $output_format->setValue("csv");

        $chosen_sources = new UniForm_Form_Element_Text('chosen_sources');
        $chosen_sources->setLabel('CAMERAS')
                    ->setDescription('CREATE_EXPORT_CAMERA_SELECT_DESCRIPTION');

        /*$nbr_of_sections= new UniForm_Form_Element_Hidden('nbr_of_sections');
        $nbr_of_sections->setValue(0);*/

        //This is moved by jquery to be placed next to submit button.
        $preview = new UniForm_Form_Element_Button('preview');
        $preview->setLabel('PREVIEW')
          ->setAttrib('class', 'secondaryAction ui-button ui-widget ui-state-default ui-corner-all ui-button-text-only');

        $submit = new UniForm_Form_Element_Submit('submit');
        $submit->setAttrib('class', 'primaryAction ui-button ui-widget ui-state-default ui-corner-all ui-button-text-only');
        if ($this->_action == "create") {
            $submit->setLabel('CREATE_EXPORT')
                ->setAttrib("title", $this->getView()->translate('CREATE_AND_STORE_EXPORT'));
        } elseif ($this->_action == "quick-export") {
            $submit->setLabel('GENERATE');
        } else {
            $submit->setLabel('UPDATE_EXPORT');
        }

        $this->addElements(array(
            $name,
            $company_id,
            $export_type,
            $time_interval_type,
            $time_resolution,
            $manual_time_interval,
            $output_format,
            $chosen_sources,
            $aggregate));
        //if ($this->_hasEventData) {
            $this->addElement($application_id);
        //}

        if (isset($company_id)) {
            $this->addElement($company_id);
        }
        $basic_group = array(
            'name',
            'company_id',
            'export_type',
            'time_interval_type',
            'manual_time_interval',
            'output_format',
            'application_id'
            );
        if (!$this->_hasEventData) {
            $basic_group[] = 'time_resolution';
            $basic_group[] = 'aggregate';
        }
        $this->addDisplayGroup($basic_group, 'basic-group', array('legend' => _T('EXPORT_SETUP')));
        if ($this->_hasEventData) {
            $this->addDisplayGroup(array(
                //'application_id',
                'time_resolution',
                'aggregate'), 'type-group', array('legend' => _T('EXPORT_TYPE_DETAILS')));
        }

        $this->addDisplayGroup(array(
            'chosen_sources'), 'content-group', array('legend' => _T('EXPORT_CONTENT')));
        $this->addElements(array($preview, $submit));
    }
}
