<?php

class Export_Model_Export extends Zend_Db_Table_Row_Abstract {

    public function getDataSources() {
        $exportCameras = $this->getExportCameraRows();
        $cameraDb = Core_Model_Cameras::getInstance();
        $cameras = array();
        foreach ($exportCameras as $ec) {
            $cameras[] = $cameraDb->getCamera($ec->camera_id);
        }
        return $cameras;
    }

    public function getExportCameraRows() {
        $exportCameraDb = Export_Model_ExportCameras::getInstance();
        $select = $exportCameraDb->select();
        $select->where('export_id = ?', $this->export_id);
        $exportCameras = $exportCameraDb->fetchAll($select);
        return $exportCameras;
    }

    public function getURL()
    {
        $base_url = Zend_Registry::get('config')->resources->frontController->baseUrl;
        $port = '';
        $protocol = 'http://';
        if ($_SERVER['SERVER_PORT'] == '443') {
            $protocol = 'https://';
        }
        else if ($_SERVER['SERVER_PORT'] != '80') {
            $port = ':' . $_SERVER['SERVER_PORT'];
        }
        $address = $_SERVER["SERVER_NAME"].$port.$base_url."/api/?method=export.GetExport&id=";
        $permalink = $this->export_id.$this->access_key;
        return $protocol.$address.$permalink;
    }

    public function touch()
    {
        $this->last_accessed = new Zend_Db_Expr('NOW()');
        $this->times_accessed = $this->times_accessed + 1;
        $this->save();
    }

    public function render($sources_for_preview = null, $row_limit = null)
    {
        $locale = new Zend_Locale('sv');
        Zend_Registry::set('Zend_Locale', $locale);
        $outputFormat = $this->output_format;
        $timeFormat = $this->time_interval_type;
        $timeResolution = $this->time_resolution;
        $exportType = $this->export_type;
        $aggregatorId = $this->aggregator_id;
        $firstDateSelected = new Zend_Date($this->fix_interval_start);
        $lastDateSelected = new Zend_Date($this->fix_interval_end);
        $settings['type'] = $exportType;
        if ($this->aggregator_id!= 0) {
            $settings['aggregate'] = 1;
        }
        if (isset($sources_for_preview)) {
            $cameras = $sources_for_preview;
        } else {
            $cameras = $this->getDataSources();
        }

        $timeFilterId = 0; //For now.

        if ($timeFormat == "man") {
            $timeFormat = $manualTimeInterval;
            if (!CM_Interval::isValidCode($timeFormat)) {
                $this->render("badtime");
                return;
            }
        } elseif ($timeFormat != "fix") {
            $inter = new CM_Interval(Zend_Date::now(), $timeFormat);
            if (!is_null($inter)) {
                $firstDateSelected = $inter->getStartDate();
                $lastDateSelected = $inter->getStopDate();
            } else {
                // $todo: do something proper here
                $firstDateSelected = new Zend_Date();
                $lastDateSelected = new Zend_Date();
            }
        }

        if ($outputFormat == "csv") {
            $class = "WebReport_Export_Fine";
        } else {
            $class = "WebReport_Export_" . ucfirst($outputFormat);
        }
        $exportRenderer = new $class();

        $timeFilterDb = new Core_Model_CountData_TimeFilters();
        $timeFilter = $timeFilterDb->fetchById((int) $timeFilterId);
        /*$countDataSets = new Core_Model_CountData_Sets();
        $countDataSet = $countDataSets->find($countDataSetId)->current();*/

        /*if ($aggregatorId > 0) {q
            $dataAggregatorDb = new Charts_Model_DataAggregatorList();
            $aggregator = $dataAggregatorDb->getAggregator($aggregatorId);

            $aggregator->setData($countDataSet);
            $aggregator->setPeriod($firstDateSelected, $lastDateSelected);

            if($timeFilter != null){
                $aggregator->setTimeFilter($timeFilter);
            }

            $data_tmp = $aggregator->renderData();

            $labels = $aggregator->getXLabel();

            $descriptions = array("bin" => "Time Segment") + $aggregator->getDataLabels(false);

            $unit = $aggregator->getDisplayUnit();
            if ($unit != "") {
                foreach ($data_tmp as $data) {
                    foreach (array_keys($descriptions) as $label) {
                        $data->$label .= $unit;
                    }
                }
            }

            $ind = 0;
            foreach ($labels as $label)
                $data_tmp[$ind++]->bin = $label;

            $exportRenderer->setHeaders($descriptions)->setData($data_tmp);
        } else {*/
            //$countDataDb = $countDataSet->getCountData();
        if ($exportType == Export_Model_Exports::COUNT_DATA_EXPORT) {
            $dataModel = new Core_Model_CountData();
            $dataModel->addCameras($cameras);
            $dataModel->initDatabaseSelect();
            //$unitDataAggregator = new Charts_Model_DataAggregator_Unit();
            //$unitDataAggregator->setCountData($dataModel);
            //$unitDataAggregator->setWantedResolution($timeResolution);
            //r_log($firstDateSelected->toString());
            //r_log($lastDateSelected->toString());
            //$unitDataAggregator->setPeriod($firstDateSelected, $$firstDateSelected);
            //$data = $unitDataAggregator->renderData(false);
            //$labels = $unitDataAggregator->getExportTimestampLabels();
            //r_log($unitDataAggregator->getXLabelsRaw());
            //$exportRenderer->setTimestampLabels($labels);
            $data = $dataModel->getData($timeResolution, $firstDateSelected, $lastDateSelected, $timeFilter, $row_limit, true);
            $descriptions = $dataModel->getDescriptionsForLastFetchedData2();
        } elseif ($exportType == Export_Model_Exports::EVENT_DATA_EXPORT) {
            $dataModel = new Core_Model_EventData();
            $dataModel->addCameras($cameras);
            $data = $dataModel->getData($firstDateSelected, $lastDateSelected, false, $row_limit);
            $descriptions = $dataModel->getDescriptionsForLastFetchedData();
            $settings['aggregate'] = 0;
            //för imorgon: Fixa headers, fixa omräkning till UTC timestamp på tidsperiod.
        } else {
            throw new WebReportException("Unknown export type.");
        }
        $exportRenderer->setSettings($settings);
        $exportRenderer->setHeaders($descriptions)->setData($data);
        //}
        $exportRenderer->render(null, $row_limit);
    }
}
