<?php

class Export_Model_Exports extends Talking_Db_Table {

    protected $_name = 'export';
    protected $_primary = 'export_id';
    protected $_rowClass = 'Export_Model_Export';

    const COUNT_DATA_EXPORT = 1;
    const EVENT_DATA_EXPORT = 2;

    public function getRowById($id)
    {
        $select = $this->select();
        $select->where('export_id = ?', $id);
        $row = $this->fetchRow($select);
        return $row;
    }

    public function fetchAllForCompany($company)
    {
        $select = $this->select();
        $select->where('company_id = ?', $company->company_id);
        $exports = $this->fetchAll($select);
        return $exports;
    }

    public function tryFetchExport($id, $key)
    {
        $select = $this->select();
        $select->where('access_key = ?', $key);
        $row = $this->fetchAll($select)->current();
        if ($row->export_id != $id) {
            return null;
        }
        return $row;
    }

    public static function getTranslatedNameOfType($type)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        if ($type==Export_Model_Exports::COUNT_DATA_EXPORT) {
            return _('TIME_AGGREGATED_EXPORT');
        } elseif ($type==Export_Model_Exports::EVENT_DATA_EXPORT) {
            return _('RAW_EVENT_DATA_EXPORT');
        }
    }

    public function createRowDataFromPost($values, $existing = null)
    {
        if (!isset($existing)) {
            $key_length = rand(8, 15);
            $key = substr("k". md5(uniqid(rand(), true)), 0, $key_length);
        } else {
            $key = $existing->access_key;
        }
        if ($values['time_interval_type'] == "man") {
            $timeIntervalType = $values['manual_time_interval'];
        } else {
            $timeIntervalType = $values['time_interval_type'];
        }
        $fixStart =  $fixEnd = null;
        if ($values['time_interval_type'] == "fix") {
            $fixStart = $values["time-interval-start"];
            $fixEnd = $values["time-interval-stop"];
        }
        if ($values['export_type'] == 2) {
            $timeResolution = null;
        } else {
            $timeResolution = $values["time_resolution"];
        }
        $exportsDb = Export_Model_Exports::getInstance();
        $exportData = array(
                'name' => $values["name"],
                'export_type' => $values["export_type"],
                'application_id' => $values["application_id"],
                'company_id' => $values["company_id"],
                'output_format' => $values["output_format"],
                'time_interval_type' => $timeIntervalType,
                'fix_interval_start' => $fixStart,
                'fix_interval_end' => $fixEnd,
                'time_resolution' => $timeResolution,
                'aggregator_id' => $values["aggregate"],
                'access_key' => $key
                );

        return $exportData;
    }
}
