<?php

class Export_Model_LegacyPermalinkListGenerator extends Core_Model_Generic_AbstractListView {

    public function __construct()
    {
        parent::__construct();

        $this->_extendedColumns = false;
        $this->_module = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();
        if ($this->_module == "overview") {
            $this->_extendedColumns = true;
        }
        $this->_timeIntervals = Core_Model_Factory_TimeInterval::makeSelections();
        $this->_timeResolutions = array_merge(
            Core_Model_Application_PeopleCounter::getTimeResolutions(),
            Core_Model_Application_StereoPeopleCounter::getTimeResolutions(),
            Core_Model_Application_Queue2::getTimeResolutions(),
            Core_Model_Application_Demographics::getTimeResolutions());
        if ($this->_isMultiCompany) {
            $this->_companyDb = Core_Model_Companies::getInstance();
        }
    }

    public function getHeaders()
    {
        $headers = array(_T("URL"),_T("FORMAT"),_T("TIME_FILTER"),_T("RESOLUTION"),_T("LAST_ACCESSED"));
        if ($this->_isMultiCompany) {
            $headers[] = _T("COMPANY");
        }
        return $headers;
    }

    protected function _dataRowToTableRow($row)
    {
        $data_decoded = (array) json_decode($row->permalink_data);
        $url = $row->getApi();
        $timeFormat = $data_decoded['timeFormat'];
        if (isset($this->_timeIntervals[$timeFormat])) {
            $readbleTimeFormat = $this->_timeIntervals[$timeFormat];
        }
        $timeFormatAppend = "";
        if (!isset($readbleTimeFormat)) {
            $readbleTimeFormat = _('MANUAL');
            $timeFormatAppend = ": [ " . $data_decoded['timeFormat'] . " ]";
        } elseif ($readbleTimeFormat == "FIXED_INTERVAL") {
            $timeFormatAppend = ": [ " . $data_decoded['firstDateSelected'] . " - " . $data_decoded['lastDateSelected'] . " ]";
        }
        $ret = array();
        $ret[] = $url;
        $ret[] = $data_decoded['$outputFormat'];
        $ret[] = $this->translate($readbleTimeFormat) . $timeFormatAppend;
        $ret[] = $this->_timeResolutions[$data_decoded['timeResolution']];
        $ret[] = $row->last_api_read;
        if ($this->_isMultiCompany) {
            $ret[] = $this->_companyDb->getCompany($row->company_id)->name;
        }
        return $ret;
    }

    public function removeText(){
        return _T('DO_YOU_REALLY_WANT_TO_REMOVE_THE_MIGRATED_PERMALINK_"%1$s"?');
    }

    protected function _dataRowToRowId($row){
        return $row->permalink_id;
    }

    public function noDataInformation(){
        return "No migrated permalinks, TODO: Hide this table!";
    }

    public function getActions($row){
        $actions = array();
        $actions[] = parent::CreateAction(_T('VIEW_EXPORT'), null, 'view', null, null, null, null, null, $row->getApi());
        $actions[] = parent::CreateAction(_T('DELETE_PERMANENT_LINK'), 'delete-legacy-permalink', 'delete', null, null, null, 'export', 'index');
        return $actions;
    }
}
