<?php

class Export_Model_StoredExportsListGenerator extends Core_Model_Generic_AbstractListView {

    protected $_hasEventData;

    public function __construct()
    {
        parent::__construct();

        $this->_extendedColumns = false;
        $front = Zend_Controller_Front::getInstance();
        $this->_module = $front->getRequest()->getModuleName();
        if ($this->_module == "overview") {
            $this->_extendedColumns = true;
        }
        $this->_timeIntervals = Core_Model_Factory_TimeInterval::makeSelections();
        $this->_timeResolutions = array_merge(
            Core_Model_Application_PeopleCounter::getTimeResolutions(),
            Core_Model_Application_StereoPeopleCounter::getTimeResolutions(),
            Core_Model_Application_Queue2::getTimeResolutions(),
            Core_Model_Application_Demographics::getTimeResolutions());
        if ($this->_isMultiCompany) {
            $this->_companyDb = Core_Model_Companies::getInstance();
        }
        //$this->_hasEventData = Core_Model_EventData::userHasVisibleEventData($this->_user);

    }

    public function getHeaders()
    {
       //$headers = array(_T("NAME"),_T("URL"),_T("TIME_FILTER"),_T("RESOLUTION"),_T("TIMES_ACCESSED"),_T("LAST_ACCESSED"));
        $headers = array(_T("NAME"),_T("TIME_FILTER"),_T("RESOLUTION"),_T("TIMES_ACCESSED"),_T("LAST_ACCESSED"),_T("API_ID"));
        /*if ($this->_hasEventData) {
            $headers[] = _T("TYPE");
        }*/
        if ($this->_isMultiCompany) {
            $headers[] = _T("COMPANY");
        }
        return $headers;
    }

    protected function _dataRowToTableRow($row)
    {
        //$url = $row->getURL();
        $timeFilter = $row->time_interval_type;
        if (isset($this->_timeIntervals[$timeFilter])) {
            $readbleTimeFormat = $this->_timeIntervals[$timeFilter];
        }
        $timeFilterAppend = "";
        if (!isset($readbleTimeFormat)) {
            $readbleTimeFormat = _('MANUAL');
            $timeFilterAppend = ": [ " . $row->time_interval_type . " ]";
        } elseif ($readbleTimeFormat == "FIXED_INTERVAL") {
            $timeFilterAppend = ": [ " . $row->fix_interval_start . " - " . $row->fix_interval_end . " ]";
        }

        if (!isset($row->last_accessed) || $row->last_accessed == "") {
            $last_accessed = $this->translate('NEVER');
        } else {
            $last_accessed = $row->last_accessed;
        }
        if (!isset($row->time_resolution) || $row->time_resolution == "") {
            $timeResolution = "n/a";
        } else {
            $timeResolution = $this->_timeResolutions[$row->time_resolution];
        }
        $ret = array();
        $ret[] = $row->name;
        //$ret[] = $url;
        //$ret[] = $row->output_format;
        $ret[] = $this->translate($readbleTimeFormat) . $timeFilterAppend;
        $ret[] = $timeResolution;
        $ret[] = $row->times_accessed;
        $ret[] = $last_accessed;
        $ret[] = $row->export_id . $row->access_key;
        /*if ($this->_hasEventData) {
            $ret[] =  Export_Model_Exports::getTranslatedNameOfType($row->export_type);
        }*/
        if ($this->_isMultiCompany) {
            $ret[] = $this->_companyDb->getCompany($row->company_id)->name;
        }
        return $ret;
    }

    public function removeText(){
        return _T('DO_YOU_REALLY_WANT_TO_REMOVE_STORED_EXPORT_"%1$s"?');
    }

    protected function _dataRowToRowId($row){
        return $row->export_id;
    }

    public function noDataInformation(){
        return _("NO_STORED_EXPORTS_FOUND.");
    }

    public function getActions($row){
        $actions = array();
        $actions[] = parent::CreateAction(_T('VIEW_EXPORT'), null, 'view-external export', null, null, null, null, null, $row->getURL());
        $actions[] = parent::CreateAction(_T('EDIT_EXPORT'), 'edit-export', 'edit', null, null, null, 'export', 'export');
        $actions[] = parent::CreateAction(_T('COPY_URL_TO_CLIPBOARD'), null, 'clipboard', null, null, null, null, null, $row->getURL());
        $actions[] = parent::CreateAction(_T('DELETE_EXPORT'), 'delete-export', 'delete', null, null, null, 'export', 'export');
        return $actions;
    }
}
