<?php

class Heatmap_IndexController extends WebReport_Controller_Action
{

    public function indexAction()
    {
        $form = new UniForm_Form(); // force include of css
        $camerasDb = new Core_Model_Cameras();
        $heatmapApp = new Core_Model_Application_Heatmap();

        $identity = Zend_Auth::getInstance()->getIdentity();
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($identity->company_id)->current();
        if($identity->multi_company){
            $cameras = $camerasDb->getCameras($heatmapApp);
        }else{
            $cameras = $camerasDb->getCameras($heatmapApp, $company);
        }
        $this->view->headScript()->appendFile($this->view->baseUrl("/library/javascript/excanvas.compiled.js"));

        $this->view->headTitle()->prepend("HEATMAPS");
        $this->view->headTitle()->prepend("BROWSE");

        $this->view->cameras = $cameras;
    }

    public function maskListAction()
    {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        $ret = array();
        $camera_id = (int)$this->_getParam("id",0);

        //this will throw an error width incorrect privileges, and this is what we want.
        $cameraDb = new Core_Model_Cameras();
        $camera = $cameraDb->getCamera($camera_id);

        $identity = Zend_Auth::getInstance()->getIdentity();
        $maskDb = new Heatmap_Model_Masks();
        $where = $maskDb
            ->getAdapter()
            ->quoteInto('camera_id = ? and auto = 0', $camera_id);
        $masks = $maskDb->fetchAll($where);

        foreach($masks as $mask) {
            $ret[] = array(
                "heatmap_mask_id" => $mask->heatmap_mask_id,
                "name" => $mask->name,
                "mask" => $mask->mask,
                "camera_id" => $mask->camera_id
            );
        }

        echo Zend_Json::encode($ret);
    }

    public function maskAjaxAction()
    {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        if (!$this->getRequest()->isPost() || !isset($_POST['action'])) {
            echo "[]";
            return;
        }
        //fixme: validation of post
        $translate = Zend_Registry::get('Zend_Translate');
        $maskDb = new Heatmap_Model_Masks();
        $ret = array(
            "status" => "fail",
            "heatmap_mask_id" => -1
        );
        switch($_POST['action']) {
            case "add":
                $data = array(
                    "name" => $_POST['mask-name'],
                    "auto" => 0,
                    "mask" => $_POST['mask'],
                    "camera_id" => $_POST['camera_id']
                );
                $ret['status'] = "ok";
                $ret['msg'] = $translate->_("MASK_WAS_ADDED.");
                $ret['heatmap_mask_id'] = $maskDb->insert($data);
                break;
            case "edit":
                $where = $maskDb
                    ->getAdapter()
                    ->quoteInto('heatmap_mask_id = ?', $_POST['heatmap_mask_id']);
                $data = array(
                    "name" => $_POST['mask-name'],
                    "mask" => $_POST['mask']
                );
                $maskDb->update($data,$where);
                $ret["msg"] = $translate->_("MASK_WAS_SAVED.");
                $ret['status'] = "ok";
                $ret['heatmap_mask_id'] = $_POST['heatmap_mask_id'];
                break;
            case "delete":
                $where = $maskDb
                    ->getAdapter()
                    ->quoteInto('heatmap_mask_id = ?', $_POST['heatmap_mask_id']);
                $maskDb->delete($where);
                $ret['msg'] = $translate->_("MASK_WAS_DELETED.");
                $ret['status'] = "ok";
                break;
            case "nop":
                $ret['status'] = "nop";
                break;
            default:
                throw new WebReport_Error("Invalid action");
        }

        echo Zend_Json::encode($ret);
    }

    public function heatmapsAction()
    {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        $camerasDb = new Core_Model_Cameras();
        $heatmapApp = new Core_Model_Application_Heatmap();

        $id = (int)$this->_getParam("id",0);
        $date = $this->_getParam("date");
        $delta = $this->_getParam("delta", 5*60);
        $interval= $this->_getParam("interval", null);
        $isInterval = $this->_getParam("delta", "1") == "0";

        $downscaled = (int)$this->_getParam("downscaled", 0);

        $identity = Zend_Auth::getInstance()->getIdentity();
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($identity->company_id)->current();
        $company = $identity->multi_company ? null : $company;
        $cameras = $camerasDb->getCameras($heatmapApp, $company, array($id));

        if(!$isInterval){
            $interval = array (0, 24);
        } else if( !($interval[0] >= 0 && $interval[0] <= 25)  ||  !($interval[1] >= 0 && $interval[1] <= 25)   ) {
            throw new WebReport_Exception("Invalid interval!");
        }

        $cameraImagesDb = new Core_Model_CameraImages();
        $imageSelect = $cameraImagesDb->select();

        $camera = current($cameras);
        $h = array();
        if ($camera) {
            $imageSelect->where('camera_id = ?', $camera->camera_id)
                    ->where("DATE_FORMAT(time,'%Y%m%d') = ?", $date)
                    ->order('time');

            $images = $cameraImagesDb->fetchAll($imageSelect);

            // if no image for current day select another
            if (count($images) == 0) {
                $imageSelect = $cameraImagesDb->select();
                $imageSelect->where('camera_id = ?', $camera->camera_id)
                    ->where("time > STR_TO_DATE(?,'%Y%m%d')", $date)
                    ->order('time ASC')
                    ->limit(1,0);
                $images = $cameraImagesDb->fetchAll($imageSelect);

            }
            $maskDb = new Heatmap_Model_Masks();
            $heatmapsDb = new Heatmap_Model_Heatmaps();

            $h["name"] = $camera->name;
            $h["id"] = $camera->camera_id;

            $all = array();

            $db = $heatmapsDb;
            $fields = array("camera_id", "max", "time","width");
            if ($downscaled) {
                $fields[] = "data_ds4";
            } else {
                $fields[] = "data";
            }

            $fields[] = 'heatmap_mask_id';

            $select = $heatmapsDb->select($fields);
            if($isInterval) $delta = 60*60;
            $select->where('camera_id = ?', $id)
                ->where('delta = ?', $delta)
                ->where("time > DATE_ADD(STR_TO_DATE(?,'%Y%m%d'), INTERVAL '".$interval[0]."' HOUR)", $date)
                ->where("time <= DATE_ADD(STR_TO_DATE(?,'%Y%m%d'), INTERVAL '".$interval[1]."' HOUR)", $date)
                ->order('time');
            $heatmaps = $db->fetchAll($select);
            if(!$isInterval){
                foreach ($heatmaps as $heatmap) {
                    r_log("Found heatmap.");
                    $d = array();
                    if ($downscaled) {
                        $d['data'] = $heatmap->data_ds4;
                    } else {
                        $d['data'] = $heatmap->data;
                    }

                    if(!is_null($heatmap->heatmap_mask_id) && $heatmap->heatmap_mask_id>0) {
                        $d['mask'] = $maskDb->find($heatmap->heatmap_mask_id)->current()->mask;
                    } else {
                        $d['mask'] = null;
                    }

                    $d['max'] = $heatmap->max;
                    $all[substr($heatmap->time,11,5)] = $d; // add time as key in format hh:mm
                }
                $h['heatmaps'] = $all;
            } else {
                $d = array();
                $hm = $heatmapsDb->createRow();
                $hm->delta = 60*60;
                if ($downscaled) {
                    $t = microtime(true);
                    $d['data'] = $hm->mean($heatmaps,true)->data_ds4;
                    $t = microtime(true) - $t;
                } else {
                    $t = microtime(true);
                    $d['data'] = $hm->mean($heatmaps)->data;
                    $t = microtime(true) - $t;
                }
                $d['max'] = $hm->max;
                if(count($heatmaps) > 0 && !is_null($heatmaps[0]->heatmap_mask_id) && $heatmaps[0]->heatmap_mask_id>0) {
                    $d['mask'] = $maskDb->find($heatmaps[0]->heatmap_mask_id)->current()->mask;
                } else {
                    $d['mask'] = null;
                }
                $hour = ($interval[1] < 10 ? "0" .  $interval[1] : $interval[1]) . ":00";
                $h['heatmaps'] = array( $hour => $d);
            }
            $h['wasInterval'] = $isInterval;
            $imTimes = array();
            $imFilenames = array();
            foreach ($images as $image) {
                $imTimes[] = substr($image->time,11,5);
                $imFilenames[] = $image->filename;
            }
            $h['images'] = array("times" => $imTimes, "filenames" => $imFilenames);

        }

        echo Zend_Json::encode($h);
    }

   /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {
        $this->_translate->_('MOD_HEATMAP_NAME');
        $this->_translate->_('MOD_HEATMAP_MENU_BROWSE_HEATMAPS');
        $this->_translate->_('MOD_HEATMAP_MENU_SETTINGS');
    }

}
