<?php
require_once(APPLICATION_PATH . "/abc.php");
class Installation_IndexController extends Zend_Controller_Action
{

    protected $_translate = null;

    public function init()
    {
        $t = Zend_Registry::get('Zend_Translate');

        $this->_helper->layout->setLayout('installation');
        $view = $this->view;
        $view->headTitle()->setTranslator($t);
        $view->headTitle("", Zend_View_Helper_Placeholder_Container_Abstract::SET);
        $view->headTitle('AXIS Store Data Manager', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $view->headTitle('INSTALLATION', Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $view->headTitle()->setSeparator(' - ');


        $steps = array();

        $view->headTitle()->enableTranslation();

        $steps[] = $t->_("CHOOSE_LANGUAGE");
        $steps[] = $t->_("CHECK_IF_SYSTEM_REQUIREMENTS_ARE_FULFILLED");
        $steps[] = $t->_("LICENSE_AGREEMENT");
        $steps[] = $t->_("REGISTRATION_CODE");
        $steps[] = $t->_("DATABASE_CONFIGURATION");
        $steps[] = $t->_("USER_AND_COMPANY_INFORMATION");
        $steps[] = $t->_("FINISH_INSTALLATION");
        $this->view->steps = $steps;
    }

    public function indexAction()
    {
        $this->_helper->redirector('language');
    }

    public function languageAction()
    {
        if (Zend_Auth::getInstance()->hasIdentity()) {
            Zend_Auth::getInstance()->clearIdentity();
        }

        $this->view->headTitle("CHOOSE_LANGUAGE", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $this->view->step = 1;

        $locale = new Zend_Locale("en");

        $availableLanguages = Zend_Registry::get('languages');
        $languages = array();
        foreach ($availableLanguages as $avLang) {
            $languages[$avLang] = ucfirst($locale->getTranslation($avLang,'language', $avLang));
        }

        $language = $this->_getParam('lang');

        $this->view->languages = $languages;

        $sessionNamespace = new Zend_Session_Namespace('WebReport_Installation');

        if ($language) {
            $sessionNamespace->language = $language;
            $this->_helper->redirector('requirements');
        }
    }

    public function requirementsAction()
    {
        $this->view->step = 2;

        $this->view->headTitle("SYSTEM_REQUIREMENTS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $t = Zend_Registry::get('Zend_Translate');

        $environment = array();
        $environment[$t->_("PHP_VERSION") . " >= 5.4"] = version_compare(PHP_VERSION, '5.4', '>=');
        $environment[$t->_("config.ini_IS_WRITABLE")] = is_writable( APPLICATION_PATH . "/config.ini");
        $environment[$t->_("license.ini_IS_WRITABLE")] = is_writable( APPLICATION_PATH . "/license.ini");

        $environment[sprintf($t->_('CACHE_DIRECTORY_%1$s_IS_WRITABLE'), realpath(APPLICATION_PATH . '/../cache'))] = is_writable( realpath(APPLICATION_PATH . "/../cache"));

        $this->view->environment = $environment;

        $extensions = array("gd", "pcre", "PDO", "pdo_mysql", "Reflection", "session", "SimpleXML", "SPL", "standard", "zlib", "openssl", "curl");
        $extensionsAvailable = array();
        foreach ($extensions as $ext) {
            $extensionsAvailable[$ext] = extension_loaded($ext);
        }
        $this->view->extensions = $extensionsAvailable;

        $settings = array();
        
        $set = array("setting" => "safe_mode", "recommended" => $t->_("OFF"));
        if (ini_get("safe_mode")) { $set['value'] = $t->_("ON"); $set['ok'] = false; }
        else { $set['value'] = $t->_("OFF"); $set['ok'] = true; }
            $settings[] = $set;

        $set = array("setting" => "short_open_tag", "recommended" => $t->_("ON"));
        if (ini_get("short_open_tag")) { $set['value'] = $t->_("ON"); $set['ok'] = true; }
        else { $set['value'] = $t->_("OFF"); $set['ok'] = false; }
            $settings[] = $set;

        $set = array("setting" => "memory_limit", "recommended" => $t->_("32M_OR_MORE"));
        if (intval(ini_get("memory_limit")) >= 32) { $set['value'] = ini_get("memory_limit"); $set['ok'] = true; }
        else { $set['value'] = ini_get("memory_limit"); $set['ok'] = false; }
            $settings[] = $set;


        $set = array("setting" => "register_globals", "recommended" => $t->_("OFF"));
        if (ini_get("register_globals")) { $set['value'] = $t->_("ON"); $set['ok'] = false; }
        else { $set['value'] = $t->_("OFF"); $set['ok'] = true; }
            $settings[] = $set;

        //$settings[] = array("setting" => "session.auto_start", "recommended" => $t->_("OFF"));
        $set = array("setting" => "session.auto_start", "recommended" => $t->_("OFF"));
        if (ini_get("session.auto_start")) { $set['value'] = $t->_("ON"); $set['ok'] = false; }
        else { $set['value'] = $t->_("OFF"); $set['ok'] = true; }
            $settings[] = $set;



        $set = array("setting" => "file_uploads", "recommended" => $t->_("ON"));
        if (ini_get("file_uploads")) { $set['value'] = $t->_("ON"); $set['ok'] = true; }
        else { $set['value'] = $t->_("OFF"); $set['ok'] = false; }
            $settings[] = $set;





        $this->view->settings = $settings;


        $form = new UniForm_Form();
    }

    public function licenseAction()
    {
        $this->view->step = 3;
        $form = new UniForm_Form();
    }

    public function registrationAction()
    {
        $this->view->step = 4;
        $form = new Installation_Form_Registration();
        $t = Zend_Registry::get('Zend_Translate');
        $config = Zend_Registry::get('config');
        $license = Zend_Registry::get('config-license');
        $this->view->form = $form;
        $config->general->usersCanSetNbrOfChannels = "1";
        $config->general->channelAdminHash = "";
        $config->general->httpProxy = "";
        $config->general->httpProxyPort = "";
        $config->general->httpProxyUser = "";
        $config->general->httpProxyPass = "";

        try {
            $writer = new Zend_Config_Writer_IniNice();
            $writer->setConfig($config)
                ->setFilename(APPLICATION_PATH . '/config.ini');

            $writer->write(); // throws exception if not ok
        } catch (Zend_Config_Exception $e) {
            if(substr($e->getMessage(), 0,11) === "Invalid key") {
                $form->addError($t->_("THE_CONFIG_FILE_SEEMS_CORRUPT._PLEASE_REINSTALL_TrueView_DataManager<sup>&reg;</sup>."));
            } else {
                $form->addError($t->_("COULD_NOT_WRITE_TO_CONFIG_FILE._MAKE_SURE_IT_HAS_CORRECT_PERMISSIONS."));
            }
            return;
        }

        if ($this->_request->isPost()) {
            $license->registrationCode = $_POST['registration_code'];
            $license->installationId = $_POST['installation_id'];
            $license->activationCode = $_POST['activation_code'];

            $writer = new Zend_Config_Writer_IniNice();
            $writer->setConfig($config)
                ->setFilename(APPLICATION_PATH . '/config.ini');

            $writer->write();

            $licence_writer = new Zend_Config_Writer_Ini();
            $licence_write->setConfig($config)
                ->setFileName(APPLICATION_PATH . '/licence.ini');
            $license_write->write();

            $this->_helper->redirector('database');
        }

    }

    public function registrationAjaxAction()
    {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        if (!$this->getRequest()->isPost()) {
            echo "[]";
            return;
        }

        $config = Zend_Registry::get('config');
        $ret = array();

        /*if (!extension_loaded("cmwebreport")) {
            $ret['error'] = true;
            $ret['type'] = 'cmwebreport-extension-not-loaded';
            echo Zend_Json::encode($ret);
            return;
        }*/

        $response = cm_installid2($_POST['registrationCode']);

        if (preg_match("/^[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}$/", $response)) {
            $ret['error'] = false;
            $ret['installationId'] = $response;
        } else {
            $ret['error'] = true;
            $ret['type'] = 'invalid-code';
        }
        echo Zend_Json::encode($ret);

    }


    public function registrationWithActivationAjaxAction()
    {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        if (!$this->getRequest()->isPost()) {
            echo "[]";
            return;
        }

        $config = Zend_Registry::get('config-license');

        $ret = array();
        $actcode = $_POST['activationCode'];
        $regcode = $_POST['registrationCode'];
        $response = cm_isvalid_activation2($regcode,$actcode);

        if ($response !== true) {
            $ret['error'] = true;
            $ret['type'] = 'invalid-actcode';
        } else {
            $ret['error'] = false;
        }

        echo Zend_Json::encode($ret);
    }

    public function connectionTestAjaxAction() {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        if(!$this->getRequest()->isPost()) {
            echo Zend_Json::encode(array("connection" => false,"message" => "invalid post"));
            return;
        }

        try {
            $import_connection=array(
                "host" => $_POST['database_import_params_host'],
                "port" => $_POST['database_import_params_port'],
                "username" => $_POST['database_import_params_username'],
                "password" => $_POST['database_import_params_password'],
                "dbname" => $_POST['database_import_params_dbname']
            );
            $db_import = new Installation_Model_DatabaseImport($import_connection,NULL);
            $db_import->testConnection();
        } catch(Exception $e) {
            echo Zend_Json::encode(array("connection"=>false, "message" => $e->getMessage()));
            return;
        }

        echo Zend_Json::encode(array("connection" => true, "message" => "success"));
    }

    public function registrationSaveParametersAjaxAction()
    {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        if (!$this->getRequest()->isPost()) {
            echo "[]";
            return;
        }

        $config = Zend_Registry::get('config-license');
        $config->registrationCode = $_POST['registrationCode'];
        $config->installationId = $_POST['installationId'];
        $config->activationCode = $_POST['activationCode'];
        try {
            $writer = new Zend_Config_Writer_IniNice();
            $writer->setConfig($config)
                ->setFilename(APPLICATION_PATH . '/license.ini');
            $writer->write();
        } catch (Exception $e) {
            echo Zend_Json::encode(array("error" => true, "message" => $e->getMessage()));
            return;
        }

        echo Zend_Json::encode(array("error" => false));
    }

    public function databaseAction()
    {
        $this->view->step = 5;
        $this->view->headTitle("Setup database", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $t = Zend_Registry::get('Zend_Translate');
        $form = new Installation_Form_Database();
        $config = Zend_Registry::get('config');
        if($config == null){
            $form->addError($t->_("LOST_SESSION_ON_SERVER._PLEASE_START_THE_INSTALLATION_OVER."));
            return;
        }

        $this->view->form = $form;


        try {

            $writer = new Zend_Config_Writer_IniNice();
            $writer->setConfig($config)
                ->setFilename(APPLICATION_PATH . '/config.ini');

            $writer->write(); // throws exception if not ok
        } catch (Zend_Config_Exception $e) {
            if(substr($e->getMessage(), 0,11) === "Invalid key") {
                $form->addError($t->_("THE_CONFIG_FILE_SEEMS_CORRUPT._PLEASE_REINSTALL_TrueView_DataManager<sup>&reg;</sup>."));
            } else {
                $form->addError($t->_("COULD_NOT_WRITE_TO_CONFIG_FILE._MAKE_SURE_IT_HAS_CORRECT_PERMISSIONS."));
            }

            if ($this->getRequest()->isPost()) {
                $form->populate($_POST);
            }
            return;

        }

        $ok=false;


        if ($this->getRequest()->isPost()) {


            if ($form->isValid($_POST)) {
                set_time_limit(60*20);
                $values = $form->getValues();

                $config->database->params->host = $values['database_params_host'];
                $config->database->params->username = $values['database_params_username'];
                $config->database->params->password = $values['database_params_password'];
                $config->database->params->dbname = $values['database_params_dbname'];
    
                if($values['database_newdb']) {
                    $config->database->params->dbname = '';
                }

                $assignedPassword = false;
                /*
                 * If password is blank the user can set it.
                 */
                if($values['database_params_username'] == 'root' && $values['database_params_password'] == $values['database_params_password_c']) {
                    $c = mysql_connect("localhost", "root", "");

                    $pw = @mysql_escape_string($values['database_params_password']);
                    $response = mysql_query("SET PASSWORD FOR 'root'@'localhost' = PASSWORD('" . $pw . "')");
                    mysql_query("FLUSH PRIVILEGES;");
                    $assignedPassword = true;
                }

                $writer->setConfig($config);
                $writer->write();
                $conn = null;
                $db = null;
                $dbExists = false;
                try {

                    $sqlfile = APPLICATION_PATH . '/modules/installation/sql/installation.sql';
                    $buffer = @file_get_contents($sqlfile);

                    if(!$buffer) {
                        throw new WebReport_Exception(sprintf($t->_('COULDN\'T_LOAD_THE_FILE_%1$s.'), $sqlfile));
                    }
                    $db = Zend_Db::factory($config->database);

                    if($values['database_newdb']) {
                        try {
                            $conn = $db->getConnection();
                            /*$version = $conn->query('SELECT VERSION()')->fetch();
                            $over_574 = version_compare($version[0], "5.7.4");
                            if ($over_574 == 1) {
                                $conn->exec('SET GLOBAL sql_mode=""'); //This is now handled in installation script by changing my.ini. Leaving it here if other settings must be set depening on mysql version.
                            }*/
                            $response = $conn->exec('CREATE DATABASE ' . $values['database_params_dbname'] . ' character set utf8 collate utf8_bin');
                            error_log($response);
                            $config->database->params->dbname = $values['database_params_dbname'];
                            $writer->setConfig($config);
                            $writer->write();
                            $db = Zend_Db::factory($config->database);
                        } catch (Exception $e) {
                            error_log($e->getMessage());
                            error_log(get_class($e));
                            $form->addError($t->_("ERROR_CREATING_A_NEW_DATABASE._MAKE_SURE_THE_DATABASE_DOES_NOT_ALREADY_EXIST_AND_THAT_THE_DATABASE_NAME_IS_VALID."));
                            $ok=false;
                        }
                    }
                    $db->getConnection();
                    $db->setFetchMode(Zend_Db::FETCH_OBJ);
                    $tables = $db->fetchOne('SHOW TABLES');
                    if($tables && strlen($tables) > 0) {
                        $form->addError($t->_("DATABASE_IS_NON_EMPTY."));
                        $ok = false;
                    } else {
                        $conn = $db->getConnection();
                        //Zend_Registry::set('db',$db);

                        if((int) $values['database_import'] == 0) {

                            $queries = $this->_splitSql($buffer);

                            foreach ($queries as $query)
                            {
                                if ($query != '' && $query {0} != '#') {
                                    $conn->query($query);
                                }
                            }
                            Zend_Db_Table::setDefaultAdapter($db);
                            $json_file = APPLICATION_PATH . '/modules/installation/acl/acl.json';
                            $a = new Installation_Model_AclDbSync($json_file);
                            $a->sync();
                        }

                        $ok = true;
                    }
                } catch (WebReport_Exception $e) {
                    $form->addError($t->_($e->getMessage()));
                    $ok=false;
                } catch (Exception $e) {
                    error_log($e->getMessage());
                    error_log(get_class($e));
                    $form->addError($t->_("INVALID_DATABASE_PARAMETERS._MAKE_SURE_THE_HOST_IS_CORRECT_AND_THAT_THE_USER_HAVE_CORRECT_DATABASE_PRIVILEGES._ALSO_MAKE_SURE_THE_DATABASE_EXISTS,_OTHERWISE_YOU_CAN_CHOOSE_TO_CREATE_ONE_IN_THE_FORM."));
                    $ok=false;
                    return;
                }

                if((int) $values['database_import'] == 1) {
                    try {
                        $import_connection=array(
                            "host" => $values['database_import_params_host'],
                            "port" => $values['database_import_params_port'],
                            "username" => $values['database_import_params_username'],
                            "password" => $values['database_import_params_password'],
                            "dbname" => $values['database_import_params_dbname']
                        );
                        $db_import = new Installation_Model_DatabaseImport($import_connection,$conn);
                        $db_import->import();
                        Zend_Db_Table::setDefaultAdapter($db);
                        $json_file = APPLICATION_PATH . '/modules/installation/acl/acl.json';
                        $a = new Installation_Model_AclDbSync($json_file);
                        $a->sync();
                        $this->_helper->redirector('finish');
                        $ok=false; //just to control flow;
                    } catch (WebReport_Exception $e) {
                        $form->addError($t->_($e->getMessage()));
                        $ok=false;
                    } catch (Exception $e) {
                        $form->addError($t->_("IMPORT_EXCEPTION: ") . $e->getMessage());
                        $ok=false;
                    }
                }
                if($ok==true) {
                    $this->_helper->redirector('user-and-company');
                } else {
                    if($assignedPassword) {
                        $c = mysql_connect("localhost", "root", $values['database_params_password']);

                        $pw = @mysql_escape_string($values['database_params_password']);
                        mysql_query("UPDATE mysql.user SET password=PASSWORD('') WHERE user = 'root';");
                        mysql_query("FLUSH PRIVILEGES;");
                    }
                }
            } else {
                $form->populate($_POST);
                if(strlen($password) > 0) {
                    $form->populate(array("database_params_password" => "hejsan"));
                }
            }
        }
    }

    public function userAndCompanyAction()
    {
        $this->view->step = 6;

        $form = new Installation_Form_CompanyAndUser();

        if ($this->getRequest()->isPost()) {
            set_time_limit(120);
            if ($form->isValid($_POST)) {
                $values = $form->getValues();

                $companiesDb = new Core_Model_Companies();
                $company = $companiesDb->createCompanyAndSave(array(
                    'name' => $values['name']
                ));

                /*
                 * TODO:
                 * This is done also in CompaniesController addAction()
                 * and very similar to Folder Creation in FoldersController addAction().
                 * Make it utilize same code!
                 */
                $folderName = $company->name . '-root';
                $dynamicSalt = CM_User::createSalt();
                $cameraGroupsDb = new Core_Model_CameraGroups();
                $cameraGroupsData = array(
                    'name' => $folderName,
                    'username' => $company->name . '-root-ccid',
                    'password' => md5(STATIC_SALT . 'defaulT_Pass0' . $dynamicSalt),
                    'password_salt' => $dynamicSalt,
                    'description' => '',
                    'email' => $values['email']
                );
                $cameraGroup = $cameraGroupsDb->createRow($cameraGroupsData);
                $cameraGroupId = $cameraGroup->save();

                $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
                $companyCameraGroupData = array(
                    "company_id" => $company->company_id,
                    "camera_group_id" => $cameraGroupId
                );
                $companyCameraGroup = $companyCameraGroupDb->createRow($companyCameraGroupData);
                $companyCameraGroup->save();

                $foldersDb = new Core_Model_Folders();
                $folder = $foldersDb->createRow(array(
                    'company_id' => $company->company_id,
                    'name' => $folderName,
                    'description' => '',
                    'parent_id' => NULL,
                    'in_sync' => FALSE,
                    'camera_group_id' => $cameraGroupId));

                $folder->save();

                $document = Core_Model_DocumentFactory::add($cameraGroupId, "camera_group", $folder);

                $usersDb = new Core_Model_Users();
                $dynamicSalt = CM_User::createSalt();

                //$aclRolesDb = new Core_Model_AclRoles();
                //$aclRole = $aclRolesDb->fetchRow($aclRolesDb->select()->order('acl_role_id')->limit(1));

                $userDataArray = array(
                    'forename' => $values['forename'],
                    'surname' => $values['surname'],
                    'company_id' => $company->company_id,
                    //'acl_role_id' => $aclRole->acl_role_id,
                    'email' => $values['email'],
                    'password' => md5(STATIC_SALT . $values['password'] . $dynamicSalt),
                    'password_salt' => $dynamicSalt,
                    'digest_helper_password' => md5($values['email'] . ':' . APPLICATION_REALM . ':' . $values['password']),
                    'created_date' => new Zend_Db_Expr('NOW()'),
                    'multi_company' => 1
                );


                $user = $usersDb->createRow($userDataArray);
                $user->save();

                $user->created_by = $user->user_id;
                $user->save();

                $db = Zend_Registry::get('db');
                $t = Zend_Registry::get('Zend_Translate');
                $conn = $db->getConnection();
                $writer = new Zend_Log_Writer_Stream(APPLICATION_PATH . "/../data/install_upgrade.log");
                $logger = new Zend_Log($writer);


                $u = new Installation_Model_DatabaseUpdate($conn, $t, $logger);
                $u->sync($logger,true);

                $json_file = APPLICATION_PATH . '/modules/installation/acl/acl.json';

                $a = new Installation_Model_AclDbSync($json_file);
                $a->sync($logger);

                $this->_helper->redirector('finish');

            } else {
                $form->populate($_POST);
            }
        }

        $this->view->form = $form;

    }




    public function finishAction()
    {
        $config = Zend_Registry::get('config');
        $config->general->installationCompleted = '1';
        $writer = new Zend_Config_Writer_IniNice();
        $writer->setConfig($config)
            ->setFilename(APPLICATION_PATH . '/config.ini');

        $writer->write();
        $this->view->step = 7;
    }

    public function _splitSql($sql)
    {
        $sql = trim($sql);
        $sql = preg_replace("/\n\#[^\n]*/", '', "\n".$sql);
        $buffer = array();
        $ret = array();
        $in_string = false;

        for ($i = 0; $i < strlen($sql) - 1; $i++) {
            if ($sql[$i] == ";" && !$in_string) {
                $ret[] = substr($sql, 0, $i);
                $sql = substr($sql, $i+1);
                $i = 0;
            }

            if ($in_string && ($sql[$i] == $in_string) && $buffer[1] != "\\") {
                $in_string = false;
            } elseif (!$in_string && ($sql[$i] == '"' || $sql[$i] == "'") && (!isset ($buffer[0]) || $buffer[0] != "\\")) {
                $in_string = $sql[$i];
            }

            if (isset ($buffer[1])) {
                $buffer[0] = $buffer[1];
            }

            $buffer[1] = $sql[$i];
        }

        if (!empty($sql)) {
            $ret[] = trim($sql);
        }

        return ($ret);
    }

    private function _translatableStrings()
    {
        $t = Zend_Registry::get('Zend_Translate');
        $this->view->translate('STEP_%1$s');
        $this->view->translate('CHECK_AGAIN');
        $this->view->translate('NEXT_STEP');
        $this->view->translate('INSTALLATION');
        $this->view->translate('HOST_NAME');
        $this->view->translate('USER_NAME');
        $this->view->translate('PASSWORD');
        $this->view->translate('E-MAIL');
        $this->view->translate('DATABASE_NAME');
        $this->view->translate('YOUR_COMPANY');
        $this->view->translate('FORENAME');
        $this->view->translate('SURNAME');
        $this->view->translate('CONFIRM_PASSWORD');
        $this->view->translate('ADMINISTRATIVE_USER');
        $this->view->translate('COMPANY_NAME');
        $this->view->translate('SYSTEM_REQUIREMENTS');
        $this->view->translate('THE_PASSWORD_IS_TOO_WEAK._INCLUDE_SOME_SYMBOLS_AND_NUMBERS_AND_TRY_AGAIN.');
        $this->view->translate('isEmpty'); // not empty validator message
        $this->view->translate('INSTALLATION_ID');
        $this->view->translate('REGISTER');
        $this->view->translate('ACTIVATION_CODE');
        $this->view->translate('WORKING..');
        $this->view->translate('IMPORT_USER_NAME');
        $this->view->translate('IMPORT_PASSWORD');
        $this->view->translate('IMPORT_DATABASE_NAME');
        $this->view->translate('TARGET_DATABASE_NOT_EMPTY.');
        $this->view->translate('DONE.');
        $this->view->translate('_NEW_RESOURCES');
        $this->view->translate('_NEW_ROLES');
        $this->view->translate('_NEW_RESOURCES');
        $this->view->translate('_NEW_GROUPS');
        $this->view->translate('_NEW_ASSEMBLIES');
        $this->view->translate('_NEW_ACTIONS');
        $this->view->translate('RUNNING_ACL_SYNC');
        $this->view->translate('NOTICE');
    }

}
