<?php

class Installation_Model_AclDbSync {
    private static $_action_extra = array('param1_key','param1_value','param2_key','param2_value', 'param3_key','param3_value');

    public static function generateJsonFromDb() {
        return Zend_Json::encode(self::generateArrayFromDb());
    }

    public static function generateArrayFromDb() {
        $ret = array();

        $resourcesDb = new Core_Model_AclResources();
        $select = $resourcesDb->select()->order("text ASC");
        $resources = $resourcesDb->fetchall($select);

        foreach($resources as $resource) {

            $cur = array(
               //"resourceName" => $resource->text,
               "module" => $resource->findParentCore_Model_Modules()->alias,
               "controller" => $resource->controller,
               "groups" => array()
            );

            $groupsDb = new Core_Model_AclPrivilegeGroups();
            $groups = $resource->findDependentRowset($groupsDb, null, $groupsDb->select()->order('text ASC'));

            foreach($groups as $group) {
                $cur_group = array(
                    "assembly" => array()
                );

                $assembliesDb = new Core_Model_AclPrivilegeGroupAssemblies();
                $assemblies = $group->findDependentRowset($assembliesDb, null, $assembliesDb->select()->order('text ASC'));
                foreach($assemblies as $assembly) {
                    $cur_assembly = array(
                        "optional" => $assembly->optional,
                        "value" => $assembly->value,
                        "action" => array(),
                        "acl_role" => array()
                    );

                   $actions = $assembly->findCore_Model_AclPrivilegeGroupAssemblyPrivilege();
                   foreach($actions as $action) {
                        $action_params = $action->findParentCore_Model_AclPrivileges();
                        $cur_action = array();

                        foreach(self::$_action_extra as $extra) {
                            if($action_params->$extra) {
                                $cur_action[$extra] = $action_params->$extra;
                            }
                        }
                        $cur_action_add = array("name" => $action_params->action);
                        if(count($cur_action))
                            $cur_action_add['extra'] = $cur_action;
                        $cur_assembly['action'][] = $cur_action_add;
                    }

                    $roles = $assembly->findCore_Model_AclRolePrivilegeGroupAssembly();
                    foreach($roles as $role) {
                        if($role->acl_role_id > 3) continue;
                        $cur_assembly['acl_role'][] = $role->findParentCore_Model_AclRoles()->name;
                    }
                    $cur_assembly['acl_role'] = array_unique($cur_assembly['acl_role']);


                    $cur_group["assembly"][$assembly->text] = $cur_assembly;
                }
                $cur['groups'][$group->text] = $cur_group;
            }


            $ret[$resource->text] = $cur;
        }
        return array('resources' => $ret);
    }

    private $_aclNew = null;

    public function __construct($json_file) {
        $f = fopen($json_file,'rb');
        $raw = fread($f,filesize($json_file));
        $this->_aclNew = Zend_Json::decode($raw);
    }

    public function sync(Zend_Log $logger=null) {

        if(is_null($logger)) {
            $logger = new Zend_Log();
            $logger->addWriter(new Zend_Log_Writer_Null());
        }
        
        $func = new Installation_Model_AclDbSyncFunctions();

        $current_all = $this->generateArrayFromDb();

        $current = $current_all['resources'];
        $new = $this->_aclNew['resources'];

        $new_acl_resources = array();
        $new_acl_groups = array();
        $new_acl_assemblies = array();
        $new_acl_actions = array();
        $new_acl_roles = array();

        foreach($new as $resourcekey => $resource) {
            $newresource = false;

            if(!array_key_exists($resourcekey, $current)) {
                $new_acl_resources[] = array(
                    'text' => $resourcekey,
                    'controller' => $resource['controller'],
                    'module' => $resource['module']);
                $newresource = true;
            } else {
                $func->_setResourceModule($resourcekey,$resource);
            }             


            foreach($resource['groups'] as $groupkey => $group) {
                $newgroup = false;

                if($newresource || !array_key_exists($groupkey,$current[$resourcekey]['groups'])) {
                    $new_acl_groups[] = array(
                        'resource' => $resourcekey,
                        'group' => $groupkey
                    );
                    $newgroup = true;
                }

                foreach($group['assembly'] as $assemblykey => $assembly) {
                    $newassembly = false;
                    if($newgroup || !array_key_exists($assemblykey, $current[$resourcekey]['groups'][$groupkey]['assembly'])) {
                        $new_acl_assemblies[] = array (
                                'resource' => $resourcekey,
                                'group' => $groupkey,
                                'assembly' => $assemblykey,
                                'optional' => array_key_exists('optional',$assembly) ? $assembly['optional'] : '1',
                                'value' => array_key_exists('value',$assembly) ? $assembly['value'] : '1'
                        );
                        $newassembly = true;
                    }

                    foreach($assembly['action'] as $actions_arr) {
                        $action = $actions_arr['name'];
                        if(isset($actions_arr['extra'])) {
                            $additional_keys = $actions_arr['extra'];
                        } else {
                            $additional_keys = null;
                        }

                        $new_keys = false;
                        $new_action = false;

                        $old_keys = null;

                        if(!$newassembly) {
                            $i=0;
                            for(;$i<count($current[$resourcekey]['groups'][$groupkey]['assembly'][$assemblykey]['action']);$i++) {
                                if($current[$resourcekey]['groups'][$groupkey]['assembly'][$assemblykey]['action'][$i]['name'] == $action)
                                    break;
                            }
                            if($i == count($current[$resourcekey]['groups'][$groupkey]['assembly'][$assemblykey]['action']))
                                $new_action = true;
                            elseif(isset($current[$resourcekey]['groups'][$groupkey]['assembly'][$assemblykey]['action'][$i]['extra']))
                                $old_keys = $current[$resourcekey]['groups'][$groupkey]['assembly'][$assemblykey]['action'][$i]['extra'];
                        }

                        if(is_null($additional_keys) && !is_null($old_keys)) {
                            $additional_keys = array();
                            $new_keys = true;
                        } elseif(!is_null($additional_keys) && is_null($old_keys)) {
                            $new_keys = true;
                        } elseif(!is_null($additional_keys) && !is_null($old_keys)) {
                            foreach(self::$_action_extra as $extra) {
                                if(isset($old_keys[$extra]) && isset($additional_keys[$extra]) && $old_keys[$extra] != $additional_keys[$extra]) {
                                    $new_keys = true;
                                    break;
                                }
                                if(isset($old_keys[$extra]) && !isset($additional_keys[$extra])) {
                                    $new_keys = true;
                                    break;
                                }
                                if(!isset($old_keys[$extra]) && isset($additional_keys[$extra])) {
                                    $new_keys = true;
                                    break;
                                }
                            }
                        }

                        if( $newassembly || $new_action || $new_keys ) {
                            $new_acl_actions[] = array (
                                'resource' => $resourcekey,
                                'group' => $groupkey,
                                'assembly' => $assemblykey,
                                'action' => $action,
                                'additional_keys' => $additional_keys
                            );
                        }
                    }

                    foreach($assembly['acl_role'] as $acl_role) {
                        if($newassembly || !in_array($acl_role, $current[$resourcekey]['groups'][$groupkey]['assembly'][$assemblykey]['acl_role'])) {
                            foreach($new_acl_roles as $new_acl_role) {
                                if( $new_acl_role['resource'] == $resourcekey && $new_acl_role['group'] == $groupkey
                                    && $new_acl_role['assembly'] == $assembly && $new_acl_role['acl_role'] == $acl_role) {
                                    continue;
                                    }
                            }
                            $new_acl_roles[] = array (
                                'resource' => $resourcekey,
                                'group' => $groupkey,
                                'assembly' => $assemblykey,
                                'acl_role' => $acl_role
                            );
                        }
                    }
                }

            }
        }

        //the diff is now sorted. just insert the new stuff
        $logger->notice(count($new_acl_resources) . "_NEW_RESOURCES");
        foreach($new_acl_resources as $new_acl_resource) {
            try {
                $func->_addResource($new_acl_resource);
            } catch(WebReport_Exception $e) {
                $logger->warn('FAILED_TO_ADD_RESOURCE:' . $e->getMessage());
                continue;
            } catch(Exception $e) {
                $logger->err('ERROR: ' . $e->getMessage());
                return;
            }
            $logger->notice("NEW_RESOURCE_" .$new_acl_resource['text']);
        }

        $logger->notice(count($new_acl_groups) . "_NEW_GROUPS");
        foreach($new_acl_groups as $new_acl_group) {
            try {
                $func->_addGroup($new_acl_group);
            } catch(WebReport_Exception $e) {
                $logger->warn('FAILED_TO_ADD_GROUP:' . $e->getMessage());
                continue;
            } catch(Exception $e) {
                $logger->err('ERROR: ' . $e->getMessage());
                return;
            }
            $logger->notice("NEW_GROUP_" . $new_acl_group['group']);
        }

        $logger->notice(count($new_acl_assemblies) . "_NEW_ASSEMBLIES");
        foreach($new_acl_assemblies as $new_acl_assembly) {
            try {
                $func->_addAssembly($new_acl_assembly);
            } catch(WebReport_Exception $e) {
                $logger->warn('FAILED_TO_ADD_ASSEMBLY:' . $e->getMessage());
                continue;
            } catch(Exception $e) {
                $logger->err('ERROR: ' . $e->getMessage());
                return;
            }
            $logger->notice("NEW_ASSEMBLY_" . $new_acl_assembly['assembly']);
        }

        $logger->notice(count($new_acl_actions) . "_NEW_ACTIONS");
        foreach($new_acl_actions as $new_acl_action) {
            try {
                $func->_addAction($new_acl_action,self::$_action_extra);
            } catch(WebReport_Exception $e) {
                $logger->warn('FAILED_TO_ADD_ACTION:' . $e->getMessage());
                continue;
            } catch(Exception $e) {
                $logger->err('ERROR: ' . $e->getMessage());
                return;
            }
            $logger->notice("NEW_ACTION_" . $new_acl_action['action']);
        }

        $logger->notice(count($new_acl_roles) . "_NEW_ROLES");
        $compDb = new Core_Model_Companies();
        $companies = $compDb->fetchAll();
        foreach($new_acl_roles as $new_acl_role) {
            foreach($companies as $company){
                try {
                    $func->_addRole($new_acl_role,$company->company_id);
                } catch(WebReport_Exception $e) {
                    $logger->warn('FAILED_TO_ADD_ROLE:' . $e->getMessage());
                    continue;
                } catch(Exception $e) {
                    $logger->err('ERROR: ' . $e->getMessage());
                    return;
                }
            }
            $logger->notice("NEW_ROLE_" . $new_acl_role['acl_role'] . '_FOR_' . $new_acl_role['assembly']);
        }

        $logger->notice("DONE.");
        // @todo sync companies default acl roles

    }
}


