<?php

class Installation_Model_DatabaseImport {

    private $_foreignConnection = null;
    private $_connRemote = null;
    private $_connLocal = null;
    private $_connectionDetails = array();
    private $_logger = null;
    private $_translator = null;

    public function __construct($connection_details, $local=null, $logger=null) {
        $this->_connectionDetails = $connection_details;
        $this->_connect();
        $this->_connLocal = $local;

        if(is_null($logger)) {
            $logger = new Zend_Log();
            $logger->addWriter(new Zend_Log_Writer_Stream(APPLICATION_PATH . '/../data/session/database_log'));
        }

        $this->_logger = $logger;
        $this->_translator = Zend_Registry::get('Zend_Translate');
    }

    private function _connect() {
        $this->_foreignConnection = Zend_Db::factory('Pdo_Mysql',$this->_connectionDetails);
        $this->_foreignConnection->getConnection();
        $this->_connRemote = $this->_foreignConnection->getConnection();
    }

    public function testConnection() {
        try {
            $this->_connRemote->query("SELECT 1 as TEST;");
        } catch (Exception $e) {
            throw new WebReport_Exception("CONNECTION_TEST_FAILED.");
        }
    }

    public function validateTargetDatabase() {
        $res=$this->_connLocal->query('SHOW TABLES')->fetchAll();
        if(count($res)>0) {
            throw new WebReport_Exception("TARGET_DATABASE_NOT_EMPTY.");
        }
    }

    public function validateSourceDatabase() {
        // some assertions that this really is a webreport database
        // FIXME: Incomplete test
        $base_tables = array('user','company','acl_privilege','camera','company_camera_group','count_data_table');
        foreach($base_tables as $table) {
            $res=$this->_connRemote->query("SHOW TABLES LIKE  '" . $table . "'")->fetchAll();
            if(count($res) != 1)
                throw new WebReport_Exception("DOES_NOT_LOOK_LIKE_A_DATAMANAGER_DATABASE.");
        }
        return true;
    }

    public function importDatabase() {
        set_time_limit(300);

        $this->_connRemote->exec("USE " . $this->_connectionDetails['dbname']);
        $tables = $this->_connRemote->query("SHOW TABLES")->fetchAll();
        $create_array = array();
        foreach($tables as $table) {
            $r = $this->_connRemote->query("SHOW CREATE TABLE `" . $table[0] . "`")->fetchAll();
            if(count($r)!=1) {
                throw new WebReport_Exception("CREATE_TABLE_DUMP_YIELDED_MULTIPLE_TABLES");
            }
            $create_array[$table[0]] = $r[0][1];
            unset($r);
        }
        unset($tables);

        $this->_logger->notice("STRUCTURE DUMP COMPLETE, CREATING TABLES");
        $this->_connLocal->exec("SET @OLD_FOREIGN_KEY_CHECK=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0");

        foreach($create_array as $table => $create_stmt) {
            $this->_connLocal->exec($create_stmt);
            $this->_connLocal->exec("ALTER TABLE `" . $table .  "` AUTO_INCREMENT = 1");
            $columns = $this->_connRemote->query("DESCRIBE `$table`")->fetchAll();
            $this->_logger->notice("CREATING_TABLE " . $table);
            $colarray = array();
            foreach($columns as $column) {
                $colarray[] = $column[0];
            }
            unset($columns);

            $chunksize = 400;
            $offset = 0;
            $loop = true;
            while($loop){
                $rows = $this->_connRemote->query("SELECT * FROM `" . $table . "` LIMIT $offset, $chunksize");
                $offset +=$chunksize;
                $n_insert = 0;
                $insert_values = array();
                $loop = false;
                while($row = $rows->fetch()) {
                    $valarray = array();
                    foreach ($colarray as $col){
                        $valarray[] = $this->_connLocal->quote($row[$col]);
                    }
                    $n_insert++;

                    $insert_values[] = "(". implode(",", $valarray) . ")";

                    if($n_insert >= 500) {
                        $insert_stmt = "INSERT INTO `" . $table . "` (`". implode("`,`",$colarray) . "`) VALUES " . implode(",",$insert_values);
                        $this->_connLocal->exec($insert_stmt);

                        unset($insert_stmt);
                        $insert_values = array();
                        $n_insert = 0;
                        $loop = true;
                    }
                    unset($row);
                }
                if($n_insert>0) {
                    $insert_stmt = "INSERT INTO `" . $table . "` (`". implode("`,`",$colarray) . "`) VALUES " . implode(",",$insert_values);
                    $this->_connLocal->exec($insert_stmt);
                    unset($insert_stmt);
                    $loop = true;
                }
                unset($rows);
            }
        }
        $this->_connLocal->exec("SET FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS");

        //Zend_Debug::dump($create_array);
        return true;
    }

    public function importSucceded() {

        return false;
    }

    public function upgradeDatabase() {
        $logger = $this->_logger;

        $u = new Installation_Model_DatabaseUpdate($this->_connLocal, $this->_translator, $logger);
        $u->sync($logger,true);

        $logger->notice("RUNNING_ACL_SYNC");
        $json_file = APPLICATION_PATH . '/modules/installation/acl/acl.json';

        $a = new Installation_Model_AclDbSync($json_file);
        $a->sync($logger);

        return $u->test($logger);
    }


    public function import() {
        $logger = $this->_logger;
        $logger->notice("TESTING_CONNECTION");
        $this->testConnection();

        $logger->notice("VALIDATING_TARGET_DATABASE");
        $this->validateTargetDatabase();

        $logger->notice("VALIDATING_SOURCE_DATABASE");
        $this->validateSourceDatabase();

        $logger->notice("IMPORTING_DATABASE");
        $this->importDatabase();

        $logger->notice("TESTING_DATABASE");
        $this->importSucceded();

        $logger->notice("UPGRADING_DATABASE");
        $this->upgradeDatabase();

        //throw new WebReport_Exception("EVERTHING_PASSED");
    }
}

