<?php

class NoTranslator {
    public function _($a) {
        return $a;
    }
}
class Installation_Model_DatabaseUpdate {

    private $_update_directory = null;
    private $_db_update = array();
    private $_conn = null;
    private $_t = null;


    public function __construct($conn, $translator, $logger = null, $scan_files = true) {
        $this->_update_directory = APPLICATION_PATH . '/modules/installation/models/DatabaseUpdateEntry';
        $this->_t = $translator ? $translator:  new NoTranslator();

        $this->_conn = $conn;

        if(is_null($logger)) {
            $logger = new Zend_Log();
            $logger->addWriter(new Zend_Log_Writer_Null());
        }
        $this->_logger = $logger;
        if ( $scan_files ) {
            //make list of all db_update
            $dir = new DirectoryIterator($this->_update_directory);
            $subdirs = array();

            foreach($dir as $fileinfo) {
                if($fileinfo == "." || $fileinfo == "..")
                    continue;

                if(substr($fileinfo,0,2) == "DM"){
                    $subdirs[] = $fileinfo->__toString();
                } else {
                    $logger->warn("Skipping file/folder " . $fileinfo . " since it does not have the 'DM' prefix.");
                }

            }

            sort($subdirs);
            $this->clearUpdates();
            foreach($subdirs as $subdir){
                $update_classes = array();
                foreach(new DirectoryIterator($this->_update_directory . "/" . $subdir . "/" ) as $file){
                    if($file == "." || $file == ".." || $file == "info.txt")
                        continue;
                    if ( substr($file, -4)  == ".php" ) {
                        $update_classes[] = "Installation_Model_DatabaseUpdateEntry_". $subdir. "_" . substr($file,0,-4);
                    }
                };
                sort($update_classes);

                foreach($update_classes as $class) {
                    try {
                        if(class_exists($class)) {
                            //$this->_db_update[] = new $class($conn,$translator,$logger);
                            $this->addUpdate($class);
                        } else {
                            $logger->notice("Failed to find update '$class'. This is a fatal error, aborting upgrade.");
                            throw new Exception("Class $class not found.");
                        }
                    } catch (Exception $e) {
                        throw $e;
                    }
                }
            }
        }
    }

    public function clearUpdates() {
        $this->_db_update = array();
    }

    public function addUpdate($className) {
        $this->_db_update[] = (new $className($this->_conn,$this->_t, $this->_logger));
    }

    private function _getLatestUpdateId() {
        $q = $this->_conn->query("SHOW TABLES LIKE 'database_version'")->fetchAll();

        if(count($q) != 1) {
            return -1;
        }

        $q = $this->_conn->query("SELECT * FROM database_version ORDER BY database_version_id DESC LIMIT 1")->fetchAll();
        if(count($q) == 0) {
            return -1;
        }
        return $q[0]['applied_patch_number'];
    }

    public function test(Zend_Log $logger = null) {

        if(is_null($logger)) {
            $logger = new Zend_Log();
            $logger->addWriter(new Zend_Log_Writer_Null());
        }

        $all_good = true;
        foreach($this->_db_update as $update) {
            try {
                if($update->testApplied())
                        $logger->warn( $update->description(). "-" .$this->_t->_("ALREADY_APPLIED"));
                    else {
                        $logger->warn( $update->description(). "-" .$this->_t->_("NOT_APPLIED"));
                        $all_good = false;
                }
            } catch (WebReport_Exception $e) {
                    $logger->err($update->description()."-".$this->_t->_("APPLY_TEST_FAILED"));
                    $logger->err($e->getMessage());
            }
        }
        return $all_good;
    }


    public function sync(Zend_Log $logger = null, $ok_to_fail = false, $start_id = false) {
        if(is_null($logger)) {
            $logger = new Zend_Log();
            $logger->addWriter(new Zend_Log_Writer_Null());
        }

        $latest_db_update = $this->_getLatestUpdateId();
        if ( $start_id !== FALSE ) $latest_db_update = $start_id;
        if($latest_db_update != -1)
            $logger->notice("Starting from patch #" . $latest_db_update);

        $ok_to_fail_until_update_id = 0;

        if($ok_to_fail) {
            $ok_to_fail_until_update_id = 9999;
        }

        if($latest_db_update == -1 && !$ok_to_fail) {
            // since we do not now what the database version is.. its ok to fail an update
            // until we get to 2.2.0 ie update id 12
            $ok_to_fail = true;
            $ok_to_fail_until_update_id = 10;
        }

        foreach($this->_db_update as $update) {
            set_time_limit(0);
            if($update->getId() < $latest_db_update){
                continue;
            }

            if($update->getId() > $ok_to_fail_until_update_id) {
                $ok_to_fail = false;
            } else {
                try {
                    if($ok_to_fail && $update->testApplied() && !$update->alwaysRun()) {
                        $logger->warn( $update->description(). " - Patch was already applied");
                        continue;
                    }
                } catch (WebReport_Exception $e) {
                    $logger->err($update->description()." - Patch was already applied");
                    $logger->err($e->getMessage());
                }
            }
            try {
                $logger->warn( $update->description(). " - Starting patch process.");
                $update->apply();
            } catch (WebReport_Exception $e) {
                if($ok_to_fail) {
                    $logger->warn( $update->description(). " - Apply failed, but fail was allowed - " . $e->getMessage());
                    continue;
                } else {
                    $logger->err( $update->description(). " - Apply failed - " . $e->getMessage());
                    break;
                }
            } catch(Exception $e) {
                 $logger->err($update->description()." - Error while applying");
                 $logger->err($e->getMessage());
                 break;
            }
            $logger->info($update->description(). " - Done!");
        }

    }
}
