<?php
include_once 'DatabaseUpdateEntryAbstract.php';

abstract class Installation_Model_DatabaseUpdateEntryAbstract {

    protected $_id;
    protected $_conn;
    protected $_t;
    protected $_alwaysRun = false;
    protected $_logger = null;
    protected $_prepared = false;

    abstract protected function _preAssert();
    abstract protected function _postAssert();
    abstract protected function _update();
    abstract protected function _description();
    
    public function __construct($conn, $translator, $logger=null) {
        $this->_conn = $conn;
        $this->_t = $translator;
        $this->_id = substr(get_class($this),-4);

        if(!is_null($logger)) {
            $this->_logger = $logger;    
        }
    }

    public function testApplied() {
        try {
            if ( !$this->_prepared ) {
                $this->_prepared = true;
                $this->_prepare();
            }
            return $this->_postAssert();
        } catch(Exception $e) {
            throw new WebReport_Exception("TEST_FAILED " . $e->getMessage());
        }
    }

    public function apply() {
        if ( !$this->_prepared ) {
            $this->_prepared = true;
            $this->_prepare();
        }

        if($this->_preAssert() == true) {
            if($this->_update() == false)
                throw new WebReport_Exception('UPDATE_FAILED.');
            if($this->_postAssert() == false)
                throw new Exception('POST_UPDATE_ASSERTION_FAILED.');
        }

        else if($this->_postAssert() == false)
                throw new WebReport_Exception('PRE_AND_POST_UPDATE_ASSERTION_FAILED.');
    }

    public function getId() {
        return (int) $this->_id;
    }

    public function description() {
        return "Patch #" . $this->_id . ": " . $this->_description();
    }

    public function alwaysRun() {
        return $this->_alwaysRun;
    }

    public function foreignKeyExists($table, $name){
        $q = $this->_conn->query("SHOW CREATE TABLE `" . $table . "`")->fetchAll();
        $pos =  strpos($q[0]["Create Table"],"CONSTRAINT `" . $name . "`");
        return $pos !== FALSE;
    }

    public function getTables($table,$tablesQuery){
        if(is_null($tablesQuery)) return array($table);
        $all = array();
        $q = $this->_conn->query($tablesQuery)->fetchAll();
        foreach( $q as $row){
            $all[] = $row[0];
        }
        return $all;
    }

    public function _prepare() {

    }
}
