<?php
class Pos_IndexController extends WebReport_Controller_Action
{

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = $this->_group ?$this->_group->group_id : null;


        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse", 'folder' => $groupId), "text" => _T('POS_IMPORTS'));

        if($this->_group && $this->_group->canEdit($this->_user))
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", 'folder' => $groupId), "text" => _T('ADD_IMPORT'));

        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

        $this->view->headTitle("POS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function addAction($action = "add", $job = null)
    {
        if($action == 'add')
            $title = $this->_translate->_('ADD');
        else
            $title = $this->_translate->_('EDIT');
        $title .= ' ' . $this->_translate->_('POS_IMPORT_JOB');

        if (!$this->getRequest()->isPost()) {
            Zend_Session::namespaceUnset('Pos');
            $this->view->title = $title . ": " . sprintf($this->_translate->_('STEP_%1$s'), 1);
            $form = new Pos_Form_AddPosDataStep1();
            if($action == "edit") {
                $form->populate($job->toArray());
                $form->populate(array("file" => $job->file_url, "enc" => $job->encoding, "header_enabled" => $job->use_header));
            }
            $this->view->form = $form;
            $this->render("add-step1");
        } else {
            $session = new Zend_Session_Namespace('Pos');
            if ($_POST['step'] == 1) {
                $form = new Pos_Form_AddPosDataStep1($action);
                if ($form->isValid($_POST)) {
                    $oldTimeLimit = (int) ini_get('max_execution_time');
                    set_time_limit(600);
                    $values = $form->getValues();

                    if(strlen($values['csv_file'])) {
                        $upload = new Zend_File_Transfer_Adapter_Http();
                        $file = $upload->getFileName();
                        $tmp  = APPLICATION_PATH . "/../public/" . Zend_Registry::get('config')->general->publicTempFolder . "/";
                        $path = $tmp . uniqid('pos_');
                        error_log($path);

                        mkdir($path);
                        copy($file, $path.'/'.$values['csv_file']);

                        $session->uploaded = TRUE;
                    } else {
                        $path = $values['file'];
                        $session->uploaded = FALSE;
                    }
                    $pos = new Pos_Model_PosPath(NULL, $path, $encoding=$_POST['enc'], $heading=($_POST['header_enabled'] == 1));
                    if(!$pos->tryReadStructure() || $pos->tryUpdateSyncFile(FALSE) || !$pos->tryVerifyHeaders()){
                        $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
                        $form->populate($_POST);
                        $this->view->form = $form;
                        $this->view->form->getElement('file')->addError($pos->getLatestTranslatedError());
                        $this->render("add-step1");
                        set_time_limit($oldTimeLimit);
                        return;
                    }
                    set_time_limit($oldTimeLimit);
                    $session->separator = $pos->getSeparator();
                    $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

                    $safeHeaders = array();
                    foreach ($pos->getHeaders() as $header){
                        $safe = preg_replace('/\s/','_' , preg_replace('/\r\n|\r|\n/', '', $header));
                        $safeHeaders[$safe] = $header;
                    }
                    $this->view->safeHeaders = $safeHeaders;
                    $session->safeHeaders = $safeHeaders;

                    $form = new Pos_Form_AddPosDataStep2($action, $data=$pos->csv->data, $safeheaders=$safeHeaders);

                    if($pos->offset && $_POST['header_enabled'] == 1) {
                        $this->view->warning = $this->_translate->_("THE_FIRST_LINE_WAS_POORLY_FORMATTED_AND_HAD_TO_BE_SKIPPED._THESE_MAY_NOT_BE_YOUR_REAL_HEADERS_BUT_THAT_DOES_NOT_MATTER._CHOOSE_THE_CORRECT COLUMNS_AND_IT_WILL_ALL_WORK_OUT.");
                    }
                    $this->view->title = $title . ": " . sprintf($this->_translate->_('STEP_%1$s'), 2);
                    $this->view->arr_fields = $pos->getFields();
                    $this->view->arr_headers = $pos->getHeaders();

                    $this->view->arr_examples =$pos->getExamples();
                    foreach($_POST as $k => $v){
                        $session->$k = $v;
                    }
                    $session->arr_fields = $this->view->arr_fields;
                    $session->arr_examples = $this->view->arr_examples;
                    $session->arr_headers = $this->view->arr_headers;
                    $session->pos_csv_data = $pos->csv->data;
                    $session->file = $path;
                    $session->header_enabled = $_POST["header_enabled"];
                    $this->view->pos = $pos;
                    $this->view->form = $form;
                    $this->view->mapping = array();
                    if($action == 'edit') {
                        $form->populate($job->toArray());
                        $mapping = array();
                        foreach($job->getMapping() as $header => $value) {
                            $safe = preg_replace('/\s/','_' , preg_replace('/\r\n|\r|\n/', '', $value));
                            $mapping[$header] = $safe;
                        }
                        $this->view->mapping = $mapping;
                        if($job->store_filter != null)
                            $this->view->store_filter = array($mapping['storeid'] => $job->store_filter);
                    }

                    $this->render("add-step2");
                } else {
                    $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
                    $form->populate($_POST);
                    $this->view->form = $form;
                    $this->render("add-step1");
                }
            } else if ($_POST['step'] == 2) { // step 2
                $form = new Pos_Form_AddPosDataStep2($action, $data=$session->pos_csv_data, $safeHeaders=$session->safeHeaders);
                $this->view->safeHeaders = $session->safeHeaders;
                $this->view->arr_examples = $session->arr_examples;
                $this->view->arr_headers = $session->arr_headers;
                $values = $_POST;
                if(isset($values['mapping'])) {
                    $safe_array = $values['mapping'];
                    $map_array = array();
                    foreach($values['mapping'] as $header => $value)
                        $map_array[$safeHeaders[$header]] = $value;
                } else {
                    $map_array = $session->mapping;
                }
                $is_map = (current($map_array) == '' && count(array_unique($map_array)) == 1) ? false : true;
                $session->store_filter = $_POST["store_id"][array_search('storeid', $safe_array)];

                $jobs = new Pos_Model_ImportJobs();

                $format_check = $jobs->testDateTimeFormat($map_array, $_POST["date_format"],
                    $_POST["time_format"], (array)$session->arr_headers, 
                    (array)$session->arr_examples);
                if($format_check === false) {
                    $form->addError($this->_translate->_("COULD_NOT_FIND_TIME_FIELDS._DATETIME_OR_DATE_AND_TIME_NEEDED."));
                    $this->view->store_filter = $_POST["store_id"];
                } else if (count($format_check)) {
                    foreach($format_check as $k=>$v) {
                        $form->getElement($k)->addError($this->_translate->_("BAD_FORMAT"));
                    }
                    $format_check = false;
                    $this->view->store_filter = $_POST["store_id"];
                } else {
                    //Check if first line is header or data by checking if date/time format fits
                    /*$format_check = $jobs->testDateTimeFormat($safe_array, $_POST["date_format"],
                        $_POST["time_format"], (array)$session->arr_headers, 
                        (array)$session->arr_headers);
                    if(count($format_check))
                        $session->header_enabled = 1;
                    else {
                        $session->header_enabled = 0;
                        $new_mapping = array();
                        foreach($map_array as $k=>$v) {
                            $ind = array_search($k, (array)$session->arr_headers);
                            $new_mapping[$ind] = $v;
                        }
                        $map_array = $new_mapping;
                        $safe_array = $new_mapping;
                    }*/
                }
                if ($is_map && $format_check !== false) {
                    $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
                    foreach($_POST as $k => $v){
                        $session->$k = $v;
                    }
                    $session->mapping = $map_array;
                    $form = new Pos_Form_AddPosDataStep3($action, null, $session->uploaded);
                    $this->view->title = $title . ": " . sprintf($this->_translate->_('STEP_%1$s'), 3);
                    if($action == 'edit') {
                        $pop["camera_id"] = $job->getCameras();
                        $form->populate($pop);
                    }
                    $this->view->form = $form;
                    $this->render("add-step3");
                } else {
                    $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
                    $form->populate($_POST);
                    $this->view->form = $form;
                    $this->view->arr_fields = $session->arr_fields;
                    $this->view->arr_headers = $session->arr_headers;
                    $this->view->arr_examples = $session->arr_examples;
                    $this->view->mapping = array_flip($safe_array);
                    $this->render("add-step2");
                }
            } else if ($_POST['step'] == 3) { // step 3
                $form = new Pos_Form_AddPosDataStep3($action);
                if ($form->isValid($_POST)) {   //final submit
                    $values = $form->getValues();

                    if ($action == 'add') {
                        $pos = new Pos_Model_ImportJobs();
                        $job = $pos->createRowFromNamespace($session);
                        $session->_job_id = $job->pos_data_id;
                        Core_Model_DocumentFactory::add($job->pos_data_id, "importjob", $this->_group);
                    } else {
                        $job->name = $session->name;
                        $job->file_url = $session->file;
                        $job->schedule = $session->schedule;
                        $job->separator = $session->separator;
                        $job->use_header = $session->header_enabled;
                        $job->mapping = json_encode(array_flip($session->mapping));
                        $job->date_format = $session->date_format;
                        $job->time_format = $session->time_format;
                        $job->country = $session->country;
                        $job->currency = $session->currency;
                        $job->company_id = $session->company_id;
                        $job->store_filter = $session->store_filter;
                        $job->save();
                    }
                    $job->clearCameras();
                    if(isset($values['camera_id'])){
                        $job->addCamerasById($values['camera_id']);
                    }

                    if(isset($values['virtual_camera_id'])){
                    }
                    if(isset($values['count_data_formula_set_id'])){
                    }
                    if(isset($values['count_data_filter_group_id'])){
                    }
                    #$pos = new Pos_Model_ImportJobs();
                    #$pos->importCsv3($values);
                    #$session->values3 = $form->getValues();

                    if($session->uploaded) {
                        $this->_helper->redirector('import', 'index', 'pos', array('id' => $job->pos_data_id, 'folder' => $this->_group->group_id));
                    }

                    $this->_helper->redirector('browse','index','pos', array('folder' => $this->_group->group_id));
                } else {
                    $this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 3), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
                    $form->populate($_POST);
                    $this->view->form = $form;
                    $this->render("add-step3");
                }
            }
        }
    }

    public function browseAction()
    {
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
            $this->_user, $this->_group, $this->_helper, $this->_request,
            'Pos_Model_ImportJobs',
            'Pos_Model_PosListGenerator');
        #$this->view->title = $this->_translate->_('BROWSE_POS_DATA');
        #$identity = Zend_Auth::getInstance()->getIdentity();
        #$jobDb = new Pos_Model_ImportJobs();
        #$jobs = $jobDb->fetchAllJobs($identity->multi_company ? null : $identity->company_id);
        #$this->view->list = Pos_Model_PosListGenerator::CreateView($jobs);
        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function editAction(){
        $posId = (int)$this->_getParam("id");
        $pos = new Pos_Model_ImportJobs();
        $job = $pos->find($posId)->current();
        $document = Core_Model_DocumentFactory::fetch($posId, "importjob");
        if (!$job || !$document->canEdit($this->_user))
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_POS_DATA_SET_WITH_SUCH_ID_EXISTS.'));
        $this->addAction("edit", $job);
    }

    public function importAction()
    {
        set_time_limit(600);

        $posId = (int)$this->_getParam("id");
        if (!$posId)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_POS_DATA_SET_WITH_SUCH_ID_EXISTS.'));
        #error_log("Dont forget to check privileges here. grep id: hj3k12j3");
        $pos = new Pos_Model_ImportJobs();
        $set = $pos->find($posId)->current();
        $document = Core_Model_DocumentFactory::fetch($posId, "importjob");
        if (!$set || !$document->canEdit($this->_user))
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_POS_DATA_SET_WITH_SUCH_ID_EXISTS.'));

        try {
            $rowsCount = $set->run();
        } catch(WebReport_Exception $e) {
            $this->_flashMessenger->addMessage($e->getMessage());
            $this->_helper->redirector('browse', 'index', 'pos', array('folder' => $this->_group->group_id));
            return;
        }

        #$this->view->rows_count = $rowsCount;
        $this->_flashMessenger->addMessage($this->_translate->_('IMPORT_FINISHED_SUCCESSFULLY.'));
        $this->_helper->redirector('browse','index','pos', array('folder' => $this->_group->group_id));
    }

    public function viewAction()
    {
        $posId = (int)$this->_getParam("id");
        $document = Core_Model_DocumentFactory::fetch($posId, "importjob");
        if(!$document || !$document->canView($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('THIS_IMPORT_DOES_NOT_EXIST,_OR_IS_NOT_VISIBLE_TO_YOU.'));
            $this->_helper->redirector('browse','index','pos', array('folder' => $this->_group->group_id));
            return;
        } 

        $pos = new Pos_Model_ImportJobs();
        $set = $pos->find($posId)->current();
        if (!$set)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_POS_DATA_SET_WITH_SUCH_ID_EXISTS.'));
        $this->view->name = $set->name;
        $this->view->company_id = $set->company_id;

        $usersDb = new Core_Model_Companies();
        $company = $usersDb->find($set->company_id)->current();
        $this->view->company = $company->name;

        $usersDb = new Core_Model_Users();
        $user = $usersDb->find($set->created_by)->current();
        $this->view->created_by = $user->forename . ' ' . $user->surname;
        $this->view->created_date = $set->created_date;
        $this->view->currency = $set->currency ? $set->currency : _T("NONE");

        $this->view->schedule = $set->schedule;
        if ($set->schedule == 24*60*60)
            $this->view->schedule = _T("EVERY_DAY");
        else if ($set->schedule == 01*60*60)
            $this->view->schedule = _T("EVERY_HOUR");
        else if ($set->schedule == -1)
            $this->view->schedule = _T("NEVER");

        $this->view->file_url = $set->file_url;
        $this->view->store_filter = $set->store_filter;

        $countries = new Talking_Countries();
        $country = $set->country;
        if ($country) {
            foreach ($countries->getContinentCountries() as $countries) {
                if (array_key_exists($country , $countries)) {
                    $country = $countries[$country];
                }
            }
        }
        $this->view->country = $country ? $country : '-';

        $encodings = $pos->getEncodings();
        $this->view->encoding = $encodings[$set->encoding];

        $this->view->cameras = $set->findManyToManyRowset('Core_Model_Cameras', 'Pos_Model_SetCamera');
        $this->view->virtualCameras = $set->findManyToManyRowset('Core_Model_VirtualCameras', 'Pos_Model_SetCamera');
        $this->view->formulaSets = $set->findManyToManyRowset('Core_Model_CountData_FormulaSets', 'Pos_Model_SetCamera');
        $this->view->filterGroups = $set->findManyToManyRowset('Core_Model_CountData_FilterGroups', 'Pos_Model_SetCamera');

    }

    public function deleteAction()
    {
        $posId = (int)$this->_getParam("id");

        $pos = new Pos_Model_ImportJobs();
        $set = $pos->find($posId)->current();
        $identity = Zend_Auth::getInstance()->getIdentity();
        $document = Core_Model_DocumentFactory::fetch($posId, "importjob");


        if (!$set || !$document->canEdit($this->_user))
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_POS_DATA_SET_WITH_SUCH_ID_EXISTS.'));

        //delete Pos Camera Set
        $camSet = new Pos_Model_PosCameraSet();
        $camSet = $camSet->delete(array('pos_data_id = ?' => $posId));

        //delete Pos_data_pos1
        $db = Zend_Registry::get('db');

        $tableName = "pos_data_pos1";
        $select = $db->select()
            ->from("information_schema.columns", array("COLUMN_NAME"))
            ->where("table_name = ?", $tableName)
            ->where('column_name LIKE ?', 'p' . $posId . '\_%');

        foreach ($db->fetchAll($select) as $s) {
            $query = "ALTER TABLE " . $tableName . " DROP COLUMN " . $s->COLUMN_NAME;
            $db->query($query);
        }

        $set->delete();
        Core_Model_DocumentFactory::remove($posId, 'importjob');
        $this->_flashMessenger->addMessage($this->_translate->_('IMPORT_DELETED.'));
        $this->_helper->redirector('browse','index','pos', array('folder' => $this->_group->group_id));
    }

/*
    public function indexAction()
    {
        $camerasDb = new Core_Model_Cameras();
        $countDataEventsDb = new Core_Model_CountData_Events();

        $db = Zend_Registry::get('db');
        $select = $db->select()->from(array("c" => $camerasDb->info('name')), array("name"))
            ->join(array("cde" => $countDataEventsDb->info('name')), "c.camera_id = cde.camera_id", array("cnttime", "count", "cnttype"))
            ->order('cde.cnttime DESC');

        $page = $this->_getParam('page');

        if (is_null($page)) $page = 1;
        else $page = (int)$page;

        $messagePaginator = new Zend_Paginator(new Zend_Paginator_Adapter_DbSelect($select));
        $messagePaginator->setCurrentPageNumber($this->_getParam('page'));
        $messagePaginator->setItemCountPerPage(50);

        $this->view->messagePaginator = $messagePaginator;
    }

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
 */
    private function _translatableStrings() {
        $this->_translate->_('MOD_POS_NAME');

        $this->_translate->_("MOD_POS_MENU_CONFIG");

        $this->_translate->_("POINT_OF_SALE");
        $this->_translate->_("VIEW");
        $this->_translate->_("EDIT");

        $this->_translate->_('ADD_POS');
        $this->_translate->_('BROWSE_POS');

        $this->_translate->_('SET_NAME');
        $this->_translate->_('CAMERAS');
    }

}
