<?php

class Pos_Form_AddPosDataStep1 extends UniForm_Form
{

    protected $_action;

    /**
     * Constructor
     *
     * @see Zend_Form
     * @param string $action Can be add or edit
     * @param mixed $options
     * @return void
     */
    public function __construct($action = "add", $options = null)
    {
        $this->_action = $action;
        parent::__construct($options);
    }

    public function init()
    {
        parent::init();

        //$this->addElementPrefixPath('Reports_Form_ReportAdd_General_Validate', 'reports/forms/ReportAdd/General/Validate', 'validate');

        $this->setAttrib("id", "general-form");                

        $companiesDb = new Core_Model_Companies();

        $identity = Zend_Auth::getInstance()->getIdentity();
        if (!$identity->multi_company) {
            $company = $companiesDb->find($identity->company_id);
            $company_id = new UniForm_Form_Element_Hidden('company_id');
            $company_id->setValue($identity->company_id);
        } else {
            $company_id = new UniForm_Form_Element_Select('company_id');
            $company_id->setLabel(_T('COMPANY'))
                ->addMultiOption(0, _T("SELECT_A_COMPANY."))
                ->setRequired()
                ->setValue(1) //dev
                ->addValidator(new Zend_Validate_GreaterThan(0))
                ->setErrorMessages(array($this->getView()->translate(_T("MUST_BELONG_TO_A_COMPANY."))));

            $companies = $companiesDb->fetchAll($companiesDb->select()->where('active = ?', 1)->order('name'));                                 
            foreach ($companies as $company) {
                $company_id->addMultiOption($company->company_id, $company->name);
            }
            if(isset($this->_companyId)) $company_id->setValue($this->_companyId);
        }             

        $name = new UniForm_Form_Element_Text('name');
        $name->setLabel(_T('NAME'))
            ->setRequired()                        
            ->addFilter('StringTrim');                  

        $schedule = new UniForm_Form_Element_Select('schedule');
        $schedule->setLabel(_T("SCHEDULE"))
            ->setDescription(_T("HOW_OFTEN_DATA_SHALL_BE_PULLED_FROM_LOCAL_FILE_PATH"))
            ->addMultiOption(-1, _T("NEVER"))  
            ->addMultiOption(24*60*60, _T("EVERY_DAY"))
            ->addMultiOption(01*60*60, _T("EVERY_HOUR"));

        $csv_file = new UniForm_Form_Element_File('csv_file');
        $csv_file->setLabel(_T('FILE_CONTAINING_POS_DATA'))
            ->addValidator('Extension', true, 'csv');

        $file_source = new UniForm_Form_Element_Text('file');
        $file_source->setLabel(_T('DIRECTORY_CONTAINING_POS_DATA'))
            ->setDescription(_T("DIRECTORY_CONTAINING_ONE_OR_MULTIPLE_POS_DATA_FILES"));

                /*$separate_char = new UniForm_Form_Element_Text('separate_char');
        $separate_char->setLabel("Separate_char")
                        ->setDescription("Autodetect per default. In most cases you don't have to fill in this.")
                ->setAttrib('size', '2');*/

        $header_enabled = new UniForm_Form_Element_Checkbox('header_enabled');
        $header_enabled->setLabel(_T("USE_HEADER"))
            ->setDescription(_T("CHECK_THIS_IF_THE_FIRST_LINE_IN_THE_CSV_FILE_IS_A_HEADER"))
            ->setValue(1);

        $currency = new UniForm_Form_Element_Text('currency');
        $currency->setLabel(_T("CURRENCY"))
            ->setDescription(_T("THIS_IS_FOR_DISPLAY_PURPOSE._USE_SYMBOLS_LIKE_€_AND_$,_OR_TEXT_LIKE_USD_OR_EURO."))
            ->setValue("€");

        $step = new UniForm_Form_Element_Hidden('step');
        $step->setRequired()
            ->setValue(1);

        $enc = new UniForm_Form_Element_Select('enc');
        $enc->setLabel(_T("ENCODING"));

        $jobsDb = new Pos_Model_ImportJobs();
        foreach($jobsDb->getEncodings() as $val=>$description)
            $enc->addMultiOption($val, $description);
        $enc->setValue("utf8");

        $country = new UniForm_Form_Element_Select('country');
        $country->setLabel(_T('COUNTRY'));

        $countries = new Core_Model_Countries();

        $country->addMultiOption("", _T("SELECT_A_COUNTRY."));
        foreach ($countries->getCountries() as $key => $value) 
            $country->addMultiOption($key, $value);

        $this->addElements(array($company_id, $name, $schedule, $step,$currency));
        $this->addDisplayGroup(array('company_id','name','schedule', 'step','currency'), 'general-group', array("legend" => _T("GENERAL_INFORMATION")));

        $this->addElements(array($csv_file, $file_source, $header_enabled, $country, $enc));
        $this->addDisplayGroup(array('csv_file', 'file', 'header_enabled', 'country', 'enc'), 'file-group', array("legend" => _T("FILE")));

        $submit = new UniForm_Form_Element_Submit('submit');
        $submit->setLabel(_T("NEXT_STEP"))
            ->setAttrib("id", "page-submit");

        $this->addElement($submit);

    }

    /*public function init()
    {
        parent::init();

        $action = $this->_action;

        $name = new UniForm_Form_Element_Text('name');
        $name->setLabel('SET_NAME')
            ->setRequired()
            ->addFilter('StringTrim');

        $submit = new UniForm_Form_Element_Submit('submit');
        $submit->setLabel('SUBMIT');

        $peopleCounterApp = new Core_Model_Application_PeopleCounter();
        $countDataDb = new Core_Model_CountData(Zend_Auth::getInstance()->getIdentity()->company_id, $peopleCounterApp);
        $cameraRows = $countDataDb->getCameraNames();

        $cameras = new UniForm_Form_Element_MultiCheckbox('cameras');
        $cameras->setLabel('CAMERAS')
            ->setRequired();
        foreach ($cameraRows as $cameraId => $cameraName) {
            $cameras->addMultiOption($cameraId, $cameraName);
        }

        $this->addElements(array($name, $cameras)); //, $password_confirm));
        $this->addDisplayGroup(array('name', 'cameras'), 'general-group');


        $this->addElements(array($submit));
    }*/

}
