<?php

class Pos_Model_CSV extends Talking_Db_Table
{
    protected $_name = 'pos_tag';

    //returns array of CSV file fields names
    public static function get_header_fields($file_name, $encoding='utf8', $separator=',', $enclose_char='"', $escape_char='\\')
    {        
        return self::load_line(null, $file_name, 1, $encoding, $separator, $enclose_char, $escape_char);
    }

    public static function get_examples($file_name, $encoding='utf8', $separator=',', $enclose_char='"', $escape_char='\\')
    {
        return self::load_line(null, $file_name, 2, $encoding, $separator, $enclose_char, $escape_char);
    }

    public static function get_line($file_name, $line_num=1)
    {
        $line = '';
        $fpointer = fopen($file_name, "r");
        if ($fpointer)
        {
            for($i=1; $i<=$line_num; $i++)
            {
                $line = fgets($fpointer); //get a line which number is equal to $line_num
            }
        }
        return $line;
    }

    public static function load_line($db, $file_name, $line_num, $encoding='utf8', $separator=',', $enclose_char='"', $escape_char='\\')
    {
        $arrColumns = array();
        $line = self::get_line($file_name, $line_num);
        $arrColumns = self::convert_line($db, $file_name, $encoding, $separator, $enclose_char, $escape_char); 

        if( !empty($line) )
        {
            $filename = tempnam(APPLICATION_PATH . "/../data/session/", 'csv');
            if($filename)
            {
                self::write_file($filename, $line);
                $arrColumns = self::convert_line($db, $filename, $encoding, $separator, $enclose_char, $escape_char);
                self::delete_file($filename);
            }
        }
        return $arrColumns;
    }

    public static function convert_line($db, $file_name, $encoding='utf8', $separator=',', $enclose_char='"', $escape_char='\\')
    {
        $rez = array();

        $fQuickCSV = new Pos_Model_CSVImport($db);

        $fQuickCSV->table_name = sprintf("temp_%s_%d", date("d_m_Y_H_i_s"), rand(1, 100));
        $fQuickCSV->file_name = $file_name;
        $fQuickCSV->use_csv_header = false;
        $fQuickCSV->make_temporary = true;
        $fQuickCSV->field_separate_char = $separator;
        $fQuickCSV->encoding = $encoding;
        $fQuickCSV->field_enclose_char = $enclose_char;
        $fQuickCSV->field_escape_char = $escape_char;

        $fQuickCSV->import();

        if( !empty($fQuickCSV->error) ) {
            exit( $fQuickCSV->error );
        }

        if( 0 == $fQuickCSV->rows_count )
        {
            exit( 'CSV::convert_line returned 0 rows which is not good' );
        }

        $db = $fQuickCSV->getAdapter(); 
        $query = $db->select()->from($fQuickCSV->table_name);
        $fRow = $db->fetchRow($query);
        //include "dBug.php"; //for dev
        //new dBug($fRow);
        return $fRow; 
        //$db->query("SELECT * FROM `".$fQuickCSV->table_name."` LIMIT 1");
        //return $db->getRow();
    }

    //gets count of each possible CSV separator
    //returns most appropriative separator (by max count in the string)
    public static function try_separators($line, $default=null)
    {
        $csv_separators = array(1=>",", ";", "|", "\\", "/", "#", "!", "*", "-");
        $max_similar = 0;
        $index       = 0;
        foreach($csv_separators as $i=>$sep)
        {
            $count = substr_count($line, $sep);
            if($max_similar<$count)
            {
                $max_similar=$count;
                $index = $i;
            }
        }
        if($index>0)
        {
            return $csv_separators[$index];
        }
        elseif( !empty( $default ) )
        {
            return $default;
        }

        return null;
    }

    //returns all fields of [tableName]
    function get_table_fields($tableName )
    {
        $db = $this->getAdapter();
        $metadata = $db->describeTable($tableName);

        //$columnNames = array_keys($metadata);
        //return $columnNames;

        $select = "SELECT pos_tag_id, name from ".$tableName;
        $valueGroups = $db->fetchAll($select);
        //new dBug($valueGroups);
        $ret = array();
        $k = 0;
        foreach ($valueGroups as $vg) {
            $ret[$k]['id'] = $vg->pos_tag_id;
            $ret[$k]['name'] = $vg->name;
            $k++;
        }
        //new dBug($ret);
        return $ret;


        $metadata = $db->fetchAll();
        $columnNames = array_keys($metadata);
        return $columnNames;

        $arrFields = array();
        if( empty($tableName) )
        {
            return false;
        }

        $db->query("SHOW TABLES LIKE '".$tableName."'");

        if( 0 == $db->getRowsCount())
        {
            return false;
        }

        $db->query("SHOW COLUMNS FROM ".$tableName);


        while( $row = mysql_fetch_array($db->fResult) )
        {
            $arrFields[] = trim( $row[0] );
        }

        return $arrFields;
    }

    //writes data in a file
    static function write_file($filename, $data)
    {
        $fp = fopen($filename, 'w');
        if($fp)
        {
            fwrite($fp, $data);
            fclose($fp);
            return true;
        }
        return false;
    }

    //writes data in the end of a file
    static function append_file($filename, $data)
    {
        $fp = fopen($filename, 'a');
        if($fp)
        {
            fwrite($fp, $data);
            fclose($fp);
            return true;
        }
        return false;
    }

    //OS independent deletion of a file
    static function delete_file($filename)
    {
        if(file_exists($filename))
        {
            $os = php_uname();
            if(stristr($os, "indows")!==false)
                return exec("del ".$filename);
            else
                return unlink($filename);
        }
        return true;
    }

    function detect_line_ending($file)
    {
        $s = file_get_contents($file);
        if( empty($s) ) return null;

        if( substr_count( $s,  "\r\n" ) ) return '\r\n'; //Win
        if( substr_count( $s,  "\r" ) )   return '\r';   //Mac
        return '\n'; //Unix
    }

}
