<?php

/*
 Quick_CSV_import class provides interface to a quick CSV file to MySQL database import. Much quicker (10-100 times) that line by line SQL INSERTs.
 version: 1.5
 author: Skakunov Alexander <i1t2b3@gmail.com>
 date: 23.08.2006
 description:
   1. Before importing, you MUST: 
     - establish connection with MySQL database and select database;
     - define CSV filename to import from.
   2. You CAN define several additional import attributes:
     - use CSV header or not: if yes, first line of the file will be recognized as CSV header, and all database columns will be called so, and this header line won't be imported in table content. If not, the table columns will be calles as "column1", "column2", etc
     - separate char: character to separate fields, comma [,] is default
     - enclose char: character to enclose those values that contain separate char in text, quote ["] is default
     - escape char: character to escape special symbols like enclose char, back slash [\] is default
     - encoding: string value which represents MySQL encoding table to parse files with. It's strongly recomended to use known values, not manual typing! Use get_encodings() method (or "SHOW CHARACTER SET" query) to ask the server for the encoding tables list
     - arr_csv_columns is an assoc. array, 'name' goes for field name and 'type' for its SQL type like 'BIGINT UNSIGNED NOT NULL AUTO_INCREMENT'
     - additional_create contains any SQL to add in the end of the CREATE TABLE statement, like 'PRIMARY KEY(`id`)'
   3. You can read "error" property to get the text of the error after import. If import has been finished successfully, this property is empty.
*/


class Pos_Model_CSVImport extends Talking_Db_Table
{
    
  protected $_name = 'pos_tag';  
  protected $_posDateTimeTag;
  
  var $table_name; //where to import to
  var $file_name;  //where to import from
  var $use_csv_header; //use first line of file OR generated columns names
  var $line_separate_char; //character(s) to separate lines (usually \n ) 
  var $field_separate_char; //character to separate fields
  var $field_enclose_char; //character to enclose fields, which contain separator char into content
  var $field_escape_char;  //char to escape special symbols
  var $error; //error message
  var $pos_id; //pos id
  var $arr_csv_columns; //array of columns
  var $arr_csv_columns2; //array of columns
  var $arr_csv_columns_to_load; //array of columns
  var $table_exists; //flag: does table for import exist
  var $make_temporary; //flag: does table for import exist
  var $additional_create;
  var $is_full_import;
  var $rows_count; //how many rows has been imported
  var $encoding; //encoding table, used to parse the incoming file. Added in 1.5 version
  var $fDB; //database object ref
  
  function Quick_CSV_import($fDB, $file_name="")
  {          
    if(!is_object($fDB) || empty($fDB->link))
    {
      throw new Exception('Wrong database object');
    }
    $this->fDB = $fDB;
      $this->fDB->fError = array(); // Added this to prevent flooding error log.
    $this->file_name = $file_name;
    $this->arr_csv_columns = array();
    $this->arr_csv_columns2 = array();    
    $this->line_separate_char = '\n';
    $this->field_separate_char = ",";
    $this->field_enclose_char  = "\"";
    $this->field_escape_char   = "\\";
    $this->table_exists = false;
  }
  
  function import()
  {
    if( empty($this->table_name) )
      $this->table_name = "temp_".date("d_m_Y_H_i_s");
    
    if( !$this->table_exists )
    {
      $this->create_import_table();
    }   

    if( !in_array( $this->line_separate_char, array('\n', '\r', '\r\n' ) ) )
    {
        $this->line_separate_char = '\n';
    }
    
    if(empty($this->arr_csv_columns_to_load))
      $this->arr_csv_columns_to_load = $this->arr_csv_columns;
    
    $fields = array();
    foreach($this->arr_csv_columns_to_load as $column)
    {
      $field = '@dummy';
      if( is_array($column) )
      {
        $field = '`'.$column['name'].'`';
      }
      elseif( '' != trim($column) )
      {
        $field = '`'.$column.'`';
      }
      $fields[] = $field;
    }

    /* change start. Added in 1.5 version */
    if("" != $this->encoding && "default" != $this->encoding)
      $this->set_encoding();
    /* change end */

      $sql = "LOAD DATA LOCAL INFILE '".@mysql_escape_string($this->file_name).
             "' IGNORE INTO TABLE `".$this->table_name.
             "` FIELDS TERMINATED BY '".@mysql_escape_string($this->field_separate_char).
             "' OPTIONALLY ENCLOSED BY '".@mysql_escape_string($this->field_enclose_char).
             "' ESCAPED BY '".@mysql_escape_string($this->field_escape_char).
             "' LINES TERMINATED BY '". $this->line_separate_char .
             "' ".
             ($this->use_csv_header ? " IGNORE 1 LINES " : "")
             ."(".implode(",", $fields).")";
             
        $db = $this->getAdapter();
        $res = $db->query($sql);

        // get rid of double quotes if any
        foreach($fields as $field)
        {
            $query = "UPDATE " . $this->table_name . " SET " . $field . "= REPLACE(" . $field . ",'\"','')";
            $db->query($query);
        }
      $this->fDB->fError = array(); // Added this to prevent flooding error log.
        $this->error = $this->fDB->fError;
      if(empty($this->error)) //OK!
      {
        $query = $db->select()->from($this->table_name, 'count(*)'); 
        $this->error = $this->fDB->fError;
        if(empty($this->error)) //OK!
        {            
            $numRows = $db->fetchOne($query);            
            $this->rows_count = $numRows;
            
            if($this->is_full_import) {
                $this->import_data();  
            }
         }
      }
    
  }
  

  public function import_data() {
        $db = $this->getAdapter();
  
        $pos_id = $this->pos_id; //get pos id

        $query = $db->select()->from($this->table_name); 

        // exec query
        $rs = $db->query($query);

        // fetch
        $rowset = array();		
        $mapping = array_values($this->arr_csv_columns2);

        $this->set_posDateTimeTag();
        //$nbrOfTypes = count(array_filter($mapping))-1;
        $cntId = array();
        $tagDb = new Pos_Model_Tags();
        while ($row = $rs->fetch()) {
                $rowset = array_values((array) $row);
                $dataList = array();
                //$dateTimeList = array();                    
                foreach($mapping as $i=>$column)
                {
                        //if column selected
                        if(!empty($column)) {                                
                            //$posDateTimeTag = $posTags->parseDateTime($column, $rowset[$i]);
                            $rowfield = $rowset[$i];                            
                            $pDT = $this->parseDateTime($column, $rowfield );
                            if(!$pDT) { //add ordinary fields
                                if(!isset($cntId[$column])){
                                    $tag = $tagDb->find($column)->current();
                                    $cntId[$column] = $tag->getCntId();
                                }
                                $dataList[$cntId[$column]] = $rowfield ;
                            }
                        }
                }

                if ($this->_posDateTimeTag) {
                    $datetime = $this->getDateTime();
                    //$datetime = $this->_posDateTimeTag->get_dateTime();
                } else
                    exit("You need to configure a valid DateTime Tag for your CSV file");                   

                if(empty($dataList))
                    exit("You need to configure a Sales column");  
                
                Core_Model_PosData::addData($pos_id, $datetime, $dataList);            
            }

  }
  
  public function set_posDateTimeTag() {
        $this->_posDateTimeTag = new Pos_Model_Tag();
  }
  
  function getDateTime() {
      $tag = $this->_posDateTimeTag;
      $datetime = $tag->get_dateTime();     
      if($datetime) {
          return $datetime;
      }
      $date = $tag->get_date();
      $time = $tag->get_time();
      $timeFrom = $tag->get_fromTime();
      $timeTo = $tag->get_toTime();
      $datetime = $tag->get_time();
      if($date) {
          if($timeFrom && $timeTo) {
                $yourdate = $date->addTime($time);
                $quarter = round(date('i', $yourdate / 15)) * 15;
                $roundeddate = mktime(date("H",$yourdate), $quarter, date("s",$yourdate), date("n",$yourdate), date("j",$yourdate), date("Y",$yourdate));
          } else {
              if($time) {
                if(strlen($time) == 5) $time .= ':00';
                $datetime = $date->addTime($time);
                //new dbug($datetime);
                //exit;
                return $datetime;
                }
                /*$fText = str_replace("%mm", $date->get(Zend_Date::MINUTE), $fText);
		$fText = str_replace("%m", $date->get(Zend_Date::MINUTE_SHORT), $fText);
		$fText = str_replace("%HH", $date->get(Zend_Date::HOUR), $fText);
		$fText = str_replace("%H", $date->get(Zend_Date::HOUR_SHORT), $fText);
		$fText = str_replace("%hh", $date->get(Zend_Date::HOUR_AM), $fText);
		$fText = str_replace("%h", $date->get(Zend_Date::HOUR_SHORT_AM), $fText);

		$fText = str_replace("%ss", $date->get(Zend_Date::SECOND), $fText);
		$fText = str_replace("%s", $date->get(Zend_Date::SECOND_SHORT), $fText);

		$fText = str_replace("%TTTTT", $date->get(Zend_Date::TIME_FULL), $fText);
		$fText = str_replace("%TTTT", $date->get(Zend_Date::TIME_LONG), $fText);
		$fText = str_replace("%TTT", $date->get(Zend_Date::TIME_MEDIUM), $fText);
		$fText = str_replace("%TT", $date->get(Zend_Date::TIME_SHORT), $fText);*/              
          }
     }
  }  
    
  function parseDateTime($cid, $value) {       
      $posTags = new Pos_Model_Tags();
    $dt = $posTags->isDateTime($cid, $value);
    if ($dt) {                    
            $this->_posDateTimeTag->set_dateTime($dt);
    } else {
        $dt = $posTags->isDate($cid, $value);
        if ($dt) {                    
            $this->_posDateTimeTag->set_date($dt);                    
        } else {
            $dt = $posTags->isTime($cid, $value);
            if ($dt) {                    
            $this->_posDateTimeTag->set_time($dt);
            } else {
                $dt = $posTags->isFromTime($cid, $value);
                if ($dt) {                    
                $this->_posDateTimeTag->set_fromTime($value);
                } else {
                    $dt = $posTags->isToTime($cid, $value);
                    if ($dt) {                    
                    $this->_posDateTimeTag->set_toTime($value);
                    } else {
                        return false;
                    }
                }
            }
        }            
    }
      
      return $this->_posDateTimeTag;
  }
 
  //returns array of CSV file columns
  function get_csv_header_fields()
  {     
    $this->arr_csv_columns = array();
    $fpointer = fopen($this->file_name, "r");
    if ($fpointer)
    {
      $arr = fgetcsv($fpointer, 10*1024, $this->field_separate_char);
      if(is_array($arr) && !empty($arr))
      {
        if($this->use_csv_header)
        {
          foreach($arr as $k => $val)
            //if(''!=trim($val))
              //$this->arr_csv_columns[] = array('name'=>$val, 'type'=>'TEXT'); //old with name
              $this->arr_csv_columns[] = array('name'=>'column'.($k+1), 'type'=>'TEXT');
        }
        else
        {
          foreach($arr as $k => $val)
          {
            $this->arr_csv_columns[] = array('name'=>'column'.($k+1), 'type'=>'TEXT');
          }
        }
      }
      unset($arr);
      fclose($fpointer);
    }
    else
      $this->error = "file cannot be opened: ".(""==$this->file_name ? "[empty]" : @mysql_escape_string($this->file_name));
    return $this->arr_csv_columns;
  }
  
  function create_import_table()
  {
    $sql = "CREATE ".($this->make_temporary ? 'TEMPORARY':'')." TABLE IF NOT EXISTS ".$this->table_name." (";
    if(empty($this->arr_csv_columns))
      $this->get_csv_header_fields();
    
    if(!empty($this->arr_csv_columns))
    {
      $arr = array();
      foreach($this->arr_csv_columns as $i=>$column)
        $arr[] = "`".$column['name']."` ".$column['type'];
      if( !empty($this->additional_create) )
        $arr[] = $this->additional_create;
      $sql .= implode(",", $arr);
      $sql .= ")";
        $db = $this->getAdapter();            
        $res = $db->query($sql);       
      //$res = $this->fDB->query($sql);
      $this->fDB->fError = array(); // Added this to prevent flooding error log.
      $this->error = $this->fDB->fError;
      $this->table_exists = empty($this->error);
    }
  }
  
  /* change start. Added in 1.5 version */
  //returns recordset with all encoding tables names, supported by your database
  function get_encodings()
  {           
	$db = $this->getAdapter();            
        return $db->fetchAll("SHOW CHARACTER SET"); 
                
    /*$rez = array();
    $sql = "SHOW CHARACTER SET";
    $res = $this->fDB->query($sql);
    if($this->fDB->fRowsNumber > 0 && empty($this->fDB->fError))
    {
      while ($row = $this->fDB->getRow())
      {
        $rez[$row["Charset"]] = ("" != $row["Description"] ? $row["Description"] : $row["Charset"]); //some MySQL databases return empty Description field
      }
    }
    return $rez;*/
  }
  
  /* change start. Added in 1.5 version */
  //returns recordset with all encoding tables names, supported by your database
  function get_column($column_name, &$whole_count, $page=1, $limit=0)
  {
    $arrColumns = array();
    $sql = sprintf(
           "SELECT SQL_CALC_FOUND_ROWS DISTINCT `%1\$s`
            FROM `%2\$s`
            WHERE `%1\$s` <> ''
            ORDER BY `%1\$s`
           %3\$s", 
           $column_name,
           $this->table_name,
           ($limit > 0 ? 'LIMIT '.($page-1)*$limit.', '.$limit : '')
           );
    $res = $this->fDB->query($sql);
    if( !empty($this->fDB->fError) )
    {
      $this->error = $this->fDB->fError;
      return $arrColumns;
    }

    $arrColumns = $this->fDB->getColumn($column_name);
    $sql = "SELECT FOUND_ROWS()";
    $res = $this->fDB->query($sql);
    $whole_count = $this->fDB->getField();
    return $arrColumns;
  }
  
  //defines the encoding of the server to parse to file
  function set_encoding($encoding="")
  {
    if("" == $encoding)
      $encoding = $this->encoding;
    $sql = "SET SESSION character_set_database = " . $encoding; //'character_set_database' MySQL server variable is [also] to parse file with rigth encoding
    $res = @mysql_query($sql);
    return mysql_error();
  }
  /* change end */

}

?>
