<?php

class Pos_Model_ImportJob extends Talking_Db_Table_Row
{
    protected $_dependentTables = array('Pos_Model_Pos');

    public function getCameras(){
        $cams = array();
        foreach($this->findDependentRowset('Pos_Model_PosCameraSet') as $k => $v){
            $cams[] = $v->camera_id;
        }
        return $cams;
    }

    public function getMapping(){
        return json_decode($this->mapping);
    }

    public function cnttime($data, $fmtDate, $fmtTime, $fn = ""){
        $mapping = $this->getMapping();
        if(array_key_exists('date', $mapping)){
            $lblDate = $mapping->date;
            $date = DateTime::CreateFromFormat($fmtDate["value"], substr($data[$lblDate], 0, strlen($fmtDate["title"])));
            if(!$date)
                throw new WebReport_Exception_ImportError("Bad date format in $fn.");
            if(array_key_exists('time', $mapping)){
                $lblTime = $mapping->time;
                $time = DateTime::CreateFromFormat($fmtTime["value"], substr($data[$lblTime], 0, strlen($fmtTime["title"])));
                if(!$time) 
                    throw new WebReport_Exception_ImportError("Bad time format in $fn.");
                $timeStr = substr($time->format('His'),0,4);
            }else{
                $timeStr = "1200";
            }
            $cnttime = new Zend_Date($date->format('Ymd') . $timeStr, "yyyyMMddHHmm");
            $cnttime = $cnttime->add('00:15:00', Zend_Date::TIMES)->toString("yyyyMMddHHmm");
        } else if(array_key_exists('datetime', $mapping)) {
            $datetime = DateTime::CreateFromFormat($fmtDate["value"] . "*" . $fmtTime["value"], $data[$mapping->datetime]);
            
            if(!$datetime)
                throw new WebReport_Exception_ImportError("Bad time format in $fn.");
            $cnttime = substr($datetime->format('YmdHis'),0,12);
        } else {
            throw new WebReport_Exception_ImportError("Could not find time fields in $fn. DateTime or Date & Time needed.");
        }
        return $cnttime;
    }

    public function addCamerasById($ids){
        foreach($ids as $id){
            $this->addCameraById($id);
        }
    }

    public function clearCameras() {
        $setDb = Pos_Model_PosCameraSet::getInstance();
        $select = $setDb->select()->where('pos_data_id = ' . $this->pos_data_id);
        foreach($setDb->fetchAll($select) as $cam){
            if($cam) $cam->delete();
        }
    }

    public function addCameraById($cameraId){
        $setDb = new Pos_Model_PosCameraSet();
        $select = $setDb->select()->where('pos_data_id =?', $this->pos_data_id)->where('camera_id = ?', $cameraId);
        if($setDb->fetchRow($select)){
            return;
        }
        $row = $setDb->createRow(array(
            'pos_data_id' => $this->pos_data_id,
            'camera_id' => $cameraId
        ));
        $row->save();
    }

    public function getDataList($row){
        $data = array();
        $mapping = $this->getMapping();
        if(isset($mapping->storeid) && $this->store_filter != null &&
             $row[$mapping->storeid] != $this->store_filter) {
            return null;
        }
        if(isset($mapping->sales))
            $data[Core_Model_PosData::TYPE_SALES] = $row[$mapping->sales];
        if(isset($mapping->trans))
            $data[Core_Model_PosData::TYPE_TRANSACTIONS] = $row[$mapping->trans];
        return $data;
    }

    public function runIfTime(){
        if($this->schedule <= 0) return false;
        if(time() - $this->last_sent >= $this->schedule){
            $this->last_sent = time();
            $this->save();
            $this->run();
        }
        return true;
    }

    public function getTimeFormat() {
        $jobs = new Pos_Model_ImportJobs();
        $formats = $jobs->getTimeFormats();
        return $formats[$this->time_format];
    }

    public function getDateFormat() {
        $jobs = new Pos_Model_ImportJobs();
        $formats = $jobs->getDateFormats();
        return $formats[$this->date_format];
    }

    public function run(){
        $path = new Pos_Model_PosPath($this);
        $dateFormat = $this->getDateFormat();
        $timeFormat = $this->getTimeFormat();

        if($path->tryUpdateSyncFile()) {
            foreach($path->getFilesToUpdate(TRUE) as $k => $v){
                $path->checkFileAsUpdated($k);
                error_log("Running ImportJob for $v");
                $import = new Pos_Model_ParseCSV();
                $import->encoding($input=$this->encoding, $output='utf8');
                $import->delimiter = $this->separator;
                $import->heading = $this->use_header == 1;
                while($import->auto($v) == null) {
                    if($import->offset == null)
                        $import->offset = 0;
                    $import->offset++;
                }
                foreach($import->data as $d => $data){
                    $stats = $this->getDataList($data);
                    if(!$stats) continue;
                    $cnttime = $this->cnttime($data, $dateFormat, $timeFormat, $v);
                    foreach(array("-"," ",":") as $symbol)
                        $cnttime = str_replace($symbol,"",$cnttime);

                    Core_Model_CountData::addDataForSeparatedString((object)array(
                        'username'     => Core_Model_CountData::TYPE_POS,
                        'pass'         => NULL,
                        'version'      => 3, 
                        'cameraName'   => '#'.$this->pos_data_id,
                        'serial'       => '000000000000',
                        'regcode'      => 'NO_REGCODE',
                        'fields'       => 10, #Arbitrary number > 9
                        'stats'        => $stats,
                        'cnttime'      => $cnttime,
                        'nbrOfTypes'   => count($stats) 
                    ));
                    //Core_Model_PosData::addData($this->pos_data_id, $this->cnttime($data, $dateFormat, $timeFormat, $v), $this->getDataList($data));
                }
                if(!$path->tryWriteSyncFile()){
                    throw new WebReport_Exception("Could not save sync file!");
                }
            }
        } else {
        }
    }
}
