<?php

class Pos_Model_PosPath {

    protected $_latestError;
    protected $_data;
    protected $_readStructure;
    protected $_folder;
    protected $_syncFilePath;
    protected $_headers;
    protected $_fields;
    protected $_examples;
    protected $_separator;
    protected $_needsUpdate;
    protected $_row;
    public $offset;

    const FirstRuleOfPosClub = "### THIS FOLDER IS INDEXED BY WEBREPORT ###";
    const IndexedFileHead    = "### THIS FOLDER IS INDEXED BY WEBREPORT ###\r\nChanging anything in this file or its directory \r\nmight cause severe problems to Web Report.\r\n\r\n";

    // The convention in this file is that all functions starting
    // with try or _try shuold set _lastError to an error message
    // and return FALSE if an error occurs.

    public function __construct($row, $folder = FALSE, $encoding='utf8', $heading = true){
        $this->_latestError = "NO_ERROR.";
        $this->_data = array('files' => array());
        $this->_readStructure = FALSE;
        $this->_examples = array();
        $this->_fields = array();
        $this->_headers = array();
        $this->_encoding = $encoding;
        $this->_heading = $heading;
        $this->offset = FALSE;
        $this->_row = $row;
        if(!$folder) $folder = $row->file_url;
        $slashed = str_replace("\\","/",$folder);
        $this->_folder = rtrim($slashed,'/');
    }

    public function getHeaders(){
        return $this->_headers;
    }

    public function getFields(){
        return $this->_fields;
    }

    public function getExamples(){
        return $this->_examples;
    }

    public function getSeparator(){
        return $this->_separator;
    }

    public function checkFileAsUpdated($key){
            $md5 = md5_file($this->_folder . $key);
            $this->_data['files'][$key]['md5'] = $md5;
    }

    public function getFilesToUpdate($absolute = false){
        if($absolute){
            $f = array();
            foreach($this->_needsUpdate as $k => $file){
                $f[$k] = $this->_folder . "/" . $file;
            }
            return $f;
        }
        return $this->_needsUpdate;
    }

    public function tryReadStructure(){
        if(!is_dir($this->_folder)) {
            $this->_latestError = _T("THIS_DIRECTORY_DOES_NOT_EXIST_ON_THE_SERVER.");
            return FALSE;
        }

        if(!$this->_tryReadSyncFile()){
            if(!$this->tryCreateSyncFile()){
                return FALSE;
            } else {
                return TRUE;
            }
        }
        /*
        if($this->syncFileExists()){
            if(!$this->_tryReadSyncFile()){
                return FALSE;
            }
        } else if(!$this->tryCreateSyncFile()){
            return FALSE;
        }
        // Sync file should exist by now

        if(!$this->tryUpdateSyncFile(TRUE)){
            return FALSE;
        }

        */
        return TRUE;
    }

    protected function syncFileExists() {
        return TRUE && $this->_row->sync_info;
    }

    protected function _tryReadSyncFile($onlyTry = FALSE){
        if(!$this->_row) {
            $this->_latestError = _T("COULD_NOT_READ_CONTENT_OF_FOLDER.");
            return FALSE;
        }
        $file = $this->_row->sync_info;
        if(strlen($file) > 0 && !$onlyTry) {
            $this->_data = json_decode($file, TRUE);
        }

        return TRUE;
    }

    public function tryVerifyHeaders(){
        $header = NULL;

        if(count($this->_data['files']) == 0) {
            $this->_latestError = _T("THE_DIRECTORY_DOES_NOT_CONTAIN_ANY_CSV_FILES._THE_DIRECTORY_MUST_CONTAIN_EXAMPLE_DATA_IN_CSV_FORMAT.");
            return FALSE;
        }
        foreach($this->_data['files'] as $file){
            $csv = new Pos_Model_ParseCSV();
            $csv->encoding($input=$this->_encoding, $output='utf8');
            $csv->heading = $this->_heading;
            $i = 0;
            $this->offset = false;
            while($csv->auto($this->_folder . $file['filename']) == null && $i++ < 100) {
                if($csv->offset == null)
                    $csv->offset = 0;
                $csv->offset++;
                $this->offset = true;
            }
            if($csv->delimiter == null) {
                $this->_latestError = _T('COULD_NOT_PARSE_FILE._CHECK_THAT_THE_ENCODING_IS_CORRECT.');
                return FALSE;
            }
            $this->_separator = $csv->delimiter;
            $this->_fields = Pos_Model_Fields::GetFields();
            $this->_headers = array_keys($csv->data[floor(count($csv->data)/2)]);
            $this->_examples = array_values($csv->data[floor(count($csv->data)/2)]);
            $this->csv=$csv;
        }
        return true;
    }

    public function tryCreateSyncFile(){
        if($this->_row && $this->_row->sync_info && !$this->_tryReadSyncFile()){
            $this->_latestError = 'CORRUPT_STRUCTURE._SOMETHING_SEEMS_SUSPICIOUS_IN_THE_FILE_STRUCTURE.';
            return FALSE;
        }
        return TRUE;
    }

    public function tryUpdateSyncFile($write = TRUE){
        if(!$this->_tryReadSyncFile()) {
            $this->_data = array( "files" => array());
            //return FALSE;
        }

        $files = glob($this->_folder . "/*csv");

        $needsUpdate = array();
        foreach($files as $file){
            $file = str_replace($this->_folder,"", $file);
            if(!array_key_exists($file,$this->_data['files'])){
                $this->_data['files'][$file] = array('filename' => $file, 'md5' => "");
            } else {
            }
        }
        foreach($this->_data['files'] as $indexed){
            $path = $indexed['filename'];
            if(!file_exists($this->_folder . '/' . $path)) {
                $this->_data['files'][$path]['md5'] = "";
                continue;
            }
            $md5 = $indexed['md5'];
            $newMd5 = md5_file($this->_folder . "/" . $path);
            if($md5 != $newMd5) {
                $needsUpdate[$path] = $path;
            }
        }

        $this->_needsUpdate = $needsUpdate;
        foreach($needsUpdate as $path){
        }

        if(!$write || !$this->tryWriteSyncFile()){
            return FALSE;
        }

        return TRUE;
    }

    public function tryWriteSyncFile(){
        $data = json_encode($this->_data);
        if(!$this->_row || !$this->_row->pos_data_id) {
            return FALSE;
        }
        $this->_row->sync_info = $data;
        $this->_row->save();
        return TRUE;
    }

    public function getLatestTranslatedError(){
        $t = Zend_Registry::get('Zend_Translate');
        return $t->_($this->_latestError);
    }

    private function translatableString(){
        $t->_('NO_ERROR');
        $t->_('COULD_NOT_READ_CONTENT_OF_FOLDER.');
        $t->_('COULD_NOT_WRITE_TO_FOLDER.');
        $t->_('COULD_NOT_UPDATE_SYNC_FOLDER.');
        $t->_('THE_CURRENT_FOLDER_IS_NOT_AN_INDEXED_FOLDER.');
        $t->_('CORRUPT_STRUCTURE._SOMETHING_SEEMS_SUSPICIOUS_IN_THE_FILE_STRUCTURE.');
    }
}
