<?php

class Reports_Pdf_Drawer{
    private $page, $font, $fontsize;
    public function __construct($page, $font, $fontsize) {
        $this->page = $page;
        $this->font = $font;
        $this->fontsize = $fontsize;
    }

    public function text_left($string, $x,$y,$fontsize = null) {
        if(!$fontsize) $fontsize = $this->fontsize;
        self::write_text_left($this->page, $string, $fontsize, $this->font, $x, $y);
    }

    public function text_right($string, $x,$y,$cellWidth) {
        self::write_text_right($this->page, $string, $this->fontsize, $this->font, $x, $y, $cellWidth);
    }

    public function drawRect($page, $color, $x1, $x2, $y1, $y2){
        $black = new Zend_Pdf_Color_Rgb(0,0,0);
        $page->setFillColor($color)
             ->setLineColor($color)
             ->setLineWidth(0)
             ->drawRectangle($x1, $y1, $x2, $y2)
             ->setLineWidth(1)
             ->setFillColor($black)
             ->setLineColor($black);
    }

    public function write_text_centered($page, $string, $font_size, $font, $x, $y)
    {
        $text_width = self::get_text_width($string, $font_size, $font);
        $page->setFont($font, $font_size);
        $page->drawText($string, $x - $text_width/2, $y, 'utf-8');
    }

    public function write_text_center($page, $string, $font_size, $font, $x, $y, $width)
    {
        $fsize = $font_size;
        $text_width = 10000;
        $cuts = 0;

        $text_width = self::get_text_width($string, $fsize, $font);
        if ( $text_width *1.02 >= $width ) {
            $fsize = $font_size;
            $string = substr($string,0,3);
            $text_width = self::get_text_width($string, $fsize, $font);
        }
        $page->setFont($font, $fsize);
        $page->drawText($string, $x + ($width-$text_width)/2, $y, 'utf-8');
    }

    public static function write_text_left($page, $string, $font_size, $font, $x, $y)
    {
        $text_width = self::get_text_width($string, $font_size, $font);
        $page->setFont($font, $font_size);
        $page->drawText($string, $x, $y, 'utf-8');
    }

    public static function write_text_right($page, $string, $font_size, $font, $x, $y, $cellWidth)
    {
        $text_width = self::get_text_width($string, $font_size, $font);
        $page->setFont($font, $font_size);
        $x += $cellWidth - $text_width;
        $page->drawText($string, $x, $y, 'utf-8');
    }

    public static function get_text_width($string, $fontSize, $font)
    {
        $drawingString = iconv('utf-8', 'UTF-16BE//IGNORE', utf8_decode($string));
        $characters = array();
        for ($i = 0; $i < strlen($drawingString); $i++) {
            $characters[] = (ord($drawingString[$i++]) << 8 ) | ord($drawingString[$i]);
        }
        $glyphs = $font->glyphNumbersForCharacters($characters);
        $widths = $font->widthsForGlyphs($glyphs);
        $stringWidth = (array_sum($widths) / $font->getUnitsPerEm()) * $fontSize;
        return $stringWidth;
    }


}
