<?php

class Reports_Pdf_Page extends Zend_Pdf_Page
{

    const MARGIN_TOP = 50;
    const MARGIN_BOTTOM = 60;
    const MARGIN_LEFT = 30;
    const MARGIN_RIGHT = 30;	
    const CELL_MARGIN = 30;

    const COMPANY_LOGO_WIDTH = 120;

    const TEXT_ALIGN_LEFT = 'left';
    const TEXT_ALIGN_RIGHT = 'right';
    const TEXT_ALIGN_CENTER = 'center';

    const CELL_TITLE_FONT_SIZE = 12;
    const CELL_TEXT_SIZE = 10;

    const HEADER_FONT_SIZE = 10;
    const FOOTER_FONT_SIZE = 10;

    const TITLE_FONT_SIZE_LARGE = 38;
    const TITLE_FONT_SIZE_MEDIUM = 28;
    const TITLE_FONT_SIZE_SMALL = 20;

    protected $_page;	
    protected $_cells;
    protected $_show_logo;
    protected $_landscape;
    protected $_pageFrame;
    protected $_width;
    protected $_height;
    protected $_margin_top;	
    protected $_generateDate = Null;

    public function getGenerateDate(){
        return $this->_generateDate;
    }

    public function __construct(Reports_Model_Report_Page $page, $size, $show_logo = FALSE, $generateDate = Null, $font = Null)
    {
        $this->_generateDate = $generateDate;
        parent::__construct($size);
        $this->_page = $page;
        $this->_show_logo = $show_logo;
        if ( $font == null ) throw new Exception("Font (with utf-8 support) must be set before rendering");
        $this->font = $font;
        $this->setFont($this->font, 10);

        $template = $page->getTemplate();

        $this->_landscape = $page->getLandscape(); 
        if ($page->isTitleEnabled()) {
            $this->_margin_top = 90;
        }else{
            $this->_margin_top = self::MARGIN_TOP;
        }
        
        if($this->_landscape){
            $this->rotate($this->getWidth()/2, $this->getWidth()/2, pi()/2);
            $this->_width=$this->getHeight();
            $this->_height=$this->getWidth();
        }else{
            $this->_width=$this->getWidth();
            $this->_height=$this->getHeight();
        }
        $this->_pageFrame = new Reports_Pdf_Frame(self::MARGIN_LEFT, self::MARGIN_BOTTOM,
                $this->_width - self::MARGIN_LEFT - self::MARGIN_RIGHT,
                $this->_height - $this->_margin_top - self::MARGIN_BOTTOM);

        $this->_drawTitle();

        $frames = $this->_calculateFrames($template);		
        $smallest = $this->_calculateCells($frames);

        foreach ($this->_cells as $cell) {
            if(method_exists($cell, "limitTextSize") && $cell->getTextSize() < $smallest+4){
                $cell->limitTextSize($smallest);
            }
            $cell->render();
        }
    }

    protected function _calculateCells($frames){
        $smallest = 100;
        $this->_cells = array();
        foreach ($this->_page->getCells() as $cell) {
            $className = "Reports_Pdf_Page_Cell_" . ucfirst($cell->getType());
            $frame = $frames[$cell->getTemplate()->getId()];
            $pdfCell = new $className($this, $cell, $frames[$cell->getTemplate()->getId()], false);
            if(method_exists($pdfCell, "getTextSize") && $pdfCell-> getTextSize() < $smallest)
                $smallest = $pdfCell-> getTextSize();
            $this->_cells[] = $pdfCell;	
        }
        return $smallest;
    }

    protected function _calculateFrames(Reports_Model_Template $template) 
    {
        if( $this->_landscape){
            $frames = array();
            $f = $this->_pageFrame;
            $width = $f->getWidth();
            $height = $f->getHeight();
            $rows = $template->getRows();
            $rowCount = count($rows);

            $gridWidth = 0;
            $gridHeight = $rowCount;
            if ($rowCount > 0) {
                $cells = current($rows)->getCells();
                foreach ($cells as $cell) {
                    $colspan = $cell->getColspan();
                    if ($colspan <= 1) {
                        $gridWidth += 1;
                    }	else {
                        $gridWidth += $colspan;
                    }
                }
                
                $x = $f->getX()+$width;
                for ($i = 0; $i < $rowCount; $i++) {
                    $row = $rows[$i];
                    $cells = $row->getCells();
                    $cellCount = count($cells);
                    $y = $f->getY2();
                    // FIXME: rowspan don't really work that well

                    for ($j = 0; $j < $cellCount; $j++) {
                        $cell = $cells[$j];
                        $h = $cell->getWidth();
                        $w = $row->getHeight();

                        $rowspan = $cell->getRowspan();
                        $colspan = $cell->getColspan();
                        if ($rowspan == 0) {
                            $rowspan = 1;
                        } 

                        if ($colspan == 0) {
                            $colspan = 1;
                        } 

                        $k = 1;
                        for (; $k < $rowspan; $k++) {
                            $h += $rows[$k]->getHeight();
                        }
                        $h = $h/100*($height - ($gridWidth-$colspan)*self::CELL_MARGIN);

                        $w = $w/100*($width - ($gridHeight-$rowspan)*self::CELL_MARGIN);

                        $frame = new Reports_Pdf_Frame($x-$w, $y-$h, $w, $h);
                        $frames[$cell->getId()] = $frame;
                        $y = $y - $h - self::CELL_MARGIN;
                        if ($i < $rowCount-1) {
//                            $y -= self::CELL_MARGIN;	
                        }				

                    }
                    $x = $x - $w - self::CELL_MARGIN;
                }
            }
 
        }else{
            $frames = array();
            $f = $this->_pageFrame;
            $width = $f->getWidth();
            $height = $f->getHeight();
            $rows = $template->getRows();
            $rowCount = count($rows);
            $y = $f->getY2();

            $gridWidth = 0;
            $gridHeight = $rowCount;
            if ($rowCount > 0) {
                $cells = current($rows)->getCells();
                foreach ($cells as $cell) {
                    $colspan = $cell->getColspan();
                    if ($colspan <= 1) {
                        $gridWidth += 1;
                    }	else {
                        $gridWidth += $colspan;
                    }
                }

                for ($i = 0; $i < $rowCount; $i++) {
                    $row = $rows[$i];
                    $cells = $row->getCells();
                    $cellCount = count($cells);
                    $x = $f->getX();

                    // FIXME: rowspan don't really work that well

                    for ($j = 0; $j < $cellCount; $j++) {
                        $cell = $cells[$j];
                        $w = $cell->getWidth();
                        $h = $row->getHeight();
                        $rowspan = $cell->getRowspan();
                        $colspan = $cell->getColspan();
                        if ($rowspan == 0) {
                            $rowspan = 1;
                        } 

                        if ($colspan == 0) {
                            $colspan = 1;
                        } 

                        $k = 1;
                        for (; $k < $rowspan; $k++) {
                            $h += $rows[$k]->getHeight();
                        }
                        $h = $h/100*($height - ($gridHeight-$rowspan)*self::CELL_MARGIN);

                        $w = $w/100*($width - ($gridWidth-$colspan)*self::CELL_MARGIN);

                        $frame = new Reports_Pdf_Frame($x, $y-$h, $w, $h);
                        $frames[$cell->getId()] = $frame;

                        $x += $w + self::CELL_MARGIN;
                    }
                    $y -= $row->getHeight()/100*($height - ($rowCount-1)*self::CELL_MARGIN);
                    if ($i < $rowCount-1) {
                        $y -= self::CELL_MARGIN;	
                    }				


                }
            }
        }
        return $frames;		
    }

    public function getLandscape(){
        return $this->_landscape;
    }


    protected function _drawTitle()
    {
        $page = $this->_page;
        if ($page->isTitleEnabled()) {
            $title = $page->getTitle();
            $size = self::TITLE_FONT_SIZE_MEDIUM;
            switch ($title->getFontSize()) {
                case Reports_Model_Report_Page::FONT_SIZE_SMALL : 
                    $size = self::TITLE_FONT_SIZE_SMALL;
                    break;
                case Reports_Model_Report_Page::FONT_SIZE_LARGE : 
                    $size = self::TITLE_FONT_SIZE_LARGE;
                    break;
            }

            $this->setFont($this->getFont(), $size);
            $f = $this->_pageFrame;			
            $title = $page->getTitle();
            $text = $title->getText();
            $bottom = $this->drawTextBox($text, $f->getX1(), $f->getY2() - $size - 10, $f->getX2(), $title->getAlign());
            $diff = $f->getY2() - $size - $bottom;
            $this->_pageFrame->decreaseHeight($diff+25);
        }
    }

    private function _drawFooterHeader($fh, $y, $fontsize) {
        $font = $this->getFont();                
        $this->setFillColor(new Zend_Pdf_Color_GrayScale(0.0));
        $this->setFont($font, $fontsize);                
        $this->drawAlignedText($fh->getText(), self::MARGIN_LEFT, $y, $this->_width-self::MARGIN_LEFT, $fh->getAlign(),$font,$fontsize);
    }	

    public function drawHeader($company) 
    {
        $diff = $this->_margin_top/2.0 + self::HEADER_FONT_SIZE/2.0;              	
        $this->_drawFooterHeader($this->_page->getHeader(), $this->_height-$diff, self::HEADER_FONT_SIZE);
        $this->setLineColor(new Zend_Pdf_Color_Grayscale(0.6));
        //$this->drawLine($this->getWidth()-self::MARGIN_LEFT, $this->getHeight()-$diff-35, self::MARGIN_LEFT, $this->getHeight()-$diff-35);
    }       

    public function drawFooter() 
    {
        $diff = self::FOOTER_FONT_SIZE+45;
        $this->_drawFooterHeader($this->_page->getFooter(), $diff, self::FOOTER_FONT_SIZE);                
    }

    public function drawRectangleFromArray($array) {
        return $this->drawRectangle($array[0][0], $array[0][1], $array[1][0], $array[1][1]);
    }

    public function drawLogo($company) {
        $this->_drawLogo($company);
    }


    public function drawCognimaticsHeader() { // Copyright Cognimatics

        $font = $this->getFont();
        $this->setFillColor(new Zend_Pdf_Color_GrayScale(0.5));
        $this->setFont($font, 7);                
        //$this->drawAlignedText("TrueView Web Report® by Cognimatics", self::MARGIN_LEFT, $this->getHeight()-12, $this->getWidth()-self::MARGIN_LEFT, "left");                    

    }

    public function drawCognimaticsFooter($cogLogo = false) { // Copyright Cognimatics
        $diff = self::HEADER_FONT_SIZE+20;
        if($this->_landscape){
            $yPos=40;
        }else{
            $yPos=50;
        }
        $this->drawLine($this->_width-self::MARGIN_LEFT, $yPos, self::MARGIN_LEFT, $yPos);
        $footer = $this->_page->getFooter();
        $date = new Zend_Date();
        if(!is_null($this->_generateDate))
            $date = $this->_generateDate;
        //TODO: generated datum kan vara annat om generateOldReports
        $tz = $date->get(Zend_Date::GMT_DIFF_SEP); 
        $date = $date->get('yyyy-MM-dd HH:mm'); 
        $pageNr = $footer->getPageNumber();
        $font = $this->getFont();
        $this->setFont($font, self::FOOTER_FONT_SIZE); 
        $this->drawText("Generated: $date $tz",self::MARGIN_LEFT,$yPos - self::FOOTER_FONT_SIZE-2);
        $this->_write_text_centered(utf8_encode("Page $pageNr"), $font, self::FOOTER_FONT_SIZE, $this->_width/2, $yPos-self::FOOTER_FONT_SIZE);
        if ($cogLogo) {
            $this->setFont($font, 7); 

            $this->drawText("Powered by", $this->_width - 185,$yPos-9);
            $this->setFont($font, self::FOOTER_FONT_SIZE);
            $year = date("Y");
            $this->drawText(("Cognimatics " . $year . " - For live reports use TrueView Web Report ®"), self::MARGIN_LEFT, $yPos - self::FOOTER_FONT_SIZE*2 - 10,'UTF-8');

            $imgPath = APPLICATION_PATH . "/../public/templates/standard/images/cognimatics.png";   
            //list($width, $height) = getimagesize($imgPath);

            $logo_im = Zend_Pdf_Image::imageWithPath($imgPath);
            $pageHeight = $this->_height;
            $pageWidth = $this->_width;
            $imageHeight = 26;
            $imageWidth = 168;

            $topPos = $yPos - 10;
            $bottomPos = $topPos - $imageHeight;
            $leftPos = $pageWidth-190;
            $rightPos = $leftPos + $imageWidth;                

            $this->drawImage($logo_im, $leftPos, $bottomPos, $rightPos, $topPos);  

        }
    }

    public function _drawLogo($company) 
    {
        $logoFilename = false;
        $alt = "";
        $width = 0;        
        $leftPos = self::MARGIN_LEFT;
        $config = Zend_Registry::get('config');       

        // company logo
        $imgPath = APPLICATION_PATH . "/../public/logos/" . $company->logo_filename;
        if ($this->_show_logo && $company->logo_enable == "1" && file_exists($imgPath)) {
            list($width, $height) = getimagesize($imgPath);
            if ( $this->_show_logo == 1 ) { // left
                $this->_drawLogoImage($imgPath, $leftPos);
                $leftPos += 135;
            } else if ( $this->_show_logo == 2 ) {
                $pos = $this->_width / 2 - self::COMPANY_LOGO_WIDTH / 2;
                $this->_drawLogoImage($imgPath, $pos);
            } else {
                $pos = $this->_width - self::COMPANY_LOGO_WIDTH - self::MARGIN_RIGHT;
                $this->_drawLogoImage($imgPath, $pos);
            }
        }
        if (isset($config->logo) && 
                isset($config->logo->logo_in_reports) && 
                (bool)$config->logo->logo_in_reports && 
                isset($config->logo->logo)) {
            $imgPath = APPLICATION_PATH . "/../public/logos/" . $config->logo->logo;
            if (file_exists($imgPath)) {
                $logoFilename = $config->logo->logo;
                list($width, $height) = getimagesize($imgPath);
                $this->_drawLogoImage($imgPath, $leftPos);
            }
        }
    }

    public function _drawLogoImage($imagePath, $leftPos) {
        $logo_im = Zend_Pdf_Image::imageWithPath($imagePath);
        $pageHeight = $this->_height;
        $pageWidth = $this->_width;
        $imageHeight = 45;
        $imageWidth = self::COMPANY_LOGO_WIDTH;//$logo_im->getPixelWidth();
        

        $topPos = $pageHeight-5;
        $bottomPos = $topPos - $imageHeight;
        $rightPos = $leftPos + $imageWidth;
        $this->drawImage($logo_im, $leftPos, $bottomPos, $rightPos, $topPos);
    }        

    public function drawAlignedText($text, $x1, $y1, $x2 = null, $position = self::TEXT_ALIGN_LEFT, $font, $fontSize, $encoding = null){
        if( $encoding == null ) $encoding = Talking_Pdf::$encoding;

        $bottom = $y1; // could do the same for vertical-centering
        switch ($position) {
            case self::TEXT_ALIGN_LEFT :
                $this->drawText($text,$x1,$y1,$encoding);
                break;
            case self::TEXT_ALIGN_RIGHT :
                if (null === $x2) {
                    throw new Exception ( "Cannot right-align text horizontally, x2 is not provided" );
                }
                $textWidth = $this->_get_text_width($text, $font, $fontSize);
                $this->drawText($text,$x2-$textWidth,$y1,$encoding);
                break;
            case self::TEXT_ALIGN_CENTER :
                if (null === $x2) {
                    throw new Exception ( "Cannot center text horizontally, x2 is not provided" );
                }
                $textWidth = $this->_get_text_width($text, $font, $fontSize);
                $this->_write_text_centered($text,$font,$fontSize,$x1+($x2-$x1)/2,$y1);
                break;
            default :
                throw new Exception ( "Invalid position value \"$position\"" );
        }
    }

    protected function _write_text_centered($string, $font, $font_size, $x, $y)
    {
        $this->setFont($font, $font_size);

        $text_width = $this->_get_text_width($string, $font, $font_size);

        $this->drawText($string, $x - $text_width/2, $y, 'utf-8');
    }

    protected function _get_text_width($string, $font, $fontSize)
    {
        $drawingString = iconv('UTF-8', 'UTF-16BE//IGNORE', $string);
        $characters = array();
        for ($i = 0; $i < strlen($drawingString); $i++) {
            $characters[] = (ord($drawingString[$i++]) << 8 ) | ord($drawingString[$i]);
        }
        $glyphs = $font->glyphNumbersForCharacters($characters);
        $widths = $font->widthsForGlyphs($glyphs);
        $stringWidth = (array_sum($widths) / $font->getUnitsPerEm()) * $fontSize;
        return $stringWidth;
 
    }
    public function drawTextBox($text, $x1, $y1, $x2 = null, $position = self::TEXT_ALIGN_LEFT, $lineHeight = 1.3, $encoding = null){
        if( $encoding == null )  {
            $encoding = Talking_Pdf::$encoding;
        }


        $lines = explode("\n", $text);

        $bottom = $y1;
        $lineHeight = $this->getFontSize() * $lineHeight;
        foreach( $lines as $line ){
            preg_match_all('/([^\s]*\s*)/i', $line, $matches);

            $words = $matches[1];

            $lineText = '';
            $lineWidth = 0;
            $prevLine = '';
            foreach( $words as $word ){
                $lineText .= $word;
                $lineWidth = $this->getTextWidth($lineText);
                $url_regex = "#((http|https|ftp)://(\S*?\.\S*?))(\s|\;|\)|\]|\[|\{|\}|,|\"|'|:|\<|$|\.\s)#ie";

                if(preg_match($url_regex, $word)) {
                    // $word is URL create link

                    $target = Zend_Pdf_Action_URI::create($word);
                    $linkWidth = $this->getTextWidth($word);

                    if($lineWidth >= $x2-$x1) {
                        $annotation = Zend_Pdf_Annotation_Link::create(
                                $x1,
                                $bottom,
                                $x1 + $linkWidth,
                                $bottom - $lineHeight,
                                $target
                                );
                    }
                    else {
                        $annotation = Zend_Pdf_Annotation_Link::create(
                                $x1 + $lineWidth - $linkWidth,
                                $bottom + $lineHeight,
                                $x1 + $lineWidth,
                                $bottom,
                                $target);
                    }

                    $this->attachAnnotation($annotation);
                }

                if( $lineWidth >= $x2-$x1 ){
                    $this->drawAlignedText($prevLine, $x1, $bottom, $x2, $position, $this->_font, self::CELL_TEXT_SIZE, $encoding );
                    $bottom -= $lineHeight;
                    $lineText = $word;
                    $line = '';
                    $lineWidth = $this->getTextWidth($word);
                }
                $prevLine = $lineText;
            }
            $this->drawAlignedText( $lineText, $x1, $bottom, $x2, $position, $this->_font, self::CELL_TEXT_SIZE, $encoding );
            $bottom -= $lineHeight;
        }

        return $bottom;
    }


    function getTextWidth($string, $font = null, $fontSize = null)
    {
        if( !isset($encoding) || $encoding == null ) $encoding = Talking_Pdf::$encoding;

        if( !$font ){
            $font = $this->getFont();
            $fontSize = $this->getFontSize();
        } elseif( $font instanceof Zend_Pdf_Resource_Font ){
            if( $fontSize === null ) throw new Talking_Pdf_Exception('The fontsize is unknown');
        }

        if( !$font instanceof Zend_Pdf_Resource_Font ){
            throw new Exception('Invalid resource passed');
        }

        $drawingString = iconv('UTF-8', 'UTF-16BE//IGNORE', $string);
        $characters = array();
        for ($i = 0; $i < strlen($drawingString); $i++) {
            $characters[] = (ord($drawingString[$i++]) << 8 ) | ord($drawingString[$i]);
        }
        $glyphs = $font->glyphNumbersForCharacters($characters);
        $widths = $font->widthsForGlyphs($glyphs);
        $stringWidth = (array_sum($widths) / $font->getUnitsPerEm()) * $fontSize;
        return $stringWidth;
    }

}



