<?php

class Reports_Pdf_Page_Cell_Datasource extends Reports_Pdf_Page_Cell_Abstract
{

    const WANTED_FONTSIZE = 9;
    const SMALLEST_FONTSIZE = 6;
    const WANTED_NUMBERSIZE = 7;
    const SMALLEST_NUMBERSIZE = 3;

    const MARGIN = 1.5;

    protected $_maxLabels = 6;
    protected $_inch = 2.54;

    protected $_includeTable;
    protected $_drawWeather;
    protected $_table = NULL;

    protected $_textSize;
    protected $_numberSize;
    protected $_weatherIconSize = 25;
    protected $_printErrorMessage = false;

    protected $_chartRenderer = null;

    public function render()
    {
        set_time_limit(60);
        $tic = time(NULL);
        $date = $this->_page->getGenerateDate();
        parent::render();

        $this->_page->setFont($this->_page->getFont(), Reports_Pdf_Page::CELL_TEXT_SIZE);

        try {
            $this->_chartData = $this->_cell->getDataSource();
        } catch(Exception $e){
            $margin = 5;
            $this->_page->drawTextBox("An image could not be generated here. The chart selected for this cell does not exist anymore", $this->_drawingFrame->getX1()+$margin, $this->_drawingFrame->getY2()-Reports_Pdf_Page::CELL_TEXT_SIZE-$margin, $this->_drawingFrame->getX2()-$margin);
            return;
        }
        
        $this->_chartRenderer   = $this->_cell->getRenderInstance();
        $this->_chartRenderer   ->setGenerateDate($date);
        $this->_chartRenderer   ->_renderCommon($this->_chartData);

        $this->_drawWeather     = Reports_Model_WeatherServiceAbstract::WeatherIsEnabled() && $this->_cell->getHasWeather();
        $this->_includeTable    = $this->_cell->getIncludeTable();
/*
        if($this->_includeTable){
            $tableRenderer      = new Charts_Model_ChartRender_Table();
            $tableRenderer->setGenerateDate($date);
            $this->_tableData   = $tableRenderer->render($this->_chartData);
        }
 */
        $this->_prep();
        $this->_drawingFrame->decreaseHeight2($this->_drawingHeight);

        $fn = APPLICATION_PATH . "/../data/session/" . uniqid();
        $this->_chartRenderer->setWidth($this->_drawingFrame->getWidth()*$this->_inch);
        $this->_chartRenderer->setHeight($this->_drawingFrame->getHeight()*$this->_inch);
        $this->_chartRenderer->_drawLegend = false;
        $this->_chartRenderer->_includeTable = $this->_includeTable;

        try {
            $new_fn = $this->_chartRenderer->renderToFile($this->_chartData, $fn);
            $pdf_im = Zend_Pdf_Image::imageWithPath($new_fn);
            $this->_page->drawImage($pdf_im,$this->_drawingFrame->getX1(), $this->_drawingFrame->getY1(), $this->_drawingFrame->getX2(), $this->_drawingFrame->getY2());
            unlink($new_fn);
        } catch(Exception $e){
            $margin = 5;
            $this->_page->drawTextBox("An image could not be generated here. This might be caused by missing read/write access to data/session in the application folder.", $this->_drawingFrame->getX1()+$margin, $this->_drawingFrame->getY2()-Reports_Pdf_Page::CELL_TEXT_SIZE-$margin, $this->_drawingFrame->getX2()-$margin);
            return;
        }

        $this->_origo = $this->_chartRenderer->_origo;
        $this->_drawStuff();

    }

    protected function _prep(){
        //200 is according to chartrender png (2*dx)
        $this->_width       = $this->_drawingFrame->getWidth()-(200/$this->_inch);
        $this->_height      = $this->_drawingFrame->getHeight()-(200/$this->_inch);
        $this->_font        = $this->_page->getFont();
        $this->_labels      = $this->_chartRenderer->_dataLabels;
        $this->_labelsRaw   = $this->_chartRenderer->_dataLabelsRaw;
        $this->_nbrTimes    = count($this->_labels);
        $this->_cellWidth   = $this->_width/$this->_nbrTimes;
        $this->_y2          = $this->_drawingFrame->getY1();
        $this->_translate   = Zend_Registry::get('Zend_Translate');

        $textSize = $this->_makeLabelsFit($this->_cellWidth);

        //cut labels if text got too small
        if($textSize < self::SMALLEST_FONTSIZE){
            $this->_cutLabels();
            $textSize = $this->_makeLabelsFit($this->_cellWidth);
        }

        //skip labels if text is still too small
        $this->_skippedLabels = 0;
        $count = 0;
        while($textSize < self::SMALLEST_FONTSIZE && count($this->_labels)>2){
            $count++;
            $this->_skippedLabels = pow(2,$count)-1;
            $this->_skipEveryOtherLabel();
            $textSize = $this->_makeLabelsFit($this->_cellWidthAfterSkipping);
        }

        $this->_textSize            = $textSize;
        $this->_lineWidth           = $textSize/12;
        $this->_cellHeight          = $this->_textSize + 2*self::MARGIN + $this->_lineWidth;
        $this->_marginAboveSources  = $this->_textSize*2;
        $this->_marginAboveErrorMsg = $this->_textSize;
        $this->_marginAboveWeather  = 0; //initial value
        $this->_errorMsgHeight      = 0; //initial value
        $this->_boxSize             = ($this->_cellHeight - $this->_lineWidth) * (7/12);
        $this->_boxMargin           = ($this->_cellHeight - $this->_lineWidth) * (2/12);
        $this->_lineSize            = ($this->_cellHeight - $this->_lineWidth) * (2/12);
        $this->_shadowSize          = ($this->_cellHeight - $this->_lineWidth) * (1/12);

        if($this->_drawWeather){
            $this->_prepWeather();
        }else{
            $this->_weatherFits = false;
            $this->_weatherHeight= 0;
        }

        $tableHeight = 0;
        if($this->_includeTable){
            $tableHeight            = $this->_prepTable();
        }

        $this->_columns             = $this->_getColumns();
        $this->_legendHeight        = count($this->_columns[0])*($textSize+self::MARGIN*2);
        $this->_drawingHeight       = $this->_cellHeight + $tableHeight + $this->_marginAboveSources + $this->_legendHeight + $this->_weatherHeight + $this->_marginAboveWeather + $this->_errorMsgHeight + $this->_marginAboveErrorMsg;
    }

    protected function _prepWeather(){
        $this->_weatherTime             = $this->_get_weather_time($this->_cell->getWeatherTimePeriod());
        $generateDate                   = $this->_page->getGenerateDate();
        $this->_weatherData             = array();
        $this->_weatherError            = "";
        $dayFit                         = $this->_chartRenderer->_resolution == 1 && $this->_weatherTime['resolution'] == 'd';
        $weekFit                        = $this->_chartRenderer->_resolution == 2 && $this->_weatherTime['resolution'] == 'w';
        $minutesFit                     = $this->_chartRenderer->_resolution == 6 && $this->_weatherTime['resolution'] == 'd' && $this->_nbrTimes < 20;
        $this->_weatherFits             = ($dayFit || $weekFit || $minutesFit)   && $this->_skippedLabels == 0;
        $this->_weatherFontSize         = (2/3)*$this->_textSize;

        //If page has date set that differs from todays date subtract the difference from weathertime.
        if ($generateDate!=null) {
            $now = new Zend_date;
            $dateDiff = $now->sub($generateDate);
            $this->_weatherTime['date'] = $this->_weatherTime['date']->sub($dateDiff);
        }


        $data = $this->_getWeather();

        //If all weatherData objects have errors (code <10) no weather will be drawn.
        $this->_allErrCode = true;
        foreach ($data as $d){
            if($d->getCode() >= 10){
                $this->_allErrCode = false;
            }
        }
        if ($this->_allErrCode) {
            $this->_weatherError = $this->_translate->_('COULD_NOT_GET_DATA_FROM_WEATHER_SERVICE,_CHECK_WEATHER_LOGS_IN_WEB_REPORT_FOR_DETAILS.');//Add details to logs in next commit
            $this->_weatherHeight = 0;
            $this->_weatherIconSize = $this->_textSize*1.5;
            $this->_weatherFits = false;
        }else{
        // see if weather really matches the labels
            if ($this->_weatherFits){
                $allLabelErrors = true;
                $noLabelErrors = true;
                //Match weathertime with pdf labels.
                foreach($this->_labelsRaw as $label){
                    $foundMatching = false;
                    foreach($data as $d){
                        if (strtolower(mb_substr($d->getTime(),0,3,'UTF-8')) == strtolower(mb_substr($label,0,3,'UTF-8'))
                            || ((int) mb_substr($d->getTime(),0,2,'UTF-8'))-1 == mb_substr($label,0,2,'UTF-8')
                            || ((int) mb_substr($d->getTime(),0,2,'UTF-8'))+1 == mb_substr($label,0,2,'UTF-8')){
                            $this->_weatherData[] = $d;
                            $foundMatching = true;
                            $allLabelErrors = false;
                            break;
                        }
                    }
                    if ($foundMatching == false){
                        $noLabelErrors = false;
                        $errorDataIcon = new Reports_Model_WeatherData(0,0,0,10,false,"Label could not be matched to weather data.");
                        $this->_weatherData[] = $errorDataIcon;
                    }
                }
                if($allLabelErrors == true){
                    $this->_weatherError = $this->_translate->_('CHART_X-AXIS_LABELS_COULD_NOT_BE_MATCHED_TO_WEATHER_DATA.');
                    $this->_weatherFits = false;
                }elseif($allLabelErrors == false && $noLabelErrors == false){
                    $this->_weatherError = $this->_translate->_('ONE_OR_MORE_CHART_X-AXIS_LABELS_COULD_NOT_BE_MATCHED_TO_WEATHER_DATA.');
                }
            }

            if ($this->_weatherFits){
                $this->_weatherIconSize = min($this->_cellWidth*0.6, $this->_weatherIconSize);
            } else {
                $this->_weatherIconSize =  min($this->_width/12, $this->_height/6);
                $this->_marginAboveWeather = $this->_textSize*2;
                $this->_weatherData = $data;
            }
            $this->_weatherHeight= $this->_weatherIconSize + $this->_weatherFontSize*2.2;
            if($this->_weatherIconSize < 15){
                $this->_weatherHeight -= $this->_weatherFontSize*2.2; //dont include text
            }
        }
        if(!$this->_weatherError==""){
            $this->_printErrorMessage = true;
            $this->_errorMsg = $this->_weatherError;
            $this->_errorMsgHeight==$this->_textSize*2.5; //margins included
        }else{
            $this->_printErrorMessage = false;
        }
    }

    protected function _cutLabels(){
        if(!preg_match('/\d/', $this->_labels[0])){
            foreach($this->_labels as $k => $label){
                if(strlen($label)>4)
                    $this->_labels[$k]=mb_substr($label,0,3,'UTF-8');
            }
        }
    }

    protected function _makeLabelsFit($width){
        $longestLabel       = $this->_labels[0];
        $textSize           = self::WANTED_FONTSIZE;
        foreach($this->_labels as $label){
            $labelLength = $this->_get_text_width($label, $textSize);
            $longestLabelLength = $this->_get_text_width($longestLabel, $textSize);
            if($labelLength > $longestLabelLength)
                $longestLabel = $label;
        }
        while($this->_get_text_width($longestLabel, $textSize) > $width){
            $textSize--;
        }
        return $textSize;
    }

    protected function AlmostAmChartDurationFormat($value) {
        //$trans = Zend_Registry::get('Zend_Translate');
        $m = $value % 60;
        $h = ($value-$m) / 60;
        if($h < 1) return $m."m";
            return $h."h ".$m."m";
    }

    protected function _prepTable(){
        //former tablerenderer
        $data = array();
        $data['isTime'] = Array();
        for($j=0; $j < $this->_chartRenderer->_dataLength; $j++) {
            for($i=0; $i < $this->_chartRenderer->_numberOfDataSources; $i++) {
                $data[$i][$j] = $this->_chartRenderer->_dataSourceData[$i][$j];
                if(!$j) $data['isTime'][$i] = false;
                if($this->_chartRenderer->_dataSourceType[$i]=='DURATION'){
                    if(!$j) $data['isTime'][$i] = true;
                    $data[$i][$j] = $this->AlmostAmChartDurationFormat($data[$i][$j]);
                }
            }
        }
        $this->_tableData = $data;

        //calculate font size for numbers inside table
        $this->_numberSize    = self::WANTED_NUMBERSIZE;
        $longestNumber = $this->_tableData[0][0];
        for($r=0; $r < $this->_chartRenderer->_numberOfDataSources; $r++){
            for($c=0; $c < $this->_nbrTimes; $c++){
                $numberLength = $this->_get_text_width($this->_tableData[$r][$c],$this->_numberSize);
                $longestNumberLength = $this->_get_text_width($longestNumber, $this->_numberSize);
                if($numberLength > $longestNumberLength)
                    $longestNumber = $this->_tableData[$r][$c];
            }
        }
        while($this->_get_text_width($longestNumber, $this->_numberSize) > $this->_cellWidth){
            $this->_numberSize--;
        }

        //round off to large numbers
        if($this->_numberSize < self::SMALLEST_NUMBERSIZE){
            $this->_numberSize = self::SMALLEST_NUMBERSIZE;
            for($r=0; $r < $this->_chartRenderer->_numberOfDataSources; $r++){
                if($this->_tableData['isTime'][$r]){
                    for($c=0; $c < $this->_nbrTimes; $c++){
                        $number = $this->_tableData[$r][$c];
                        $numberWidth = $this->_get_text_width($number, $this->_numberSize);
                        while($numberWidth > $this->_cellWidth && !strpos(" ",$number)){
                            $number = $this->_tableData[$r][$c];
                            $this->_tableData[$r][$c] = $this->_shortenTime($number);
                        }
                    }
                }else{
                    for($c=0; $c < $this->_nbrTimes; $c++){
                        $pot = 0;
                        $number = $this->_tableData[$r][$c];
                        $numberWidth = $this->_get_text_width($number, $this->_numberSize);
                        while($numberWidth > $this->_cellWidth & $number>800){
                            $number = round($number/1000);
                            $pot++;
                        }
                        if($pot == 0){
                            $this->_tableData[$r][$c] = $number;
                        }else if($pot == 1){
                            $this->_tableData[$r][$c] = $number . 'k';
                        }else if($pot == 2){
                            $this->_tableData[$r][$c] = $number . 'M';
                        }else{
                            $this->_tableFits = false;
                        }
                    }
                }
            }
        }

        $biggestFont            = max($this->_numberSize, $this->_textSize);
        $this->_cellHeight      = $biggestFont + 2*self::MARGIN + $this->_lineWidth;
        $tableHeight            = $this->_cellHeight * $this->_chartRenderer->_numberOfDataSources;
        return $tableHeight;
    }

    protected function _shortenTime($time){
        $ind=max(strpos(" ",$time));
        return mb_substr($time,0,$ind,'UTF-8'); //If you get a mergeconflict here, this is the correct version
    }

    protected function _skipEveryOtherLabel(){
        $labels = Array();
        for($k = 0; $k < count($this->_labels); $k += 2)
            $labels[$k/2] = $this->_labels[$k];
        $this->_labels = $labels;
        $gainedWidth = ($this->_skippedLabels * $this->_cellWidth)*(2/5)*2;
        $this->_cellWidthAfterSkipping = $gainedWidth + $this->_cellWidth;
        $this->_cellSpace = ($this->_skippedLabels * $this->_cellWidth)*(1/5);
    }

    protected function _getColumns(){
        $columns = Array(Array());
        switch ($this->_chartRenderer->_numberOfDataSources){
            case 1:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                break;
            case 2:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                break;
            case 3:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                break;
            case 4:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[3];
                break;
            case 5:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[4];
                break;
            case 6:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[4];
                $columns[1][2]=$this->_chartRenderer->_dataSourceTitles[5];
                break;
            case 7:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[4];
                $columns[1][2]=$this->_chartRenderer->_dataSourceTitles[5];
                $columns[2][0]=$this->_chartRenderer->_dataSourceTitles[6];
                break;
            case 8:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[4];
                $columns[1][2]=$this->_chartRenderer->_dataSourceTitles[5];
                $columns[2][0]=$this->_chartRenderer->_dataSourceTitles[6];
                $columns[2][1]=$this->_chartRenderer->_dataSourceTitles[7];
                break;
            case 9:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[4];
                $columns[1][2]=$this->_chartRenderer->_dataSourceTitles[5];
                $columns[2][0]=$this->_chartRenderer->_dataSourceTitles[6];
                $columns[2][1]=$this->_chartRenderer->_dataSourceTitles[7];
                $columns[2][2]=$this->_chartRenderer->_dataSourceTitles[8];
                break;
            case 10:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[0][3]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[4];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[5];
                $columns[1][2]=$this->_chartRenderer->_dataSourceTitles[6];
                $columns[2][0]=$this->_chartRenderer->_dataSourceTitles[7];
                $columns[2][1]=$this->_chartRenderer->_dataSourceTitles[8];
                $columns[2][2]=$this->_chartRenderer->_dataSourceTitles[9];
                break;
            case 11:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[0][3]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[4];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[5];
                $columns[1][2]=$this->_chartRenderer->_dataSourceTitles[6];
                $columns[1][3]=$this->_chartRenderer->_dataSourceTitles[7];
                $columns[2][0]=$this->_chartRenderer->_dataSourceTitles[8];
                $columns[2][1]=$this->_chartRenderer->_dataSourceTitles[9];
                $columns[2][2]=$this->_chartRenderer->_dataSourceTitles[10];
                break;
            case 12:
                $columns[0][0]=$this->_chartRenderer->_dataSourceTitles[0];
                $columns[0][1]=$this->_chartRenderer->_dataSourceTitles[1];
                $columns[0][2]=$this->_chartRenderer->_dataSourceTitles[2];
                $columns[0][3]=$this->_chartRenderer->_dataSourceTitles[3];
                $columns[1][0]=$this->_chartRenderer->_dataSourceTitles[4];
                $columns[1][1]=$this->_chartRenderer->_dataSourceTitles[5];
                $columns[1][2]=$this->_chartRenderer->_dataSourceTitles[6];
                $columns[1][3]=$this->_chartRenderer->_dataSourceTitles[7];
                $columns[2][0]=$this->_chartRenderer->_dataSourceTitles[8];
                $columns[2][1]=$this->_chartRenderer->_dataSourceTitles[9];
                $columns[2][2]=$this->_chartRenderer->_dataSourceTitles[10];
                $columns[2][3]=$this->_chartRenderer->_dataSourceTitles[11];
                break;

        }
        return $columns;
    }

    public function interpolate($hex1,$hex2,$scale) {
        $hex1="$hex1";
        $hex2="$hex2";
        if(strlen($hex1) == 3) {
            $r1 = hexdec(substr($hex1,0,1).substr($hex1,0,1));
            $g1 = hexdec(substr($hex1,1,1).substr($hex1,1,1));
            $b1 = hexdec(substr($hex1,2,1).substr($hex1,2,1));
        } else {
            $r1 = hexdec(substr($hex1,0,2));
            $g1 = hexdec(substr($hex1,2,2));
            $b1 = hexdec(substr($hex1,4,2));
        }
        if(strlen($hex2) == 3) {
            $r2 = hexdec(substr($hex2,0,1).substr($hex2,0,1));
            $g2 = hexdec(substr($hex2,1,1).substr($hex2,1,1));
            $b2 = hexdec(substr($hex2,2,1).substr($hex2,2,1));
        } else {
            $r2 = hexdec(substr($hex2,0,2));
            $g2 = hexdec(substr($hex2,2,2));
            $b2 = hexdec(substr($hex2,4,2));
        }
        $channel=dechex($r1*(1-$scale)+$r2*$scale);
        if(strlen($channel)<2){
            $channel = "0" . "$channel";
        }
        $ret=$channel;

        $channel=dechex($g1*(1-$scale)+$g2*$scale);
        if(strlen($channel)<2){
            $channel = "0" . "$channel";
        }
        $ret=$ret . $channel;
        $channel=dechex($b1*(1-$scale)+$b2*$scale);
        if(strlen($channel)<2){
            $channel = "0" . "$channel";
        }
        $ret=$ret . $channel;
        return new Zend_Pdf_Color_Html("#".$ret);
    }


    protected function _drawStuff(){
        // (x1,y1) Origo i grafen
        $frameWidth = $this->_drawingFrame->getX2() - $this->_drawingFrame->getX1();
        $frameHeight= $this->_drawingFrame->getY2() - $this->_drawingFrame->getY1();
        $this->_x1  = $this->_drawingFrame->getX1() + $frameWidth * $this->_origo['x'];
        $this->_y1  = $this->_drawingFrame->getY2() - $frameHeight* $this->_origo['y'];

        $this->_drawLabels();
        if($this->_drawWeather && $this->_weatherFits)
            $this->_drawWeather();
        if($this->_includeTable)
            $this->_drawTable();
        $this->_drawLegend();
        if($this->_drawWeather && !$this->_weatherFits)
            $this->_drawWeather();
        if ($this->_printErrorMessage){
            $this->_drawErrorMessage();
        }
    }

    protected function _write_text_centered($string, $font_size, $x, $y)
    {
        $this->_page->setFont($this->_font, $font_size);

        $text_width = $this->_get_text_width($string, $font_size);

        $this->_page->drawText($string, $x - $text_width/2, $y, 'utf-8');
    }

    protected function _get_text_width($string, $fontSize)
    {
        $drawingString = iconv('UTF-8', 'UTF-16BE//IGNORE', $string);
        $characters = array();
        for ($i = 0; $i < strlen($drawingString); $i++) {
            $characters[] = (ord($drawingString[$i++]) << 8 ) | ord($drawingString[$i]);
        }
        $glyphs = $this->_font->glyphNumbersForCharacters($characters);
        $widths = $this->_font->widthsForGlyphs($glyphs);
        $stringWidth = (array_sum($widths) / $this->_font->getUnitsPerEm()) * $fontSize;
        return $stringWidth;
    }

    protected function _get_weather_time($time_period_code) {
        $date = new Zend_Date();
        $length = strlen($time_period_code);

        $ret = array();

        $code = $time_period_code[$length-1];
        if($code == 'w' ||
                $code == 'd') {
            $ret['resolution'] = $code;

            $offset = sscanf($time_period_code, "%d" . $code);
            $offset = $offset[0];

            if($offset > 0) {
                throw new Webreport_Exception("Time offset must be negative.");
            }

            switch($code) {
                case 'w':
                    $date->addWeek($offset);
                    break;
                case 'd':
                    $date->addDay($offset);
                    break;
            }

        } else {
            throw new Webreport_Exception("Invalid time period code.");
        }

        $ret['date'] = $date;

        return $ret;
    }

    public function _drawLabels(){
        $lightGray      = new Zend_Pdf_Color_Html("#F7F7F7");
        $this->_black   = new Zend_Pdf_Color_Html("#000000");
        $this->_white   = new Zend_Pdf_Color_Html("#FFFFFF");

        $cellWidth      = $this->_cellWidth;
        $cellHeight     = $this->_cellHeight;
        $fontSize       = $this->_textSize;
        $margin         = ($cellHeight-$fontSize)/2;
        if($this->_drawWeather && $this->_weatherFits)
            $cellHeight += $this->_weatherHeight;
        $page           = $this->_page;
        $x1             = $this->_x1;
        $x2             = $this->_x1 + $this->_width;
        $y1             = $this->_y1;

        //remove old labels
        $page->setFillColor($this->_white)
            ->setLineColor($this->_white)
            ->drawRectangle($x1,$y1,$x2,$y1-15)
            ->setFillColor($this->_black)
            ->setLineColor($this->_black);
        //yPos for writing text
        $yPos = $y1 - $margin - $this->_textSize;

        //draw labels if skipLabels is true
        if($this->_skippedLabels != 0){
            $xPos =  $this->_x1+$cellWidth/2;
            $cellWidth = $this->_cellWidthAfterSkipping;
            foreach($this->_labels as $k => $label){
                $this->_write_text_centered($label, $this->_textSize, $xPos, $yPos);
                $xPos += $cellWidth/2;
                $xPos2 = $xPos+$this->_cellSpace;
                if(count($this->_labels) == $k+1){
                    if($xPos > $x2)   $xPos2 = $xPos;
                    else                        $xPos2 = $x2;
                }
                $page->setFillColor($lightGray)
                    ->setLineColor($lightGray)
                    ->setLineWidth($this->_lineWidth)
                    ->drawRectangle($xPos, $y1, $xPos2, $y1-$cellHeight)
                    ->setFillColor($this->_black)
                    ->setLineColor($this->_black);
                $xPos = $xPos2 + $cellWidth/2;
            }
        }else{ //draw labels reggularely
            $xPos = $this->_x1 + $cellWidth/2;
            foreach($this->_labels as $k => $label){
                if($k % 2 != 0){
                    $xPos1 = $xPos - $cellWidth/2;
                    $xPos2 = $xPos + $cellWidth/2;
                    $page->setFillColor($lightGray)
                        ->setLineColor($lightGray)
                        ->setLineWidth($this->_lineWidth)
                        ->drawRectangle($xPos1, $y1, $xPos2, $y1-$cellHeight)
                        ->setFillColor($this->_black)
                        ->setLineColor($this->_black);
                }
                $this->_write_text_centered($label,$this->_textSize,$xPos,$yPos);
                $xPos += $cellWidth;
            }
        }
        $page->drawLine($x1,$y1,$x2,$y1);
    }

    public function _drawWeatherIcon($icon,$x,$y){
        $x1 = $x - $this->_weatherIconSize/2;
        $x2 = $x + $this->_weatherIconSize/2;
        $fontSize = $this->_weatherFontSize;
        $margin = $fontSize * 0.1;
        $this->_page->drawImage($icon->getImage(), $x1, $y-$this->_weatherIconSize, $x2, $y);
        if($icon->isRealData()) {
            $y1         = $y - $this->_weatherIconSize - $fontSize/2;
            $string1    = utf8_decode($icon->getTemperature().utf8_encode('°C'));
            $y2         = $y1 - $fontSize - $margin;
            $string2    = utf8_decode($icon->getPrecipitationText());
            $transp     =  max(0.6 - $icon->getPrecipitation() * 5,0);
            if($this->_weatherIconSize < 15){
               return;
            }
            $this->_write_text_centered($string1, $fontSize, $x, $y1);

            if($icon->getPrecipitation() == 0 && $this->_weatherFits)
                return;

            if($this->_cellWidth > $this->_get_text_width($string2, $fontSize)){
                $transpColor= new Zend_Pdf_Color_Rgb($transp,$transp,$transp);
                $this->_page->setFillColor($transpColor);
                $this->_write_text_centered($string2, $fontSize, $x, $y2);
            }
            $this->_page->setFillColor($this->_black);
        }
    }
    public function _getWeather(){
        $date = $this->_weatherTime['date'];
        $res  = $this->_weatherTime['resolution'];
        $weatherService = Reports_Model_WeatherServiceAbstract::GetWeatherService();
        switch($res) {
            case 'w':
                $data = $weatherService->getWeekWeather($date, $this->_cell->getWeatherCoord());
                break;
            case 'd':
                $data = $weatherService->getDayWeather($date, $this->_cell->getWeatherCoord());
                break;
        }
        return $data;
    }

    public function _drawWeather(){
        if ($this->_allErrCode){
            $x = $this->_x1 + $this->_width/2;
            $y = $this->_y2 + $this->_weatherHeight;
            $this->_drawErrorMessage($x,$y);
            return;
        }
        if($this->_weatherFits){
            $x      = $this->_x1 + $this->_cellWidth/2;
            $y      = $this->_y1 - $this->_cellHeight;
            foreach($this->_weatherData as $d){
                    $this->_drawWeatherIcon($d, $x, $y);
                    $x += $this->_cellWidth;
            }
        }else{
            $space   = ($this->_width - count($this->_weatherData)*$this->_weatherIconSize) / (count($this->_weatherData)+1);
            $x = $this->_x1 + $space + $this->_weatherIconSize/2;
            $y = $this->_y1 - $this->_cellHeight - $this->_marginAboveSources - $this->_legendHeight - $this->_marginAboveWeather;
            if($this->_includeTable){
                $y -= $this->_cellHeight*$this->_chartRenderer->_numberOfDataSources;
            }
            foreach($this->_weatherData as $d) {
                $this->_write_text_centered($d->getTime(),$this->_weatherFontSize, $x, $y);
                $this->_drawWeatherIcon($d, $x, $y);
                $x += $this->_weatherIconSize+$space;
            }
        }
    }

    protected function _drawErrorMessage(){
        $errorIconSize = $this->_textSize;
        $iconTextMargin = $this->_textSize;
        $x = $this->_x1 - $errorIconSize*0.5;
        $y = $this->_y1 - $this->_cellHeight - $this->_marginAboveSources - $this->_legendHeight - $this->_marginAboveWeather - $this->_weatherHeight - $this->_marginAboveErrorMsg - $this->_textSize;
        if($this->_includeTable){
            $y -= $this->_cellHeight*$this->_chartRenderer->_numberOfDataSources;
            $x1 = $x - $errorIconSize;
            $x2 = $x;
        }else{
            $x1 = $x;
            $x2 = $x + $errorIconSize;
        }

        $image_path = APPLICATION_PATH . WEATHER_ICON_PATH . 'icon_error_warning.png';
        $error_icon = Zend_Pdf_Image::imageWithPath($image_path);
        $this->_page->drawImage($error_icon, $x1, $y-$errorIconSize+$this->_textSize, $x2, $y+$this->_textSize);
        $this->_page->setFont($this->_font, $this->_textSize);
        $this->_page->drawText($this->_weatherError, $x1+$errorIconSize+$iconTextMargin, $y,'utf-8');
    }

    protected function _drawRect($color, $x1, $y1, $x2, $y2){
        $this->_page->setFillColor($color)
                    ->setLineColor($color)
                    ->drawRectangle($x1, $y1, $x2, $y2)
                    ->setFillColor($this->_black)
                    ->setLineColor($this->_black);
     }

    protected function _drawNumber($number, $x, $y){
        $darkGray           = new Zend_Pdf_Color_Rgb(0.8,0.8,0.8);
        if($number == 0 ){
            $this->_page->setFillColor($darkGray)
                        ->setLineColor($darkGray);
        }else{
            $this->_page->setFillColor($this->_black)
                        ->setLineColor($this->_black);
        }
        $this->_write_text_centered($number, $this->_numberSize, $x, $y);
    }

    public function _drawTable(){
        $fontSize           = $this->_numberSize;
        $margin             = ($this->_cellHeight - $fontSize)/2;

        //colors
        $boxColorsBright    = $this->_chartRenderer->_dataSourceColors;
        $boxColorsDark      = $this->_chartRenderer->_dataSourceColors;
        foreach($boxColorsBright as $k=>$color)
            $boxColorsBright[$k]=$this->interpolate($color,"FFFFFF",0.95);
        foreach($boxColorsDark as $k=>$color)
            $boxColorsDark[$k]=$this->interpolate($color,"FFFFFF",0.85);

        $y = $this->_y1 - $this->_cellHeight;
        if($this->_weatherFits) $y -= $this->_weatherHeight;
        for($r=0; $r < $this->_chartRenderer->_numberOfDataSources; $r++){
            $colorCount = 0;
            $y1         = $y  - $this->_cellHeight*($r) - $this->_lineWidth;
            $y2         = $y1 - $this->_cellHeight;
            $ytext      = $y2 + $margin;
            $xbox       = $this->_x1-$this->_boxSize-$this->_shadowSize-$margin;
            $ybox       = $y2 + ($this->_cellHeight - $this->_boxSize)/2;
            $this->_drawRect($boxColorsBright[$r], $xbox-$margin, $y1, $this->_x1+$this->_width, $y2);
            $this->_drawSourceSymbol($r, $xbox, $ybox);
            for($c=0; $c < $this->_nbrTimes; $c++){
                $x1 = $this->_x1 + $c*$this->_cellWidth;
                $xtext = $x1 + $this->_cellWidth/2;
                if($colorCount % 2 != 0)
                    $this->_drawRect($boxColorsDark[$r], $x1, $y1, $x1+$this->_cellWidth, $y2);
                $this->_drawNumber($this->_tableData[$r][$c],$xtext,$ytext);
                $colorCount++;
            }
            $this->_page->setLineColor(new Zend_Pdf_Color_Html('#' . $this->_chartRenderer->_dataSourceColors[$r]))
                 ->drawLine($xbox-$margin, $y1, $this->_x1+$this->_width, $y1)
                 ->setLineColor($this->_black);
        }
    }

    public function _drawSourceSymbol($sourceIndex, $x, $y){
        $shadow             = new Zend_Pdf_Color_Rgb(0.5, 0.5, 0.5);
        $shadowSize         = $this->_shadowSize;
        $color              = new Zend_Pdf_Color_Html('#' . $this->_chartRenderer->_dataSourceColors[$sourceIndex]);
        if($this->_chartRenderer->_dataSourceAppearance[$sourceIndex] == 'line'){
            $y1 = $y + ($this->_boxSize-$this->_lineSize)/2;
            $y2 = $y1 + $this->_lineSize;
        }else if ($this->_chartRenderer->_dataSourceAppearance[$sourceIndex] == 'bar'){
            $y1 = $y;
            $y2 = $y1 + $this->_boxSize;
        }
        $x2 = $x + $this->_boxSize;
        $this->_drawRect($shadow, $x+$shadowSize, $y1-$shadowSize, $x2+$shadowSize, $y2-$shadowSize);
        $this->_drawRect($color, $x, $y1, $x2, $y2);
    }

    public function _drawLegend(){
        $boxTextMargin  = $this->_textSize;
        $columnWidth    = $this->_width/count($this->_columns);
        $maxTextWidth   = $columnWidth - $this->_boxSize - $boxTextMargin*2;
        $boxTextDiff    = ($this->_textSize- ($this->_boxSize+$this->_shadowSize))/2;
        $margin         = ($this->_cellHeight - $this->_numberSize)/2;

        $x = $this->_x1;
        $y = $this->_y1 - $this->_cellHeight - $this->_marginAboveSources - $this->_textSize;
        if($this->_includeTable){
            $x -= $this->_boxSize+$this->_shadowSize+$margin;
            $y -= $this->_cellHeight*$this->_chartRenderer->_numberOfDataSources;
        }
        if($this->_weatherFits)
            $y -= $this->_weatherHeight;

        $sourceIndex = 0;
        foreach($this->_columns as $i=>$column){
            $xPos = $x + $columnWidth*$i;
            $yPos = $y;
            foreach($column as $j=> $source){
                if($source){
                    $this->_drawSourceSymbol($sourceIndex, $xPos, $yPos);

                    //cut source text if too long
                    $cut = false;
                    while($this->_get_text_width($source, $this->_textSize) >= $maxTextWidth){
                        $source = substr($source,0,-2);
                        $cut = true;
                    }
                    if($cut)
                        $source = $source . "...";

                    $this->_page->setFont($this->_page->getFont(), $this->_textSize);
                    $this->_page->drawText($source, $xPos+$this->_boxSize+$boxTextMargin, $yPos+$boxTextDiff,'utf-8');
                    $yPos -= $this->_textSize + self::MARGIN;
                    $sourceIndex++;
                }
            }
        }
    }
}
