<?php

class Reports_ExportController extends WebReport_Controller_Action
{
    protected $_user;

    public function init()
    {
    	parent::init();

        $usersDb = new Core_Model_Users();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

    	$this->view->headTitle("EXPORT_DATA", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function indexAction()
    {
    	if (!$this->getRequest()->isPost()) {
    		$form = new Reports_Form_ExportData($this->_user);
    		$this->view->form = $form;
    		$this->render("step1");
    		$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 1), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    	} else { // request is post
    		if ((int)$_POST['step'] == 1) {
    			$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    			$form = new Reports_Form_ExportData($this->_user);

    			if ($form->isValid($_POST)) {
    				$values = $_POST;
    				$exportNamespace = new Zend_Session_Namespace('WebReport_DataExport');
    				$identity = Zend_Auth::getInstance()->getIdentity();
    				$count_data_set_id = 0;
    				if (isset($values['count_data_set_id'])) { $count_data_set_id = $values['count_data_set_id']; }

    				if ($count_data_set_id == 0) {
    					$form->addError($this->_translate->_('NO_COUNT_DATA_SET_IS_SELECTED'));
    					$this->view->form = $form;
    					$this->render("step1");
    					return;
    				}

    				$exportNamespace->format = $values['format'];

    				$companiesDb = new Core_Model_Companies();
    				$applicationsModel = new Core_Model_Applications();

    				$countDataSets = new Core_Model_CountData_Sets();
    				$countDataSet=$countDataSets->find($count_data_set_id)->current();
    				$countDataDb = $countDataSet->getCountData();
    				$company = $companiesDb->find($identity->company_id)->current();

    				try{
    					$countDataDb->initDatabaseSelect();
    					$this->view->emptyCountDataSet = false;
    				}
    				catch (WebReport_Exception_NoFieldsToSelect $e){
    					$form = new Reports_Form_ExportData($this->_user);
    					$this->view->form = $form;
    					$this->view->emptyCountDataSet = true;
    					$form->addError('THE_CHOSEN_COUNT_DATA_SET_DID_NOT_YIELD_ANY_COUNT_DATA.');
    					$this->render("step1");
    				}
    				
    				if(!$this->view->emptyCountDataSet){
    					$this->view->timeResolutions = $countDataDb->getTimeResolutions();
    					$this->view->timeIntervals = Core_Model_Factory_TimeInterval::makeSelections();
    					foreach($this->view->timeIntervals as $key => &$value){
    						$value = $this->_translate->_($value);
    					}
    					$this->view->exportActions = array(
    					    "table" => $this->_translate->_("CREATE_TEMPORARY_COUNT_DATA_TABLE"),
    					    "link" => $this->_translate->_("CREATE_PERMANENT_LINK_TO_COUNT_DATA_TABLE"),
    					    );

    					$firstDate = $countDataDb->getFirstDate();
    					$lastDate = $countDataDb->getLastDate();
    					$sevenDaysBeforeLast = clone $lastDate;
    					$sevenDaysBeforeLast->sub(7, Zend_Date::DAY);

    					if ($sevenDaysBeforeLast->compare($firstDate) > 0) {
    						$firstDateSelected = $sevenDaysBeforeLast;
    					} else {
    						$firstDateSelected = $firstDate;
    					}

    					$this->view->firstDate = $firstDate;
    					$this->view->lastDate = $lastDate;
    					$this->view->firstDateSelected = $firstDateSelected;
    					$this->view->lastDateSelected = $lastDate;
    					$exportNamespace->timeFilterId = 0;
    					$exportNamespace->data_aggregator_id = 0;
    					$exportNamespace->countDataDb = serialize($countDataDb);
    					$exportNamespace->countDataSetId = $count_data_set_id;
    					
    					$this->view->showTimeResolution = true;
    					
    					$this->render("step2");
    				}
    			} else {

    				$form->populate($_POST);
    				$this->view->form = $form;
    				$this->render("step1");

    			}

    		} else { // step == 2
    			$exportNamespace = new Zend_Session_Namespace('WebReport_DataExport');
    			$name = $exportNamespace->name;
    			
    			$timeFormat = $_POST['time-interval-type'];
    			$manualTimeInterval = $_POST['manual-time-interval'];
    			
    			$firstDateSelected = new Zend_Date();
    			$firstDateSelected->setDate($_POST['time-interval-start'], "yyyy-MM-dd");
    			$lastDateSelected = new Zend_Date();
    			$lastDateSelected->setDate($_POST['time-interval-stop'], "yyyy-MM-dd");
    			if($timeFormat == "man"){
    				$timeFormat = $manualTimeInterval;
    				if(!CM_Interval::isValidCode($timeFormat)){
    					$this->render("badtime");
    					return;
    				}
    			}
    			$this->view->headTitle(sprintf($this->_translate->_('STEP_%1$s'), 2), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    			$this->_helper->layout->disableLayout();    //disable layout
    			$this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering
    			if($timeFormat != "fix"){
    				$interval = new CM_Interval(Zend_Date::now(), $timeFormat);
    				$firstDateSelected = $interval->getStartDate();
    				$lastDateSelected = $interval->getStopDate();			
    			}

    			$timeResolution = $_POST['time-resolution'];
    			$action = $_POST['export-action'];
    			
    			$countDataSetId = $exportNamespace->countDataSetId;
    			$countDataSetDb = new Core_Model_CountData_Sets();
    			$countDataSet = $countDataSetDb->find($countDataSetId)->current();
    			$countData = $countDataSet->getCountData();
    			$sourceCount = $countData->getNumberOfSources();
    			
    			$timeFilterId = $exportNamespace->timeFilterId;
    			$aggregatorId = $exportNamespace->data_aggregator_id;
    			$outputFormat = $exportNamespace->format;
    			
    			$load = Core_Model_CountData::estimateQueryLoad($timeResolution, $firstDateSelected, $lastDateSelected, null,$sourceCount);

                                $allowOverload = false;    			
                                if($allowOverload && !Core_Model_CountData::acceptLoad($load)){
    				$this->render("overload");
    			}else{				
    				if($action == "table"){
                        error_log("$countDataSetId\n$outputFormat\n$aggregatorId\n$timeFilterId\n$timeResolution\n$timeFormat\n$firstDateSelected\n$lastDateSelected");
    					Reports_Model_ReportCompiler::renderReport($countDataSetId,
                                                                   $outputFormat,
                                                                   $aggregatorId,
                                                                   $timeFilterId,
                                                                   $timeResolution,
                                                                   $timeFormat,
                                                                   $firstDateSelected, 
                                                                   $lastDateSelected);
    				}else if($action == "link"){
    					$permaDb = new Reports_Model_Permalinks();
    					$all = $permaDb->fetchAll();
    					$auth =  Zend_Auth::getInstance()->getIdentity();
    					
    					$filter = $permaDb->addPermalink($outputFormat,$auth->company_id,$countDataSetId,$aggregatorId,$timeFilterId,$timeResolution, $timeFormat, $_POST['time-interval-start'], $_POST['time-interval-stop']);
    					$this->view->fullAddress = $filter->getApi($this->view);
    					$this->render("api");
    				}else{
    					throw new WebReport_Exception("An undefined export action was specified.");
    				}
    			}
    		}
    	}
    }

    public function validTimeCodeAjaxAction() {
    	$this->_helper->layout->disableLayout();
    	$this->_helper->viewRenderer->setNoRender();
    	
    	try {
    		$valid = CM_Interval::isValidCode($_POST['code']);
    	} catch(Exception $e) {
    		echo Zend_Json::encode(array("valid"=>false, "message" => $e->getMessage()));
    		return;
    	}
    	
    	echo Zend_Json::encode(array("valid" => $valid, "message" => "success"));
    }

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {
    	_("EXPORT_DATA");
    	_('STEP_%1$s');
    	_('NO_DATA_IS_SELECTED.');
    	_('NO_COUNT_DATA_SET_IS_SELECTED');
    	_("TOO_MUCH_DATA_SELECTED._DESELECT_A_FEW_CAMERAS_AND_TRY_AGAIN.");
    	_("THE_CHOSEN_COUNT_DATA_SET_DID_NOT_YIELD_ANY_COUNT_DATA");
        _('ENABLE_TITLE');
        _('DATA_AGGREGATOR');
        _("SELECT_A_TIME_INTERVAL.");
        _("CURRENT_DAY");
        _("CURRENT_WEEK");
        _("CURRENT_MONTH");
        _("CURRENT_YEAR");
        _("LAST_DAY");
        _("LAST_WEEK");
        _("LAST_MONTH");
        _("LAST_YEAR");
        _("MANUAL_TIME_INTERVAL");
        _("FIXED_INTERVAL");
        _("FIXED_DATE_INTERVAL");
        _("MANUAL_INTERVAL");
        _("isEmpty");
        _("NO_COUNT_DATA_SET_IS_SELECTED.");
        _("LETTER");
    	_("NONE");
        _("A_PERMANENT_LINK_WAS_CREATED_FOR_THE_REPORT.");
    	_("THE_REPORT_CAN_BE_REACHED_AT:");
    	_("TIME_FILTER");
    	_("FILTER_CODE");
    	_("SELECT_A_REPORT");
    	_("CHOOSE_FREQUENCY");
		_("MONDAY");
		_("TUESDAY");
		_("WEDNESDAY");
		_("THURSDAY");
		_("FRIDAY");
		_("SATURDAY");
		_("SUNDAY");
		_("CHOOSE_DAY_OF_MONTH.");
		_("CHOOSE_DAY_OF_WEEK.");
		_("CHOOSE_TIME.");
		_("LAST_DAY_OF_MONTH");
		_("CHOOSE_TIMEZONE.");
		_("EVERY_MONTH");
    	_("EVERY_WEEK");
    	_("EVERY_DAY");
    	_("HOW_OFTEN_SHOULD_THIS_SUBSCRIPTION_BE_SENT");
    	_("BODY_TEXT_TO_BE_INCLUDED_IN_EMAIL.");
    	_("CHOOSE_WHO_WILL_RECIVE_REPORT.");
    	_("SEND_FREQUENCY");
    	_("MAIL_BODY_TEXT");
    	_("CHOOSE_RECIVER_ROLES");
    	_("WEEK_USAGE");_("WEEK_USAGE(RELATIVE)");
    	_("MONTH_USAGE");_("MONTH_USAGE(RELATIVE)");
    	_("HOUR_USAGE");_("HOUR_USAGE(RELATIVE)");
        _("15_MINUTE_USAGE");
        _("DAY_OF_MONTH_USAGE");_("DAY_OF_MONTH_USAGE(RELATIVE)");
    }
}

