<?php

class Reports_IndexController extends WebReport_Controller_Action
{
    protected $_user, $_group;
   
    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();
        
        $usersDb = new Core_Model_Users();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_REPORTS');

        if($this->_group && $this->_group->canEdit($this->_user))
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", "folder" => $groupId), "text" => 'ADD_REPORT');

        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

        $this->view->headTitle("REPORTS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function browseAction()
    {
        $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();
        $this->view->dateFormat = strtolower($dateFormat);
        $this->view->dateFormat = str_replace("yyyy", "yy", $this->view->dateFormat);
        $this->view->dateFormat = $this->view->dateFormat ? $this->view->dateFormat : "yy-mm-dd";
        $this->view->list = WebReport_Controller_Helper_Browse::getController($this->_user, $this->_group, $this->_helper, $this->_request,
                                                                              'Reports_Model_ReportMapper',
                                                                              'Reports_Model_ReportListGenerator');
        
        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

	public function copyAction() {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

		$identity = Zend_Auth::getInstance()->getIdentity();

		$id = (int)$this->_getParam("id");

        $document = Core_Model_DocumentFactory::fetch($id, "report");
        if ( !$document->canEdit($this->_user) ) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_COPY_THIS_REPORT.'));
            $this->_helper->redirector('browse','index','reports');
            return;
        }

        $folder = $document->getGroup();
        if ( !$folder->canEdit($identity) ) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_COPY_THIS_REPORT.'));
            $this->_helper->redirector('browse','index','reports');
            return;
        }
        
		$reportMapper = new Reports_Model_ReportMapper();
		$report = $reportMapper->find($id);
		if (!$report ) {
			throw new WebReport_Exception_ParameterError($this->_translate->_('NO_REPORT_WITH_SUCH_ID_EXISTS.'));
		}

        $newReport = $report->createCopyOf($identity);
        Core_Model_DocumentFactory::add($newReport->getId(), "report", $folder);

		$this->_helper->redirector('browse', 'index', 'reports', array('folder' => $this->_group->group_id));
    }

	public function addAction() {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector('browse','index','reports');
            return;
        }

        $this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/json2.js'))
            ->appendFile($this->view->baseUrl('/modules/reports/javascript/jquery/plugins/jquery.reportsEditor.js'));

        /* Load company */
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($this->_user->company_id)->current();
        $this->view->company = $company;

        /* Load templates */
        $templateMapper = new Reports_Model_TemplateMapper();
        $templates = $templateMapper->fetchAll();
        $this->view->templates = $templates;

        /* Load Count data sets */
        $setsDb = new Core_Model_CountData_Sets();
        $this->view->countDataSets = $setsDb->fetchAll();

        $form = new Reports_Form_ReportAdd_General($company);
        $config = Zend_Registry::get('config');
        if(isset($config->report))
          $form->populate(array("coglogo_enabled" => $config->report->coglogo_enabled));
        $this->view->form = $form;

        $templateCellForm = new Reports_Form_ReportAdd_TemplateCell($company);
        $this->view->templateCellForm = $templateCellForm;

        $pageForm = new Reports_Form_ReportAdd_Page();
        $this->view->pageForm = $pageForm;

        $this->view->folder = $this->_group->group_id;
    }

    public function editAction()
    {
        $this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/json2.js'));
        $this->view->headScript()->appendFile($this->view->baseUrl('/modules/reports/javascript/jquery/plugins/jquery.reportsEditor.js'));

        $identity = Zend_Auth::getInstance()->getIdentity();

        $id = (int)$this->_getParam("id");

        $document = Core_Model_DocumentFactory::fetch($id, "report");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_REPORT.'));
            $this->_helper->redirector('browse','index','reports');
            return;
        }

        $reportMapper = new Reports_Model_ReportMapper();
        $report = $reportMapper->find($id);
        if (!$report ) {
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_REPORT_WITH_SUCH_ID_EXISTS.'));
        }

        /* Load company */
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($identity->company_id)->current();
        $this->view->company = $company;

        /* Load templates */
        $templateMapper = new Reports_Model_TemplateMapper();
        $templates = $templateMapper->fetchAll();
        $this->view->templates = $templates;

        /* Load Count data sets */
        $setsDb = new Core_Model_CountData_Sets();
        $this->view->countDataSets = $setsDb->fetchAll();


        $form = new Reports_Form_ReportAdd_General($company);
        $form->populate(
            array(                    
                "company_id" => $this->_group->company_id,
                "id" => $report->getId(),
                "name" => $report->getName(),
                "description" => $report->getDescription(),
                "header_enabled" => $report->isPageHeaderEnabled(),
                "header_text" => $report->getPageHeader()->getUnformattedText(),
                "header_align" => $report->getPageHeader()->getAlign(),
                "footer_enabled" => $report->isPageFooterEnabled(),
                "footer_text" => $report->getPageFooter()->getUnformattedText(),
                "footer_align" => $report->getPageFooter()->getAlign(),
                "logo_enabled" => $report->showLogo(),
                "coglogo_enabled" => $report->isCogLogoEnabled()
                ));
        $this->view->form = $form;

        $templateCellForm = new Reports_Form_ReportAdd_TemplateCell($company);
        $this->view->templateCellForm = $templateCellForm;

        $pageForm = new Reports_Form_ReportAdd_Page();
        $this->view->pageForm = $pageForm;

        $this->view->pages = $report->getPages();

        $this->view->folder = $this->_group->group_id;

        $this->renderScript("index/add.phtml");

    }

    public function deleteAction()
    {
        $id = (int)$this->_getParam('id');

        $document = Core_Model_DocumentFactory::fetch($id, "report");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_DELETE_THIS_REPORT.'));
            $this->_helper->redirector('browse','index','reports', array('folder' => $this->_group->group_id));
            return;
        }

        $reportMapper = new Reports_Model_ReportMapper();
        $report = $reportMapper->find($id);

        if (!$report) {
            throw new WebReport_Exception($this->_translate->_('NO_REPORT_WITH_SUCH_ID_EXISTS.'));
        }

        if($report->usedInSubscription()) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_DELETE_THIS_REPORT_BECAUSE_IT_IS_USED_IN_A_SUBSCRIPTION.'));
            $this->_helper->redirector('browse','index','reports', array('folder' => $this->_group->group_id));
            return;
        }

        $this->_flashMessenger->addMessage($this->_translate->_('REPORT_DELETED.'));
        Core_Model_DocumentFactory::remove($report->getId(), "report");
        $reportMapper->delete($report);

        $this->_helper->redirector('browse', 'index', 'reports', array('folder' => $this->_group->group_id));
    }

    public function saveAction()
    {

        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        if (!$this->_request->isPost()) {
            echo "[]";
            exit;
        }

        $data = $this->_getParam("data");
        $data  = Zend_Json::decode($data, Zend_Json::TYPE_OBJECT);

        $reportMapper = new Reports_Model_ReportMapper();

        $general = $data->general;
        $id = $general->id;

        if ($id) {
            $report = $reportMapper->find($id);
            $header = $report->getPageHeader();
            $footer = $report->getPageFooter();
        } else {
            $report = new Reports_Model_Report();
            $header = new Reports_Model_Report_Page_Header();
            $footer = new Reports_Model_Report_Page_Footer();
        }
                
        $identity = Zend_Auth::getInstance()->getIdentity();
        $companiesDb = new Core_Model_Companies();                
        $company = $companiesDb->find($this->_group->company_id)->current();
        $report
            ->setName($general->name)
            ->setDescription($general->description)
            ->setSize($general->size)
            ->setCompany($company);

        // Report header
        $report->enablePageHeader($general->header->enabled);
        if (!$report->isPageHeaderEnabled()) {
            $header->reset();
        } else {
            $header->setText($general->header->text)
                ->setAlign($general->header->align);
        }
        if ( $general->header->logo ) {
            $report->showLogo($general->header->logo_align);
        } else {
            $report->showLogo(0);
        }
        $header->setCompany($company);
        $report->setPageHeader($header);


        // Report footer
        $report->enablePageFooter($general->footer->enabled);
        if (!$report->isPageFooterEnabled()) {
            $footer->reset();
        } else {
            $footer->setText($general->footer->text)
                ->setAlign($general->footer->align);
        }
        $footer->setCompany($company);
        $report->setPageFooter($footer);
        $report->enableCogLogo($general->coglogo_enabled);



        // receivers
        // receivers are disabled
        /*if ($id) { // update receivers
          $userIds = $general->userIds;
          $users = $report->getReceivers();
          $us = array();
          $usersToDelete = array();
          foreach ($users as $u) {
          $uid = $user->user_id;
          if (($index = array_search($uid, $userIds, false)) === false) {
          $usersToDelete[] = $u;
          } else {
          unset($userIds[$index]);
          $us[] = $u;
          }
          }

          $usersDb = new Core_Model_Users();
          $users = $usersDb->find($userIds);
          foreach ($users as $u) {
          $us[] = $u;
          }

          foreach ($usersToDelete as $utd) {
          $utd->delete();
          }

          $report->setReceivers($us);
          } else {
          $users = new Core_Model_Users();
          $us = array();
          $users = $users->find($userIds);
          foreach ($users as $u) {
          $us[] = $u;
          }
          $report->setReceivers($us);
          } */


        // store the template and template cells to avoid multiple database requests later
        $templateMapper = new Reports_Model_TemplateMapper();
        $templateStore = $templateMapper->fetchAll(array("order" => "template_id"));
        $templates = array();
        foreach ($templateStore as $template) {
            $templates[(int)$template->getId()] = $template;
        }

        $templateCellMapper = new Reports_Model_Template_Row_CellMapper();
        $templateCellsStore = $templateCellMapper->fetchAll(array("order" => "template_row_cell_id"));
        $templateCells = array();
        foreach ($templateCellsStore as $templateCell) {
            $templateCells[(int)$templateCell->getId()] = $templateCell;
        }

        $reportPages = $report->getPages();
        $pageMapper = new Reports_Model_Report_PageMapper();
        foreach ($reportPages as $rp) {
            $pageMapper->delete($rp);
        }
        // TODO: instead of removing all pages and inserting them again, update existing
        $report->setPages(array());    // remove all pages

        $pages = array();
        $index = 0;
        foreach ($data->pages as $p) {
            $page = new Reports_Model_Report_Page();
            $page->setReport($report)
                ->setTemplate($templates[$p->templateId])
                ->setOrder($index)
                ->setLandscape($p->landscape);

            $index++;
            $page->enableTitle($p->enableTitle);
            $title = new Reports_Model_Report_Page_Title();
            if ($page->isTitleEnabled()) {
                $title->setText($p->title)
                    ->setAlign($p->titleAlign)
                    ->setFontSize($p->titleSize);
            }
            $page->setTitle($title);

            $countDataSetsDb = new Core_Model_CountData_Sets();
            $aggregators = new Charts_Model_DataAggregatorList();

            $timefilters = new Core_Model_CountData_TimeFilters();

            //file_put_contents('/tmp/cells',print_r(array($p),true),FILE_APPEND);
            foreach ($p->cells as $c) {
                $type = $c->type;
                $className = "Reports_Model_Report_Page_Cell_" . ucfirst($type);

                $cell = new $className();
                $cell->enableTitle($c->enableTitle);

                $title = new Reports_Model_Report_Page_Cell_Title($c->title);
                $title->setCompany($company);
                $cell->setTitle($title);

                $cell->setTemplate($templateCells[(int)$c->templateRowCellId]);

                switch ($type) {
                case Reports_Model_Report_Page_Cell::TYPE_TABLE:
                    $cell->setTableId($c->tableId);
                    //$cell->setHasWeather($c->hasWeather);
                    //$cell->setWeatherEnabled($c->weatherEnabled);
                    //$cell->setWeatherLocation($c->weatherLocation);
                    //$cell->setWeatherTimePeriod($c->weatherTimePeriod);
                    break;
                case Reports_Model_Report_Page_Cell::TYPE_TEXT:
                    $cell->setText($c->text);
                    break;
                case Reports_Model_Report_Page_Cell::TYPE_WARNINGS:
                    break;
                case Reports_Model_Report_Page_Cell::TYPE_DATASOURCE:
                    $cell->setDataSourceId($c->dataSourceId);
                    $cell->setIncludeTable($c->includeTable);
                    $cell->setHasWeather($c->hasWeather);
                    if ( isset($c->weatherEnabled) ) $cell->setWeatherEnabled($c->weatherEnabled);
                    $cell->setWeatherLocation($c->weatherLocation);
                    $cell->setWeatherTimePeriod($c->weatherTimePeriod);
                    break;
                default:
                    throw new WebReport_Exception("Invalid cell type: " . $type);
                }
                $cell->setPage($page);
                $page->addCell($cell);
            }

            $pages[] = $page;
        }

        $report->setPages($pages);
        $reportMapper->save($report);
        Core_Model_DocumentFactory::add($report->getId(), "report", $this->_group);

        echo Zend_Json::encode(array("success" => true, "report_id" => $report->getId()));

    }


    public function previewReportAction()
    {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        $id = (int)$this->_getParam("id");

        $identity = Zend_Auth::getInstance()->getIdentity();

        $reportMapper = new Reports_Model_ReportMapper();
        $report = $reportMapper->find($id);

        if (!$report) {
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_REPORT_WITH_SUCH_ID_EXISTS.'));
        }
        $oldTimeLimit = (int) ini_get('max_execution_time');
        set_time_limit(300);

        $dateStr = $this->_getParam("date");

        $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();
        //$dateFormat = strtolower($dateFormat);
        //$dateFormat = str_replace("yyyy", "yy", $dateFormat);
        $dateFormat = str_replace("/","-", $dateFormat);

        if ($dateStr){
            $date = new Zend_Date($dateStr,$dateFormat);
            $now = new Zend_Date;
            $hour = $now->get(Zend_Date::HOUR);
            $minute = $now->get(Zend_Date::MINUTE);
            $second = $now->get(Zend_Date::SECOND);
            $date->set($hour,Zend_Date::HOUR)->set($minute,Zend_Date::MINUTE)->set($second,Zend_Date::SECOND);
        } else {
            $date = new Zend_Date();
        }
        //$date->sub('24:00:00', Zend_Date::TIMES);

        //If $date is null $date is set to now deeper in the report creation.
        $renderer = new Reports_Renderer_Pdf($report,$date);
        $filename = str_replace(' ', '_', $report->name);
        $this->_response->setHeader("Content-type", "application/pdf");
        $this->_response->setHeader("Content-Disposition", 'attachment; filename="' . $filename . '.pdf"');
        $this->view->dateformat = $dateFormat;
        echo $renderer->render();
        set_time_limit($oldTimeLimit);
        Core_Model_Generic_AbstractListView::GiveLoaderCookie($this->_request);
    }

    public function previewPageAction()
    {
        $config = Zend_Registry::get('config');

        // start with deleting old files
        $tmpFolder  = APPLICATION_PATH . "/../public/" . $config->general->publicTempFolder . "/";

        // Filetypes to check (you can also use *.*)
        $fileTypes = 'pdf-prev-*.pdf';
        $expireTime = 20; // in minutes

        // Find all files of the given file type
        foreach (glob($tmpFolder . $fileTypes) as $filename) {
            // Read file creation time
            $creationTime = filectime($filename);
            $fileAge = time() - $creationTime;

            // Is the file older than the given time span?
            if ($fileAge > ($expireTime * 60)){
                unlink($filename);
            }
        }

        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        // Create new PDF

        if (!$this->_request->isPost()) {
            echo "[]";
            exit;
        }


        $identity = Zend_Auth::getInstance()->getIdentity();
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($identity->company_id)->current();

        $data = stripslashes($this->_getParam("data"));
        $data  = Zend_Json::decode($data, Zend_Json::TYPE_OBJECT);


        $general = $data->general;
        $report = new Reports_Model_Report();
        $header = new Reports_Model_Report_Page_Header();
        $footer = new Reports_Model_Report_Page_Footer();
        $report->setCompany($company);

        // Report header
        $report->enablePageHeader($general->header->enabled);
        if (!$report->isPageHeaderEnabled()) {
            $header->reset();
        } else {
            $header->setText($general->header->text)
                ->setAlign($general->header->align);
        }
        $header->setCompany($company);
        $report->setPageHeader($header);

        // Report footer
        $report->enablePageFooter($general->footer->enabled);
        if (!$report->isPageFooterEnabled()) {
            $footer->reset();
        } else {
            $footer->setText($general->footer->text)
                ->setAlign($general->footer->align);
        }
        $footer->setCompany($company);
        $report->setPageFooter($footer);


        // store the template and template cells to avoid multiple database requests later
        $templateMapper = new Reports_Model_TemplateMapper();
        $p = $data->page;
        $template = $templateMapper->find($p->templateId);

        $page = new Reports_Model_Report_Page();

        $page->setReport($report)
            ->setTemplate($template)
            ->setOrder(0);
        $page->enableTitle($p->enableTitle);
        $title = new Reports_Model_Report_Page_Title();
        if ($page->isTitleEnabled()) {
            $title->setText($p->title)
                ->setAlign($p->titleAlign)
                ->setFontSize($p->titleSize);
        }
        $page->setTitle($title);

        $countDataSetsDb = new Core_Model_CountData_Sets();
        foreach ($p->cells as $c) {
            $type = $c->type;
            $className = "Reports_Model_Report_Page_Cell_" . ucfirst($type);

            $cell = new $className();
            $cell->enableTitle($c->enableTitle);

            $title = new Reports_Model_Report_Page_Cell_Title($c->title);
            $title->setCompany($company);
            $cell->setTitle($title);

            $cell->setTemplate($templateCells[(int)$c->templateRowCellId]);
            switch ($type) {
            case Reports_Model_Report_Page_Cell::TYPE_CHART:
            case Reports_Model_Report_Page_Cell::TYPE_TABLE:
                $countDataSet = $countDataSetsDb->find($c->countDataSetId)->current();
                $cell->setTimeInterval($c->dateTimeInterval)
                    ->setCountDataSet($countDataSet);

                if ($cell->getTimeInterval() == Reports_Model_Report_Page_Cell_CountData_Abstract::TIME_INTERVAL_FIX) {
                    // TODO:
                    // $c['fixedDateIntervalFrom'];
                    // $c['fixedDateIntervalTo'];
                }

                if ($type == Reports_Model_Report_Page_Cell::TYPE_CHART) {
                    $cell->setChartType($c->chartType);
                }

                break;
            case Reports_Model_Report_Page_Cell::TYPE_TEXT:
                $cell->setText($c->text);
                break;
            case Reports_Model_Report_Page_Cell::TYPE_WARNINGS:
                break;
            case Reports_Model_Report_Page_Cell::TYPE_DATASOURCE:
                $cell->setDataSourceId($c->data_source_id);
                $cell->setHasWeather($c->has_weather);
                $cell->setWeatherLocation($c->weather_location);
                $cell->setWeatherTimePerdio($c->weather_time_period);
                break;
            default:
                throw new WebReport_Exception("Invalid cell type: " . $type);
            }
            $cell->setPage($page);
            $page->addCell($cell);
        }

        $pages[] = $page;


        $report->setPages($pages);

        $folder = $config->general->publicTempFolder;

        $filename = "pdf-prev-" . md5(uniqid(mt_rand(), true)) . ".pdf";

        $success = true;

        try {
            $report->renderToPdf(realpath(APPLICATION_PATH . "/../public/" . $folder)  . "/" . $filename);
        } catch (Zend_Pdf_Exception $e) {
            $success = false;
        }

        echo Zend_Json::encode(array(
                                     'success' => $success,
                                     'filename' => $this->view->baseUrl($folder . "/" . $filename)
                                     ));

    }

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {
        $this->_translate->_("MOD_REPORTS_NAME");
        $this->_translate->_("MOD_REPORTS_MENU_EXPORT_DATA");
        $this->_translate->_("MOD_REPORTS_MENU_REPORTS");

        $this->_translate->_("PAGE_HEADER_ENABLED");
        $this->_translate->_("PAGE_FOOTER_ENABLED");
        $this->_translate->_("PAGE_FOOTER");
        $this->_translate->_("PAGE_HEADER");
        $this->_translate->_("ALIGN");
        $this->_translate->_("TEXT");
        $this->_translate->_("LEFT");
        $this->_translate->_("RIGHT");
        $this->_translate->_("CENTER");
        $this->_translate->_("BROWSE_REPORTS");
        $this->_translate->_("HANDLE_REPORTS");

        $this->_translate->_("ADD_REPORT");
        $this->_translate->_("BROWSE_REPORTS");

        $this->_translate->_("TITLE");
        $this->_translate->_("CELL_TYPE");
        $this->_translate->_("CHART");
        $this->_translate->_("TABLE");
        $this->_translate->_("SELECT_A_COUNT_DATA_SET.");


        $this->_translate->_("OK");
        $this->_translate->_("PAGE_SETTINGS");
        $this->_translate->_("ENABLE_PAGE_TITLE");
        $this->_translate->_("PAGE_TITLE");
        $this->_translate->_("SIZE");
        $this->_translate->_("SMALL");
        $this->_translate->_("MEDIUM");
        $this->_translate->_("LARGE");
        $this->_translate->_("LARGEST");

        $this->_translate->_("INCLUDE_TABLE");
        $this->_translate->_("SHOW_WEATHER");
        $this->_translate->_("LOCATION");

        $this->_translate->_("PERSONS_THAT_WILL_GET_THE_REPORT_BY_E-MAIL");

        $this->_translate->_("DAY_USE");
        $this->_translate->_("DATA_SOURCE");

        $this->_translate->_("CELL_TITLE");
        $this->_translate->_("PREVIEW");
        $this->_translate->_("SEND_TO_RECEIVERS");
        $this->_translate->_("THIS_SUBSCRIPTION_CONTAINS_A_REPORT_THAT_IS_NOT_VISIBLE_TO_YOU._EDITING_THE_SUBSCRIPTION_WILL_REMOVE_THE_REPORT_FROM_THE_SUBSCRIPTION_AND_THE_CHANGE_IS_IRREVERSIBLE.");
    }

}

