<?php

class Reports_SubscribeController extends WebReport_Controller_Action
{
    protected $_user;

    public function init() {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $usersDb = new Core_Model_Users();

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();

        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();       


        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse"), "text" => 'BROWSE_SUBSCRIPTIONS');

        if($this->_group && $this->_group->canEdit($this->_user)) {
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", "folder" => $this->_group->group_id), "text" => 'ADD_SUBSCRIPTION');
        }

        $items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->menu->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

        $this->view->headTitle("REPORT_SUBSCRIPTIONS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function browseAction() {
        $this->view->list = WebReport_Controller_Helper_Browse::getController(
                $this->_user, $this->_group, $this->_helper, $this->_request,
                'Reports_Model_Subscriptions',
                'Reports_Model_SubscriptionListGenerator'); 

        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function addAction() {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector("browse", "subscribe", "reports", array("folder" => $this->_group->group_id));
            return;
        }

        /*$this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/json2.js'));
          $this->view->headScript()->appendFile($this->view->baseUrl('/modules/reports/javascript/jquery/plugins/jquery.subscriptionEditor.js'));*/

        $this->view->headTitle("ADD", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $form = new Reports_Form_AddEditSubscription();
        $this->view->form = $form;

        if ($this->getRequest()->isPost()) {
            if ($form->isValid($_POST)) {
                $values = $form->getValues();

                $subscriptions = new Reports_Model_Subscriptions();

                $identity = Zend_Auth::getInstance()->getIdentity();

                $subscriptionData = array(
                        'name' => $values['name'],
                        'company_id' => $values['company_id'],
                        'report_id' => $values['report_id'],
                        'send_frequency' => $values['send_frequency'],
                        'message_body' => $values['message_body'],
                        'send_day' => $values['send_day'],
                        'send_weekday' => $values['send_weekday'],
                        'send_hour' => $values['send_hour'],
                        'send_days' => implode(',',$values['send_days']),
                        'timezone' => $values['timezone']
                        );

                /*if (!$userData->multi_company) {
                  $subscriptionData['company_id'] = $identity->company_id;
                  }*/

                $subscription = $subscriptions->createRow($subscriptionData);
                $subscription->save();
                Core_Model_DocumentFactory::add($subscription->subscription_id, "subscription", $this->_group);

                $subscriptionReceivers = new Reports_Model_SubscriptionReceivers();
                if(array_key_exists("acl_role_id", $values) && count($values['acl_role_id']) > 0) {
                    foreach($values['acl_role_id'] as $acl_id) {
                        $receiverData = array(
                                'subscription_id' => $subscription->subscription_id,
                                'acl_role_id' => $acl_id
                                );
                        $receiver = $subscriptionReceivers->createRow($receiverData);
                        $receiver->save();
                    }
                }

                foreach(explode("\n", $values['mail_text_area']) as $mail) {
                    $receiverData = array(
                            'subscription_id' => $subscription->subscription_id,
                            'mail' => $mail
                            );
                    $receiver = $subscriptionReceivers->createRow($receiverData);
                    $receiver->save();
                }

                $this->_flashMessenger->addMessage($this->_translate->_('SUBSCRIPTION_ADDED.'));
                $this->_helper->redirector('browse','subscribe','reports');
            } else {
                $form->populate($_POST);
            }
        }
        // get country of user and set timezone according to that. default: GB.
        if(Zend_Auth::getInstance()->getIdentity()->country)
            $country = Zend_Auth::getInstance()->getIdentity()->country;
        else
            $country = 'GB';

        $countryToTimezone = Zend_Registry::get('Zend_Locale')->getTranslationList('TimezoneToTerritory');
        $form->getElement('timezone')->setValue($countryToTimezone[$country]);

        $this->view->form = $form;
    }

    public function editAction() {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_SUBSCRIPTION.'));
            $this->_helper->redirector("browse", "subscribe", "reports", array("folder" => $this->_group->group_id));
            return;
        }

        $this->view->headScript()->appendFile($this->view->baseUrl('/library/javascript/json2.js'));
        $this->view->headScript()->appendFile($this->view->baseUrl('/modules/reports/javascript/jquery/plugins/jquery.subscriptionEditor.js'));

        $sub_id = (int)$this->_request->getParam('id');
        $subscriptionDb = new Reports_Model_Subscriptions();
        $subscription = $subscriptionDb->find($sub_id)->current();
        $subscriptionReceivers = new Reports_Model_SubscriptionReceivers();

        if (!$subscription)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_SUCH_SUBSCRIPTION_EXISTS.'));

        $reportDocument = Core_Model_DocumentFactory::fetch($subscription->report_id, "report");


        if($reportDocument && !$reportDocument->canView($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_SUBSCRIPTION_BECAUSE_IT_CONTAINS_A_REPORT_THAT_IS_NOT_VISIBLE_TO_YOU.')); 
            $this->_helper->redirector('browse', 'subscribe', 'reports');
        }

        if (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $subscription->company_id)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_SUCH_SUBSCRIPTION_EXISTS.'));

        $aclDb = new Core_Model_AclRoles();
        $acls = $aclDb->fetchAll($aclDb->select()->where("company_id = ?" ,(int) $subscription->company_id));

        $form = new Reports_Form_AddEditSubscription('edit', $acls, null);

        $this->view->form = $form;

        $this->view->headTitle(sprintf($this->_translate->_('EDIT_"%1$s"'), $subscription->name), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        if ($this->getRequest()->isPost()) {

            if ($form->isValid($_POST)) {
                $values = $form->getValues();

                $subscription->name = $values['name'];
                $subscription->company_id = $values['company_id'];
                $subscription->report_id = $values['report_id'];
                $subscription->send_frequency = $values['send_frequency'];
                $subscription->message_body = $values['message_body'];

                // time to send
                $subscription->send_day = $values['send_day'];
                $subscription->send_weekday = $values['send_weekday'];
                $subscription->send_hour = $values['send_hour'];

                // add weekdays
                if(isset($values['send_days']))
                    $subscription->send_days = implode(',',$values['send_days']);
                else
                    $subscription->send_days = NULL;

                // timezone
                $subscription->timezone = $values['timezone'];

                // an edit resets last_send
                if($values['unsend']) $subscription->last_send = null;

                $subscription->save();

                $receivers = $subscriptionReceivers->fetchAll($subscriptionReceivers->select()
                        ->where('subscription_id = ?', $sub_id));

                foreach($receivers as $reciver) {
                    $reciver->delete();
                }

                if(isset($values['acl_role_id'])) {
                    foreach($values['acl_role_id'] as $acl_id) {
                        $reciverData = array(
                                'subscription_id' => $subscription->subscription_id,
                                'acl_role_id' => $acl_id
                                );
                        $reciver = $subscriptionReceivers->createRow($reciverData);
                        $reciver->save();
                    }
                }
                foreach(explode("\n", $values['mail_text_area']) as $mail) {
                    $reciverData = array(
                            'subscription_id' => $subscription->subscription_id,
                            'mail' => $mail
                            );
                    $reciver = $subscriptionReceivers->createRow($reciverData);
                    $reciver->save();
                }

                $this->_flashMessenger->addMessage($this->_translate->_('SUBSCRIPTION_UPDATED.'));
                $this->_helper->redirector('browse','subscribe','reports');

            } else {
                $form->populate($_POST);

            }
        } else {
            //populate the rows
            $aclreceivers = array();
            $textreceivers = array();


            $receivers = $subscriptionReceivers->fetchAll($subscriptionReceivers->select()
                ->where('subscription_id = ?', $sub_id));

            foreach($receivers as $receiver) {
                if($receiver->acl_role_id){
                    $acl_id = $receiver->acl_role_id;
                    $userDb = Core_Model_Users::getInstance();
                    $select = $userDb->select()
                                     ->from($userDb,array("email"))
                                     ->where("acl_role_id = ?", $acl_id)
                                     ->where("active = 1");
                    $acl_receivers = $userDb ->fetchAll($select);
                    foreach($acl_receivers as $r){
                        $textreceivers[] = $r['email'];
                    }
                }else{
                    $textreceivers[] = $receiver->mail;
                }
            }

            $form->populate($subscription->toArray());
            $form->populate(array('acl_role_id' => $aclreceivers));
            $form->populate(array('mail_text_area' => implode("\n", $textreceivers)));

            // populate multicheckbox
            $form->populate(array('send_days' => explode(',', $subscription->send_days)));
        }


    }

    public function sendAction() {
        $sub_id = (int)$this->_request->getParam('id');
        $subscriptionDb = new Reports_Model_Subscriptions();
        $subscription = $subscriptionDb->find($sub_id)->current();

        if (!$subscription ||
                (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $subscription->company_id))
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_SUCH_SUBSCRIPTION_EXISTS.'));
        $succ = false;
        try {
            $subscription->send();
            $succ = true;
        } catch (Exception $e) {
            $this->_flashMessenger->addMessage($this->_translate->_('SUBSCRIPTION_SEND_FAIL:') .' '. $e->getMessage());
        }
        if($succ)
            $this->_flashMessenger->addMessage($this->_translate->_('SUBSCRIPTION_SENT.'));
        $this->_helper->redirector('browse','subscribe','reports', array("folder" => $this->_group->group_id));
    }

    public function removeAction() {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_REMOVE_THIS_SUBSCRIPTION.'));
            $this->_helper->redirector("browse", "subscribe", "reports", array("folder" => $this->_group->group_id));
            return;
        }

        $sub_id = (int)$this->_request->getParam('id');
        $subscriptionDb = new Reports_Model_Subscriptions();
        $subscription = $subscriptionDb->find($sub_id)->current();

        if (!$subscription ||
                (!Zend_Auth::getInstance()->getIdentity()->multi_company && Zend_Auth::getInstance()->getIdentity()->company_id != $subscription->company_id))
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_SUCH_SUBSCRIPTION_EXISTS.'));

        Core_Model_DocumentFactory::remove($subscription->subscription_id, "subscription");
        $subscription->delete();

        $this->_flashMessenger->addMessage($this->_translate->_('SUBSCRIPTION_REMOVED.'));
        $this->_helper->redirector('browse','subscribe','reports');
    }

    public function viewAction() {
        $sub_id = (int) $this->_request->getParam('id');
        $subscriptionDb = new Reports_Model_Subscriptions();
        $subscription = $subscriptionDb->find($sub_id)->current();

        $this->view->headTitle(sprintf($this->_translate->_('VIEW_"%1$s"'), $subscription->name),
                Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $this->view->subscription = $subscription;
    }

    public function translatableString(){
        $this->translate->_("EDIT_SUBSCRIPTION");
        $this->translate->_("SUBSCRIPTION_NAME");
        $this->translate->_("SELECT_A_REPORT.");
        $this->translate->_("A_SEND_FREQUENCY_MUST_BE_SPECIFIED");
        $this->translate->_("CHOOSE_DAY");
        $this->translate->_("CHOOSE_DAY_OF_WEEK");
        $this->translate->_("CHOOSE_TIME");
        $this->translate->_("CONTAINS_ONE_OR_MORE_INVALID_EMAIL_ADDRESSES");
        $this->translate->_("BROWSE_SUBSCRIPTIONS");
        $this->translate->_("NEVER");
        $this->translate->_("RECEIVER_ROLES");
    }

}
