<?php

class Reports_Form_AddEditSubscription extends UniForm_Form
{
    protected $action;
    protected $acls = null;

    public function __construct($action = 'add', $acls = null, $options = null)
    {
        $this->action = $action;
        $this->acls = $acls;
        parent::__construct($options);
    }

    public static function get_timezone_list()
    {
        $tz_list = array();

        static $regions = array('Africa' => DateTimeZone::AFRICA,
            'America' => DateTimeZone::AMERICA,
            'Antarctica' => DateTimeZone::ANTARCTICA,
            'Asia' => DateTimeZone::ASIA,
            'Atlantic' => DateTimeZone::ATLANTIC,
            'Europe' => DateTimeZone::EUROPE,
            'Indian' => DateTimeZone::INDIAN,
            'Pacific' => DateTimeZone::PACIFIC,
            'Australia' => DateTimeZone::AUSTRALIA);

        foreach($regions as $name => $mask)
            foreach(DateTimeZone::listIdentifiers($mask) as $tz)
                $tz_list[] = $tz;

        asort($tz_list);

        return $tz_list;
    }

    public static function GetTimeZoneSelect($element_id, $label) {
        $timezone = new UniForm_Form_Element_Select($element_id);
        $timezone->setLabel($label);

        $tz_list = self::get_timezone_list();

        foreach($tz_list as $tz) {
            $timezone->addMultiOption($tz, preg_replace("/\_/", " ", $tz));
        }

        return $timezone;
    }

    public function init() {
        parent::init();

        $receiversDb = new Reports_Model_SubscriptionReceivers();

        $name = new UniForm_Form_Element_Text('name');
        $name->setRequired()->addFilter('StringTrim')->setLabel('SUBSCRIPTION_NAME');

        $company_id = new UniForm_Form_Element_Select("company_id");

        $companiesDb = new Core_Model_Companies();
        $usersDb = new Core_Model_Users();

        $user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        if (!$user->multi_company) {
            $companiesDb = new Core_Model_Companies();
            $company = $companiesDb->find($user->company_id);
            $company_id = new UniForm_Form_Element_Hidden('company_id');
            $company_id->setValue($user->company_id);
        } else {

            $company_id = new UniForm_Form_Element_Select('company_id');
            $company_id->setLabel('COMPANY')
                ->addMultiOption(0, "SELECT_A_COMPANY.")
                ->setRequired()
                ->addValidator(new Zend_Validate_GreaterThan(0))
                ->setErrorMessages(array($this->getView()->translate("MUST_BELONG_TO_A_COMPANY.")));

            $companies = $companiesDb->fetchAllActive();
            foreach ($companies as $company) {
                $company_id->addMultiOption($company->company_id, $company->name);
            }
        }

        $report_id = new UniForm_Form_Element_Select('report_id');
        $report_id->setLabel('REPORT')
            ->addMultiOption(0,'SELECT_A_REPORT.')
            ->setRequired()
            ->addValidator(new Zend_Validate_GreaterThan(0))
            ->setErrorMessages(array($this->getView()->translate("SELECT_A_REPORT.")));

        $reportsMapper = new Reports_Model_ReportMapper();

        $reports = $reportsMapper->fetchAllVisible($user);
        foreach($reports as $report){
            if($report){
                $report_id->addMultiOption($report->getId(), $report->getName());
            }else{
                error_log("ReportsMapper returned list with report = null, in add edit subscriptions.");
            }
        }

        $mail_text_area = new UniForm_Form_Element_Textarea('mail_text_area');
        $mail_text_area_validator = new Reports_Form_Validate_Email();
        $mail_text_area_validator->setMessage("CONTAINS_ONE_OR_MORE_INVALID_EMAIL_ADDRESSES");
        $mail_text_area->setLabel("EMAIL_ADDRESSES:")
            ->setDescription("SPECIFY_RECEIVERS_BY_THEIR_EMAIL_ADDRESS,_ONE_PER_ROW.")
            ->addValidator($mail_text_area_validator);

        $send_frequency = new UniForm_Form_Element_Select('send_frequency');
        $send_frequency_validator = new Zend_Validate_InArray(array("w","d","y","m","never"));
        $send_frequency_validator->setMessage("A_SEND_FREQUENCY_MUST_BE_SPECIFIED");
        $send_frequency->setLabel('SEND_FREQUENCY')
            ->setDescription('HOW_OFTEN_SHOULD_THIS_SUBSCRIPTION_BE_SENT')
            ->setRequired()
            ->addMultiOption(0,'CHOOSE_FREQUENCY')
            ->addValidator($send_frequency_validator);

        $frequencies = Reports_Model_Subscription::getReadableFrequencies();

        foreach($frequencies as $key => $value) {
            $send_frequency->addMultiOption($key,$value);
        }

        $weekdays = array('mon' => 'MONDAY', 
            'tue' => 'TUESDAY', 
            'wed' => 'WEDNESDAY', 
            'thu' => 'THURSDAY', 
            'fri' => 'FRIDAY', 
            'sat' => 'SATURDAY', 
            'sun' => 'SUNDAY');

        $send_days = new UniForm_Form_Element_MultiCheckbox('send_days');
        $send_days->setLabel('SEND_DAYS');

        foreach($weekdays as $key => $value) {
            $send_days->addMultiOption($key, $value);
        }

        // populate multicheckbox
        if($this->action == 'add') {
            $send_days->setValue(array_keys($weekdays));
        }

        $send_day = new UniForm_Form_Element_Select('send_day');
        $send_day_validator = new Zend_Validate_InArray(range(1,31));
        $send_day_validator->setMessage("CHOOSE_DAY");
        $send_day->setLabel('SEND_DAY')
            ->addMultiOption(0, "CHOOSE_DAY_OF_MONTH.")
            ->addValidator($send_day_validator);

        foreach(range(1,30) as $day_of_month)
            $send_day->addMultiOption($day_of_month, "$day_of_month");
        $send_day->addMultiOption(31, "31 (" . $this->getView()->translate('LAST_DAY_OF_MONTH') . ')');

        $send_weekday = new UniForm_Form_Element_Select('send_weekday');
        $send_weekday_validator = new Zend_Validate_InArray(array_keys($weekdays));
        $send_weekday_validator->setMessage("CHOOSE_DAY_OF_WEEK");
        $send_weekday->setLabel('SEND_WEEKDAY')
            ->addMultiOption(0, 'CHOOSE_DAY_OF_WEEK.')
            ->addValidator($send_weekday_validator);

        foreach($weekdays as $key => $value)
            $send_weekday->addMultiOption($key, $value);

        $send_hour = new UniForm_Form_Element_Select('send_hour');
        $send_hour_validator = new Zend_Validate_InArray(range(0,23));
        $send_hour_validator->setMessage("CHOOSE_TIME");
        $send_hour->setLabel('SEND_HOUR')
            ->addMultiOption(24, 'CHOOSE_TIME.')
            ->addValidator($send_hour_validator);

        foreach(range(0,23) as $hour)
            $send_hour->addMultiOption($hour, sprintf("%02d:00", $hour));


        $timezone = self::GetTimeZoneSelect("timezone", _T("TIMEZONE"));

        $body = new UniForm_Form_Element_Textarea('message_body');
        $body->setLabel('MAIL_BODY_TEXT')
            ->setDescription('BODY_TEXT_TO_BE_INCLUDED_IN_EMAIL.');

        $submit = new UniForm_Form_Element_Submit('submit');

        if($this->action == 'add')
            $submit->setLabel(_T('ADD'));
        else {
            $submit->setLabel(_T('SAVE'));
        }
        $names = array('name','company_id','report_id','send_frequency', 'send_days', 'send_day', 'send_weekday', 'send_hour', 'timezone', 'message_body');
        $elements = array($name,$company_id,$report_id,$send_frequency,$send_days,$send_day,$send_weekday,$send_hour,$timezone,$body,$mail_text_area);

        if($this->action == 'edit') {
            $unsend = new UniForm_Form_Element_Checkbox('unsend');
            $unsend->setLabel(_T("MARK_AS_UNSENT."))
                ->setDescription(_T("CHECK_THIS_BOX_IF_YOU_WANT_THE_REPORT_LAST_SENT_DATE_TO_BE_SET_TO_NEVER."));
            $names[] = 'unsend';
            $elements[] = $unsend;
        }

        $this->addElements($elements);

        $this->addDisplayGroup($names,'subscription');
        $this->addDisplayGroup(array('mail_text_area'),'receivers');

        $this->addElements(array($submit));

    }

    private function _translateableStrings() {
        $this->_translate('SEND_DAY');
        $this->_translate('SEND_DAYS');
        $this->_translate('SEND_WEEKDAY');
        $this->_translate('SEND_HOUR');
        $this->_translate('TIMEZONE');
        $this->_translate('MAIL_BODY_TEXT');
        $this->_translate('EMAIL_ADDRESSES:');
        $this->_translate("SPECIFY_RECEIVERS_BY_THEIR_EMAIL_ADDRESS,_ONE_PER_ROW.");
        $this->_translate('BODY_TEXT_TO_BE_INCLUDED_IN_EMAIL.');
    }
}
