<?php

class Reports_Form_ReportAdd_General extends UniForm_Form
{
    protected $_company;

    /**
     * Constructor
     *
     * @see Zend_Form
     * @param mixed $options
     * @return void
     */
    public function __construct($company, $options = null)
    {

        $c = false;
        if ($company instanceof Core_Model_Company) {
            $c = $company;
        } else if (is_numeric($company)) {
            $companyDb = new Core_Model_Companies();
            $c = $companyDb->find((int)$company)->current();
        }

        if (!$c || !($c instanceof Core_Model_Company)) {
            throw new WebReport_Exception("Not valid company given");    
        }        
        $this->_company = $c;

        parent::__construct($options);
    }

    public function populate($data) {
        $aligns = array("left", "left", "center", "right");
        $data["logo_align"] = $aligns[$data["logo_enabled"]];
        $data["logo_enabled"] = $data["logo_enabled"] ? 1 : 0;
        parent::populate($data);
    }

    public function createAlignElements($base, $label, $validator = false) {
        $enabled = new UniForm_Form_Element_Checkbox($base."_enabled");
        $enabled->setLabel($label);

        $align = new UniForm_Form_Element_Radio($base."_align");
        $align
            ->setLabel("ALIGN")
            ->addMultiOption(Talking_Pdf_Page::TEXT_ALIGN_LEFT, "LEFT")
            ->addMultiOption(Talking_Pdf_Page::TEXT_ALIGN_CENTER, "CENTER")
            ->addMultiOption(Talking_Pdf_page::TEXT_ALIGN_RIGHT, "RIGHT")
            ->setValue(Talking_Pdf_Page::TEXT_ALIGN_CENTER);

        $text = new UniForm_Form_Element_Text($base."_text");
        $text->setLabel('TEXT')
            ->setAllowEmpty(false);
        if ( $validator ) {
            $text->addValidator($validator);
        }

        $this->addElements(array($enabled, $align, $text));
    }

    public function init()
    {
        parent::init();

        $this->addElementPrefixPath('Reports_Form_ReportAdd_General_Validate', 'reports/forms/ReportAdd/General/Validate', 'validate');

        $this->setAttrib("id", "general-form");

        $companiesDb = new Core_Model_Companies();

        $identity = Zend_Auth::getInstance()->getIdentity();
        if (!$identity->multi_company) {
            $company = $companiesDb->find($identity->company_id);
            $company_id = new UniForm_Form_Element_Hidden('company_id');
            $company_id->setValue($identity->company_id);
        } else {
            $company_id = new UniForm_Form_Element_Select('company_id');
            $company_id->setLabel('COMPANY')
                ->addMultiOption(0, "SELECT_A_COMPANY.")
                ->setValue($this->_company->company_id)
                ->setRequired()
                ->addValidator(new Zend_Validate_GreaterThan(0))
                ->setErrorMessages(array($this->getView()->translate("MUST_BELONG_TO_A_COMPANY.")));

            $companies = $companiesDb->fetchAll($companiesDb->select()->where('active = ?', 1)->order('name'));                                 
            foreach ($companies as $company) {
                $company_id->addMultiOption($company->company_id, $company->name);
            }
            if(isset($this->_company)) $company_id->setValue($this->_company->company_id);
        }                

        $name = new UniForm_Form_Element_Text('name');
        $name->setRequired()
            ->setLabel("NAME");

        $id = new UniForm_Form_Element_Hidden('id');
        $id->setRequired()
            ->setValue(0)
            ->addValidator('Digits');

        $description = new UniForm_Form_Element_Textarea('description');
        $description->setLabel("DESCRIPTION");

        $weather_enabled = new UniForm_Form_Element_Hidden('weather_enabled');
        $weather_enabled->setValue(Reports_Model_WeatherServiceAbstract::WeatherIsEnabled());

        $size = new UniForm_Form_Element_Select('size');
        $size->setLabel("SIZE");
        $size->addMultiOption(Zend_Pdf_Page::SIZE_A4, "A4")
            ->addMultiOption(Zend_Pdf_Page::SIZE_LETTER, "LETTER")
            ->setValue(Zend_Pdf_Page::SIZE_A4);
        $this->addElements(array($name, $description, $size, $id, $weather_enabled));
        $this->addDisplayGroup(array('name', 'description','size','id', 'weather_enabled'), 'general-group', array("legend" => "GENERAL_INFORMATION"));

        //Enable Company Logo
        /*
        if (!$company_id->getValue()) {
            $logo_link = "<a href='../../admin/companies/browse" . $company_id->getValue(). "'>Set Company logo</a>";
        } else {
            $logo_link = "<div name='logo_enabled'><a href='../../admin/companies/logo/id/" . $company_id->getValue(). "'>Set Company logo</a></div>";
        }
        */
        $this->addDisplayGroup(array('name', 'description','size','id', 'weather_enabled'), 'general-group', array("legend" => "GENERAL_INFORMATION"));

        $this->createAlignElements('logo', _T("SHOW_COMPANY_LOGO"));
        $this->removeElement("logo_text");
        $this->getElement("logo_enabled")
            ->setDescription(_T("IF_A_COMPANY_LOGO_IS_AVAILABLE_THIS_OPTION_WILL_SHOW_THE_LOGO_IN_THE_UPPER_LEFT_CORNER."));

        $this->createAlignElements('header', _T("PAGE_HEADER_ENABLED"), "HeaderEnabled");

        $this->addDisplayGroup(array('header_enabled', 'header_align', 'header_text','logo_enabled', 'logo_align'), 'header-group', array("legend" => "PAGE_HEADER", "class" => "clearfix inlineLabels"));

        $this->createAlignElements("footer", _T("PAGE_FOOTER_ENABLED"), "FooterEnabled");
        $identity = Zend_Auth::getInstance()->getIdentity();
        if($identity->multi_company) {
            $coglogo_enabled = new UniForm_Form_Element_Checkbox('coglogo_enabled');
            $coglogo_enabled->setLabel("COGNIMATICS_LOGO_ENABLED");
        } else 
            $coglogo_enabled = new UniForm_Form_Element_Hidden('coglogo_enabled');

        
        $this->addElements(array($coglogo_enabled));
        $this->addDisplayGroup(array('footer_enabled', 'footer_align', 'footer_text', 'coglogo_enabled'), 'footer-group', array("legend" => "PAGE_FOOTER", "class" => "clearfix inlineLabels"));
    }
}
