<?php

class Reports_Form_ReportAdd_TemplateCell extends UniForm_Form
{

	protected $_company;

	/**
	 * Constructor
	 *
	 * @see Zend_Form
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($company, $options = null)
	{
		$c = false;
		if ($company instanceof Core_Model_Company) {
			$c = $company;
		} else if (is_numeric($company)) {
			$companyDb = new Core_Model_Companies();
			$c = $companyDb->find((int)$company)->current();
		}

		if (!$c || !($c instanceof Core_Model_Company)) {
			throw new WebReport_Exception("Not valid company given");
		}

		$this->_company = $c;

		// pure lameness makes us put super constructor last, otherwise the init method won't get called in correct order
		parent::__construct($options);

	}

    public function isValid($data) {
        if ( $data["cell_type"] != "datasource"  ||
            $data["has_weather"] == 0 ) {
                $weather_location = $this->getElement("weather_location");
                $weather_location->removeValidator("NotEmpty");
            }
        return parent::isValid($data);
    }

	public function init()
	{
		parent::init();

		$this->addElementPrefixPath('Reports_Form_ReportAdd_TemplateCell_Validate', 'reports/forms/ReportAdd/TemplateCell/Validate', 'validate');

		$this->setAttrib("id", "edit-cell-form");

        $usersDb = new Core_Model_Users();
        $user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

		$company = $this->_company;

		$title_enable = new UniForm_Form_Element_Checkbox('cell_title_enable');
		$title_enable->setLabel('ENABLE_TITLE');

		$title = new UniForm_Form_Element_Text('cell_title');
		$title->setLabel("CELL_TITLE")
			->setAllowEmpty(false)
			->addValidator('TitleEnabled');


		$sets = $company->getCountDataSets();
		//$count_data_set_id = $sets->getFormElement('cell_count_data_set_id', "COUNT_DATA_SET", false, "SELECT_A_COUNT_DATA_SET.");
		//$count_data_set_id->addValidator("CountDataSetRequired");

		$timeFilterDb = new Core_Model_CountData_TimeFilters();
		$time_filter = $timeFilterDb->getFormElementForCompany('time_filter','TIME_FILTER',false,"NONE",$company);

		$type = new UniForm_Form_Element_Radio('cell_type');
    	$type->setLabel("CELL_TYPE");
		$type->addMultiOption(Reports_Model_Report_Page_Cell::TYPE_DATASOURCE, "CHART")
			->addMultiOption(Reports_Model_Report_Page_Cell::TYPE_TABLE, "TABLE")
			->addMultiOption(Reports_Model_Report_Page_Cell::TYPE_TEXT, "TEXT")
            ->addMultiOption(Reports_Model_Report_Page_Cell::TYPE_WARNINGS, "WARNINGS")
			->setValue(Reports_Model_Report_Page_Cell::TYPE_DATASOURCE);

        $data_source_id = new UniForm_Form_Element_Select('data_source_id');
        $data_source_id
            ->setLabel("CHART")
            ->addValidator('DataSourceRequired')
            ->addMultiOption(0,"NONE");
        $dataSourceListDb = new Charts_Model_ChartDataList();
        $dataSourceListSorted  = $dataSourceListDb->fetchAllVisibleSorted($user);

        $data_source_id->addMultiOptions($dataSourceListSorted);

        $include_table = new UniForm_Form_Element_Checkbox('include_table');
        $include_table->setLabel("INCLUDE_TABLE");

		$chart_type = new UniForm_Form_Element_Radio('cell_chart_type');
		//$chart_type->setLabel("CHART_TYPE");
		$chart_type
			->addMultiOption("line", "LINE_CHARTS")
			->addMultiOption("column", "COLUMN_CHART")
            ->addMultiOption("pie", "PIE_CHART")
			->setValue("line");

        $has_weather = new UniForm_Form_Element_Checkbox('has_weather');
        $has_weather->setLabel("SHOW_WEATHER");
        
        $weather_location = new UniForm_Form_Element_Text('weather_location');
        $weather_location->setLabel("LOCATION")
            ->setAllowEmpty(false)
            ->addValidator('NotEmpty');
        
        $weather_time_period = new UniForm_Form_Element_Select('weather_time_period');
        $weather_time_period->setLabel("TIME_PERIOD");

        //move this to factory somewhere
        $duration = array(
                          "w" => "WEEK",
                          "d" => "DAY");
        $period = array(
                        "0" => "CURRENT_%dd",
                        "-1" => "LAST_%dd",
                        "-2" => "TWO_%ddS_AGO",
                        "-3" => "THREE_%ddS_AGO",
                        "-4" => "FOUR_%ddS_AGO"
                        );

        $tDb = Charts_Model_Tables::getInstance();
        $tablesSorted  = $tDb->fetchAllVisibleSorted($user);
        $table_id = new UniForm_Form_Element_Select('table_id');
        $table_id
            ->setLabel(_T("TABLE"))
            ->addMultiOption(0,"NONE");

        $table_id->addMultiOptions($tablesSorted);

        $possible_time_codes = array();
        foreach($duration as $t => $dur) {
            $group = array();
            foreach($period as $p => $format) {
                $group[$p . $t] = str_replace("%dd", $dur, $format);
                $possible_time_codes[] = $p . $t;
            }
            $weather_time_period->addMultiOptions(array($dur."S" => $group));
        }

        

		//('y' for years, 'm' for months, 'w' for weeks, 'd' for days, e.g. '-1y -1m'), or null for no limit.
		//$time_interval = Core_Model_Factory_TimeInterval::makeSelect('cell_time_interval');

        //$manual_interval = new

		//$time_interval->addValidator("TimeInterval", false, array("fix","man"));

		/*$time_resolution = new UniForm_Form_Element_Select('cell_time_resolution');
          $time_resolution
          ->setLabel("TIME_RESOLUTION");*/
        //$manual_time_interval= new UniForm_Form_Element_Text('manual_time_interval');
        //$manual_time_interval
        //    ->setLabel("MANUAL_INTERVAL")
        //    ->addValidator("TimeInterval", true);
        //FIXME add validator

        //$date_interval = new UniForm_Form_Element_DateInterval('cell_date_interval');
		//$date_interval->setLabel("FIXED_DATE_INTERVAL")
		//	->addValidator("DateInterval");


		$text = new UniForm_Form_Element_Textarea('cell_text');
		$text->setLabel("TEXT");

		$this->addElements(array($title_enable, $title, $include_table, $time_filter, $type, $chart_type, $has_weather, $weather_location, $weather_time_period, $table_id, $text, $data_source_id/* , $time_resolution*/));
		$this->addDisplayGroup(array('cell_type', 'cell_title_enable','cell_title', 'cell_count_data_set_id', 'cell_chart_type', 'cell_time_interval', 'cell_date_interval', 'manual_time_interval','table_id', 'data_aggregator','time_filter', /*, 'cell_time_resolution',*/ 'cell_text', 'data_source_id', 'include_table', 'has_weather', 'weather_location', 'weather_time_period'), 'box-group');

		$submit = new UniForm_Form_Element_Submit('submit');
		$submit->setLabel("OK");
		$this->addElement($submit);


	}





}

