<?php

class Reports_Model_Report extends WebReport_Model_Abstract
{

	protected $_id = null;
	protected $_pages = array();
	protected $_company = null;
	protected $_receivers = array();
	protected $_name = "";
	protected $_description = "";
	protected $_size;
	
	protected $_pageHeaderEnabled = false;
	protected $_pageHeader = null;
	
	protected $_pageFooterEnabled = false;
	protected $_pageFooter = null;
    protected $_show_logo = false;
	protected $_coglogo_enabled = false;

    public function setId($id)
    {
        $this->_id = (int) $id;
        return $this;
    }

    public function getId()
    {
        return $this->_id;
    }

    public function showLogo($val = null) {
        if($val === null) return $this->_show_logo;
        $this->_show_logo = $val;
    }


	public function setCompany(Core_Model_Company $company)
	{
		$this->_company = $company;
        foreach($this->_pages as $page) {
            $page->setCompany($company);
        }
		return $this;
	}

	public function getCompany()
	{
		return $this->_company;
	}

	public function getPages()
	{
            return $this->_pages;
	}

        public function getPageCount() {
            return count($this->_pages);
        }

	public function setPages($pages)
	{
		$this->_pages = array();
		foreach ($pages as $page) {
			$this->addPage($page);
		}
		return $this;
	}

	public function addPage(Reports_Model_Report_Page $page)
	{
		$pageNumber = count($this->_pages)+1;
		$h = $this->getPageHeader();
		$f = $this->getPageFooter();
		$header = new Reports_Model_Report_Page_Header($h->getUnformattedText(), $h->getAlign(), $h->getFontSize(), $h->getCompany(), $this);
		$header->setPageNumber($pageNumber);
                $header->setReport($this);
		$footer = new Reports_Model_Report_Page_Footer($f->getUnformattedText(), $f->getAlign(), $f->getFontSize(), $f->getCompany(), $this);
		$footer->setPageNumber($pageNumber); 	
                $footer->setReport($this);
		$page->setHeader($header)
			->setFooter($footer);	
				
		$this->_pages[] = $page;
		return $this;
	}
	
	public function setName($name) 
	{
		$this->_name = $name;
		return $this;	
	}
	
	public function getName()
	{
		return $this->_name;	
	}
	
	
	public function setDescription($description) 
	{
		$this->_description = $description;
		return $this;	
	}
	
	public function getDescription()
	{
		return $this->_description;	
	}
	
	
	public function isPageHeaderEnabled()
	{
		return $this->_pageHeaderEnabled;	
	}
	
	public function enablePageHeader($enable = true)
	{
		$this->_pageHeaderEnabled = (bool)$enable;	
		return $this;
	}
	
	public function isPageFooterEnabled()
	{
		return $this->_pageFooterEnabled;	
	}

	public function setPageHeader(Reports_Model_Report_Page_Header $header)
	{
		$this->_pageHeader = $header;
		return $this;	
	} 	
	
	public function getPageHeader() {
        return $this->_pageHeader;	
	}
	
	public function enablePageFooter($enable = true)
	{
		$this->_pageFooterEnabled = $enable;	
		return $this;
	}


	public function setPageFooter(Reports_Model_Report_Page_Footer $footer)
	{
		$this->_pageFooter = $footer;
		return $this;	
	} 	
	
	public function getPageFooter()
	{
		return $this->_pageFooter;	
	}

	
	public function getReceivers()
	{
		return $this->_receivers;		
	}
	
	public function addReceiver(Core_Model_User $receiver)
	{
		$this->_receivers[] = $receiver;
		return $this;	
	}
	
	public function setReceivers($receivers)
	{
		$this->_receivers = $receivers;
		return $this;	
	}
	
	public function setSize($size)
	{
		$this->_size = $size;
		return $this;	
	}
	
	public function getSize()
	{
		return $this->_size;	
	}

	public function enableCogLogo($enabled = true)
	{
		$this->_coglogo_enabled = $enabled;
		return $this;
	}
	
	public function isCogLogoEnabled()
	{
		return $this->_coglogo_enabled;
	}

    public function usedInSubscription()
    {
        $subscriptionsDb = new Reports_Model_Subscriptions();

        $select = $subscriptionsDb->select()
            ->where('report_id = ?', $this->id);

        if(count($subscriptionsDb->fetchAll($select)) > 0)
            return TRUE;

        return FALSE;
    }

    public function createCopyOf($user) {
        $mapper = new Reports_Model_ReportMapper();
        $pageMapper = new Reports_Model_Report_PageMapper();
        $reportRow = $mapper->findAsRow($this->_id);
        $report = $mapper->find($this->_id);

        $db = $mapper->getDbTable();
        $new = $db->createRow();
        $new->setFromArray($reportRow->toArray());
        $new->report_id = NULL;
        $names = array();
        $visibles = $mapper->fetchAllVisible($user);
        foreach ( $visibles as $visible ) {
            $names[$visible->name] = 1;
        }
        $i = 0;
        $nameBase = preg_replace("/([a-z\b]*) \(copy( \d*)?\)/", "$1",  $new->name);
        while ( True ) {
            if ( $i == 0 ) $candidate = $nameBase . " (copy)";
            else $candidate = $nameBase . " (copy $i)";

            if ( !isset($names[$candidate]) ) break;

            $i += 1;
        }
        $new->name = $candidate;
        $new->save();
        $newReport = $mapper->find($new->report_id);
        $mapper->save($newReport);
        $pages = $report->getPages();
        foreach ( $pages as $idx => $page ) {
            $page->setReport($newReport);
            $page->setId(null);
            $page->setOrder($idx); // don't know why this is not perserved.
            foreach ( $page->getCells() as $cell ) {
                $cell->setId(null);
            }
            $pageMapper->save($page);
        }

        return $newReport;
    }
}
