<?php

class Reports_Model_Report_Page extends WebReport_Model_Abstract
{

	/**
	 * Align text of provided coordinates
	 */
	const TEXT_ALIGN_LEFT = 'left';
	const TEXT_ALIGN_RIGHT = 'right';
	const TEXT_ALIGN_CENTER = 'center';
	

	/**
	 * Font sizes
	 */
	const FONT_SIZE_SMALL = 'small';
	const FONT_SIZE_MEDIUM = 'medium';
	const FONT_SIZE_LARGE = 'large';


	protected $_id = null;
    protected $_landscape = false;
	protected $_cells = array();
	
	protected $_titleEnabled = false;
	protected $_title = null;
	protected $_order = null;
	
	protected $_template = null; 
	protected $_report = null;
	
	protected $_header;
	protected $_footer;


    public function setId($id)
    {
        $this->_id = $id === null ? null :(int) $id;
        return $this;
    }

    public function getId()
    {
        return $this->_id;
    }
    
    public function getLandscape(){
        return $this->_landscape;
    }
    
    public function setLandscape($bool){
        $this->_landscape = $bool;
        return $this;
    }

	public function enableTitle($enable = true) 
    {
        $this->_titleEnabled = (bool)$enable;
        return $this;
    }
    
	public function isTitleEnabled() 
	{
        return $this->_titleEnabled;
	}
   
	public function setTitle(Reports_Model_Report_Page_Title $title)
	{
		$this->_title = $title;
		return $this;	
	} 	
	
	public function getTitle()
	{
		return $this->_title;	
	}
	
	public function getCells()
	{
		return $this->_cells;
	}

    public function setCompany($company) {
        foreach($this->_cells as $cell) $cell->setCompany($company);
    }

	public function getCellWithTemplate(Reports_Model_Template_Row_Cell $t)
	{
		foreach ($this->getCells() as $c) {
			if ($c->getTemplate()->getId() == $t->getId()) {
				return $c;	
			}	
		}	
		return false;
	}

	public function setCells($cells)
	{
		$this->_cells = $cells;
		return $this;
	}

	public function addCell($cell)
	{
		$this->_cells[] = $cell;
		return $this;
	}
	

	public function setOrder($order)
	{
		$this->_order = (int)$order;
		return $this;	
	} 	
	
	public function getOrder()
	{
		return $this->_order;	
	}

	public function setTemplate(Reports_Model_Template $template)
	{
		$this->_template = $template;
		return $this;
	}
	
	public function getTemplate()
	{
		return $this->_template;
	}
	 
	 public function setReport(Reports_Model_Report $report)
	 {
	 	$this->_report = $report;
	 	return $this; 
	 } 
	 
	 public function getReport()
	 {
	 	return $this->_report;
	 }
	 
	 public function getHeader()
     {
	 	return $this->_header;
	 }
	 
	 public function setHeader(Reports_Model_Report_Page_Header $header)
	 {
	 	$this->_header = $header;
	 	return $this;
	 }

	 public function getFooter()
	 {
	 	return $this->_footer;
	 }
	 
	 public function setFooter(Reports_Model_Report_Page_Footer $footer)
	 {
	 	$this->_footer = $footer;
	 	return $this;
	 }
	 
	 public function toHtml()
	 {
	 	
    	$rows = $this->getTemplate()->getRows();
    	$s = '<table cellpadding="0" cellspacing="0" id="report-page-' . $this->getId() . '">';
    	foreach ($rows as $row) {
    		$cells = $row->getCells();
    		$s .= '<tr  style="height:' . $row->height . '%">';
    		foreach ($cells as $cell) {
				$c = $this->getCellWithTemplate($cell);
				$class = "";
				if (false === $c) {
					$class = "empty";
				} else {
					switch ($c->getType()) {
						case Reports_Model_Report_Page_Cell::TYPE_TEXT:
                                                case Reports_Model_Report_Page_Cell::TYPE_TABLE:
                                                case Reports_Model_Report_Page_Cell::TYPE_WARNINGS:
                                                case Reports_Model_Report_Page_Cell::TYPE_DATASOURCE:
							$class = $c->getType();
							break;
						case Reports_Model_Report_Page_Cell::TYPE_CHART:
							$class = $c->getChartType() . "-" . $c->getType();
                                                        break;
					}
				}
				$colspan = ($cell->getColspan() ? ' colspan="' . $cell->getColspan() . '"' : "");
				$rowspan = ($cell->getRowspan() ? ' rowspan="' . $cell->getRowspan() . '"' : "");
				$s .= '<td abbr="template-row-cell-' . $cell->getId() . '"' . $colspan . $rowspan . ' style="width: ' . $cell->getWidth() . '%" class="' . $class . '"></td>';
    		}
    		$s .= "</tr>";
    	}
    	$s .= "</table>";
    	return $s;
	 }


         public function toJson()
         {

             $cells = $this->getCells();
             $c = array();
             foreach ($cells as $cell) {
                 $a = array(
                     "id" => $cell->getId(),
                     "type" => $cell->getType(),
                     "enableTitle" => $cell->isTitleEnabled(),
                     "title" => $cell->getTitle()->getUnformattedText(),
                     "templateRowCellId" => $cell->getTemplate()->getId(),
                 );

                 switch ($cell->getType()) {
                     case Reports_Model_Report_Page_Cell::TYPE_TEXT:
                         $a["text"] = $cell->getText();
                         break;
                     case Reports_Model_Report_Page_Cell::TYPE_DATASOURCE:
                         $a["dataSourceId"] = $cell->getDataSourceId();
                         $a["includeTable"] = $cell->getIncludeTable();
                         $a["hasWeather"] = $cell->getHasWeather();
                         $a["weatherLocation"] = $cell->getWeatherLocation();
                         $a["weatherTimePeriod"] = $cell->getWeatherTimePeriod();
                         $a["weatherEnabled"] = Reports_Model_WeatherServiceAbstract::WeatherIsEnabled();
			break;
                     case Reports_Model_Report_Page_Cell::TYPE_TABLE:
                         $a["tableId"] = $cell->getTableId();
                         break;
                     case Reports_Model_Report_Page_Cell::TYPE_CHART:
                         $a["countDataSetId"] = $cell->getCountDataSet()->count_data_set_id;
                         $a["dateTimeInterval"] = $cell->getTimeInterval();
                         $a["aggregator"] = Charts_Model_DataAggregatorList::getAggregatorID($cell->getAggregator());
                         $a["manualTimeInterval"] = $cell->getManualTimeInterval();
                         $a["timeFilter"] = !is_null($cell->getTimeFilter()) ? $cell->getTimeFilter()->filter_id : null;
                         //$a["fixedDateIntervalfrom"] = $cell->fixed
                         if ($cell->getType() === Reports_Model_Report_Page_Cell::TYPE_CHART) {
                             $a["chartType"] = $cell->getChartType();
                         }
                         break;
                 }

                 $c[] = $a;
             }
             return Zend_Json::encode(array(
                 "id" => $this->getId(),
                 "enableTitle" => $this->isTitleEnabled(),
                 "title" => $this->getTitle()->getUnformattedText(),
                 "titleAlign" => $this->getTitle()->getAlign(),
                 "titleSize" => $this->getTitle()->getFontSize(),
                 "templateId" => $this->getTemplate()->getId(),
                 "landscape" => $this->getLandscape(),
                 "cells" => $c,
             ));
         }

}
