<?php

abstract class Reports_Model_Report_Page_Cell_Abstract extends WebReport_Model_Abstract
{

	const TYPE_CHART = 'chart';
	const TYPE_TABLE = 'table';
	const TYPE_TEXT = 'text';
        const TYPE_DATASOURCE = 'datasource';
        const TYPE_WARNINGS = 'warnings';

	const TIME_INTERVAL_FIX = 'fix';
        const TIME_INTERVAL_MANUAL = 'man';

	protected $_id;

	/**
	 * Cell parent page.
	 *
	 * @var Reports_Model_Report_Page
	 */
	protected $_page;

	/**
	 * Cell template.
	 *
	 * @var Reports_Model_Template_Row_Cell
	 */
	protected $_cellTemplate = null;

	/**
	 * Cells title.
	 *
	 * @var Reports_Model_Report_Page_Cell_Title
	 */
	protected $_title = null;
	protected $_titleEnabled = false;


	public function setId($id)
	{
		$this->_id = $id;	
		return $this;
	}	
	
	public function getId()
	{
		return $this->_id;
	}

	/**
	 * Setter for cell title.
	 *
	 * @param Reports_Model_Report_Page_Cell_Title $title
	 */
	public function setTitle(Reports_Model_Report_Page_Cell_Title $title)
	{
		$this->_title = $title;
		return $this;
	}

	/**
	 * Getter for cells title.
	 *
	 * @return Reports_Model_Report_Page_Cell_Title
	 */
	public function getTitle()
	{
		return $this->_title;
	}


	/**
	 * Getter for cell type
	 *
	 * @return string;
	 */
	public function getType() {
		$name = get_class($this);
		$pos = strrchr($name, "_");
		if ($pos === false) {
			throw new WebReport_Model_Exception("Invalid class: " . get_class($this));
		}
		return strtolower(substr($pos, 1));
	}


    public function setCompany($company) {
        return $this;
    }

	/**
	 * Setter for cell template.
	 *
	 * @param Reports_Model_Template_Row_Cell $template
	 */
	public function setTemplate(Reports_Model_Template_Row_Cell $template) {
		$this->_cellTemplate = $template;
	}

	/**
	 * Getter for cell template.
	 *
	 * @return Reports_Model_Template_Row_Cell
	 */
	public function getTemplate()
	{
		return $this->_cellTemplate;
	}

	/**
	 * Set parent page.
	 *
	 * @param Reports_Model_Report_Page $page
	 */
	public function setPage(Reports_Model_Report_Page $page)
	{
		$this->_page = $page;
		return $this;
	}

	public function getPage()
	{
		return $this->_page;
	}



	public function enableTitle($enable = true) 
        {
        $this->_titleEnabled = $enable;
        return $this;
        }
    
	public function isTitleEnabled() 
	{
        return $this->_titleEnabled;
	}
   
}

