<?php

abstract class Reports_Model_Report_Page_Cell_CountData_Abstract extends Reports_Model_Report_Page_Cell_Abstract
{

	const TIME_INTERVAL_FIX = 'fix';
        const TIME_INTERVAL_MANUAL = 'man';

	/**
	 * Cell count data set
	 *
	 * @var Core_Model_CountData_Set
	 */
	protected $_countDataSet = null;

	protected $_timeInterval;
	protected $_resolution;

        protected $_aggregator = null;

        protected $_manualTimeInterval = null;

        protected $_dataunit = null;

	protected $_timeFilter = null;

        protected $_fixed_time_from = null;
        protected $_fixed_time_to = null;

        public function setAggregator(Charts_Model_DataAggregatorAbstract $aggregator=null)
        {
            if(isset($aggregator))
                $this->_aggregator = $aggregator;
            return $this;
        }

        public function setFixedTimeInterval($from,$to) {
            $this->_fixed_time_from = $from;
            $this->_fixed_time_to = $to;
            return $this;
        }

        public function getFixedTimeIntervalFrom() {
            return $this->_fixed_time_from;
        }

        public function getFixedTimeIntervalTo() {
            return $this->_fixed_time_to;
        }


        public function setManualTimeInterval($value)
        {
            $this->_manualTimeInterval = $value;
            return $this;
        }


        public function getAggregator() {
            return $this->_aggregator;
        }

        public function hasAggregator() {
            return !is_null($this->_aggregator);
        }

        public function setTimeFilter(Core_Model_CountData_TimeFilter $timeFilter=null)
        {
            $this->_timeFilter = $timeFilter;
            return $this;
        }

	public function getTimeFilter() {
            return $this->_timeFilter;
        }

        public function hasTimeFilter() {
            return !is_null($this->_timeFilter);
        }

	/**
	 * Setter for cells count data set
	 *
	 * @param Core_Model_CountData_Set $set
	 */
	public function setCountDataSet(Core_Model_CountData_Set $set)
	{
		$this->_countDataSet = $set;
		return $this;
	}

	/**
	 * Getter for cells count data set
	 *
	 * @return Core_Model_CountData_Set
	 */
	public function getCountDataSet()
	{
		return $this->_countDataSet;
	}

	public function setTimeInterval($interval)
	{
		$this->_timeInterval = $interval;
		return $this;
	}

        public function getManualTimeInterval() {
            return $this->_manualTimeInterval;
        }

	public function getTimeInterval()
	{
            return $this->_timeInterval;
	}

        private $_firstDate = null;
        private $_lastDate = null;


        private function _compute_dates(Zend_Date $base = null)
        {
            $intervalCode = "";
            if($this->_timeInterval == Reports_Model_Report_Page_Cell_CountData_Abstract::TIME_INTERVAL_FIX) {
                $this->_firstDate = new Zend_Date($this->_fixed_time_from);
                $this->_lastDate = new Zend_Date($this->_fixed_time_to);
                return true;
            } else if($this->_timeInterval == Reports_Model_Report_Page_Cell_CountData_Abstract::TIME_INTERVAL_MANUAL) {
                $intervalCode = $this->getManualTimeInterval();
            } else {
                $intervalCode = $this->getTimeInterval();
            }

            if(is_null($base))
                $base = new Zend_Date();

            $interval = new CM_Interval($base, $intervalCode);
            $this->_firstDate = $interval->getStartDate();
            $this->_lastDate = $interval->getStopDate();

            //fixme: should not be in final release
            return true;
        }

        protected $_data = null;
        protected $_xlabel = null;
        protected $_dataLabels = null;

        public function getPrettyResolution(Zend_Translate $t){
            switch($this->_aggregator->getResolution()){
                case 'ts1q':
                    return $t->_("QUARTER");
                case 'ts1m':
                    return $t->_("MONTH");
                case 'ts1w':
                    return $t->_("WEEK");
                case 'ts1d':
                    return $t->_("DAY");
                case 'ts1h':
                    return $t->_("HOUR");
                case 'ts30min':
                    return $t->_("HALF_HOUR");
                return $t->_("UNKNOWN_UNIT");

            }
        }

        public function renderData() {
            $usingUnit = false;
            if(!$this->_aggregator) {
                $this->_aggregator = new Charts_Model_DataAggregator_Unit();
                $usingUnit = true;
            }

            $this->_compute_dates();

            $this->_aggregator->setData($this->_countDataSet);
            $this->_aggregator->setPeriod($this->_firstDate, $this->_lastDate);
            if($this->_timeFilter != null){
                $this->_aggregator->setTimeFilter($this->_timeFilter);
            }
            $this->_data = $this->_aggregator->renderData();
            $this->_xlabel = $this->_aggregator->getXLabel();
            $this->_dataLabels = $this->_aggregator->getDataLabels();
            $this->_dataunit = $this->_aggregator->getDisplayUnit();
            $this->_resolution = $this->_aggregator->getResolution();

            $translate = Zend_Registry::get('Zend_Translate');

            $this->_dataunit = $translate->_($this->_dataunit);

            foreach($this->_xlabel as  $key => $value){
                $this->_xlabel[$key] = $translate->_($value);
            }
            $days = $this->_lastDate->sub($this->_firstDate);
            $days = $days->get(Zend_Date::DAY);
            

            return array(
                'data' => $this->_data,
                'xlabel' => $this->_xlabel,
                'dataLabels' => $this->_dataLabels,
                'unit' => $this->_dataunit,
                'res' => $this->_resolution,
                'isUnit' => $usingUnit,
                'days' => (int)$days,
                'ylabel' => $translate->_('COUNT').($usingUnit ? "/".$this->getPrettyResolution($translate) : ""),
                'xtitle' => $translate->_('TIME')                
            );
        }
}

