<?php

class Reports_Model_Report_Page_CellMapper extends WebReport_Model_Mapper_Abstract
{
	
	protected $_dbTableClass = "Reports_Model_DbTable_Report_Page_Cell";


    public function save($cell)
    {
    	if (!($cell instanceof Reports_Model_Report_Page_Cell_Abstract)) {
    		throw new WebReport_Model_Mapper_Exception("Cell must be instance of Reports_Model_Report_Page_Cell_Abstract");
    	}
 
        $data = array(
            'report_page_id' 		=> $cell->getPage()->getId(),
            'template_row_cell_id' 	=> $cell->getTemplate()->getId(),
            'type'			=> $cell->getType(),
            'title_enabled'		=> $cell->isTitleEnabled(),
            'title'			=> $cell->getTitle()->getUnformattedText()
        );

        $type = $cell->getType();
        switch ($type) {
            case Reports_Model_Report_Page_Cell::TYPE_TEXT:
                $data['text'] = $cell->getText();
                break;
            case Reports_Model_Report_Page_Cell::TYPE_WARNINGS:
                break;
            case Reports_Model_Report_Page_Cell::TYPE_DATASOURCE:
                $data['data_source_id'] = $cell->getDataSourceId();
                $data['include_table'] = $cell->getIncludeTable();
                $data['has_weather'] = $cell->getHasWeather();
                $data['weather_location'] = $cell->getWeatherLocation();
                $data['weather_time_period'] = $cell->getWeatherTimePeriod();
                break;
            case Reports_Model_Report_Page_Cell::TYPE_TABLE:
                $data['table_id'] = $cell->getTableId();
                break;

            default:
                throw new WebReport_Exception("Invalid cell type: " . $type);
        }




       if (null === ($id = $cell->getId())) {
            $id = $this->getDbTable()->insert($data);
            $cell->setId($id);
        } else {
            $this->getDbTable()->update($data, array('report_page_cell_id = ?' => $id));
        }
    }

	public function fetchAllForPage(Reports_Model_Report_Page $page)
	{
		$cells = $this->fetchAll(array("where" => "report_page_id = ". (int)$page->getId(), "template" => $page->getTemplate()));
		
		foreach ($cells as $cell) {
			$cell->setPage($page);
		}
		return $cells;
	}

	protected function _populateFromRow($row, $deepPopulate = true, $params = array()) 
	{
		switch($row->type) {
			case Reports_Model_Report_Page_Cell_Abstract::TYPE_TABLE:
				$entry = new Reports_Model_Report_Page_Cell_Table();
				break;
			case Reports_Model_Report_Page_Cell_Abstract::TYPE_TEXT:
				$entry = new Reports_Model_Report_Page_Cell_Text();
				break;
			case Reports_Model_Report_Page_Cell_Abstract::TYPE_WARNINGS:
				$entry = new Reports_Model_Report_Page_Cell_Warnings();
                break;
            case Reports_Model_Report_Page_Cell_Abstract::TYPE_DATASOURCE:
                $entry = new Reports_Model_Report_Page_Cell_Datasource();
                break;
            default:
				$entry = new Reports_Model_Report_Page_Cell_Text();
                $entry->setText("This cell could not properly be loaded. It might be because it contains a chart with old format.");
				break;	
		}
		$title = new Reports_Model_Report_Page_Cell_Title($row->title);
		$entry
			->setId($row->report_page_cell_id)
			->setTitle($title)
			->enableTitle((bool)$row->title_enabled);
			
		if (isset($params["template"])) {
			$template = $params["template"];
			$cellTemplate = $template->getRowCellWithId($row->template_row_cell_id);
			$entry->setTemplate($cellTemplate);	
		}

		switch($row->type) {
		    case Reports_Model_Report_Page_Cell_Abstract::TYPE_TEXT:
				$entry->setText($row->text);
				break;
            case Reports_Model_Report_Page_Cell_Abstract::TYPE_WARNINGS:
                $identity = Zend_Auth::getInstance()->getIdentity();
              	$companiesDb = new Core_Model_Companies();
                $company = $companiesDb->find($identity->company_id)->current();
                $entry->setCompany($company);
                break;
            case Reports_Model_Report_Page_Cell_Abstract::TYPE_DATASOURCE:
                $entry->setDataSourceId($row->data_source_id);
                $entry->setHasWeather((bool)$row->has_weather);
                $entry->setIncludeTable((bool)$row->include_table);
                $entry->setWeatherEnabled($a["weatherEnabled"] = Reports_Model_WeatherServiceAbstract::WeatherIsEnabled());
                $entry->setWeatherLocation($row->weather_location);
                $entry->setWeatherTimePeriod($row->weather_time_period);
                break;
            case Reports_Model_Report_Page_Cell_Abstract::TYPE_TABLE:
                $entry->setTableid($row->table_id);
                break;
		}

      return $entry;
	}

}

