<?php

class Reports_Model_Report_Page_Label
{
	
	/**
	 * Count data set of title
	 *
	 * @var Core_Model_CountData_set
	 */
	protected $_countDataSet;

	/**
	 * Company of the title
	 *
	 * @var Core_Model_Company
	 */
	protected $_company;

	/**
	 * Text align
	 *
	 * @var string
	 */
	protected $_align;



	/**
	 * Unformatted text
	 *
	 * @var string
	 */
	protected $_unformattedText = '';

	/**
	 * Font size
	 *
	 * @var int
	 */
	protected $_fontSize;

        protected $_report;

	/**
	 * Constructor for title.
	 *
	 * @param Core_Model_Company $company
	 * @param Core_Model_CountData_Set $countDataSet
	 */
	public function __construct($text = '',
                $align = Reports_Model_Report_Page::TEXT_ALIGN_LEFT,
                $fontSize = Reports_Model_Report_Page::FONT_SIZE_MEDIUM,
                Core_Model_Company $company = null)
	{
		$this
			->setText($text)
			->setAlign($align)
			->setFontSize($fontSize);
			
                //$this->_report = $report;
		if ($company) {
			$this->setCompany($company);
		}

	}

	/*
	 * Reset to defaults.
	 */
	public function reset()
	{
		$this
			->setText("")
			->setAlign(Reports_Model_Report_Page::TEXT_ALIGN_LEFT)
			->setFontSize(Reports_Model_Report_Page::FONT_SIZE_MEDIUM)
			->setCompany(null);
	}

	/**
	 * Set title company
	 *
	 * @param Core_Model_Company $company
	 */
	public function setCompany($company)
	{
		if (($company instanceof Core_Model_Company) || is_null($company)) {
			$this->_company = $company;
		} else {
			throw new WebReport_Model_Exception('$company must be an instance of Core_Model_Company, ' . get_class($company). ' given');	
		}
		return $this;

	}

	/**
	 * Getter for titles company.
	 *
	 * @return Core_Model_Company
	 */
	public function getCompany()
	{
		if (is_null($this->_company)) {
			throw new WebReport_Exception("No company set");
		}
		return $this->_company;
	}

	/**
	 * Setter for the count data set
	 *
	 * @param Core_Model_CountData_Set $countDataSet
	 */
	public function setCountDataSet($countDataSet)
	{
		if (($countDataSet instanceof Core_Model_CountData_Set) || is_null($counDataSet)) {
			$this->_countDataSet = $countDataSet;
		} else {
			throw new WebReport_Model_Exception('$countDataSet must be an instance of Core_Model_CountData_Set');	
		}
		return $this;
	}

	/**
	 * Getter for titles count data set.
	 *
	 * @return Core_Model_CountData_Set
	 */
	public function getCountDataSet()
	{
		if (!$this->_countDataSet) {
			throw new WebReport_Exception("No count data set set");
		}

		return $this->_countDataSet;
	}

	/**
	 * Setter for (unformatted) text.
	 */
	public function setText($text)
	{
		$this->_unformattedText = $text;
		return $this;
	}

	/**
	 * returns formatted text
	 *
	 * @return string
	 */
	public function getText()
	{
		return $this->_transform();
	}

	/**
	 * Getter for unformatted text
	 *
	 * @return string
	 */
	public function getUnformattedText()
	{
		return $this->_unformattedText;
	}


	/**
	 * Setter for title align
	 *
	 * @param string $align
	 */
	public function setAlign($align)
	{
		$this->_align = $align;
		return $this;
	}
	
	/**
	 * Getter for title align
	 * 
	 * @return string
	 */
	public function getAlign()
	{
		return $this->_align;
	}

	/**
	 * Setter for font size
	 * 
	 * @param int $size
	 * @return <type> 
	 */
	public function setFontSize($size)
	{
		$this->_fontSize = $size;
		return $this;
	}

	/**
	 * Getter for font size.
	 * 
	 * @return int
	 */
	public function getFontSize()
	{
		return $this->_fontSize;
	}


	protected function _transform()
	{
		$fText = $this->getUnformattedText();

		if (strpos($fText, "%cn")!==FALSE) {
			$companyName = $this->getCompany()->name;
			$fText = str_replace("%cn", $companyName, $fText);
		}

		$date = new Zend_Date();
		$fText = str_replace("%yy", $date->get(Zend_Date::YEAR_SHORT), $fText);
		$fText = str_replace("%y", $date->get(Zend_Date::YEAR), $fText);
		$fText = str_replace("%MMMMM", $date->get(Zend_Date::MONTH_NAME_NARROW), $fText);
		$fText = str_replace("%MMMM", $date->get(Zend_Date::MONTH_NAME), $fText);
		$fText = str_replace("%MMM", $date->get(Zend_Date::MONTH_NAME_SHORT), $fText);
		$fText = str_replace("%MM", $date->get(Zend_Date::MONTH), $fText);
		$fText = str_replace("%M", $date->get(Zend_Date::MONTH_SHORT), $fText);
		$fText = str_replace("%SS", $date->get(Zend_Date::DAY_SUFFIX), $fText);
		$fText = str_replace("%dd", $date->get(Zend_Date::DAY), $fText);

		$fText = str_replace("%FFFFF", $date->get(Zend_Date::DATE_FULL), $fText);
		$fText = str_replace("%FFFF", $date->get(Zend_Date::DATE_LONG), $fText);
		$fText = str_replace("%FFF", $date->get(Zend_Date::DATE_MEDIUM), $fText);
		$fText = str_replace("%FF", $date->get(Zend_Date::DATE_SHORT), $fText);
		$fText = str_replace("%F", $date->get(Zend_Date::DATES), $fText);

		$fText = str_replace("%c", $date->get(Zend_Date::ISO_8601), $fText);

		$fText = str_replace("%ww", $date->get(Zend_Date::WEEK), $fText);
		$fText = str_replace("%EEEE", $date->get(Zend_Date::WEEKDAY), $fText);
		$fText = str_replace("%EEE", $date->get(Zend_Date::WEEKDAY_SHORT), $fText);
		$fText = str_replace("%EE", $date->get(Zend_Date::WEEKDAY_NAME), $fText);
		$fText = str_replace("%E", $date->get(Zend_Date::WEEKDAY_NARROW), $fText);
		$fText = str_replace("%ww", $date->get(Zend_Date::WEEK), $fText);
		$fText = str_replace("%e", $date->get(Zend_Date::WEEKDAY_DIGIT), $fText);


		$fText = str_replace("%mm", $date->get(Zend_Date::MINUTE), $fText);
		$fText = str_replace("%m", $date->get(Zend_Date::MINUTE_SHORT), $fText);
		$fText = str_replace("%HH", $date->get(Zend_Date::HOUR), $fText);
		$fText = str_replace("%H", $date->get(Zend_Date::HOUR_SHORT), $fText);
		$fText = str_replace("%hh", $date->get(Zend_Date::HOUR_AM), $fText);
		$fText = str_replace("%h", $date->get(Zend_Date::HOUR_SHORT_AM), $fText);

		$fText = str_replace("%ss", $date->get(Zend_Date::SECOND), $fText);
		$fText = str_replace("%s", $date->get(Zend_Date::SECOND_SHORT), $fText);

		$fText = str_replace("%TTTTT", $date->get(Zend_Date::TIME_FULL), $fText);
		$fText = str_replace("%TTTT", $date->get(Zend_Date::TIME_LONG), $fText);
		$fText = str_replace("%TTT", $date->get(Zend_Date::TIME_MEDIUM), $fText);
		$fText = str_replace("%TT", $date->get(Zend_Date::TIME_SHORT), $fText);

		return $fText;

	}

}
