<?php

class Reports_Model_ReportMapper extends WebReport_Model_Mapper_Abstract
{
    protected $_dbTableClass = "Reports_Model_DbTable_Report";

    public function save(Reports_Model_Report $report, $recursive = true)
    {
        $footer = $report->getPageFooter();
        $header = $report->getPageHeader();
       
        $data = array(
            'name'            => $report->getName(),
            'description'    => $report->getDescription(),
              'company_id'    => $report->getCompany()->company_id,
              'size'            => $report->getSize(),
              'header_enabled' => (int)$report->isPageHeaderEnabled(),
              'header_text' => $header->getUnformattedText(),
              'header_align' => $header->getAlign(),
              'footer_enabled' => (int)$report->isPageFooterEnabled(),
              'footer_text' => $footer->getUnformattedText(),
              'footer_align' => $footer->getAlign(),
              'show_logo' => $report->showLogo(),
              'coglogo_enabled' => (int)$report->isCogLogoEnabled()
        );

        if (null === ($id = $report->getId())) {
            $id = $this->getDbTable()->insert($data);
            $report->setId($id);
        } else {
           $this->getDbTable()->update($data, array('report_id = ?' => $id));
        }
        
      if ($recursive) {
          $pages = $report->getPages();
          $pageMapper = new Reports_Model_Report_PageMapper();
          foreach ($pages as $page) {
              $pageMapper->save($page);
          }            
       }
    }

    

    protected function _populateFromRow($row, $deepPopulate = true, $params = array()) {
        $entry = new Reports_Model_Report();
              $entry->setId($row->report_id)
                ->setName($row->name)
                ->setDescription($row->description)
                ->enablePageHeader((bool)$row->header_enabled)
                ->enablePageFooter((bool)$row->footer_enabled)
                ->setSize($row->size)
                ->enableCogLogo((bool)$row->coglogo_enabled);

        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($row->company_id)->current();
        $entry->setCompany($company);
        
        $header = new Reports_Model_Report_Page_Header($row->header_text, $row->header_align);
        $header->setCompany($company);

        $footer = new Reports_Model_Report_Page_Footer($row->footer_text, $row->footer_align);
        $footer->setCompany($company);
        
        $entry->setPageHeader($header)
            ->setPageFooter($footer);

        $entry->showLogo($row->show_logo);
        
        
        if ($deepPopulate) {
            $pageMapper = new Reports_Model_Report_PageMapper();
            $pages = $pageMapper->fetchAllForReport($entry);
            $entry->setPages($pages);
        } 
 
      return $entry;
    }

    public function fetchAllVisible($user)
    {
        $groupsDb = new Core_Model_Folders();

        $groups = $groupsDb->getViewableGroups($user);

        $reports = array();

        foreach($groups as $group)
            $reports = array_merge($reports, $this->fetchReports($group));

        return $reports;
    }

    public function fetchReports($group) {
        $documents = $group->getDocuments('report');

        $reports = array();
        foreach($documents as $doc)
            $reports[] = $this->find($doc->id);

        return $reports;
    }

    public function fetchDocuments($group) {
        $documents = $group->getDocuments('report');
        $reportsDb = new Reports_Model_DbTable_Report();

        $reports = array();
        foreach($documents as $doc) {

            $report = $reportsDb->find($doc->id)->current();

            if($report)
                $reports[] = $report;
        }    

        return $reports;
    }
}

