<?php

class Reports_Model_Subscription extends Zend_Db_Table_Row_Abstract
{
    // left to void for now.. will need to send and crap later.
    protected $_constructTime;

    public static function getReadableFrequencies() {
        return array("m"=>_T("EVERY_MONTH"), 
        "w"=>_T("EVERY_WEEK"), 
        "d"=>_T("EVERY_DAY"), 
        "never" => _T("NEVER"));
    }

    public function init(){
        $this->_constructTime = new Zend_Date();
        $this->_constructTime->setTimezone($this->timezone);
    }

    public function getLastSent(){
        if(!$this->last_send || substr($this->last_send,0,4) == "1970") return "never";
        return $this->last_send . " (" . $this->timezone . ")";
    }

    public function getTimeZone() {
        return str_replace('_', ' ', $this->timezone);
    }

    public function getCurrentTime(){
        return $this->_constructTime . " (" . $this->timezone . ")";
    }

    public function timeToSend() {
        $timezone = $this->timezone;

        if($this->send_frequency == 'never') 
            return false;

        $digit_to_weekday = array(0 => 'sun',
            1 => 'mon',
            2 => 'tue',
            3 => 'wed',
            4 => 'thu',
            5 => 'fri',
            6 => 'sat');

        $now = $this->_constructTime;

        if($this->last_send)
            $last_send = new Zend_Date($this->last_send, 'yyyy-MM-dd');
        else
            $last_send = new Zend_Date('1900-01-01', 'yyyy-MM-dd');

        if($this->send_frequency == "d") {
            $send_days = explode(',', $this->send_days);
            $this_day = $digit_to_weekday[$now->get(Zend_Date::WEEKDAY_DIGIT)];
            $this_hour = $now->get(Zend_Date::HOUR_SHORT);

            if($now->compareDate($last_send) != 0 && 
                in_array($this_day, $send_days) && 
                $this_hour >= $this->send_hour){
                    return true;
                }
        }

        else if($this->send_frequency == "w") {
            $send_day = $this->send_weekday;
            $this_day = $digit_to_weekday[$now->get(Zend_Date::WEEKDAY_DIGIT)];
            $this_hour = $now->get(Zend_Date::HOUR_SHORT);

            if($now->compareWeek($last_send) != 0 &&
                $send_day == $this_day &&
                $this_hour >= $this->send_hour)
                return true;
        }

        else if($this->send_frequency == "m") {
            $month_days = $now->get(Zend_Date::MONTH_DAYS);

            $send_day = $this->send_day > $month_days ? $month_days : $this->send_day;
            $this_day = $now->get(Zend_Date::DAY_SHORT);

            $this_hour = $now->get(Zend_Date::HOUR_SHORT);

            if($now->compareMonth($last_send) != 0 &&
                $send_day == $this_day &&
                $this_hour >= $this->send_hour)
                return true;
        }

        return false;
    }

    public function updateSend() {
        $this->last_send = $this->_constructTime->toString("yyyy-MM-dd");
        return $this;
    }

    public function reportName() {
        $reportMapper = new Reports_Model_ReportMapper();
        $report = $reportMapper->find($this->report_id);
        return $report->name;
    }

    public function readableDuration() {
        $read = $this->getReadableFrequencies();
        return $read[$this->send_frequency];
    }

    public function companyName() {
        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($this->company_id)->current();
        return $company->name;
    }

    private function _get_recivers() {
        $userDb = new Core_Model_Users();
        $rec_groups = $this->_get_recivers_groups();

        $ids = array();

        foreach($rec_groups as $group) {
            $ids[] = $group->acl_role_id;
        }

        if(count($ids) == 0) return array();
        $users = $userDb->fetchAll($userDb->select()->where('acl_role_id IN (?)', $ids)->where('active'));

        if(count($users) == 0) return array();
        $companyDb = new Core_Model_Companies();
        if(!$companyDb->getCompany($users->current()->company_id)->active) return array();
        return $users;
    }

    private function _get_recivers_groups() {
        $subscriptionReceivers = new Reports_Model_SubscriptionReceivers();
        $receivers = $subscriptionReceivers->fetchAll($subscriptionReceivers->select()
            ->where('subscription_id = ?', $this->subscription_id));
        return $receivers;
    }


    public function getReciversShort() {
        return $this->_get_recivers_groups();
    }

    public function getReciversLong() {
        return $this->_get_recivers();
    }


    public function getNonAclReceivers(){
        $mails = array();
        foreach($this->_get_recivers_groups() as $rec){
            if(!is_null($rec->mail) && strlen($rec->mail) > 4){
                $mails[] = $rec->mail;
            }
        }
        return $mails;
    }

    public function getReceiversText() {
        $mails = array();
        foreach($this->_get_recivers() as $user){
            $mails[] = $user->email;
        }

        return array_merge($mails, $this->getNonAclReceivers());
    }

    protected function _delete()
    {
        $receiveDb = new Reports_Model_SubscriptionReceivers();
        $receivers = $receiveDb->getAdapter()->quoteInto('subscription_id = ?', $this->subscription_id);
        $receiveDb->delete($receivers);
    }

    public function send() {
        $reportsDb = new Reports_Model_ReportMapper();
        $report = $reportsDb->find($this->report_id);
        Core_Model_User::SetDefaultDateFormat($this->company_id);

        /*
         * change language to that of company
         */
        $old_locale = Zend_Registry::get('Zend_Locale');

        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($this->company_id)->current();

        /*
         * get locale from territory
         */
        $locale_name = Zend_Locale::getLocaleToTerritory($company->country == null ? 'GB' : $company->country);
        $locale = new Zend_Locale($locale_name);
        $translate = Zend_Registry::get('Zend_Translate');

        if($locale->getLanguage() == 'sv' || $locale->getLanguage() == 'en')
            $translate->setLocale($locale);

        Zend_Registry::set('Zend_Locale', $locale);
        Zend_Registry::set('Zend_Translate', $translate);
        if($company->week_start) Core_Model_User::SetDefaultStartOfWeek($company->week_start);
        if($company->date_format != "auto") Core_Model_User::SetDefaultDateFormat($company->date_format);
        //$report->initDataFromDatabase();


        $config = Zend_Registry::get('config');
        $receivers = $this->_get_recivers();
        $report->setCompany($company);
        $now = $this->_constructTime;
        $reportPdf = new Reports_Renderer_Pdf($report, $now);
        //$reportPdf = $report->renderToPdf();

        $mailParams = $config->mail->params->toArray();

        if($mailParams['auth'] == 'none')
            $mailParams = array('port' => $mailParams['port']);

        $transport = new Zend_Mail_Transport_Smtp($config->mail->host, $mailParams);

        $mail = new WebReport_Mail();

        /*foreach ($receivers as $rec) {
            $add = trim($rec->email);
            if(filter_var($add, FILTER_VALIDATE_EMAIL))
                $mail->addTo($add, $rec->forename . " " . $rec->surname);
        }*/
        foreach ($this->getNonAclReceivers() as $rec) {
            $add = trim($rec);
            if(filter_var($add, FILTER_VALIDATE_EMAIL))
                $mail->addBcc($add, $add);
        }

        $mail->setFrom($config->mail->from->address, $config->mail->from->name);
        $mail->setSubject('Web Report - ' . ($this->name));
        $mail->setBodyText($this->message_body ? ($this->message_body) : '');
        $pdfAtt = $mail->createAttachment($reportPdf->render());
        $pdfAtt->type = 'application/pdf';
        $pdfAtt->filename = str_replace(' ', '_', $report->name) . '.pdf';

        /*
         * reset language to user language
         */
        $translate->setLocale($old_locale);

        Zend_Registry::set('Zend_Locale', $old_locale);
        Zend_Registry::set('Zend_Translate', $translate);
        $mail->setCaller(WebReport_Mail::Subscription);
        $mail->send($transport, $this->name);
        Core_Model_User::SetDefaultStartOfWeek();
        Core_Model_User::SetDefaultDateFormat("auto");
        return true;
    }
}
