<?php

class Reports_Model_Template_RowMapper extends WebReport_Model_Mapper_Abstract
{
	
	protected $_dbTableClass = "Reports_Model_DbTable_Template_Row";

   public function save(Application_Model_Template_Row $row, $recursive = true)
   {
        $data = array(
            'height' 		=> $row->getHeight(),
            'order'			=> $row->getOrder()
        );
        
        $template = $row->getTemplate();
        if ($template) {
        		$data['template_id'] = $template->getId();
        }

        if (null === ($id = $row->getId())) {
            unset($data['id']);
            $this->getDbTable()->insert($data);
        } else {
            $this->getDbTable()->update($data, array('template_row_id = ?' => $id));
        }
        
        if ($recursive) {
        		$cells = $row->getCells();
        		$cellMapper = new Reports_Model_Template_Row_CellMapper();
        		
        		foreach ($cells as $cell) {
        				$cellMapper->save($cell, true);
        		}
        	
        }
    }

	protected function _populateFromRow($row, $populateDeep = true, $params = array()) 
	{
		$entry = new Reports_Model_Template_Row();
      $entry->setId($row->template_row_id)
      	->setHeight($row->height)
      	->setOrder($row->order);
      	
      if ($populateDeep) {
			$cellMapper = new Reports_Model_Template_Row_CellMapper();
			$cells = $cellMapper->fetchAllForRow($entry);
			foreach ($cells as $cell) {
				$cell->setRow($entry);	
			}
			$entry->setCells($cells);      	
      }
      
      return $entry;
	}

	public function fetchAllForTemplate(Reports_Model_Template $template) 
	{
		$id = $template->getId();
		
		if (null === $id) {
			throw new WebReport_Exception("Id must be set for template");	
		}
		return $this->fetchAll(array("where" => "template_id = " . $id, "order" => "order"));

	}

}

