<?php

/*
 * This class defines weather for a single datapoint.
 */

const WEATHER_ICON_PATH = '/../public/weather/';

class Reports_Model_WeatherData
{
    protected $_codeToImage = 
        array(
              0   => 'wsymbol_9999_error_warning', //connection error to web
              1   => 'wsymbol_9999_error_warning', //connection error to cache
              2   => 'wsymbol_9999_error_warning', //host reported error
              3   => 'wsymbol_9999_empty', //future request error
              4   => 'wsymbol_9999_error_warning', //server does not have data for this day
              5   => 'wsymbol_9999_error_warning', //no data, json decode might have failed. Bad data?
              10  => 'wsymbol_9999_empty', //does not match label
              395 => 'wsymbol_0012_heavy_snow_showers',
              392 => 'wsymbol_0016_thundery_showers',
              389 => 'wsymbol_0024_thunderstorms',
              386 => 'wsymbol_0016_thundery_showers',
              377 => 'wsymbol_0021_cloudy_with_sleet',
              374 => 'wsymbol_0013_sleet_showers',
              371 => 'wsymbol_0012_heavy_snow_showers',
              368 => 'wsymbol_0011_light_snow_showers',
              365 => 'wsymbol_0013_sleet_showers',
              362 => 'wsymbol_0013_sleet_showers',
              359 => 'wsymbol_0018_cloudy_with_heavy_rain',
              356 => 'wsymbol_0010_heavy_rain_showers',
              353 => 'wsymbol_0009_light_rain_showers',
              350 => 'wsymbol_0021_cloudy_with_sleet',
              338 => 'wsymbol_0020_cloudy_with_heavy_snow',
              335 => 'wsymbol_0012_heavy_snow_showers',
              332 => 'wsymbol_0020_cloudy_with_heavy_snow',
              329 => 'wsymbol_0020_cloudy_with_heavy_snow',
              326 => 'wsymbol_0011_light_snow_showers',
              323 => 'wsymbol_0011_light_snow_showers',
              320 => 'wsymbol_0019_cloudy_with_light_snow',
              317 => 'wsymbol_0021_cloudy_with_sleet',
              314 => 'wsymbol_0021_cloudy_with_sleet',
              311 => 'wsymbol_0021_cloudy_with_sleet',
              308 => 'wsymbol_0018_cloudy_with_heavy_rain',
              305 => 'wsymbol_0010_heavy_rain_showers',
              302 => 'wsymbol_0018_cloudy_with_heavy_rain',
              299 => 'wsymbol_0010_heavy_rain_showers',
              296 => 'wsymbol_0017_cloudy_with_light_rain',
              293 => 'wsymbol_0017_cloudy_with_light_rain',
              284 => 'wsymbol_0021_cloudy_with_sleet',
              281 => 'wsymbol_0021_cloudy_with_sleet',
              266 => 'wsymbol_0017_cloudy_with_light_rain',
              263 => 'wsymbol_0009_light_rain_showers',
              260 => 'wsymbol_0007_fog',
              248 => 'wsymbol_0007_fog',
              230 => 'wsymbol_0020_cloudy_with_heavy_snow',
              227 => 'wsymbol_0019_cloudy_with_light_snow',
              200 => 'wsymbol_0016_thundery_showers',
              185 => 'wsymbol_0021_cloudy_with_sleet',
              182 => 'wsymbol_0021_cloudy_with_sleet',
              179 => 'wsymbol_0013_sleet_showers',
              176 => 'wsymbol_0009_light_rain_showers',
              143 => 'wsymbol_0006_mist',
              122 => 'wsymbol_0004_black_low_cloud',
              119 => 'wsymbol_0003_white_cloud',
              116 => 'wsymbol_0002_sunny_intervals',
              113 => 'wsymbol_0001_sunny'
              );

    protected $_time; // time to be displayed.
    protected $_temp; // temp in centigrade
    protected $_prec; // amount of precipitation in mm/h
    protected $_code; // code describing the weather.
    protected $_is_real;
    protected $_errMess; //error message

  
    public function __construct($time, $temp, $prec, $code , $is_real = true, $errMess = "") {
        $this->_time = $time;
        $this->_temp = $temp;
        $this->_prec = $prec;
        $this->_code = $code;
        $this->_is_real = $is_real;
        $this->_errMess = $errMess;
    }

    public function __toString()
    {
        return sprintf("Time: %s\nTemperature: %dC\nPrecipitation: %fmm/h\nCode: %d",
                       $this->_time, $this->_temp, $this->_prec, $this->_code);
    }

    /*
     * get an image describing the weather
     */
    public function getImage()
    {
        $image_path = APPLICATION_PATH . WEATHER_ICON_PATH . $this->_codeToImage[$this->_code] . '.png';

        $weather_icon = Zend_Pdf_Image::imageWithPath($image_path);
        
        return $weather_icon;
    }

    /*
     * get a string describing the time.
     */
    public function getTime()
    {
        return $this->_time;
    }

    /*
     * get temperature in centigrade.
     */
    public function getTemperature()
    {
        return $this->_temp;
    }

    /*
     * get precipitation in mm/h.
     */
    public function getPrecipitation()
    {
        return $this->_prec;
    }

    /*
    * get image code.
    */
    public function getCode()
    {
        return $this->_code;
    }

    /*
    * get error message.
    */
    public function getErrorMessage()
    {
        return $this->_errMess;
    }

    public function getPrecipitationText(){
        $t = Zend_Registry::get('Zend_Translate');
        if($this->_prec > 50) return utf8_encode($t->_("VIOLENT_RAIN"));
        if($this->_prec > 10) return utf8_encode($t->_("HEAVY_RAIN"));
        if($this->_prec > 2.5) return utf8_encode($t->_("MODERATE_RAIN"));
        if($this->_prec > 0) return utf8_encode($t->_("LIGHT_RAIN"));
        return $t->_("NO_RAIN");
    }

    /*
     * true if $this is real data.
     */
    public function isRealData()
    {
        return $this->_is_real;
    }
}


