<?php

abstract class Reports_Model_WeatherServiceAbstract {
    private static function GetProviderFromConfig(){
            $config = Zend_Registry::get('config');
            return $config->weather->use_service;
    }

    public static function GetWeatherService($provider = NULL){
        if($provider == NULL) $provider =  self::GetProviderFromConfig();
        if($provider == 'www.worldweatheronline.com') return new Reports_Model_WorldWeatherOnlineWeatherService();
        if($provider == 'demonstration') return new Reports_Model_DemonstrationWeatherService();
        throw new WebReport_Exception_WeatherError("Could not find weather service implementation for '" . $provider ."'.");
    }

    protected function _forcedSource(){
        $config = Zend_Registry::get('config');
        return $config->weather->forced_source;
    }

    protected function _getKeys(){
        $config = Zend_Registry::get('config');
        $keysMerged = $config->weather->key;
        if($keysMerged == "" || $keysMerged == NULL) return array();
        $keys = explode(',',$keysMerged);
        foreach($keys as $k => $v){
            $keys[$k] = trim($v);
        }
        return $keys;
    }

    abstract public function getDayWeather($time, $position);
    abstract public function getWeekWeather($time, $position);
    abstract public function testService();

    public static function WeatherIsEnabled(){
        $provider = self::GetProviderFromConfig();
        $identity = Zend_Auth::getInstance()->getIdentity();
        $companyDb = Core_Model_Companies::getInstance();
        if($identity)
            $company = $companyDb->find($identity->company_id)->current();
        else
            return false;
        return !($provider == "none" || $provider == "" || $provider == NULL) &&
                ($company->weather_enable || $identity->multi_company);
    }
}
