<?php
class Sets_SetsController extends WebReport_Controller_Action
{
    protected $_user;

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $usersDb = Core_Model_Users::getInstance();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        //TODO: Submenu is now in layout per module, not controller, so should be defined in module, not here.
        $items = array();
        $items[] = array("urlOptions" => array(
            'module' => $module,
            "controller" => "index",
            "action" => "index"),
            "text" => _T('LIST_SETS'),
            "tooltip" => _T('LIST_SETS_TOOLTIP'));
        /*$items[] = array("urlOptions" => array(
            'module' => $module,
            "controller" => 'sets',
            "action" => "add-export-set"),
            "text" => _T('ADD_SOURCE_SET'),
            "tooltip" => _T('ADD_SOURCE_SET_TOOLTIP'));*/
        $items[] = array("urlOptions" => array(
            'module' => $module,
            "controller" => 'sets',
            "action" => "add-store"),
            "text" => _T('ADD_STORE'),
            "tooltip" => _T('ADD_STORE_TOOLTIP'));

        $set_id = $this->_request->getParam('id');
        if ($set_id) {
            $setsDb = new Core_Model_CameraSets();
            $this->_set = $setsDb->find($set_id)->current();
        }


        $this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
        $this->view->module = $module;
    }

    /*
     * Parses POST values from add-store.phtml to get a better structure
     * for the sections, in which section id dependence is removed.
     */
    protected function parseStoreSectionValues($values)
    {
        $parsedSectionValues = array();
        $sectionIndex = 0;
        $cameraIndex = 0;
        foreach ($values as $key => $value) {
            if (substr($key, 0, 12) == "section_name") {
                $section_name = $value;
                $section_nbr = substr($key, 13, 14);
                $parsedSectionValues["sections"][] = array('name'=>$section_name);
                foreach ($values as $key => $value) {
                    if ($key == "section_cameras_" . $section_nbr) {
                        $cameras = explode(',', $value);
                        if ($cameras[0]!=null) {
                            foreach ($cameras as $camera_id) {
                                $cameraArray = array('id'=>$camera_id, 'on_perimeter'=>0, 'reverse'=>0);
                                foreach ($values as $key => $value) {
                                    if ($key == "section_camera_perimeter" . $camera_id . $section_nbr) {
                                        $cameraArray['on_perimeter']=1;
                                    } elseif ($key == "section_camera_reverse" . $camera_id . $section_nbr) {
                                        $cameraArray['reverse']=1;
                                    }
                                }
                                $parsedSectionValues["sections"][$sectionIndex]["cameras"][$cameraIndex] = $cameraArray;
                                $cameraIndex++;
                            }
                        }
                    } elseif ($key == "section_id_" . $section_nbr) {
                        $parsedSectionValues["sections"][$sectionIndex]["section_id"] = $value;
                    }
                }
                $sectionIndex++;
            }
        }
        return $parsedSectionValues;
    }

    private function locationArrayFromString($location)
    {
        $full_location_array = explode("[", $location, 2);
        $location_str = $full_location_array[0];
        $location_coord = rtrim($full_location_array[1], "]");
        $location_array = explode(",", $location_str, 3);
        $location_city = $location_array[0];
        //TODO: Handle location entries consiting of four parts (London has double regions for example.);
        if (sizeof($location_array)==2) {
            $location_region = null;
            $location_country = $location_array[1];
        } else {
            $location_region = $location_array[1];
            $location_country = $location_array[2];
        }
        return array(
            "all" => $location,
            "city" => $location_city,
            "region" => $location_region,
            "country" => $location_country,
            "coordinates" => $location_coord
        );
    }

    private function prepareRepopulation($values)
    {
        $location_array = $this->locationArrayFromString($values["location"]);
        for ($i=1; $i<=$values["nbr_of_sections"]; $i++) {
            $cam_array = array();
            $cameras = explode(",", $values["section_cameras_" . $i]);
            foreach ($cameras as $camera_id) {
                if (isset($values["section_camera_reverse" . $camera_id . $i])) {
                    $reverse_count = 1;
                } else {
                    $reverse_count = 0;
                }
                $cam_array[] = array(
                    "id" => $camera_id,
                    "reverse_count" => $reverse_count
                );
            }
            $populate_data[] = array(
                "section_name" => $values["section_name_" . $i],
                "cameras" => $cam_array,
            );
        }
        return $extra_populate_data = array(
            'sales_region' => $values["sales_region"],
            'location' => $location_array,
            'store_cameras' => $values["store_cameras"],
            'sections' => $populate_data
        );
    }

    private function saveStore($values, $store = null)
    {
        if (!isset($store)) {
            $create_new = true;
        } else {
            $create_new = false;
            $cameraSet = $store;
            $store_section_id = intval($values["store_section_id"]);
        }
        $parsedSectionValues = $this->parseStoreSectionValues($values);
        $nbr_of_sections = intval($values["nbr_of_sections"]);

        $cameraSetDb = Core_Model_CameraSets::getInstance();
        $cameraSetData = array(
                'name' => $values["name"],
                'camera_set_type_id' => 1, //Store
                'company_id' => $values["company_id"],
                );
        if ($create_new) {
            $cameraSet = $cameraSetDb->createRow($cameraSetData);
        } else {
            $cameraSet->setFromArray($cameraSetData);
        }
        $cameraSetId = $cameraSet->save();

        $salesRegionDb = Core_Model_CameraSet_CameraSetType1SalesRegions::getInstance();
        //Sales region handling.
        if (is_numeric($values["sales_region"])) {
            $sales_region_id = $values["sales_region"];
        } else {
            $salesRegionData = array(
                    'name' => $values["sales_region"]
                );
            $salesRegion = $salesRegionDb->createRow($salesRegionData);
            $sales_region_id = $salesRegion->save();
        }

        $storeDb = Core_Model_CameraSet_CameraSetType1s::getInstance();
        //TODO: Handle incorrect/empty location entries.
        $location_array = $this->locationArrayFromString($values["location"]);
        $storeData = array(
                'camera_set_id' => $cameraSetId,
                'location_city' => $location_array["city"],
                'location_region' => $location_array["region"],
                'location_country' => $location_array["country"],
                'location_coordinates' => $location_array["coordinates"],
                'sales_region_id' => $sales_region_id,
                'day_start_offset' => $values["day_start_offset"],
                );
        if ($create_new) {
            $storeMetadata = $storeDb->createRow($storeData);
        } else {
            $storeMetadata = $cameraSet->getSetMetadataRow();
            $storeMetadata->setFromArray($storeData);
        }
        $storeMetadata->save();

        Core_Model_CameraSet_CameraSetType1SalesRegions::deleteUnusedSalesRegions();

        $storeSectionDb = Core_Model_CameraSet_CameraSetType1Sections::getInstance();
        if ($create_new) {
            $storeMainSectionData = array(
                    'camera_set_id' => $cameraSetId,
                    'name' => null,
                    'is_store' => 1,
                    );
            $storeMainSection = $storeSectionDb->createRow($storeMainSectionData);
        } else {
            //No need to update anything but $storeMainSection is needed to check cameras.
            $storeMainSection = $storeSectionDb->getSectionById($store_section_id);
        }
        $storeMainSectionId = $storeMainSection->save();
        $storeSectionCameraDb = Core_Model_CameraSet_CameraSetType1SectionCameras::getInstance();
        //Add store cameras (and perhaps remove on edit)
        $cameraDb = Core_Model_Cameras::getInstance();

        $camera_to_be_deleted = false;
        $existingMainSectionCameras = $storeMainSection->getSectionCameras();
        if ($existingMainSectionCameras->count() > 0) {
            $camera_to_be_deleted = true; //Might not have to delete anything, but check must be run.
        }
        if (isset($values['store_cameras']) && $values['store_cameras']!="") {
            $storeCameraIds = explode(',', $values['store_cameras']);
            foreach ($storeCameraIds as $cameraId) {
                $camera = $cameraDb->getCamera($cameraId);
                if (!$camera) {
                    throw new WebReport_Exception("Could not find store camera with id: $cameraId");
                } else {
                    if ($camera_to_be_deleted) {
                        $cameras_to_save[] = $camera;
                    }
                    $existing_camera = $storeSectionCameraDb->getRowByCameraAndSectionId($camera->camera_id, $storeMainSectionId);
                    $storeSectionCameraData = array(
                        'camera_id' => $camera->camera_id,
                        'camera_set_type1_section_id' => $storeMainSectionId,
                        'reverse_count' => 0,
                        );
                    if ($existing_camera != null) {
                        $storeSectionCamera = $existing_camera;
                        $storeSectionCamera->setFromArray($storeSectionCameraData);
                    } else {
                        $storeSectionCamera = $storeSectionCameraDb->createRow($storeSectionCameraData);
                    }
                    $storeSectionCamera->save();
                }
            }
        }
        if ($camera_to_be_deleted) {
            foreach ($existingMainSectionCameras as $existingMainSectionCamera) {
                $delete = true;
                $existing_id = $existingMainSectionCamera->camera_id;
                if (isset($cameras_to_save)) {
                    foreach ($cameras_to_save as $camera_to_save) {
                        if ($camera_to_save->camera_id == $existing_id) {
                            $delete = false;
                        }
                    }
                }
                if ($delete) {
                    $existingMainSectionCamera->delete();
                }
            }
        }
        $existingSections = $storeDb->getContentById($cameraSetId);
        if (isset($existingSections['sections'])) {
            $existingSections = $existingSections['sections'];
        } else {
            $existingSections = array();
        }
        //Add section cameras (and perhaps remove on edit)
        if ($nbr_of_sections) {
            for ($i=0; $i<$nbr_of_sections; $i++) {
                $camera_to_be_deleted = false;
                $cameras_to_save = array();
                $sectionData = array(
                        'camera_set_id' => $cameraSetId,
                        'name' => $parsedSectionValues["sections"][$i]["name"],
                        'is_store' => 0,
                    );
                $section_id = $parsedSectionValues["sections"][$i]["section_id"];
                if ($section_id != null) {
                    $storeSection=$storeSectionDb->getSectionById($section_id);
                    $storeSectionsToBeSaved[] = $storeSection;
                    $storeSection->setFromArray($sectionData);
                } else {
                    $storeSection = $storeSectionDb->createRow($sectionData);
                }
                $storeSectionId = $storeSection->save();
                $existingSectionCameras = $storeSection->getSectionCameras();
                $sectionCameras = $parsedSectionValues["sections"][$i]["cameras"];
                if ($existingSectionCameras->count() > 0) {
                    $camera_to_be_deleted = true; //Might not have to delete anything, but check must be run.
                }
                if (isset($sectionCameras) && $sectionCameras != "") {
                    foreach ($sectionCameras as $sectionCamera) {
                        $camera = $cameraDb->getCamera($sectionCamera["id"]);
                        if ($camera_to_be_deleted) {
                            $cameras_to_save[] = $camera;
                        }
                        if (!$camera) {
                            r_log($cameraName, "Could not find section camera with name: ");
                        } else {
                            $existing_camera = $storeSectionCameraDb->getRowByCameraAndSectionId($camera->camera_id, $section_id);
                            $storeSectionCameraData = array(
                                'camera_id' => $camera->camera_id,
                                'camera_set_type1_section_id' => $storeSectionId,
                                'reverse_count' => $sectionCamera["reverse"]
                                );
                            if ($existing_camera != null) {
                                $storeSectionCamera = $existing_camera;
                                $storeSectionCamera->setFromArray($storeSectionCameraData);
                            } else {
                                $storeSectionCamera = $storeSectionCameraDb->createRow($storeSectionCameraData);
                            }
                            $storeSectionCamera->save();
                        }
                    }
                }
                if ($camera_to_be_deleted) {
                    foreach ($existingSectionCameras as $existingSectionCamera) {
                        $delete = true;
                        $existing_id = $existingSectionCamera->camera_id;
                        if (isset($cameras_to_save)) {
                            foreach ($cameras_to_save as $camera_to_save) {
                                if ($camera_to_save->camera_id == $existing_id) {
                                    $delete = false;
                                }
                            }
                        }
                        if ($delete) {
                            $existingSectionCamera->delete();
                        }
                    }
                }
            }
        }
        if (!empty($existingSections)) {
            foreach ($existingSections as $existingSection) {
                $delete = true;
                $existing_id = $existingSection["id"];
                if (isset($storeSectionsToBeSaved)) {
                    foreach ($storeSectionsToBeSaved as $storeSectionToSave) {
                        if ($storeSectionToSave->camera_set_type1_section_id == $existing_id) {
                            $delete = false;
                        }
                    }
                }
                if ($delete) {
                    $select = $storeSectionDb->select();
                    $select->where('camera_set_type1_section_id = ?', $existing_id);
                    $section_to_be_deleted = $storeSectionDb->fetchRow($select);
                    $section_to_be_deleted->delete();
                }
            }
        }
    }

    private function fetchAvailableDataForUser($user) {
        $cameraDb = Core_Model_Cameras::getInstance();
        $salesRegionDb = Core_Model_CameraSet_CameraSetType1SalesRegions::getInstance();
        $storeDb = Core_Model_CameraSet_CameraSetType1s::getInstance();
        $storeCamerasDb = Core_Model_CameraSet_CameraSetType1SectionCameras::getInstance();
        if ($user->multi_company) {
            $allAvailableCameras = $cameraDb->fetchAll();
            $salesRegions = $salesRegionDb->fetchAll();

        } else {
            $companyDb = Core_Model_Companies::getInstance();
            $company = $companyDb->getCompany($user->company_id);
            $cameras_array = $cameraDb->getCameras(null, $company);
            foreach ($cameras_array as $camera) {
                $select = $cameraDb->select();
                $camera_row = $cameraDb->fetchAll($select->where("camera_id = ?", $camera->camera_id))->current();
                $allAvailableCameras[] = $camera_row;
            }
            $salesRegions = $salesRegionDb->getSalesRegionsByCompanyId($user->company_id);
        }
        $allStoreCameras = $storeCamerasDb->fetchAll();
        $storeCameraIds = array();
        foreach ($allStoreCameras as $storeCamera) {
            $parentSet = $storeCamera->getParentSection()->getParentCameraSet();
            if ($parentSet != $this->_set) {
                $storeCameraIds[] = $storeCamera->camera_id;
            }
        }
        $storeCameraIds = array_unique($storeCameraIds);
        $ret["cameras"] = $allAvailableCameras;
        $ret["cameras_used_in_stores"] = $storeCameraIds;
        $ret["sales_regions"] = $salesRegions;
        return $ret;
    }

    public function fetchEnableAllAction()
    {
        $metadata_row = $this->_set->getSetMetadataRow();
        $fetch_status = $metadata_row->getFetchStatus();
        foreach ($fetch_status["disabled"] as $camera) {
            $camera->fetch_me = 1;
            $camera->save();
        }
        $this->_flashMessenger->addMessage($this->_translate->_('ENABLED_FETCH_OF_DATA_TO_Xense_FOR_ALL_CAMERAS_IN_STORE.'));
        $this->_helper->redirector('index', 'index', 'sets');
    }

    public function fetchDisableAllAction()
    {
        $metadata_row = $this->_set->getSetMetadataRow();
        $fetch_status = $metadata_row->getFetchStatus();
        foreach ($fetch_status["enabled"] as $camera) {
            $camera->fetch_me = 0;
            $camera->save();
        }
        $this->_flashMessenger->addMessage($this->_translate->_('DISABLED_FETCH_OF_NEW_DATA_TO_Xense_FOR_ALL_CAMERAS_IN_STORE.'));
        $this->_helper->redirector('index', 'index', 'sets');
    }

    public function addStoreAction()
    {
        $this->view->title = $this->_translate->_("ADD_STORE");
        //API call in view to get locations needs to know baseUrl
        $this->view->baseUrl = $this->_request->getBaseUrl();

        $availableData = $this->fetchAvailableDataForUser($this->_user);

        $this->view->allAvailableCameras = $availableData["cameras"];
        $this->view->camerasInStores = $availableData["cameras_used_in_stores"];
        $this->view->salesRegions = $availableData["sales_regions"];

        $form = new Sets_Form_AddEditStore();

        if ($this->_request->isPost()) {
            $values = $_POST;
            if ($form->isValid($values)) {
                $this->saveStore($values);
                $this->_flashMessenger->addMessage($this->_translate->_('STORE_ADDED_SUCCESSFULLY.'));
                $this->_helper->redirector('index', 'index', 'sets');

            } else {
                $extra_populate_data = $this->prepareRepopulation($values);
                $this->view->extra_populate_data = $extra_populate_data;
                $form->populate($_POST);
            }
        }
        $this->view->form = $form;
    }

    public function editAction()
    {

        $this->view->baseUrl = $this->_request->getBaseUrl();

        $availableData = $this->fetchAvailableDataForUser($this->_user);

        $this->view->allAvailableCameras = $availableData["cameras"];
        $this->view->camerasInStores = $availableData["cameras_used_in_stores"];
        $this->view->salesRegions = $availableData["sales_regions"];

        $form = new Sets_Form_AddEditStore("edit");
        $this->_helper->viewRenderer->setRender('add-store');

        $this->view->title =$this->_translate->_('EDIT_STORE');

        if (!$this->getRequest()->isPost()) {
            $storeDB = Core_Model_CameraSet_CameraSetType1s::getInstance();
            $metaDataRow = $storeDB->getMetaDataById($this->_set->camera_set_id);

            $location_str = $metaDataRow->location_city
            . ", " . $metaDataRow->location_region
            . ", " . $metaDataRow->location_country
            . " [" . $metaDataRow->location_coordinates ."]";

            $storeCameras = $this->_set->getSetContent();
            if (isset($storeCameras["store_camera_ids"])) {
                $store_camera_ids = implode(",", $storeCameras["store_camera_ids"]);
            } else {
                $store_camera_ids = null;
            }

            $sections = $metaDataRow->getSections();

            $populate_data = array();
            foreach ($sections as $section) {
                $cam_array = array();
                if ($section->is_store==0) {
                    $cameras = $section->getSectionCameras();
                    foreach ($cameras as $camera) {
                        $cam_array[] = array(
                            "id" => $camera->camera_id,
                            "reverse_count" => $camera->reverse_count
                        );
                    }
                    $populate_data[] = array(
                        "section_id" => $section->camera_set_type1_section_id,
                        "section_name" => $section->name,
                        "cameras" => $cam_array,
                    );
                } else {
                    $store_section_id = $section->camera_set_type1_section_id;
                }
            }
            $data = array('name' => $this->_set->name,
                          'company_id' => $this->_set->company_id,
                          'day_start_offset' => substr($metaDataRow->day_start_offset, 0, 5),
                          'store_section_id' => $store_section_id
            );
            //Selectizes doesn't work well with regular form poulate, so that's done specifically in view script.
            $extra_populate_data = array(
                'sales_region' => $metaDataRow->sales_region_id,
                'location' => array(
                        "all" => $location_str,
                        "city" => $metaDataRow->location_city,
                        "region" => $metaDataRow->location_region,
                        "country" => $metaDataRow->location_country,
                        "coordinates" => $metaDataRow->location_coordinates
                    ),
                'store_cameras' => $store_camera_ids,
                'sections' => $populate_data
            );
            $this->view->extra_populate_data = $extra_populate_data;
            $form->populate($data);
        } else {
            $values = $_POST;
            if ($form->isValid($values)) {
                $store = $this->_set;
                $this->saveStore($values, $store);
                $this->_flashMessenger->addMessage($this->_translate->_('STORE_UPDATED.'));
                $this->_helper->redirector('index', 'index', 'sets');
            } else {
                $extra_populate_data = $this->prepareRepopulation($values);
                $form->populate($values);
                $this->view->extra_populate_data = $extra_populate_data;
            }
        }

        $this->view->form = $form;
    }

    public function deleteAction()
    {
        $type = $this->_set->camera_set_type_id;
        if ($this->_set) {
            $this->_set->delete();
        }
        if ($type == 1) {
            $this->_flashMessenger->addMessage($this->_translate->_('STORE_REMOVED_SUCCESSFULLY.'));
        } else {
            r_log("Set of this type shouldn't exist!");
        }
        $this->_helper->redirector('index', 'index', 'sets');
    }
}
