<?php

class Sets_Form_AddEditStore extends UniForm_Form
{

    protected $_action;
    protected $_company_id;
    /**
    * Constructor
    *
    * @see Zend_Form
    * @param string $action Can be add or edit
    * @param mixed $options
    * @return void
    */
    public function __construct($action = "add", $options = null, $company_id = null)
    {
        $this->_company_id = $company_id;
        $this->_action = $action;
        parent::__construct($options);
    }

    public function isValid($values)
    {
        $validation_result = parent::isValid($values);
        if ($values["nbr_of_sections"] != 0) {
            for ($i=1; $i<=$values["nbr_of_sections"]; $i++) {
                if ($values["section_name_" . $i] == "") {
                    $validation_result = false;
                }
            }
        }
        return $validation_result;
    }

    public function init()
    {
        parent::init();

        $action = $this->_action;

        $name = new UniForm_Form_Element_Text('name');
        $name->setLabel('NAME')
          ->setRequired()
          ->addFilter('StringTrim');

        $identity = Zend_Auth::getInstance()->getIdentity();

        if ($identity->multi_company) {
            $companiesDb = new Core_Model_Companies();
            $activeCompanies = $companiesDb->fetchAllActive();
            $company_id = new UniForm_Form_Element_Select('company_id');
            $company_id->setLabel('COMPANY')
              ->setRequired()
              ->addValidator(new Zend_Validate_GreaterThan(0)) // check if user belongs to company
              ->setErrorMessages(array($this->getView()->translate("USER_MUST_BELONG_TO_A_COMPANY.")));

            foreach ($activeCompanies as $company) {
                $company_id->addMultiOption($company->company_id, $company->name);
            }
        } else {
            $company_id = new UniForm_Form_Element_Hidden('company_id');
            $company_id->setValue(Zend_Auth::getInstance()->getIdentity()->company_id);
        }

/*
 * Store specific form elements
 */
        $sales_region = new UniForm_Form_Element_Text('sales_region');
        $sales_region->setLabel('SALES_AREA')
                    ->setDescription('THE_STORE_SALES_AREA_CAN_BE_SET_FREELY_BY_WRITING._IT_WILL_SUGGEST_PREVIOUSLY_SET_SALES_AREAS_TO_HELP_SETTING_UP_CONSISTENT_FILTERING_OPTIONS.');

        $location = new UniForm_Form_Element_Text('location');
        $location->setLabel('LOCATION')
                    ->setDescription('A_VALID_SEARCH_RESULT_MUST_BE_SELECTED_TO_GENERATE_CORRECT_LOCATION_DATA_FOR_THE_STORE._THE_CHOSEN_SEARCH_RESULT_WILL_BE_USED_TO_SET_THE_STORES_CITY,_REGION_AND_COUNTRY.');

        $day_start_offset = new UniForm_Form_Element_Select('day_start_offset');
        $day_start_offset->setLabel('DAY_START_HOUR')
                    ->setDescription('WHEN_AGGREGATING_DATA_FOR_A_DATE_THIS_HOUR_WILL_BE_USED_AS_THE_BEGINNING_OF_THE_DATE_INSTEAD_OF_00:00._MOST_COMMONLY_USED_IF_STORE_IS_OPEN_OVER_MIDNIGHT.')
                    ->addMultiOptions(array(
                        "-06:00"=>"18:00",
                        "-05:00"=>"19:00",
                        "-04:00"=>"20:00",
                        "-03:00"=>"21:00",
                        "-02:00"=>"22:00",
                        "-01:00"=>"23:00",
                        "00:00"=>"00:00 (" . $this->getView()->translate('DEFAULT') . ")",
                        "01:00"=>"01:00",
                        "02:00"=>"02:00",
                        "03:00"=>"03:00",
                        "04:00"=>"04:00",
                        "05:00"=>"05:00",
                        "06:00"=>"06:00",))
                    ->setValue("00:00");

//TODO add store sections.
        $add_section_button = new UniForm_Form_Element_Button('add_section_button');
        $add_section_button->setLabel('ADD_DEPARTMENT');

        $nbr_of_sections= new UniForm_Form_Element_Hidden('nbr_of_sections');
        $nbr_of_sections->setValue(0);

        $store_section_id= new UniForm_Form_Element_Hidden('store_section_id');
        $store_section_id->setValue(null);

        $cameras_template = new UniForm_Form_Element_Text('cameras_template');
        $cameras_template->setLabel('STORE CAMERAS');

        $submit = new UniForm_Form_Element_Submit('submit');
        if ($action == "add") {
            $submit->setLabel('ADD_STORE');
        } else {
            $submit->setLabel('SAVE_STORE');
        }

        $this->addElements(array(
            $name,
            $company_id,
            $sales_region,
            $location,
            $day_start_offset,
            $add_section_button,
            $nbr_of_sections,
            $store_section_id,
            $cameras_template));
        if (isset($company_id)) {
            $this->addElement($company_id);
        }

        $this->addDisplayGroup(array('name', 'company_id'), 'basic-group', array('legend' => _T('STORE_BASICS')));
        $this->addDisplayGroup(array(
            'sales_region',
            'location',
            'day_start_offset'), 'type-group', array('legend' => _T('STORE_DETAILS')));
        $this->addDisplayGroup(array('add_section_button', 'cameras_template'), 'content-group', array('legend' => _T('STORE_CONTENT')));
        $this->addElements(array($submit));
    }
}
