<?php

include_once 'core/models/CameraGroups.php';
include_once 'core/models/CompanyCameraGroup.php';
include_once 'core/models/Companies.php';

class Sources_DataStorageManagementController extends WebReport_Controller_Action
{
    protected $_user, $_group;

    public function init()
    {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $request = $this->_request;
        //$group = $this->_group;
        $folder = $request->getParam('folder');
        $id = $request->getParam('id');
        $user = $this->_user;
        $helper = $this->_helper;

        //If no group detected but cookie is set redirect to make group available.
        if (!isset($id) && !isset($folder) && isset($_COOKIE[WebReport_Controller_Helper_Browse::COOKIE_NAME])) {
            foreach ($this->view->messages as $message) {
                $this->_flashMessenger->addMessage($this->_translate->_($message));
            }
            $helper->redirector($request->getActionName(),
                                $request->getControllerName(),
                                $request->getModuleName(),
                                array('folder' => $_COOKIE[WebReport_Controller_Helper_Browse::COOKIE_NAME]));
        }

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = Core_Model_Folders::getInstance();
        $this->_group = $groupsDb->find($groupId)->current();

        $usersDb = Core_Model_Users::getInstance();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        $items = array();
        //$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "browse-camera-groups"), "text" => 'BROWSE_CAMERA_GROUPS');

        //if($this->_group && $this->_group->canEdit($this->_user)) 
        //    $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add-camera-group", "folder" => $groupId), "text" => 'ADD_CAMERA_GROUP');

        //TODO: Submenu is now in layout per module, not controller, so should be defined in module, not here.
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "index", "action" => "index"), "text" => _T('BROWSE_SOURCES'), 'tooltip' => _T('BROWSE_SOURCES_TOOLTIP'));
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "data-storage-management", "action" => "merge"), "text" => _T('MERGE_CAMERAS'), 'tooltip' => _T('MERGE_CAMERAS_TOOLTIP'));
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "folders", "action" => "browse", "folder" => $this->_group->group_id), "text" => _T('EDIT_FOLDERS'), 'tooltip' => _T('EDIT_FOLDERS_TOOLTIP'));

        //$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

        $features = new Core_Model_Features();

        $this->view->dynamic_channels = $features->dynamicChannels();
        $this->view->module = $module;
        $this->view->headTitle("SOURCES", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function fetchToggleAction()
    {
        $cameraDb = Core_Model_Cameras::getInstance();
        $cameraId = $this->_getParam("id");
        $camera = $cameraDb->find($cameraId)->current();
        if ($camera->fetch_me) {
            $camera->fetch_me = 0;
            $this->_flashMessenger->addMessage($this->_translate->_('NEW_DATA_FOR_THIS_CAMERA_CAN_NO_LONGER_BE_FETCHED_BY_TrueView Xense.'));
        } else {
            $camera->fetch_me = 1;
            $this->_flashMessenger->addMessage($this->_translate->_('DATA_FOR_THIS_CAMERA_CAN_NOW_BE_FETCHED_BY_TrueView Xense.'));
        }
        $camera->save();
        $document = Core_Model_DocumentFactory::fetch($camera->camera_group_id, "camera_group");
        $folder = $document->getGroup();
        $this->_helper->redirector('view-camera', 'data-storage-management', 'sources', array('id' => $cameraId));
    }

    public function browseCameraGroupsAction()
    {
        $this->view->list = WebReport_Controller_Helper_Browse::getController($this->_user, $this->_group, $this->_helper, $this->_request,
            'Core_Model_CameraGroups',
            'Admin_Model_CameraGroupListGenerator');
        if($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function editCameraGroupAction()
    {
        $this->view->headTitle("CAMERA_GROUPS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $groupId = $this->_getParam("id");
        $this->view->headTitle("EDIT", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $document = Core_Model_DocumentFactory::fetch($groupId, "camera_group");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_CAMERA_GROUP.'));
            $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin');
            return;
        }

        if (!$groupId)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $groupsDb = Core_Model_CameraGroups::getInstance();
        $group = $groupsDb->find($groupId)->current();

        if (!$group)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $companyGroupDb = Core_Model_CompanyCameraGroup::getInstance();
        if (!$this->_user->multi_company) {
            $cg = $companyGroupDb->fetchRow($companyGroupDb->select()->where('camera_group_id = ?', $group->camera_group_id)->where('company_id = ?', $this->_user->company_id));
            if (!$cg) {
                throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));
            }
        }

        $form = new Admin_Form_AddEditCameraGroup("edit", $group);

        if ($this->_request->isPost()) {
            $values = $_POST;
            if ($form->isValid($values)) {
                if ($this->_user->multi_company) {
                    $rows = $companyGroupDb->fetchAll($companyGroupDb->select()->where("camera_group_id = ?", $group->camera_group_id));
                    foreach ($rows as $row) {
                        $row->delete();
                    }

                    $row = $companyGroupDb->createRow(array("company_id" => $this->_group->company_id, "camera_group_id" => $group->camera_group_id));
                    $row->save();
                }

                $group->name = $values['name'];
                $group->email = $values['email'];
                $group->no_data_timeout = $values['no_data_timeout'];
                $group->no_connection_timeout = $values['no_connection_timeout'];
                $group->username = $values['username'];
                $group->description = $values['description'];
                $group->save();

                $this->_flashMessenger->addMessage($this->_translate->_('CAMERA_GROUP_UPDATED.'));
                $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin', array('folder' => $this->_group->group_id));

            } else {
                $form->populate($_POST);
            }
        } else {
            $tableCompanies = $group->findDependentRowset("Core_Model_CompanyCameraGroup");
            $values = $group->toArray();
            $values['company_id'] = array();
            foreach ($tableCompanies as $tc) {
                $values['company_id'][] = $tc->company_id;
            }
            $form->populate($values);
        }

        $this->view->form = $form;
    }

    public function changeCameraGroupPasswordAction()
    {
        $this->view->headTitle("CAMERA_GROUPS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $form = new Admin_Form_ChangeCameraGroupPassword();

        $groupId = $this->_getParam("id");

        $document = Core_Model_DocumentFactory::fetch($groupId, "camera_group");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_CAMERA_GROUP.'));
            $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin');
            return;
        }

        if (!$groupId)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $cameraGroupsDb = Core_Model_CameraGroups::getInstance();
        $group = $cameraGroupsDb->find($groupId)->current();

        if (!$group)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $companyGroupDb = Core_Model_CompanyCameraGroup::getInstance();
        if (!$this->_user->multi_company) {
            $cg = $companyGroupDb->fetchRow($companyGroupDb->select()->where('camera_group_id = ?', $group->camera_group_id)->where('company_id = ?', $this->_user->company_id));
            if (!$cg) {
                throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));
            }
        }


        $this->view->headTitle(sprintf($this->_translate->_('CHANGE_PASSWORD_FOR_"%1$s"'), $group->name), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $this->view->name = $group->name;

        if ($this->_request->isPost()) {
            $values = $_POST;

            if ($form->isValid($values)) {
                $dynamicSalt = CM_User::createSalt();
                $group->password = md5(STATIC_SALT . $values['password'] . $dynamicSalt);
                $group->password_salt = $dynamicSalt;
                $group->save();

                $this->_flashMessenger->addMessage(sprintf($this->_translate->_('PASSWORD_IS_CHANGED_FOR_THE_CAMERA_GROUP_"%1$s".'), $group->name));
                $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin');
            }

        }

        $this->view->form = $form;
    }

    public function addDataForFile($filename, $cameraGroup) {
        $appDb  = new Core_Model_Applications();
        $unique = $this->_getParam('unique');

        $cache = Zend_Registry::get('cache');
        $status = array('max' => 0, 'cur' => 0, 'stage' => 1);
        $cache->save(json_encode($status), "importstatus_$unique");
        $text = "";
        $fp = gzopen($filename, "r");
        while ($data = gzread($fp, 1024*128)) {
            $text .= $data;
        }
        $status = array('max' => 0, 'cur' => 0, 'stage' => 2);
        $cache->save(json_encode($status), "importstatus_$unique");
        $jsons = json_decode($text);
        $i = 0;
        $status = array('max' => 0, 'cur' => 0, 'stage' => 3);
        $cache->save(json_encode($status), "importstatus_$unique");
        $meta = $jsons->meta;
        $cntdata = $jsons->cntdata;
        $eventdata = $jsons->eventdata;
        $idmeta = array();
        $i  = 0;
        foreach ($cntdata as $cd) {
            foreach ($meta->cntdata as $id => $camera) {
                $i = $i + 1;
            }
        }
        foreach ($meta->eventdata as $id => $camera) {
            $i = $i + count($eventdata->$id->data);
            if (isset($eventdata->$id->openinghours)) {
                $i = $i + count($eventdata->$id->openinghours);
            }
        }
        $status = array('max' => $i, 'cur' => 0, 'stage' => 4);
        $cache->save(json_encode($status), "importstatus_$unique");
        $i = 0;
        foreach ($meta->eventdata as $id => $camera) {
            $app = $appDb->findWithId($camera->app);
            if ($app->hasEventData()) {
                $ed_app = $app->asEventDataApplication();
                $dbTable = $ed_app->getEventDataTable();
                $indata = array (
                    'protocol' => 1,
                    'camera_name' => $camera->name,
                    'camera_serial' => $camera->serial,
                    'camera_regcode' => $camera->regcode,
                    'data_struct' => $ed_app->getDataColumns(),
                    'utc_offset' => $camera->utc_offset,
                    'utc_time_start' => null,
                    'utc_time_end' => null
                );
                foreach ($eventdata->$id->data as $key => $ed) {
                    $ed = (array) $ed;
                    unset($ed[$ed_app->getIdColumn()]);
                    unset($ed['camera_id']);
                    foreach ($indata["data_struct"] as $index => $field) {
                        $rowdata[$key][$index] = $ed[$field];
                    }
                    if (!isset($indata['utc_time_start']) || $indata['utc_time_start'] > $ed['t_start']) {
                        $indata['utc_time_start'] = $ed['t_start'];
                    }
                    if (!isset($indata['utc_time_end']) || $indata['utc_time_end'] < $ed['t_end']) {
                        $indata['utc_time_end'] = $ed['t_end'];
                    }
                }
                $indata['data'] = $rowdata;
                set_time_limit(30);
                $import_info = array('status' => $status, 'unique' => $unique);
                $new_camera_id = Core_Model_EventData::addImportData($indata, $app, $cameraGroup, $import_info);
                $status = json_decode($cache->load("importstatus_$unique"), true);
                $i = $status['cur'];
                $openingHoursDb = Core_Model_OpeningHours::getInstance();
                foreach ($eventdata->$id->openinghours as $oh) {
                    $oh = (array) $oh;
                    unset($oh['opening_hour_id']);
                    unset($oh['last_update']);
                    $oh['camera_id'] = $new_camera_id;
                    $oh_row = $openingHoursDb->createRow($oh);
                    try {
                        $oh_row->save();
                    } catch (Exception $e) {
                        r_log($e->getMessage());
                        throw new WebReport_Exception($e->getMessage());
                    }
                    $i = $i + 1;
                    $status['cur'] = $i;
                    $cache->save(json_encode($status), "importstatus_$unique");
                }
            } else {
                throw new WebReport_Exception("Export faulty. Camera defined as EventData in export does not have an EventData application.");
            }
        }
        foreach ($cntdata as $d) {
            $d = (array)$d;
            foreach ($meta->cntdata as $id => $camera) {
              $app = $appDb->findWithId($camera->app);
                if (!$app->hasEventData()) {
                    session_write_close();
                    $serial = $camera->serial;
                    $regcode = $camera->regcode;
                    if (strlen($regcode) > 4) $serial = "$serial::$regcode";
                    $base = array(null,null,'3',$camera->name, $serial, $d['cnttime']);
                    $map = $app->getCntMapping();
                    if ($map == null || count($map) == 0) {
                        r_log("no map!");
                        $message = $this->_translate->_('ONE_OR_MORE_SOURCES_WERE_NOT_SUPPORTED_BY_IMPORT');
                        continue;
                    }
                    $base[] = count($map);
                    foreach ($map as $cnt => $label) {
                        $base[] = $label;
                        if (isset($d["c$id" . "_" . $cnt])) {
                            $base[] = $d["c$id" . "_" . $cnt];
                        } else {
                            throw new WebReport_Exception("CntMapping does not match import data.");
                        }
                    }
                    set_time_limit(30);
                    $i = $i + 1;
                    $status['cur'] = $i;
                    Core_Model_CountData::addDataForSeparatedString($base, false, false, $cameraGroup, true, false, $camera->app);
                } else {
                    throw new WebReport_Exception("Export faulty. Camera defined as CountData in export does have an EventData application.");
                }
            }
            $cache->save(json_encode($status), "importstatus_$unique");
        }
        if (isset($message)) {
            $this->_flashMessenger->addMessage($message);
        }
        $cache->save(json_encode($status), "importstatus_$unique");
    }

    public function checkImportStatusAction() {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        $unique = $this->_getParam('unique');
        header('Content-type: application/json');
        $cache = Zend_Registry::get('cache');
        $status = $cache->load("importstatus_$unique");
        if($status) echo $status;
        else echo json_encode(array("stage" => 0));
    }

    public function importCntFilesAction()
    {
        $this->view->headTitle(_T("IMPORT_DATA_FROM_FILE"), Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $cameraGroupId = $this->_group->camera_group_id;

        $document = Core_Model_DocumentFactory::fetch($cameraGroupId, "camera_group");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_EDIT_THIS_CAMERA_GROUP.'));
            $this->_helper->redirector('browse-cameras','data-storage-management','admin');
            return;
        }

        if (!$cameraGroupId)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $cameraGroupsDb = new Core_Model_CameraGroups();
        $group = $cameraGroupsDb->find($cameraGroupId)->current();

        if (!$group)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $companyGroupDb = new Core_Model_CompanyCameraGroup();
        if (!$this->_user->multi_company) {
            $cg = $companyGroupDb->fetchRow($companyGroupDb->select()->where('camera_group_id = ?', $cameraGroupId)->where('company_id = ?', $this->_user->company_id));
            if (!$cg) {
                throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));
            }
        }

        $camerasDb = Core_Model_Cameras::getInstance();
        $countDataTablesDb = Core_Model_CountData_Tables::getInstance();

        $groupName = $group->name;

        $this->view->groupName = $groupName;

        $form = new Admin_Form_ImportCntFilesToCameraGroup();
        $this->view->status = $this->_request->getParam("unique_monitor");

        $db = Zend_Registry::get('db');
        if ($this->_request->isPost()) {
            $values = $_POST;

            if ($form->isValid($values)) {
                $camerasDb = Core_Model_Cameras::getInstance();
                $upload = new Zend_File_Transfer_Adapter_Http();
                $files = $upload->getFileInfo();
                $logger = Zend_Registry::get('Zend_Logger_FireBug');

                $serial = $values['serial'];
                $type   = $values['type'];
                foreach ($files as $file => $info) {
                    $filename = $info['tmp_name'];
                    if ($filename == '')
                        continue;
                    if($type == 'json') {
                        $this->addDataForFile($filename, $group);
                    } else if($type == 'cnt') {
                        self::ImportCnt($serial, $group, $filename, $values['timezone'], $logger);
                    }
                }
                $this->_flashMessenger->addMessage($this->_translate->_('THE_FILE_WAS_SUCCESSFULLY_IMPORTED.'));
                $this->_helper->redirector('index', 'index', 'sources');
            } else {
                $form->populate($values);
            }
        }

        $this->view->form = $form;
    }

    public static function ImportCnt($serial, $group, $filename, $timezone, $logger = FALSE, &$added = FALSE) {
        if ($logger) {
            $opts =  array("verbose" => true, "logger" => $logger);
        } else {
            $opts = array("verbose" => false);
        }
        $cnt = new WebReport_FileFormats_Cnt($filename, $opts);
        $original_timezone = date_default_timezone_get();
        date_default_timezone_set($timezone);
        $blocks = $cnt->getBlocks();
        foreach ($blocks as $block) {
            $application = $block->getApplication();
            if (!$application || $application->getId() != 1) {
                throw new Exception("Not valid application. Only People Counters are valid");
            }
            $cameraName = $block->getName();
            $cntSets = $block->getCntSets();

            foreach ($cntSets as $set) {
                set_time_limit(0);
                $delta = $set->getDelta();
                $startTime = $set->getStartTime();
                $endTime = $startTime + $delta*(count($set->getData())-1)/1000;

                $data = $set->getData();
                $i = 0;
                foreach ($data as $d) {
                    $time = $startTime + $i*$delta/1000;
                    $dbTime = date('Y-m-d H:i:s', $time);
                    $input = (object)array(
                        "username" => NULL,
                        "pass"     => NULL,
                        "version"  => 3,
                        "cameraName" => $cameraName,
                        "serial"   => $serial,
                        "regcode"  => "?",
                        "fields"   => 8+count($d),
                        "stats"    => $d,
                        "cnttime" => $dbTime,
                        "nbrOfTypes" => count($d)
                    );
                    if ($group) {
                        Core_Model_CountData::addDataForSeparatedString($input, NULL, false, $group, true);
                    }
                    if ($added !== FALSE) {
                        $added[] = $input;
                    }

                    $i++;
                }
            }
        }

        date_default_timezone_get($original_timezone);
    }

    public function addCameraGroupAction()
    {
        $form = new Admin_Form_AddEditCameraGroup();
        $this->view->headTitle("ADD", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector('browse-cameras','data-storage-management','admin');
        }

        if ($this->_request->isPost()) {
            $values = $_POST;

            if ($form->isValid($values)) {
                $dynamicSalt = CM_User::createSalt();

                $cameraGroupsDb = Core_Model_CameraGroups::getInstance();
                $cameraGroupsData = array(
                    'name' => $values['name'],
                    'username' => $values['username'],
                    'password' => md5(STATIC_SALT . $values['password'] . $dynamicSalt),
                    'password_salt' => $dynamicSalt,
                    'description' => $values['description'],
                    'email' => $values['email']
                );

                $cameraGroup = $cameraGroupsDb->createRow($cameraGroupsData);
                $cameraGroup->save();
                $cameraGroupId = $cameraGroup->camera_group_id;
                $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();

                if ($this->_user->multi_company) {
                    $compGroup = $companyCameraGroupDb->createRow(array(
                        'company_id' => $this->_group->company_id,
                        'camera_group_id' => $cameraGroupId
                    ));
                    $compGroup->save();
                } else {
                    $compGroup = $companyCameraGroupDb->createRow(array(
                        'company_id' => $this->_user->company_id,
                        'camera_group_id' => $cameraGroupId
                    ));
                    $compGroup->save();
                }

                $document = Core_Model_DocumentFactory::add($cameraGroup->camera_group_id, "camera_group", $this->_group);

                $this->_flashMessenger->addMessage($this->_translate->_('CAMERA_GROUP_ADDED.'));
                $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin', array('folder' => $this->_group->group_id));

            } else {
                $form->populate($_POST);
            }
        }

        $this->view->form = $form;
    }

    public function deleteCameraGroupAction()
    {
        set_time_limit(300);
        $groupId = $this->_getParam("id");

        $document = Core_Model_DocumentFactory::fetch($groupId, "camera_group");
        if(!$document->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_DELETE_THIS_CAMERA_GROUP.'));
            $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin');
            return;
        }

        if (!$groupId)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $cameraGroupsDb = Core_Model_CameraGroups::getInstance();
        $group = $cameraGroupsDb->find($groupId)->current();

        if (!$group)
            throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $groupName = $group->name;

        if (!$this->_user->multi_company) {
            $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
            $companyGroups = $companyCameraGroupDb->fetchAll($companyCameraGroupDb->select()->where('camera_group_id = ?', $groupId));

            if (count($companyGroups) == 1) {
                $cg = $companyGroups->current();
                if ($cg->company_id != $this->_user->company_id) {
                    $this->_flashMessenger->addMessage($this->_translate->_("YOUR_COMPANY_DOES_NOT_USE_THIS_CAMERA_GROUP.") . " ". $this->_translate->_("YOU_ARE_THEREFORE_NOT_ALLOWED_TO_DELETE_IT."));
                    $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin');
                    return;
                } else {
                    $group->delete();
                }
            } else {
                $this->_flashMessenger->addMessage($this->_translate->_("IT_IS_NOT_ONLY_YOUR_COMPANY_THAT_USES_THIS_CAMERA_GROUP.") . " ". $this->_translate->_("YOU_ARE_THEREFORE_NOT_ALLOWED_TO_DELETE_IT."));
                $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin');
                return;
            }
        } else {
            $group->delete();
        }

        Core_Model_DocumentFactory::remove($groupId, "camera_group");
        $this->_flashMessenger->addMessage($this->_translate->_('CAMERA_GROUP_DELETED_WITH_ALL_ITS_DATA.'));
        $this->_helper->redirector('browse-camera-groups', 'data-storage-management', 'admin', array('folder' => $this->_group->group_id));
        set_time_limit(30);
    }

    public function addTestCameraAction() {
        $form = new Admin_Form_AddTestCamera();

        $this->view->form = $form;
    }

    public function browseCamerasAction()
    {
        /*
         *This function now also creates a folder view by mimicing WebReport_Controller_Helper_Browse class and utilizing it's cookie. 
         *Direct implementation of that class wasn't feasible.
         */
        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();

        $request = $this->_request;
        $group = $this->_group;
        $folder = $request->getParam('folder');
        $user = $this->_user;
        $helper = $this->_helper;

        if(!isset($folder) && isset($_COOKIE[WebReport_Controller_Helper_Browse::COOKIE_NAME])) {
            $helper->redirector($request->getActionName(),
                                $request->getControllerName(),
                                $request->getModuleName(),
                                array('folder' => $_COOKIE[WebReport_Controller_Helper_Browse::COOKIE_NAME]));
        }
        if($group && $group->canView($user)) {
            /*
             * Create list and get objects in Group.
             */
            $this->view->headTitle("CAMERAS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
            $applicationsModel = Core_Model_Applications::getInstance();

            $this->view->title = $this->_translate->_("BROWSE_CAMERAS");

            $camerasDb = new Core_Model_Cameras();
            $cameras = $camerasDb->fetchActiveCameras();
            $this->view->active_channels = count($cameras);

            $cameraGroupId = $group->camera_group_id;
            $this->view->cameraGroupId = $cameraGroupId;

            if(!$cameraGroupId) {
                $visible = $camerasDb->fetchAllVisible($this->_user);
                $editable = $camerasDb->fetchAllEditable($this->_user);
            } else {
                $document = Core_Model_DocumentFactory::fetch($cameraGroupId, 'camera_group');

                $cameraGroupsDb = new Core_Model_CameraGroups();

                $cameraGroup = $cameraGroupsDb->find($cameraGroupId)->current();

                if(!$cameraGroup) {
                    $this->view->list = Sources_Model_CameraListGenerator::CreateView(NULL, $group);
                    setcookie(WebReport_Controller_Helper_Browse::COOKIE_NAME, $group->group_id, time()+WebReport_Controller_Helper_Browse::DURATION, '/');
                    return;
                }

                if($document->canEdit($this->_user)) {
                    $visible = array();
                    $editable = $cameraGroup->getCameras();
                } else if($document->canView($this->_user)) {
                    $editable = array();
                    $visible = $cameraGroup->getCameras();
                } else {
                    $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_VIEW_CAMERA_GROUP.'));
                    $this->_helper->redirector('browse-cameras', 'data-storage-management');
                    return;
                }

            }
            //    $visible = array_diff($visible, $editable);

            $ret = array();
            $can_edit = array();
            foreach ($visible as $c) {
                $application = $applicationsModel->findWithId($c->application_id);
                if ($application) {
                    $can_edit[$c->camera_id] = FALSE;
                    $ret[$application->getName()][] = $c;
                }
            }

            foreach ($editable as $c) {
                $application = $applicationsModel->findWithId($c->application_id);
                if ($application) {
                    $can_edit[$c->camera_id] = TRUE;
                    $ret[$application->getName()][] = $c;
                }
            }

            $this->view->cameras = $ret;
            $this->view->editable = $can_edit;
            $this->view->list = Sources_Model_CameraListGenerator::CreateViews($ret, $group);

            setcookie(WebReport_Controller_Helper_Browse::COOKIE_NAME, $group->group_id, time()+WebReport_Controller_Helper_Browse::DURATION, '/');
        }
        else {
            /*
             * Generate root view.
             */
            $this->view->title = $this->_translate->_("BROWSE_CAMERAS");
            $this->view->list = Sources_Model_CameraListGenerator::CreateView(NULL, WebReport_View_Helper_GenericListView::ROOT_FOLDER);
            setcookie(WebReport_Controller_Helper_Browse::COOKIE_NAME, 0, time()+WebReport_Controller_Helper_Browse::DURATION, '/');
        }

    }


    public function deleteCameraAction()
    {
        $id = (int)$this->_getParam('id');

        if (!$id) {
            throw new WebReport_Exception($this->_translate->_('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));
        }

        $camerasDb = new Core_Model_Cameras();
        $camera = $camerasDb->find($id)->current();

        if (!$camera) {
            throw new WebReport_Exception($this->_translate->_('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));
        }

        $document = Core_Model_DocumentFactory::fetch($camera->camera_group_id, "camera_group");
        if(!$document->canEdit($this->_user)) {
            throw new WebReport_Exception($this->_translate->_('YOU_CANNOT_DELETE_THIS_CAMERA.'));
        }
        $camera_group_id = $camera->camera_group_id;
        $camera->delete();
        $this->_flashMessenger->addMessage($this->_translate->_('CAMERA_REMOVED.'));
        $document = Core_Model_DocumentFactory::fetch($camera_group_id, "camera_group");
        $folder = $document->getGroup();
        $this->_helper->redirector('index', 'index', 'sources', array('folder' => $folder->group_id));
    }

    public function viewCameraAction()
    {
        $this->view->headTitle("CAMERAS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $this->view->headTitle("VIEW", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $id = (int)$this->_getParam('id');

        if (!$id) {
            throw new WebReport_Exception("No camera found. Incorrect or missing ID.");
        }

        $camerasDb = new Core_Model_Cameras();
        $cameraGroupsDb = new Core_Model_CameraGroups();

        $camera = $camerasDb->find($id)->current();

        if (!$camera) {
            throw new WebReport_Exception($this->_translate->_('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));
        }
        $cameraGroup = $camera->findParentRow('Core_Model_CameraGroups');
        $document = Core_Model_DocumentFactory::fetch($cameraGroup->camera_group_id, "camera_group");

        if(!$document->canView($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_VIEW_THIS_CAMERA.'));
            $this->_helper->redirector('browse-cameras','data-storage-management','admin');
            return;
        }

        $this->view->headTitle($camera->name, Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $applications = new Core_Model_Applications();
        $application = $applications->findWithId($camera->application_id);

        $db = Zend_Registry::get('db');

        $this->view->name = $camera->name;
        $this->view->cameraGroup = $cameraGroup->name;
        $this->view->firstDate = $camera->first_date;
        $this->view->lastDate = $camera->last_date;
        $this->view->application = $application->getName();
        $this->view->serial = $camera->serial ? $camera->serial : _T("UNKNOWN");
        $this->view->regcode = $camera->regcode ? $camera->regcode : _T("UNKNOWN");
        $this->view->ip = $camera->ip;
        $this->view->reportWarnings = $camera->report_warnings;

        if($this->_user->multi_company) {
            $companies = $cameraGroup->findManyToManyRowset('Core_Model_Companies', 'Core_Model_CompanyCameraGroup');
            $compName = function($c) { return $c['name']; };
            $companies = array_map($compName, $companies->toArray());
            $this->view->companies = $companies;
        }    

        $this->view->id = $id;
        $this->view->fetch_me = $camera->fetch_me;

        $warnings = array();

        if ($this->view->aclUrl(array("module" => "admin", "controller" => "warnings", "action" => "browse"), null, 'default')) {
            $warningsDb = new Core_Model_Warnings();
            $warnings = $warningsDb->fetchAll($warningsDb->select()->where('camera_id = ?', $camera->camera_id)->order("created_date DESC"));
            foreach ($warnings as $warning) {
                $warning->setCamera($camera);    
            }
        }

        $this->view->cameraWarnings = $warnings;

        $doc = Core_Model_DocumentFactory::fetch($cameraGroup->camera_group_id, 'camera_group');
        $foldersDb = new Core_Model_Folders();
        $folder = $foldersDb->find($doc->group_id)->current();

        $this->view->folder = $folder->name;

        $this->view->isCountData = $application instanceof Core_Model_Application_CountDataAbstract;
        $this->view->isEventData = $application instanceof Core_Model_Application_EventDataAbstract;

    }

    public function editCameraAction()
    {

        $this->view->headTitle("CAMERAS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $this->view->headTitle("EDIT", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        $form = new Sources_Form_AddEditCamera("edit");

        $id = (int)$this->_getParam('id');

        if (!$id) {
            throw new WebReport_Exception($this->_translate('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));
        }

        $camerasDb = new Core_Model_Cameras();

        if ($this->_user->multi_company) {
            $camera = $camerasDb->find($id)->current();
        } else {
            $cameraGroupsDb = new Core_Model_CameraGroups();
            $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();
            $db = $camerasDb->getAdapter();
            $select = $db->select();
            $select->from(array("cams" => $camerasDb->info('name')), array("camera_id", "name"))
                ->join(array("cg" => $cameraGroupsDb->info('name')), "cams.camera_group_id = cg.camera_group_id", array())
                ->join(array("ccg" => $companyCameraGroupDb->info('name')), "ccg.camera_group_id = cg.camera_group_id", array())
                ->where("cams.camera_id = ?", $id)
                ->where("ccg.company_id = ?", $this->_user->company_id);
            $camera = $db->fetchRow($select);
        }

        if (!$camera) {
            throw new WebReport_Exception($this->_translate('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));
        }

        // TODO: ugly way of doing this, wonder why I did it?
        $camera = $camerasDb->find($camera->camera_id)->current();

        $this->view->headTitle($camera->name, Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        if ($this->getRequest()->isPost()) {
            $sameNameSelect = $camerasDb
                ->select()
                ->where("camera_id != ?",$camera->camera_id)
                ->where("camera_group_id = ?",$camera->camera_group_id)
                ->where("name = ?", $_POST['name']);
            $notUnique = count($camerasDb->fetchAll($sameNameSelect));
            if ($form->isValid($_POST) && !$notUnique) {
                $values = $_POST;

                $logger = Zend_Registry::get('loggers')->getLogger('user');
                if($camera->name != $values["name"]){
                    $logger->notice($this->_user->forename." ".$this->_user->surname.' changed camera name from "'.$camera->name.'" to "'.$values['name'].'" on a camera with id '.$camera->camera_id);
                }
                if($camera->serial != $values['serial']){
                    $logger->notice($this->_user->forename." ".$this->_user->surname.' changed camera serial from "'.$camera->serial.'" to "'.$values['serial'].'" on a camera with id '.$camera->camera_id);
                }
                if($camera->report_warnings != $values['report_warnings']){
                    $logger->notice($this->_user->forename." ".$this->_user->surname.' changed "Report warnings" from '.$camera->report_warnings.' to '.$values['report_warnings'].' on a camera with id '.$camera->camera_id);
                }

                $camera->name = $values['name'];
                $camera->serial = $values['serial'];
                $camera->report_warnings = $values['report_warnings'];
                $camera->save();

                $this->_helper->redirector('index', 'index', 'sources');

            } else {
                if ($notUnique) {
                    $form->getElement("name")
                        ->addError(_T("THIS_NAME_IS_ALREADY_TAKEN_BY_ANOTHER_CAMERA_IN_THE_CURRENT_CAMERA_GROUP."));
                }
                $form->populate($_POST);
            }

        } else {
            $form->populate($camera->toArray());
        }

        $this->view->form = $form;
    }

    public function checkSyncStatusAction(){
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        if(!$this->_user->multi_company)
            throw new WebReport_Exception($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_EDIT_THIS_CAMERA.'));


        $id = (int)$this->_getParam('id');
        $camerasDb = new Core_Model_Cameras();
        $camera = $camerasDb->find($id)->current();

        if(!$camera)
            throw new WebReport_Exception($this->_translate->_('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));

        //check what company the camera belongs to
        $db = $camerasDb->getAdapter();
        $select = $db->select();
        $select ->from('group',array('company_id','group_id'))
            ->join('document','document.group_id = group.group_id',array('type','id','group_id'))
            ->join('camera','document.id=camera.camera_group_id',array('sync_me','camera_id','camera_group_id'))
            ->where('document.type = ?', 'camera_group')
            ->where('camera.camera_id=?',$id);
        $company_id = $db->fetchRow($select)->company_id;

        //check if the company has any other cameras on sync
        $select = $db->select();
        $select ->from('camera',array('camera_group_id','sync_me','camera_id'))
            ->join('document','camera.camera_group_id = document.id',array('type', 'id','group_id'))
            ->join('group','document.group_id = group.group_id',array('company_id'))
            ->where('document.type = ?','camera_group')
            ->where('group.company_id = ?',$company_id)
            ->where('camera.sync_me = 1');
        $has_no_cameras = ($db->fetchRow($select))==null;
        $this->_helper->Json->sendJson(array('has_no_cams' => $has_no_cameras));

    }

    /*
     * Toggle if camera should be synced with Cogadmin or not.
     */
    public function toggleCameraAction() {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        if(!$this->_user->multi_company)
            throw new WebReport_Exception($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_EDIT_THIS_CAMERA.'));


        $id = (int)$this->_getParam('id');
        $camerasDb = new Core_Model_Cameras();
        $camera = $camerasDb->find($id)->current();

        if(!$camera)
            throw new WebReport_Exception($this->_translate->_('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));


        $camera->sync_me^=TRUE;
        $camera->save();

        if($camera->sync_me){
            if($camera->hasValidCertificate()){
                $status = 'active';
            }else{
                $status = 'tentative';
                $has_tentative_file = APPLICATION_PATH . "/../public/tmp/has_tentative_cams";
                file_put_contents($has_tentative_file, 1);
            }
        }else{
            $status = 'inactive';
        }

        $this->_helper->Json->sendJson(array('status' => $status));
    }

    public function uploadSynchronizationAction() {
        if(!$this->_user->multi_company)
            throw new WebReport_Exception($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_PERFORM_THIS_ACTION.'));

        if($this->_request->isPost()) {
            $upload = new Zend_File_Transfer_Adapter_Http();
            $upload->receive();
            $content = file_get_contents($upload->getFileName(), NULL, NULL, 0, 10*1024*1024);

            $json_data = base64_decode($content);
            $data = json_decode($json_data);

            if(is_null($json_data) || is_null($data)) {
                $this->_flashMessenger->addMessage($this->_translate->_('COULDNT_DECODE_FILE.'));
            }
            foreach($data as $serial => $obj) {
                $cameras = Core_Model_Cameras::getInstance();
                $select = $cameras->select()->where('serial = ?', $serial); 

                foreach($cameras->fetchAll($select) as $camera) {
                    $camera->signature = json_encode($obj);
                    $camera->save();
                    $this->_flashMessenger->addMessage(sprintf($this->_translate->_('UPDATED_"%1$s"'), $camera->name));
                }
            }
        }
        $this->_helper->redirector('browse-cameras', 'data-storage-management', 'admin');
    }

    public function downloadRegistrationAction() {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        $this->_response->setHeader('Content-Type', 'application/octet-stream');
        $this->_response->setHeader('Content-Disposition', 'attachment; filename="wr_offline_reg_'.time().'.b64"');

        if(!$this->_user->multi_company)
            throw new WebReport_Exception($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_PERFORM_THIS_ACTION.'));

        echo base64_encode(json_encode(array(
            'url' => WebReport_Url::getBaseUrl(),
            'registration_code' => Zend_Registry::get('config-license')->registrationCode,
            'identifier' => Zend_Registry::get('config')->cogadmin->identifier
        )));
    }

    public function downloadSynchronizationAction() {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();
        $this->_response->setHeader('Content-Type', 'application/octet-stream');
        $this->_response->setHeader('Content-Disposition', 'attachment; filename="wr_synchronization_'.time().'.b64"');

        if(!$this->_user->multi_company)
            throw new WebReport_Exception($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_PERFORM_THIS_ACTION.'));

        // Get cameras that should be synced.
        $camerasDb = new Core_Model_Cameras();
        $select = $camerasDb->select()->where('sync_me = 1');
        $cameras = $camerasDb->fetchAll($select);

        $config = Zend_Registry::get('config');

        $channels = array();
        foreach($cameras as $camera) {
            $channels[$camera->serial] = $camera->getCertificate();
        }

        echo base64_encode(json_encode(array(
            'key' => $config->cogadmin->key,
            'channels' => $channels
        )));
    }

    public function mergeStep0Action() {

        $this->view->headTitle("MERGE", Zend_View_Helper_Placeholds_Container_Abstract::PREPEND);

        $form = new Sources_Form_MergeCamera($step=0);
        $this->view->form = $form;
        if($this->getRequest()->isPost()) {
            if($form->isValid($_POST)) {
                $values = $_POST;
                $this->_helper->redirector('merge-step1/app/' + $values['app_id']);
            } else {
                $form->populate($_POST);
            }
        } else {
            $form->populate(array());
        }
    }

    public function mergeAction()
    {
        $this->view->headTitle("MERGE", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);

        /* @var $logger Zend_Logger_FireBug() */
        $logger = Zend_Registry::get("Zend_Logger_FireBug");
        $step = 0;
        $application_id = -1;

        if($this->getRequest()->isPost()) {
            if(isset($_POST["step"])) {
                $step = (int) $_POST["step"];
            }
            if(isset($_POST["application_id"])) {
                $application_id = (int) $_POST["application_id"];
            }
        }

        $logger->info($step);
        $logger->info($application_id);

        $form = new Sources_Form_MergeCamera("merge", $step,$application_id);

        $this->view->form = $form;

        if ($this->getRequest()->isPost() && $step == 2) {
            if ($form->isValid($_POST)) {
                $values = $_POST;
                $logger->info("valid");
                $cameras = new Core_Model_Cameras();
                $db = Zend_Registry::get('db');

                $source = $cameras->getCamera((int) $values['merge_camera_source_id']);
                $target = $cameras->getCamera((int) $values['merge_camera_target_id']);

                /*
                 * calculate new first_date and new last_date
                 */
                $srcFirstDate = new Zend_Date($source->first_date, 'yyyy-MM-dd HH:mm:ss');
                $tgtFirstDate = new Zend_Date($target->first_date, 'yyyy-MM-dd HH:mm:ss');
                $newFirstDate = $srcFirstDate->compare($tgtFirstDate) < 0 ? $srcFirstDate : $tgtFirstDate;

                $srcLastDate = new Zend_Date($source->last_date, 'yyyy-MM-dd HH:mm:ss');
                $tgtLastDate = new Zend_Date($target->last_date, 'yyyy-MM-dd HH:mm:ss');
                $newLastDate = $srcFirstDate->compare($tgtLastDate) > 0 ? $srcLastDate : $tgtLastDate;

                if($source->application_id != $target->application_id)
                    throw new Exception("applications differ");

                // figure out types
                $apps = new Core_Model_Applications();
                $app = $apps->findWithId($source->application_id);

                $count_data_table = new Core_Model_CountData_Tables();

                $target_fields = $app->getSqlQueryForInTypes($target);
                $source_fields = $app->getSqlQueryForInTypes($source);

                $target_table = $count_data_table->find($target->count_data_table_id)->current()->table;
                $source_table = $count_data_table->find($source->count_data_table_id)->current()->table;

                $data = $app->getRawData(array($source,$target));

                $db->beginTransaction();

                $commit = true;
                $err = false;
                if((int) $values['dry_run']) {
                    $commit = false;
                }

                try {
                    foreach($data as $row) {
                        if ( is_null($row->$source_fields[0]) && is_null($row->$target_fields[0]) )
                            continue;

                        if ( !is_null($row->$source_fields[0]) &&  is_null($row->$target_fields[0]) ) {
                            // add source data to target
                            $update_data = array();
                            for($i=0; $i < count($source_fields); $i++) {
                                $update_data[$target_fields[$i]] = $row->$source_fields[$i];
                            }
                            $db->update($target_table, $update_data, array('cnttime = ?' => $row->cnttime));
                        } else if (  is_null($row->$source_fields[0]) && !is_null($row->$target_fields[0]) ) {
                            // all is well nothing to be done
                        } else if ( !is_null($row->$source_fields[0]) && !is_null($row->$target_fields[0]) ) {
                            switch((int) $values['handle_dup']) {
                            case 0: // interupt
                                throw new Exception("Duplicate data");
                                break;
                            case 1: // sum data
                                $update_data = array();
                                for($i=0; $i < count($source_fields); $i++)
                                    $update_data[$target_fields[$i]] = (string) ((int) $row->$source_fields[$i]) + ((int) $row->$target_fields[$i]);
                                $db->update($target_table, $update_data, array('cnttime = ?' => $row->cnttime));
                                break;
                            case 2: // use target data 'do nothing'
                                break;
                            case 3: // use source data
                                $update_data = array();
                                for($i=0; $i < count($source_fields); $i++)
                                    $update_data[$target_fields[$i]] = $row->$source_fields[$i];
                                $db->update($target_table, $update_data, array('cnttime = ?' => $row->cnttime));
                                break;
                            }
                        }
                    }
                } catch (Exception $e) {
                    $commit = false;
                    $err = true;
                    $logger->err($e);
                    r_log($e->getMessage(), "error: ");
                }
                if($commit && !$err) {
                    $db->commit();

                    $target->first_date = $newFirstDate->toString('yyyy-MM-dd HH:mm:ss');
                    $target->last_date = $newLastDate->toString('yyyy-MM-dd HH:mm:ss');
                    $target->save();

                    $this->_flashMessenger->addMessage($this->_translate->_('MERGE_WAS_SUCCESSFUL.'));
                    $logger->info($this->_translate->_('MERGE_WAS_SUCCESSFUL.'));
                } else if (!$commit && !$err) {
                    $db->rollBack();
                    $this->_flashMessenger->addMessage($this->_translate->_('MERGE_WAS_SUCCESSFUL_DRY_RUN.'));
                    $logger->info($this->_translate->_('MERGE_WAS_SUCCESSFUL_DRY_RUN.'));
                } else if (!$commit && $err ) {
                    $db->rollBack();
                    $this->_flashMessenger->addMessage($this->_translate->_('MERGE_WAS_ABORTED_NO_DATA_CHANGE.'));
                    $logger->info($this->_translate->_('MERGE_WAS_ABORTED_NO_DATA_CHANGE.'));
                } else {
                    throw new Exception("Unhandled case commit:". $commit ."err:". $err);
                }

                if((int) $values['remove_source'] && $commit) {
                    $source->delete();
                    $logger->info($this->_translate->_('REMOVED_SOURCE_CAMERA.'));
                    $this->_flashMessenger->addMessage($this->_translate->_('REMOVED_SOURCE_CAMERA.'));
                }
                //Zend_Debug::dump($this->_flashMessenger->getMessages());
                $this->_helper->redirector('index', 'index', 'sources', array('folder'=>$_COOKIE[WebReport_Controller_Helper_Browse::COOKIE_NAME]));

            } else {
                $form->populate($_POST);
                $logger->warn("invalid");
            }
        } else {
            $form->populate(array());
        }

        $this->view->form = $form;
    }

    /*
     * Dump camera data as JSON.
     */
    public function dumpCameraAction()
    {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        $id = (int)$this->_getParam('id');
        $camerasDb = new Core_Model_Cameras();
        $camera = $camerasDb->find($id)->current();


        if (!$camera) {
            throw new WebReport_Exception($this->_translate->_('NO_CAMERA_WITH_SUCH_ID_EXISTS.'));
        }

        $this->exportCamerasToJson(array($camera), $camera->name);
    }

    /*
     * Dump camera group data as JSON.
     */
    public function dumpCameraGroupAction()
    {
        $this->_helper->layout->disableLayout();
        $this->_helper->viewRenderer->setNoRender();

        $id = (int)$this->_getParam('folder');

        $camerasDb = Core_Model_Cameras::getInstance();
        $cameraGroupDb = Core_Model_CameraGroups::getInstance();
        $cameraGroup = $cameraGroupDb->find($this->_group->camera_group_id)->current();

        $select = $camerasDb->select()->where('camera_group_id = ?', $cameraGroup->camera_group_id);
        $cameras = $camerasDb->fetchAll($select);

        $this->exportCamerasToJson($cameras, $this->_group->name);
    }

    private function exportCamerasToJson($cameras,$title ) {
        header('Content-Description: File Transfer');
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="'.$title.'.json.gz"');
        header('Content-Transfer-Encoding: binary');
        header('Connection: Keep-Alive');
        header('Expires: 0');
        $meta = array();
        $cntdata = new Core_Model_CountData();
        foreach ($cameras as $camera) {
            if ($camera->hasEventData()){
                $datatype = "eventdata";
            } else {
                $datatype = "cntdata";
            }
            $utc_offset = $camera->utc_offset;
            //To get valid json, might not really be necessary
            if (!isset($utc_offset)) {
                $utc_offset = "";
            }
            $meta[$datatype][$camera->camera_id] = array(
                'name' => $camera->name,
                'id'   => $camera->camera_id,
                'app' => $camera->application_id,
                'serial' => $camera->serial,
                'regcode' => $camera->regcode,
                //'data_last_utc_timestamp' => $camera->data_last_utc_timestamp,
                'utc_offset' => $utc_offset);
            $document = Core_Model_DocumentFactory::fetch($camera->camera_group_id, "camera_group");
            if (!$document->canView($this->_user)) {
                throw new WebReport_Exception($this->_translate->_('YOU_CANNOT_DUMP_THIS_CAMERA.'));
            }
            if ($camera->hasEventData()) {
                $eventdata = new Core_Model_EventData();
                $openingHours = Core_Model_OpeningHours::getInstance();
                $eventdata->addCameras($camera);
                $eventdataOutput[$camera->camera_id]['data'] = $eventdata->getData(null, null, true);
                $eventdataOutput[$camera->camera_id]['openinghours'] = $openingHours->fetchAllOpeningHoursForCameraId($camera->camera_id)->toArray();
            } else {
                $cntdata->addCamera($camera);
            }
        }
        $cntdata_cameras = $cntdata->getCameras();
        if (!empty($cntdata_cameras)) {
          $cntdata->initDatabaseSelect(true);
          $appDb = new Core_Model_Applications();
          $app = $appDb->findWithId($camera->application_id);
          $res = $app->getTimeResolutions();
          $res = preg_grep("/^ts\d+min$/", array_keys($res));
          $res = str_replace('min', '', str_replace('ts','',$res));
          sort($res);
          $res = "ts" . $res[0] . "min";
          $cntdataOutput = $cntdata->getData($res);
        } else {
          $cntdataOutput = array();
        }

        //To get valid json, might not really be necessary
        if (!isset($eventdataOutput)) {
            $eventdataOutput = array();
        }
        $data = array(
            'meta' => $meta,
            'cntdata' => $cntdataOutput,
            'eventdata' => $eventdataOutput
        );
        $data = json_encode($data);
        echo gzencode($data, 9, FORCE_GZIP);
    }

    /**
     * Function to catch translatable strings in Poedit.
     *
     * @return void
     */
    private function _translatableStrings() {


        $this->_translate->_('BROWSE_CAMERA_GROUPS');
        $this->_translate->_('ADD_CAMERA_GROUP');

        $this->_translate->_('COMPANIES');
        $this->_translate->_('CAMERA_GROUP_NAME');
        $this->_translate->_('USER');
        $this->_translate->_('USER_PASSWORD');
        $this->_translate->_('DATA_STORAGE');
        $this->_translate->_('HANDLE_CAMERA_GROUPS');
        $this->_translate->_('USERNAME_MUST_BE_UNIQUE_AND_NON_EMPTY.');
        $this->_translate->_('USERNAME_AND_PASSWORD_DOES_NOT_NEED_TO_BE_THE_SAME_AS_YOUR_ACCOUNT_CREDENTIALS.');

        $this->_translate->_('BROWSE_CAMERAS');
        $this->_translate->_('VIEW_CAMERA');
        $this->_translate->_('EDIT_CAMERA');
        $this->_translate->_('DELETE_CAMERA');
        $this->_translate->_('HANDLE_CAMERAS');
        $this->_translate->_('DATA_STORAGE_AND_MANAGEMENT');
        $this->_translate->_('ALSO_KNOWN_AS_THE_MAC_ADDRESS_OF_THE_CAMERA.');
        $this->_translate->_('SERIAL_NUMBER');
        $this->_translate->_("UNCHECK_TO_DISABLE_REPORTING_OF_WARNINGS.");
        $this->_translate->_("REPORT_WARNINGS");
        $this->_translate->_("MERGE_CAMERAS");
        $this->_translate->_("MERGE");
    }



}
