<?php

class Sources_FoldersController extends WebReport_Controller_Action
{
    protected $_user, $_group, $_cameraGroup;

    public function init()
    {
        parent::init();

        $form = new UniForm_Form(); // include uni-form.css

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = new Core_Model_Folders();
        $this->_group = $groupsDb->find($groupId)->current();

        if (isset($this->_group)) {
            $cameraGroupsDb = Core_Model_CameraGroups::getInstance();
            $this->_cameraGroup = $cameraGroupsDb->find($this->_group->camera_group_id)->current();
            $group_id = $this->_group->group_id;
        } else {
            $group_id = 0;
        }
        $usersDb = new Core_Model_Users();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        //TODO: Submenu is now in layout per module, not controller, so should be defined in module, not here.
        $items = array();
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "index", "action" => "index"), "text" => _T('BROWSE_SOURCES'), 'tooltip' => _T('BROWSE_SOURCES_TOOLTIP'));
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "data-storage-management", "action" => "merge"), "text" => _T('MERGE_CAMERAS'), 'tooltip' => _T('MERGE_CAMERAS_TOOLTIP'));
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "folders", "action" => "browse", "folder" => $group_id), "text" => _T('EDIT_FOLDERS'), 'tooltip' => _T('EDIT_FOLDERS_TOOLTIP'));

        /*if(!$this->_group && $this->_user->multi_company || $this->_user->administrator)
            $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add", "folder" => $groupId), "text" => 'ADD_ROOT_FOLDER');*/
            //$items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "add"), "text" => 'ADD_ROOT_FOLDER');

        //$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);
        $this->view->headTitle("SOURCES", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
        $this->view->headTitle("FOLDERS", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function browseAction()
    {
        $group = $this->_group;
        $cname = WebReport_Controller_Helper_Browse::COOKIE_NAME;
        $dur = WebReport_Controller_Helper_Browse::DURATION;

        $folder = $this->_request->getParam('folder');
        if(!isset($folder) && isset($_COOKIE[$cname])) {
            foreach ($this->view->messages as $message) {
                $this->_flashMessenger->addMessage($this->_translate->_($message));
            }
            $this->_helper->redirector('browse','folders','sources', array('folder' => $_COOKIE[$cname]));
        }

        if($group && !$group->canView($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_VIEW_THIS_FOLDER.'));
            $this->_helper->redirector('browse','folders','sources');
            return;
        }

        if(!$group) {
            setcookie($cname, 0, time()+$dur, '/');
            $group = WebReport_View_Helper_GenericListView::ROOT_FOLDER;
        } else
            setcookie($cname, $group->group_id, time()+$dur, '/');

        $this->view->group = $group;
        $this->view->cameraGroup = $this->_cameraGroup;
        $this->view->user = $this->_user;

        if ($this->_flashMessenger->hasMessages())
            $this->view->messages = $this->_flashMessenger->getMessages();
    }

    public function addAction() {
        /* This function has a lot of ugly comments as I've tried to keep most of the two step code (to add user permissions) without actually using it. */

        $user = Zend_Auth::getInstance()->getIdentity();

        $return_to = $this->_request->getParam('return_to');
        $return_action = $this->_request->getParam('return_action');
        $redirect_options[] = array();
        $redirect_options = array(
                "action" => $return_action,
                "controller" => $return_to,
                "module" => "sources");


        if($this->_group && !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_SAVE_TO_THIS_FOLDER.'));
            $this->_helper->redirector($redirect_options["action"],$redirect_options["controller"],$redirect_options["module"],array('folder' => $this->_group->group_id));
            return;
        }

        if(!$this->_group && !$this->_user->multi_company && !$this->_user->administrator) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_CREATE_ROOT_FOLDERS._ONLY_FOLDER_MANAGERS_CAN_CREATE_FOLDERS_IN_THE_ROOT.'));
            $this->_helper->redirector($redirect_options["action"],$redirect_options["controller"],$redirect_options["module"], array('folder' => 0));
            return;
        }

        if(!$this->getRequest()->isPost()) {
            //$this->view->title = $this->_translate->_('ADD_SUBFOLDER') . ": " . sprintf($this->_translate->_('STEP_%1$s'), 1);
            $this->view->title = $this->_translate->_('ADD_FOLDER');
            $form = new Sources_Form_AddGroupStep1();
            if($this->_group)
                $form->removeElement('company_id');

        } else {
            $values = $_POST;
            //if(array_key_exists('step', $values) && $values['step'] == 1) {
                $form = new Sources_Form_AddGroupStep1();
                if($this->_group)
                    $form->removeElement('company_id');

                if($form->isValid($values)) {
                    $companiesDb = new Core_Model_Companies();
                    if($this->_group)
                        $company = $companiesDb->find($this->_group->company_id)->current();
                    else
                        $company = $companiesDb->find($values['company_id'])->current();

                    $users = $company->fetchAllActiveUsers();
                    $groups = $company->fetchAllGroups();

                    //$this->view->form = new Admin_Form_AddGroupStep2($values, $users, $groups);

                    //$this->view->title = $this->_translate->_('ADD_FOLDER') . ": " . sprintf($this->_translate->_('STEP_%1$s'), 2);

                } else {
                    $form->populate($values);
                }
            //} else {
                //$oldValues = json_decode($values['values'], true);
                $oldValues = $values;
                $companiesDb = new Core_Model_Companies();

                if($this->_group)
                    $company = $companiesDb->find($this->_group->company_id)->current();
                else
                    $company = $companiesDb->find($oldValues['company_id'])->current();

                $users = $company->fetchAllActiveUsers();
                $groups = $company->fetchAllGroups();

                //$form = new Admin_Form_AddGroupStep2($oldValues, $users, $groups);

                if($form->isValid($values)) {
                    if(!$user->multi_company &&
                       $user->company_id != $company->company_id) {
                        throw new WebReport_Exception_ParameterError($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_ADD_THIS_FOLDER.'));
                    }


                    /*
                     * TODO:
                     * This is done very similarily in CompaniesController addAction()
                     * and InstallationController userAndCompanyAction()
                     * Make it utilize same code!
                     */
                    /*
                     * Add camera group.
                     */
                    $dynamicSalt = CM_User::createSalt();

                    $cameraGroupsDb = Core_Model_CameraGroups::getInstance();
                    $cameraGroupsData = array(
                        'name' => $oldValues['name'], //Will get same name as folder.
                        'username' => $oldValues['username'],
                        'no_data_timeout' => $oldValues['no_data_timeout'],
                        'no_connection_timeout' => $oldValues['no_connection_timeout'],
                        'password' => md5(STATIC_SALT . $oldValues['password'] . $dynamicSalt),
                        'password_salt' => $dynamicSalt,
                        'description' => $oldValues['description'], //Will get same description as folder.
                        'email' => $oldValues['email']
                    );

                    $cameraGroup = $cameraGroupsDb->createRow($cameraGroupsData);
                    $cameraGroupId = $cameraGroup->save();

                    /*
                     * Create group (folder).
                     */
                    $groupsDb = new Core_Model_Folders();

                    if($this->_group)
                        $company_id = $this->_group->company_id;
                    else
                        $company_id = $oldValues['company_id'];

                    $groupData = array('company_id' => $company_id,
                                       'name' => $oldValues['name'],
                                       'description' => $oldValues['description'],
                                       'camera_group_id' => $cameraGroupId,
                                       'camera_conn_enabled' => $oldValues['conn_enabled']);

                    if(!$this->_group)
                        $groupData['parent_id'] = NULL;
                    else
                        $groupData['parent_id'] = $this->_group->group_id;

                    $group = $groupsDb->createRow($groupData);
                    $group->save();

                    /*
                     * Add document.
                     */
                    $document = Core_Model_DocumentFactory::add($cameraGroupId, "camera_group", $group);


                    // /*
                    //  * Add users.
                    //  */
                    // if(array_key_exists('folder_members', $values)) {
                    //     $usersDb = new Core_Model_Users();

                    //     $addUsers = $values['folder_members'];

                    //     $creators = array();
                    //     if(array_key_exists('creators', $values)) {
                    //         $creators = $values['creators'];
                    //     }

                    //     /*
                    //      * Add viewers.
                    //      */
                    //     foreach(array_diff($addUsers, $creators) as $user_id)
                    //         $group->add($usersDb->find($user_id)->current(), 'view');

                    //     foreach(array_diff($addUsers, $creators) as $user_id)
                    //         $group->add($usersDb->find($user_id)->current(), 'view');
                    //     foreach(array_diff($addUsers, $creators) as $user_id)
                    //         $group->add($usersDb->find($user_id)->current(), 'view');
                    //     foreach(array_diff($addUsers, $creators) as $user_id)
                    //         $group->add($usersDb->find($user_id)->current(), 'view');
                    //     foreach(array_diff($addUsers, $creators) as $user_id)
                    //         $group->add($usersDb->find($user_id)->current(), 'view');
                    //     /*
                    //      * Add creators.
                    //      */
                    //     foreach(array_intersect($addUsers, $creators) as $user_id)
                    //         $group->add($usersDb->find($user_id)->current(), 'create');

                    // }

                    $companyCameraGroupDb = new Core_Model_CompanyCameraGroup();

                    if ($this->_user->multi_company) {
                        $company_id = $oldValues['company_id']; //Creating a root folder with company chosen in previous form.
                        if(is_null($company_id)){
                            $company_id = $this->_group->company_id; //Not creating a root folder, group available.
                        }
                        $compGroup = $companyCameraGroupDb->createRow(array(
                            'company_id' => $company_id,
                            'camera_group_id' => $cameraGroupId
                        ));
                        $compGroup->save();
                    } else {
                        $compGroup = $companyCameraGroupDb->createRow(array(
                            'company_id' => $this->_user->company_id,
                            'camera_group_id' => $cameraGroupId
                        ));
                        $compGroup->save();
                    }


                    $this->_flashMessenger->addMessage($this->_translate->_('FOLDER_ADDED.'));
                    $this->_helper->redirector($redirect_options["action"],$redirect_options["controller"],$redirect_options["module"], array('folder' => $group->group_id));

                } else {
                    $form->populate($values);
                }

        }

        $this->view->form = $form;
    }

    public function editAction() {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_EDIT_THIS_FOLDER.'));
            $this->_helper->redirector('browse','folders','sources', array('folder' => $this->_group->group_id));
        }

        $form = new Sources_Form_EditGroup($this->_group);

        if(!$this->getRequest()->isPost()) {

            $this->view->title = sprintf($this->_translate->_('EDIT_FOLDER_BASICS_FOR_"%1$s".'), $this->_group->name);

            $data = array('name' => $this->_group->name,
                          'description' => $this->_group->description,);
            $form->populate($data);
        } else {
            $values = $_POST;
            if($form->isValid($values)) {
                $this->_group->name = $values['name'];
                $this->_group->description = $values['description'];
                $this->_group->save();

                $this->_flashMessenger->addMessage($this->_translate->_('FOLDER_UPDATED.'));
                $this->_helper->redirector('browse','folders','sources', array('folder' => $this->_group->group_id));
            } else {
                $form->populate($values);
            }
        }

        $this->view->form = $form;
    }

    public function editCcSettingsAction() {
        if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_EDIT_THIS_FOLDER.'));
            $this->_helper->redirector('browse','folders','sources', array('folder' => $this->_group->group_id));
        }

        //$companiesDb = new Core_Model_Companies();
        //$company = $companiesDb->find($this->_group->company_id)->current();

        $cameraGroup = $this->_cameraGroup;
        $group = $this->_group;

        if (!$cameraGroup)
        throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        //$users = $company->fetchAllActiveUsers();

        $form = new Sources_Form_EditGroupCCSettings($this->_group);
        $this->view->group = $this->_group;
        $this->view->user = $this->_user;

        if(!$this->getRequest()->isPost()) {

            $this->view->title = sprintf($this->_translate->_('EDIT_CONNECTION_SETTINGS_FOR_"%1$s".'), $this->_group->name);

            $data = array('username' => $cameraGroup->username,
                          'no_data_timeout' => $cameraGroup->no_data_timeout,
                          'no_connection_timeout' => $cameraGroup->no_connection_timeout,
                          'email' => $cameraGroup->email,);

            $form->populate($data);
        } else {
            $values = $_POST;
            if($form->isValid($values)) {
                if($values['conn_enabled']){
                    $cameraGroup->username = $values['username'];
                    $cameraGroup->no_data_timeout = $values['no_data_timeout'];
                    $cameraGroup->no_connection_timeout = $values['no_connection_timeout'];
                    $cameraGroup->email = $values['email'];
                    $dynamicSalt = CM_User::createSalt();
                    if (!$values['password']==""){
                            $cameraGroup->password = md5(STATIC_SALT . $values['password'] . $dynamicSalt);
                            $cameraGroup->password_salt = $dynamicSalt;
                    }
                } else {
                    //Remove password on camera connection disable so cameras can't connect.
                    $cameraGroup->password = "";
                    $cameraGroup->password_salt = $dynamicSalt;
                }
                $cameraGroup->save();

                $group->camera_conn_enabled = $values['conn_enabled'];
                $group->save();

                $this->_flashMessenger->addMessage($this->_translate->_('FOLDER_UPDATED.'));
                $this->_helper->redirector('browse','folders','sources', array('folder' => $this->_group->group_id));
            } else {
                $form->populate($values);
            }
        }

        $this->view->form = $form;
    }

    public function editUserPermissionsAction() {
        /*if(!$this->_group || !$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_EDIT_THIS_FOLDER.'));
            $this->_helper->redirector('browse','folders','sources', array('folder' => $this->_group->group_id));
        }

        $companiesDb = new Core_Model_Companies();
        $company = $companiesDb->find($this->_group->company_id)->current();

        //$cameraGroup = $this->_cameraGroup;

        //if (!$cameraGroup)
        //throw new WebReport_Exception_ParameterError($this->_translate->_('NO_CAMERA_GROUP_WITH_SUCH_ID_EXISTS.'));

        $users = $company->fetchAllActiveUsers();

        $form = new Sources_Form_EditGroupUserPermissions($users,$this->_group);

        $this->view->user = $this->_user;

        if(!$this->getRequest()->isPost()) {

            $data = array();

            $members = $this->_group->getUsers(false);

            $data['folder_members'] = array();
            $data['creators'] = array();

            foreach($members['viewers'] as $viewer)
                $data['folder_members'][] = $viewer->user_id;

            foreach($members['creators'] as $creator) {
                $data['folder_members'][] = $creator->user_id;
                $data['creators'][] = $creator->user_id;
            }

            $form->populate($data);
        } else {
            $values = $_POST;
            if($form->isValid($values)) {

                if(!array_key_exists('creators', $values))
                    $values['creators'] = array();

                if(!array_key_exists('folder_members', $values))
                    $values['folder_members'] = array();

                $add_viewer = array_flip(array_diff($values['folder_members'], $values['creators']));
                $add_creator = array_flip(array_intersect($values['folder_members'], $values['creators']));

                foreach($users as $user) {
                    if(array_key_exists($user->user_id, $add_viewer))
                        $this->_group->add($user, 'view');
                    else if(array_key_exists($user->user_id, $add_creator))
                        $this->_group->add($user, 'create');
                    else
                        $this->_group->remove($user);
                }

                $this->_flashMessenger->addMessage($this->_translate->_('FOLDER_UPDATED.'));
                $this->_helper->redirector('browse','folders','sources', array('folder' => $this->_group->group_id));
            } else {
                $form->populate($values);
            }
        }

        $this->view->form = $form;
    */
    }

    public function deleteAction()
    {
        if(!$this->_group->canEdit($this->_user)) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_DONT_HAVE_PERMISSION_TO_DELETE_THIS_FOLDER.'));
            $this->_helper->redirector('browse','folders','sources', array('folder' => $this->_group->group_id));
        }

        /*if(count($this->_group->getChildren()) > 0 ||
           count($this->_group->getDocuments()) > 0) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_DELETE_A_FOLDER_CONTAINING_DOCUMENTS_OR_FOLDERS.'));
            $this->_helper->redirector('browse', 'folders','admin', array('folder' => $this->_group->group_id));
        }*/

        if(count($this->_group->getChildren()) > 0 ||
           count($this->_cameraGroup->getCameras()) > 0) {
            $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_DELETE_A_FOLDER_CONTAINING_CAMERAS_OR_SUBFOLDERS.'));
            $this->_helper->redirector('browse', 'folders','sources', array('folder' => $this->_group->group_id));
        }

        if(($p = $this->_group->getParent()))
            $parent_id = $p->group_id;
        else
            $parent_id = 0;

        $this->_group->delete();
        $this->_cameraGroup->delete();
        $this->_flashMessenger->addMessage($this->_translate->_('FOLDER_REMOVED.'));
        $this->_helper->redirector('browse','folders','sources', array('folder' => $parent_id));
    }

    public function moveAction()
    {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        $groupsDb = new Core_Model_Folders();
        $dstGroupId = (int) $this->_request->getParam('destination');
        $dstGroup = $groupsDb->find($dstGroupId)->current();

        $cameraDb = new Core_Model_Cameras();
        $cameraId = (int) $this->_request->getParam('camera');
        $camera = $cameraDb->find($cameraId)->current();

        //Check permissions
        if(!$this->_group || !$dstGroup ||
           !$this->_group->canEdit($this->_user) ||
           !$dstGroup->canEdit($this->_user) ) {
               $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_MOVE_THIS_CAMERA.'));
               echo json_encode(array('status' => 'error', 'message' => 'move failed - permissions'));
               return;
        }

        //Check so that there is no camera with the same name in target.
        $camerasInCameraGroup = $cameraDb->select()->where("camera_group_id=?",$dstGroup->camera_group_id,"String")->query();
        foreach ($camerasInCameraGroup as $destCamera) {
            if ($destCamera->name===$camera->name){
               $this->_flashMessenger->addMessage($this->_translate->_('MOVE_FAILED._CAMERA_WITH_THE_SAME_NAME_ALREADY_EXISTS_IN_DESTINATION_FOLDER.'));
               echo json_encode(array('status' => 'error', 'message' => 'move failed - camera already exist in target'));
               return;
            }
        }

        $this->_flashMessenger->addMessage($this->_translate->_('MOVE_SUCCESSFUL.'));
        $camera->camera_group_id = $dstGroup->camera_group_id;
        $camera->save();
        echo json_encode(array('status' => 'success', 'message' => 'move succeeded'));
        return;
    }

    public function searchAction()
    {
        $this->_helper->layout->disableLayout();    //disable layout
        $this->_helper->viewRenderer->setNoRender(); //suppress auto-rendering

        $q = @mysql_escape_string($this->_request->getParam('q'));

        // Also add in models/Documents.php
        $actions = array(
            //'camera_group' => array('module' => 'admin', 'controller' => 'data-storage-management', 'action' => 'browse-camera-groups'),
            'camera' => array('module' => 'sources', 'controller' => 'data-storage-management', 'action' => 'browse-cameras'),
            /*'count_data_set' => array('module' => 'count-data', 'controller' => 'count-data-sets', 'action' => 'browse'),
            'virtual_camera' => array('module' => 'count-data', 'controller' => 'virtual-cameras', 'action' => 'browse'),
            'formula_set' => array('module' => 'count-data', 'controller' => 'formulas', 'action' => 'browse'),
            'chart' => array('module' => 'charts', 'controller' => 'chart-producer', 'action' => 'browse'),
            'table' => array('module' => 'charts', 'controller' => 'table-creator', 'action' => 'browse'),
            'report' => array('module' => 'reports', 'controller' => 'index', 'action' => 'browse'),
            'importjob' => array('module' => 'pos', 'controller' => 'index', 'action' => 'browse'),
            */'folder' => array('module' => 'sources', 'controller' => 'folders', 'action' => 'browse'),
            //'subscription' => array('module' => 'reports', 'controller' => 'subscribe', 'action' => 'browse')
        );
        $types = array(
            //'camera_group' => $this->_translate->_('CAMERA_GROUP'),
            'camera' => $this->_translate->_('CAMERA'),
            /*'count_data_set' => $this->_translate->_('COUNT_DATA_SET'),
            'virtual_camera' => $this->_translate->_('VIRTUAL_CAMERA'),
            'formula_set' => $this->_translate->_('FORMULA_SET'),
            'chart' => $this->_translate->_('CHART'),
            'table' => $this->_translate->_('TABLE'),
            'report' => $this->_translate->_('REPORT'),
            'importjob' => $this->_translate->_('POS'),
            */'folder' => $this->_translate->_('FOLDER'),
            //'subscription' => $this->_translate->_('SUBSCRIPTION')
        );

        //$documentsDb = new Core_Model_Documents();
        $foldersDb = new Core_Model_Folders();
        $camerasDb = new Core_Model_Cameras();

        //$all_results = array_slice(array_merge(array_merge($documentsDb->searchDocuments($q), $foldersDb->searchFolders($q)),$camerasDb->searchCameras($q)),0,10);
        $all_results = array_slice(array_merge(array_merge($foldersDb->searchFolders($q)),$camerasDb->searchCameras($q)),0,10);
        $results=array();
        /*
         * Add URLs.
         */
        foreach($all_results as $result) {
            $params = $actions[$result->type] + array('folder' => $result->group_id);
            if(isset($result->extra)) $params += $result->extra;
            $result->url = $this->view->aclUrl($params, null, 'default');
            $result->type_translated = $types[$result->type];
            if(isset($result->url)) $results[]=$result;
        }

        $this->_helper->Json->sendJson($results);

    }
}
