<?php

/*include_once 'core/models/CameraGroups.php';
include_once 'core/models/CompanyCameraGroup.php';
include_once 'core/models/Companies.php';*/

class Sources_IndexController extends WebReport_Controller_Action
{
    protected $_user, $_group;

    public function init() {
        parent::init();

        $module = $this->getRequest()->module;
        $controller = $this->getRequest()->controller;
        $action = $this->getRequest()->action;

        $groupId = (int) $this->_request->getParam('folder');
        $groupsDb = Core_Model_Folders::getInstance();
        $this->_group = $groupsDb->find($groupId)->current();

        if (isset($this->_group)) {
            $group_id = $this->_group->group_id;
        } else {
            $group_id = 0;
        }

        $usersDb = Core_Model_Users::getInstance();
        $this->_user = $usersDb->find(Zend_Auth::getInstance()->getIdentity()->user_id)->current();

        $items = array();
        //TODO: Submenu is now in layout per module, not controller, so should be defined in module, not here.
        $items[] = array("urlOptions" => array('module' => $module, "controller" => $controller, "action" => "index"), "text" => _T('BROWSE_SOURCES'), 'tooltip' => _T('BROWSE_SOURCES_TOOLTIP'));
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "data-storage-management", "action" => "merge"), "text" => _T('MERGE_CAMERAS'), 'tooltip' => _T('MERGE_CAMERAS_TOOLTIP'));
        $items[] = array("urlOptions" => array('module' => $module, "controller" => "folders", "action" => "browse", "folder" => $group_id), "text" => _T('EDIT_FOLDERS'), 'tooltip' => _T('EDIT_FOLDERS_TOOLTIP'));

        //$items = WebReport_Module_MenuItems_Abstract::filterNotAllowedItems($items);
        $this->view->submenu = WebReport_Module_MenuItems_Abstract::translate($items);

        $features = new Core_Model_Features();

        $this->view->dynamic_channels = $features->dynamicChannels();
        $this->view->module = $module;
        $this->view->headTitle("SOURCES", Zend_View_Helper_Placeholder_Container_Abstract::PREPEND);
    }

    public function indexAction()
    {
        /*
         *This function now also creates a folder view by mimicing WebReport_Controller_Helper_Browse class and utilizing it's cookie.
         *Direct implementation of that class wasn't feasible.
         */
        if ($this->_flashMessenger->hasMessages()) {
            $this->view->messages = $this->_flashMessenger->getMessages();
        }
        $request = $this->_request;
        $group = $this->_group;
        $folder = $request->getParam('folder');
        $user = $this->_user;
        $helper = $this->_helper;

        $this->view->title = $this->_translate->_("SOURCES");

        if(!isset($folder) && isset($_COOKIE[WebReport_Controller_Helper_Browse::COOKIE_NAME])) {
            if(isset($this->view->messages)){
                foreach ($this->view->messages as $message) {
                    $this->_flashMessenger->addMessage($this->_translate->_($message));
                }
            }
            $helper->redirector($request->getActionName(),
                                $request->getControllerName(),
                                $request->getModuleName(),
                                array('folder' => $_COOKIE[WebReport_Controller_Helper_Browse::COOKIE_NAME]));
        }
        if($group && $group->canView($user)) {
            /*
             * Create list and get objects in Group.
             */

            $applicationsModel = Core_Model_Applications::getInstance();

            $camerasDb = new Core_Model_Cameras();
            $cameras = $camerasDb->fetchActiveCameras();
            $this->view->active_channels = count($cameras);

            $cameraGroupId = $group->camera_group_id;
            $this->view->cameraGroupId = $cameraGroupId;

            if(!$cameraGroupId) {
                $visible = $camerasDb->fetchAllVisible($this->_user);
                $editable = $camerasDb->fetchAllEditable($this->_user);
            } else {
                $document = Core_Model_DocumentFactory::fetch($cameraGroupId, 'camera_group');

                $cameraGroupsDb = new Core_Model_CameraGroups();

                $cameraGroup = $cameraGroupsDb->find($cameraGroupId)->current();

                if(!$cameraGroup) {
                    $this->view->list = Sources_Model_CameraListGenerator::CreateView(NULL, $group);
                    setcookie(WebReport_Controller_Helper_Browse::COOKIE_NAME, $group->group_id, time()+WebReport_Controller_Helper_Browse::DURATION, '/');
                    return;
                }

                if($document->canEdit($this->_user)) {
                    $visible = array();
                    $editable = $cameraGroup->getCameras();
                } else if($document->canView($this->_user)) {
                    $editable = array();
                    $visible = $cameraGroup->getCameras();
                } else {
                    $this->_flashMessenger->addMessage($this->_translate->_('YOU_CANNOT_VIEW_CAMERA_GROUP.'));
                    $this->_helper->redirector('browse-cameras', 'data-storage-management');
                    return;
                }

            }
            //    $visible = array_diff($visible, $editable);

            $ret = array();
            $can_edit = array();
            foreach ($visible as $c) {
                $application = $applicationsModel->findWithId($c->application_id);
                if ($application) {
                    $can_edit[$c->camera_id] = FALSE;
                    $ret[$application->getName()][] = $c;
                }
            }

            foreach ($editable as $c) {
                $application = $applicationsModel->findWithId($c->application_id);
                if ($application) {
                    $can_edit[$c->camera_id] = TRUE;
                    $ret[$application->getName()][] = $c;
                }
            }

            $this->view->cameras = $ret;
            $this->view->editable = $can_edit;
            $this->view->list = Sources_Model_CameraListGenerator::CreateViews($ret, $group);

            setcookie(WebReport_Controller_Helper_Browse::COOKIE_NAME, $group->group_id, time()+WebReport_Controller_Helper_Browse::DURATION, '/');
        }
        else {
            /*
             * Generate root view.
             */
            $this->view->list = Sources_Model_CameraListGenerator::CreateView(NULL, WebReport_View_Helper_GenericListView::ROOT_FOLDER);
            setcookie(WebReport_Controller_Helper_Browse::COOKIE_NAME, 0, time()+WebReport_Controller_Helper_Browse::DURATION, '/');
        }

    }

}
