<?php

class Sources_Form_AddGroupStep1 extends UniForm_Form
{
    public function __construct($options = null)
    {
        parent::__construct($options);
    }

    public function isValid($data) {
        if ($data["conn_enabled"] == 0 ) {
                $username = $this->getElement("username");
                $username->clearValidators();
                $username->setRequired(0);

                $password = $this->getElement("password");
                $password->clearValidators('PasswordConfirmation');
                $password->setRequired(0);

                $password_confirm = $this->getElement("password_confirm");
                $password_confirm->clearValidators('PasswordConfirmation');
                $password_confirm->setRequired(0);
            }
        return parent::isValid($data);
    }

    public function init()
    {
        parent::init();

        $this->setAttrib("id", "add-folder-form");
        $this->addElementPrefixPath('CM_Validate_User', 'CM/Validate/User/', 'validate');
        $this->addElementPrefixPath('WebReport_Validate_User', 'WebReport/Validate/User/', 'validate');

        $name = new UniForm_Form_Element_Text('name');
        $name->setLabel('NAME')
            ->setRequired()
            ->addFilter('StringTrim');

        $usernameValidator = new Zend_Validate_Db_NoRecordExists('camera_group', 'username');

        $user = Zend_Auth::getInstance()->getIdentity();

        if(!$user->multi_company) {
            $company_id = new UniForm_Form_Element_Hidden('company_id');
            $company_id->setValue($user->company_id);
        } else {
            $company_id = new UniForm_Form_Element_Select('company_id');
            $company_id->setLabel(_T('COMPANY'))
                ->setRequired()
                ->setErrorMessages(array($this->getView()->translate(_T("MUST_BELONG_TO_A_COMPANY."))));

            $companiesDb = new Core_Model_Companies();
            $companies = $companiesDb->fetchAllActive();
            $company_ids = array();
            foreach($companies as $company) {
                $company_id->addMultiOption($company->company_id, $company->name);
                $company_ids[] = $company->company_id;
            }
                
            $company_id_validator = new Zend_Validate_InArray($company_ids);
            $company_id->addValidator($company_id_validator);
        }


        $description = new UniForm_Form_Element_Textarea('description');
        $description->setLabel(_T('DESCRIPTION'))
            ->addFilter('StringTrim');

        $conn_enabled = new UniForm_Form_Element_Checkbox('conn_enabled');
        $conn_enabled->setLabel('ENABLE_CONNECTIONS_TO_THIS_FOLDER');
        
        $username = new UniForm_Form_Element_Text('username');
        $username->setLabel('FOLDER_CONNECTION_IDENTIFIER')
            ->setRequired()
            ->setAttrib("autocomplete", "off")
            ->addFilter('StringTrim')
            ->addValidator($usernameValidator)
            ->setErrorMessages(array("FOLDER_CONNECTION_IDENTIFIER_MUST_BE_UNIQUE_AND_NON_EMPTY."))
                ->setDescription("THE_FOLDER_CONNECTION_IDENTIFIER_AND_PASSWORD_IS_USED_WHEN_ADDING_CAMERAS_TO_THIS_FOLDER._NOTE_THAT_ON_OLDER_CAMERA_APPLICATION_VERSIONS_THIS_PARAMETER_MIGHT_BE_CALLED_'USER_NAME'.");


        $password = new UniForm_Form_Element_Password('password');
        $password->setLabel('FOLDER_CONNECTION_PASSWORD')
            ->addValidator('PasswordConfirmation')
            ->setRequired();

        $password_confirm = new UniForm_Form_Element_Password('password_confirm');
        $password_confirm->setLabel('CONFIRM_PASSWORD')
            ->setRequired();

        $no_data_timeout = new UniForm_Form_Element_Text('no_data_timeout');
        $no_data_timeout->setLabel(_T("HOURS_WITHOUT_COUNT_DATA_ALLOWED"))
            ->addValidator("Int")
            ->addValidator(new Zend_Validate_GreaterThan(0))
            ->setDescription(_T("A_WARNING_WILL_BE_ISSUED_IF_YOUR_COUNTER_NO_LONGER_SENDS_NEW_COUNTING_DATA_TO_DataManager._THIS_SETTING_IS_THE_NUMBER_OF_HOURS_ALLOWED_TO_PASS_WITHOUT_SENDING_A_WARNING."))
            ->setValue("48");
        $no_connection_timeout = new UniForm_Form_Element_Text('no_connection_timeout');
        $no_connection_timeout->setLabel(_T("HOURS_WITHOUT_COMMUNICATION_ALLOWED"))
            ->addValidator("Int")
            ->addValidator(new Zend_Validate_GreaterThan(0))
            ->setDescription(_T("A_WARNING_WILL_BE_ISSUED_IF_YOUR_COUNTER_NO_LONGER_IS_ABLE_TO_CONTACT_DataManager._THIS_SETTING_IS_THE_NUMBER_OF_HOURS_ALLOWED_TO_PASS_WITHOUT_SENDING_A_WARNING."))
            ->setValue("8");
        $email = new UniForm_Form_Element_Text('email');
        $email->setLabel(_T("EMAIL_FOR_WARNINGS"))
            ->setDescription(_T("IF_YOU_WANT_WARNINGS_SENT_AS_EMAILS,_WRITE_THE RECEIVER_ADDRESS_HERE._THIS_REQUIRES_SMTP_TO_BE_CONFIGURED_BY_THE_SERVER_ADMINISTRATOR."));

        $step = new UniForm_Form_Element_Hidden('step');
        $step->setValue(1);

        $submit = new UniForm_Form_Element_Submit('submit');
        $submit->setLabel(_T('SAVE_FOLDER'));

        $this->addElements(array($name, $company_id, $description, $conn_enabled, $username, $password, $password_confirm, $no_data_timeout, $no_connection_timeout, $email, $step));
        $this->addDisplayGroup(array('name', 'company_id', 'description'),'basics',array('legend' => 'FOLDER_BASICS'));
        $this->addDisplayGroup(array('conn_enabled','username','password','password_confirm', 'no_data_timeout', 'no_connection_timeout', 'email', 'step'), 'connection-group',array('legend' => 'CONNECTION_SETTINGS'));

        $this->addElements(array($submit));
    }
}
