<?php

class Sources_Form_MergeCamera extends UniForm_Form
{
    protected $_action;
    protected $_step;
    protected $_application_id;

    public function __construct($action = "merge",$step = 0, $application_id=-1, $options = null)
    {
        $this->_action = $action;
        $this->_step = $step;
        $this->_application_id = $application_id;

        parent::__construct($options);
    }

    public function init()
    {

        parent::init();

        $submit = new UniForm_Form_Element_Submit('submit');
        $submit->setLabel('SUBMIT');
  
        switch($this->_step) {
            case 0:
                $application = new UniForm_Form_Element_Select('application_id');
                $applications = new Core_Model_Applications();
                $application->setLabel('SELECT_APPLICATION');

                $step = new UniForm_Form_Element_Hidden('step');
                $step->setValue('1');
                $this->addElement($step);
                
                $apps = $applications->fetchAllThatHasVisibleDataInDb();

                foreach($apps as $app) {
                    $application->addMultiOption($app->getId(),$app->getName());
                }

                $this->addElement($application);
                $this->addDisplayGroup(array("application_id"),'chart-group');



                break;
            case 1:
            case 2:

                $this->addElementPrefixPath('CM_Validate', 'CM/Validate','validate');

                $identity = Zend_Auth::getInstance()->getIdentity();
                $camerasDB = new Core_Model_Cameras();

                $applications = new Core_Model_Applications();
                $app = $applications->findWithId($this->_application_id);

                if ($identity->multi_company) {
                    $cams = $camerasDB->getCameras($app,null,array(),true);
                } else {
                    $companiesDB = new Core_Model_Companies();
                    $company = $companiesDB->find($identity->company_id)->current();
                    $cams = $camerasDB->getCameras($app, $company, array(), true);
                }

                $notEqValidator = new CM_Validate_ValueNotEqual();
                $notEqValidator->depend = "merge_camera_target_id";
                $merge_camera_first = new UniForm_Form_Element_Select('merge_camera_source_id');
                $merge_camera_first
                    ->setLabel('MERGE_SOURCE')
                    ->addMultiOption(0,"SELECT_SOURCE")
                    ->setRequired()
                    ->addValidator($notEqValidator);

                $notEqValidator2 = new CM_Validate_ValueNotEqual();
                $notEqValidator2->depend = "merge_camera_source_id";
                $merge_camera_second = new UniForm_Form_Element_Select('merge_camera_target_id');
                $merge_camera_second
                    ->setLabel('MERGE_TARGET')
                    ->addMultiOption(0,"SELECT_TARGET")
                    ->setRequired()
                    ->addValidator($notEqValidator2);
                foreach($cams as $cam) {
                    $merge_camera_first->addMultiOption($cam->camera_id,$cam->name . " (".$cam->group.")");
                    $merge_camera_second->addMultiOption($cam->camera_id,$cam->name . " (".$cam->group.")");
                }

                $dry_run = new UniForm_Form_Element_Checkbox('dry_run');
                $dry_run->setLabel('DRY_RUN')->setDescription('DONT_DO_ANYTHING_CHECK_FOR_DOUBLE_DATA');

                $remove_source = new UniForm_Form_Element_Checkbox('remove_source');
                $remove_source->setLabel("REMOVE_SOURCE")->setDescription('REMOVE_SOURCE_CAMERA_IF_MERGE_WAS_SUCCESS');

                $handle_dup = new UniForm_Form_Element_Select("handle_dup");
                $handle_dup
                    ->setLabel("HANDLE_DUPLICATES")
                    ->setRequired()->setDescription('MERGE_STRATEGY_FOR_DUPLICATE_ENTRIES');

                $handle_options = array(
                    '0' => 'INTERUPT_MERGE',
                    '1' => 'SUM_DATA',
                    '2' => 'USE_TARGET_DATA',
                    '3' => 'USE_SOURCE_DATA');

                foreach($handle_options as $id => $desc) {
                    $handle_dup->addMultiOption($id,$desc);
                }


                $this->addElements(array($merge_camera_first, $merge_camera_second, $dry_run, $remove_source,$handle_dup));
                $this->addDisplayGroup(array("merge_camera_source_id","merge_camera_target_id","dry_run","remove_source","handle_dup"),'chart-group');
                
                $step = new UniForm_Form_Element_Hidden('step');
                $step->setValue('2');
                $application = new UniForm_Form_Element_Hidden("application_id");
                $application->setValue($this->_application_id);
                $this->addElements(array($step,$application));
        }
        $this->addElement($submit);
    }

    private function _translatableStrings() {
        $this->_translate->_('DRY_RUN');
        $this->_translate->_('MERGE_SOURCE');
        $this->_translate->_('SELECT_SOURCE');
        $this->_translate->_('SELECT_APPLICATION');
        $this->_translate->_('MERGE_TARGET');
        $this->_translate->_('SELECT_TARGET');
        $this->_translate->_('DONT_DO_ANYTHING_CHECK_FOR_DOUBLE_DATA');
        $this->_translate->_('REMOVE_SOURCE_CAMERA_IF_MERGE_WAS_SUCCESS');
        $this->_translate->_('INTERUPT_MERGE');
        $this->_translate->_('SUM_DATA');
        $this->_translate->_('USE_TARGET_DATA');
        $this->_translate->_('USE_SOURCE_DATA');
    }
}
