<?php

class Sources_Model_CameraListGenerator extends Core_Model_Generic_AbstractListView {
    protected $_editableCameras;
    public function __construct()
    {
        parent::__construct();

        $camerasDb = new Core_Model_Cameras();

        $this->_editableCameras = $camerasDb->fetchAllEditableNames($this->_user);

        $features = new Core_Model_Features();

        $this->_dynamic_channels = $features->dynamicChannels();

        $this->_extendedColumns = false;
        $this->_module = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();
        if ($this->_module == "overview") {
            $this->_extendedColumns = true;
        }
    }

    public function getHeaders()
    {
        if ($this->_dynamic_channels) {
            return array(_T("NAME"),_T("IP_NUMBER"),_T("MAC"),_T("FIRST_DATE"),_T("LAST_DATE"),_T("STATUS"),_T("LAST_CONTACT"));
        } elseif ($this->_extendedColumns) {
            $ret = array(_T("NAME"),_T("APPLICATION"),_T("COMPANY"),_T("FOLDER"),
                _T("IP_NUMBER"),_T("MAC"),_T("FIRST_DATE"),_T("LAST_DATE"),_T("LAST_CONTACT"));
            if(!$this->_user->multi_company) {
                unset($ret[2]); //Remove company;
            }
            return $ret;
        } else {
            return array(_T("NAME"),_T("IP_NUMBER"),_T("MAC"),_T("FIRST_DATE"),_T("LAST_DATE"),_T("LAST_CONTACT"));
        }

    }
    /*
    protected function _nbrOfDays($date) {
        return floor(($date - time()) / 60/60/24);
    }
    */

    protected function _dataRowToTableRow($row){

        $mins = $row->minutesSinceContact();
        if($mins > -1) {
            if($mins >= 60*48) {
                $title = floor($mins/60/24).' '.$this->translate('DAYS_AGO');
            } else if( $mins >= 60*2){
                $title = floor($mins/60).' '.$this->translate('HOURS_AGO');
            } else if($mins > 0){
                $title = floor($mins).' '.$this->translate('MINUTES_AGO');
            } else {
                $title = $this->translate('A_MOMENT_AGO');
            }
        } else {
            $title = $this->translate('UNKNOWN');
        }

        if($mins)
            $content = str_pad(floor($mins), 10, $pad_string = "0",  $pad_type = STR_PAD_LEFT);
        else
            $content = 'UNKNOWN';

        $status = NULL;
        if($this->_dynamic_channels) {
            // Channel status.
            $channel = $row->getChannelStatus();

            switch($channel['status']) {
            case Abc::NO_SIGNATURE:
                if($channel['trial']) 
                    $status = sprintf($this->translate("TRIAL_MODE,_EXPIRES_IN_%d_DAYS."),$channel['trial_days']) ;
                else
                    $status = $this->translate("TRIAL_EXPIRED.");
                break;

            case Abc::VALID_SIGNATURE:
                $dys = $row->getDaysUntilExpire();
                if($dys >= 0)
                    $status = sprintf($this->translate('EXPIRES_IN_%1$s_DAYS.'), $dys);
                else
                    $status = sprintf($this->translate('EXPIRED_%1$s_DAYS_AGO.'), -$dys);
                break;
            case Abc::INVALID_SIGNATURE:
                $status = $this->translate("INVALID_SIGNATURE.");
                break;
            case Abc::TAMPERED:
                $status = $this->translate("UNKNOWN_ERROR.");
                break;
            case Abc::WRONG_SERIAL:
                $status = $this->translate("INCORRECT_SERIAL.");
                break;
            case Abc::EXPIRED:
                $status = sprintf($this->translate('EXPIRED_%1$s_DAYS_AGO.'), $row->getDaysUntilExpire());
                break;
            default:
                $status = $this->translate("UNKNOWN_ERROR.");
            }
        }

        if ($this->_extendedColumns) {
            $metaData = $row->getMetaData();
            $applicationsModel = Core_Model_Applications::getInstance();
            $application = $applicationsModel->findWithId($row->application_id);
            if ($application) {
                $application_name=$application->getName();
            }
            $ret = array(
                $row->name,
                $application_name,
                $metaData["company"]["name"],
                $metaData["folder"]["path"],
                $row->ip == "" ? $this->translate("NOT_AVAILABLE") :  $row->ip,
                $row->serial == "" ? $this->translate("NOT_AVAILABLE") : $row->serial, 
                $row->first_date,
                $row->last_date,
                self::CreateCellWithClassAndTitle($title,array("last_contact", $row->getConnectionLevel(), "ui-hover"), $title));
            if(!$this->_user->multi_company) {
                unset($ret[2]); //Remove company;
            }
        } else {
            $ret = array(
                $row->name,
                $row->ip == "" ? $this->translate("NOT_AVAILABLE") :  $row->ip,
                $row->serial == "" ? $this->translate("NOT_AVAILABLE") : $row->serial, 
                $row->first_date,
                $row->last_date,
                //$status,
                self::CreateCellWithClassAndTitle($title,array("last_contact", $row->getConnectionLevel(), "ui-hover"), $title));
            //if (!$this->_dynamic_channels) {
            //    unset($ret[5]);
            //}
        }



        return $ret;
    }

    protected function _dataRowToRowId($row){
        return $row->camera_id;
    }

    public function removeText(){
        return _T('DO_YOU_REALLY_WANT_TO_REMOVE_THE_CAMERA_"%1$s"?');
    }

    public function noDataInformation(){
        return _T("THIS_FOLDER_IS_EMPTY");
    }

    public function getActions($row){
        $actions = array();
        $actions[] = parent::CreateAction(_T('VIEW_CAMERA'),'view-camera', 'view', null, null, null, 'sources','data-storage-management');

        if(array_key_exists($row->camera_id, $this->_editableCameras)) {
            $actions[] = parent::CreateAction(_T('EDIT_CAMERA'), 'edit-camera', 'edit', null, null, null, 'sources', 'data-storage-management');
            $actions[] = parent::CreateAction(_T('DELETE_CAMERA'),'delete-camera', 'delete', null, null, null, 'sources', 'data-storage-management');
        }

        /*if ($row->fetch_me) {
            $actions[] = parent::CreateAction(_T('FETCH_OF_NEW_CAMERA_DATA_TO_TrueView Xense&reg;_ENABLED._CLICK_TO_DISABLE.'), 'fetch-toggle', 'fetch-disable', null, null, null, 'sources', 'data-storage-management');
        } else {
            $actions[] = parent::CreateAction(_T('FETCH_OF_CAMERA_DATA_TO_TrueView Xense&reg;_DISABLED._CLICK_TO_ENABLE.'), 'fetch-toggle', 'fetch-enable', null, null, null, 'sources', 'data-storage-management');
        }*/

        if($this->_user->multi_company && $this->_dynamic_channels) {
            if($row->sync_me) {
                $text = _T('INACTIVATE_CAMERA');
                $status = 'active';

                if(!$row->hasValidCertificate())
                    $status .= ' tentative';
            }
            else {
                $text = _T('ACTIVATE_CAMERA');
                $status = 'inactive';
            }

            $actions[] = parent::CreateAction($text, 'toggle-camera', "toggle-active $status", null, null, null, 'sources', 'data-storage-management');
        }

        return $actions;
    }
}
