<?php

abstract class AmCharts_Data_Abstract {

	protected $_series = array();
	protected $_xLabels;

	protected $_labels;

	protected $_graphAttributes;

	public function __construct()
	{
		$this->_labels = null;
		$this->init();
		$this->_graphAttributes = array();
	}

	public function init()
	{
		
	}

	public function appendLabels(AmCharts_Labels $labels)
	{
		$this->_labels = $labels;
	}


	public function addGraphAttribute($attribute, $value)
	{
		$this->_graphAttributes[$attribute] = $value;
	}

	public function addSerie($id, $param, $description = "serie") {
		if ($param instanceof AmCharts_Data_Serie) {
			$this->_series[$id] = $param;
		} else {
			$this->_series[$id] = new AmCharts_Data_Serie($param, $description);
		}
	}

	public function getSerie($id)
	{
		if (!array_key_exists($id, $this->_series)) {
			throw new AmCharts_Exception("Serie with id $id doesn't exist.");
		} else {
			return $this->_series[$id];
		}
	}

	public function removeSerie($id)
	{
		if (!array_key_exists($id, $this->_series)) {
			throw new AmCharts_Exception("Serie id doesn't exist.");
		} else {
			unset($this->_series[$id]);
		}
	}

	public function setXLabels($labels)
	{
		if (is_array($labels)) {
			$this->_xLabels = new AmCharts_Data_Labels($labels);
		} else if (is_numeric) {
			$this->_xLabels = new AmCharts_Data_Labels($labels);
		} else if ($labels instanceof AmCharts_Data_Labels) {
			$this->_xLabels = $labels;
		} else {
			throw new AmCharts_Exception("Not valid x labels");
		}
	}

	public function getXLabels()
	{
		return $this->_xLabels;
	}


	public function __toString()
	{

		$xml = new SimpleXMLElement("<chart/>");

		if (count($this->_series) > 0) {
			
			$series = $xml->addChild("series");

			foreach ($this->getXLabels()->getTags() as $key => $value) {
				$valueNode = $series->addChild("value", $value);
				$valueNode->addAttribute("xid", $key);
			}


			$graphs = $xml->addChild("graphs");

			foreach ($this->_series as $id => $serie) {
				$graph = $graphs->addChild("graph");
				$graph->addAttribute("title", $serie->getDescription());
				$graph->addAttribute("gid", $id);
				foreach ($this->_graphAttributes as $attribute => $value) {
					$graph->addAttribute($attribute, $value);
				}
				$options = $serie->getOptions();
				if ($options) {
					foreach ($options as $key => $value) {
						$graph->addAttribute($key, $value);
					}
				}
				foreach ($serie->getTags() as $key => $value) {
					if (!is_null($value)) {
						$valueNode = $graph->addChild("value", $value);
						$valueNode->addAttribute("xid", $key);
					}
				}
			}

		}

		if ($this->_labels && count($this->_labels->getLabels()) > 0) {

			$labels = $xml->addChild("labels");

			foreach ($this->_labels as $label) {
				$labelElement = $labels->addChild("label");
				$labelElement->addChild("x", $label->getX());
				$labelElement->addChild("y", $label->getY());
//				$labelElement->addChild("text", "<![CDATA[" . $label->getText() . "]]>");
				$labelElement->addChild("text", $label->getText());
				if ($label->getAlign())
					$labelElement->addChild("align", $label->getAlign());

				if ($label->getRotate())
					$labelElement->addChild("rotate", $label->getRotate());

				if ($label->getTextSize())
					$labelElement->addChild("text_size", $label->getTextSize());

				if ($label->getTextColor())
					$labelElement->addChild("text_color", $label->getTextColor());

				if ($label->getWidth())
					$labelElement->addChild("text_color", $label->getWidth());

			}
		}


		$xml = $xml->asXML();

		$firstNewLine = strpos($xml, "\n");
		if ($firstNewLine !== false) {
			$xml = substr($xml, $firstNewLine);
		}

		$xml = str_replace("\r\n", "", $xml); // for windows
		$xml = str_replace("\n", "", $xml); // for unix

		return $xml;

	}

}