<?php

class AmCharts_Label {

	protected $_text;
	protected $_x;
	protected $_y;
	protected $_rotate = null;
	protected $_width = null;
	protected $_align = null;
	protected $_textColor = null;
	protected $_textSize = null;

	const ALIGN_CENTER = 'center';
	const ALIGN_RIGHT = 'right';
	const ALIGN_LEFT = 'left';

	const ROTATE = 'rotate';
	const WIDTH = 'width';
	const ALIGN = 'align';
	const TEXT_COLOR = 'textColor';
	const TEXT_SIZE = 'textSize';

	public function __construct($text, $x = 0, $y = 0, $params = array())
	{
		$this->setText($text);

		if (!$x) $x = 0;
		$this->setX($x);
		if (!$y) $y = 0;
		$this->setY($y);

		if (count($params) > 0) {
			foreach ($params as $key => $value) {
				switch ($key) {
					case self::ROTATE:
						$this->_rotate = $value;
						break;
					case self::WIDTH:
						$this->_width = $value;
						break;
					case self::ALIGN:
						$this->_align = $value;
						break;
					case self::TEXT_COLOR:
						$this->_textColor = (string)$value;
						break;
					case self::TEXT_SIZE:
						$this->_textSize =  $value;
						break;
					default:
						// ignore unrecognized configuration directive
						break;
				}
			}
		}


	}


	public function __call($method, $parameters)
	{
		//for this to be a setSomething or getSomething, the name has to have
		//at least 4 chars as in, setX or getX
		if(strlen($method) < 4)
			throw new AmCharts_Exception('Method does not exist');

		//take first 3 chars to determine if this is a get or set
		$prefix = substr($method, 0, 3);

		//take last chars and convert first char to lower to get required property
		$suffix = substr($method, 3);
		$suffix[0] = strtolower($suffix[0]);

		$suffix = '_' . $suffix;

		if($prefix == 'get') {
			if ($this->_hasProperty($suffix) && count($parameters) == 0)
				return $this->$suffix;
			else
				throw new Exception('Getter does not exist');
		}

		if ($prefix == 'set') {
			if ($this->_hasProperty($suffix) && count($parameters) == 1)
				$this->$suffix = $parameters[0];
			else
				throw new Exception('Setter does not exist');
		}
	}

	protected function _hasProperty($property)
	{
		$properties = get_class_vars(get_class($this));
		return array_key_exists($property, $properties);
	}

}


