<?php

abstract class AmCharts_Settings_Abstract extends Zend_Config
{

	public function __construct()
	{
		parent::__construct(array(), true);
		$this->init();
	}

	public function init() {

	}



	public function __toString()
	{
		$xml = $this->asXml();

		$firstNewLine = strpos($xml, "\n"); // delete the <?xml... crap
		if ($firstNewLine !== false) {
			$xml = substr($xml, $firstNewLine);
		}

		$xml = str_replace("\r\n", "", $xml); // for windows
		$xml = str_replace("\n", "", $xml); // for unix

		return $xml;
	}

	/**
	 * Set chart colors.
	 *
	 * @param array $colors
	 */
	public function setColors(array $colors)
	{
		$this->colors = implode(", ", $colors);
		return $this;
	}

	public function setTextSize($size) {
		$this->textSize = $size;
		return $this;
	}

	public function setTextColor($color)
	{
		$this->textColor = $color;
		return $this;
	}

	public function setRedraw($redraw)
	{
		if ($redraw)
			$this->redraw = "true";
		else
			$this->redraw = "false";

		return $this;
	}

	protected function _initSection($sections)
	{
		if (!is_array($sections))
			$sections = array($sections);

		$prev = $this;

		foreach ($sections as $section) {
			if (!is_array($prev->$section)) {
				$prev->$section = array();
				$prev = $prev->$section;
			}
		}

	}


	public function asXml($formatWithLineBreaks = false)
	{

        $xml = new SimpleXMLElement('<settings/>');

		foreach ($this as $sectionName => $data) {
			
			if (!($data instanceof Zend_Config)) {
				$xml->addChild(Talking_Toolbox_String::toUnderscores($sectionName), (string) $data);
				continue;
			}

			$child = $xml->addChild(Talking_Toolbox_String::toUnderscores($sectionName));
			$this->_addBranch($data, $child);
		}

		$xml->asXml();
		if ($formatWithLineBreaks) {
			$dom = dom_import_simplexml($xml)->ownerDocument;
			$dom->formatOutput = true;

			$xmlString = $dom->saveXML();
		}
		return $xml->asXML();
	}

    /**
     * Add a branch to an XML object recursively
     *
     * @param  Zend_Config      $config
     * @param  SimpleXMLElement $xml
     * @return void
     */
    protected function _addBranch(Zend_Config $config, SimpleXMLElement $xml)
    {
        foreach ($config as $key => $value) {
            if ($value instanceof Zend_Config) {
                $child = $xml->addChild($key);
                $this->_addBranch($value, $child);
            } else {
                $xml->addChild(Talking_Toolbox_String::toUnderscores($key), (string) $value);
            }
        }
    }


	/* utility functions for fast set of settings */

	/**
	 * Set the plot area margins
	 *
	 * @param array $values Associative array with keys top, right, bottom and values the margins
	 * @return AmCharts_Settings_Abstract
	 */
	public function setPlotAreaMargins($values) {
		$this->_initSection(array('plotArea','margins'));
		foreach ($values as $key => $value) {
			$this->plotArea->margins->$key = $value;
		}
		return $this;
	}

	/**
	 * Sets the legend align
	 *
	 * @param string $align
	 * @return AmCharts_Settings_Abstract
	 */
	public function setLegendAlign($align) {
		$this->_initSection('legend');
		$this->legend->align = $align;
		return $this;
	}

	public function setGraphBullets($type, $size = null)
	{
		$this->_initSection(array('graphs', 'graph'));
		$this->graphs->graph->bullet = $type;
		if ($size) {
			$this->graphs->graph->bulletSize = $size;
		}
		return $this;
	}


	public function setGraphBulletsSize($size)
	{
		$this->_initSection(array('graphs', 'graph'));
		$this->graphs->graph->bulletSize = $size;
		return $this;
	}


}