<?php

include_once("pChart/class/pData.class.php");
include_once("pChart/class/pDraw.class.php");
include_once("pChart/class/pImage.class.php");
include_once("pChart/class/pPie.class.php");

class CM_ChartingPie {

    protected $_width;
    protected $_height;
    protected $_format;
    protected $_title;
    protected $_data; // array of CM_ChartData

    protected $_yaxis_name;
    const _font = "/fonts/Forgotte.ttf";//"/fonts/Lucida_Grande_Bold.ttf";

    protected $_basedir;
    public function __construct() {
        $this->_format = "png";
        $this->_basedir = dirname(__FILE__);
        $this->_basedir .= "/../pChart";
        $this->_yaxis_name = "Persons";
        $this->_data = array();

    }

    public function setYlabel($label) 
    {
        $this->_yaxis_name = $label;
        return $this;
    }
    
    public function getYlabel($label)
    {
        return $this->_yaxis_name;
    }
    
    public function setData($data) {
        $this->_data = $data;
    }

    public function addData($data) {
        $this->_data[] = $data;
    }

    public function setXlabel($labels, $name) {
        $this->_xlabels = $labels;
        $this->_xlabel = $name;
    }

    private function _renderTheme_test()
    {

    }
    const _text_size = 20;
    const _text_space = 10;
    const _x_spacing = 20;
    const _y_spacing = 20;
    
    const _bot_margin = 80;
    const _top_margin = 80;
    const _right_margin = 150;
    const _left_margin = 150;
    
    private function _computeRadius($cols,$rows,$n,$width,$height) {
        if($cols*$rows <$n) return -1; // invalid due to
        
        $width = $width - self::_right_margin - self::_left_margin;
        $height = $height - self::_top_margin - self::_bot_margin;
        
        $d_x = $width/$cols-($cols>1?1:0)*self::_x_spacing;
        $d_y = $height/$rows-($rows>1?1:0)*self::_y_spacing-self::_text_size-self::_text_space;

        return min(array((int) $d_x/2,(int) $d_y/2));
    }
    
    public function prep($width, $height, $title=null)
    {
        //TODO remove log
        //$logger= new Zend_Log();
        //$logger->addWriter(new Zend_Log_Writer_Stream('/tmp/charting_log'));
        
        $graph_data = new pData();
        $pies = array();
        
        $chartingData = new pData();
        $chartingData->loadPalette($this->_basedir."/palettes/summer.color", TRUE);

        $unit = null;
        foreach ($this->_data as $key => $cdata) {
            $legend =  $cdata->getLegend();
            $chartingData->addPoints($cdata->getY(), $legend);
            //$logger->notice(print_r($cdata->getY(),true));
            
            if($cdata->hasUnit())
                    $unit = $cdata->getUnit();
            
            //file_put_contents('/tmp/miniprof',print_r(array('legend',$cdata->GetType()),true),FILE_APPEND);
            switch ($cdata->GetType()) {
                case "line":
                case "column":
                    throw new WebReport_Exception("Not valid chart type for chart: " . $cdata->GetType());
                    break;
                case "pie":
                    $new_pie = array("legend" => $legend, "data" => new pData());
                    $new_pie['data']->loadPalette($this->_basedir."/palettes/summer.color", TRUE);
                    $ydata =$cdata->getY();
                    $xdata = $this->_xlabels;
                    for($i = 0; $i < count($ydata);$i++){
                        if(abs($ydata[$i]) < 0.3){
                            unset($ydata[$i]);
                            unset($xdata[$i]);
                        }
                    }
                    $new_pie['data']->addPoints($ydata,$legend);
                    $new_pie['data']->addPoints($xdata, $this->_xlabel);
                    $new_pie['data']->setAbscissa($this->_xlabel);
                    //throw new Exception(print_r($ydata,true).print_r($this->_xlabels,true));
                    $pies[] = $new_pie;
                    break;
                default:
                    break;
            }
        }

        //set up drawing surface
        $chartingSurface = new pImage($width, $height, $chartingData);

        $chartingSurface->setFontProperties(array("FontName"=>$this->_basedir.self::_font,"FontSize"=>20));
        $TextSettings = array("Align"=>TEXT_ALIGN_MIDDLEMIDDLE
            , "R"=>0, "G"=>0, "B"=>0);
        
        if(!is_null($title))
            $chartingSurface->drawText($width/2, 18, $title, $TextSettings);
        
        $chartingData->addPoints($this->_xlabels, $this->_xlabel);
        $chartingData->setAbscissa($this->_xlabel);

        $i=0;
        $n=count($pies);
        
        $nrows = 1;
        $ncols = 1;
        $best_nrows=0;
        $best_ncols=0;
        $best_r = 0;
        for($j=1;$j<$n+1;$j++) {
            for($i=1;$i<$n+1;$i++) {
                $r_new = $this->_computeRadius($i,$j,$n,$width,$height);
                if($r_new > $best_r) {
                    $best_r = $r_new;
                    $best_nrows=$j;
                    $best_ncols=$i;
                }
            }
        }
        
        if(($best_nrows==0 && $best_ncols == 0) || $best_r==0)
            throw new WebReport_Exception("NO VALID LAYOUT FOUND");
        
        $k=0;
        for($j=0;$j<$best_nrows;$j++) {
            if($k==$n) break;
            //$y_center=($height)/($best_nrows+1)*($j+1); // simple and working
            if($j==0 && $best_nrows>1) {
                $y_center = self::_top_margin + $best_r;
            } elseif($j+1==$best_nrows) {
                $y_center = $height-self::_bot_margin-$best_r;
            } elseif($best_nrows==1) {
                $y_center = $height/2;
            } else {
                $y_center = self::_top_margin + $best_r + ($height-2*$best_r-self::_bot_margin-self::_top_margin)/($best_nrows-1)*$j;
            }
            
                    
            $y_text=$y_center - $best_r - self::_text_space;
            
            for($i=0;$i<$best_ncols;$i++) {
                if($k==$n) break;
                //$x_center=($width)/($best_ncols+1)*($i+1); // simple and working
                if($i==0 && $best_ncols>1) {
                    $x_center = self::_left_margin + $best_r;
                } elseif($i+1==$best_ncols) {
                    $x_center = $width-self::_left_margin-$best_r;
                } if($best_ncols==1) {
                    $x_center = $width/2;
                } else {
                    $x_center = self::_left_margin + $best_r + ($width-2*$best_r-self::_left_margin-self::_right_margin)/($best_ncols-1)*$i;
                }
                

                $y_text=$y_center - $best_r - self::_text_space;
                
                $x_text=$x_center;
                
                $pies[$k]['x']=$x_center;
                $pies[$k]['y']=$y_center;
                $pies[$k]['x_text']=$x_text;
                $pies[$k]['y_text']=$y_text;
                
                $k++;
            }
        }
           
        
        foreach($pies as $pie_data) {

            $pie=$pie_data['legend'];
            $pieChart = new pPie($chartingSurface, $pie_data['data']);
            $pie_data['data']->setSerieDrawable($pie, true);
            $pieSettings = array(
                "DrawLabels"=>True,
                "Radius" => $best_r
            );
            //$logger->notice(print_r(array($pie_data['x'], $pie_data['y'],$pie_data['x_text'], $pie_data['y_text'],$best_r),true));
            
            
            $pieChart->draw3DPie($pie_data['x'], $pie_data['y'], $pieSettings);
            $chartingSurface->drawText($pie_data['x_text'], $pie_data['y_text'],$pie,array("R"=>0,"G"=>0,"B"=>0,"Align"=>TEXT_ALIGN_TOPMIDDLE));
        }
 
        $chartingSurface->setFontProperties(array("R"=>0,"G"=>0,"B"=>0,"FontName"=>$this->_basedir.self::_font,"FontSize"=>16));
        $chartingSurface->setGraphArea(80,40,$width-80,$height-110);
        
        //$pieSurface = new pPie($chartingSurface,$chartingData);
        //$pieSurface->draw2DPie($width/2, $height/2, array("SecondPass"=>False));

        
        //legend is left
        $Config = array("Margin"=>6, "Alpha"=>30, "BoxSize"=>5,
            "Style"=>LEGEND_BOX, "Mode"=>LEGEND_HORIZONTAL
        );
        
        //TODO warn when box grows to large.
        $chartingSurface->setFontProperties(array("FontName"=>$this->_basedir.self::_font, "FontSize"=>16));
        //$logger->notice(print_r($chartingSurface->DataSet->getData(),true));
        
        //$chartingSurface->drawLegend(80, $height-40, $Config);
        
        
        $this->_chartingSurface = $chartingSurface;
    }

    public function toFile($filename, $width, $height, $title="Chart") {
    
        $this->prep($width,$height, $title);

        $this->_chartingSurface->render($filename);

    }

    public function stroke($width, $height, $title="Chart") {

        $this->prep($width,$height, $title);
        
        $this->_chartingSurface->stroke();
        
    }

}
