<?php

class CM_Interval {
    private $_duration_array = null;
    private $_baseDate = null;
    private $_startDate = null;
    private $_stopDate = null;
    private $_nres;

    private static function _parseDuration ($duration) {
        $duration_elements = preg_split("/[^+\-0-9ymwd]/",$duration);
        $nres = preg_match_all('/([+-]?[0-9]+)([ymwd]{1})/', $duration , $res, PREG_SET_ORDER);
	// TODO: remove me

        //file_put_contents('/tmp/cells',print_r(array($duration_elements,$duration,$nres),true),FILE_APPEND);//(array($duration_elements,$duration,$nres));

        if(!$nres || count($duration_elements) != $nres || $nres > 2)
            return null;
        $ret = array();
        if($nres==2) {
            if($res[0][2] != $res[1][2]) // mixing types not allowed
                return null;
            $ret = array(
                "resolution" => $res[0][2],
                "from" => intval($res[0][1]),
                "to" => intval($res[1][1]));
        } else if ($nres==1) {
            $ret = array(
                "resolution" => $res[0][2],
                "from" => intval($res[0][1]),
                "to" => intval($res[0][1])+1
                );
        } else {
            throw new Webreport_Exception("Duration out of bounds : " . $duration);
        }
        //file_put_contents('/tmp/cells',print_r(array("ret",$ret),true),FILE_APPEND);
        return $ret;
    }

    public static function isValidCode($duration) {
        return !is_null(self::_parseDuration($duration));
    }

    public function toReadable() {
        $this->_do_computations();
        $ret ="";
        $longerPeriod = (($this->_duration_array['to'] - $this->_duration_array['from']) != 1);
        switch ($this->_duration_array["resolution"]) {
            case 'y':
                if($longerPeriod){
                    $ret = "[" . $this->_startDate->toString(Zend_Date::YEAR). " - " . $this->_stopDate->toString(Zend_Date::YEAR) . "]";
                } else {
                    $ret = "[" .$this->_startDate->toString(Zend_Date::YEAR) . "]";
                }
                break;
            case 'm':
                if($longerPeriod){
                    $ret = "[" . $this->_startDate->toString(Zend_Date::YEAR . ' ' . Zend_Date::MONTH_NAME);
                    if ($this->_startDate->equals($this->_stopDate, Zend_Date::YEAR)) {
                        $ret = $ret . " - " . $this->_stopDate->toString(Zend_Date::MONTH_NAME) . "]";
                    } else {
                        $ret = $ret . " - " . $this->_stopDate->toString(Zend_Date::YEAR . ' ' . Zend_Date::MONTH_NAME) . "]";
                    }
                } else {
                    $ret = "[" . $this->_startDate->toString(Zend_Date::YEAR . ' ' . Zend_Date::MONTH_NAME) . "]";
                }
                break;
            case 'w':
                $translate = Zend_Registry::get('Zend_Translate');
                $midweek = clone $this->_startDate;
                $midweek = $midweek->add(2, Zend_Date::DAY);
                if ($longerPeriod) {
                    $midweek_end = clone $this->_stopDate;
                    $midweek_end = $midweek_end->add(2, Zend_Date::DAY);
                    $ret = "[" . date("o",$this->_startDate->getTimestamp()). ' ' . $translate->_('WEEK') . ' ' . $midweek->toString(Zend_Date::WEEK);
                    if ($this->_startDate->equals($this->_stopDate, Zend_Date::YEAR)) {
                        $ret = $ret . " - " . $translate->_('WEEK') . ' ' . $midweek_end->toString(Zend_Date::WEEK) . "]";
                    } else {
                        $ret = $ret . " - " . date("o",$this->_stopDate->getTimestamp()). ' ' . $translate->_('WEEK') . ' ' . $midweek_end->toString(Zend_Date::WEEK) . "]";
                    }
                } else {
                    $ret = "[" . date("o",$this->_startDate->getTimestamp()). ' ' . $translate->_('WEEK') . ' ' . $midweek->toString(Zend_Date::WEEK) . "]";

                }
                break;
            case 'd':
                $dateFormat = Core_Model_User::GetCurrentUsersDateFormat();
                if($longerPeriod){
                    $ret = "[" . $this->_startDate->toString($dateFormat). " - " . $this->_stopDate->toString($dateFormat) . "]";
                } else {
                    $ret = "[" .$this->_startDate->toString($dateFormat) . "]";
                }
                break;
            default:
                break;

        }
        return $ret;
    }

    //This function is deprecated, but still used when %a is used in chart names to allow older configurations to still export in the same way.
    public function legacyToReadable() {
        $this->_do_computations();
        $ret ="";
        switch ($this->_duration_array["resolution"]) {
            case 'y':
                $ret = $this->_startDate->toString(Zend_Date::YEAR);
                break;
            case 'm':
                $ret = $this->_startDate->toString(Zend_Date::YEAR . '-' . Zend_Date::MONTH_NAME);
                break;
            case 'w':
                $translate = Zend_Registry::get('Zend_Translate');
                $midweek = clone $this->_startDate;
                $midweek = $midweek->add(2, Zend_Date::DAY);
                $ret = date("o",$this->_startDate->getTimestamp()).
                                ' ' . $translate->_('WEEK') . ' ' . $midweek->toString(Zend_Date::WEEK);
                break;
            case 'd':
                $ret = $this->_startDate->toString(Zend_Date::DATE_SHORT);
                break;
            default:
                break;

        }
        return $ret;
    }

    public function __construct(Zend_Date $baseDate, $duration) {
        $this->setDurationArray($duration);
        $this->_baseDate = $baseDate;
    }

    private function _reset_computations() {
        $this->_startDate = null;
        $this->_stopDate = null;
    }

    public static function firstDayOfWeek(){
        $day = Core_Model_User::GetCurrentUsersStartOfWeek();
        if($day == "mon") return "monday";
        if($day == "sun") return "sunday";
        $locale = Zend_Registry::get('Zend_Locale');
        $weekList = Zend_Locale_Data::getList($locale,'week');

        if($weekList['weekendEnd'] == "sun"){
           return "monday";
       }else{
           return "sunday";
       }
    }
    
    private function _do_computations() {
        if(!is_null($this->_startDate) && !is_null($this->_stopDate)) {
            return false; // no need everything should be up to date
        }
        
        $this->_startDate = null;//new Zend_Date($this->_baseDate);
        $this->_stopDate = null;//new Zend_Date($this->_baseDate);
        
        switch ($this->_duration_array["resolution"]) {
            case 'y':
                $this->_startDate = new Zend_Date($this->_baseDate->get('yyyy'), 'yyyy'); 
                $this->_stopDate = clone $this->_startDate;
                $this->_startDate->addYear($this->_duration_array["from"]);
                $this->_stopDate->addYear($this->_duration_array["to"]);
                break;
            case 'm':
                $this->_startDate = new Zend_Date($this->_baseDate->get('yyyyMM'), 'yyyyMM');
                $this->_stopDate = clone $this->_startDate;
                $this->_startDate->addMonth($this->_duration_array["from"]);
                $this->_stopDate->addMonth($this->_duration_array["to"]);
                break;
            case 'w':
                $weekstart = self::firstDayOfWeek();
                $weekstartNumber = $weekstart == 'monday' ? 1 : 0;
                $startDate = clone $this->_baseDate;
                for($i = 0; $i<8; $i++){
                    if($startDate->get(Zend_Date::WEEKDAY_DIGIT) != $weekstartNumber){
                        $startDate=$startDate->subDay(1);
                    }else{
                        $this->_startDate = $startDate;
                        $this->_stopDate = clone $startDate;
                        $this->_startDate=$this->_startDate->addWeek($this->_duration_array["from"]);
                        $this->_stopDate=$this->_stopDate->addWeek($this->_duration_array["to"]);
                        break;
                    }
                }
                break;
            case 'd':
                $this->_startDate = clone $this->_baseDate;
                $this->_startDate->addDay($this->_duration_array["from"]);
                $this->_stopDate = clone $this->_baseDate;
                $this->_stopDate->addDay($this->_duration_array["to"]);
 
                break;
            default:
                throw new WebReport_Exception('not valid duration');
        }
        
        $this->_startDate->setTime("00:00:00");
        if($this->_startDate->get(Zend_Date::HOUR) != 0){
            error_log("Error: CM_Interval failed to set correct time, will try to fix it");
            r_log("start date before fix: " . $this->_startDate);
            //This is probably because some DST change, Zend then gets an "off-by-1-error"
            if($this->_startDate->get(Zend_Date::HOUR) == 23) $this->_startDate->addHour(1);
            if($this->_startDate->get(Zend_Date::HOUR) == 01) $this->_startDate->subHour(1);
            r_log("start date after fix: " . $this->_startDate);
        }

        $this->_stopDate->setTime("00:00:00");
        if($this->_stopDate->get(Zend_Date::HOUR) != 0){
            error_log("Error: CM_Interval failed to set correct time, will try to fix it");
            r_log("stop date before fix: " . $this->_stopDate);
            //This is probably because some DST change, Zend then gets an "off-by-1-error"
            if($this->_stopDate->get(Zend_Date::HOUR) == 23) $this->_stopDate->addHour(1);
            if($this->_stopDate->get(Zend_Date::HOUR) == 01) $this->_stopDate->subHour(1);
            r_log("stop date after fix: " . $this->_stopDate);
        }
        $this->_stopDate->addSecond(-1);
        return true;
    }

    public function getStartDate() {
        $this->_do_computations();
        return $this->_startDate;
    }

    public function getStopDate() {
        $this->_do_computations();
        return $this->_stopDate;
    }
    
    public function getBaseDate(){
        return $this->_baseDate;
    }

    public function setDurationArray($duration) {
        $parsed_dur = $this->_parseDuration($duration); 
        
        if(is_null($parsed_dur))
             throw new WebReport_Exception("not a valid duration code: ");
        
        $this->_reset_computations();
        $this->_duration_array = $parsed_dur;
        
        return $this;
    }

}
