<?php

require_once 'PHPUnit/Autoload.php';
require_once 'RsaSign.php';

class RsaSignTest extends PHPUnit_Framework_TestCase
{
    private $msg, $keys;

    function setUp() {
        $this->keys = gen_keys();
        $this->msg = new Message('a simple message');
    }

    function tearDown() {
        unset($this->keys);
        unset($this->msg); 
    }

    function testCanonical() {
        $d = array(
            "one" => array(1, 2, 3, 4, 5),
            "two" => 123,
            "three" => "string",
            "four" => array("a" => "a", "b" => array(1, 2, 3, 4)),
            "five" => 6
        );

        $this->assertEquals($d, json_decode(canonical($d), TRUE));
    }

    function testCanonicalOrder() {
        $d1 = array("one" => 1, "two" => 2);
        $d2 = array("two" => 2, "one" => 1);

        $this->assertEquals(canonical($d1), canonical($d2));
    }

    function testCanonicalSmall() {
        $this->assertEquals('', json_decode(canonical(''), TRUE));
    }

    function testCanonicalFloat() {
        $this->setExpectedException('InvalidArgumentException');

        canonical(array(0 => 0.1));
    }

    function testCanonicalIntKey() {
        $this->setExpectedException('InvalidArgumentException');

        canonical(array(0 => 0, "test" => "test"));
    }

    function testCanonicalWrongType() {
        $this->setExpectedException('InvalidArgumentException');

        canonical(array("a" => json_decode('{"a":"b"}')));
    }

    function testSignMessage() {
        $this->msg->sign($this->keys['private']);

        $this->assertTrue($this->msg->verify($this->keys['public']));
    }

    function testTamperedMessage() {
        $this->msg->sign($this->keys['private']);
        $this->msg->data = 'another message';

        $this->assertFalse($this->msg->verify($this->keys['public']));
    }

    function testSendMessage() {
        $this->msg->sign($this->keys['private']);
        
        $rec_msg = Message::loadJson($this->msg->saveJson());
        
        $this->assertTrue($rec_msg->verify($this->keys['public']));
    }

    function testVerifyUnsigned() {
        $this->setExpectedException('BadMethodCallException');
        $this->msg->verify($this->keys['public']);
    }

    function testStrangeMessage() {
        $this->setExpectedException('InvalidArgumentException');
        $msg = new Message(0.1);
    }

    function testInvalidKeys() {
        $this->setExpectedException('InvalidArgumentException');

        $this->msg->sign(123);
        $this->msg->verify(123);
    }

    function testLoadSettings() {
        Settings::load();
    }
}

?>
