<?php

/* Modified version of http://www.rmauger.co.uk/2010/01/keeping-your-html-valid-with-zend-framework-tidy-and-firebug/ */

class Talking_Controller_Plugin_TidyOutput extends Zend_Controller_Plugin_Abstract
{
    /**
     * @var tidy|null
     */
    protected $_tidy;

    /**
     * @var array
     */
    protected $_tidyConfig = array('indent'            =>true,
                                   'indent-attributes' => false,
                                   'output-xhtml'      => true,
                                   'drop-proprietary-attributes' => true,
                                   'wrap'              => 0,

    );

    /**
     * @var string
     */
    protected $_tidyEncoding = 'UTF8';

	/**
	 * @var Zend_Log
	 */
	protected $_logger = null;


    public function setConfig(array $config)
    {
        $this->_tidyConfig = $config;
    }

    public function setEncoding($encoding)
    {
        if (!is_string($encoding)) {
            throw new InvalidArgumentException('Encoding must be a string');
        }
        $this->_tidyEncoding = $encoding;
    }

	public function setLogger($logger)
	{
		$this->_logger = $logger;
	}

    protected function getTidy($string = null)
    {
        if (null === $this->_tidy) {
            if (null === $string) {
                $this->_tidy = new tidy();
            } else {
                $this->_tidy = tidy_parse_string($string,
                                                 $this->_tidyConfig,
                                                 $this->_tidyEncoding);
            }
        }
        return $this->_tidy;
    }


    public function dispatchLoopShutdown()
    {
        $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
        if ($viewRenderer->getNoRender()) {
            return;    
        };

		$request = $this->getRequest() ;
		if (!$request->isXmlHttpRequest()) {
			$response = $this->getResponse();

			$headers = $response->getHeaders();
			foreach ($headers as $h) {
				if (strtolower($h['name']) == "content-type" && $h["value"] != "text/html") {
					return;
				}
			}
			$tidy     = $this->getTidy($response->getBody());
			if ($this->_logger) {
				$tidy->diagnose();
				$lines = (explode("\n", $tidy->errorBuffer));
				array_pop($lines);
				foreach ($lines as $line) {
					$this->_logger->log($line, Zend_Log::INFO);
				}

			}

			$tidy->cleanRepair();
			$response->setBody((string) $tidy);
		}




	}
}