<?php

// TODO clean up, throw exceptions instead of echoing strings

class Talking_Filesystem_File
{
	/**
	 * this returns an array of all of the files of a set type in a directory path
	 * if type is an array then it will return all files of the types in the array ( $types = array('png', 'jpg', 'gif'); )
	 *
	 * @param string $path, the filepath to search
	 * @param mixed $type, the file extension to return
	 * @param string $appendPath, the path to append to the returned files
	 */
	static function getFilesByType($path, $type = false, $appendPath = false)
	{
		if(is_dir($path)){
			$dir = scandir($path); //open directory and get contents
			if(is_array($dir)){ //it found files
				$returnFiles = false;
				foreach ($dir as $file){
					if($type){ //validate the type
						$fileParts = explode('.', $file);
						if(is_array($fileParts)){
							$c = count($fileParts) - 1; //arrays always start with 0
							$fileType = $fileParts[$c];

							//check whether the filetypes were passed as an array or string
							if(is_array($type)){
								if(in_array($fileType, $type)){
									$returnFiles[] = $appendPath . $file;
								}
							}else{
								if($fileType == $type){
									$returnFiles[] = $appendPath . $file;
								}
							}
						}
					}else{ //the type was not set.  return all files and directories
						$returnFiles[] = $file;
					}
				}

				if ($returnFiles){
					return $returnFiles;
				}
			}
		}
	}

	/**
	 * creates a new file from a string
	 *
	 * @param string $path
	 * @param string $content
	 * @return string
	 */
	static function saveFile($path, $content)
	{
		$content = stripslashes($content);
		try {
			file_put_contents($path, $content);
			return 'Your page was saved successfully';
		} catch(Talking_Exception $e) {
			return 'Sorry, there was an error saving your page';
		}
	}

	/**
	 * rename the selected file
	 *
	 * @param string $source
	 * @param string $newName
	 */
	static function rename($source, $newName)
	{
    	if(file_exists($source)){
    		rename($source, $newName);
    	}
	}

	/**
	 * copy a file
	 *
	 * @param string $source
	 * @param string $target
	 * @return bool
	 */
	static function copy( $source, $target )
    {
        if (file_exists( $source )){
			return copy($source, $target);
        }
    }

	/**
	 * check if path exists
	 *
	 * @param string $path
	 */
	static function exists($path)
	{
		return file_exists($path);
	}

    /**
     * move a file
     *
     * @param string $source
     * @param string $target
     */
    static function move($source, $target)
    {
    	if(file_exists($source)){
    		rename($source, $target);
    	}
    }

    /**
     * delete a file
     *
     * @param string $path
     */
    static function delete($path)
    {
    	@unlink($path);
    }

    /**
     * this function cleans up the filename
     * it strips ../ and ./
     * it spaces with underscores
     *
     * @param string $fileName
     */
    static function cleanFilename($fileName)
    {
        $fileName = str_replace('../', '', $fileName);
        $fileName = str_replace('./', '', $fileName);
        $fileName = str_replace(' ', '_', $fileName);
        return $fileName;
    }

	/**
	 * From http://se2.php.net/realpath
	 * Because realpath() does not work on files that do not
	 * exist, I wrote a function that does.
	 * It replaces (consecutive) occurences of / and \\ with
	 * whatever is in DIRECTORY_SEPARATOR, and processes /. and /.. fine.
	 * Paths returned by get_absolute_path() contain no
	 * (back)slash at position 0 (beginning of the string) or
	 * position -1 (ending)
	 *
	 * If $forWeb is true DIRECTORY_SEPARATOR will be changed to /
	 *
	 * @param string $path
	 * @param bool $forWeb
	 * @return string
	 */

	static function getAbsolutePath($path, $forWeb = false)
	{
		$dirSep = DIRECTORY_SEPARATOR;
		if ($forWeb) {
			$dirSep = "/";
		}

		$addBeginningSeparator = false;
		if (substr($path, 0, 1) == '/' || substr($path, 0, 2) == '\\')
			$addBeginningSeparator = true;

		$path = str_replace(array('/', '\\'), $dirSep, $path);
		$parts = array_filter(explode($dirSep, $path), 'strlen');
		$absolutes = array();
		foreach ($parts as $part) {
			if ('.' == $part) continue;
			if ('..' == $part) {
				array_pop($absolutes);
			} else {
				$absolutes[] = $part;
			}
		}

		$path = implode($dirSep, $absolutes);

		if ($addBeginningSeparator)
			$path = $dirSep . $path;

		return $path;
	}

}