<?php

class Talking_Image
{
	const BORDER_PLACEMENT_OUTSIDE = 'outside';
	const BORDER_PLACEMENT_INSIDE = 'inside';

	const ARROW_SIDES_END = 'end';
	const ARROW_SIDES_BEGINNING = 'beginning';
	const ARROW_SIDES_BOTH = 'both';

	protected $_width;
	protected $_height;
	protected $_image;

	protected $_font = "arial.ttf";

	/* between 0 and 100, 100 is the best */
	protected $_antialiasQuality = 100;
	
	/**
	 * Initializes image with white background
	 *
	 * @param int $width
	 * @param int $height
	 */
	public function __construct($width, $height, $r = 255, $g = 255, $b = 255)
	{
		$this->_width = $width;
		$this->_height = $height;

		$image = imagecreatetruecolor($width,$height);
		if (!$image) {
			throw new Exception("Couldn't create image.");
		}

		$this->_image = $image;

		$color = $this->_allocateColor($r, $g, $b);
		imagefilledrectangle($this->_image, 0, 0, $width, $height, $color);

		$this->_image = $image;
	}

	public function setFont($font) {
		$this->_font = $font;
	}

	public function drawText($x, $y, $text, $size, $r = 0, $g = 0, $b = 0, $angle = 0)
	{
        $color = $this->_allocateColor($r, $g, $b);
		imagettftext($this->_image, $size, $angle, $x, $y, $color, $this->_font, $text);
	}


	function drawRectangle($x1, $y1, $x2, $y2, $r, $g, $b, $filled = false, $drawBorder=true)
	{
		if ( $x2 < $x1 ) { list($x1, $x2) = array($x2, $x1); }
		if ( $y2 < $y1 ) { list($y1, $y2) = array($y2, $y1); }

		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		$rectangleColor = $this->_allocateColor($this->_image, $r, $g, $b);
		if ($filled) {
			imagefilledrectangle($this->_image, round($x1), round($y1), round($x2), round($y2), $rectangleColor);
		}

		if ( $drawBorder ) {
			$x1=$x1-.2;$y1=$y1-.2;
			$x2=$x2+.2;$y2=$y2+.2;
			$this->drawLine($x1,$y1,$x2,$y1,$r,$g,$b);
			$this->drawLine($x2,$y1,$x2,$y2,$r,$g,$b);
			$this->drawLine($x2,$y2,$x1,$y2,$r,$g,$b);
			$this->drawLine($x1,$y2,$x1,$y1,$r,$g,$b);
		}
	}

	function drawTriangle($x1, $y1, $x2, $y2, $x3, $y3, $r, $g, $b, $filled = false, $drawBorder=true)
	{
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		if ($filled) {
			throw new Exception("Not implemented.");
		}

		$trangleColor = $this->_allocateColor($this->_image, $r, $g, $b);

		if ( $drawBorder ) {
			$x1=$x1-.2;$y1=$y1-.2;
			$x2=$x2+.2;$y2=$y2+.2;
			$x3=$x3-.2;$y3=$y3-.2;
			$this->drawLine($x1,$y1,$x2,$y2,$r,$g,$b);
			$this->drawLine($x2,$y2,$x3,$y3,$r,$g,$b);
			$this->drawLine($x3,$y3,$x1,$y1,$r,$g,$b);
		}
	}


	public function addBorder($size = 3, $r = 0, $g = 0, $b = 0, $placement = self::BORDER_PLACEMENT_OUTSIDE)
    {
		if ($placement == self::BORDER_PLACEMENT_INSIDE) {
			$newHeight = $this->_height;
			$newWidth = $this->_width;
		} elseif ($placement == self::BORDER_PLACEMENT_OUTSIDE) {
			$newHeight = $this->_height+2*$size;
			$newWidth = $this->_width+2*$size;
		} else {
			throw Exception("Not valid border placement.");
		}

		$withBorder = imagecreatetruecolor($newWidth,$newHeight);
		$bgColor = $this->_allocateColor($withBorder, $r, $g, $b);
		imagefilledrectangle($withBorder, 0, 0, $newWidth, $newHeight, $bgColor);

		if ($placement == self::BORDER_PLACEMENT_INSIDE) {
			imagecopy($withBorder, $this->_image, 0, 0, 0, 0, $this->_width, $this->_height);
		} elseif ($placement == self::BORDER_PLACEMENT_OUTSIDE) {
			imagecopy($withBorder, $this->_image, $size, $size, 0, 0, $this->_width, $this->_height);
		}
		
		imagedestroy($this->_image);

		$this->_width = $newWidth;
		$this->_height = $newHeight;

		$this->_image = $withBorder;
	}

	function drawEllipse($xc, $yc, $height, $width, $r, $g, $b)
	{
		/*
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		$colorCircle = $this->_allocateColor($this->_image, $r, $g, $b);
		$step = 360 / (2 * 3.1418 * max($width,$height));

		for($i = 0; $i <= 360; $i=$i+$step) {
			$x = cos($i*M_PI/180) * $height + $xc;
			$y = sin($i*M_PI/180) * $width + $yc;
			$this->_drawAntialiasPixel($x, $y, $r, $g, $b);
		}*/
		$this->drawArc($xc, $yc, $height, $width, 0, 360, $r, $g, $b);
	}

	function drawArc($xc, $yc, $height, $width, $startAngle, $endAngle, $r, $g, $b)
	{
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		if ( $endAngle < $startAngle ) { list($startAngle, $endAngle) = array($endAngle, $startAngle); }

		$colorCircle = $this->_allocateColor($this->_image, $r, $g, $b);
		$step = 360 / (2 * 3.1418 * max($width,$height));

		for($i = $startAngle; $i <= $endAngle; $i=$i+$step) {
			$x = cos($i*M_PI/180) * $height + $xc;
			$y = sin($i*M_PI/180) * $width + $yc;
			$this->_drawAntialiasPixel($x, $y, $r, $g, $b);
		}
	}


	function drawCircle($xc, $yc, $radius, $r = 0, $g = 0, $b = 0)
	{
		$this->drawEllipse($xc, $yc, $radius*2, $radius*2, $r, $g, $b);
	}


	function drawArrow($x1, $y1, $x2, $y2, $r, $g, $b, $sides = self::ARROW_SIDES_END, $lineWidth = 1)
	{
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		if (round($x1) == round($x2)) {
			if ($sides == self::ARROW_SIDES_END || $sides == self::ARROW_SIDES_BOTH) {
				$this->drawLine($x2, $y2, $x2-4, $y2-4, $r, $g, $b, $lineWidth);
				$this->drawLine($x2, $y2, $x2+4, $y2-4, $r, $g, $b, $lineWidth);
			}
			if ($sides == self::ARROW_SIDES_BEGINNING || $sides == self::ARROW_SIDES_BOTH) {
				$this->drawLine($x1, $y1, $x1-4, $y1+4, $r, $g, $b, $lineWidth);
				$this->drawLine($x1, $y1, $x1+4, $y1+4, $r, $g, $b, $lineWidth);
			}
		} else if (round($y1) == round($y2)) {
			if ($sides == self::ARROW_SIDES_END || $sides == self::ARROW_SIDES_BOTH) {
				$this->drawLine($x2, $y2, $x2-4, $y2-4, $r, $g, $b, $lineWidth);
				$this->drawLine($x2, $y2, $x2-4, $y2+4, $r, $g, $b, $lineWidth);
			}
			if ($sides == self::ARROW_SIDES_BEGINNING || $sides == self::ARROW_SIDES_BOTH) {
				$this->drawLine($x1, $y1, $x1+4, $y1+4, $r, $g, $b, $lineWidth);
				$this->drawLine($x1, $y1, $x1+4, $y1-4, $r, $g, $b, $lineWidth);
			}
		} else {
			return;
		}





		$this->drawLine($x1, $y1, $x2, $y2, $r, $g, $b, $lineWidth);
		
		
	}


	function drawLine($x1, $y1, $x2, $y2, $r, $g, $b, $lineWidth = 1) {
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		$distance = sqrt(($x2-$x1)*($x2-$x1)+($y2-$y1)*($y2-$y1));
		if ( $distance == 0 )
			return;

		$xStep = ($x2-$x1) / $distance;
		$yStep = ($y2-$y1) / $distance;

		for($i=0; $i <= $distance; $i++) {
			$x = $i * $xStep + $x1;
			$y = $i * $yStep + $y1;

			if ($x >= 0 && $x < $this->_width && $y >= 0 && $y <= $this->_height) {
				if ( $lineWidth == 1 ) {
					$this->_drawAntialiasPixel($x,$y,$r,$g,$b);
				} else {
					$startOffset = -$lineWidth/2;
					$endOffset = $lineWidth/2;
					for($j = $startOffset; $j <= $endOffset; $j++) {
						$this->_drawAntialiasPixel($x+$j, $y+$j, $r, $g, $b);
					}
				}
			}
		}
	}

	public function __destruct()
	{
		imagedestroy($this->_image);
	}

	public function render($path = null)
	{
		if ($path) {
			imagepng($this->_image, $path);
		} else {
			header('Content-type: image/png');
			imagepng($this->_image);
		}
	}


	/* Private functions for internal processing */
	private function _drawAntialiasPixel($x, $y, $r, $g, $b, $alpha = 100)
	{
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);
		
		$xi   = floor($x);
		$yi   = floor($y);

		if ( $xi == $x && $yi == $y) {
			if ( $alpha == 100 ) {
				$colorAliased = $this->_allocateColor($this->_image, $r, $g, $b);
				imagesetpixel($this->_image, $x, $y, $colorAliased);
			} else
				$this->drawAlphaPixel($x, $y, $alpha, $r, $g, $b);
		} else {
			$inverseAntialiasQuality = 100 - $this->_antialiasQuality;
			$alpha1 = (((1 - ($x - floor($x))) * (1 - ($y - floor($y))) * 100) / 100) * $alpha;
			if ( $alpha1 > $inverseAntialiasQuality ) { $this->_drawAlphaPixel($xi,$yi,$alpha1,$r,$g,$b); }
			$alpha2 = ((($x - floor($x)) * (1 - ($y - floor($y))) * 100) / 100) * $alpha;
			if ( $alpha2 > $inverseAntialiasQuality ) { $this->_drawAlphaPixel($xi+1,$yi,$alpha2,$r,$g,$b); }
			$alpha3 = (((1 - ($x - floor($x))) * ($y - floor($y)) * 100) / 100) * $alpha;
			if ( $alpha3 > $inverseAntialiasQuality ) { $this->_drawAlphaPixel($xi,$yi+1,$alpha3,$r,$g,$b); }
			$alpha4 = ((($x - floor($x)) * ($y - floor($y)) * 100) / 100) * $alpha;
			if ( $alpha4 > $inverseAntialiasQuality ) { $this->_drawAlphaPixel($xi+1,$yi+1,$alpha4,$r,$g,$b); }
		}
	}

	private function _drawAlphaPixel($x, $y, $alpha, $r, $g, $b)
	{
		
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		if ($x < 0 || $y < 0 || $x >= $this->_width || $y >= $this->_height)
			return;

		$rgb2 = imagecolorat($this->_image, $x, $y);
		$r2   = ($rgb2 >> 16) & 0xFF;
		$g2   = ($rgb2 >> 8) & 0xFF;
		$b2   = $rgb2 & 0xFF;

		$iAlpha = (100 - $alpha)/100;
		$alpha  = $alpha / 100;

		$ra   = floor($r*$alpha + $r2*$iAlpha);
		$ga   = floor($g*$alpha + $g2*$iAlpha);
		$ba   = floor($b*$alpha + $b2*$iAlpha);

		$aliasedColor = $this->_allocateColor($ra, $ga, $ba);
		imagesetpixel($this->_image, $x, $y, $aliasedColor);
    }

	private function _allocateColor($r, $g, $b, $factor = 0)
	{
		$r = $r + $factor;
		$g = $g + $factor;
		$b = $b + $factor;
		$r = min(max($r, 0), 255);
		$g = min(max($g, 0), 255);
		$b = min(max($b, 0), 255);

		return imagecolorallocate($this->_image, $r, $g, $b);
	}

}
