<?php

abstract class Talking_Pdf_Chart_Data_Abstract {

	protected $_series = array();
	protected $_xLabels;

	protected $_min = null;
	protected $_max = null;

	public function __construct()
	{
		$this->_labels = null;
		$this->init();
	}

	public function init()
	{

	}

	public function addSerie($id, $param, $description = "serie") {
		if ($param instanceof Talking_Pdf_Chart_Data_Serie) {
			$this->_series[$id] = $param;
		} else {
			$this->_series[$id] = new Talking_Pdf_Chart_Data_Serie($param, $description);
		}
	}

	public function getSerie($id)
	{
		if (!array_key_exists($id, $this->_series)) {
			throw new Talking_Exception("Serie with id $id doesn't exist.");
		} else {
			return $this->_series[$id];
		}
	}

	public function getSeries()
	{
		return $this->_series;
	}

	public function removeSerie($id)
	{
		if (!array_key_exists($id, $this->_series)) {
			throw new Talking_Exception("Serie id doesn't exist.");
		} else {
			unset($this->_series[$id]);
		}
	}

	public function setXLabels($labels)
	{
		if (is_array($labels)) {
			$this->_xLabels = new Talking_Pdf_Chart_Data_Labels($labels);
		} else if (is_numeric) {
			$this->_xLabels = new Talking_Pdf_Chart_Data_Labels($labels);
		} else if ($labels instanceof Talking_Pdf_Chart_Data_Labels) {
			$this->_xLabels = $labels;
		} else {
			throw new Talking_Exception("Not valid x labels");
		}
	}

	public function getXLabels()
	{
		return $this->_xLabels;
	}

	private function _initMinAndMax()
	{
		$max = (int)(PHP_INT_MAX+1);
		$min = PHP_INT_MAX;
		$series = $this->getSeries();
		foreach ($series as $serie) {
			$values = $serie->getTags();
			foreach ($values as $v) {
				if ($v > $max) $max = $v;
				if ($v < $min) $min = $v;
			}

		}
		$this->_min = $min;
		$this->_max = $max;
	}


	public function getMin()
	{
		if (is_null($this->_min)) {
			$this->_initMinAndMax();
		}
		return $this->_min;
	}

	public function getMax()
	{
		if (is_null($this->_max)) {
			$this->_initMinAndMax();
		}
		return $this->_max;
	}

	public function getNumberOfValues()
	{
		$serie = current($this->_series);
		return count($serie->getTags());
		
	}

}