<?php

class Talking_Pdf_Chart_Line extends Talking_Pdf_Chart_Abstract
{

	protected $_chartType = "line";

	public function render(Talking_Pdf_Page $page, Talking_Pdf_Frame $frame = null)
	{
		if (!$frame) {
			$frame = $this->_frame;
		}

		if (!$frame) {
			throw new Talking_Exception("Frame must be set");
		}

		$settings = $this->getSettings();

		$orgStyle = $page->getStyle();

		$data = $this->getData();
		/* @var $data Talking_Pdf_Chart_Data_Line */

		$min = $data->getMin();
		$max = $data->getMax();
		if ($min > 0) {
			$min = 0;
		}

		$nbrOfVals = $data->getNumberOfValues();

		$series = $data->getSeries();

		$heightUnit = $frame->getHeight()/($max-$min);
		$zeroY = $frame->getY() + $frame->getHeight() + $min*$heightUnit;

		$fontSize = 10;
		$page->setFont(Zend_Pdf_Font::fontWithName(Zend_Pdf_Font::FONT_HELVETICA), $fontSize);
		/* draw Y markers */
		$valueTextWidths = array();
		for ($i = $min; $i <= $max; $i++) {
			$textProps = $page->getTextProperties((string)$i);
			$valueTextWidths[$i] = $textProps['text_width'];
		}

		$style = new Zend_Pdf_Style();
		$style->setLineColor(new Zend_Pdf_Color_GrayScale(0.5));
		$style->setLineWidth(1);
		$page->setStyle($style);


		$maxTextWidth = max($valueTextWidths);
		$fontHeight = $page->getFontHeight();

		/*HEREIWAS: $nNbrsToWrite = ($max-$min)*$fontHeight;*/

		for ($i = $min; $i <= $max; $i++) {

			$page->drawLine(
				$frame->getX() + $maxTextWidth + 4,
				$frame->getY() + $frame->getHeight() - ($i-$min)*$heightUnit,
				$frame->getX() + $maxTextWidth + 11,
				$frame->getY() + $frame->getHeight() - ($i-$min)*$heightUnit
			);
			$textWidth = $valueTextWidths[$i];

			$page->drawText($i,
							$frame->getX() + $maxTextWidth - $textWidth,
							$frame->getY() + $frame->getHeight() - ($i-$min)*$heightUnit - $fontHeight/2 - 1);
		}

		$frame->setX($frame->getX() + $maxTextWidth + 11);
		$frame->setWidth($frame->getWidth() - $maxTextWidth - 11);
		$widthUnit = $frame->getWidth()/($nbrOfVals);


		$page->drawLine($frame->getX(), $frame->getY(), $frame->getX(), $frame->getY() + $frame->getHeight());
		$page->drawLine($frame->getX(), $zeroY, $frame->getX()+$frame->getWidth(), $zeroY);

		$style->setLineColor(new Zend_Pdf_Color_GrayScale(0.5));
		$style->setLineWidth(1);
		$page->setStyle($style);


		$j = 0;
		$colors = $settings->getColors();
		$colorsCount = count($colors);

		$x = 0;
		/* draw X markers */
		for ($i = 0; $i <= $nbrOfVals; $i++) {
			$page->drawLine(
				$frame->getX() + $x*$widthUnit,
				$frame->getY() + $frame->getHeight() - (0-$min)*$heightUnit,
				$frame->getX() + $x*$widthUnit,
				$frame->getY() + $frame->getHeight() - (0-$min)*$heightUnit + 7
			);
			$x++;
		}

		foreach ($series as $serie) {
			/* @var $serie Talking_Pdf_Chart_Data_Serie */
			$serieData = $serie->getValues();

			$style->setLineColor(new Zend_Pdf_Color_Html($colors[$j%$colorsCount]));
			
			$page->setStyle($style);

			$x = 0;
			$keys = array_keys($serieData);
			$vals = array_values($serieData);


			for ($i = 0; $i < $nbrOfVals-1; $i++) {
				$val = $vals[$i];
				$nVal = $vals[$i+1];
				if ($val === null || $nVal === null) {
					$x++;
					continue;
				}

				$page->drawLine(
					$frame->getX() + $x*$widthUnit + $widthUnit/2,
					$frame->getY() + $frame->getHeight() - ($val-$min)*$heightUnit,
					$frame->getX() + ($x+1)*$widthUnit + $widthUnit/2,
					$frame->getY() + $frame->getHeight() - ($nVal-$min)*$heightUnit
				);

				$x++;
			}
			$j++;
		}

		$page->setStyle($orgStyle);


	}

}