<?php

class Talking_Pdf_Table_Cell {

	protected $_width;
	protected $_height;
	protected $_recommendedWidth;
	protected $_recommendedHeight;
	protected $_text;
	protected $_align;
	protected $_vAlign;
	protected $_lineHeight = 1.1;

	protected $_textProperties;
	
	/**
	 * Cell padding
	 *
	 * @var array (padding-top,padding-right,padding-bottom,padding-left)
	 */
	protected $_padding;

	/**
	 * Cell borders
	 *
	 * @var array (Talking_Pdf position=>array('color','width','dashing_pattern'))
	 */
	protected $_border;


	/**
	 * Checks if Cell contains a Text Element
	 *
	 * @return bool
	 */
	public function hasText(){
		return !is_null($this->_text);
	}

	public function getRecommendedWidth()
	{
		return $this->_recommendedWidth;
	}

	public function getRecommendedHeight()
	{
		return $this->_recommendedHeight;
	}

	/**
	 * Set Cell Background Color
	 *
	 * @param Zend_Pdf_Color $color
	 */
	public function setBackgroundColor(Zend_Pdf_Color $color)
	{
		$this->_bgColor = $color;
	}

	/**
	 * Set Cell Text Color
	 *
	 * @param Zend_Pdf_Color $color
	 */
	public function setColor(Zend_Pdf_Color $color){
		$this->_color = $color;
	}

	/**
	 * Set Cell Padding
	 *
	 * @param Talking_Pdf $position
	 * @param int $value padding
	 */
	public function setPadding($position, $value){
		$this->_padding[$position] = $value;
	}

	/**
	 * Get Cell Padding
	 *
	 * @param Talking_Pdf $position
	 * @return int
	 */
	public function getPadding($position){
		if(isset($this->_padding[$position])){
			return $this->_padding[$position];
		} else {
			return false;
		}
	}

	/**
	 * Set Horizontal Alignment
	 *
	 * @param Talking_Pdf $align
	 */
	public function setAlignment($align){
		$this->_align = $align;
	}

	/**
	 * Get Alienment
	 *
	 * @return Talking_Pdf
	 */
	public function getAlignment(){
		return $this->_align;
	}

	/**
	 * Set Vertical Alignment
	 *
	 * @param Talking_Pdf $align
	 */
	public function setVAlignment($align){
		$this->_vAlign=$align;
	}

	public function getVAlignment(){
		return $this->_vAlign;
	}

	/**
	 * Get Cell Border
	 *
	 * @param Talking_Pdf $position
	 * @return Zend_Pdf_Style $style
	 */
	public function getBorder($position){
		if(isset($this->_border[$position])){
			return $this->_border[$position];
		} else{
			return false;
		}
	}

	/**
	 * Set cell border properties
	 *
	 * @param Talking_Pdf $position
	 * @param Zend_Pdf_Style $style
	 */
	public function setBorder($position, Zend_Pdf_Style $style){
		$this->_border[$position]=$style;
	}

	/**
	 * Set Cell Borders
	 *
	 * @param array(array(Talking_Pdf position, Zend_Pdf_Styles style))
	 */
	public function setBorders($borders){
		$this->_border=$borders;
	}

	/**
	 * Remove Cell Border
	 *
	 * @param Talking_Pdf $position
	 */
	public function removeBorder($position){
		unset($this->_border[$position]);
	}

	/**
	 * Set Cell Width
	 *
	 * @param int $value
	 */
	public function setWidth($value){
		$this->_width=$value;
	}

	/**
	 * Get Cell Width
	 *
	 * @return int
	 */
	public function getWidth(){
		return $this->_width;
	}

	/**
	 * Set Cell Height
	 *
	 * @param int $value
	 */
	public function setHeight($value){
		$this->_height=$value;
	}

	/**
	 * Get Cell Height
	 *
	 * @return int
	 */
	public function getHeight(){
		return $this->_height;
	}

	/**
	 * Add text to cell
	 *
	 * @param string $text
	 * @param Talking_Pdf $align Horizontal Alignment
	 * @param Talking_Pdf $valign Vertical Alignment
	 */
	public function setText($text, $align=null, $valign=null)
	{
		$this->_text = $text;
		if ($align) {
			$this->_align = $align;
		}
		
		if ($valign) {
			$this->_vAlign = $valign;
		}
	}

	/**
	 * Get Cell Text Element
	 *
	 * @return array(text,width,lines)
	 */
	public function getText()
	{
		return $this->_text;
	}

	/**
	 * Pre-render cell to get recommended width and height
	 *
	 * @param Talking_Pdf_Page $page
	 * @param int $posX
	 * @param int $posY
	 * @param bool $inContentArea
	 */
	public function preRender(Talking_Pdf_Page $page, $posX, $posY) 
	{
		if (!$this->_width) {
			$width = $page->getWidth()-$posX;
		} else{
			$width = $this->_width;
		}

		$maxWidth = $width;// - ($this->_padding[Talking_Pdf::LEFT] + $this->_padding[Talking_Pdf::RIGHT])-(+$this->_getBorderLineWidth(Talking_Pdf::LEFT)+$this->_getBorderLineWidth(Talking_Pdf::RIGHT));

		if ($this->_text) {

			$props = Talking_Pdf_Page_TextProperties($page->getFont(), $page->getFontSize(), $maxWidth);

			if (!$this->_width) {
				$this->_recommendedWidth = $props->getWidth() + ($this->_padding[Talking_Pdf::LEFT] + $this->_padding[Talking_Pdf::RIGHT]);//+$this->_getBorderLineWidth(Talking_Pdf::LEFT)+$this->_getBorderLineWidth(Talking_Pdf::RIGHT);
			} else {
				$this->_recommendedWidth = $props->getMaxWidth();
			}
			
			if (!$this->_height) {
				if ($this->_textLineSpacing) {
					$height=$this->_textLineSpacing*count($text_props['lines'])+$text_props['height'];
				} else {
					$height = $props->getHeight();
				}
				
				$this->_recommendedHeight = $height + ($this->_padding[Talking_Pdf::TOP]+$this->_padding[Talking_Pdf::BOTTOM]);
			}
			$this->_textProperties = $props;
		} else{
			throw new Talking__Pdf_Exception("not defined","preRender()");
		}
	}

	/**
	 * Render Cell
	 *
	 * @param Talking_Pdf_Page $page
	 * @param int $posX
	 * @param int $posY
	 */
	public function render(Talking_Pdf_Page $page, $posX, $posY) {
//		$this->_renderBackground($page, $posX, $posY);
		$this->_renderText( $page, $posX, $posY);
		$this->_renderBorder($page,$posX, $posY);
	}

	private function _renderText(Talking_Pdf_Page $page, $posX, $posY){
		if (!$this->_text) {
			return;
		}

		if(count($this->_textProperties->getLines()) > 1) {
			$lineHeight = $page->getFontHeight() * $this->_lineHeight;
			
			switch ($this->_vAlign){
				case Talking_Pdf::BOTTOM:
					$y_inc=$posY-$this->_textLineSpacing;
					$rev_lines=array_reverse($this->_text['lines']);
					foreach ($rev_lines as $line){
						$page->drawText($line,$this->_getTextPosX($posX), $this->_getTextPosY($page,$y_inc));
						$y_inc-=$line_height;
					}
					break;
				default:
					$y_inc=$posY-$this->_textLineSpacing;
					foreach ($this->_text['lines'] as $line){
						$page->drawText($line,$this->_getTextPosX($posX), $this->_getTextPosY($page,$y_inc));
						$y_inc+=$line_height;
					}
					break;
			}

			//@@TODO VERTICAL POSITIONING OF MULTI-LINED TEXT
			$y_inc=$posY-$this->_textLineSpacing; //@@TODO HACK
			$this->_vAlign=Talking_Pdf::TOP; //@@TODO ONLY TOP ALIGNEMENT IS VALID AT THIS MOMENT
			foreach ($this->_text['lines'] as $line){
				$page->drawText($line,$this->_getTextPosX($posX), $this->_getTextPosY($page,$y_inc));
				$y_inc+=$line_height;
			}


		}
		else{
			//write single line of text
			$page->drawText($this->_text['text'],$this->_getTextPosX($posX), $this->_getTextPosY($page,$posY));
		}
		//reset style
		$page->setStyle($page->getDefaultStyle());
	}


	private function _renderBorder(Talking_Pdf_Page $page, $posX, $posY) {
		if(!$this->_border) return;

		foreach ($this->_border as $key=>$style){
			$page->setStyle($style);
			switch ($key) {
				case Talking_Pdf::TOP:
					$page->drawLine(
									$posX, $posY-$this->_getBorderLineWidth(Talking_Pdf::TOP)/2,
									$posX+$this->_width, $posY-$this->_getBorderLineWidth(Talking_Pdf::TOP)/2,
									true
									);
					break;
				case Talking_Pdf::BOTTOM:
					$page->drawLine(
									$posX, $posY + $this->_height+$this->_getBorderLineWidth(Talking_Pdf::BOTTOM)/2,
									$posX+$this->_width, $posY + $this->_height+$this->_getBorderLineWidth(Talking_Pdf::BOTTOM)/2,
									true
									);
					break;
				case Talking_Pdf::RIGHT:
					//@@TODO INCLUDE BORDER LINE WIDTH??
					$page->drawLine(
									$posX+$this->_width,$posY,
									$posX+$this->_width,$posY+ $this->_height,
									true
								    );
					break;
				case Talking_Pdf::LEFT:
					//@@TODO INCLUDE BORDER LINE WIDTH??
					$page->drawLine(
									$posX, $posY,
									$posX, $posY+$this->_height,
									true
									);
					break;
			}
			//reset page style
			$page->setStyle($page->getDefaultStyle());
		}
	}

	/**
	 * Positions text horizontally (x-axis) adding alignment
	 * Default alignment: LEFT
	 * @param int $posX
	 * @return int
	 */
	private function _getTextPosX($posX){
		$x=0;
		switch ($this->_align){
			case Talking_Pdf::RIGHT:
				$x=$posX+$this->_width-$this->_text['width']-$this->_padding[Talking_Pdf::RIGHT]-$this->_getBorderLineWidth(Talking_Pdf::RIGHT)/2;
				break;
			case Talking_Pdf::CENTER:
				$x=$posX+$this->_width/2-$this->_text['width']/2;
				break;
			default: //LEFT
				$x=$posX+$this->_padding[Talking_Pdf::LEFT]+$this->_getBorderLineWidth(Talking_Pdf::LEFT)/2;
				break;
		}
		return $x;
	}

	/**
	 * Positions text vertically (y-axis) adding vertical alignment
	 * Default alignment: TOP
	 * @param Talking_Pdf_Page $page
	 * @param int $posY
	 * @return int
	 */
	private function _getTextPosY(Talking_Pdf_Page $page,$posY){
		$y=0;
		$line_height=/*$page->getFontHeight()+*/$this->_textLineSpacing;

		switch ($this->_vAlign){
			case Talking_Pdf::BOTTOM:
				$y=$posY+$this->_height-$this->_padding[Talking_Pdf::BOTTOM];
				break;
			case Talking_Pdf::MIDDLE:
				$y=$posY+$this->_height/2-$line_height/2;
				break;
			default: //TOP
				$y=$posY+$line_height+$this->_padding[Talking_Pdf::TOP];
				break;
		}
		return $y;
	}


	private function _getBorderLineWidth($position){
		if(isset($this->_border[$position])){
			$style=$this->_border[$position];
			$width=$style->getLineWidth();
		}
		else{
			$width= 0;
		}
		return $width;
	}

}
