<?php

class Talking_Pdf_TextProperties
{
	
	protected $_lines;
	protected $_text;
	
	protected $_font;
	protected $_fontSize;
	protected $_maxWidth;	

	public function __construct($text, $font, $fontSize, $maxWidth = PHP_INT_MAX)
	{
		$this->_text = $text;
		$this->_font = $font;
		$this->_fontSize = $fontSize;
		$this->_maxWidth = $maxWidth;
		
		$this->_splitLines();
		
	}

	protected function _getTextWidth($string)
	{
	     $drawingString = iconv('UTF-8', 'UTF-16BE//IGNORE', $string);
	     $characters = array();
	     for ($i = 0; $i < strlen($drawingString); $i++) {
	         $characters[] = (ord($drawingString[$i++]) << 8 ) | ord($drawingString[$i]);
	     }
	     $font = $this->_font;
	     $glyphs = $font->glyphNumbersForCharacters($characters);
	     $widths = $font->widthsForGlyphs($glyphs);
	     $stringWidth = (array_sum($widths) / $font->getUnitsPerEm()) * $this->_fontSize;
	     return $stringWidth;
	 }
	 
	protected function _splitLines()
	{
		$this->_lines = array();
		
		$lines = explode(PHP_EOL, $text);
        
		foreach( $lines as $line ){
			preg_match_all('/([^\s]*\s*)/i', $line, $matches);
            
			$words = $matches[1];
            
			$lineText = '';
			$lineWidth = 0;
			$prevLine = '';

			$ready = false;			
			
			foreach( $words as $word ) {
				
				$lineWidth = $this->_getTextWidth($lineText);
				if( $lineWidth >= $this->_maxWidth ){
            	$this->_lines[] = $lineText;
					$lineText = $word;
					$lineWidth = $this->_getTextWidth($word);
				}
			}
			
			$i = 0;
			$prevLine = '';
			do {
				$word = $words[$i];
				$lineText .= $word;
				$lineWidth = $this->_getTextWidth($lineText);

				if( $lineWidth >= $this->_maxWidth ){
            	$this->_lines[] = $prevLine;
					$lineText = "";
					$lineWidth = 0;
				} else {
					$i++;	
				}
				$prevLine = $lineText;

				if ($i >= count($words)) {
					$ready = true;	
				}
			} while(!$ready);
			
		}
	}

	
	public function getLines()
	{
		return $this->_lines;	
	}
	
}