<?php

class Talking_Toolbox_Array
{

	/**
	 * Replace some keys in an array to some others
	 *
	 * @param array $array Original array
	 * @param array $reps Associative with old keys as keys and new keys as values
	 * @return array Array with new keys
	 */
	public static function replaceKeys($array, $reps)
	{
		$return = array();
		foreach ($array as $key => $value) {
			$newKey = $key;

			if (array_key_exists($key, $reps)) {
				$newKey = $reps[$key];
			}
			$return[$newKey] = $value;
		}
		return $return;
	}

	/**
	 * Taken from http://snipplr.com/view.php?codeview&id=3491 with some modifications
	 * The main function for converting to an XML document.
	 * Pass in a multi dimensional array and this recursively loops through and builds up an XML document.
	 *
	 * @param array $data
	 * @param string $rootNodeName - what you want the root node to be - defaultsto data.
	 * @param SimpleXMLElement $xml - should only be used recursively
	 * @return string XML
	 */
	public static function toXml($data, $rootNodeName = 'data', $xml = null)
	{
		// turn off compatibility mode as simple xml throws a wobbly if you don't.
		if (ini_get('zend.ze1_compatibility_mode') == 1) {
			ini_set ('zend.ze1_compatibility_mode', 0);
		}

		if ($xml == null) {
			$xml = simplexml_load_string("<?xml version='1.0' encoding='utf-8'?><$rootNodeName />");
		}

		// loop through the data passed in.
		foreach($data as $key => $value) {
			// no numeric keys in our xml please!
			if (is_numeric($key)) {
				// make string key...
				$key = "unknownNode_". (string) $key;
			}

			// replace anything not alpha numeric
			$key = preg_replace('/[^a-z]/i', '', $key);

			// if there is another array found recrusively call this function
			if (is_array($value)) {
				$node = $xml->addChild($key);
				// recrusive call.
				Talking_Toolbox_Array::toXml($value, $rootNodeName, $node);
			} else {
				// add single node.
                                $value = htmlentities($value);
				$xml->addChild($key,$value);
			}

		}
		// pass back as string. or simple xml object if you want!
		return $xml->asXML();
	}

	/**
	 * Gets max in multi demonsional array
	 *
	 * @param array $array
	 * @return mixed
	 */
	public static function multiMax(array $array) {
		$return = (int)(PHP_INT_MAX+1);
		foreach( $array as $value ) {
			if (is_null($value)) break;
			if( is_array($value) ) {
				$subvalue = self::multiMax($value);
				
				if( $subvalue > $return ) {
					$return = $subvalue;
				}
				
			} elseif($value > $return) {
				$return = $value;
			}
		}

		return $return;
	}

	/**
	 * Gets min of multi dimensional array
	 * 
	 * @param array $array
	 * @return mixed
	 */
	public static function multiMin(array $array) {
		$return = PHP_INT_MAX;
		foreach( $array as $value ) {
			if (is_null($value)) break;
			elseif( is_array($value) ) {
				$subvalue = self::multiMin($value);
				
				if( $subvalue < $return ) {
					$return = $subvalue;
				}
				
			} elseif($value < $return) {
				$return = $value;
			}
		}

		return $return;
	}

	/**
	 * Merges any number of arrays / parameters recursively, replacing
	 * entries with string keys with values from latter arrays.
	 * If the entry or the next value to be assigned is an array, then it
	 * automagically treats both arguments as an array.
	 * Numeric entries are appended, not replaced, but only if they are
	 * unique
	 *
	 * calling: result = Talking_Toolbox_Array::mergeRecursiveDistinct(a1, a2, ... aN)
	**/
   public static function mergeRecursiveDistinct () {
    $arrays = func_get_args();
  $base = array_shift($arrays);
  if(!is_array($base)) $base = empty($base) ? array() : array($base);
  foreach($arrays as $append) {
    if(!is_array($append)) $append = array($append);
    foreach($append as $key => $value) {
      if(!array_key_exists($key, $base) and !is_numeric($key)) {
        $base[$key] = $append[$key];
        continue;
      }
      if(is_array($value) or is_array($base[$key])) {
        $base[$key] = self::mergeRecursiveDistinct($base[$key], $append[$key]);
      } else if(is_numeric($key)) {
        if(!in_array($value, $base)) $base[] = $value;
      } else {
        $base[$key] = $value;
      }
    }
  }
  return $base;
   }

}