<?php

class Talking_Toolbox_String
{

	/**
	 * Converts a string in Underscore case to Camel case
	 *
	 * @param string $name  A string in Underscore case (this_is_underscore_case)
	 * @return string  The string in Camel case (thisIsCamelCase)
	 * @see toUnderscores()
	 */
	static public function toCamel( $name )
	{
		return @preg_replace("/_([a-z])/e", "strtoupper('\\1')", $name);
	}

	/**
	 * Converts a string in Camel case to Underscore case
	 *
	 * @param string $name A string in Camel case (thisIsCamelCase)
	 * @return string The string in Underscore case (this_is_underscore_case)
	 * @see toCamel()
	 */
	static public function toUnderscores( $name )
	{
		return strtolower(preg_replace('/([a-z])([A-Z])/', "$1_$2", $name));
	}

	/**
	 * Converts a string to an URL friendly version
	 *
	 * @param string $s
	 * @return string url friendly variant of $s
	 */
	static public function toCleanUrl($s)
	{
		if(!$s) return 'page';

		$s = strtolower(htmlentities($s, ENT_QUOTES, $GLOBALS['CHARSET']));
		$s = preg_replace('/&(.)(?:acute|cedil|circ|ring|tilde|uml|grave|elig|slash);/', '\\1', $s);
		$s = preg_replace('/\W+/', '_', html_entity_decode($s));
		$s = preg_replace('/_{2,}/','_',$s);
		$s = trim($s,'_');

		if(!$s) $s = 'page';

		return $s;
	}


}
