<?php
/**
 * Uni-Form extension to Zend Framework
 *
 * LICENSE
 *
 * Due to the license of Uni-form these classes are release under a Creative
 * Commons License.
 *
 * @category   UniForm
 * @package    UniForm_Form
 * @copyright  Copyright (c) 2008 Gustaf Lindqvist (http://www.lookwhostalkingnow.com)
 * @license    http://creativecommons.org/licenses/by-sa/2.5/ Creative Commons License.
 * @version	   0.5
 */

/** Zend_Form */
require_once 'Zend/Form.php';

/**
 * Create a form with the smae markup as Uni-Form (http://sprawsm.com/uni-form/).
 *
 * @see	       Zend_Form
 */

class UniForm_Form extends Zend_Form 
{

	/**
	 * Path to UniForm css files
	 *
	 * @var string
	 */
	protected static $_cssDirectory = null;

	/**
	 * Initialize form with view helpers and markup.
	 *
	 * @return void
	 */
	public function init()
	{

		parent::init();

		$view = $this->getView();
		$view->addHelperPath('UniForm/View/Helper', 'UniForm_View_Helper');

		if (self::$_cssDirectory) {
			$view->headLink()
				->appendStylesheet(self::$_cssDirectory . '/uni-form.css')
				->appendStylesheet(self::$_cssDirectory . '/uni-form-lte-ie7.css', "screen", 'lte IE 7');
		};

		// Path setting for custom decorations MUST ALWAYS be first!
		$this->addElementPrefixPath('UniForm_Form_Decorator', 'UniForm/Form/Decorator/', 'decorator');
		$this->addPrefixPath('UniForm_Form_Decorator', 'UniForm/Form/Decorator/', 'decorator');

		$this->setAttrib('accept-charset', 'UTF-8');
    
		$this->setDecorators(
			array(
				array('UniFormFormErrors', array("class" => "dummy")),
				array('Description', array('escape' => false, 'class' => 'form-hint', 'tag'=>'p')),
				'FormElements',
				array('Form', array('class' => 'uniForm'))
			)
		);

	}


	/**
	 * Adds Uni-form display group for settings <fieldset>-class.
	 *
	 * @see Zend_Form::addDisplayGroup()
	 */
	public function addDisplayGroup(array $elements, $name, $options = null) {
		if (is_null($options))
		$options = array();

		if (!isset($options['displayGroupClass']))
		$options['displayGroupClass'] = 'UniForm_Form_DisplayGroup';

		parent::addDisplayGroup($elements, $name, $options);
	}

	/**
	 * Adds error as class on the element div-wrappers when errors.
	 *
	 * @see Zend_Form::isValid
	 */
	public function isValid($formData) {
		$isValid = parent::isValid($formData);
		if(!$isValid) {
			$errors = $this->getErrors();
			foreach($errors as $fn => $error) {
				if(empty($error)) continue;
				$el = $this->getElement($fn);
				$el->markAsError();
			}
		}
		return $isValid;
	}

	/**
	 * Set css path if you want automagically including of the css files
	 *
	 * @param string $path Path to UniForm css dir relative to basedir
	 */
	public static function setCssDirectory($path)
	{
		$path = Talking_Filesystem_File::getAbsolutePath($path, true);
        self::$_cssDirectory = $path;
	}

	/**
	 * Append stylesheets
	 *
	 * @return void
	 */
	public static function appendStylesheets()
	{
		$path = Talking_Filesystem_File::getAbsolutePath($path, true);
        self::$_cssDirectory = $path;
	}

}