<?php

class WebReport_Acl extends Zend_Acl {

	protected static $_instance = null;

	private function __construct()
	{}

	private function __clone()
	{}

	protected function init()
	{

		$this->deny();
		$this->add(new Zend_Acl_Resource('default:index'));
		$this->add(new Zend_Acl_Resource('default:error'));
		$this->add(new Zend_Acl_Resource('admin:login'));
		$this->add(new Zend_Acl_Resource('api:index'));
		$this->add(new Zend_Acl_Resource('admin:index'));
		$this->add(new Zend_Acl_Resource('admin:about'));

		$user = Zend_Auth::getInstance()->getIdentity();

		if ($user) {
			$usersDb = new Core_Model_Users();
			$user = $usersDb->find($user->user_id)->current();
			$this->createAcl($user);
		} else {
			$this->addRole(new Zend_Acl_Role(0));
			$this->allow(0, "default:index", "index");
			$this->allow(0, "default:error", "error");
			$this->allow(0, "admin:login", "index");
			$this->allow(0, "admin:login", "logout");
			$this->allow(0, "api:index");
		}


	}


	public function createAcl(Core_Model_User $user) {
		$roleId = $user->acl_role_id;
		$db = Zend_Registry::get('db');

/*$sql = "SELECT SUBSTR(module.alias,5) as module, acl_resource.controller, acl_privilege.acl_privilege_id, action, param1_key AS p1k, param1_value AS p1v, param2_key AS p2k, param2_value AS p2v, param3_key AS p3k, param3_value AS p3v
					FROM module
					  JOIN acl_resource ON acl_resource.module_id = module.module_id
					JOIN acl_privilege_group ON acl_resource.acl_resource_id = acl_privilege_group.acl_resource_id
					JOIN acl_privilege_group_assembly ON acl_privilege_group.acl_privilege_group_id = acl_privilege_group_assembly.acl_privilege_group_id
					JOIN acl_privilege_group_assembly_privilege ON acl_privilege_group_assembly_privilege.acl_privilege_group_assembly_id = acl_privilege_group_assembly.acl_privilege_group_assembly_id
					JOIN acl_privilege ON acl_privilege.acl_privilege_id = acl_privilege_group_assembly_privilege.acl_privilege_id
					JOIN acl_role_privilege_group_assembly ON acl_role_privilege_group_assembly.acl_privilege_group_assembly_id = acl_privilege_group_assembly.acl_privilege_group_assembly_id
					JOIN acl_role ON acl_role_privilege_group_assembly.acl_role_id = acl_role.acl_role_id
					WHERE acl_role.acl_role_id = ?";*/
		$sql =  "SELECT SUBSTR(module.alias,5) as module, acl_resource.controller, acl_privilege.acl_privilege_id, action, param1_key AS p1k, param1_value AS p1v, param2_key AS p2k, param2_value AS p2v, param3_key AS p3k, param3_value AS p3v
			FROM `acl_role_privilege_group_assembly`
			LEFT JOIN acl_privilege_group_assembly ON acl_role_privilege_group_assembly.acl_privilege_group_assembly_id = acl_privilege_group_assembly.acl_privilege_group_assembly_id
			LEFT JOIN acl_privilege_group_assembly_privilege ON acl_privilege_group_assembly.acl_privilege_group_assembly_id = acl_privilege_group_assembly_privilege.acl_privilege_group_assembly_id
			LEFT JOIN acl_privilege ON acl_privilege_group_assembly_privilege.acl_privilege_id = acl_privilege.acl_privilege_id
			LEFT JOIN acl_resource ON acl_privilege.acl_resource_id = acl_resource.acl_resource_id
			LEFT JOIN module ON acl_resource.module_id = module.module_id
			WHERE acl_role_id = ? ORDER BY controller";
		
		$rows = $db->fetchAll($sql, $roleId);
        

		$this->addRole(new Zend_Acl_Role($roleId));

		foreach ($rows as $row) {
			if (!$this->has($row->module . ':' . $row->controller)) {
				$this->add(new Zend_Acl_Resource($row->module . ':' . $row->controller));
			}
		}
		

		$this->deny();
		$this->allow($roleId, 'default:index', "index");
		$this->allow($roleId, 'default:error', "error");
		$this->allow($roleId, 'admin:login', "index");
		$this->allow($roleId, 'admin:login', "logout");

		$this->allow($roleId, 'admin:index', "index"); // dashboard
		$this->allow($roleId, 'admin:about', "index"); // about page
		//$this->allow($roleId, "api:index");

		if (APPLICATION_ENVIRONMENT == 'development') {

			if (!$this->has("pos:index")) {
				$this->add(new Zend_Acl_Resource("pos:index"));
			}
			$this->allow($roleId, 'pos:index');

			if (!$this->has("admin:acl-admin")) {
				$this->add(new Zend_Acl_Resource("admin:acl-admin"));
			}
			$this->allow($roleId, 'admin:acl-admin');
		}

		foreach ($rows as $row) {
			$action = $row->action;

			$pks = array();

			if ($row->p1k != '') $pks[$row->p1k] = $row->p1v;
			if ($row->p2k != '') $pks[$row->p2k] = $row->p2v;
			if ($row->p3k != '') $pks[$row->p3k] = $row->p3v;

			$pksCount = count($pks);
			if ($pksCount > 0) {
				if ($pksCount > 1) ksort($pks);
				foreach ($pks as $key => $value) {
					$action = $action . ':' . $key . ':' . $value;
				}
			}

			$this->allow($roleId, $row->module . ':' . $row->controller, $action);
			//echo "allow: " . $role['role_id'] . " to " . $role['module_alias'] . ':' . $role['resource_alias'] . " and " . $role['privilege_alias'] . "<br />";
		}

	}






	/**
	 * Check if url options is allowed.
	 *
	 * @see Zend_View_Helper_Url
	 * @param array $urlOptions
	 * @return bool
	 */
	public function isUrlOptionsAllowed($urlOptions, $name = null)
	{

		$user = Zend_Auth::getInstance()->getIdentity();

		$front = Zend_Controller_Front::getInstance();
		$router = $front->getRouter();
		if (!is_null($name)) {
			$route = $router->getRoute($name);
			$defs = $route->getDefaults();
			$urlOptions['module'] = $defs['module'];
			$urlOptions['controller'] = $defs['controller'];
			$urlOptions['action'] = $defs['action'];
		}
		
		$roleId = 0;
		if ($user) {
			$roleId = $user->acl_role_id;
		}

		if (isset($urlOptions['module'])) { $moduleName = $urlOptions['module']; unset($urlOptions['module']); }
		else $moduleName = 'default';
		if (isset($urlOptions['controller'])) { $controllerName = $urlOptions['controller']; unset($urlOptions['controller']); }
		else $controllerName = 'index';
		if (isset($urlOptions['action'])) { $actionName = $urlOptions['action']; unset($urlOptions['action']); }
		else $actionName = 'index';


		if (!$this->hasRole($roleId) || !$this->has($moduleName . ':' . $controllerName)) {
			return false;
		}


		$nbrOfPars = count($urlOptions);
		
		if ($nbrOfPars > 0) {
			ksort($urlOptions);
			$total = $nbrOfPars;
			for ($i = $nbrOfPars; $i >= 1; --$i) {
				
				$choose = $i;
				$c = new WebReport_Combination($total,$choose);
				$nbrOfCombs = WebReport_Combination::choose($total,$choose);
				$optKeys = array_keys($urlOptions);

				for ($j = 0; $j < $nbrOfCombs; ++$j) {
					$testStr = $actionName;
					foreach ($c->data as $number) {
						$key = $optKeys[$number];
						$value = $urlOptions[$key];
						$testStr .= ':' . $key . ':' . $value;
					}
					
					if ($this->isAllowed($roleId, $moduleName . ':' . $controllerName, $testStr)) {
						return true;
					}
					$c = $c->successor();
				}

			}
		}


		if ($this->isAllowed($roleId, $moduleName . ':' . $controllerName, $actionName)) {
			return true;
		}

		return false;
	}



	public static function getInstance()
	{

		if (null === self::$_instance) {
			self::$_instance = new self();
			self::$_instance->init();
		}

		return self::$_instance;
	}

}
