<?php

class WebReport_Api
{

    const FORMAT_JSON = 'json';
    const FORMAT_JSONP = 'jsonp';
    const FORMAT_PRETTY_JSON = 'prettyjson';

    const CODE_OK = 0;
    const CODE_OTHER_ERROR = 1;
    const CODE_CMDECRYPT_ERROR = 2;
    const CODE_INVALID_METHOD = 3;
    const CODE_AUTH_FAILED = 4;
    const CODE_INVALID_FORMAT = 5;
    const CODE_INVALID_PARAMETERS = 6;
    const CODE_PERMISSION_DENIED = 7;

    protected $_methods = array();

    public function register($resource)
    {
        if (!($resource instanceof WebReport_Api_Resource_Abstract))
        {
            throw new WebReport_Api_Exception("resource must be instance of WebReport_Api_Method_Abstract", WebReport_Api::CODE_OTHER_ERROR);
        }

        $methods = $resource->availableMethods();

        $this->_methods = array_merge($this->_methods, $methods);
    }

    public function getMethod($module, $method)
    {
        $m = $module . "." . $method;

        if (array_key_exists($m, $this->_methods)) {
            $className = $this->_methods[$m];

            return new $className();
        } else {
            throw new WebReport_Api_Exception("Invalid method.", WebReport_Api::CODE_INVALID_METHOD);
        }
    }

}
