<?php

class WebReport_Combination
{
	protected $n = 0;
	protected $k = 0;
	public $data = null;

	public function __construct($n, $k, array $arr = null)
	{
		if ($n < 0 || $k < 0) // normally n >= k
		throw new Exception("Negative parameter in constructor");

		$this->n = $n;
		$this->k = $k;
		$this->data = array();

		if ($arr) {
			for ($i = 0; $i < count($arr); ++$i)
				$this->data[$i] = $arr[$i];

			if (!$this->isValid()) {
				throw new WebReport_Exception("Bad value from array");
			}
		} else {
			for ($i = 0; $i < $k; ++$i)
				$this->data[$i] = $i;
		}

	}


	public function isValid()
	{
		if (count($this->data) != $this->k)
		return false; // corrupted

		for ($i = 0; $i < $this->k; ++$i)  {
			if ($this->data[$i] < 0 || $this->data[$i] > $this->n - 1)
			return false; // value out of range

			for ($j = $i+1; $j < $this->k; ++$j)
			if ($this->data[$i] >= $this->data[$j])
			return false; // duplicate or not lexicographic
		}

		return true;
	} // IsValid()

	public function __toString()
	{
		$s = "{ ";
		for ($i = 0; $i < $this->k; ++$i) {
			$s .= $this->data[$i] . " ";
		}
		$s .= " }";
		return $s;
	} // ToString()

	public function successor()
	{
		if ($this->data[0] == $this->n - $this->k)
		return null;

		$ans = new WebReport_Combination($this->n, $this->k);

		for ($i = 0; $i < $this->k; ++$i)
		$ans->data[$i] = $this->data[$i];

		for ($i = $this->k - 1; $i > 0 && $ans->data[$i] == $this->n - $this->k + $i; --$i);

		++$ans->data[$i];

		for ($j = $i; $j < $this->k - 1; ++$j) {
			$ans->data[$j+1] = $ans->data[$j] + 1;
		}

		return $ans;
	} 

	public static function choose($n, $k)
	{
		if ($n < 0 || $k < 0) {
			throw new WebReport_Exception("Invalid negative parameter");
		}
		if ($n < $k) {
			return 0;  // special case
		}
		if ($n == $k) {
			return 1;
		}

		$delta = 0;
		$iMax = 0;
		if ($k < $n-$k) { // ex: Choose(100,3)
			$delta = $n-$k;
			$iMax = $k;
		} else {         // ex: Choose(100,97)
			$delta = $k;
			$iMax = $n-$k;
		}

		$ans = $delta + 1;

		for ($i = 2; $i <= $iMax; ++$i) {
			$ans = ($ans * ($delta + $i)) / $i;
		}

		return $ans;
	} 

} 
