<?php

class WebReport_Export_Fine extends WebReport_Export_Abstract
{

    public function render($filename = null, $row_limit = null)
    {
        if (!$filename) {
            $filename = "WebReport-Csv-" . date("Ymdhis");
        }

        $headers = $this->getHeaders();
        $data = $this->getData();
        $source_header = "Source";
        $cameraDb = Core_Model_Cameras::getInstance();
        $appDb = Core_Model_Applications::getInstance();
        $sources = array_slice($headers, 1);
        //TODO: Move aggregation to earlier step, shouldn't be done here.
        if ($this->_settings['aggregate']) {
            $aggregated_data = array();
            $types = array();
            foreach ($data as $k => $row) {
                $aggregated_data[$k] = array();
                foreach ($row as $key => $value) {
                    if ($key != "cnttime" &&  strpos($key, 'exists') == false) {
                        $key_array = explode("_", $key, 2);
                        $cam = $key_array[0];
                        $source = $sources[$key];
                        $pos = strrpos($source, "(");
                        if ($pos !== false) {
                            $type = substr($source, $pos+1, strlen($source)-$pos-2);
                        } else {
                            $type = $key_array[1];
                        }
                        $types[] = $type;
                        if (!isset($aggregated_data[$k][$type])) {
                            $aggregated_data[$k][$type]['count'] = $value;
                        } else {
                            $aggregated_data[$k][$type]['count'] += $value;
                        }
                        if (!isset($aggregated_data[$k]['app_name'])) {
                            $cam_id = (int) substr($cam, 1);
                            $camera = $cameraDb->getCamera($cam_id);
                            $app = $appDb->findWithId($camera->application_id);
                            $aggregated_data[$k][$type]['app_name'] = $app->getName();
                        }
                    } elseif ($key == "cnttime") {
                        $aggregated_data[$k]["cnttime"] = $value;
                    }
                }
            }
            $source_header = "Source type";
            $data = $aggregated_data;
            $types = array_unique($types);
        }
        ob_start();
        $response = Zend_Controller_Front::getInstance()->getResponse();
        $response->setHeader("Content-Type", "text/plain");
        $response->setHeader("Content-Disposition", 'filename="' . $filename . '.txt"');

        if ($this->_settings['type'] == Export_Model_Exports::COUNT_DATA_EXPORT) {
            $ck = array_keys($sources);
            $headers = array($headers['cnttime'], $source_header, "Type", "Count");
        } elseif ($this->_settings['type'] == Export_Model_Exports::EVENT_DATA_EXPORT) {
            $sources = array_slice($headers, 2);
            $ck = array_keys($sources);
            $pos = strrpos($headers['event_id_header'], "(");
            unset($ck['event_id_header']);
            if ($pos !== false) {
                $event_id_header = substr($headers['event_id_header'], $pos+1, strlen($headers['event_id_header'])-$pos-2);
                $event_id_column = substr($headers['event_id_header'], 0, $pos);
            }
            $headers = array($headers['cnttime'], "Source", $event_id_header, "Value type", "Value");
            foreach ($ck as $key => $value) {
                if (strpos(trim($value), trim($event_id_column)) !== false) {
                    $pos = strpos($value, "_");
                    if ($pos !== false) {
                        $event_id_key = substr($value, $pos+1, strlen($value));
                    }
                    unset($ck[$key]);
                }
            }
        }
        echo implode(", ", $headers) . "\n";
        $rows_printed=0;
        foreach ($data as $row) {
            $toEcho = array();
            $row = (array)$row;
            if ($this->_settings['aggregate']) {
                foreach ($types as $type) {
                    if (!isset($row_limit) || $rows_printed < $row_limit) {
                        echo  $row['cnttime']       .", " .
                              $row[$type]['app_name'] ." (sum), " .
                              $type                 .", " .
                              $row[$type]['count']  ."\n";
                        $rows_printed++;
                    }
                }
            } else {
                foreach ($ck as $k) {
                    if (isset($row[$k])) { //If no value for now, don't do anything.
                        if (!isset($row_limit) || $rows_printed < $row_limit) {
                            $source = $sources[$k];
                            $type = "?";
                            $pos = strrpos($source, "(");
                            if ($pos !== false) {
                                $type = substr($source, $pos+1, strlen($source)-$pos-2);
                                $source = substr($source, 0, $pos);
                            }
                            $echo_str = $row['cnttime'] .", " . trim($source) . ", ";
                            if ($this->_settings['type'] == Export_Model_Exports::EVENT_DATA_EXPORT) {
                                $pos = strpos($k, "_");
                                if ($pos !== false) {
                                    $cam = substr($k, 0, $pos);
                                }
                                $echo_str .= $row[$cam . "_" . $event_id_key]   .", ";
                            }
                            $echo_str .= $type .", " . $row[$k] . "\n";
                            echo  $echo_str;
                            $rows_printed++;
                        }
                    }
                }
            }
        }
        if (isset($row_limit) && $row_limit <= $rows_printed) {
            echo "...\n";
            echo "#### Preview: Export truncated after $row_limit rows. ####";
        } elseif ($rows_printed==0) {
            echo "\n";
            echo "#### No data found for selected time interval and export content. ####";
        }
        ob_end_flush();
    }
}
