<?php

class WebReport_Formula {

	protected $_operators = array("+", "-", "*", "/", "^");
	protected $_operatorsAssociative = array("(" => "none", ")" => "none", "+" => "left", "-" => "left", "/" => "left", "*" => "left", "^" => "right");
	protected $_operatorsPrecedence = array("(" => 0, ")" => 0, "+" => 5, "-" => 5, "/" => 8, "*" => 8, "^" => 9);

	protected $_cameras;
    protected $_cameras_clean;
        protected $_cameras_cleaner;
	protected $_camerasOutTypes = array();
	protected $_formula;

	public function __construct($infixExpression) {
		$infixExpression = trim($infixExpression);
		$this->_formula = $infixExpression;
		$this->_infixToPostfix($infixExpression);
	}

	public function toReadable($shortname=NULL)
	{
        if( $shortname == NULL)
            $shortname = $this->_formula;
		$return = str_replace(array_keys($this->_cameras), array_values($this->_cameras), $shortname);
		return $return;

	}
    
    public function toReadable_notags($shortname=NULL)
	{
        if( $shortname == NULL)
            $shortname = $this->_formula;
		$return = str_replace(array_keys($this->_cameras_clean), array_values($this->_cameras_clean), $shortname);
		return $return;
	}
    
    public function toReadableNameOnly($shortname=NULL)
	{
        if( $shortname == NULL)
            $shortname = $this->_formula;
		$return = str_replace(array_keys($this->_cameras_cleaner), array_values($this->_cameras_cleaner), $shortname);
		return $return;
	}
    
    
	protected function _infixToPostfix($expr)
	{
		$operators = $this->_operators;
		$assoc = $this->_operatorsAssociative;
		$prec = $this->_operatorsPrecedence;
		$cameras = array();
		$expr = explode(" ", $expr);
		$output = array();
		$opStack = array();
		foreach ($expr as $token) {
			if (is_numeric($token)) {
				$output[] = $token;
			} elseif (preg_match("/c([0-9]+)_([A-z_]+)/", $token)) { // camera
				$cameras[] = $token;
				$output[] = $token;
			} elseif (preg_match("/v([0-9]+)_([A-z_]+)/", $token)) { // virtual camera
				$output[] = $token;
			} elseif (preg_match("/f([0-9]+)/", $token)) { // formula
				$output[] = $token;
			} elseif (in_array($token, $operators)) {
				if (empty($opStack)) {
					$opStack[] = $token;
				} else {
					$op1 = $token;
					$op2 = $this->_arrayLast($opStack) ;
					while ($op2 &&
						(
							($assoc[$op1] == "left" && $prec[$op1] <= $prec[$op2]) ||
							($assoc[$op1] == "right" && $prec[$op1] < $prec[$op2])
						)
					) {
						$op2 = array_pop($opStack);
						$output[] = $op2;
						$op2 = $this->_arrayLast($opStack);
					}
					$opStack[] = $op1;

				}
			} elseif ($token == "(") {
				$opStack[] = $token;
			} elseif ($token == ")") {
				$output[] = $token;
				$top = $this->_arrayLast($opStack);
				while ($top != "(") {
					if ($top === false) {
						throw new Exception("paranthesis do not match");
					}
					$output[] = array_pop($opStack);
					$top = $this->_arrayLast($opStack);
				}
				$output[] = array_pop($opStack);

			} else {
				throw new WebReport_Exception("Not valid token: $token");
			}


		}

		while (!is_null($op = array_pop($opStack))) {
			$output[] = $op;
		}

		$cameraTypes = array_unique($cameras);
		$cts = array();
		foreach ($cameraTypes as $ct) {
			$usPos = strpos($ct, "_");
			$id = (int)substr($ct, 1, $usPos-1);
			$type = substr($ct, $usPos+1);
			if (!isset($cts[$id])) {
				$cts[$id][] = $type;
			} elseif (!in_array($type, $cts[$id])) {
				$cts[$id][] = $type;
			}
		}

		$camerasDb = new Core_Model_Cameras();
		$cameraIds = array_keys($cts);
		$select = $camerasDb->select()->from($camerasDb, array("camera_id", "name"));
		$cameras = $camerasDb->getAdapter()->fetchPairs($select);

		$c = array();
        $c_clean = array();

		$translate = Zend_Registry::get('Zend_Translate');
		$camerasOutTypes = array();

		foreach ($cameraTypes as $ct) {
			$usPos = strpos($ct, "_");
            $id = (int)substr($ct, 1, $usPos-1);
            if(!array_key_exists($id, $cameras)) continue;
			$type = substr($ct, $usPos+1);

			$camerasOutTypes[$id][] = $type;
			$transIndex = "COUNT_TYPE_OUT_" . strtoupper($type);
			$c[$ct] = '<span class="camera">' . 
				$cameras[$id] . ' <span class="count-type">(' .
				$translate->_($transIndex) . ")</span></span>";
            $c_clean[$ct] = $cameras[$id] ." (" . $translate->_($transIndex) . ")";
            $c_cleaner[$ct] = $cameras[$id];
		}
		$this->_cameras = $c;
        $this->_cameras_clean = $c_clean;
        $this->_cameras_cleaner = $c_cleaner;
		foreach ($camerasOutTypes as $cameraId => $types) {
			$this->_camerasOutTypes[$cameraId] = array_unique($types);
		}

		return implode(" ", $output);

	}

	public function getCameraIdsAndOutTypesInvolved()
	{
		return $this->_camerasOutTypes;
	}

	protected function _postfixToInfix($expr)
	{
		$operators = $this->_operators;
		$assoc = $this->_operatorsAssociative;
		$prec = $this->_operatorsPrecedence;

		$stack = array();
		$expr = explode(" ", $expr);

		foreach ($expr as $token) {

			if (is_numeric($token)) {
				$stack[] = $token;
			} elseif (preg_match("/v([0-9]+)_([A-z_]+)/", $token)) {
				$stack[] = $token;
			} elseif (preg_match("/v([0-9]+)_([A-z_]+)/", $token)) {
				$stack[] = $token;
			} elseif (preg_match("/f([0-9]+)/", $token)) {
				$stack[] = $token;
			} elseif (in_array($token, $operators)) {
				$l = array_pop($stack);
				$r = array_pop($stack);
				$token = $r . " " . $token . " " . $l;
				$stack[] = $token;
			} elseif ($token == ")") {
				$v = array_pop($stack);
				$stack[] = $v . " )";
			} elseif ($token == "(") {
				$v = array_pop($stack);
				$stack[] = "( " . $v;
			}
		}
		return implode(" ", $stack);

	}

	private function _arrayLast($arr) {
		$count = count($arr);
		if ($count == 0) return false;
		else return $arr[$count-1];
	}

/*
	private function _maskParanthesis($str) {
		$parStack = 0;
		$temp = "";

		for ($i = 0; $i < strlen($str); $i++) {
			$char = $str{$i};
			if ($char == "(") {
				$parStack++;
			}
			if ($char == ")") {
				$parStack--;
			}

			if ($parStack == 0) {
				$temp = $temp . $char;
			} else {
				$temp = $temp . "(";
			}
		}
		if ($parStack != 0) {
			throw new Exception("Paranthesis do not match");
		}
		return $temp;
	}
*/

}
