<?php

class WebReport_Log_Writer_Db extends Zend_Log_Writer_Abstract
{
    /*
    */
    const MAX_MESSAGES = 10000;
    /**
     * Database adapter instance
     * @var Zend_Db_Adapter
     */
    private $_db;

	/**
	 * Log alias.
	 * @var string
	 */
	private $_alias;

	/**
	 * Company id for log
	 * @var int
	 */
	private $_companyId;

    /**
     * Name of the log table in the database
     * @var string
     */
    private $_table;

    /**
     * Relates database columns names to log data field keys.
     *
     * @var null|array
     */
    private $_columnMap;

	/**
	 * Log table row for caching.
	 *
	 * @var Zend_Db_Table_Row
	 */
	private $_dbRow;

    /**
     * Class constructor
     *
     * @param Zend_Db_Adapter $db   Database adapter instance
     * @param string $table         Log table in database
     * @param array $columnMap
     */
    public function __construct($db, $alias, $companyId, $table, $columnMap = null)
    {
        $this->_db    = $db;
		$this->_alias = $alias;
		$this->_companyId = $companyId;
        $this->_table = $table;
        $this->_columnMap = $columnMap;
    }

    /**
     * Create a new instance of Zend_Log_Writer_Db
     *
     * @param  array|Zend_Config $config
     * @return Zend_Log_Writer_Db
     * @throws Zend_Log_Exception
     */
    static public function factory($config)
    {
        $config = self::_parseConfig($config);
        $config = array_merge(array(
            'db'        => null,
			'companyId' => null,
			'alias'		=> null,
            'table'     => null,
            'columnMap' => null,
        ), $config);

        if (isset($config['columnmap'])) {
            $config['columnMap'] = $config['columnmap'];
        }

        return new self(
            $config['db'],
			$config['alias'],
			$config['companyId'],
            $config['table'],
            $config['columnMap']
        );
    }

    /**
     * Formatting is not possible on this writer
     */
    public function setFormatter(Zend_Log_Formatter_Interface $formatter)
    {
        require_once 'Zend/Log/Exception.php';
        throw new Zend_Log_Exception(get_class() . ' does not support formatting');
    }

    /**
     * Remove reference to database adapter
     *
     * @return void
     */
    public function shutdown()
    {
        $this->_db = null;
    }

    /**
     * Write a message to the log.
     *
     * @param  array  $event  event data
     * @return void
     */
    protected function _write($event)
    {
        if ($this->_db === null) {
            require_once 'Zend/Log/Exception.php';
            throw new Zend_Log_Exception('Database adapter is null');
        }

        if ($this->_columnMap === null) {
            $dataToInsert = $event;
        } else {
            $dataToInsert = array();
            foreach ($this->_columnMap as $columnName => $fieldKey) {
                $dataToInsert[$columnName] = $event[$fieldKey];
            }
        }



        $logs = new Core_Model_Logs();
		if (!$this->_dbRow) {
			$log = $logs->fetchRowWithAlias($this->_alias);
			$this->_dbRow = $log;
		} else {
			$log = $this->_dbRow;
		}

		$companyId = $this->_companyId;

		if (!$companyId) {
			// user maybe have logged in since logger were initialized.
			// check for that
			if (Zend_Auth::getInstance()->hasIdentity()) {
				$companyId = Zend_Auth::getInstance()->getIdentity()->company_id;
			}
		}

		if ($companyId <= 0) {
			$companyId = new Zend_Db_Expr('NULL');
		}

        $dataToInsert['log_id'] = $log->log_id;

        $dataToInsert['company_id'] = $companyId;

 
        $numToRemove = $log->getNumberOfMessages() - self::MAX_MESSAGES + 1;
        if ($numToRemove>0){ 
            $this->_db->query("delete from $this->_table where log_id = $log->log_id order by log_message_id asc limit $numToRemove");
        }
        $this->_db->insert($this->_table, $dataToInsert);
    }
}
