<?php


class WebReport_Loggers
{

    /**
     *
     * @var Core_Model_Logs
     */
    protected $_logTable = NULL;

    protected $_logObjects = NULL; // Lazy loading deluxe
    
    protected $_loggers = array();

    public function getLogger($alias)
    {
        $logger = null;
        if (array_key_exists($alias, $this->_loggers)) {
            $logger = $this->_loggers[$alias];
        } else {
            $logObjects = $this->_getLogObjects();
            $logObject = null;
            $logTable = $this->_getLogTable();

            if (array_key_exists($alias, $logObjects)) {
                $logObject = $logObjects[$alias];
            } else {
                $log = $logTable->createRow();
                $log->text = strtoupper($alias) . "_LOG";
                $log->alias = $alias;
                $log->save();
                $this->_logObjects[$alias] = $log;
                $logObject = $log;
            }


            $auth = Zend_Auth::getInstance();

            $companyId = NULL;

            if ($auth->hasIdentity()) {
                $identity = $auth->getIdentity();
                $companyId = $identity->company_id;
            }
            
            $columnMapping = array("date" => "timestamp", "priority_id" => "priority", "message" => "message");
            $writer = new WebReport_Log_Writer_Db($logTable->getAdapter(), $logObject->alias, NULL, 'log_message', $columnMapping);
            $logger = new Zend_Log($writer);
            $loggers[$alias] = $logger;
        }

        return $logger;
    }

    public function __get($alias)
    {
        return $this->getLogger($alias);
    }

    protected function _getLogObjects()
    {
        if ( is_null($this->_logObjects) ) {
            $this->_logObjects = array();

            $logs = $this->_getLogTable();

            $logObject = $logs->fetchAll();

            foreach ($logObject as $log) {
                $this->_logObjects[$log->alias] = $log;
            }

        }

        return $this->_logObjects;
    }

    protected function _getLogTable()
    {
        if ( is_null($this->_logTable) ) {
            $this->_logTable = new Core_Model_Logs();
        }

        return $this->_logTable;
    }
}


