<?php

abstract class WebReport_Model_Abstract 
{

	protected $_mapperClass;
	protected $_mapper;
	

    public function __construct(array $options = null)
    {
        if (is_array($options)) {
            $this->setOptions($options);
        }
    }

    public function __set($name, $value)
    {
        $method = 'set' . $name;
        if (('mapper' == $name) || !method_exists($this, $method)) {
            throw new Exception('Invalid ' . get_class($this) . ' property');
        }
        $this->$method($value);
        return $this;
    }

    public function __get($name)
    {

        $method = 'get' . ucfirst($name);
        if (!method_exists($this, $method)) {
            throw new Exception('Invalid ' . get_class($this) . ' property');
        }
        return $this->$method();
    }

	public function getMapper()
	{

		if (null === $this->_mapper) {
			$this->_mapper = $this->setMapper($this->_mapperClass);
		}
		return $this->_mapper;
	}
	
    public function setMapper($mapperClass)
    {

        if (is_string($mapperClass)) {
            $mapperClass = new $mapperClass();
        }
        
        if (!$mapperClass instanceof WebReport_Model_Mapper_Abstract) {
            throw new Exception('Invalid model mapper provided');
        }

        $this->_mapper = $mapperClass;
        return $mapperClass;
    }


    public function setOptions(array $options)
    {
        $methods = get_class_methods($this);
        foreach ($options as $key => $value) {
            $method = 'set' . ucfirst($key);
            if (in_array($method, $methods)) {
                $this->$method($value);
            }
        }
        return $this;
    }

}