<?php

abstract class WebReport_Model_Mapper_Abstract  {
	
	
	protected $_dbTableClass = null;
	protected $_dbTable;

    public function setDbTable($dbTable)
    {

        if (is_string($dbTable)) {
            $dbTable = new $dbTable();
        }
        
        if (!$dbTable instanceof Zend_Db_Table_Abstract) {
            throw new Exception('Invalid table data gateway provided');
        }

        $this->_dbTable = $dbTable;
        return $this;
    }

    public function getDbTable()
    {
        if (null === $this->_dbTable) {
            $this->setDbTable($this->_dbTableClass);
        }
        return $this->_dbTable;
    }
	
	/**
	  *  $populateEntryRecursive can be set to false for performence boost
	  */
    public function find($id, $deepPopulate = true, $params = array())
    {
        $result = $this->getDbTable()->find($id);

        if (0 == count($result)) {
            return null;
        }
        
        $row = $result->current();
       
        return $this->_populateFromRow($row, $deepPopulate, $params);
    }

	/**
	  *  $populateEntryRecursive can be set to false for performence boost
	  */
    public function findAsRow($id, $deepPopulate = true, $params = array())
    {
        $result = $this->getDbTable()->find($id);

        if (0 == count($result)) {
            return null;
        }
        
        $row = $result->current();

        return $row;
    }

    public function fetchAll($options = array())
    {
    	$where = null; $order = null; $count = null; $offset = null; $deepPopulate = true;
    	if (!is_array($options)) {
    		throw new WebReport_Model_Exception("Invalid options, must be an array");	
    	}
    	foreach ($options as $key => $val) {
    		switch ($key) {
    			case "where" : $where = $val; unset($options["where"]); break;
    			case "order" : $order = $val;	unset($options["order"]); break;
    			case "count" : $count = $val;	unset($options["count"]); break;
				case "offset" : $offset = $val; unset($options["offset"]); break;
				case "deepPopulate" : $deepPopulate = $val; unset($options["deepPopulate"]); break;

    		}
 		}

      $resultSet = $this->getDbTable()->fetchAll($where, $order, $count, $offset);
      return $this->_populateFromRows($resultSet, $deepPopulate, $options);
    }

	abstract protected function _populateFromRow($row, $deepPopulate = true, $params = array());

	protected function _populateFromRows($set, $deepPopulate = true, $params = array()) {
        $entries   = array();
        foreach ($set as $row) {
            $entries[] = $this->_populateFromRow($row, $deepPopulate, $params);
        }		
        return $entries;
	}
	
	public function delete($entity)
	{
		if (!($entity instanceof WebReport_Model_Abstract)) {
			throw new WebReport_Model_Mapper_Exception("Parameter must be instance of WebReport_Model_Abstract");
		}
     $result = $this->getDbTable()->find($entity->getId());

     if (0 == count($result)) {
          return false;
     }
        
     $row = $result->current();
     $row->delete();
     return true;
	}
	
}
