<?php

/**
 * Validator for a .cnt file.
 *
 * @category  WebReport
 * @package   WebReport_Validate
  */
class WebReport_Validate_File_CntFile extends Zend_Validate_Abstract
{
    /**
     * @const string Error constants
     */
    const NOT_VALID_CNT_FILE    = 'notValidCntFile';

    /**
     * @var array Error message template
     */
    protected $_messageTemplates = array(
        self::NOT_VALID_CNT_FILE    => "This is not a valid .cnt file"
    );

    /**
     * @var array Error message template variables
     */
    protected $_messageVariables = array(
        'notValidCntFile'  => '_notValidCntFile'
    );

    protected $_notValidCntFile;


    /**
     * Defined by Zend_Validate_Interface
     *
     * Returns true if and only if  $value is a valid cnt file
     *
     * @param  string $value Real cnt file to check
     * @param  array  $file  File data from Zend_File_Transfer
     * @return boolean
     */
    public function isValid($value, $file = null)
    {
        // Is file readable ?
        #require_once 'Zend/Loader.php';
        if (!Zend_Loader::isReadable($value)) {
            return $this->_throw($file, self::NOT_READABLE);
        }

		$cnt = new WebReport_FileFormats_Cnt($value,
			array(
				WebReport_FileFormats::VERBOSE => false,
				WebReport_FileFormats::LOGGER => Zend_Registry::get('Zend_Logger_FireBug'),
				WebReport_FileFormats::ONLY_VALIDATE => true
			));
		
		if (!$cnt->isValid()) {
			$this->_throw($file, self::NOT_VALID_CNT_FILE);
		}
		
        if (count($this->_messages) > 0) {
            return false;
        }

        return true;
    }

    /**
     * Throws an error of the given type
     *
     * @param  string $file
     * @param  string $errorType
     * @return false
     */
    protected function _throw($file, $errorType)
    {
        if ($file !== null) {
            $this->_value = $file['name'];
        }

        $this->_error($errorType);
        return false;
    }
}
