<?php

class WebReport_View_Helper_DataTables extends Zend_View_Helper_Abstract
{

    public $view;

    public function dataTables($tableSelector = "<table>", $options = array(), $dataTablesPath = false)
    {
        if (!$dataTablesPath) {
            $dataTablesPath = $this->view->baseUrl('/library/javascript/jquery/plugins/DataTables-1.10.10/');
        }

        $this->view->headScript()->appendFile($dataTablesPath . "datatables.min.js");
        $this->view->headLink()->appendStylesheet($dataTablesPath . "datatables.min.css");

        $defaults = array(
            "bAutoWidth" => false,
            "bStateSave" => true,
            "oLanguage" =>
            array(
                "sProcessing" => $this->view->translate("DataTables: sProcessing"),
                "sLengthMenu" => $this->view->translate("DataTables: sLengthmenu"),
                "sZeroRecords" => $this->view->translate("DataTables: sZeroRecords"),
                "sInfo" => $this->view->translate("DataTables: sInfo"),
                "sInfoEmpty" => $this->view->translate("DataTables: sInfoEmpty"),
                "sInfoFiltered" => $this->view->translate("DataTables: sInfoFiltered"),
                "sSearch" => $this->view->translate("DataTables: sSearch") . ":",
                "oPaginate" =>
                array(
                    "sFirst" => $this->view->translate("DataTables: sFirst"),
                    "sPrevious" => $this->view->translate("DataTables: sPrevious"),
                    "sNext" => $this->view->translate("DataTables: sNext"),
                    "sLast" => $this->view->translate("DataTables: sLast"),
                )
            ),
            "buttons" => array("copy","excel","pdf")
        );

        $options = Talking_Toolbox_Array::mergeRecursiveDistinct($defaults, $options);

        $this->view->headScript()->captureStart();
        ?>
$(function() {
    var opts = <?= Zend_Json::encode($options); ?>;
    var $t = $("<?= $tableSelector; ?>");
    $t.each(function(i, val) {
        //Init options for dataTables
        var o = jQuery.extend(true, {}, opts),
        $$ = $(this),
        $ths = $$.find("thead th"),
        n = $ths.size(),
        p = o.aoColumns.length;

        if (o.aoColumns && p < n) {
            for (var i = p; i < n; i++) {
                o.aoColumns.push(null);
            }
        }
        $table = $$.DataTable(o);
        $$.prev().prepend($("<span />", { "class" : "icon" }).append('<i class="fa fa-search"></i>'));
        $table.buttons().container().insertBefore($$.parent().find(".dataTables_filter"));
        $copy = $$.parent().find(".buttons-copy");
        $copy.children("span").html("").append('<i class="fa fa-clipboard"></i>').attr("title","<?=$this->view->translate('COPY_TO_CLIPBOARD')?>");
        $export_excel = $$.parent().find(".buttons-excel");
        $export_excel.children("span").html("").append('<i class="fa fa-file-excel-o"></i>').attr("title","<?=$this->view->translate('EXPORT_TO_XLSX')?>");
        $export_pdf = $$.parent().find(".buttons-pdf");
        $export_pdf.children("span").html("").append('<i class="fa fa-file-pdf-o"></i>').attr("title","<?=$this->view->translate('EXPORT_TO_PDF')?>");
    });
});

function translateTableSort(e) {
    if ($(this).attr('aria-sort') == 'ascending') {
        title = '<?=$this->view->translate('SORT_BY_xxCOLUMNxx_DESCENDING')?>';
        title = title.replace("xxCOLUMNxx", $(this).text());
        $(this).attr('title', title);
    } else {
        title = '<?=$this->view->translate('SORT_BY_xxCOLUMNxx_ASCENDING')?>';
        title = title.replace("xxCOLUMNxx", $(this).text());
        $(this).attr('title', title);
    }
}

$(function() {
    $('.list-table').each(function() {
        $(this).find('[aria-label]').not("th:first")
        .each(translateTableSort)
        .each(function(index) {
            $(this).click(translateTableSort)
        });
        $(this).find("th:first").each(function() {
            $(this).css({'background-image':'none','cursor':'default'});
        });
    });
});
<?php
$this->view->headScript()->captureEnd();
}




    public function setView(Zend_View_Interface $view)
    {
        $this->view = $view;
    }

}
