<?php

class WebReport_View_Helper_Tutorial extends Zend_View_Helper_Abstract
{

    public $view;

    public function setView(Zend_View_Interface $view)
    {
        $this->view = $view;
    }

    public function tutorial($is_multi)
    {
        $bootstrap_tour = $this->view->baseUrl('/library/');

        $this->view->headScript()->appendFile($bootstrap_tour . "javascript/bootstrap-tour-standalone.js");
        $this->view->headLink()->appendStylesheet($bootstrap_tour . "css/bootstrap-tour-standalone.css");

        $this->view->headScript()->captureStart();
        ?>
        $(function(){
            var tour_steps = [
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: "",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_WELCOME_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_WELCOME_DESCRIPTION")?>",
                orphan: true,
                backdrop: true,
            },
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: "#header",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_HEADER_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_HEADER_DESCRIPTION")?>",
                placement: "bottom",
                onNext:function() {
                    showTableIfHidden();
                },
            },
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: ".dataTable tr:first",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_DESCRIPTION")?>",
                placement: "top",
                reflex:true,
            },
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: ".dataTables_filter:first",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_FILTERS_EXPORT_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_FILTERS_EXPORT_DESCRIPTION")?>",
                placement: "left",
                onNext: function() {
                    clearTableFilter();
                },
            },
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: ".dataTable:first td:first",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_ACTIONS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_ACTIONS_DESCRIPTION")?>",
                placement: "right",
            },
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: ".show-hide-icon:first",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_SHOWHIDE_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_SHOWHIDE_DESCRIPTION")?>",
                placement: "right",
                onNext: function() {
                    hideTableIfShown();
                },
                onPrev: function() {
                    showTableIfHidden();
                    clearTableFilter();
                },
            },
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: "#show-all",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_SHOWALL_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_DATATABLE_SHOWALL_DESCRIPTION")?>",
                placement: "right",
                reflex: true,
                onNext: function() {
                    $("#show-all").click();
                },
                onPrev: function() {
                    showTableIfHidden();
                },
            },
            {
                path:"<?= $this->view->baseUrl('overview')?>",
                element: "#sources",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_SOURCES_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_SOURCES_DESCRIPTION")?>",
                placement: "bottom",
                reflex: true,
            },
            {
                path:"<?= $this->view->baseUrl('sources/index/index/folder/0/')?>",
                element: "#submenu li:first",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_BROWSE_SOURCES_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_BROWSE_SOURCES_DESCRIPTION")?>",
                placement: "bottom",
                reflex: true,
            },
            {
                path:"<?= $this->view->baseUrl('sources/index/index/folder/0/')?>",
                element: ".directory-listing .folder-child:first",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_FOLDERS_LISTING_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_FOLDERS_LISTING_DESCRIPTION")?>",
                placement: "right",
                orphan: true,
                onNext: function() {
                    var href = $(".directory-listing .folder-child:first").parent().attr("href"),
                        first_folder_id = href.substr(-1);
                    document.location.href = "<?= $this->view->baseUrl('sources/index/index/folder/')?>" + first_folder_id;
                },
                reflex: true,
            },
            {
                element: ".lists-cell",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_FOLDER_CONTENTS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_FOLDER_CONTENTS_DESCRIPTION")?>",
                placement: "top",
                orphan: true,
                onShown: function() {
                    if (!isOnPage('sources','index','index')) {
                        goBackOrReset();
                    }
                },
            },
            {
                element: ".options-list :nth-child(2)",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_DIRECTORY_LIST_OPTIONS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_DIRECTORY_LIST_OPTIONS_DESCRIPTION")?>",
                placement: "right",
                reflex: true,
                orphan: true,
                onShown: function() {
                    if (isOnPage('sources','folders','browse')) {
                        tour.next();
                    } else if (!isOnPage('sources','index','index')) {
                        goBackOrReset();
                    }
                },
                onNext: function() {
                    var href = $(".directory-listing .folder-current:first").parent().attr("href");
                    if(typeof(href) !== 'undefined') {
                        first_folder_id = href.substr(-1);
                        document.location.href = "<?= $this->view->baseUrl('sources/folders/browse/')?>" + + first_folder_id;
                    }
                }
            },
            {
                element: "#submenu li:last",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_EDIT_FOLDERS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_EDIT_FOLDERS_DESCRIPTION")?>",
                placement: "right",
                onShown: function() {
                    if (isOnPage('sources','index','index')) {
                        tour.prev();
                    } else if (!isOnPage('sources','folders','browse')) {
                        goBackOrReset();
                    }
                },
                onPrev: function() {
                    var href = $(".directory-listing .folder-current:first").parent().attr("href");
                    if(typeof(href) !== 'undefined') {
                        first_folder_id = href.substr(-1);
                        document.location.href = "<?= $this->view->baseUrl('sources/index/index/folder/')?>" + first_folder_id;
                    }
                }
            },
            {
                element: "#folder-view-list",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_VIEW_EDIT_FOLDERS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_VIEW_EDIT_FOLDERS_DESCRIPTION")?>",
                placement: "right",
                orphan: true,
                onShown: function() {
                    if (!isOnPage('sources','folders','browse')) {
                        goBackOrReset();
                    }
                },
            },
            {
                element: "#sets",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_SETS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_SETS_DESCRIPTION")?>",
                placement: "bottom",
                reflex: true,
            },
            {
                path: "<?= $this->view->baseUrl('sets')?>",
                element: "#exports",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_EXPORTS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_EXPORTS_DESCRIPTION")?>",
                placement: "bottom",
                reflex: true,
            },
            {
                path: "<?= $this->view->baseUrl('export')?>",
                element: "#user-admin-menu",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_ADMIN_MENU_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_ADMIN_MENU_DESCRIPTION")?>",
                placement: "left",
                orphan: true,
            },
            {
                element: "#top-right-dropdown",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_ADMIN_MENU_CONTENTS_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_ADMIN_MENU_CONTENTS_DESCRIPTION")?>",
                placement: "left",
                orphan: true,
                onShow: function() {
                    setTimeout(function(){$("#user-admin-menu a").jqDropdown('show')},0);
                },
                onPrev: function() { $('#user-admin-menu').jqDropdown('show', false)}
            },
            {
                element: "#replay-introduction",
                title: "<?= $this->view->translate("INTRO_TUTORIAL_FINAL_TITLE")?>",
                content: "<?= $this->view->translate("INTRO_TUTORIAL_FINAL_DESCRIPTION")?>",
                placement: "left",
                reflex: true,
                orphan: true,
                onShow: function() {setTimeout(function(){$("#user-admin-menu a").jqDropdown('show')},0);},
            },
            ];
            function clearTableFilter() {
                $searchbox = $(".dataTables_filter:first").find("input");
                table = $searchbox.parent().parent().next().dataTable();
                table.fnFilter('');
                $searchbox.val('');
            }
            function showTableIfHidden() {
                $element = $(".show-hide-icon:first");
                if ($element.hasClass("fa-chevron-circle-down")) {
                    $(".show-hide-icon:first").click();
                }
            }
            function hideTableIfShown() {
                $element = $(".show-hide-icon:first");
                if ($element.hasClass("fa-chevron-circle-up")) {
                    $(".show-hide-icon:first").click();
                }
            }
            /*
             * These can be used if we want to add an tutorial for adding a first camera.
             *{
                // path: <?= $this->view->baseUrl('sources/folders/edit-cc-settings/')?>
                element: "#conn_enabled",
                    title: "<?= $this->view->translate("INTRO_TUTORIAL_CLICK_ENABLE_CC_TITLE")?>",
                    content: "<?= $this->view->translate("INTRO_TUTORIAL_CLICK_ENABLE_CC_DESCRIPTION")?>",
                    placement: "right",
                    orphan: true,
                    reflex: true,
            },
            {
                element: "#username",
                    title: "<?= $this->view->translate("INTRO_TUTORIAL_FILL_CC_SETTINGS_TITLE")?>",
                    content: "<?= $this->view->translate("INTRO_TUTORIAL_FILL_CC_SETTINGS_DESCRIPTION")?>",
                    placement: "right",
                    onHide: function(tour){
                        validateCCSettings(tour);
                    },
            },
            {
                element: "#password",
                    title: "<?= $this->view->translate("INTRO_TUTORIAL_FILL_CC_SETTINGS_TITLE")?>",
                    content: "<?= $this->view->translate("INTRO_TUTORIAL_FILL_CC_SETTINGS_DESCRIPTION")?>",
                    placement: "right",
                    onHide: function(tour){
                        validateCCSettings(tour);
                    },
                    onShown: function(tour){
                        checkPreviousStepValid(tour);
                    },
            },
            {
                element: "#password_confirm",
                    title: "<?= $this->view->translate("INTRO_TUTORIAL_FILL_CC_SETTINGS_TITLE")?>",
                    content: "<?= $this->view->translate("INTRO_TUTORIAL_FILL_CC_SETTINGS_DESCRIPTION")?>",
                    placement: "right",
                    onHide: function(tour){
                        validateCCSettings(tour);
                    },
                    onShown: function(tour){
                        checkPreviousStepValid(tour);
                    },
            },
            {
                element: "#primary",
                    title: "<?= $this->view->translate("INTRO_TUTORIAL_SUBMIT_CC_SETTINGS_TITLE")?>",
                    content: "<?= $this->view->translate("INTRO_TUTORIAL_SUBMIT_CC_SETTINGS_DESCRIPTION")?>",
                    placement: "left",
                    onShown: function(tour){
                        checkPreviousStepValid(tour);
                    },
                    reflex: true,
            },
             *
             * These are helpfunctions needed for validation of formfields during tutorial. Used in commented code above.
             *
            var invalidStep = -1;
                redirectInProgress = false;
            function validateCCSettings(tour) {
                if (!redirectInProgress) {
                    var step = tour._options.steps[tour.getCurrentStep()];
                    var $attachedEl = typeof inputSelector == 'undefined' ? $(step.element) : $(inputSelector);
                    if ($attachedEl.length > 0 && $attachedEl.is('input')) {
                        if ($attachedEl.hasClass('textInput')) {
                            console.log("CHECKING VALIDATION!");
                            var val = $attachedEl.val();
                            console.log(val);
                            if (val.length == 0) {
                                invalidStep = tour.getCurrentStep();
                                createError($attachedEl, "Value is required and can't be empty.");
                            } else {
                                if ($attachedEl.attr('id') == 'password_confirm') {
                                    if($("#password").val() != $("#password_confirm").val()) {
                                        invalidStep = tour.getCurrentStep();
                                        createError($attachedEl, "Password and password confirmation do not match.");
                                    }
                                }
                                removeError($attachedEl);
                            }
                        }
                    }

                    return invalidStep == -1;
                }
                redirectInProgress = false;
            }
            function checkPreviousStepValid(tour) {
                // .goTo only seems to work in the onShown step event so I had to put this check
                // on the next step's onShown event in order to redisplay the previous step with
                // the error
                console.log(invalidStep);
                if (invalidStep > -1) {
                    var tempStep = invalidStep;
                    redirectInProgress = true;
                    invalidStep = -1;
                    tour.goTo(tempStep);
                }
            }
            function createError($element, message) {
                $ctrlHolder = $element.parent(".ctrlHolder");
                $prevError = $ctrlHolder.find(".errorField");
                if ($prevError.length==0) {
                    $ctrlHolder.prepend($('<p/>').addClass("errorField")
                        .append($('<strong/>')
                        .html(message)
                        )
                    )
                }
            }
            function removeError($element) {
                $ctrlHolder = $element.parent(".ctrlHolder");
                $prevError = $ctrlHolder.find(".errorField");
                if($prevError.length!=0) {
                    $ctrlHolder.find(".errorField").remove();
                }
            }
            * End helpfunctions
            */

            function isOnPage(module,controller,action) {
                <?php $request_params = Zend_Controller_Front::getInstance()->getRequest()->getParams(); ?>
                request_module = "<?= $request_params['module'] ?>";
                request_controller = "<?= $request_params['controller'] ?>";
                request_action = "<?= $request_params['action'] ?>";
                if (request_module == module && request_controller == controller && request_action == action) {
                    return true;
                }
                return false;
            }
            function goBackOrReset() {
                referrer = document.referrer;
                if(referrer.indexOf("datamanager") > -1) {
                    history.back();
                } else {
                    tour.goTo(0);
                }
            }


            $.each(tour_steps, function(i, step){
                step['title'] = '<div class="title-text">' + step['title'] + '</div>';
                step['title'] += '<span class="step-numbering">'+(i+1)+'/'+tour_steps.length+'</span>';
            });

            var tour = new Tour({
                steps: tour_steps,
                keyboard: false,
                storage: window.localStorage,
                debug: false, //Set to true to get some console output for debugging.
                template: "<div class='popover tour'> \
                <div class='arrow'></div> \
                <h3 class='popover-title'></h3> \
                <div class='popover-content'></div> \
                    <div class='popover-navigation'> \
                    <button class='awesome' data-role='end'><?= $this->view->translate("END_TOUR")?></button> \
                    <div class='btn-group'> \
                        <button class='btn awesome' data-role='prev'>« <?= $this->view->translate("PREV")?></button> \
                        <button class='btn awesome' data-role='next'><?= $this->view->translate("NEXT")?> »</button> \
                    </div> \
                </div> \
                </div>"
            });

            tour.init();
            tour.start()

            //Add replay introduction item to user-admin-dropdown.
            $("#dropdown-help-header").parent().after($('<li/>').
                append($('<a/>').
                    attr('id','replay-introduction').
                    attr('href','#').
                    text("<?= $this->view->translate("REPLAY_INTRODUCTION")?>").
                    attr('title', "<?= $this->view->translate("REPLAY_INTRODUCTION_TOOLTIP")?>").
                    click(function() {
                        tour.restart();
                    })
                    )
                );

        });



        <?php
        $this->view->headScript()->captureEnd();
    }

}
?>
