(function ($) {

    // In retrospect I think UI widget design pattern is more suitble.
    // This design pattern is good but is missing calling sibling functions ie this.update()
    // all functions also require return this.each() and data strapping that violates DRY principle. --eriksod

    var methods = {
        "init" : function (settings) {
            var settings = $.extend(true, $.fn.webreportAmChart.defaults, settings);
            return this.each(function () {
                var $$ = $(this),
                    data = $$.data("webreport-amcharts");
                if(data) {
                    return;
                }
                if(!data) {
                    data = {
                        settings : settings,
                        graphs : {}
                    };
                    $$.data("webreport-amcharts", data);
                }

                data.amchart = new AmCharts.AmSerialChart();
                data.legend = new AmCharts.AmLegend();
                data.legend.valueText ="";
                data.amchart.balloon.cornerRadius = 6;
                data.amchart.balloon.borderThickness = 0;
                data.amchart.balloon.fillAlpha = 0.85;
                data.amchart.balloon.verticalPadding = 5;

                data.amchart.addLegend(data.legend);
                //apply the defaults to the amchart
                for(var key in data.settings.amChartDefaults) {
                    data.amchart[key] = data.settings.amChartDefaults[key];
                }
                data.formatNumber = AmCharts.formatNumber;

                data.amchart.write($$.attr('id'));
                if(data.settings.data) $$.webreportAmChart('update', data.settings.data);
            });
        },
        "begin-update": function() {
            return this;
        },
        "update" : function(chartData) {
            return this.each(function () {
                var $$ = $(this),
                    data = $$.data("webreport-amcharts");
                if(!data)
                    $.error( 'jQuery.webreportAmChart not initialized\n' );

                for(var graph in data.graphs) {
                    data.amchart.removeGraph(data.graphs[graph]);
                    delete data.graphs[graph];
                }

                data.amchart.valueAxes = [];
                var myAxes = {};
                for(var axes in chartData.valueAxes) {
                    var axis = new AmCharts.ValueAxis();
                    for(var setting in chartData.valueAxes[axes]){
                       axis[setting] = chartData.valueAxes[axes][setting];
                    }
                    data.amchart.addValueAxis(axis);
                    myAxes[axes] = axis;
                }

                data.amchart.dataProvider = chartData.data;
                for(var graph in chartData.graphs) {
                    if(data.graphs[graph] === undefined) {
                        data.graphs[graph] = new AmCharts.AmGraph();
                        var limitLabels = chartData.graphs.length * data.settings.graphDefaults.length;

                        for(var key in data.settings.graphDefaults) {
                            data.graphs[graph][key] = data.settings.graphDefaults[key];
                        }
                        data.amchart.addGraph(data.graphs[graph]);
                    }
                    for(var key in chartData.graphs[graph]) {
                        data.graphs[graph][key] = chartData.graphs[graph][key];
                    }

                    data.graphs[graph]['valueAxis'] = myAxes[chartData.graphs[graph].valueAxis]
                    if(chartData.customSettings['hideNumbers'])  data.graphs[graph].labelText = "";
                    data.graphs[graph].bulletSize = (2000 / ( 250 + chartData.data.length));
                }

                for(var setting in chartData.settings) {
                       data.amchart[setting] = chartData.settings[setting];
                }

                for(var setting in chartData.categoryAxis) {
                       data.amchart.categoryAxis[setting] = chartData.categoryAxis[setting];
                }

                data.amchart.numberFormatter.thousandsSeparator = chartData.customSettings['thousandSeparator'];
                if(chartData.customSettings['formatNumbers']){
                    AmCharts.formatNumber = function(a,b,c,d,e){
                        if(a > 99999) return (parseInt(a/1000,10)).toString()  + "k";
                        else if(a > 999) return (parseInt(a/100,10)/10.0).toString().substring(0,4)  + "k";
                        return a;
                    }
                } else {
                    AmCharts.formatNumber = data.formatNumber;
                }

                if(chartData.customSettings['chartScroll']){
                    var scroll = new AmCharts.ChartScrollbar();
                    data.amchart.addChartScrollbar(scroll);
                }
                if(chartData.customSettings['chartCursor']){
                    var cursor = new AmCharts.ChartCursor();
                    for(var v in chartData.customSettings['chartCursor']){
                        cursor[v] = chartData.customSettings['chartCursor'][v];
                    }
                    data.amchart.addChartCursor(cursor);
                }
                /*
                $$.mouseenter(function(){
                    for(var graph in chartData.graphs){
                        data.graphs[graph].labelText = chartData.customSettings['hideNumbers'] ? "" : "";
                    }
                    data.amchart.validateData();
                    data.amchart.validateNow();
                });
                $$.mouseleave(function(){
                    for(var graph in chartData.graphs){
                        data.graphs[graph].labelText = chartData.customSettings['hideNumbers'] ? "" : "[[title]]";
                    }
                    data.amchart.validateData();
                    data.amchart.validateNow();
                });
                */

                if(chartData.hasError && window.console)
                    window.console.log("Error is " + chartData.error);

                data.amchart.validateData();

                data.amchart.validateNow();

            });
        },
        "destroy" : function() {
             return this.each(function () {
                var $$ = $(this),
                    data = $$.data("webreport-amcharts");
                if(!data)
                    $.error( 'jQuery.webreportAmChart not initialized\n' );

                data.chart.destroy();
                for(graph in data.graphs) {
                    delete data.graphs[graph];
                }
                $("#" + data.amchartDiv).remove();
                $$.data("webreport-amcharts",undefined);
            });
        },
        "resize" : function(options) {
            return this.each(function () {
                var $$ = $(this),
                    data = $$.data("webreport-amcharts");
                if(!data)
                    $.error( 'jQuery.webreportAmChart not initialized\n' );
                if(options) {
                    if(options.height)
                        $$.height(options.height);
                    if(options.width)
                        $$.width(options.width);
                }
                data.amchart.invalidateSize();
            });
        }
        /*"add-resize" : function(height) {
            return this.each(function () {
                var $$ = $(this),
                    data = $$.data("webreport-amcharts");
                if(!data)
                    $.error( 'jQuery.webreportAmChart not initialized\n' );
                if(data.
            });
        },*/
        /*
        "method-skelleton" : function() {
            return this.each(function () {
                var $$ = $(this),
                    data = $$.data("webreport-amcharts");
                if(!data)
                    $.error( 'jQuery.webreportAmChart not initialized\n' );

            });
        },
        */
    }

    $.fn.webreportAmChart = function( method ) {
        if ( methods[method] ) {
            return methods[method].apply( this, Array.prototype.slice.call( arguments, 1 ));
        } else if ( typeof method === 'object' || ! method ) {
            return methods.init.apply( this, arguments );
        } else {
            $.error( 'Method ' +  method + ' does not exist on jQuery.webreportAmChart' );
        }
    }

     $.fn.webreportAmChart.defaults =  {
        data: undefined,
        amChartDefaults: {
            categoryField : "ts",
            marginLeft : 47,
            marginTop : 30
        },
        graphDefaults: {
            balloonText : "[[category]]: [[description]]",
            title : "unset",
            labelText : "[[value]]",
            lineAlpha : 1,
            lineThickness : 1,
            fillAlphas : 0,
            type : "column"
        },
        customButtonAttr: {
            "font-family": "verdana",
            "font-size": "14px"
        }
     }

     var fakeuuid = function () {
        this.old_keys = {'start': 'start'},
            ret = 'start';
        while(ret in this.old_keys) {
            //http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript
            ret = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
                var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
                    return v.toString(16);
                });
        }
        this.old_keys[ret] = ret;
        return ret;
    };

 })(jQuery);
