$(function(){
(function($, undefined) {

function zeroPad(n, digits) {
    n = n.toString();
    while (n.length < digits) {
        n = '0' + n;
    }
    return n;
}
$.heatmapsBrowser = function(options)
{
    options = $.extend(true, {}, {
        heatmapsUrl : '/tjenis',
        maskListUrl : '/kempeTorsk',
        imagesBaseUrl : '/guleboj',
        showError : false,
        "strings" : {
            "onlyFourHeatmaps" : "Only four heatmaps can be shown at the same time."
        }
    }, options);

    var strings = options.strings,
        lameBrowser = $.browser.msie && (parseInt($.browser.version.substr(0, $.browser.version.indexOf(".")),10) <= 8); // IE <= v8

    var slideAction = "slide";
    if (lameBrowser) slideAction = "stop";

    var colors = [
        [ 0, 0,0],  [  0, 90,255], [  0,162,255], [  0,234,255],
        [  0,255,234], [  0,255,162], [  0,255, 90], [  0,255, 18],
        [ 18,255,  0], [ 90,255,  0], [162,255,  0], [234,255,  0],
        [255,234,  0], [255,162,  0], [255, 90,  0], [255, 18,  0], [255, 0,0]
    ];

    var cols = [],
        i = 0,
        colorSteps = colors;

    for (i = 0; i < 16; i++) {
        cols.push([0, 0, 0]);
    }
    for (i = 16; i < 256; i++) {
        var m = i % 16;
        var ind = Math.floor(i / 16),
            from = colorSteps[ind],
            to = colorSteps[ind+1];
        var newCol = [];
        for (var j = 0; j < 3; j++) {
            newCol.push(Math.round((to[j]*m + from[j]*(16-m))/16));
        }
        cols.push(newCol);
    }

    var minDate = "3077-12-12",
        maxDate = "1832-01-01";
    var $camerasWrapper = $("#heatmap-cameras"),
        $cameras = $camerasWrapper.find("input[type=checkbox]").each(function() {
        var $$ = $(this);
        var min = $$.attr("data-first-date").split(" "),
            max = $$.attr("data-last-date").split(" ");
        if (min[0] < minDate) minDate = min[0];
        if (max[0] > maxDate) maxDate = max[0];
    });

    minDate = new Date(parseInt(minDate.substr(0,4), 10), parseInt(minDate.substr(5,2), 10)-1, parseInt(minDate.substr(8,2), 10));
    maxDate = new Date(parseInt(maxDate.substr(0,4), 10), parseInt(maxDate.substr(5,2), 10)-1, parseInt(maxDate.substr(8,2), 10));

    var $time = $("#time-text"),
        $day = $("#day-text");

    var mapsMap = {};

    var $maps = $("#heatmaps .heatmap").each(function() {
        var $$ = $(this);
        $$.css("position", "relative");
        mapsMap[$$.attr("data-id")] = $$;
    });

    var maps = {};
    var images = {};
    var masks = {};
    var manual_mask = {};

    var stretch = false;

    var $timeSlider = $("#time-slider");
    var $timeSliderInterval = $("#time-slider-interval").hide();

    var $timeResolution = $("#time-resolution");

    $("#stretch").bind('click', function(e) {
        stretch = $(this).attr("checked");
        var fakeUi = {value : $timeSlider.slider('value'), values : $timeSliderInterval.slider('values')}
        $timeSlider.slider('option', slideAction).call($timeSlider, false, fakeUi);
    });

    var dayValue = -1;

    var runningAjaxRequests = {};

    var cancelAllRequests = function()
    {
        for (var id in runningAjaxRequests) {
            runningAjaxRequests[id].abort();
        }
    }

    var latestLoaded = {};

    var loadMap = function(id, day)
    {
        runningAjaxRequests[id] = $.ajax({
            url : options.heatmapsUrl,
            dataType : 'json',
            data : {
                downscaled : 1,
                date : day,
                id : id,
                isInterval : $timeResolution.val() == "0",
                interval : $timeSliderInterval.slider('option', 'values'),
                delta : $timeResolution.val()
            },
            success : function(data) {
                $timeSlider.slider('enable');
                maps[data.id] = {
                    "data" : data.heatmaps,
                    "downscaled" : true,
                    "wasInterval" : data.wasInterval
                };
                images[data.id] = {times : data.images.times, filenames : data.images.filenames};

                var fakeUi = {value : $timeSlider.slider('value'), values : $timeSliderInterval.slider('values')}
                $timeSlider.slider('option', slideAction).call($timeSlider, false, fakeUi);

                if (!lameBrowser) { // it goes crapilicous slow for ie to draw with vml so just make it only show the downscaled version
                    runningAjaxRequests[id] = $.ajax({
                        url : options.heatmapsUrl,
                        dataType : 'json',
                        data : {
                            downscaled : 0,
                            date : day,
                            id : id,
                            isInterval : $timeResolution.val() == "0",
                            interval : $timeSliderInterval.slider('option', 'values'),
                            delta : $timeResolution.val()
                        },
                        success : function(data) {
                            if (data == null || !$.isPlainObject(data)) return;
                            delete runningAjaxRequests[id];
                            maps[data.id] = {
                                "data" : data.heatmaps,
                                "downscaled" : false,
                                "wasInterval" : data.wasInterval
                            };

                            var fakeUi = {value : $timeSlider.slider('value'), values : $timeSliderInterval.slider('values')}
                            $timeSlider.slider('option', slideAction).call($timeSlider, false,fakeUi);
                        },
                        error : function(x,e) {
                            // @todo: nicer error
                            if(e != "abort" && showError)
                                alert('An error occured. Might be a good to try to logout and login. (1, ' + e + ')');
                        }
                    });
                }
            },
            error : function(x,e) {
                // @todo: nicer error
                if(e != "abort" && showError)
                    alert('An error occured. Might be a good to try to logout and login. (2, ' + e + ')');
            }
        });
    }


    var $datepicker = $("#day-picker").datepicker({
        minDate : minDate,
        maxDate : maxDate,
        onSelect: function(dateText, inst) {
            cancelAllRequests();
            var day =  inst.selectedYear + zeroPad(inst.selectedMonth+1,2) + zeroPad(inst.selectedDay,2);
            if (day == dayValue) return;
            $day.text(inst.selectedYear + "-" + zeroPad(inst.selectedMonth+1,2) + "-" + zeroPad(inst.selectedDay,2));
            dayValue = day;
            $timeSlider.slider('disable');

            $camerasWrapper.find("input:checkbox").each(function() {
                var $$ = $(this);
                if ($$.is(":checked")) {
                    var id = $(this).attr("data-id");
                    loadMap(id, day);
                }else{
                    $$.trigger('change');
                }
            });
        }
    });


    var curDay = new Date(),
        prevTimeRes = $timeResolution.val(),
        secsToday = curDay.getHours() * 60 * 60 + curDay.getMinutes() * 60;

    var sliderOpts ={
        value : Math.max(Math.floor(secsToday/prevTimeRes), 0),
        min : 1,
        max : (60*60*24)/prevTimeRes
    };

    sliderOpts['stop'] = function(e,ui) {
        if($timeResolution.val() != 0) return;
        $timeSlider.slider('option', 'slide').call(this,e,ui, 'stop');
    }
    sliderOpts[ slideAction ] = function(e, ui, action) {
            var start = new Date().getTime();
            var delta = $timeResolution.val();

            if(delta != 0){
                var time = new Date(ui.value * delta * 1000),
                    from = new Date((ui.value-1) * delta * 1000);
            } else {
                var val = $timeSliderInterval.slider('option', 'values');
                if(ui.values && ui.values.length == 2) val = ui.values; // Use real ui if ranged slider.
                var time = new Date(val[1] * 60 * 60 * 1000),
                from = new Date(val[0] * 60 * 60 * 1000);
            }
            time = zeroPad(time.getUTCHours(),2) + ":" + zeroPad(time.getUTCMinutes(),2);
            from = zeroPad(from.getUTCHours(),2) + ":" + zeroPad(from.getUTCMinutes(),2);

            $time.text(from + "-" + ((time == "00:00") ? "24:00" : time) );

            if(delta == 0) {
                if(action == 'stop'){
                    $camerasWrapper.find("input:checkbox").each(function() {
                        var $$ = $(this);
                        if ($$.is(":checked")) {
                            var id = $(this).attr("data-id");
                            loadMap(id,dayValue);
                        }
                    });
                    return;
                }
            }

            for (var id in mapsMap) {
                var $p = mapsMap[id].find(".canvas"),
                    $img = mapsMap[id].find("img").eq(0);
                gc = $p[0].getContext;
                if(!gc){
                    alert("An error occured. This is probably caused by using an old browser. We recommend Chrome, Firefox or IE9.");
                } else {
                    ctx = $p[0].getContext('2d');
                }


                $p[0].width = 320;
                $p[0].height = 240;
                ctx.fillStyle = "rgba(0,0,0,0)";
                ctx.clearRect(0,0,320,240);
                ctx.fillRect(0,0,320,240);
                var map = maps[id],
                    ims = images[id],
                    mask = masks[id] ? maskOps.decompress(masks[id]) : undefined;
                if (typeof map == "undefined") {
                    map = {data : {}};
                    ims = {times : []};
                }

                var imgsrc = "",
                    imlen = ims.times.length;

                if (imlen == 0) {
                } else if (imlen == 1) {
                    imgsrc = ims['filenames'][0];
                } else {
                    var times = ims['times'];
                    if (time < times[0]) {
                        imgsrc = ims['filenames'][0];
                    } else if (time > times[imlen-1]) {
                        imgsrc = ims['filenames'][imlen-1];
                    } else {
                        for (var i = 0; i < imlen-1; i++) {
                            if (time >= times[i] && time <= times[i+1]) {
                                imgsrc = ims['filenames'][i];
                                break;
                            }
                        }
                    }
                }

                if (imgsrc != "") {
                    $img.show().attr("src", options.imagesBaseUrl + "/" + imgsrc);
                } else {
                    $img.hide();
                }

                var wasInterval = map['wasInterval'];
                if (!wasInterval && typeof map['data'][time] == "undefined" ) {
                    // fill black
                    ctx.fillStyle = "rgba(0,0,0,0.2)";
                    ctx.fillRect(0,0,320,240);
                } else {

                    var downscaled = map['downscaled'];
                        ss = downscaled ? 4:1,
                        vals = null,
                        max = 1;
                    if(wasInterval && time == "00:00") time = "24:00";
                    if(map['data'][time]) {
                        vals = map['data'][time]['data'];
                        max = map['data'][time]['max'];

                        if( map['data'][time]['mask'] && !manual_mask[id]) {
                            if(!map['data'][time]['mask_uncompressed'])
                                map['data'][time]['mask_uncompressed'] = maskOps.decompress(map['data'][time]['mask']);
                            mask = map['data'][time]['mask_uncompressed'];
                        }
                    }

                    if(vals == null && wasInterval) {
                        if(action == 'stop' &&  delta == 0) loadMap(id,dayValue);
                        continue;
                    }

                    var s = 4;

                    if (max == 0) max = 1; // not divide by zero

                    if (downscaled) {
                        s = 16;
                    }
                    var w = 320/s,
                        h = 240/s;

                    if(mask) {
                        max = 0;
                        for (var y=0; y<h;y++) {
                            for(var x=0; x<w; x++) {
                                var g = parseInt(vals.substr((y*w+x)*2,2),16);

                                if(mask[(y*w*ss+x)*ss] == 0) {
                                    continue;
                                }
                                if(g > max) max = g;
                            }
                        }
                    }

                    for (var y = 0; y < h; y++) {
                        for (var x = 0; x < w; x++) {
                            var g,
                                alpha = 0.5;

                            if(mask && !mask[(ss*(y*w*ss+x))]) {
                                g = 0 /*255 debug*/;
                            } else
                                g = parseInt(vals.substr((y*w+x)*2,2),16);

                            if (stretch) {
                                g = Math.floor(g/max * 255);
                            }
                            var col = cols[g];
                            if (g < 16) {
                                alpha = 0.2;
                            }
                            
                            ctx.fillStyle = "rgba(" + col[0] + "," + col[1] + "," + col[2] + ", " + alpha + ")";
                            ctx.fillRect(x*s,y*s,s,s);
                        }
                    }

                }

            }
            var end = new Date().getTime();
            var time = end - start;
        };
    $timeSlider.slider(sliderOpts);
    sliderOpts['min'] = 0;
    sliderOpts['max'] = 24;
    sliderOpts['range'] = true;
    sliderOpts['values'] = [8, 18];
    $timeSliderInterval.slider(sliderOpts);
    $cameras.bind("selectmap", function(e) {
        var $$ = $(this);
        var id = parseInt($$.attr("data-id"),10);

        if ($camerasWrapper.find(":checked").length > 4) {
            alert(strings["onlyFourHeatmaps"]);
            $$.attr("checked", false);
        } else {
            if ($$.attr("checked")) {
                var ne = '';
                ne += '<div class="heatmap" data-id="' + id + '" id="map-' + id + '">';
                ne += '  <div class="wrap">';
                ne += '    <img id="image-' + id + '" width="320" height="240" />';
                ne += '    <canvas class="canvas">a</canvas>';
                ne += '  </div>';
                ne += '  <div class="label">' + $.trim($$.closest("label").text()) + '</div>';
                ne += '</div>';

                var $ne = $(ne);
                mapsMap[id] = $ne;
                $ne.appendTo("#heatmaps");

                cancelAllRequests();
                dayValue = -1; // set to -1 so we force reload
                var date = $datepicker.datepicker( "getDate" );

                // @todo: this will force all maps to be reload. only need to reload the new map
                $datepicker.datepicker("option","onSelect").call(this, null, {
                    selectedMonth : date.getMonth(),
                    selectedYear : date.getFullYear(),
                    selectedDay : date.getDate()
                });

            } else {
                delete mapsMap[id];
                delete maps[id];
                delete images[id];
                $("#map-" + id).remove();
            }
        }
    }).bind("click", function() {
        $(this).trigger("selectmap");
    })

    var maskButtons = {};
    $("div#mask input[type=submit]").each( function () {
        var $button = $(this);
        maskButtons[$button.val()] = function () {
            $button.trigger("click");
        };
    }).parents("div.ctrlHolder").hide();
    $("div#mask").hide().dialog({
            autoOpen : false,
            draggable : true,
            modal : true,
            buttons: maskButtons,
            width: 550,
    });

    //var heatmapMask = function(){ return $("#mask-area").heatmapMasking(); };
    var heatmapMask = $("#mask-area").heatmapMasking() ; // This does not work i IE, check it!

    $("div#mask input[name=auto_mask]").click(function(e) {
        if($(this).is(":checked")) {
            $(this).parents("div.ctrlHolder").siblings("div.ctrlHolder.manual-mask").slideUp();
        } else {
            $(this).parents("div.ctrlHolder").siblings("div.ctrlHolder.manual-mask").slideDown();
        }
    });

    $("div#mask div.mask.ctrlHolder").hide();

    $("div#mask ul li").click(function (e) {

            var $option = $("form#mask-form select[name=heatmap_mask_id] option:selected");
                maskid = parseInt($option.val(), 10),
                action = $(this).attr("class"),
                img_url = $("form#mask-form").data("image-url");

            e.preventDefault();
            $("form#mask-form input[name=action]").val(action);
            switch(action) {
                case "add":
                   $("div#mask div.mask.ctrlHolder").slideDown();
                   $("form#mask-form input[name=mask-name]").val("");
                   heatmapMask.setMask(img_url, [2]);
                   break;
                case "delete":
                    if(maskid == -1) {
                        alert(strings["maskMustBeChoosen"]);
                        break;
                    }
                    $("form#mask-form").submit();
                    break;
                case "edit":
                    if(maskid == -1) {
                        alert(strings["maskMustBeChoosen"]);
                        break;
                    }
                    $("form#mask-form input[name=mask-name]").val($option.text())
                    $("div#mask div.mask.ctrlHolder").slideDown();
                    heatmapMask.setMask(img_url, maskOps.mask2array($option.data("heatmapMask"),true));
                    break;
                default:
                    break;
            }
    });

    $("div#mask select[name=heatmap_mask_id]").change(function(e) {
        $("form#mask-form input[name=action]").val("nop");
        $("div#mask div.mask.ctrlHolder").slideUp();
    });

    $("form#mask-form").bind("refresh-data", function(e,data) {
            if(data===undefined) data={};
            var $$ = $(this),
                id = data.id || parseInt($("input[name=camera_id]", $$).val(),10);

            data = $.extend(true, {}, {"auto_mask" : true, "heatmap_mask_id": -1}, data);

            if(data.id) {
                $("input[name=camera_id]", $$).val(data.id);
            }
            //figure out if auto is checked
            $("input[name=auto_mask]", $$).attr("checked", data.auto_mask);
            if(data.auto_mask) {
                $("div.ctrlHolder.manual-mask", $$).slideUp();
            } else {
                $("div.ctrlHolder.manual-mask", $$).slideDown();
            }
            //ajax request load mask-data
            if(data.msg) {
                $("<div/>", {"id": "OKMsg"})
                    .append($("<p/>").text(data.msg))
                    .insertAfter($("select[name=heatmap_mask_id]",$$).parent())
                    .fadeOut(3000, function() {$(this).remove();});
            }

            $.ajax({
                url: options.maskListUrl + '/id/' + id,
                dataType: 'json',
                success: function(data_ret) {
                   var $select = $("select[name=heatmap_mask_id]", $$);
                   $("option.mask", $select).remove();
                   $.each(data_ret, function() {
                        $("<option />", {"class" : "mask" })
                            .text(this.name)
                            .val(this.heatmap_mask_id)
                            .attr("selected", this.heatmap_mask_id == data.heatmap_mask_id)
                            .data("heatmapMask", this.mask)
                            .appendTo($select);
                   });
                },
                error : function(x, e) {
                    // @todo: nicer error
                    if(e != "abort" && showError)
                        alert('An error occured. Might be a good to try to logout and login.');
                }
            });
    });

    $("form#mask-form").submit(function (e) {
        var $$ = $(this),
            data = {};
        e.preventDefault();

        if($("input[name=action]").val() == "nop") {
            var $option = $("select[name=heatmap_mask_id] option:selected", $$);
            // not sure there is a point to this anymore since they are kinda stored elsewhere, maybe for loading.
            $("#camera-box-" + $("input[name=camera_id]", $$).val()).data("mask",
                {
                    "auto_mask" : $("input[name=auto_mask]:checked", $$).val() == "1",
                    "heatmap_mask_id" : $("select[name=heatmap_mask_id] option:selected", $$).val()
                }
            );

            masks[$("input[name=camera_id]", $$).val()] = $("select[name=heatmap_mask_id] option:selected", $$).data("heatmapMask");
            manual_mask[$("input[name=camera_id]", $$).val()] = $("input[name=auto_mask]:checked", $$).val() != "1";
            $("div#mask").dialog('close');
            $timeResolution.trigger('change');
            return;
        }

        $("input[name=mask]", this).val(maskOps.array2mask(heatmapMask.getMask(),true));

        $.ajax({
            url : $$.attr("action"),
            dataType : 'json',
            type : 'POST',
            data : $$.serialize(),
            success : function(data) {
                $$.trigger("refresh-data", $.extend(true,{},{"auto_mask": false}, data));
            },
            error : function() {
                // @todo: nicer error
                if(e != "abort" && showError)
                    alert('An error occured. Might be a good to try to logout and login.')
            }
        });

        $("div#mask div.mask.ctrlHolder").slideUp();
        $("input[name=action]", this).val('nop');
    });

    $("div#heatmap-cameras label span").click(function (e) {
        e.preventDefault();
        var id = $(this).siblings("input:last").data("id");
        $("form#mask-form").data("image-url", options.imagesBaseUrl + "/" + images[id + ""].filenames[images[id + ""].filenames.length-1]);
        $("div#mask").dialog("open");
        $("form#mask-form input[name=camera_id]").val(id);
        $("form#mask-form").trigger("refresh-data", $(this).siblings("input:last").data("mask") || {});
    });

    var $timeWrapper = $("#time-wrapper");
    $timeResolution.change(function() {
        var onADay = 24*60*60,
            delta = $(this).val(),
            nbrOfSteps = onADay/delta;


        if(delta == onADay) {
            $timeWrapper.slideUp('fast');
        } else {
            $timeWrapper.slideDown('fast');
        }

        if(delta == 0) {
            $timeSlider.slideUp('fast');
            $timeSliderInterval.slideDown('fast');
        } else if( delta == onADay) {
            $timeSliderInterval.slideUp('fast');
            $timeSlider.slideUp('fast');
        } else {
            $timeSlider.slideDown('fast');
            $timeSliderInterval.slideUp('fast');
        }

        if (delta == 0) {
            nbrOfSteps = onADay / (60*60);
            delta = prevTimeRes;

        }

        var sliderVal = $timeSlider.slider('option', 'value');
        $timeSlider.slider('option', 'max', nbrOfSteps)
            .slider('option', 'value', Math.round(sliderVal * prevTimeRes/delta)); // select time that is in the same interval

        prevTimeRes = delta;

        cancelAllRequests();
        dayValue = -1; // set to -1 so we force reload
        var date = $datepicker.datepicker( "getDate" );

        // @todo: this will force all maps to be reload. only need to reload the new map
        $datepicker.datepicker("option","onSelect").call(this, null, {
            selectedMonth : date.getMonth(),
            selectedYear : date.getFullYear(),
            selectedDay : date.getDate()
        });

    });

    // trigger first requests
    $datepicker.datepicker("option","onSelect").call(this, null, {
        selectedMonth : maxDate.getMonth(),
        selectedYear : maxDate.getFullYear(),
        selectedDay : maxDate.getDate()
    });
    $cameras.trigger('selectmap');
}

})(jQuery);
});
