<?php

class Bootstrap extends Zend_Application_Bootstrap_Bootstrap
{

    protected function _initSession()
    {
        Zend_Session::start();
        if(!function_exists("r_log")) {
            function r_log($obj, $pre=""){
                error_log($pre . " " . print_r($obj,true));
            }
        }
    }

    protected function _initAutoLoader(array $options = array())
    {
        $autoloader = Zend_Loader_Autoloader::getInstance();
        $autoloader->registerNamespace('CM_');
        $autoloader->registerNamespace('Talking_');
        $autoloader->registerNamespace('UniForm_');
        $autoloader->registerNamespace('WebReport_');
        $autoloader->registerNamespace('Zend_');

        // Modules to be loaded for installation to work.
        $modules = array(
            array('name' => 'Installation',
                  'path' => '/modules/installation'),
            array('name' => 'Core',
                  'path' => '/modules/core'),
            array('name' => 'Charts',
                  'path' => '/modules/charts'),
            array('name' => 'Reports',
                  'path' => '/modules/reports')
        );

        foreach($modules as $module) {
            $resourceLoader = new Zend_Loader_Autoloader_Resource(array(
                'basePath' => APPLICATION_PATH . $module['path'],
                'namespace' => ucfirst($module['name'])
            ));

            $resourceLoader->addResourceType('plugin', 'plugins/', 'Plugin')
                ->addResourceType('form', 'forms/', 'Form')
                ->addResourceType('model', 'models/', 'Model');
        }


    }

    protected function _initDb(array $options = array())
    {

        $options = $this->getOption('database');
        $db = Zend_Db::factory($options['adapter'], $options['params']);
        $db->setFetchMode(Zend_Db::FETCH_OBJ);
        Zend_Db_Table_Abstract::setDefaultAdapter($db);
        Zend_Registry::set('db', $db);

        return $db;
    }

    protected function _initView(array $options = array())
    {
        $this->bootstrap('Constants');
        $view = new Zend_View();

        $view->doctype('XHTML1_STRICT');
        $view->setEncoding('utf-8');

        $view->addHelperPath('CM/View/Helper', 'CM_View_Helper');
        $view->addHelperPath('WebReport/View/Helper', 'WebReport_View_Helper');
        $view->addHelperPath('Talking/View/Helper', 'Talking_View_Helper');
        $view->addScriptPath(MODULES_PATH . '/core/views/scripts');

        $view->doctype('XHTML1_STRICT');

        // Add it to the ViewRenderer
        $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('ViewRenderer');
        $viewRenderer->setView($view);

        // Return it, so that it can be stored by the bootstrap
        return $view;
    }

    protected function _initForms(array $options = array())
    {
        $this->bootstrap('FrontController');
        $front = $this->getResource('FrontController');
        $this->bootstrap('view');
        $view = $this->getResource('view');
        $this->bootstrap('languages');
        $translate = $this->getResource('languages');
        Zend_Form::setDefaultTranslator($translate);
        UniForm_Form::setCssDirectory($front->getBaseUrl() . '/templates/' . $view->template() . '/css/uni-form');
    }

    protected function _initLanguages(array $options = array())
    {
        $options = $this->getOption('general');
        $languages = $options['languages'];
        Zend_Registry::set('languages', $languages);
        $translate = null;

        foreach ($languages as $language) {
            $languageFile = APPLICATION_PATH . '/languages/' . $language . '.mo';
            if (Talking_Filesystem_Dir::exists($languageFile)) {
                if ($translate == null) {
                    $translate = new Zend_Translate('gettext', $languageFile, $language);
                } else {
                    $translate->addTranslation($languageFile, $language);
                }
            }
        }

        $sessionNamespace = new Zend_Session_Namespace('WebReport_Installation');

        try {
            $locale = new Zend_Locale(Zend_Locale::BROWSER);
        } catch (Exception $e) {
            $locale = new Zend_Locale('en_US');
        }

        if (isset($sessionNamespace->language)) {
            $locale->setLocale($sessionNamespace->language);
        }

        if (!in_array($locale->getLanguage(), $languages)) {
            $locale->setLocale("en_" . $locale->getRegion());
        }


        Zend_Registry::set('Zend_Locale', $locale);
        $translate->setLocale($locale);
        Zend_Registry::set('Zend_Translate', $translate);

        return $translate;
    }

}
